package com.hand.hls.web.svc;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.BeanDefinitionRegistryPostProcessor;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.core.type.filter.AnnotationTypeFilter;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.Map;

import static org.springframework.util.Assert.notNull;

public class ServiceTagFactory implements BeanDefinitionRegistryPostProcessor, InitializingBean {

    private ServiceTag defaultTag;
    private HashMap<String, Map<String, Object>> tagMap = new HashMap<>(100);
    private String basePackage;

    public HashMap<String, Map<String, Object>> getTagMap() {
        return tagMap;
    }

    public void setTagMap(HashMap<String, Map<String, Object>> tagMap) {
        this.tagMap = tagMap;
    }

    public String getBasePackage() {
        return basePackage;
    }

    public void setBasePackage(String basePackage) {
        this.basePackage = basePackage;
    }

    public void afterPropertiesSet() throws Exception {
        notNull(this.basePackage, "Property 'basePackage' is required");
    }

    public ServiceTag getTag(String nameSpace, String name) {
        Map<String, Object> map = tagMap.get(nameSpace);
        if (map != null) {
            return (ServiceTag) map.get(name);
        }
        return getDefaultTag();
    }

    private ServiceTag getDefaultTag() {
        if (defaultTag == null) {
            defaultTag = new DefaultServiceTag();
        }
        return defaultTag;
    }

    public void postProcessBeanFactory(ConfigurableListableBeanFactory arg0) throws BeansException {
    }

    public void postProcessBeanDefinitionRegistry(BeanDefinitionRegistry registry) throws BeansException {
        ServiceComponentScanner scanner = new ServiceComponentScanner(registry);
        scanner.setServiceTagFactory(this);
        scanner.addIncludeFilter(new AnnotationTypeFilter(SvcTag.class));
        scanner.scan(StringUtils.tokenizeToStringArray(this.basePackage, ConfigurableApplicationContext.CONFIG_LOCATION_DELIMITERS));
    }

}
