package com.hand.hls.web.svc.components;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import java.io.File;
import java.io.FileReader;
import java.util.Map;

public class ScriptRunner {
    private ScriptEngine nashorn;
    private Logger logger;
    private PathMatchingResourcePatternResolver resolver;
    private BmOperation bmOperation;
    private String path;

    public void init(ApplicationContext context){
        
    }

    protected ScriptRunner(BmOperation bmOperation, String path) {
        this.bmOperation = bmOperation;
        nashorn = new ScriptEngineManager().getEngineByName("nashorn");
        resolver = new PathMatchingResourcePatternResolver();
        logger = LoggerFactory.getLogger(getClass());
        this.path = path;
    }

    public void prepare(String key, Object value) {
        nashorn.put(key, value);
    }

    public void prepare(Map<String, Object> values) {
        if (values != null) {
            values.forEach((k, v) -> nashorn.put(k, v));
        }
    }

    public Object run(String jsPath) throws Exception {
        try {
            nashorn.put("$bm", bmOperation);
            if (logger.isDebugEnabled()) {
                logger.debug("prepare run js,path: {}", jsPath);
            }
            File file = new File(path + jsPath);
            if (!file.exists() || file.isDirectory()) {
                throw new RuntimeException("could not found target js file");
            }
            return nashorn.eval(new FileReader(file));
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw e;
        }
    }
}