package com.hand.hls.web.svc.utils;

import com.hand.hls.web.svc.domain.Field;
import com.hand.hls.web.svc.domain.Model;
import com.hand.hls.web.svc.domain.QueryField;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.Assert;

import java.util.List;
import java.util.stream.Collectors;

/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2017-12-07
 * Time: 15:13
 */
public class ModelQueryParser {
    public static String parseQuery(Model model) {
        // a query statement can be split to 4 parts
        // 1. query part
        String query = parseFields(model.getFields(), model.getTableAlias());
        // 2. from part
        String from = parseFrom(model.getTableName(), model.getTableAlias());
        // 3. where part
        String where = parseWhere(model.getQueryFields(), model.getTableAlias());
        // 4. order part
        String order = parseOrder(model.getFields(), model.getTableAlias());

        StringBuilder sb = new StringBuilder();
        sb.append(" select ").append(query)
                .append(" from ").append(from);
        if(StringUtils.isNotEmpty(where)){
            sb.append(" where ").append(where);
        }
        if(StringUtils.isNotEmpty(order)){
            sb.append(" order by ").append(order);
        }

        return sb.toString();
    }

    private static String parseOrder(List<Field> fields, String tableAlias) {
        //todo: parse order part
        return null;
    }



    private static String parseWhere(List<QueryField> queryFields, String tableAlias) {
        if (CollectionUtils.isEmpty(queryFields)) {
            return null;
        }
        String where = StringUtils.join(queryFields.stream().map(queryField -> {
            String queryExp = queryField.getQueryExp();
            if (StringUtils.isNotEmpty(queryExp)) {
                return queryExp;
            }
            String field = queryField.getField();
            String name = queryField.getParamPath();
            Assert.notNull(field, "queryField must has field attribute");
            String queryOperation = queryField.getQueryOperation();
            Assert.notNull(queryOperation, "queryField must has queryOperation attribute");
            return (StringUtils.isNotEmpty(tableAlias) ? tableAlias + "." : "") + field + queryOperation + ":" + name;
        }).collect(Collectors.toList()), " and ");
        return where;
    }

    private static String parseFrom(String tableName, String tableAlias) {
        Assert.notNull(tableName, "must has an tableName");
        if(StringUtils.isNotEmpty(tableAlias)){
            return tableName + " " + tableAlias;
        }
        return tableName;
    }

    private static String parseFields(List<Field> fields, String alias) {
        Assert.notEmpty(fields, "fields must has 1 element at least");
        return StringUtils.join(fields.stream().map(field -> FieldParser.parseQueryField(field, alias)).collect(Collectors.toList()), " , ");
    }
}
