package hls.core.calc.controllers;


import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.exception.TokenException;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.calc.dto.HlsCalcConfig;
import hls.core.calc.dto.HlsPriceListConfigBT;
import hls.core.calc.dto.HlsPriceListConfigHd;
import hls.core.calc.dto.HlsPriceListConfigLn;
import hls.core.calc.mapper.HlsPriceListConfigHdMapper;
import hls.core.calc.service.HlsCalcConfigService;
import hls.core.calc.service.HlsPriceListConfigHdService;
import hls.core.calc.service.HlsPriceListConfigLnService;
import hls.core.csh.exception.ChangeLimitException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 租金计算器
 * 
 * @Title: Calc
 * @Description:
 * @Company:
 * @author: yang.gao01@hand-china.com
 * @date 2016.11.25
 *
 */
@Controller
@RequestMapping(value = "/hls/calcConfig")
public class HlsCalcConfigController extends BaseController {

	/**
	 * 租金计算器配置保存
	 * 
	 * @param result
	 *            BindingResult
	 * @param request
	 *            HttpServletRequest
	 * @return ResponseData
	 * @throws TokenException
	 */
	@Autowired
	private HlsCalcConfigService calcService;
	@Autowired
	private HlsPriceListConfigHdMapper hlsPriceListConfigHdMapper;
	@Autowired
	private HlsPriceListConfigHdService hlsPriceListConfigHdService;
	@Autowired
	private HlsPriceListConfigLnService hlsPriceListConfigLnService;

	@RequestMapping(value = "/submit", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData submitCalcConfig(HttpServletRequest request, @RequestBody List<HlsCalcConfig> list)
			throws TokenException {
		ResponseData rd = null;
		try {
			IRequest iRequest = createRequestContext(request);
			List<HlsCalcConfig> lists = calcService.batchUpdate(iRequest, list);
			rd = new ResponseData(lists);
		} catch (Exception e) {
			// TODO: handle exception
			rd = new ResponseData(false);
			rd.setMessage("价目表代码不可重复");
		}
		
		return rd;
	}
	
	@RequestMapping(value = "/submitSheet", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData submitSheet(HttpServletRequest request, String priceList,@RequestBody String sheets)
			throws TokenException {
		ResponseData rd = null;
		try {
			calcService.updateSheets(priceList, sheets);
			rd = new ResponseData();
		} catch (Exception e) {
			// TODO: handle exception
			rd = new ResponseData(false);
			rd.setMessage(e.getMessage());
		}
		
		return rd;
	}
	
	@RequestMapping(value = "/query")
	@ResponseBody
	public ResponseData query(HttpServletRequest request, HlsCalcConfig hcc,
			@RequestParam(defaultValue = DEFAULT_PAGE) int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
		IRequest requestCtx = createRequestContext(request);

		return new ResponseData(calcService.queryAll(requestCtx, hcc, page,
				pagesize));
	}

	@RequestMapping(value = "/queryUnSheets")
	@ResponseBody
	public ResponseData queryUnSheets(HttpServletRequest request, HlsCalcConfig hcc,
							  @RequestParam(defaultValue = DEFAULT_PAGE) int page,
							  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
		IRequest requestCtx = createRequestContext(request);

		return new ResponseData(calcService.queryUnSheets(requestCtx, hcc, page,
				pagesize));
	}

	@RequestMapping(value = "/getConfigHdInfoByPriceList")
	@ResponseBody
	public ResponseData getConfigHdInfoByPriceList(HttpServletRequest request, HlsPriceListConfigHd hcc,
							  @RequestParam(defaultValue = DEFAULT_PAGE) int page,
							  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
		IRequest requestCtx = createRequestContext(request);

		return new ResponseData(hlsPriceListConfigHdMapper.getConfigHdInfoByPriceList(hcc));
	}
	
	@RequestMapping(value = "/remove")
	@ResponseBody
	public ResponseData delete(HttpServletRequest request,
			@RequestBody List<HlsCalcConfig> list) {
		calcService.batchDelete(list);
		return new ResponseData();
	}
	
	@RequestMapping(value = "/querySheet",produces="application/javascript;charset=utf-8")
	@ResponseBody
	public String querySheet(HttpServletRequest request,HlsCalcConfig hcc) {
		IRequest requestCtx = createRequestContext(request);
		HlsCalcConfig hlsCalcConfig=calcService.selectByPrimaryKey(requestCtx, hcc);
		if(hlsCalcConfig.getSheets()==null){
			//add by wangwei 没有定义sheet时，初始化一个sheet让其可以维护
			return "[{\"name\":\"Sheet1\",\"rows\":[]}]";
		}else{
			return hlsCalcConfig.getSheets();
		}

	}

	@RequestMapping(value = "/lineConfig/query", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData queryLn(HttpServletRequest request, HlsPriceListConfigLn calcConfig,
								@RequestParam(defaultValue = DEFAULT_PAGE) int page,
								@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
		PageHelper.startPage(page, pagesize);
//		List<HlsPriceListConfigLn> list = hlsPriceListConfigLnService.select(createRequestContext(request), calcConfig, page,
//				pagesize);
		List<HlsPriceListConfigLn> list = hlsPriceListConfigLnService.selectHlsPriceListConfiglineByPriceList(calcConfig,page,pagesize);
		return new ResponseData(list);
	}

	@RequestMapping(value = "/headConfig/submit")
	@ResponseBody
	public ResponseData submit(HttpServletRequest request,@RequestBody  HlsPriceListConfigHd calcConfigs)
			throws ChangeLimitException {
		try {
			IRequest iRequest = createRequestContext(request);
			List<HlsPriceListConfigHd> lists = hlsPriceListConfigHdService.submit(iRequest, calcConfigs);
			return new ResponseData(lists);
		} catch (Exception e) {
			throw new ChangeLimitException(e.getMessage());
		}
	}

	@RequestMapping(value = "/lineConfig/delete", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData delete(@RequestBody List<HlsPriceListConfigLn> calcConfigs) {
		hlsPriceListConfigLnService.batchDelete(calcConfigs);
		return new ResponseData();
	}

	@RequestMapping(value = "/btConfig/query", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData calcBtConfigQuery(HttpServletRequest request,@RequestParam String priceList){
		List<HlsPriceListConfigBT> btList = calcService.btConfigQuery(priceList);
		return new ResponseData(btList);
	}


}
