package hls.core.calc.service.impl;

import com.hand.hap.core.IRequest;
import hls.core.calc.service.HlsCalcExcelImportUtilService;
import hls.core.calc.service.QuotationCommon;
import hls.core.csh.service.IConContractCashflowService;
import hls.core.prj.dto.HlsCusPrjQuotation;
import hls.core.prj.dto.HlsCusPrjQuotationDetails;
import hls.core.prj.mapper.HlsCusPrjQuotationMapper;
import hls.core.prj.service.PrjQuotationDetailsService;
import hls.core.prj.service.PrjQuotationService;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class ContractQuotationServiceImpl implements QuotationCommon {


    @Autowired
    private HlsCusPrjQuotationMapper prjQuotationMapper;
    @Autowired
    private IConContractCashflowService conContractCashflowService;
    @Autowired
    HlsCalcExcelImportUtilService hlsCalcExcelImportUtilService;
    @Autowired
    private PrjQuotationService prjQuotationService;
    private static final String sourceDocumentCategory = "CON_CONTRACT";
    @Autowired
    private PrjQuotationDetailsService prjQuotationDetailsService;

    @Override
    public String getSourceDocumentCategory() {
        return sourceDocumentCategory;
    }

    @Override
    public HlsCusPrjQuotation PrjQuotationSubmit(IRequest iRequest, HlsCusPrjQuotation prjQuotation) throws Exception {
        //校验公式是否被修改
        hlsCalcExcelImportUtilService.excelFormatHasChange(iRequest, prjQuotation.getPriceList(), prjQuotation.getSheets());
        Object prj = hlsCalcExcelImportUtilService.getExcelToCalcHdTable(iRequest, prjQuotation.getSheets(), prjQuotation.getPriceList(), "prj");
        HlsCusPrjQuotation prjQuotationDto = new HlsCusPrjQuotation();
        BeanUtils.copyProperties(prjQuotationDto,prj);
        if (prjQuotationDto.getLeaseTimes() == null) {
            throw new IllegalArgumentException("期数未取到！");
        }
        prjQuotationDto.setPriceList(prjQuotation.getPriceList());
        prjQuotationDto.setSheets(prjQuotation.getSheets());
        prjQuotationDto.setProjectId(prjQuotation.getSourceDocumentId());
        prjQuotationDto.setSourceDocumentCategory(prjQuotation.getSourceDocumentCategory());
        prjQuotationDto.setSourceDocumentId(prjQuotation.getSourceDocumentId());
        //获取项目项下的报价
        //当前项目下的只有一个报价
        List<HlsCusPrjQuotation> prjQuotationList = prjQuotationMapper.prjQuotationDetailQuery(prjQuotationDto);
        HlsCusPrjQuotation savePrjQuotation = new HlsCusPrjQuotation();
        if (prjQuotationList.size() > 0) {
            prjQuotationDto.setQuotationId(prjQuotationList.get(0).getQuotationId());
        }
        if (prjQuotationDto.getQuotationId() == null) {
            //新插入默认为新建
            prjQuotationDto.setStatus("NEW");
            savePrjQuotation = prjQuotationService.insertSelective(iRequest, prjQuotationDto);
        } else {
            prjQuotationService.updateByPrimaryKey(iRequest, prjQuotationDto);
            savePrjQuotation.setQuotationId(prjQuotationDto.getQuotationId());
        }
        HlsCusPrjQuotationDetails prjQuotationDetails = new HlsCusPrjQuotationDetails();
        prjQuotationDetails.setQuotation_id(savePrjQuotation.getQuotationId());
        List<HlsCusPrjQuotationDetails> detailsList = new ArrayList<>();
        detailsList = prjQuotationDetailsService.queryDetailsById(prjQuotationDetails);
        if (detailsList.size() > 0) {
            detailsList.get(0).setSheets(prjQuotationDto.getSheets());
            detailsList.get(0).set__status("update");
        } else {
            prjQuotationDetails.setSheets(prjQuotationDto.getSheets());
            detailsList.add(prjQuotationDetails);
            detailsList.get(0).set__status("add");
        }
        prjQuotationDetailsService.batchUpdate(iRequest, detailsList);

        //保存现金流表
        //如果报价上的来源单据类别是项目PRJ_PROJECT，saveCalc2PrjQuotationCashflow
        //如果报价上的来源单据类别是合同CON_CONTRACT，saveCalc2ConContractCashflow
        conContractCashflowService.saveCalc2ConContractCashflow(iRequest, prjQuotationDto);


        //修改项目表中的finace_amount字段为报价表上的finace_amount
//        HlsCusPrjProject prjProject = new HlsCusPrjProject();
//        prjProject.setProjectId(prjQuotation.getSourceDocumentId());
//        prjProject.setFinanceAmount(new Double(prjQuotationDto.getFinanceAmount()).longValue());
//        prjProjectService.updateByPrimaryKey(iRequest,prjProject);

        return prjQuotationDto;
    }
}
