package hls.core.cont.components;

import com.hand.hap.core.IRequest;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.service.ConContractService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventCommonService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.sys.service.SysUserService;
import org.activiti.engine.delegate.DelegateExecution;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

abstract class AbstractConContractEventHandler  implements SysEventCommonService {
    private static final String DOCUMENT_CATEGORY = "CON_CONTRACT";

    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private ConContractService conContractService;

    public String getMessage(String userDesc, String conName, String conNum){
        return "租赁合同："+conName+"("+conNum+") "+getSuffixMsg()+"。";
    }

    abstract String getSuffixMsg();

    abstract String getNoticeTitle();

    public long getLevel() {
        return 2L;
    }

    public String getSourceModel() {
        return SysEventCodeUtil.PROPERTITY_SOURCEMODULE_CON;
    }

    public String getRelation() {
        return SysEventCodeUtil.PROPERTITY_RELATIONTYPE_D2D;
    }

    @Override
    public String getDocumentCategory() {
        return null;
    }


    public String getDocumentType(){
        return "";
    }

    @Override
    public void process(IRequest iRequest, Long documentId, String documentCategory, String documentType, Map<String, Object> params) {
        if (params == null) {
            params = new HashMap<>();
        }
        HlsCusConContract conContract = conContractService.squerConContractById(documentId);
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String message = getMessage(sysUser.getDescription(), conContract.getContractName(), conContract.getContractNumber());
        params.put("message", message);
        params.put("noticeTitle", getNoticeTitle());
        params.put("level", getLevel());
        params.put("noticeType", "NOTICE");

        sysEventService.eventSave(iRequest, documentId, DOCUMENT_CATEGORY, getDocumentType(), getSourceModel(), getEventCode(), getRelation(), params);
    }

    @Override
    public void notice(DelegateExecution delegateExecution, String status) {

    }
}
