package hls.core.cont.controllers;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;

import com.hand.hap.attachment.exception.AttachmentException;
import hls.core.cont.dto.HlsCusConContractAttachment;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hand.hap.attachment.FileInfo;
import com.hand.hap.attachment.UpConstants;
import com.hand.hap.attachment.Uploader;
import com.hand.hap.attachment.UploaderFactory;
import com.hand.hap.attachment.controllers.AttachmentController;
import com.hand.hap.attachment.dto.AttachCategory;
import com.hand.hap.attachment.dto.Attachment;
import com.hand.hap.attachment.dto.SysFile;
import com.hand.hap.attachment.exception.StoragePathNotExsitException;
import com.hand.hap.attachment.exception.UniqueFileMutiException;
import com.hand.hap.attachment.service.IAttachCategoryService;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.util.UploadUtil;
import com.hand.hap.security.TokenUtils;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.cont.service.ConContractAttachmentService;
import hls.core.hls.bp.service.HlsSysFileService;

@Controller
public class ConContractAttachmentController extends BaseController {
    /**
     * 提示信息名.
     */
    private static final String MESSAGE_NAME = "message";
    /**
     * 提示信息 name.
     **/
    private static final String INFO_NAME = "info";
    /**
     * file对象名.
     */
    private static final String FILE_NAME = "file";

    @Autowired
    private ConContractAttachmentService service;

    @Autowired
    private ObjectMapper objectMapper;
    /**
     * 日志记录.
     **/
    private static Logger logger = LoggerFactory.getLogger(AttachmentController.class);

    @Autowired
    private HlsSysFileService fileService;

    @Autowired
    private MessageSource messageSource;
    @Autowired
    private IAttachCategoryService categoryService;

    @RequestMapping(value = "/con/contract/attach/detail/query")
    @ResponseBody
    public ResponseData queryDetail(final HlsCusConContractAttachment contractAttachment, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusConContractAttachment> list = service.queryDetail(requestContext, contractAttachment, page, pagesize);
/*        String securityKey = TokenUtils.getSecurityKey(request.getSession());
        HlsCusConContractAttachment param = new HlsCusConContractAttachment();
        for (HlsCusConContractAttachment con:list) {
            param.setContractAttachmentId(con.getContractAttachmentId());
            con.set_token(TokenUtils.generateToken(securityKey,param));
        }*/
        return new ResponseData(list);
    }

    @RequestMapping(value = "/con/contract/sign/submit")
    @ResponseBody
    public HlsCusConContractAttachment signSubmit(HlsCusConContractAttachment contractAttachment, final BindingResult result,
                                                  final HttpServletRequest request) {

        IRequest requestContext = createRequestContext(request);
        return service.signSubmit(requestContext, contractAttachment);
    }

    @RequestMapping(value = "/con/contract/attach/submit")
    @ResponseBody
    public ResponseData submit(@RequestBody final List<HlsCusConContractAttachment> contractAttachments, final BindingResult result,
                               final HttpServletRequest request) {
        getValidator().validate(contractAttachments, result);
        ResponseData rd = null;

        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestContext = createRequestContext(request);
        if (contractAttachments != null) {
            for (HlsCusConContractAttachment contractAttachment : contractAttachments) {
                if ("add".equalsIgnoreCase(contractAttachment.get__status())) {

                    for(int i=0; i<contractAttachments.size(); i++){
                        HlsCusConContractAttachment attch = contractAttachments.get(i);
                        if(attch.getBpId()==null || attch.getBpCategory()=="" || attch.getConTemplate()==null){
                            rd = new ResponseData(false);

                            rd.setMessage("请将必输项输入完整！");

                            return rd;
                        }
                    }
                    contractAttachment.setSourceType("MANUAL");
                    contractAttachment.setContractAttachmentCategory("CON_TRMPLATE");
                }
                if (contractAttachment.getConTemplate() != null) {
                    contractAttachment.setSourceId(Long.parseLong(contractAttachment.getConTemplate()));
                }
            }
        }
        rd = new ResponseData(service.batchUpdate(requestContext, contractAttachments));
        return rd;
    }

    @RequestMapping(value = "/con/contract/attach/upload", method = RequestMethod.POST, produces = "text/plain;charset=UTF-8")
    public String upload(HttpServletRequest request) throws StoragePathNotExsitException, UniqueFileMutiException, JsonProcessingException, AttachmentException {
        Map<String, Object> response = new HashMap<String, Object>();
        Uploader uploader = UploaderFactory.getMutiUploader();
        uploader.init(request);
        String sourceType = uploader.getParams("sourceType");
        String sourceKey = uploader.getParams("sourceKey");
        response.put("success", false);
        if (StringUtils.isBlank(sourceType) || StringUtils.isBlank(sourceKey)) {
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.typeorkey_empty", null, null));// TYPEORKEY_EMPTY
            response.put(INFO_NAME, messageSource.getMessage("hap.typeorkey_empty", null, null));
            return objectMapper.writeValueAsString(response);
        }
        IRequest requestContext = createRequestContext(request);
        // 设置上传参数
        AttachCategory category = categoryService.selectAttachByCode(requestContext,sourceType);

        UploadUtil.initUploaderParams(uploader, category);
        // TYPE 设置不对
        if (category == null) {
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.type_error", null, null));
            response.put(INFO_NAME, messageSource.getMessage("hap.type_error", null, null));
            return objectMapper.writeValueAsString(response);
        }
        List<FileInfo> fileInfos = uploader.upload();
        // 出错了
        if (!UpConstants.SUCCESS.equals(fileInfos.get(0).getStatus())) {
            response.put(MESSAGE_NAME, fileInfos.get(0).getStatus());
            response.put(INFO_NAME, fileInfos.get(0).getStatus());
            return objectMapper.writeValueAsString(response);
        } else {
            response.put("success", true);
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.upload_success", null, null));// uploader.getStatus()
        }
        // WebUploader 每次只上传一个文件
        FileInfo f = fileInfos.get(0);
        try {
            SysFile sysFile = UploadUtil.genSysFile(f, requestContext.getUserId(), requestContext.getUserId());

            Attachment attach = UploadUtil.genAttachment(category, sourceKey, requestContext.getUserId(),
                    requestContext.getUserId());

            fileService.insertFileAndAttach(requestContext, attach,sysFile);

            sysFile.setFilePath(null);
            TokenUtils.generateAndSetToken(TokenUtils.getSecurityKey(request.getSession(false)), sysFile);
            response.put(FILE_NAME, sysFile);
        } catch (Exception e) {
            if (logger.isErrorEnabled()) {
                logger.error("database error", e);
            }
            File file = f.getFile();
            if (file.exists()) {
                file.delete();
            }
            response.put("success", false);
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.database_error", null, null));
            response.put(INFO_NAME, messageSource.getMessage("hap.database_error", null, null));
        }
        return objectMapper.writeValueAsString(response);
    }
}
