package hls.core.cont.controllers;

import com.hand.hap.attachment.dto.Attachment;
import com.hand.hap.attachment.dto.SysFile;
import com.hand.hap.attachment.exception.FileReadIOException;
import com.hand.hap.attachment.service.IAttachmentService;
import com.hand.hap.attachment.service.ISysFileService;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.exception.TokenException;
import com.hand.hap.security.TokenUtils;
import com.hand.hap.system.controllers.BaseController;
import hls.core.cont.service.ConContractDocxService;
import hls.core.cont.service.CustomSqlService;
import org.apache.commons.lang.StringUtils;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * Created by FJM on 2017/4/16.
 */
@Controller
public class ConContractDocxController extends BaseController{


    /**
     * 文件不存在提示.
     */
    private static final String FILE_NOT_EXSIT = "未找到文件模板";

    /**
     * 文件下载默认编码.
     */
    private static final String ENC = "UTF-8";


    @Autowired
    private ISysFileService fileService;

    @Autowired
    private ConContractDocxService conContractDocxService;

    @Autowired
    private IAttachmentService attachmentService;

    @RequestMapping(value = "/docxCreate")
    public void docxCreateMethod(HttpServletRequest request, HttpServletResponse response,Long contractId,Long templetId) throws TokenException, FileReadIOException, Docx4JException {

        //待合同文本表完成后，此处的contractId参数应该换位contentId,根据contentId获取书签对应的参数的key和value，然后传入到动态sql中执行

        IRequest requestContext = createRequestContext(request);

        String SourceType = "HLS_DOC_FILE_TEMPLET";//合同文本的SourceType

        String backSourceType = "";//生成备份的合同文本对应的SourceType

        Long SourceKey = templetId;//模板的ID为对应的SourceKey

        Attachment attachment = new Attachment();

        try {

            attachment = attachmentService.selectAttachByCodeAndKey(requestContext,SourceType,SourceKey.toString());//根据sys_attachment表上的记录获取附件的信息

            SysFile fileParam = new SysFile();

            fileParam.setAttachmentId(attachment.getAttachmentId());

            SysFile sysFile = fileService.selectFiles(requestContext,fileParam,0,10).get(0);//在sys_file上获取对应的文件的信息

            if (sysFile != null && StringUtils.isNotBlank(sysFile.getFilePath())) {
                File file = new File(sysFile.getFilePath());
                if (file.exists()) {

                    response.addHeader("Content-Disposition",
                            "attachment;filename=\"" + URLEncoder.encode(sysFile.getFileName(), ENC) + "\"");

                    response.setContentType(sysFile.getFileType() + ";charset=" + ENC);

                    response.setHeader("Accept-Ranges", "bytes");

                    int fileLength = (int) file.length();//先获取模板文件的大小

                    int fileBackLength = 0;//定义备份文件的大小

                    if (fileLength > 0) {

                        InputStream inStream = new FileInputStream(file);//用inputStram输入流读取本地的docx文件

                        String copyPath = sysFile.getFilePath().concat("_back_").concat(UUID.randomUUID().toString());//定义复制模板的filepath,使用uuid拼接于文件末尾，避免备份文件重名覆盖

                        this.copyModel(copyPath,inStream);//复制模板

                        InputStream modelIs = new FileInputStream(copyPath);//用输入流读取复制后的模板

                        ServletOutputStream outputStream = response.getOutputStream();//获取response对象中的outputStream输出流

                        //conContractDocxService.createDocx(requestContext,modelIs,new File(copyPath),contractId,templetId);//生成合同文本


                      //attachmentService.insert(requestContext,attachment); 此处应该根据备份的SourceType获取到对应的CATEGORY_ID,然后先插入sys_attachment表，再根据插入之后的记录，获取attachment_id，再插入sys_file表

                        fileBackLength = (int)new File(copyPath).length();

                        SysFile backSysFile = new SysFile();

                        backSysFile.setFileName(sysFile.getFileName().concat("备份"));

                        backSysFile.setFilePath(copyPath);

                        backSysFile.setFileSize((long) fileBackLength);

                        backSysFile.setFileType(sysFile.getFileType());

                        backSysFile.setUploadDate(new Date());

                        fileService.insert(requestContext,backSysFile);

                    }

                    response.setContentLength(fileBackLength);//获取替换完成之后的文件大小



                } else {
                    response.getWriter().write(FILE_NOT_EXSIT);
                }
            } else {
                response.getWriter().write(FILE_NOT_EXSIT);
            }
        } catch (IOException e) {
            throw new FileReadIOException();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }



    /**
     *  复制文件模板,将读取源文件的输入流复制文件存入一个备用的模板文件
     * @param filePath
     * @param is
     * @throws IOException
     */
      private static synchronized void copyModel(String filePath,InputStream is) throws IOException {

          FileOutputStream fos = new FileOutputStream(filePath); //复制出一个模板

          int readData;

          byte[] b = new byte[1024];

          while((readData = is.read(b)) != -1){

              fos.write(b,0,readData);

          }

          fos.flush();

          is.close();

          fos.close();


      }



}
