package hls.core.cont.service;

import hls.core.cont.dto.HlsCusConContractCashflowH;

import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 合同调息范围
 * Created by fjm on 2017/10/25.
 */
public interface IConFloatingRangeService {

    /**
     * 返回对应的调息范围类型
     *
     * @return
     */
    String getFloatingRangeMethod();

    /**
     * 把Date类型转换成为LocalDate
     * @param date
     * @return
     */
    default LocalDate date2LocalDate(Date date) {
        Instant instant = date.toInstant();
        ZoneId zoneId = ZoneId.systemDefault();
        return instant.atZone(zoneId).toLocalDate();
    }

    /**
     * 将合同现金流以调息日期进行分组,获得应该调息的现金流集合
     * @param cashflows
     * @param localRateDate
     * @return
     */
    default List<HlsCusConContractCashflowH> splitCashflow(List<HlsCusConContractCashflowH> cashflows, LocalDate localRateDate){
        LocalDate startDate,endDate;
        Long rateChangeIndex = null;
        //取第0期现金流的dueDate为租金第一期的开始日期
        Date dueDate = cashflows.stream().filter(item -> item.getTimes().compareTo(0L) == 0).findFirst().get().getDueDate();
        startDate = date2LocalDate(dueDate).plusDays(1);
        //获取租金的现金流,并按照期数从小到大排序
        cashflows = cashflows.stream().filter(item -> item.getCfItem().compareTo(1L) == 0 && item.getCfType().compareTo(1L) == 0).sorted(Comparator.comparingLong(HlsCusConContractCashflowH::getTimes)).collect(Collectors.toList());
        for (int i = 0; i < cashflows.size(); i++) {
            if (i != 0) {
                startDate = date2LocalDate(cashflows.get(i - 1).getDueDate()).plusDays(1);
            }
            endDate = date2LocalDate(cashflows.get(i).getDueDate());
            if (startDate.isBefore(localRateDate) && localRateDate.isAfter(endDate) || localRateDate.isBefore(startDate)) {
                rateChangeIndex = new Long(i);
                break;
            }
        }
        return cashflows.stream().skip(Optional.ofNullable(rateChangeIndex).orElse(new Long(cashflows.size()))).collect(Collectors.toList());
    }

    /**
     * 根据调息范围类型,截取对应的要调整的现金流
     * @param cashflows
     * @param rateDate
     * @return
     */
    List<HlsCusConContractCashflowH> rangeCashflow(List<HlsCusConContractCashflowH> cashflows, Date rateDate);

}
