package hls.core.cont.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;

import javax.servlet.http.HttpSession;

import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.dto.DTOStatus;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.dto.*;
import hls.core.cont.exception.UserWithoutCodeException;
import hls.core.cont.mapper.*;
import hls.core.csh.dto.HlsCusCshConContract;
import hls.core.csh.exception.AmountNotEqualException;
import hls.core.csh.exception.TimesUnreasonableException;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.csh.mapper.HlsCusCshConContractMapper;
import hls.core.csh.service.ConContractCashflowHService;
import hls.core.csh.service.IConContractCashflowService;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.mapper.HlsCusHlsDocumentVersionRelMapper;
import hls.core.prj.dto.HlsCusHlsDocumentVersion;
import hls.core.prj.mapper.HlsCusHlsDocumentVersionMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.core.AppContextInitListener;
import com.hand.hap.core.IRequest;

import hls.core.cont.dto.HlsCusConContractCashflowH;
import hls.core.cont.service.ConContractChangeCommon;
import hls.core.cont.service.ConContractChangeService;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.fnd.service.FndCodingRuleValuesService;


@Service
@Transactional(rollbackFor = {Exception.class, RuntimeException.class})
public class ConContractChangeServiceImpl extends BaseServiceImpl<HlsCusConContractChangeReq> implements ConContractChangeService, AppContextInitListener {

    public static final String CF_DIRECTION_INFLOW = "INFLOW";//现金流方向流入
    public static final String BLOCK = "BLOCK";
    public static final String CON_CONTRACT_CHANGE_REQ = "con_contract_change_req";
    //存储当前请求得到的ConContractH对象
    private ThreadLocal<ConContractH> contractThreadLocal = new ThreadLocal();
    @Autowired
    private HlsCusCshConContractMapper cshConContractMapper;
    @Autowired
    private HlsCusConContractChangeReqMapper hlsCusConContractChangeReqMapper;
    @Autowired
    private HlsCusConContractGuarantorMapper conContractGuarantorMapper;
    @Autowired
    private HlsCusConContractLeaseItemMapper conContractLeaseItemMapper;
    @Autowired
    private HlsCusConContractTenantMapper conContractTenantMapper;
    @Autowired
    private HlsCusConContractHMapper conContractHMapper;
    @Autowired
    private HlsCusConContractCashflowHMapper contractCashflowHMapper;
    @Autowired
    private IConContractCashflowService conContractCashflowService;
    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private HlsCusConContractCashflowMapper conContractCashflowMapper;
    @Autowired
    private HlsCusConContractCashflowHMapper cashflowHMapper;
    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private DocumentChangeCommon documentChangeCommon;
    @Autowired
    private ConContractCashflowHService conContractCashflowHService;
    @Autowired
    private HlsCusHlsDocumentVersionRelMapper hlsDocumentVersionRelMapper;
    @Autowired
    private HlsCusHlsDocumentVersionMapper hlsDocumentVersionMapper;
    // 变更Map
    private Map<String, ConContractChangeCommon> BackupsHCommonMap = new HashMap<>();
    // 删除Map
    private Map<String, ConContractChangeCommon> deleteCommonMap = new HashMap<>();
    // 审批Map
    private Map<String, ConContractChangeCommon> methodNameCommonMap = new HashMap<>();

    private String contractStauts = "";

    /**
     * 合同变更主方法
     *
     * @param changeH 合同变更对象
     * @throws ChangeLimitException
     * @throws ClassNotFoundException
     */
    public void saveBackup(IRequest iRequest, ConContractChangeH changeH, HttpSession session) throws ChangeLimitException, ClassNotFoundException, UserWithoutCodeException, AmountNotEqualException, TimesUnreasonableException, InvocationTargetException, IllegalAccessException {
        List<HlsCusCshConContract> cshConLists = selectConContract(changeH.getContractId());//查询出起租状态的数据
        Double actualSettlement = null;
        Map<String, Object> params = new HashMap();
        // 判断该合同是否可变更
        if (cshConLists == null || cshConLists.isEmpty())
            throw new ChangeLimitException("当前合同变更中，不可再次变更");
        params.put("cshConContract", cshConLists.get(0));
        // lock con_contract
        databaseLockProvider.lock(cshConLists.get(0));
        //插入合同变更表,返回合同变更ID
        HlsCusConContractChangeReq conContractChangeReq = saveConContractChangeReq(iRequest, cshConLists.get(0), changeH.getConContractChangeReq());
        Long changeReqId = conContractChangeReq.getChangeReqId();

        // 复制该合同下相应信息放入历史数据表
        duplicate(iRequest, changeH.getContractId(), changeReqId);

        ConContractH conContractH = contractThreadLocal.get();
        //当状态为提前结清时,需要特殊处理
        if ("Y".equalsIgnoreCase(conContractChangeReq.getEarlyTerminationFlag())) {

            HashMap conContractEarlyTermination = changeH.getConContractEarlyTermination();
            //Long contractId = Long.valueOf(conContractEarlyTermination.get("contractId").toString());
            Long contractId = conContractH.getContractId();
            Double fee = Double.valueOf(conContractEarlyTermination.get("fee").toString());
            Double terminationPrincipal = Double.valueOf(conContractEarlyTermination.get("terminationPrincipal").toString());
            Double terminationInterest = Double.valueOf(conContractEarlyTermination.get("terminationInterest").toString());
            Date changeReqDate = conContractChangeReq.getChangeReqDate();
            actualSettlement = Double.valueOf(conContractEarlyTermination.get("actualSettlement").toString());

            //获取当前合同现金流的最大期数
            HlsCusConContractCashflowH cashflowH = new HlsCusConContractCashflowH();
            cashflowH.setContractId(contractId);
            cashflowH.setCfItem(1L);
            List<HlsCusConContractCashflowH> cashflowHs = contractCashflowHMapper.select(cashflowH);
            Long maxTime = 0L;
            for (HlsCusConContractCashflowH item : cashflowHs) {
                if (item.getTimes() > maxTime) {
                    maxTime = item.getTimes();
                }
                item.setCfStatus(BLOCK);
                item.set__status(DTOStatus.UPDATE);
            }
            conContractCashflowHService.batchUpdate(iRequest, cashflowHs);

            HlsCusConContractCashflowH earlyTermationCashflow = new HlsCusConContractCashflowH();//提前结清费用
            earlyTermationCashflow.setCfItem(11L);
            earlyTermationCashflow.setCfType(11L);
            earlyTermationCashflow.setContractId(contractId);
            earlyTermationCashflow.setDueDate(changeReqDate);
            earlyTermationCashflow.setDueAmount(CalculateUtil.add(terminationInterest, terminationPrincipal));
            earlyTermationCashflow.setPrincipal(terminationPrincipal);
            earlyTermationCashflow.setInterest(terminationInterest);
            earlyTermationCashflow.setTimes(maxTime);
            earlyTermationCashflow.set__status(DTOStatus.ADD);
            earlyTermationCashflow.setCfDirection(CF_DIRECTION_INFLOW);

//            HlsCusCshConContract contract = new HlsCusCshConContract();//修改当前合同状态为暂挂
//            contract.setContract_id(contractId);
//            modifyState(contract, 1);
            HlsCusConContractCashflowH feeCashflow = new HlsCusConContractCashflowH();//结清手续费
            feeCashflow.setCfItem(113L);
            feeCashflow.setCfType(3L);
            feeCashflow.setContractId(contractId);
            feeCashflow.setDueDate(changeReqDate);
            feeCashflow.setDueAmount(fee);
            feeCashflow.setTimes(maxTime);
            feeCashflow.set__status(DTOStatus.ADD);
            feeCashflow.setCfDirection(CF_DIRECTION_INFLOW);

            List<HlsCusConContractCashflowH> termationCashflows = new ArrayList<>();
            termationCashflows.add(earlyTermationCashflow);
            termationCashflows.add(feeCashflow);
            conContractCashflowHService.batchUpdate(iRequest, termationCashflows);

//            contractStauts = cshConLists.get(0).getContract_status();// 保存此合同现在所处的状态
        } else if ("Y".equalsIgnoreCase(conContractChangeReq.getChangeCashflowFlag())) {

            //修改历史数据表中的数据
            List<HlsCusConContractCashflowH> needChangeList = new ArrayList<>();
            for (HlsCusConContractCashflow item : changeH.getConContractCashflows()) {
                HlsCusConContractCashflowH cashflowH = new HlsCusConContractCashflowH();
                cashflowH.setChangeReqId(changeReqId);
                cashflowH.setCashflowId(item.getCashflow_id());
                //创建要修改的cashflowH数据
                String status = item.get__status();
                HlsCusConContractCashflowH dto = (HlsCusConContractCashflowH) DocumentChangeCommon.getHistoryDto(item);
                dto.set__status(status);
                dto.setContractId(conContractH.getContractId());
                if(DTOStatus.UPDATE.equals(status)||DTOStatus.DELETE.equals(status)) {
                    //获取新cashflowid
                    Long cashFlowId = cashflowHMapper.selectCashFlowList(cashflowH).get(0).getCashflowId();
                    dto.setCashflowId(cashFlowId);
                }
                needChangeList.add(dto);
            }
            conContractCashflowHService.batchUpdate(iRequest, needChangeList);
            //修改当前合同状态为暂挂
            HlsCusCshConContract contract = new HlsCusCshConContract();
            contract.setContract_id(changeH.getContractId());
            modifyState(contract, 1);
            //对现金流历史表中的request数据进行判断
            HlsCusConContract dto = new HlsCusConContract();
            dto.setContractId(changeH.getContractId());
            checkCashFlowInfo(dto, changeReqId);
        } else {
            // 保存此合同现在所处的状态
            contractStauts = cshConLists.get(0).getContract_status();
            //修改
            makeOver(iRequest, changeH, cshConLists, changeReqId);
            //删除
            delete(iRequest, changeH);
        }

        //启动工作流
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        if (employee.getEmployeeCode() != null) {
            String employeeCode = employee.getEmployeeCode();
            iRequest.setEmployeeCode(employeeCode);
        } else {
            throw new UserWithoutCodeException();
        }

        List<HlsCusConContractChangeReq> ccrs = new ArrayList();
        ccrs.add(conContractChangeReq);


        params.put("workFlowType", "CONTRACT_CHANGE");
        params.put("actualSettlement", actualSettlement);
        activitiStartService.start(iRequest, ccrs, params);

        //变更状态进入审批中
        modifyStateReq(iRequest, conContractChangeReq, "APPROVING");

        Long contractId = changeH.getContractId();
        sysEventService.createEvent(iRequest, contractId, "CON_CONTRACT", "", null, SysEventCodeUtil.PROPERTY_CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_CREATE);

    }

    /**
     * 删除
     *
     * @param iRequest
     * @param changeH
     * @throws ClassNotFoundException
     * @throws ChangeLimitException
     */
    private void delete(IRequest iRequest, ConContractChangeH changeH) throws ClassNotFoundException, ChangeLimitException {
        // delete删除
        Class<?> clazz = Class.forName("hls.core.cont.dto.ConContractChangeDeleteArr");
        // 获取"hls.core.cont.dto.ConContractChangeDeleteArr"的所有方法名称
        Method[] methods = clazz.getMethods();
        for (Method method : methods) {
            String methodName = method.getName();
            // 判断方法开头和结尾标志
            if (methodName.startsWith("get") && methodName.endsWith("Del")) {// Del
                // 将方法名改为全小写
                methodName = methodName.substring(3).toLowerCase();
                // 获取相应对象
                ConContractChangeCommon ChangeCommon = deleteCommonMap.get(methodName);
                if (ChangeCommon == null) {
                    throw new ChangeLimitException("当前功能正在开发中，请联系管理员.");
                } else {
                    // 删除_h表中数据
                    ChangeCommon.delete(iRequest, changeH);
                }
            }
        }
    }

    /**
     * 修改
     *
     * @param iRequest
     * @param changeH
     * @param cshConLists
     * @param changeReqId
     * @throws ChangeLimitException
     */
    private void makeOver(IRequest iRequest, ConContractChangeH changeH, List<HlsCusCshConContract> cshConLists, Long changeReqId) throws ChangeLimitException {
        // 修改合同状态为挂载
        modifyState(cshConLists.get(0), 1);
        String[] fieldName = changeH.getFieldName();
        // 修改
        for (int j = 0; j < fieldName.length; j++) {
            ConContractChangeCommon ChangeCommon = BackupsHCommonMap.get(fieldName[j]);
            if (ChangeCommon == null) {
                throw new ChangeLimitException("当前功能正在开发中，请联系管理员.");
            } else {
                // 修改信息
                ChangeCommon.dispose(iRequest, changeH, changeReqId);
            }
        }
    }

    /**
     * 修改合同变更状态
     *
     * @param changeReqId
     * @param contractId
     */
    public void changeReq(IRequest iRequest, Long changeReqId, Long contractId) {
        HlsCusConContractChangeReq changeReq = achieve(iRequest, changeReqId, contractId);
        changeReq.setStatus("APPROVED");
//		changeReq.setCreatedBy(iRequest.getUserId());
//		changeReq.setCreationDate(new Date());
        self().updateByPrimaryKey(iRequest, changeReq);
    }

    /**
     * 获取合同变更表数据
     *
     * @param changeReqId 合同变更ID
     * @param contractId  合同ID
     * @return
     */
    private HlsCusConContractChangeReq achieve(IRequest iRequest, Long changeReqId, Long contractId) {
        HlsCusConContractChangeReq changeReq = new HlsCusConContractChangeReq();
        changeReq.setChangeReqId(changeReqId);
        changeReq.setContractId(contractId);
        changeReq = self().selectByPrimaryKey(iRequest, changeReq);
//		changeReq = changeReqMapper.select(iRequest,changeReq,0,0).get(0);
        return changeReq;
    }

    /**
     * 变更审批
     *
     * @param changeReqId 合同变更ID
     * @throws ClassNotFoundException
     * @throws SecurityException
     * @throws NoSuchMethodException
     * @throws InvocationTargetException
     * @throws IllegalArgumentException
     * @throws IllegalAccessException
     */
    public void examineAndApprove(IRequest iRequest, Long changeReqId, Long contractId, String status)
            throws ClassNotFoundException, NoSuchMethodException, SecurityException, IllegalAccessException,
            IllegalArgumentException, InvocationTargetException {
        HlsCusConContractChangeReq changeReq = achieve(iRequest, changeReqId, contractId);
        Class<?> clazz = changeReq.getClass();
        // 获取changeReq对象的所有方法名称
        Method[] methods = clazz.getMethods();
        for (Method method : methods) {
            String methodName = method.getName();
            // 判断方法名称开始标志
            if (methodName.startsWith("getChange")) {
                Method m2 = clazz.getMethod(methodName);
                // 判断哪些信息进行了变更
                if ("Y".equals(m2.invoke(changeReq))) {
                    ConContractChangeCommon ChangeCommon = methodNameCommonMap.get(methodName);
                    // 调用各信息对应的serviceImpl的审批方法
                    ChangeCommon.examineAndApprove(iRequest, changeReqId, contractId, status);
                }
            }

        }
    }

    @Override
    public List<HlsCusConContractChangeReq> select(IRequest requestContext, HlsCusConContractChangeReq changeReq) {
        return hlsCusConContractChangeReqMapper.queryDetail(changeReq);
    }

    /**
     * 将该合同所对应的所有相应信息复制到备份表中
     *
     * @param contract_id
     */
    public void duplicate(IRequest iRequest, Long contract_id, Long changeReqId) {
        // 组装合同对象
        HlsCusCshConContract conContract = new HlsCusCshConContract();
        conContract.setContract_id(contract_id);
        //组装承租人对象
        HlsCusConContractGuarantor guarantor = new HlsCusConContractGuarantor();
        guarantor.setContractId(contract_id);
        //组装担保人对象
        HlsCusConContractTenant tenant = new HlsCusConContractTenant();
        tenant.setContractId(contract_id);
        //组装租赁物对象
        HlsCusConContractLeaseItem leaseItem = new HlsCusConContractLeaseItem();
        leaseItem.setContractId(contract_id);
        // 组装合同现金流对象
        HlsCusConContractCashflow conContractCashflow = new HlsCusConContractCashflow();
        conContractCashflow.setContract_id(contract_id);
        // 获取该合同下所有相应信息
        List<HlsCusCshConContract> contracts = cshConContractMapper.select(conContract);
        List<HlsCusConContractGuarantor> Guarantors = conContractGuarantorMapper.select(guarantor);
        List<HlsCusConContractLeaseItem> LeaseItems = conContractLeaseItemMapper.select(leaseItem);
        List<HlsCusConContractTenant> tenants = conContractTenantMapper.select(tenant);
        List<HlsCusConContractCashflow> cashflows = conContractCashflowMapper.select(conContractCashflow);

        HlsCusConContractChangeReq changeReq = hlsCusConContractChangeReqMapper.selectByPrimaryKey(changeReqId);
        HlsCusHlsDocumentVersion documentVersion = documentChangeCommon.beforeChangeRequest(iRequest, changeReq);
        Map param = new HashMap<>();
        // 复制
        for (int i = 0; i < contracts.size(); i++) {
            HlsCusConContractH contractHistory =  documentChangeCommon.copyHistoryCommon(iRequest, contracts.get(i), null, documentVersion.getVersionId());
            param.put(DocumentChangeCommon.FIX_COLUMN_NAME, "contractId");
            param.put(DocumentChangeCommon.FIX_COLUMN_ID, contractHistory.getContractId());
            contractThreadLocal.set(contractHistory);
        }

        for (int i = 0; i < Guarantors.size(); i++) {
            documentChangeCommon.copyHistoryCommon(iRequest, Guarantors.get(i), param, documentVersion.getVersionId());
        }
        for (int i = 0; i < LeaseItems.size(); i++) {
            documentChangeCommon.copyHistoryCommon(iRequest, LeaseItems.get(i), param, documentVersion.getVersionId());
        }
        for (int i = 0; i < tenants.size(); i++) {
            documentChangeCommon.copyHistoryCommon(iRequest, tenants.get(i), param, documentVersion.getVersionId());
        }
        for (int i = 0; i < cashflows.size(); i++) {
            documentChangeCommon.copyHistoryCommon(iRequest, cashflows.get(i), param, documentVersion.getVersionId());
        }
    }

    /**
     * 根据合同ID查询合同表信息
     *
     * @param contract_id
     * @return
     */
    public List<HlsCusCshConContract> selectConContract(Long contract_id) {
        HlsCusCshConContract conContract = new HlsCusCshConContract();
        conContract.setContract_id(contract_id);
        List<HlsCusCshConContract> lists = cshConContractMapper.selectConContract(conContract);
        return lists;
    }

    /**
     * 修改变更所处状态
     *
     * @param iRequest
     * @param conContractChangeReq
     * @param statusName
     */
    public void modifyStateReq(IRequest iRequest, HlsCusConContractChangeReq conContractChangeReq, String statusName) {

        HlsCusConContractChangeReq changeReq = new HlsCusConContractChangeReq();
        changeReq.setChangeReqId(conContractChangeReq.getChangeReqId());
        changeReq.setStatus(statusName);
        self().updateByPrimaryKeySelective(iRequest, changeReq);
    }

    /**
     * 插入合同变更表
     *
     * @param iRequest
     * @param conContract
     */
    public HlsCusConContractChangeReq saveConContractChangeReq(IRequest iRequest, HlsCusCshConContract conContract, HlsCusConContractChangeReq req) {
        HlsCusConContractChangeReq changeReq = new HlsCusConContractChangeReq();
        changeReq.setContractId(conContract.getContract_id());
        changeReq.setStatus("NEW");
        changeReq.setChangeReqDate(req.getChangeReqDate());
        changeReq.setDocumentType(conContract.getDocument_type());
        changeReq.setDocumentCategory(conContract.getDocument_category());
        changeReq.setBusinessType(conContract.getBusiness_type());
        changeReq.setChangeReqNumber(getCodeValue(iRequest, conContract));
        changeReq.setEarlyTerminationFlag(req.getEarlyTerminationFlag());
        changeReq.setChangeCashflowFlag(req.getChangeCashflowFlag());
        changeReq.setChangeTenantFlag(req.getChangeTenantFlag());
        changeReq.setChangeGuarantorFlag(req.getChangeGuarantorFlag());
        changeReq.setChangeLeaseItemFlag(req.getChangeLeaseItemFlag());
        changeReq.setChangeContractTextFlag(req.getChangeContractTextFlag());
        changeReq.setCreatedBy(iRequest.getUserId());
        changeReq.setCreationDate(new Date());
        changeReq.setDescription(req.getDescription());
        changeReq.setChangeReqUserId(req.getChangeReqUserId());
        self().insertSelective(iRequest, changeReq);
        return changeReq;
    }

    public String getCodeValue(IRequest iRequest, HlsCusCshConContract conContract) {
        Map<String, String> params = new HashMap<String, String>();
        String value = codingRuleValuesService.getCodeRuleValue(iRequest, conContract.getDocument_category(),
                conContract.getDocument_type(), conContract.getBusiness_type(), params);
        return value;
    }

    /**
     * 修改合同状态
     *
     * @param conContract
     */
    public void modifyState(HlsCusCshConContract conContract, int characteristic) {
        // 当characteristic>0修改为挂载状态，否则修改为可变更状态
        if (characteristic > 0) {
            conContract.setContract_status("PENDING");
        } else {
            if ("".equalsIgnoreCase(contractStauts) || contractStauts == null) {
                contractStauts = conContract.getContract_status();
            }
            conContract.setContract_status(contractStauts);
        }
        cshConContractMapper.updateConContractStatus(conContract);
    }

    @Override
    public void contextInitialized(ApplicationContext applicationContext) {
        Map<String, ConContractChangeCommon> map = applicationContext.getBeansOfType(ConContractChangeCommon.class);
        map.forEach((k, v) -> {
            BackupsHCommonMap.put(v.getChangeName(), v);
            String str = v.getDeleteName().toLowerCase().trim();
            deleteCommonMap.put(str, v);
            methodNameCommonMap.put(v.getMethodName(), v);
        });
    }

    /*合同变更-提前结清-判断是否有提前结清类型的正在审批中的数据*/
    @Override
    public String checkChangeStatus(Map map) {
        String flag = "N";
        HlsCusConContractChangeReq conContractChangeReq = new HlsCusConContractChangeReq();
        conContractChangeReq.setContractId(Long.parseLong(map.get("contractId").toString()));
        conContractChangeReq.setStatus("APPROVING");
        conContractChangeReq.setEarlyTerminationFlag("Y");
        List<HlsCusConContractChangeReq> list = hlsCusConContractChangeReqMapper.select(conContractChangeReq);
        if (list.size() > 0) {
            flag = "Y";
        }
        return flag;
    }

    /*合同变更-租赁计划变更-校验修改现金流信息是否合法*/
    @Transactional(propagation = Propagation.REQUIRED)
    public void checkCashFlowInfo(HlsCusConContract contract, Long changeReqId) throws AmountNotEqualException, TimesUnreasonableException {
        //查询现金流历史表中的所有现金流信息
        HlsCusConContractCashflowH cashflowH = new HlsCusConContractCashflowH();
        cashflowH.setContractId(contract.getContractId());
        cashflowH.setChangeReqId(changeReqId);
        //获取所有现金流
        List<HlsCusConContractCashflowH> listAll = cashflowHMapper.selectCashFlowList(cashflowH);
        //融资额
        Double finaceAmount = conContractMapper.selectByPrimaryKey(contract).getFinanceAmount();
        //本金总和
        Double amountSum = 0D;
        //租金的期数
        List<Long> rentTimeList = new ArrayList<>();
        //所有现金流的期数
        List<Long> timeList = new ArrayList<>();
        //校验金额以及租金期数是否重复
        for (HlsCusConContractCashflowH item : listAll) {
            if (item.getPrincipal() != null) {
                amountSum = CalculateUtil.add(amountSum, item.getPrincipal());
            }
            if (item.getCfItem() == 1 && !rentTimeList.contains(item.getTimes())) {
                rentTimeList.add(item.getTimes());
            } else if (item.getCfItem() == 1 && rentTimeList.contains(item.getTimes())) {
                throw new TimesUnreasonableException();
            }
            if (!timeList.contains(item.getTimes())) {
                timeList.add(item.getTimes());
            }
        }
        if (amountSum.compareTo(finaceAmount) != 0) {
            throw new AmountNotEqualException();
        }
        //校验期数是否连续
        for (int i = 1; i < timeList.size(); i++) {
            if (timeList.get(i) != null && timeList.get(i - 1) != null) {
                Long diff = timeList.get(i) - timeList.get(i - 1);
                if (diff.compareTo(1L) != 0) {
                    throw new TimesUnreasonableException();
                }
            } else {
                throw new TimesUnreasonableException();
            }

        }

    }

    /*合同变更抽屉-判断某一个合同是否做过变更*/
    public List<HlsCusConContractChangeReq>  checkHasChangeHistory(IRequest requestContext, HlsCusConContractChangeReq changeReq){
        return  hlsCusConContractChangeReqMapper.select(changeReq);
    }
}
