package hls.core.cont.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.HlsCusConContractTenantHMapper;
import hls.core.cont.service.ConContractService;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.hls.mapper.HlsUserRelationshipMapper;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.service.SysNoticePushService;
import hls.core.sys.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.cont.dto.HlsCusConContractTenant;
import hls.core.cont.service.ConContractEventService;
import hls.core.csh.dto.HlsCusCshConContract;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.hls.dto.HlsDocAuthority;
import hls.core.hls.dto.HlsSystemEvent;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsUserRelationship;
import hls.core.hls.dto.HlsDocRelationship;
import hls.core.hls.service.HlsSystemEventService;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.hls.service.IHlsDocAuthorityService;
import hls.core.hls.service.IHlsUserRelationshipService;
import hls.core.hls.service.HlsDocRelationshipService;


/**
 * 
 *   Copyright (C) Hand Business Consulting Services
 *          AllRights Reserved
 *   
 *   @DESCRIPTION: 合同单据事件接口实现类
 *
 *   @author: SYJ
 *
 *   @date: 2017年2月7日
 */

@Service
@Transactional(rollbackFor={Exception.class,RuntimeException.class})
public class ConContractEventServiceImpl extends BaseServiceImpl<HlsCusCshConContract>
		implements ConContractEventService  {

	@Autowired
	HlsSystemEventService hlsSystemEventService;
	@Autowired
	IHlsDocAuthorityService docAuthorityService;
	@Autowired
	IHlsUserRelationshipService hlsUserRelationshipService;
	@Autowired
	HlsSystemNoticeService hlsSystemNoticeService;
	@Autowired
	private HlsCusConContractTenantHMapper conContractTenantMapper;
	@Autowired
	private HlsEmployeeMapper hlsEmployeeMapper;
	@Autowired
	private SysNoticePushService sysNoticePushService;
	@Autowired
	private SysUserService sysUserService;
	@Autowired
	private ConContractService conContractService;
	@Autowired
	private HlsUserRelationshipMapper userRelationshipMapper;
	@Autowired
	private HlsDocRelationshipService hlsDocRelationshipService;
	@Autowired
	private SysEventService sysEventService;


	public static String SOURCE_MODULE = "CON";
	public static String SOURCE_DOCUMENT_CATEGORY = "CON_CONTRACT";
	public static String RELATION_DOCUMENT_CATEGORY = "HLS_BP_MASTER";

	public static String AUTHORITY_TYPE_QUERY = "1"; //查询
	public static String AUTHORITY_TYPE_MODIFY = "2";//修改

	public static String RELATION_TYPE_1 = "P2P";//人对人
	public static String RELATION_TYPE_2 = "P2D";//人对单据

	public static String NOTICE_MESSAGE_1 = "合同创建事件";
	public static String NOTICE_MESSAGE_2 = "合同签约事件";
	public static String NOTICE_MESSAGE_3 = "合同起租事件";
	public static String NOTICE_MESSAGE_4 = "合同结清事件";
	public static String NOTICE_MESSAGE_5 = "合同取消事件";

	public static String NOTICE_TYPE = "NOTICE";
	public static String NOTICE_URL = "cont/CON100/contract_demo.view";//合同主界面URL
	public static Long NOTICE_LEVEL = (long) 1;

	@Override
	public void contractEvent(IRequest iRequest, String eventCode,
			Long contractId, Long userId)  {

		List<HlsSystemEvent> hlsSystemEventList = new ArrayList<>();
		List<HlsDocAuthority> hlsDocAuthorityList = new ArrayList<>();
		List<HlsUserRelationship> hlsUserRelationshipList = new ArrayList<> ();
		List<HlsSystemNotice> hlsSystemNoticeList = new ArrayList<> ();
		List<HlsDocRelationship> hlsDocRelationshipList = new ArrayList<>();
		SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
		HlsCusConContract conContract = conContractService.squerConContractById(contractId);

		//HlsSystemEvent 赋值
		HlsSystemEvent hlsSystemEvent = new HlsSystemEvent();

		hlsSystemEvent.setEventCode(eventCode);
		hlsSystemEvent.setSourceDocumentCategory(SOURCE_DOCUMENT_CATEGORY);
		hlsSystemEvent.setSourceDocumentId(contractId);
		hlsSystemEvent.setEventUserId(userId);

		//HlsDocAuthority 赋值
		HlsDocAuthority hlsDocAuthority = new HlsDocAuthority();

		hlsDocAuthority.setUserId(userId); 
		hlsDocAuthority.setDocumentCategory(SOURCE_DOCUMENT_CATEGORY);
		hlsDocAuthority.setDocumentId(contractId);

		hlsDocAuthority.setAuthorityType(AUTHORITY_TYPE_MODIFY);

		hlsDocAuthority.setSourceModule(SOURCE_MODULE);
		hlsDocAuthority.setSourceEvent(eventCode);
		hlsDocAuthority.setSourceDocumentCategory(SOURCE_DOCUMENT_CATEGORY);
		hlsDocAuthority.setSourceUserId(iRequest.getUserId());

		//HlsUserRelationship 赋值
		HlsUserRelationship hlsUserRelationship = new HlsUserRelationship();

		hlsUserRelationship.setUserId(userId);
		hlsUserRelationship.setRelationType(RELATION_TYPE_2);
		hlsUserRelationship.setRelationUserId(null);

		hlsUserRelationship.setRelationDocumentCategory(SOURCE_DOCUMENT_CATEGORY);
		hlsUserRelationship.setRelationDocumentId(contractId);

		hlsUserRelationship.setSourceModule(SOURCE_MODULE);
		hlsUserRelationship.setSourceEvent(eventCode);
		hlsUserRelationship.setSourceDocumentId(contractId);
		hlsUserRelationship.setSourceUserId(iRequest.getUserId());

		//HlsDocRelationship赋值
		HlsDocRelationship hlsDocRelationship = new HlsDocRelationship();
		hlsDocRelationship.setDocument_category(RELATION_DOCUMENT_CATEGORY);
		hlsDocRelationship.setDocument_id(userId);
		hlsDocRelationship.setRelation_type("D2D");
		hlsDocRelationship.setReleation_document_category("CON_CONTRACT");
		hlsDocRelationship.setReleation_document_id(contractId);
		hlsDocRelationship.setSource_module("CON");
		hlsDocRelationship.setSource_event("CON_CONTRACT.CREATE");
		hlsDocRelationship.setSource_document_category("CON_CONTRACT");
		hlsDocRelationship.setSource_document_id(contractId);
		hlsDocRelationship.setSource_user_id(iRequest.getUserId());
		hlsDocRelationship.set__status("add");
		hlsDocRelationshipList.add(hlsDocRelationship);

		//HlsSystemNotice 赋值
		HlsSystemNotice hlsSystemNotice = new HlsSystemNotice();

		switch (eventCode) {
		case "CON_CONTRACT.CREATE":
			hlsSystemNotice.setNotice_message(sysUser.getDescription()+"创建合同"+"【"+conContract.getContractName()+"】");
			hlsSystemNotice.setNotice_title("合同创建");
			break;
		case "CON_CONTRACT.SIGN":
			hlsSystemNotice.setNotice_message("合同"+"【"+conContract.getContractName()+"】签约成功");
			hlsSystemNotice.setNotice_title("合同签约");
			break;
		case "CON_CONTRACT.INCEPT":
			hlsSystemNotice.setNotice_message("合同"+"【"+conContract.getContractName()+"】起租成功");
			hlsSystemNotice.setNotice_title("合同起租");
			break;
		case "CON_CONTRACT.TERMINATION":
			hlsSystemNotice.setNotice_message("合同"+"【"+conContract.getContractName()+"】已结清");
			hlsSystemNotice.setNotice_title("合同结清");
			break;
		case "CON_CONTRACT.CANCEL":
			hlsSystemNotice.setNotice_message("合同"+"【"+conContract.getContractName()+"】被"+sysUser.getDescription()+"取消成功");
			hlsSystemNotice.setNotice_title("合同取消");
			break;
		default:
			break;
		}

		hlsSystemNotice.setNotice_datetime(new Date());
		hlsSystemNotice.setNotice_type(NOTICE_TYPE);
		hlsSystemNotice.setNotice_url(NOTICE_URL);
		hlsSystemNotice.setNotice_level(NOTICE_LEVEL);
		hlsSystemNotice.setSource_module(SOURCE_MODULE);
		hlsSystemNotice.setSource_event(eventCode);
		hlsSystemNotice.setSource_document_category(SOURCE_DOCUMENT_CATEGORY);
		hlsSystemNotice.setSource_document_id(contractId);
		hlsSystemNotice.setSource_user_id(iRequest.getUserId());

		hlsSystemEvent.set__status("add");
		hlsDocAuthority.set__status("add");
		hlsUserRelationship.set__status("add");
		hlsSystemNotice.set__status("add");

		hlsSystemEventList.add(hlsSystemEvent);
		if(eventCode.equals("CON_CONTRACT.CREATE")) {
			hlsDocAuthorityList.add(hlsDocAuthority);
		}
		hlsUserRelationshipList.add(hlsUserRelationship);
		hlsSystemNoticeList.add(hlsSystemNotice);

		if (!eventCode.equals("CON_CONTRACT.CREATE") ) {
			hlsSystemEventService.batchSave(iRequest, hlsSystemEventList, hlsDocAuthorityList, hlsUserRelationshipList, hlsSystemNoticeList);
		} else {
			hlsDocRelationshipService.batchUpdate(iRequest,hlsDocRelationshipList);

			hlsSystemEventService.batchUpdate(iRequest, hlsSystemEventList);
			List<HlsEmployee> parentPositionUserIdList = hlsEmployeeMapper.queryParentPositionUserId(userId);

			hlsDocAuthorityList.clear();
			for (int i = 0; i < parentPositionUserIdList.size(); i++) {
				Long parentPositionUserId = parentPositionUserIdList.get(i).getUserId();
				hlsDocAuthority.setUserId(parentPositionUserId);
				hlsDocAuthority.setDocumentCategory(SOURCE_DOCUMENT_CATEGORY);
				hlsDocAuthority.setDocumentId(contractId);
				hlsDocAuthority.setAuthorityType(RELATION_TYPE_1);
				hlsDocAuthority.setSourceModule(SOURCE_MODULE);
				hlsDocAuthority.setSourceEvent(eventCode);
				hlsDocAuthority.setSourceDocumentCategory(SOURCE_DOCUMENT_CATEGORY);
				hlsDocAuthority.setSourceUserId(iRequest.getUserId());
				hlsDocAuthorityList.add(hlsDocAuthority);
			}
			docAuthorityService.batchUpdate(iRequest, hlsDocAuthorityList);
			HlsCusConContractTenant Tenant = new HlsCusConContractTenant();
			Tenant.setContractId(contractId);
			List<HlsCusConContractTenant> conContractTenant = conContractTenantMapper.selectConContractTenant(Tenant);
			hlsUserRelationshipList.clear();
			for (int i = 0; i < conContractTenant.size(); i++) {
				Long bpId = conContractTenant.get(i).getTenantBpId();
				hlsUserRelationship.setUserId(userId);
				hlsUserRelationship.setRelationType(RELATION_TYPE_2);
				hlsUserRelationship.setSourceModule(SOURCE_MODULE);
				hlsUserRelationship.setSourceEvent(eventCode);
				hlsUserRelationship.setSourceDocumentId(contractId);
				hlsUserRelationship.setSourceUserId(iRequest.getUserId());
				hlsUserRelationshipList.add(hlsUserRelationship);
				if("P2P".equalsIgnoreCase(hlsUserRelationship.getRelationType())) {
					hlsUserRelationship.setRelationUserId(null);
					HlsUserRelationship userRelationship = new HlsUserRelationship();
					userRelationship.setUserId(hlsUserRelationship.getUserId());
					userRelationship.setRelationType(hlsUserRelationship.getRelationType());
					userRelationship.setRelationUserId(hlsUserRelationship.getRelationUserId());
					List<HlsUserRelationship> list =  userRelationshipMapper.select(userRelationship);
					if(list.size() == 0) {
						hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship );
					}
				}else if("P2D".equalsIgnoreCase(hlsUserRelationship.getRelationType())){
					hlsUserRelationship.setRelationDocumentCategory(RELATION_DOCUMENT_CATEGORY);
					hlsUserRelationship.setRelationDocumentId(bpId);
					HlsUserRelationship userRelationship1 = new HlsUserRelationship();
					userRelationship1.setUserId(hlsUserRelationship.getUserId());
					userRelationship1.setRelationType(hlsUserRelationship.getRelationType());
					userRelationship1.setSourceDocumentCategory(hlsUserRelationship.getSourceDocumentCategory());
					userRelationship1.setSourceDocumentId(hlsUserRelationship.getSourceDocumentId());
					List<HlsUserRelationship> lists =  userRelationshipMapper.select(userRelationship1);
					if(lists.size() == 0) {
						hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship );
					}
				}
			}
			hlsSystemNoticeService.batchUpdate(iRequest, hlsSystemNoticeList);
		}
		if(eventCode.equals("CON_CONTRACT.CREATE")){
			sysEventService.setNoticeCache(hlsSystemNotice);
		}
	}
}
