package hls.core.cont.service.impl;


import com.hand.hap.core.IRequest;
import com.hand.hap.excel.ExcelException;
import hls.core.cont.service.ConContractExcelService;
import hls.core.csh.mapper.HlsCusCshConContractMapper;
import hls.core.csh.service.ICshConContractService;
import hls.core.fnd.dto.FndInterfaceLines;
import hls.core.fnd.mapper.FndInterfaceLinesMapper;
import hls.core.hls.service.HlsBeanRefUtilService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by syj on 2017/3/7.
 */
@Service
@Transactional
public class ConContractExcelServiceImpl implements ConContractExcelService {
    @Autowired
    @Qualifier("dataSource")
    DataSource dataSource;
    @Autowired
    FndInterfaceLinesMapper fndInterfaceLinesMapper;
    @Autowired
    HlsBeanRefUtilService hlsBeanRefUtilService;
    @Autowired
    ICshConContractService iCshConContractService;
    @Autowired
    HlsCusCshConContractMapper cshConContractMapper;

    private final int batchCount = 100;
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    Connection conn;
    PreparedStatement ps;
    ResultSet rs;
    Map<String, Long> ContractIdMap = new HashMap<String, Long>();


    public void contractExcelImport(IRequest iRequest, Long hdId) throws SQLException, ExcelException {


        insertTable(iRequest, hdId, "CON_CONTRACT");
        insertTable(iRequest, hdId, "CON_CONTRACT_CASHFLOW");
        insertTable(iRequest, hdId, "CON_CONTRACT_TENANT");
        insertTable(iRequest, hdId, "CON_CONTRACT_GUARANTOR");
        insertTable(iRequest, hdId, "CON_CONTRACT_LEASE_ITEM");
        insertTable(iRequest, hdId, "CON_CONTRACT_MORTGAGE");
        insertTable(iRequest, hdId, "CON_CONTRACT_SIGN");
        insertTable(iRequest, hdId, "CON_CONTRACT_INCEPT");

    }

    private void insertTable(IRequest iRequest, Long hdId, String sheetName) throws SQLException, ExcelException {
        //获取接口表数据
        FndInterfaceLines fndInterfaceLines = new FndInterfaceLines();

        Long user_id = iRequest.getUserId();

        fndInterfaceLines.setHeaderId(hdId);

        fndInterfaceLines.setSheetName(sheetName);
        List<FndInterfaceLines> fndInterfaceLinesList = fndInterfaceLinesMapper.select(fndInterfaceLines);

        fndInterfaceLines = fndInterfaceLinesList.get(1);
        Map<String, String> column = new HashMap<String, String>();
        Map<String, String> val = new HashMap<String, String>();

        column.clear();
        val.clear();

        column = hlsBeanRefUtilService.getFieldValueMap(fndInterfaceLines);
        int k = 1;
        while (column.get("attributes_" + k) != null) {
            k++;
        }
        String sql = "";
        String sqlValue = "";
        String sqlQuery = "";
        int batchNum = 0;
        Long contractId = 0L;
        try {
            conn = dataSource.getConnection();
            conn.setAutoCommit(false);
            ps = conn.prepareStatement("");

            for (int i = 2; i < fndInterfaceLinesList.size(); i++) {

                //合同子表是通过 contract_number 关联起来
                if (sheetName != "CON_CONTRACT") {
                    //通过合同编号找合同ID

                    contractId = ContractIdMap.get(fndInterfaceLinesList.get(i).getAttributes_1());
                    sql = "insert into " + sheetName + " (contract_id,";
                    sqlValue = "";
                    //从第二列开始插入不插入合同编号
                    for (int j = 2; j < k; j++) {

                        if (hlsBeanRefUtilService.getFieldValueMap(fndInterfaceLinesList.get(i)).get("attributes_" + j) != null) {
                            if (!"".equals(hlsBeanRefUtilService.getFieldValueMap(fndInterfaceLinesList.get(i)).get("attributes_" + j).trim())) {
                                sql = sql + column.get("attributes_" + j) + ',';
                                sqlValue = sqlValue + '\'' + hlsBeanRefUtilService.getFieldValueMap(fndInterfaceLinesList.get(i)).get("attributes_" + j) + '\'' + ',';
                            }
                        }
                    }

                    sql = sql + "OBJECT_VERSION_NUMBER,REQUEST_ID,PROGRAM_ID,CREATED_BY,CREATION_DATE,LAST_UPDATED_BY,LAST_UPDATE_DATE,LAST_UPDATE_LOGIN) values (" + contractId + "," + sqlValue + "1,-1,-1," + user_id + ",CURDATE()," + user_id + ",CURDATE()," + user_id + ")";
                    this.logger.info("插入语句：" +sql);
                } else {

                    sqlQuery = "SELECT AUTO_INCREMENT FROM INFORMATION_SCHEMA.TABLES  WHERE  TABLE_NAME = '" + sheetName + "'";
                    PreparedStatement psQuery = conn.prepareStatement(sqlQuery);
                    contractId = this.queryDate(psQuery);
                    contractId = contractId + i - 2;
                    sql = "insert into " + sheetName + " (contract_id,";
                    sqlValue = "";
                    for (int j = 1; j < k; j++) {

                        if (hlsBeanRefUtilService.getFieldValueMap(fndInterfaceLinesList.get(i)).get("attributes_" + j) != null) {
                            if (!"".equals(hlsBeanRefUtilService.getFieldValueMap(fndInterfaceLinesList.get(i)).get("attributes_" + j).trim())) {
                                sql = sql + column.get("attributes_" + j) + ',';
                                sqlValue = sqlValue + '\'' + hlsBeanRefUtilService.getFieldValueMap(fndInterfaceLinesList.get(i)).get("attributes_" + j) + '\'' + ',';
                            }
                        }
                    }

                    sql = sql + "OBJECT_VERSION_NUMBER,REQUEST_ID,PROGRAM_ID,CREATED_BY,CREATION_DATE,LAST_UPDATED_BY,LAST_UPDATE_DATE,LAST_UPDATE_LOGIN) values (" + contractId + "," + sqlValue + "1,-1,-1," + user_id + ",CURDATE()," + user_id + ",CURDATE()," + user_id + ")";
                    ContractIdMap.put(fndInterfaceLinesList.get(i).getAttributes_1(), contractId);
                    this.logger.info("插入语句：" +sql);
                }

                ps.addBatch(sql);
                ++batchNum;
                if (batchNum >= batchCount) {
                    this.insertData(ps);
                    batchNum = 0;
                }
            }
            if (batchNum > 0) {
                this.insertData(ps);
            }
            conn.commit();
        } catch (SQLException e) {
            conn.rollback();
            this.logger.info("插入数据失败，数据已回滚..");
            e.printStackTrace();
            throw e;
        } catch (ExcelException e) {
            e.printStackTrace();
            this.logger.info("插入数据失败，数据已回滚..");
            conn.rollback();
            throw e;
        } finally {
            ps.close();
            conn.close();
        }
    }


    private void insertData(PreparedStatement statement) throws ExcelException {
        try {
            statement.executeBatch();
        } catch (SQLException var3) {
            var3.printStackTrace();
            throw new ExcelException((String) null, "数据插入失败: " + var3.getMessage(), (Object[]) null);
        }
    }

    private Long queryDate(PreparedStatement statement) throws SQLException {
        Long id = 0L;
        try {
            rs = statement.executeQuery();
            while (rs.next()) {
                id = rs.getLong("AUTO_INCREMENT");
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return id;

    }
}