package hls.core.cont.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.dto.HlsCusConContractIncept;
import hls.core.cont.mapper.HlsCusConContractInceptMapper;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.cont.service.ConContractEventService;
import hls.core.cont.service.ConContractInceptService;
import hls.core.cont.service.IConContractInceptApprovingCustomService;
import hls.core.csh.dto.HlsCusCshConContract;
import hls.core.csh.mapper.HlsCusCshConContractMapper;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.wfl.service.IActivitiStartService;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by FJM on 2017/1/9.
 */
@Service
@Transactional
public class ConContractInceptServiceImpl extends BaseServiceImpl<HlsCusConContractIncept> implements ConContractInceptService {

    @Autowired
    private HlsCusCshConContractMapper cshConContractMapper;
    @Autowired
    private HlsCusConContractInceptMapper conContractInceptMapper;
    @Autowired
    private ConContractEventService eventService;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired(required = false)
    private IConContractInceptApprovingCustomService iConContractInceptApprovingCustomService;

    @Override
    public HlsCusConContractIncept insert(IRequest request, HlsCusConContractIncept record) {

        HlsCusCshConContract cshConContract = new HlsCusCshConContract();
        cshConContract.setContract_id(record.getContractId());
        List<HlsCusCshConContract> list = cshConContractMapper.select(cshConContract);

        for (HlsCusCshConContract conContract : list) {
            conContract.setContract_status("NEW");
            cshConContractMapper.updateByPrimaryKeySelective(conContract);
        }
        return super.insert(request, record);
    }


    @Override
    public void updateContractStatus(@Param("contractId") Long contractId, @Param("userId") Long userId) {
        conContractInceptMapper.updateContractStatus(contractId, userId);
    }

    @Override
    public List<HlsCusConContractIncept> approving(IRequest iRequest, List<HlsCusConContractIncept> conContractIncepts) {
        if (iConContractInceptApprovingCustomService != null && !iConContractInceptApprovingCustomService.beforeInceptApproving(iRequest, conContractIncepts)) {
            return null;
        }
        for (HlsCusConContractIncept conContractIncept : conContractIncepts) {
            conContractIncept.setStatus("NEW");
            conContractIncept.set__status("add");
        }
        Long contractId = conContractIncepts.get(0).getContractId();
        HlsCusConContract conContract = new HlsCusConContract();
        conContract.setContractId(contractId);
        conContract = conContractMapper.selectByPrimaryKey(conContract);
        if (conContract == null) {
            return null;
        }
        // lock con_contract
        databaseLockProvider.lock(conContract);
        List<HlsCusConContractIncept> list = self().batchUpdate(iRequest, conContractIncepts);

        HlsCusConContractIncept conContractIncept = conContractIncepts.get(0);
        //eventService.contractEvent(iRequest,"CON_CONTRACT.INCEPT",conContractIncept.getContractId(),iRequest.getUserId());

        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "CONTRACT_INCEPT");
        activitiStartService.start(iRequest, list, params);

        HlsCusConContractIncept contractIncept = new HlsCusConContractIncept();
        contractIncept.setStatus("APPROVING");
        contractIncept.setContractInceptId(list.get(0).getContractInceptId());
        self().updateByPrimaryKeySelective(iRequest, contractIncept);
        if (iConContractInceptApprovingCustomService != null) {
            iConContractInceptApprovingCustomService.afterInceptApproving(iRequest, conContractIncepts);
        }
        return list;
    }
}
