package hls.core.cont.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.cont.service.*;
import hls.core.csh.service.IConContractCashflowService;
import hls.core.fnd.dto.FndCompany;
import hls.core.fnd.mapper.FndCompanyMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.dto.DocumentType;
import hls.core.hls.mapper.DocumentTypeMapper;
import hls.core.hls.service.HlsBeanRefUtilService;
import hls.core.prj.dto.HlsCusPrjProject;
import hls.core.prj.dto.HlsCusPrjQuotation;
import hls.core.prj.dto.HlsCusPrjQuotationDetails;
import hls.core.prj.mapper.HlsCusPrjQuotationDetailsMapper;
import hls.core.prj.mapper.HlsCusPrjQuotationMapper;
import hls.core.prj.service.PrjProjectService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.cont.dto.HlsCusConContract;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class ConContractServiceImpl extends BaseServiceImpl<HlsCusConContract> implements ConContractService{
    public static final String PRJ_PROJECT = "PRJ_PROJECT";
    @Autowired
    private PrjProjectService prjProjectService;
    @Autowired
    private HlsBeanRefUtilService hlsBeanRefUtilService;
    @Autowired
    private DocumentTypeMapper documentTypeMapper;
    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;
    @Autowired
    private HlsCusPrjQuotationMapper prjQuotationMapper;
    @Autowired
    private IConContractCashflowService iConContractCashflowService;
    @Autowired
    private ConContractTenantService conContractTenantService;
    @Autowired
    private ConContractGuarantorService  conContractGuarantorService;
    @Autowired
    private ConContractLeaseItemService conContractLeaseItemService;
    @Autowired
    private ConContractMortgageService conContractMortgageService;
    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private ConContractEventService conContractEventService;
    @Autowired
    private HlsCusPrjQuotationDetailsMapper prjQuotationDetailsMapper;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private FndCompanyMapper fndCompanyMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;


    private static String DocumentCategory="CON_CONTRACT";
    @Override
    public HlsCusConContract saveConContractFromPrjProject(IRequest iRequest, HlsCusPrjProject prjProjectPrama){
        Long projectId=prjProjectPrama.getProjectId();
        HlsCusPrjProject prjProject=prjProjectService.selectByPrimaryKey(iRequest,prjProjectPrama);
        if(prjProject==null){
            throw new IllegalArgumentException("当前项目不存在！");
        }
        if (!prjProject.getProjectStatus().equals("APPROVED")){
            throw new IllegalArgumentException("当前项目未审批通过！");
        }
        // lock prj_project
        databaseLockProvider.lock(prjProject);

        Map<String,String> map= hlsBeanRefUtilService.getFieldValueMap(prjProject);
        HlsCusConContract conContract= new HlsCusConContract();
        hlsBeanRefUtilService.setFieldValue(conContract,map);
        //把报价上的字段也复制到合同表上
        HlsCusPrjQuotation prjQuotationPrameter=new HlsCusPrjQuotation();
        prjQuotationPrameter.setProjectId(prjProject.getProjectId());
        prjQuotationPrameter.setSourceDocumentCategory(PRJ_PROJECT);
        List<HlsCusPrjQuotation> prjQuotationList= prjQuotationMapper.prjQuotationDetailQuery(prjQuotationPrameter);
        //项目只有一个报价的情况
        if(prjQuotationList.get(0)!=null){
            Map<String,String> quotationMap=hlsBeanRefUtilService.getFieldValueMap(prjQuotationList.get(0))  ;
            hlsBeanRefUtilService.setFieldValue(conContract,quotationMap);
        }

        conContract.setDocumentCategory(DocumentCategory);
        //查看单据类型
        getDocumentType(DocumentCategory,prjProject.getBusinessType());
        conContract.setDocumentType(getDocumentType(DocumentCategory,prjProject.getBusinessType()));
        conContract.setDivision(prjProject.getDivision());
        conContract.setBusinessType(prjProject.getBusinessType());
        //获取编码规则
        Map<String, String> params = new HashMap<String, String>();
        conContract.setContractStatus("NEW");
        //暂时用项目名称赋值给合同名称
        conContract.setContractName(prjProject.getProjectName());
        conContract.setContractNumber(fndCodingRuleValuesService.getCodeRuleValue(iRequest, conContract.getDocumentCategory(), conContract.getDocumentType(), conContract.getBusinessType(), params));
        //授权规则字符串设置
        FndCompany fndCompany = new FndCompany();
        fndCompany.setCompany_id(conContract.getCompanyId());
        fndCompany.setCompanyId(conContract.getCompanyId());
        fndCompany = fndCompanyMapper.selectByPrimaryKey(fndCompany);
        conContract.setAuthorityRuleString('"'+fndCompany.getCompany_code()+'"'+"."+'"'+'"'+"."+'"'+'"'+"."+'"'+conContract.getDocumentType()+'"'+"."+'"'+conContract.getBusinessType()+'"'+"."+'"'+conContract.getDivision()+'"'+"."+'"'+'"');

        conContract.setReceivedStatus("NOT");
        conContract.setOverdueStatus("N");
        conContract.setBillingStatus("NOT"); //目前先定为开票，状态为NOT

        //插入合同表
        self().insertSelective(iRequest,conContract);
        Long contractId=conContract.getContractId();
        sysEventService.createEvent(iRequest, contractId, "CON_CONTRACT", "", null, SysEventCodeUtil.PROPERTY_CON_CONTRACT_CREATE);

//        conContractEventService.contractEvent(iRequest,"CON_CONTRACT.CREATE",contractId,iRequest.getUserId());
        //复制现金流
        iConContractCashflowService.saveCashflowFromPrjCashflow(iRequest,contractId,prjProject.getProjectId());
        //复制承租人
        conContractTenantService.saveTenantFromPrj(iRequest,contractId,prjProject.getProjectId());
        //复制担保人
        conContractGuarantorService.saveGuarantorFromPrj(iRequest,contractId,prjProject.getProjectId());
        //复制租赁物
        conContractLeaseItemService.saveLeaseItemFromPrj(iRequest,contractId,prjProject.getProjectId());
        //复制抵制押物
        conContractMortgageService.saveMortgageFromPrj(iRequest,contractId,prjProject.getProjectId());

        //新增一条prj_quotation记录，该记录复制原prj_quotation对应projectId的记录
        //修改source_document_category 为合同
        //修改source_document_id 为合同ID
        if(prjQuotationList.get(0)!=null){
               HlsCusPrjQuotation p =  prjQuotationList.get(0);
               p.setSourceDocumentCategory("CON_CONTRACT");
               p.setSourceDocumentId(contractId);
               HlsCusPrjQuotationDetails prjQuotationDetails = new HlsCusPrjQuotationDetails();
               prjQuotationDetails.setQuotation_id(p.getQuotationId());
               List<HlsCusPrjQuotationDetails> prjQuotationDetailsList = prjQuotationDetailsMapper.queryDetailsById(prjQuotationDetails);
               prjQuotationMapper.insertSelective(p);
               //对于prj_quotation_details也需要新增加一条记录
               if(prjQuotationDetailsList.size() > 0 ){
                   prjQuotationDetailsList.get(0).setQuotation_id(p.getQuotationId());
                   prjQuotationDetailsMapper.insertSelective(prjQuotationDetailsList.get(0));
               }


               //改写合同里面的quotation_id


            conContract.setQuotationId(p.getQuotationId());
            conContract.setObjectVersionNumber((long)2);

//               List<HlsCusConContract> conList = new ArrayList<HlsCusConContract>();


             self().updateByPrimaryKey(iRequest,conContract);
//               conContract.set__status("update");
//               conList.add(conContract);
//               self().batchUpdate(iRequest,conList);
        }
        return  conContract;
    }

    @Override
    public void projectApproved(IRequest requestCtx, HlsCusPrjProject prjProject, String approved) {
        //修改项目状态
        prjProjectService.updatePrjProjectApproved(requestCtx,prjProject,approved);
        //项目创建合同
        HlsCusConContract conContract=self().saveConContractFromPrjProject(requestCtx,prjProject);

    }

    private String getDocumentType(String documentCategory,String  businessType){
        DocumentType  documentType=new DocumentType();
        documentType.setDocumentCategory(documentCategory);
        documentType.setBusinessType(businessType);
        documentType.setEnabledFlag("Y");
        List<DocumentType>  documentTypeList=documentTypeMapper.selectDocumentTypeByBusinessType(documentType);
        if(documentTypeList.size()==0||documentTypeList.size()>1){
            throw new IllegalArgumentException("未找到可匹配的单据类型！");
        }


        return documentTypeList.get(0).getDocumentType();
    }

    @Override
    public HlsCusConContract squerConContractById(Long conContractId) {
        return conContractMapper.squerConContractById(conContractId);
    }
}

