package hls.core.cont.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.dto.ResponseData;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.dto.HlsCusConContractTermination;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.cont.mapper.HlsCusConContractTerminationMapper;
import hls.core.cont.service.ConContractEventService;
import hls.core.cont.service.ConContractTerminationService;
import hls.core.cont.service.IConContractTerminationCustomService;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by yang on 2017/1/6.
 */
@Service
@Transactional
public class ConContractTerminationServiceImpl extends BaseServiceImpl<HlsCusConContractTermination> implements ConContractTerminationService {

    @Autowired
    private ConContractEventService eventService;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private HlsCusConContractTerminationMapper conContractTerminationMapper;
    @Autowired(required = false)
    private IConContractTerminationCustomService iConContractTerminationCustomService;

    @Override
    public ResponseData apply(IRequest iRequest, HlsCusConContractTermination termination) throws Exception {
        if (iConContractTerminationCustomService != null && !iConContractTerminationCustomService.beforeTerminationApply(iRequest, termination)) {
            return new ResponseData(true);
        }
        ResponseData responseData = new ResponseData(true);

        HlsCusConContract conContract = new HlsCusConContract();
        conContract.setContractId(termination.getContractId());
        conContract = conContractMapper.selectByPrimaryKey(conContract);
        databaseLockProvider.lock(conContract);
        //先保存到termination表
        termination.setStatus("NEW");
        HlsCusConContractTermination ct = self().insertSelective(iRequest, termination);

        //eventService.contractEvent(iRequest,"CON_CONTRACT.TERMINATION",termination.getContractId(),iRequest.getUserId());

        // 修改cashflow
/*        Example example = new Example(HlsCusConContractCashflow.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("contract_id",termination.getContractId());
        criteria.andEqualTo("cf_status","RELEASE");

        HlsCusConContractCashflow conContractCashflow = new HlsCusConContractCashflow();
        conContractCashflow.setCf_status("APPROVING");
        cashflowMapper.updateByExampleSelective(conContractCashflow,example);*/

        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);

        List<HlsCusConContractTermination> terminations = new ArrayList<HlsCusConContractTermination>();
        terminations.add(ct);
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "CONTRACT_COMPLETE");
        activitiStartService.start(iRequest, terminations, params);

        HlsCusConContractTermination cct = new HlsCusConContractTermination();
        cct.setContractTerminationId(ct.getContractTerminationId());
        cct.setStatus("APPROVING");
        self().updateByPrimaryKeySelective(iRequest, cct);

        if (iConContractTerminationCustomService != null) {
            iConContractTerminationCustomService.afterTerminationApply(iRequest, termination);
        }
        return responseData;
    }

    @Override
    public List<HlsCusConContractTermination> queryConCompleteByConId(HlsCusConContractTermination conContractTermination) {
        return conContractTerminationMapper.queryConCompleteByConId(conContractTermination);
    }
}
