package hls.core.cont.service.impl;

import com.hand.hap.core.IRequest;
import hls.core.cont.dto.*;
import hls.core.cont.mapper.HlsCusConContractCashflowHMapper;
import hls.core.cont.mapper.HlsCusConContractHMapper;
import hls.core.cont.service.ConContractHService;
import hls.core.cont.service.IConFloatingCalcService;
import hls.core.cont.service.IConFloatingRateReqLnService;
import hls.core.csh.service.ConContractCashflowHService;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.HlsCusHlsDocumentVersionRel;
import hls.core.hls.mapper.HlsCusHlsDocumentVersionRelMapper;
import hls.core.prj.dto.HlsCusHlsDocumentVersion;
import hls.core.prj.mapper.HlsCusHlsDocumentVersionMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by fjm on 2017/10/25.
 */
@Service
public class ConFloatingCalcServiceImpl implements IConFloatingCalcService {

    protected static final String UNIT_AMT_PER_DAY = "UNIT_AMT_PER_DAY";
    public static final String CON_CONTRACT = "con_contract";
    public static final String CON_CONTRACT_H = "con_contract_h";
    public static final String CALCULATED = "CALCULATED";

    @Autowired
    private HlsCusHlsDocumentVersionRelMapper documentVersionRelMapper;

    @Autowired
    private HlsCusHlsDocumentVersionMapper documentVersionMapper;

    @Autowired
    private HlsCusConContractCashflowHMapper cashflowHMapper;

    @Autowired
    private IConFloatingRateReqLnService conFloatingRateReqLnService;

    @Autowired
    private HlsCusConContractHMapper contractHMapper;

    @Autowired
    private ConContractHService contractHService;

    @Autowired
    private ConContractCashflowHService cashflowHService;

    @Override
    public String getFloatingCalcMethod() {
        return this.UNIT_AMT_PER_DAY;
    }

    @Override
    public void conFloatingCalcMethod(IRequest request, List<HlsCusConContractCashflowH> cashflowHs, HlsCusConFloatingRateReqLn changeReq) {

        Long contractId = cashflowHs.stream().findFirst().get().getContractId();
        HlsCusConContractH conContractH = contractHMapper.selectByPrimaryKey(contractId);
        HlsCusConContractCashflowH contractCashflowH = new HlsCusConContractCashflowH();
        contractCashflowH.setContractId(contractId);
        //所有的现金历史流
        List<HlsCusConContractCashflowH> contractCashflowHs = cashflowHMapper.select(contractCashflowH);

        HlsCusHlsDocumentVersion documentVersion = new HlsCusHlsDocumentVersion();
        documentVersion.setSourceDocumentId(changeReq.getFltReqLnId());
        documentVersion.setVersionType(DocumentChangeCommon.CHANGE_REQ);
        documentVersion = documentVersionMapper.selectOne(documentVersion);
        HlsCusHlsDocumentVersionRel documentVersionRel = new HlsCusHlsDocumentVersionRel();
        documentVersionRel.setVersionId(documentVersion.getVersionId());
        List<HlsCusHlsDocumentVersionRel> documentVersionRels = documentVersionRelMapper.select(documentVersionRel);

        Long originContractId = documentVersionRels.stream().filter(item -> CON_CONTRACT.equalsIgnoreCase(item.getFromTable()) && CON_CONTRACT_H.equalsIgnoreCase(item.getToTable())).findFirst().get().getFromPk();
        Double originBaseRate = changeReq.getOldBaseRate();//原利率
        Double newBaseRate = changeReq.getNewBaseRate();//新利率
        Double subRate = CalculateUtil.sub(newBaseRate, originBaseRate);//利率差
        Double subDayRate = CalculateUtil.div(subRate, 360D, 6);//日利率调整差值
        //按期数进行排序
        cashflowHs = cashflowHs.stream().sorted(Comparator.comparingLong(HlsCusConContractCashflowH::getTimes)).collect(Collectors.toList());
        contractCashflowHs = contractCashflowHs.stream().sorted(Comparator.comparingLong(HlsCusConContractCashflowH::getTimes)).collect(Collectors.toList());
        //按期数排序之后的第一个元素
        HlsCusConContractCashflowH cashflowH = cashflowHs.stream().findFirst().get();
        //获取需要修改的现金流所在对应的原现金流集合里的索引
        int i = contractCashflowHs.indexOf(cashflowH);
        HlsCusConContractCashflowH startCashflow = contractCashflowHs.get(i - 1);
        LocalDate startDate, endDate;
        startDate = date2LocalDate(startCashflow.getDueDate());
        Double totalChangeAmount = null;
        Double finaceAmount = conContractH.getFinanceAmount();
        Double leftPrincipal = null;
        //租金的集合
        List<HlsCusConContractCashflowH> collect = contractCashflowHs.stream().filter(item -> item.getCfType() == 1 && item.getCfItem() == 1).collect(Collectors.toList());
        if (collect.contains(startCashflow)) {
            leftPrincipal = finaceAmount;
        } else {
            for (HlsCusConContractCashflowH item : collect) {
                leftPrincipal = CalculateUtil.sub(finaceAmount, item.getPrincipal());
                if (item.getCashflowId().compareTo(startCashflow.getCashflowId()) == 0)
                    break;
            }
        }
        for (int j = 0; j < cashflowHs.size(); j++) {
            HlsCusConContractCashflowH item = cashflowHs.get(j);
            if (j != 0) {
                startDate = date2LocalDate(cashflowHs.get(j - 1).getDueDate());
            }
            endDate = date2LocalDate(item.getDueDate());
            Long betweenDays = ChronoUnit.DAYS.between(startDate, endDate);
            Double mulAmount = CalculateUtil.mul(CalculateUtil.mul(leftPrincipal, subDayRate),betweenDays.doubleValue());
            item.setInterest(CalculateUtil.add(item.getInterest(),mulAmount));
            item.setDueAmount(CalculateUtil.add(item.getDueAmount(),mulAmount));
            cashflowHService.updateByPrimaryKeySelective(request,item);
            totalChangeAmount = CalculateUtil.add(totalChangeAmount,mulAmount);
        }
        conContractH.setBaseRate(newBaseRate);
        contractHService.updateByPrimaryKeySelective(request,conContractH);
        changeReq.setInterestAdjAmount(totalChangeAmount);
        changeReq.setBaseRateAdjRange(subRate);
        changeReq.setStatus(CALCULATED);
        conFloatingRateReqLnService.updateByPrimaryKeySelective(request,changeReq);//更新对应的变更记录
    }


}
