package hls.core.cont.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.AppContextInitListener;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.annotation.StdWho;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.dto.*;
import hls.core.cont.mapper.HlsCusConContractCashflowHMapper;
import hls.core.cont.mapper.HlsCusConContractHMapper;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.cont.mapper.HlsCusConFloatingRateReqLnMapper;
import hls.core.cont.service.*;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.fnd.dto.FndBaseRateSet;
import hls.core.fnd.mapper.FndBaseRateSetMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.HlsCusHlsDocumentVersionRel;
import hls.core.hls.dto.HlsDocumentVersionRel;
import hls.core.hls.mapper.HlsCusHlsDocumentVersionRelMapper;
import hls.core.prj.dto.HlsCusHlsDocumentVersion;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class ConFloatingRateReqLnServiceImpl extends BaseServiceImpl<HlsCusConFloatingRateReqLn> implements IConFloatingRateReqLnService, AppContextInitListener {

    public static final String FLOATING = "FLOATING";//浮动利率类型
    public static final String UN_CALCULATED = "UN_CALCULATED";//未计算
    public static final String CON_CONTRACT = "con_contract";
    public static final String CON_CONTRACT_H = "con_contract_h";
    public static final String ENABLED_FLAG_Y = "Y";
    public static final String PENDING = "PENDING";
    public static final String DOCUMENT_CATEGORY = "CON_FLOATING_RATE_REQ";
    public static final String DOCUMENT_TYPE = "STD";
    public static final String BUSINESS_TYPE = "CON_FLOATING_RATE_REQ";


    private static Map<String, IConFloatingRangeService> conFloatingRangeServiceMap = new HashMap<>();

    private static Map<String, IConFloatingCalcService> conFloatingCalcServiceMap = new HashMap<>();

    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private HlsCusConFloatingRateReqLnMapper mapper;
    @Autowired
    private HlsCusConContractCashflowMapper conContractCashflowMapper;
    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private HlsCusConContractCashflowHMapper contractCashflowHMapper;
    @Autowired
    private HlsCusConContractHMapper conContractHMapper;
    @Autowired
    private IConFloatingRateReqService conFloatingRateReqService;
    @Autowired
    private HlsCusHlsDocumentVersionRelMapper documentVersionRelMapper;
    @Autowired
    private FndBaseRateSetMapper baseRateSetMapper;
    @Autowired
    private ConContractService conContractService;
    @Autowired
    private DocumentChangeCommon documentChangeCommon;
    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;

    /*调息明细页面-调息列表查询*/
    public List<HlsCusConFloatingRateReqLn> rateChangeListQuery(IRequest irequest, HlsCusConFloatingRateReqLn dto, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        return mapper.rateChangeListQuery(dto);
    }

    /*调息明细页面-调息总体信息查询*/
    public HlsCusConFloatingRateReqLn rateChangeTotalQuery(IRequest irequest, HlsCusConFloatingRateReqLn dto) {
        List<HlsCusConFloatingRateReqLn> list = mapper.rateChangeListQuery(dto);
        HlsCusConFloatingRateReqLn result = new HlsCusConFloatingRateReqLn();
        Double amount = 0D;
        for (HlsCusConFloatingRateReqLn item : list) {
            if (item.getInterestAdjAmount() != null) {
                amount = CalculateUtil.add(amount, item.getInterestAdjAmount());
            }
        }
        result.setInterestAdjAmountSum(amount);
        result.setContractCount(list.size());
        return result;
    }

    /**
     * 取消计算
     * @param iRequest
     * @param floatingRateReq
     */
    @Override
    public void cancelFloatingRateReq(IRequest iRequest,@StdWho HlsCusConFloatingRateReq floatingRateReq) {
        HlsCusConFloatingRateReqLn conFloatingRateReqLn = new HlsCusConFloatingRateReqLn();
        conFloatingRateReqLn.setFltReqId(floatingRateReq.getFltReqId());
        List<HlsCusConFloatingRateReqLn> conFloatingRateReqLns = mapper.select(conFloatingRateReqLn);
        conFloatingRateReqLns.forEach(item -> {
            mapper.cancleRateChange(item);
        });

    }

    /**
     * 创建调息申请
     *
     * @param request
     * @param floatingRateReq
     */
    @Override
    public HlsCusConFloatingRateReq createFloatingRateChange(IRequest request, HlsCusConFloatingRateReq floatingRateReq) {
        //插入申请头表
        floatingRateReq.setStatus("NEW");//状态
        floatingRateReq.setFltReqDate(new Date());//日期
        floatingRateReq.setFltReqUserId(request.getUserId());//人员
        floatingRateReq.setDocumentType(DOCUMENT_TYPE);
        floatingRateReq.setDocumentCategory(DOCUMENT_CATEGORY);
        floatingRateReq.setBusinessType(BUSINESS_TYPE);
        Map<String, String> params = new HashMap<String, String>();
        floatingRateReq.setFltReqNumber(fndCodingRuleValuesService.getCodeRuleValue(request, DOCUMENT_CATEGORY, DOCUMENT_TYPE, BUSINESS_TYPE, params));//生成单据编号
        HlsCusConFloatingRateReq hlsCusConFloatingRateReq = conFloatingRateReqService.insertSelective(request, floatingRateReq);
        //获得要调息的合同的集合
        List<HlsCusConContract> floatingRateContracts = conContractMapper.selectFloatingRateContract();
        List<HlsCusConFloatingRateReqLn> floatingRateReqLns = new ArrayList<>();
        //插入调息申请行表
        floatingRateContracts.forEach(item -> {
            HlsCusConFloatingRateReqLn floatingRateReqLn = new HlsCusConFloatingRateReqLn();
            floatingRateReqLn.setContractId(item.getContractId());
            floatingRateReqLn.setFltReqId(hlsCusConFloatingRateReq.getFltReqId());
            floatingRateReqLn.setBaseRateType(item.getBaseRateType());
            floatingRateReqLn.setOldBaseRate(item.getBaseRate());
            floatingRateReqLn.setNewBaseRate(item.getNewBaseRate());
            floatingRateReqLn.setFloatingRangeMethod(item.getFloatingRangeMethod());
            floatingRateReqLn.setStatus(UN_CALCULATED);
            floatingRateReqLn = self().insertSelective(request, floatingRateReqLn);
            floatingRateReqLns.add(floatingRateReqLn);
        });
        //备份行表对应的合同下的合同数据和对应的合同现金流数据
        floatingRateReqLns.forEach(item -> {
            //由于documentChangeCommon接口中不能处理该字段,所以在dto中增加了一个字段,以便于将申请dto转为HlsDocumentVersion类型
            item.setReqUserId(hlsCusConFloatingRateReq.getFltReqUserId());
            HlsCusHlsDocumentVersion hlsDocumentVersion = documentChangeCommon.beforeChangeRequest(request, item);
            Long versionId = hlsDocumentVersion.getVersionId();
            Map param = new HashMap<>();
            Long contractId = item.getContractId();
            HlsCusConContract contract = conContractMapper.selectByPrimaryKey(contractId);
            HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
            cashflow.setContract_id(contractId);
            List<HlsCusConContractCashflow> cashflows = conContractCashflowMapper.select(cashflow);
            HlsCusConContractH conContractH = documentChangeCommon.copyHistoryCommon(request, contract, null, versionId);
            conContractH.setNewBaseRate(item.getNewBaseRate());
            param.put(DocumentChangeCommon.FIX_COLUMN_NAME, "contractId");
            param.put(DocumentChangeCommon.FIX_COLUMN_ID, conContractH.getContractId());
            cashflows.forEach(pro -> {
                documentChangeCommon.copyHistoryCommon(request, pro, param, versionId);
            });
            item.setDocumentVersionId(versionId);
            self().updateByPrimaryKeySelective(request, item);//更新行表上的versionId
        });

        return hlsCusConFloatingRateReq;
    }

    /**
     * 进行调息计算
     *
     * @param request
     * @param cusConFloatingRateReq
     */
    @Override
    public void floatingRate(IRequest request, HlsCusConFloatingRateReq cusConFloatingRateReq) {
        Long fltReqId = cusConFloatingRateReq.getFltReqId();//头ID
        HlsCusConFloatingRateReqLn param = new HlsCusConFloatingRateReqLn();
        param.setFltReqId(fltReqId);
        List<HlsCusConFloatingRateReqLn> floatingChangeLns = mapper.select(param);//找到对应的行
        floatingChangeLns.forEach(item -> {
            Long documentVersionId = item.getDocumentVersionId();//获取行上的versionId
            HlsCusHlsDocumentVersionRel documentVersionRel = new HlsCusHlsDocumentVersionRel();
            documentVersionRel.setVersionId(documentVersionId);
            List<HlsCusHlsDocumentVersionRel> documentVersionRels = documentVersionRelMapper.select(documentVersionRel);//找到对应的关系集合
            //找到关系集合中,合同对应的关系数据
            HlsDocumentVersionRel hlsDocumentVersionRel = documentVersionRels.stream().filter(pro -> CON_CONTRACT.equalsIgnoreCase(pro.getFromTable()) && CON_CONTRACT_H.equalsIgnoreCase(pro.getToTable())).findFirst().get();
            Long toPk = hlsDocumentVersionRel.getToPk();
            HlsCusConContractH conContractH = conContractHMapper.selectByPrimaryKey(toPk);
            HlsCusConContractCashflowH contractCashflowH = new HlsCusConContractCashflowH();
            contractCashflowH.setContractId(toPk);
            FndBaseRateSet fndBaseRateSet = new FndBaseRateSet();
            fndBaseRateSet.setBase_rate_type(conContractH.getBaseRateType());
            fndBaseRateSet.setEnabled_flag(ENABLED_FLAG_Y);
            fndBaseRateSet = baseRateSetMapper.selectOne(fndBaseRateSet);
            Date baseRateDate = fndBaseRateSet.getValid_from();//利率发布日
            //获取备份的历史现金流
            List<HlsCusConContractCashflowH> conContractCashflowHs = contractCashflowHMapper.select(contractCashflowH);
            IConFloatingRangeService iConFloatingRangeService = conFloatingRangeServiceMap.get(item.getFloatingRangeMethod());
            //得到需要进行分组计算的现金流
            List<HlsCusConContractCashflowH> contractCashflowHs = iConFloatingRangeService.rangeCashflow(conContractCashflowHs, baseRateDate);
            IConFloatingCalcService iConFloatingCalcService = conFloatingCalcServiceMap.get(item.getFloatingCalcMethod());
            //重算现金流需要调整的金额,并进行更新操作
            iConFloatingCalcService.conFloatingCalcMethod(request, contractCashflowHs, item);
            HlsCusConContract contract = new HlsCusConContract();
            contract.setContractId(item.getContractId());
            contract.setContractStatus(PENDING);
            //更新合同状态为暂挂
            conContractService.updateByPrimaryKeySelective(request, contract);
        });
    }

    /**
     * 调息工作流入口
     *
     * @param request
     * @param cusConFloatingRateReq
     */
    @Override
    public void floatingRateWorkFlowStart(IRequest request, HlsCusConFloatingRateReq cusConFloatingRateReq) {
        Map param = new HashMap<>();
        param.put("workFlowType", "CON_FLOATING_RATE");
        List<HlsCusConFloatingRateReq> list = new ArrayList<>();
        list.add(cusConFloatingRateReq);
        activitiStartService.start(request, list, param);
    }


    @Override
    public void contextInitialized(ApplicationContext applicationContext) {
        Map<String, IConFloatingRangeService> rangeMap = applicationContext.getBeansOfType(IConFloatingRangeService.class);
        rangeMap.forEach((k, v) -> {
            conFloatingRangeServiceMap.put(v.getFloatingRangeMethod(), v);
        });

        Map<String, IConFloatingCalcService> calcMap = applicationContext.getBeansOfType(IConFloatingCalcService.class);
        calcMap.forEach((k, v) -> {
            conFloatingCalcServiceMap.put(v.getFloatingCalcMethod(), v);
        });

    }
}