package hls.core.csh.components;

import com.hand.hap.core.IRequest;
import hls.core.sys.event.service.SysEventCommonService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.activiti.engine.delegate.DelegateExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by fjm on 2017/11/22.
 */
@Component
public class CshWriteOffLonReceiptCreditEventHandler implements SysEventCommonService {

    public static final String LON_CONTRACT= "LON_CONTRACT";

    @Autowired
    private SysEventService sysEventService;

    @Override
    public String getDocumentCategory() {
        return LON_CONTRACT;
    }

    @Override
    public String getEventCode() {
        return SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_LON_RECEIPT_CREDIT;
    }

    public String getSourceModel() {
        return SysEventCodeUtil.PROPERTITY_SOURCEMODULE_CSH;
    }

    public String getRelation() {
        return SysEventCodeUtil.PROPERTITY_RELATIONTYPE_P2D;
    }

    public Long getLevel() {
        return 2L;
    }

    public String getCurrency(Map<String, Object> params){
        return params.get("currency").toString();
    }

    public String getAmount(Map<String, Object> params){
        return params.get("amount").toString();
    }

    public String getBpName(Map<String, Object> params){
        return params.get("bpName").toString();
    }

    public String getCfName(Map<String, Object> params){
        return params.get("cfName").toString();
    }

    private String getContractName(Map<String, Object> params) {
        return params.get("contractName").toString();
    }

    public String getNoticeTitle() {
        return "融资提款";
    }

    public String getMessage(String contractName, String bpName, String cfName, String currency, String amount) {
        return "已对融资合同[" + contractName +"]"+ "(" + bpName + ")" + cfName + currency + " " + amount;
    }

    @Override
    public void process(IRequest iRequest, Long documentId, String documentCategory, String documentType, Map<String, Object> params) {
        if (params == null) {
            params = new HashMap<>();
        }
        String message = getMessage(getContractName(params), getBpName(params), getCfName(params), getCurrency(params), getAmount(params));
        params.put("message", message);
        params.put("noticeTitle", getNoticeTitle());
        params.put("level", getLevel());
        params.put("noticeType", "NOTICE");

        sysEventService.eventSave(iRequest, documentId, documentCategory, documentType, getSourceModel(), getEventCode(), getRelation(), params);
    }


    @Override
    public void notice(DelegateExecution delegateExecution, String status) {

    }
}
