package hls.core.csh.controllers;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import hls.core.cont.dto.ConAging;
import hls.core.csh.dto.HlsCusConContractCollection;
import hls.core.csh.dto.HlsCusCshPaymentReqHd;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.service.ICshPaymentReqHdService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.csh.dto.HlsCusCshPaymentReqLn;
import hls.core.csh.service.IConContractCashflowService;
import hls.core.fnd.service.FndCodingRuleValuesService;

import static hls.core.gld.components.JeTrxCommonService.map;


@Controller
public class ConContractCashflowController extends BaseController {

    @Autowired
    private IConContractCashflowService cashflowService;
    @Autowired
    private FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private ICshPaymentReqHdService iCshPaymentReqHdService;

    /**
     * 查询所有状态为签约的合同
     *
     * @param request
     * @param cashflow
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/csh/ConContractCashflow/query", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData selectList(HttpServletRequest request, final HlsCusConContractCashflow cashflow,
                                   @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        List<HlsCusConContractCashflow> list = cashflowService.selectAllConContractCashflow(cashflow);
        return new ResponseData(list);
    }


    /**
     * 创建付款申请
     *
     * @param request
     * @param list
     * @return
     */
    @Transactional
    @RequestMapping(value = "/csh/ConContractCashflow/establish", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData establish(HttpServletRequest request, @RequestBody List<HlsCusConContractCashflow> list, HttpSession session) {
        ResponseData data;
        IRequest requestContext = createRequestContext(request);
        Long companyId = ((Long) session.getAttribute("companyId"));

        if (companyId == null) {
            throw new IllegalArgumentException("公司ID获取失败!");
        }
        String value = getCodeValue(requestContext);
        try {
            List<HlsCusCshPaymentReqLn> ss = cashflowService.creatPayInfo(list, companyId, value, requestContext);
            data = new ResponseData(ss);
        } catch (Exception e) {
            data = new ResponseData(false);
            data.setMessage(e.getMessage());
        }
        return data;
    }

    public String getCodeValue(IRequest requestContext) {
        Map<String, String> params = new HashMap<String, String>();
        String value = codingRuleValuesService.getCodeRuleValue(requestContext, "CSH_PAYMENT_REQ", "PAYMENT_REQ", "PAYMENT_REQ", params);
        return value;
    }

    @RequestMapping(value = "/csh/ConContractCashflow/one/query", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData selectOne(HttpServletRequest request, final HlsCusConContractCashflow cashflow,
                                  @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        List<HlsCusConContractCashflow> list = cashflowService.selectOneConContract(cashflow);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/csh/ConContractCashflow/item/Lov/query")
    @ResponseBody
    public ResponseData selectLov(HttpServletRequest request, final HlsCusConContractCashflow cashflow,
                                  @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        List<HlsCusConContractCashflow> list = cashflowService.selectLovItem();
        return new ResponseData(list);
    }

    @RequestMapping(value = "/con/contract/cashflow/item/query")
    @ResponseBody
    public ResponseData contractCashFlowDetail(HttpServletRequest request, final HlsCusConContractCashflow cashflow,
                                               @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                               @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusConContractCashflow> list = cashflowService.contractCashFlowDetail(requestContext, cashflow, page, pagesize);
        return new ResponseData(list);
    }


    /**
     * 查询本周的逾期金额
     *
     * @param request
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/csh/ConContractCashflow/CurrentWeekOverDueAmount/query")//, method = RequestMethod.POST)
    @ResponseBody
    public Double getCurrentWeekOverDueAmount(HttpServletRequest request,
                                              @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        return cashflowService.getCurrentWeekOverDueAmount();
    }

    /**
     * 查询昨天的逾期金额
     *
     * @param request
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/csh/ConContractCashflow/YesterdayOverDueAmount/query")//, method = RequestMethod.POST)
    @ResponseBody
    public Double getYesterdayOverDueAmount(HttpServletRequest request,
                                            @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                            @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        return cashflowService.getYesterdayOverDueAmount();
    }

    /**
     * 查询今天的逾期金额
     *
     * @param request
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/csh/ConContractCashflow/TodayOverDueAmount/query")//, method = RequestMethod.POST)
    @ResponseBody
    public Double getTodayOverDueAmount(HttpServletRequest request,
                                        @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                        @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        return cashflowService.getTodayOverDueAmount();
    }

    /**
     * 查询总计的逾期金额
     *
     * @param request
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/csh/ConContractCashflow/TotalOverDueAmount/query")//, method = RequestMethod.POST)
    @ResponseBody
    public Double getTotalOverDueAmount(HttpServletRequest request,
                                        @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                        @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        return cashflowService.getTotalOverDueAmount();
    }

    /**
     * 查询上一周每一天的逾期金额
     *
     * @param request
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/csh/ConContractCashflow/LastWeekOverDueAmount/query")
    @ResponseBody
    public List<HlsCusConContractCashflow> getLastWeekOverDueAmount(HttpServletRequest request,
                                                                    @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        return cashflowService.getLastWeekOverDueAmount();
    }

    @RequestMapping("/con/contract/vat/query")
    @ResponseBody
    public ResponseData vatQuery(HttpServletRequest request, HlsCusConContractCashflow contractCashflow,
                                 @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) throws ParseException {
        IRequest iRequest = createRequestContext(request);
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        Map<String, Object> map = new HashMap<String, Object>();
        String dateFromStr = contractCashflow.getDateFromStr();
        String dateToStr = contractCashflow.getDateToStr();
        Date dueDateFrom = null;
        Date dueDateTo = null;
        if(dateFromStr != null){
            dueDateFrom = df.parse(dateFromStr);
        }
        if(dateToStr != null){
            dueDateTo = df.parse(dateToStr);
        }
        map.put("contractNumber", contractCashflow.getContract_number());
        map.put("contractName", contractCashflow.getContract_name());
        map.put("timesFrom", contractCashflow.getBankAccountNum());
        map.put("timesTo", contractCashflow.getBankBranchName());
        map.put("dueDateFrom",  dueDateFrom);
        map.put("dueDateTo",dueDateTo);
        String[] cashFlowItems = null;
        String[] status = null;
        if (contractCashflow.getCf_item_list() != null) {
            cashFlowItems = contractCashflow.getCf_item_list().split("、");
        }
        if (contractCashflow.getCf_status() != null) {
            status = contractCashflow.getCf_status().split("、");
        }

        map.put("cashFlowItems", cashFlowItems);
        map.put("status", status);

        List<HlsCusConContractCashflow> maps = cashflowService.vatQuery(iRequest, map, page, pagesize);

        return new ResponseData(maps);
    }

    @RequestMapping("/con/contract/vat/List/query")
    @ResponseBody
    public ResponseData vatQueryList(HttpServletRequest request, @RequestBody Map<String, String> map,
                                     @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                     @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest requestContext = createRequestContext(request);
        map.put("companyId", requestContext.getCompanyId().toString());
        List<HlsCusConContractCashflow> maps = cashflowService.vatQueryList(map);

        return new ResponseData(maps);
    }

    @RequestMapping("/hls/debt/query")
    @ResponseBody
    public ResponseData debtQuery(HttpServletRequest request, @RequestBody Map map,
                                  @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest iRequest = createRequestContext(request);
        List<HlsCusConContractCashflow> lists = cashflowService.debtQuery(iRequest, map, page, pagesize);
        return new ResponseData(lists);
    }

    @RequestMapping("/con/payment/approval")
    @ResponseBody
    public ResponseData paymentApproval(HttpServletRequest httpRequest, @RequestBody List<HlsCusConContractCashflow> list, HttpSession session) {
        ResponseData data = new ResponseData();
        IRequest iRequest = createRequestContext(httpRequest);
        try {
            cashflowService.paymentApproval(iRequest, session, list);
            data = new ResponseData(true);
        } catch (Exception e) {
            data = new ResponseData(false);
            data.setMessage(e.getMessage());
        }
        return data;
    }

    /**
     * 首页-资产管理-租金催收信息查询
     *
     * @param cashflow
     * @return
     * @throws ParseException
     */
    @RequestMapping(value = "/con/conHomepage/selectRentBackInfo")
    @ResponseBody
    public ResponseData selectRentBackInfo(HlsCusConContractCashflow cashflow,
     @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
     @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pageSize, HttpServletRequest request) throws ParseException {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusConContractCashflow> list = cashflowService.selectRentBackInfo(requestContext,cashflow,page,pageSize);
        return new ResponseData(list);
    }

    /**
     * 首页 资产管理 应收金额总计
     *
     * @param cashflow
     * @param session
     * @return
     * @throws ParseException
     * @throws ChangeLimitException
     */
    @RequestMapping(value = "/con/conHomepage/selectDueAmountInDiff/query")
    @ResponseBody
    public HlsCusConContractCashflow selectDueAmountInDiff(@RequestBody HlsCusConContractCashflow cashflow, HttpSession session)
            throws ParseException, ChangeLimitException {

        cashflow = cashflowService.selectDueAmountInDiff(cashflow, session);
        return cashflow;
    }

    /**
     * 首页 资产管理 阶段内到期应收金额总计
     *
     * @return
     */
    @RequestMapping(value = "/con/conHomepage/DueAmount/query")
    @ResponseBody
    public ConAging selectDueAmount(@RequestBody HlsCusConContractCashflow cashflow) throws ParseException {
        ConAging conAging = cashflowService.SelectDueAmount(cashflow);
        return conAging;
    }

    @RequestMapping(value = "/con/ast/detail/query")
    @ResponseBody
    public ResponseData astDetail(Long contract_id) {

        List<HlsCusConContractCashflow> list = cashflowService.astDetail(contract_id);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/con/contractcoll/selectHisOfRentBackIf")
    @ResponseBody
    public ResponseData selectHisOfRentBackIf(HttpServletRequest request,
                                              @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pageSize) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusConContractCollection> list = cashflowService.selectHisOfRentBackIf(requestContext, page, pageSize);
        return new ResponseData(list);
    }
    /*public List<HlsCusConContractCollection> selectHisOfRentBackIf() {

        List<HlsCusConContractCollection> clist = cashflowService.selectHisOfRentBackIf();
        return clist;
    }*/

    @RequestMapping(value = "/con/apply/detail/query")
    @ResponseBody
    public ResponseData queryConApply(Long contractId) {
        List<HlsCusCshPaymentReqHd> list = iCshPaymentReqHdService.queryApply(contractId);
        return new ResponseData(list);
    }

    /*合同变更-提前结清- 现金流grid信息查询*/
    @RequestMapping(value = "/con/contract/change/cashflow/queryCashFlowInfo")
    @ResponseBody
    public ResponseData queryReceivedWriteOff(HttpServletRequest request, final HlsCusConContractCashflow cashflow,
                                              @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest requestContext = createRequestContext(request);
        Map map = cashflowService.queryCashFlowInfo(requestContext,cashflow,page,pagesize);
        ResponseData rs = new ResponseData((List<?>) map.get("list"));
        rs.setTotal((Long) map.get("total"));
        return rs;
    }
    /*合同变更 -提前结清- 结清信息查询*/
    @RequestMapping(value = "/con/contract/change/cashflow/settleInformation")
    @ResponseBody
    public Map settleInformation(HttpServletRequest request, final HlsCusConContractCashflow cashflow,
                                 @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest requestContext = createRequestContext(request);
        return cashflowService.settleInformation(requestContext,cashflow);
    }
    /*合同变更-提前结清-获取审批中或者审批通过或者被审批拒绝的结清信息*/
    @RequestMapping(value = "/con/contract/change/cashflow/settleInformationByStatus")
    @ResponseBody
    public Map settleInformationByStatus(HttpServletRequest request, final HlsCusConContractCashflow cashflow,
                                 @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest requestContext = createRequestContext(request);
        return cashflowService.settleInformationByStatus(requestContext,cashflow);
    }
}
