package hls.core.csh.controllers;

import java.text.SimpleDateFormat;
import java.util.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import hls.core.csh.dto.HlsCusConContractCollection;
import hls.core.csh.dto.HlsCusCshConContract;
import hls.core.csh.service.ConContractCollectionService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.csh.service.ICshConContractService;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.bp.dto.HlsCusBpMasterBankAccount;


@Controller
public class CshConContractControllers extends BaseController {

    @Autowired
    private ICshConContractService conContractService;
    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private ConContractCollectionService conContractCollectionService;

    /**
     * @param request
     * @param conContract
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/csh/ConContract/query")
    @ResponseBody
    public ResponseData selectList(HttpServletRequest request, final HlsCusCshConContract conContract,
                                   @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        List<HlsCusCshConContract> list = conContractService.selectAllConContract(conContract);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/csh/contract/detail/query")
    @ResponseBody
    public ResponseData contractDetail(HttpServletRequest request, final HlsCusCshConContract conContract,
                                       @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(conContractService.contractDetail(requestContext, conContract, page, pagesize));
    }

    @RequestMapping(value = "/csh/ConContract/bank/query", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData selectLists(HttpServletRequest request, final HlsCusCshConContract conContract,
                                    @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        List<HlsCusBpMasterBankAccount> list = conContractService.selectBank(conContract);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/csh/ConContract/queryContract")
    @ResponseBody
    public ResponseData getContracts(HttpServletRequest request, final HlsCusCshConContract conContract,
                                     @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                     @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest iRequest = createRequestContext(request);
        List<HlsCusCshConContract> list = conContractService.select(iRequest, conContract, page, pagesize);
        return new ResponseData(list);
    }

    /**
     * 合同明细保存
     *
     * @param cshConContract
     * @param result
     * @param request
     * @param session
     * @return
     */
    @RequestMapping(value = "/con/contract/detail/submit")
    @ResponseBody
    public ResponseData submit(@RequestBody HlsCusCshConContract cshConContract, final BindingResult result, final HttpServletRequest request, HttpSession session) {
        ResponseData rd = new ResponseData();
        IRequest requestContext = createRequestContext(request);

        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        try{
            conContractService.save(requestContext, cshConContract);
        }catch (Exception e){
            rd = new ResponseData(false);
            rd.setMessage("租赁物信息不可重复！");
        }
        return rd;
    }

    @RequestMapping(value = "/csh/contract/amount/query")
    @ResponseBody
    public Map<String, Double> contractAmountQuery() {

        return conContractService.contractAmountQuery();
    }

    @RequestMapping(value = "/csh/contract/totalCount/query")
    @ResponseBody
    public List<HlsCusCshConContract> getTotalContractCount() {
        List<HlsCusCshConContract> conContract = conContractService.getTotalContractCount();
        return conContract;
    }

    @RequestMapping(value = "/csh/contract/diffContractStatusCount/query")
    @ResponseBody
    public List<HlsCusCshConContract> getDiffContractStatusCount() {
        List<HlsCusCshConContract> conContract = conContractService.getDiffContractStatusCount();
        return conContract;
    }

    @RequestMapping(value = "/csh/contract/terminateCount/query")
    @ResponseBody
    public List<HlsCusCshConContract> getTerminateContractCount() {
        List<HlsCusCshConContract> conContract = conContractService.getTerminateContractCount();
        return conContract;
    }

    @RequestMapping(value = "/csh/contract/stockCountAndMonthNewCount/query")
    @ResponseBody
    public List<HlsCusCshConContract> getStockContractAndMonthNewContract() {
        List<HlsCusCshConContract> conContract = conContractService.getStockContractAndMonthNewContract();
        return conContract;
    }

    @RequestMapping(value = "/csh/contract/indexContractInceptInfo/query")
    @ResponseBody
    public ResponseData getIndexContractInceptInfo(HttpServletRequest request, final HlsCusCshConContract conContract,
                                                   @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest iRequest = createRequestContext(request);
        List<HlsCusCshConContract> list = conContractService.getIndexContractInceptInfo(iRequest, conContract, page, pagesize);
        return new ResponseData(list);
    }
    /*public List<HlsCusCshConContract> getIndexContractInceptInfo(){
		List<HlsCusCshConContract> conContract = conContractService.getIndexContractInceptInfo();
		return conContract;
	}*/

	

/*@RequestMapping(value = "/csh/contract/code/value/query")
@ResponseBody
public String getCodeValue(HttpServletRequest request,final HttpSession session) {
	IRequest requestContext = createRequestContext(request);
	Map<String,String> params = new HashMap<String,String>();
	params.put("SYS_06", "sys06");
	String value = codingRuleValuesService.getCodeRuleValue(requestContext,8L, params);
		
	return  value;
}*/

    @RequestMapping(value = "/csh/contract/due/amount/query")
    @ResponseBody
    public List<HlsCusCshConContract> contractDueAmountQuery() {

        return conContractService.contractDueAmountQuery();
    }

    @RequestMapping(value = "/csh/contract/home/first/query")
    @ResponseBody
    public ResponseData contractHomeFirstQuery(HttpServletRequest request, @RequestBody Map<String, Object> map,
                                               @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                               @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpSession session) {
        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        map.put("companyId", companyId);
        return new ResponseData(conContractService.contractHomeFirstQuery(map));
    }

    @RequestMapping(value = "/csh/contract/home/secord/query")
    @ResponseBody
    public ResponseData contractHomeSecordQuery(HttpServletRequest request, @RequestBody Map<String, Object> map,
                                                @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                                @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpSession session) {
        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        map.put("companyId", companyId);
        return new ResponseData(conContractService.contractHomeSecordQuery(map));
    }

    @RequestMapping(value = "/csh/contract/home/third/query")
    @ResponseBody
    public ResponseData contractHomeThirdQuery(HttpServletRequest request, @RequestBody Map<String, Object> map,
                                               @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                               @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpSession session) {

        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        map.put("companyId", companyId);
        return new ResponseData(conContractService.contractHomeThirdQuery(map));
    }

    @RequestMapping(value = "/csh/contract/selectContractByID/query")
    @ResponseBody
    public List<HlsCusCshConContract> selectContractByID(HlsCusCshConContract cshConContract, HttpSession session) {
        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        cshConContract.setCompany_id(companyId);
        List<HlsCusCshConContract> conContract = conContractService.selectContractByID(cshConContract);
        return conContract;
    }

    @RequestMapping(value = "/csh/ConContract/comprehensive/query", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData comprehensiveQuery(HttpServletRequest request, final HlsCusCshConContract conContract,
                                           @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                           @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {

        Map<String, Object> map = new HashMap<String, Object>();

        map.put("contractNumber", conContract.getContract_number());
        map.put("contractName", conContract.getContract_name());
        map.put("bpCode", conContract.getBp_number());
        map.put("bpName", conContract.getBp_name());

        String[] statusInfo = null;
        String[] businessTypeInfo = null;
        String[] divisionInfo = null;
        if (conContract.getStatus() != null) {
            statusInfo = conContract.getStatus().split("、");
        }
        if (conContract.getBusiness_type() != null) {
            businessTypeInfo = conContract.getBusiness_type().split("、");
        }
        if (conContract.getDivision() != null) {
            divisionInfo = conContract.getDivision().split("、");
        }


        map.put("statusInfo", statusInfo);
        map.put("businessTypeInfo", businessTypeInfo);
        map.put("divisionInfo", divisionInfo);

        if (conContract.getNetFinanceAmount() == null && conContract.getVatFinanceAmount() == null) {
            if (conContract.getDocument_category() != null) {
                String[] financeAmountInfo = conContract.getDocument_category().split("、");

                if (financeAmountInfo.length == 1) {
                    if ("1000".equals(financeAmountInfo[0])) {
                        map.put("financeAmountMax", null);
                        map.put("financeAmountFrom", null);
                        map.put("financeAmountTo", 1000000d);
                    } else if ("5000".equals(financeAmountInfo[0])) {
                        map.put("financeAmountMax", null);
                        map.put("financeAmountFrom", 1000000d);
                        map.put("financeAmountTo", 5000000d);
                    } else {
                        map.put("financeAmountMax", null);
                        map.put("financeAmountFrom", 5000000d);
                        map.put("financeAmountTo", 10000000d);
                    }
                } else if (financeAmountInfo.length == 2) {
                    if ("1000".equals(financeAmountInfo[0]) && "5000".equals(financeAmountInfo[1])) {
                        map.put("financeAmountMax", null);
                        map.put("financeAmountFrom", null);
                        map.put("financeAmountTo", 5000000d);
                    } else if ("5000".equals(financeAmountInfo[0]) && "10000".equals(financeAmountInfo[1])) {
                        map.put("financeAmountMax", null);
                        map.put("financeAmountFrom", 1000000d);
                        map.put("financeAmountTo", 10000000d);
                    } else {
                        map.put("financeAmountMax", 1000000d);
                        map.put("financeAmountFrom", 5000000d);
                        map.put("financeAmountTo", 10000000d);
                    }
                } else {
                    map.put("financeAmountMax", null);
                    map.put("financeAmountFrom", null);
                    map.put("financeAmountTo", 10000000d);
                }
            }
        } else {
            map.put("financeAmountMax", null);
            map.put("financeAmountFrom", conContract.getNetFinanceAmount());
            map.put("financeAmountTo", conContract.getVatFinanceAmount());
        }


        List<HlsCusCshConContract> list = conContractService.comprehensiveQuery(map, page, pagesize);

        return new ResponseData(list);
    }

    @RequestMapping("/hls/con/receivableCollection")
    @ResponseBody
    public ResponseData receivableCollection(HlsCusBpMaster bpMaster, HttpServletRequest request) {

        return new ResponseData(conContractService.receivableCollection(bpMaster));
    }

    @RequestMapping(value = "/csh/ConContract/rent/query", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData rentQuery(HttpServletRequest request, final HlsCusCshConContract conContract,
                                  @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        Map<String, Object> map = new HashMap<String, Object>();

        map.put("contractNumber", conContract.getContract_number());
        map.put("contractName", conContract.getContract_name());
        map.put("bpNumber", conContract.getBp_number());
        map.put("bpName", conContract.getBp_name());
        map.put("cardNumber", conContract.getBpBankAccountNum());
        map.put("cardName", conContract.getBpBankAccountName());

        String[] receiveDateInfo = null;
        String[] overdueInfo = null;
        String[] moneyProjectInfo = null;
        String[] statusInfo = null;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar c = Calendar.getInstance();
        c.add(Calendar.DATE, 7);
        String weekDate = sdf.format(c.getTime());
        Calendar cd = Calendar.getInstance();
        cd.add(Calendar.DATE, 30);
        String monDate = sdf.format(cd.getTime());
        String today = sdf.format(new Date());
        if (conContract.getReceiveDateFrom() == null && conContract.getReceiveDateTo() == null) {
            if (conContract.getReceivedStatus() != null) {
                receiveDateInfo = conContract.getReceivedStatus().split("、");
                if (receiveDateInfo.length == 1) {
                    if ("0".equals(receiveDateInfo[0])) {
                        map.put("receiveDate", null);
                        map.put("receiveDateFrom", null);
                        map.put("receiveDateTo", today);
                    } else if ("7".equals(receiveDateInfo[0])) {
                        map.put("receiveDate", null);
                        map.put("receiveDateFrom", today);
                        map.put("receiveDateTo", weekDate);
                    } else {
                        map.put("receiveDate", null);
                        map.put("receiveDateFrom", today);
                        map.put("receiveDateTo", monDate);
                    }
                } else if (receiveDateInfo.length == 2) {
                    if ("0".equals(receiveDateInfo[0]) && "7".equals(receiveDateInfo[1])) {
                        map.put("receiveDate", null);
                        map.put("receiveDateFrom", null);
                        map.put("receiveDateTo", weekDate);
                    } else if ("7".equals(receiveDateInfo[0]) && "30".equals(receiveDateInfo[1])) {
                        map.put("receiveDate", null);
                        map.put("receiveDateFrom", today);
                        map.put("receiveDateTo", monDate);
                    } else {
                        map.put("receiveDate", null);
                        map.put("receiveDateFrom", null);
                        map.put("receiveDateTo", monDate);
                    }
                } else {
                    map.put("receiveDate", null);
                    map.put("receiveDateFrom", null);
                    map.put("receiveDateTo", monDate);
                }
            }
        } else {
            map.put("receiveDate", null);
            map.put("receiveDateFrom", conContract.getReceiveDateFrom());
            map.put("receiveDateTo", conContract.getReceiveDateTo());
        }

        if (conContract.getOverdueFrom() == null && conContract.getOverdueTo() == null) {
            if (conContract.getOverdueStatus() != null) {
                overdueInfo = conContract.getOverdueStatus().split("、");
                boolean flag = Arrays.asList(overdueInfo).contains("不限");
                if (!flag) {
                    map.put("overdueInfo", overdueInfo);
                }
            } else {
                map.put("overdueInfo", null);
                map.put("overdueFrom", conContract.getOverdueFrom());
                map.put("overdueTo", conContract.getOverdueTo());
            }
        } else {
            map.put("overdueInfo", conContract.getOverdueStatus());
            map.put("overdueFrom", conContract.getOverdueFrom());
            map.put("overdueTo", conContract.getOverdueTo());
        }

        if (conContract.getCf_item() != null) {
            moneyProjectInfo = conContract.getCf_item().split("、");
            boolean flag = Arrays.asList(moneyProjectInfo).contains("不限");
            if (!flag) {
                map.put("moneyProjectInfo", moneyProjectInfo);
            }
        }
        if (conContract.getWriteOffFlag() != null) {
            statusInfo = conContract.getWriteOffFlag().split("、");
            boolean flag = Arrays.asList(statusInfo).contains("不限");
            if (!flag) {
                map.put("statusInfo", statusInfo);
            }
        }
        List<HlsCusCshConContract> list = conContractService.rentQuery(map, page, pagesize);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/csh/ConContract/rentRecord/query")
    public ResponseData rentRecordQuery(HttpServletRequest request, Long contractId,
                                        @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                        @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        List<HlsCusConContractCollection> list = conContractCollectionService.rentRecordQuery(contractId, page, pagesize);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/csh/ConContract/rentRecord/submit")
    @ResponseBody
    public ResponseData submit(@RequestBody final List<HlsCusConContractCollection> conContractCollection, final BindingResult result,
                               final HttpServletRequest request) {
        getValidator().validate(conContractCollection, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(conContractCollectionService.batchUpdate(requestContext, conContractCollection));
    }

    @RequestMapping(value = "/csh/ConContract/rentRecord/remove")
    @ResponseBody
    public ResponseData remove(@RequestBody List<HlsCusConContractCollection> conContractCollection) {
        conContractCollectionService.batchDelete(conContractCollection);
        return new ResponseData();
    }

}
