package hls.core.csh.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.csh.dto.HlsCusCshDeduction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.exception.WriteOffTypeNullException;
import hls.core.csh.service.CshDeductionService;
import hls.core.csh.service.CshWriteOffService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by FJM on 2016/12/5.
 */
@Controller
public class CshDeductionController extends BaseController {

    @Autowired
    private CshDeductionService cshDeductionService;

    @Autowired
    private CshWriteOffService cshWriteOffService;

    @InitBinder
    protected void initBinder(WebDataBinder binder) {
        binder.registerCustomEditor(Date.class, new CustomDateEditor(new SimpleDateFormat("yyyy/MM/dd"), true));
    }

    @RequestMapping(value = "/csh/cshDeduction")
    @ResponseBody
    public ResponseData queryDeduction() {
        return new ResponseData();
    }

    @RequestMapping(value = "/csh/addCshDeduction")
    @ResponseBody
    public ResponseData addDeduction(HttpServletRequest request, @RequestBody List<HlsCusCshWriteOff> cshwriteoff, HttpSession session) throws ChangeLimitException {
        ResponseData rd = null;
        try {
//                List<HlsCusCshWriteOff> data = cshDeductionService.addCshDeduction(cshwriteoff);
//                rd = new ResponseData(data);
            cshWriteOffService.updateWriteOff(createRequestContext(request), cshwriteoff, session);
            rd = new ResponseData(cshwriteoff);
        } catch (IllegalArgumentException e) {
            rd = new ResponseData(false);
            rd.setMessage(e.getMessage());
            rd.setCode(e.getMessage());
        } catch (BeyondAmountLimitException e) {
            rd = new ResponseData(false);
            rd.setMessage(e.getMessage());
            rd.setCode(e.getMessage());

        } catch (WriteOffTypeNullException e) {
            rd = new ResponseData(false);
            rd.setMessage(e.getMessage());
            rd.setCode(e.getMessage());
        }
        return rd;
    }

    @RequestMapping(value = "/csh/deductionLov")
    @ResponseBody
    public ResponseData queryLov(HlsCusCshDeduction cshDeduction) {
        List<HlsCusCshDeduction> data = cshDeductionService.queryLov(cshDeduction);
        return new ResponseData(data);
    }

    @RequestMapping(value = "/csh/homePageDepositMgr")
    @ResponseBody
    public ResponseData homePageDepositMgr(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                           @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        //  List<Map> data = cshDeductionService.
        IRequest iRequest = createRequestContext(request);
        long companyId = request.getSession().getAttribute("companyId") != null ? (long) request.getSession().getAttribute("companyId") : 0;
        map.put("companyId", companyId);
        List<Map> data = cshDeductionService.homePageDepositMgr(map, page, pagesize, iRequest);
        return new ResponseData(data);
    }

    @RequestMapping(value = "/csh/homePagePoolDepositMgr")
    @ResponseBody
    public ResponseData homePagePoolDepositMgr(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                               @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        long companyId = request.getSession().getAttribute("companyId") != null ? (long) request.getSession().getAttribute("companyId") : 0;
        map.put("companyId", companyId);
        List<Map> data = cshDeductionService.homePagePoolDepositMgr(map, page, pagesize, iRequest);
        return new ResponseData(data);
    }


    @RequestMapping(value = "/csh/homePageMgr")
    @ResponseBody
    public ResponseData homePageMgr(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) throws ParseException {
        IRequest iRequest = createRequestContext(request);
        long companyId = request.getSession().getAttribute("companyId") != null ? (long) request.getSession().getAttribute("companyId") : 0;
        map.put("companyId", companyId);
        List<Map> data = cshDeductionService.homePageMgr(map, page, pagesize, iRequest);
        return new ResponseData(data);
    }

    @RequestMapping(value = "/csh/homePagePoolMgr")
    @ResponseBody
    public ResponseData homePagePoolMgr(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,

                                        @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) throws ParseException {
        IRequest iRequest = createRequestContext(request);
        long companyId = request.getSession().getAttribute("companyId") != null ? (long) request.getSession().getAttribute("companyId") : 0;
        map.put("companyId", companyId);
        List<Map> data = cshDeductionService.homePagePoolMgr(map, page, pagesize, iRequest);
        return new ResponseData(data);
    }

    @RequestMapping(value = "/csh/queryDepositPoolByBp")
    @ResponseBody
    public Map queryDepositPoolByBp(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        long companyId = request.getSession().getAttribute("companyId") != null ? (long) request.getSession().getAttribute("companyId") : 0;
        map.put("companyId", companyId);
        IRequest iRequest = createRequestContext(request);
        return cshDeductionService.queryDepositPoolByBp(map, iRequest);
    }


    @RequestMapping(value = "/csh/queryDepositPoolByCon")
    @ResponseBody
    public Map queryDepositPoolByCon(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                     @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        long companyId = request.getSession().getAttribute("companyId") != null ? (long) request.getSession().getAttribute("companyId") : 0;
        map.put("companyId", companyId);
        IRequest iRequest = createRequestContext(request);
        return cshDeductionService.queryDepositPoolByCon(map, iRequest);
    }


    @RequestMapping(value = "/csh/homePageMgrWriteOff")
    @ResponseBody
    public Map homePageMgrWriteOff(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) throws ParseException {
        IRequest iRequest = createRequestContext(request);
        long companyId = iRequest.getCompanyId();
        map.put("companyId", companyId);
        return cshDeductionService.homePageMgrWriteOff(map, page, pagesize, iRequest);
    }

    @RequestMapping(value = "/csh/homePagePoolMgrWriteOff")
    @ResponseBody
    public Map homePagePoolMgrWriteOff(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,

                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) throws ParseException {
        IRequest iRequest = createRequestContext(request);
        long companyId = iRequest.getCompanyId();
        map.put("companyId", companyId);

        return cshDeductionService.homePagePoolMgrWriteOff(map, page, pagesize, iRequest);
    }

}
