package hls.core.csh.controllers;

import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.csh.dto.HlsCusCshDeduction;
import hls.core.csh.service.IConContractCashflowService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.InitBinder;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * Created by yang on 2016/12/1.
 */
@Controller
@RequestMapping("/csh/deposit")
public class CshDepositMgrController extends BaseController {

    @Autowired
    private IConContractCashflowService service;
    private static Logger logger = LoggerFactory.getLogger(CshDepositMgrController.class);

    @InitBinder
    protected void initBinder(WebDataBinder binder){
        binder.registerCustomEditor(Date.class,new CustomDateEditor(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"),true));
    }

    @RequestMapping("/query")
    public ResponseData contractCashDepositQuery(@RequestParam(defaultValue = DEFAULT_PAGE_SIZE)int pageSize,
                                                 @RequestParam(defaultValue = DEFAULT_PAGE)int page,
                                                 final HttpServletRequest request,
                                                 final HttpServletResponse response){
        try {
            List<Map> maps = service.query4depositMgr(page,pageSize);
            return new ResponseData(maps);
        }catch (Exception e){
            logger.error(e.getMessage());
        }
        return new ResponseData(false);
    }

    @RequestMapping("/detail/query")
    public ResponseData CashDepositDetailQuery(@RequestParam Long contractId,
                                               @RequestParam Long bpId,
                                               @RequestParam(defaultValue = DEFAULT_PAGE_SIZE)int pageSize,
                                               @RequestParam(defaultValue = DEFAULT_PAGE)int page,
                                               final HttpServletRequest request,
                                               final HttpServletResponse response){
        ResponseData responseData = null;
        try{
            List<Map> maps = null;
            if(bpId != -1){
                maps = service.queryPoolDetail(bpId,page,pageSize);
            }else {
                maps = service.queryDepositDetail(contractId, page, pageSize);
            }
            responseData = new ResponseData(maps);
        }catch (Exception e){
            logger.error(e.getMessage());
            responseData = new ResponseData(false);
            responseData.setMessage("查询出现问题，请联系管理员。");
        }
        return responseData;
    }

    @RequestMapping("/pool/query")
    public ResponseData poolCashDepositQuery(@RequestParam(defaultValue = DEFAULT_PAGE_SIZE)int pageSize,
                                             @RequestParam(defaultValue = DEFAULT_PAGE)int page,
                                             final HttpServletRequest request,
                                             final HttpServletResponse response){
        ResponseData responseData = null;
        try{
            List<Map> maps = service.query4PoolDepositMgr(page,pageSize);
            responseData = new ResponseData(maps);
        }catch (Exception e){
            logger.error(e.getMessage());
            responseData = new ResponseData(false);
            responseData.setMessage("查询出现问题，请联系管理员。");
        }
        return responseData;
    }

    @RequestMapping("/recall/query")
    public ResponseData queryRecall(HlsCusCshDeduction deduction,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE)int pageSize,
                                    @RequestParam(defaultValue = DEFAULT_PAGE)int page,
                                    final HttpServletRequest request,
                                    final HttpServletResponse response){
        ResponseData responseData = null;
        try{
            List<Map> maps = service.query4Recall(deduction, page, pageSize);
            responseData = new ResponseData(maps);
        }catch (Exception e){
            logger.error(e.getMessage());
            responseData = new ResponseData(false);
            responseData.setMessage("查询出现问题，请联系管理员。");
        }
        return responseData;
    }

    @RequestMapping("/back/query")
    public ResponseData queryBack(HlsCusCshDeduction deduction,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE)int pageSize,
                                  @RequestParam(defaultValue = DEFAULT_PAGE)int page){
        ResponseData responseData = null;
        try{
            List<Map> maps = service.query4Back(deduction, page, pageSize);
            responseData = new ResponseData(maps);
        }catch (Exception e){
            logger.error(e.getMessage());
            responseData = new ResponseData(false);
            responseData.setMessage("查询出现问题，请联系管理员。");
        }
        return responseData;
    }

}
