package hls.core.csh.controllers;

import java.util.*;

import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.exception.ChangeLimitException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import hls.core.csh.exception.WriteOffTypeNullException;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.service.CshTransactionService;


@Controller
public class CshTransactionController extends BaseController {
	@Autowired
	private CshTransactionService service;
	@Autowired
	private HlsCusBpMasterMapper bpMasterMapper;

	/**
	 * 反冲查询
	 * @param cshTransaction
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/query")
	@ResponseBody
	public ResponseData query(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.queryAll(requestContext, cshTransaction, page, pagesize));
	}
	
	/**
	 * 明细查询
	 * @param transactionId
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/detail/query")
	@ResponseBody
	public ResponseData queryDetailById(Long transactionId){
		return new ResponseData(service.queryDetailById(transactionId));
	}
	/**
	 * 收款新增
	 * @param cshTransaction
	 * @param result
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody HlsCusCshTransaction cshTransaction, final BindingResult result, final HttpServletRequest request, final HttpSession session){
	        if (result.hasErrors()) {
	            ResponseData responseData = new ResponseData(false);
	            responseData.setMessage(getErrorMessage(result, request));
	            return responseData;
	        }
	        Long companyId =  (Long) session.getAttribute("companyId");
	        IRequest requestContext = createRequestContext(request);
	        HlsCusCshTransaction csh = service.updateCshTransaction(companyId, cshTransaction,requestContext);
	        List<HlsCusCshTransaction> CshTransactions = new ArrayList<HlsCusCshTransaction>();
	        CshTransactions.add(csh);
	        return new ResponseData(CshTransactions);
	}
	
	/**
	 * 反冲
	 * @param cshTransaction
	 * @param result
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/reversed/submit")
	@ResponseBody
	public ResponseData reversedSubmit(@RequestBody HlsCusCshTransaction cshTransaction, final BindingResult result, final HttpServletRequest request,
                                       final HttpSession session) {
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
		ResponseData rd = null;
        if(cshTransaction.getWriteOffFlag() == "FULL" || cshTransaction.getWriteOffFlag() =="PARTIAL"){
			rd = new ResponseData(false);
			rd.setMessage("该笔收款已有核销记录，<br />请先将核销记录反冲");
			return rd;
		}
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        service.updateReversed(requestContext,companyId,cshTransaction);
        return new ResponseData();
        
	}
	
	/**
	 * 退款查询
	 * @param cshTransaction
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/return/query")
	@ResponseBody
	public ResponseData cshReturnQuery(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request)
	{
		IRequest requestContext = createRequestContext(request);
		
		return new ResponseData(service.cshReturnQuery(requestContext, cshTransaction, page, pagesize));
	}
	
	/**
	 * 收款退款反冲
	 * @param cshTransaction
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/return/reversed/query")
	@ResponseBody
	public ResponseData cshReturnReversedQuery(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                               @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request)
	{
		IRequest requestContext = createRequestContext(request);
		
		return new ResponseData(service.cshReturnReversedQuery(requestContext, cshTransaction, page, pagesize));
	}
	
	/**
	 * 退款
	 * @param cshTransaction
	 * @param result
	 * @param request
	 * @return
	 * @throws BeyondAmountLimitException 
	 */
	@RequestMapping(value = "/csh/cshTransaction/return/submit")
	@ResponseBody
	public ResponseData returnSubmit(@RequestBody HlsCusCshTransaction cshTransaction, final BindingResult result, final HttpServletRequest request,
                                     final HttpSession session) throws BeyondAmountLimitException {
		IRequest requestContext = createRequestContext(request);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        Long companyId =  (Long) session.getAttribute("companyId");
        service.updateReturn(requestContext,companyId,cshTransaction);
        return new ResponseData();
	}
	
	/**
	 * 核销查询
	 * @param cshTransaction
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/writeOff/query")
	@ResponseBody
	public ResponseData cshWriteOffQuery(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                         @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
		IRequest requestContext = createRequestContext(request);
		List<HlsCusCshTransaction>  datas = service.cshWriteOffQuery(requestContext, cshTransaction, page, pagesize);
		return new ResponseData(datas);
	}
	
	
	/**
	 * 支付反冲查询
	 * @param cshTransaction
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/paymentreversed/query")
	@ResponseBody
	public ResponseData paymentreversedRquery(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		List<HlsCusCshTransaction> list = service.paymentreversedRquery(requestContext, cshTransaction, page, pagesize);
		return new ResponseData(list);
	}
	/**
	 * 反冲查询中付款对象下拉列表
	 * @param cshTransaction
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/paymentreversed/lov/query")
	@ResponseBody
	public ResponseData paymentreversedRqueryLov(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		List<HlsCusCshTransaction> list = service.paymentreversedRqueryLov();
		return new ResponseData(list);
	}

    /*
     资金管理-收款管理首页的6个小模块的值
     */
    @RequestMapping("/csh/cshTransaction/fundHomePage")
    @ResponseBody
    public HlsCusCshTransaction fundHomePage(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                             @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
		IRequest iRequest = createRequestContext(request);
		long companyId = request.getSession().getAttribute("companyId")!=null? (long) request.getSession().getAttribute("companyId") :0;
		map.put("companyId",companyId);
        return service.fundHomePage(iRequest, map, page, pagesize);
    }



	/*
	 资金管理-收款管理首页收款明细
	 */
	@RequestMapping("/csh/cshTransaction/collectionDetails")
	@ResponseBody
	public ResponseData collectionDetails(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
										  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		long companyId = request.getSession().getAttribute("companyId")!=null? (long) request.getSession().getAttribute("companyId") :0;
		map.put("companyId",companyId);
		return new ResponseData(service.collectionDetails(iRequest,map,page,pagesize));
	}
    /*
    * 提前结清-审批页面-lov查询收款信息*/
	@RequestMapping("/con/termination/queryCollectionInfo")
	public ResponseData queryCollectionInfo(@RequestParam(required = false)Long bpId, @RequestParam(required = false)Date transactionDateFrom, @RequestParam(required = false)Date transactionDateTo, @RequestParam(required = false)Double amountFrom, @RequestParam(required = false)Double amountTo, @RequestParam(required = false)String bpNameCondition, HttpServletRequest request, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
											@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize){
		Map map = new HashMap();
		IRequest iRequest = createRequestContext(request);
		long companyId = request.getSession().getAttribute("companyId")!=null? (long) request.getSession().getAttribute("companyId") :0;
		map.put("companyId",companyId);
		map.put("bpId",bpId);
		map.put("transactionDateFrom",transactionDateFrom);
		map.put("transactionDateTo",transactionDateTo);
		map.put("amountFrom",amountFrom);
		map.put("amountTo",amountTo);
		map.put("bpNameCondition",bpNameCondition);
		map.put("page",page);
		map.put("pagesize",pageSize);
		return new ResponseData(service.queryCollectionInfo(iRequest,map));
	}

	/**
	 *
	 * 资金管理-收款管理首页收款折线图
	 */
	@RequestMapping("/csh/cshTransaction/collectionDetailsView")
	@ResponseBody
	public ResponseData collectionDetailsView(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
										  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		long companyId = request.getSession().getAttribute("companyId")!=null? (long) request.getSession().getAttribute("companyId") :0;
		map.put("companyId",companyId);
		return new ResponseData(service.collectionDetailsView(iRequest,map,page,pagesize));
	}
	/**
	 * 现金事物-单行-交易明细
	 * @param cshTransaction
	 * @return
	 */
	@RequestMapping("/csh/cshTransaction/cashThingTransaction/query")
	@ResponseBody
	public ResponseData CashThingTransaction(HlsCusCshTransaction cshTransaction, HttpServletRequest request){
		cshTransaction.setCompanyId((Long) (request.getSession().getAttribute("companyId")!=null?request.getSession().getAttribute("companyId"):0));
		List<HlsCusCshTransaction> list = service.selectAllCashThingTransactionDetails(cshTransaction);
		return new ResponseData(list);
	}

	@RequestMapping("/csh/cshTransaction/cashThingTransaction/queryDetail")
	@ResponseBody
	public ResponseData CashThingTransactionQueryDetail(@RequestBody Map map,final HttpServletRequest request){
		List<Map> list = service.CashThingTransactionQueryDetail(map);
		return new ResponseData(list);
	}

	/**
	 * 收款综合查询
	 * @param
	 * @return
	 */
	@RequestMapping(value = "/csh/csh330aCshTransactionQuery")
	@ResponseBody
	public ResponseData csh330aCshTransactionQuery(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		List<Map> list= service.csh330aCshTransactionQuery(cshTransaction,iRequest,page,pagesize);
		return new ResponseData(list);
	}

	/**
	 * 保证金池综合查询
	 * @param
	 * @return
	 */
	@RequestMapping(value = "/csh/csh420aCshDepositPoolQuery")
	@ResponseBody
	public ResponseData csh420aCshDepositPoolQuery(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		List<Map> list= service.csh420aCshDepositPoolQuery(cshTransaction,iRequest);
		return new ResponseData(list);
	}

    /**
     * 合同保证金综合查询
     *
     * @param
     * @return
     */
    @RequestMapping(value = "/csh/csh410aConContractDepoistQuery")
    @ResponseBody
    public ResponseData csh410aConContractDepoistQuery(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        List<Map> list = service.csh410aConContractDepoistQuery(cshTransaction, iRequest);
        return new ResponseData(list);
    }


    /**
     * 合同保证金新增
     */

    @RequestMapping(value = "/csh/cshTransaction/deposit/add")
    @ResponseBody
    public ResponseData contactAddDeposit(@RequestBody HlsCusCshTransaction cshTransaction, final BindingResult result, final HttpServletRequest request,
                                          final HttpSession session) throws BeyondAmountLimitException,WriteOffTypeNullException,ChangeLimitException {
        IRequest requestContext = createRequestContext(request);

        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        Long companyId = (Long) session.getAttribute("companyId");

        service.contactAddDeposit(companyId, cshTransaction, requestContext, session);


        return new ResponseData();

    }

    /**
     * 合同保证金新增
     */

    @RequestMapping(value = "/csh/CSH421B/cshTransactionAddDpositPool")
    @ResponseBody
    public ResponseData cshTransactionAddDpositPool(@RequestBody HlsCusCshTransaction cshTransaction, final BindingResult result, final HttpServletRequest request,
                                                    final HttpSession session) throws BeyondAmountLimitException, ChangeLimitException, WriteOffTypeNullException {
        IRequest requestContext = createRequestContext(request);

        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        Long companyId = (Long) session.getAttribute("companyId");
        HlsCusCshTransaction CshTransactionR= service.cshTransactionAddDpositPool(companyId, cshTransaction, requestContext, session);
        List<HlsCusCshTransaction> CshTransactions = new ArrayList<HlsCusCshTransaction>();
        CshTransactions.add(CshTransactionR);
        return new ResponseData(CshTransactions);

    }

	/**
	 * 新增收据-收款
	 * @param cshTransaction
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/csh/cshTransaction/NotInvoiced/query")
	@ResponseBody
    public ResponseData NotInvoiced(HlsCusCshTransaction cshTransaction, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		List<HlsCusCshTransaction> list = service.selectAllNotInvoiced(cshTransaction,page,pagesize,createRequestContext(request));
    	return new ResponseData(list);
    }

	@RequestMapping(value = "/csh/cshTransaction/refundData")
	@ResponseBody
	public Map getRefundData(@RequestBody(required = false) Map map){
		Map result = new HashMap<>();
		Object transactionId = map.get("transactionId");
		Object bpId = map.get("bpId");
		HlsCusBpMaster bpMaster = bpMasterMapper.selectByPrimaryKey(bpId);
		String bpName = bpMaster.getBpName();
		result.put("bpName",bpName);
		return result;
	}


}
