package hls.core.csh.controllers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import hls.core.csh.dto.HlsCusCshPaymentReqDt;
import hls.core.csh.dto.HlsCusCshPaymentReqLn;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.mapper.HlsCusCshPaymentReqDtMapper;
import hls.core.csh.service.ICshPaymentReqDtService;
import hls.core.csh.service.ICshPaymentReqLnService;
import hls.core.hls.components.CalculateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.exception.WriteOffTypeNullException;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.service.CshWriteOffService;

@Controller
public class CshWriteOffController extends BaseController {

    public static final String WRITE_OFF_FLAG_FULL = "FULL";
    @Autowired
    private CshWriteOffService cshWriteOffService;
    @Autowired
    private HlsCusCshTransactionMapper transactionMapper;
    @Autowired
    private ICshPaymentReqLnService iCshPaymentReqLnService;
    @Autowired
    private HlsCusCshPaymentReqDtMapper cshPaymentReqDtMapper;

    /**
     * 查询账户金额是否够本次支付
     *
     * @param cshWriteOff
     * @param session
     * @return
     */
    @RequestMapping(value = "/csh/write/off/size/query")
    @ResponseBody
    public boolean querySize(final HlsCusCshWriteOff cshWriteOff, HttpSession session) {
        return cshWriteOffService.querySize(cshWriteOff, session);
    }

    /**
     * 核销查询
     *
     * @param cshWriteOff
     * @param page
     * @param pagesize
     * @param request
     * @return
     */
    @RequestMapping(value = "/csh/write/off/query")
    @ResponseBody
    public ResponseData query(final HlsCusCshWriteOff cshWriteOff, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusCshWriteOff> list = cshWriteOffService.select(requestContext, cshWriteOff, page, pagesize);
        return new ResponseData(list);
    }

    /**
     * 核销明细查询
     *
     * @param cshWriteOff
     * @param page
     * @param pagesize
     * @param request
     * @return
     */
    @RequestMapping(value = "/csh/write/off/detail/query")
    @ResponseBody
    public ResponseData queryDetail(final HlsCusCshWriteOff cshWriteOff, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusCshWriteOff> list = cshWriteOffService.queryDetail(requestContext, cshWriteOff, page, pagesize);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/csh/vat/detail/query")
    @ResponseBody
    public ResponseData vatDetail(Long cashflowId) {
        List<HlsCusCshWriteOff> list = cshWriteOffService.vatDetail(cashflowId);
        return new ResponseData(list);
    }


    /**
     * 核销反冲
     *
     * @param cshWriteOff
     * @param page
     * @param pagesize
     * @param request
     * @return
     */
    @RequestMapping(value = "/csh/write/off/reversed/query")
    @ResponseBody
    public ResponseData queryReversed(final HlsCusCshWriteOff cshWriteOff, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                      @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusCshWriteOff> list = cshWriteOffService.queryReversed(requestContext, cshWriteOff, page, pagesize);
        return new ResponseData(list);
    }


    /**
     * 核销
     *
     * @param result
     * @param request
     * @return
     * @throws BeyondAmountLimitException
     * @throws WriteOffTypeNullException
     * @throws IllegalArgumentException
     * @throws ChangeLimitException
     */
    @RequestMapping(value = "/csh/write/off/submit")
    @ResponseBody
    public ResponseData submit(@RequestBody HlsCusCshTransaction cshTransaction, final BindingResult result, final HttpServletRequest request, HttpSession session) throws BeyondAmountLimitException, IllegalArgumentException, WriteOffTypeNullException, ChangeLimitException {
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        List<HlsCusCshWriteOff> cshWriteOffs = getCshWriteOffs(cshTransaction);

        cshWriteOffService.updateWriteOff(createRequestContext(request), cshWriteOffs, session);
        return new ResponseData();

    }

    /**
     * 支付
     *
     * @param cshTransactions
     * @param result
     * @param request
     * @param session
     * @return
     * @throws BeyondAmountLimitException
     */
    @RequestMapping(value = "/csh/write/off/payment/submit")
    @ResponseBody
    public ResponseData submitPayment(@RequestBody List<HlsCusCshWriteOff> cshTransactions, final BindingResult result, final HttpServletRequest request, HttpSession session) throws BeyondAmountLimitException {
        Double amout = 0d;
        int payFlag = 0;
        ResponseData responseData = null;
        if (result.hasErrors()) {
            responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        //List<HlsCusCshWriteOff> cshWriteOffs = getCshWriteOffs(cshTransaction);
        HlsCusCshPaymentReqLn cshPaymentReqLn = new HlsCusCshPaymentReqLn();
        List<HlsCusCshPaymentReqLn> payLnList = new ArrayList<>();
        for (int i = 0; i < cshTransactions.size(); i++) {
            Double OffAmount = cshTransactions.get(i).getCshWriteOffAmount();
            cshPaymentReqLn.setPaymentReqId(cshTransactions.get(i).getPaymentReqId());
            List<HlsCusCshPaymentReqLn> cshPLnList = iCshPaymentReqLnService.queryByHnId(cshPaymentReqLn);
            Double deductAmount = 0D;
            if (cshPLnList.size() > 0) {
                for (int j = 0; j < cshPLnList.size(); j++) {
                    HlsCusCshPaymentReqLn hlsCusCshPaymentReqLn = cshPLnList.get(j);
                    payLnList.add(hlsCusCshPaymentReqLn);
                    Long payment_req_ln_id = hlsCusCshPaymentReqLn.getPayment_req_ln_id();
                    HlsCusCshPaymentReqDt cshPaymentReqDt = new HlsCusCshPaymentReqDt();
                    cshPaymentReqDt.setPaymentReqLnId(payment_req_ln_id);
                    List<HlsCusCshPaymentReqDt> cshPaymentReqDts = cshPaymentReqDtMapper.select(cshPaymentReqDt);
                    for (HlsCusCshPaymentReqDt item:cshPaymentReqDts) {
                        deductAmount = CalculateUtil.add(deductAmount,item.getDeductAmount());
                    }
                }
                cshTransactions.get(i).setCshWriteOffAmount(CalculateUtil.sub(OffAmount,deductAmount));
            }
            if (OffAmount != null) {
                amout = CalculateUtil.add(OffAmount, amout);
            }
        }
        for (int m = 0; m < payLnList.size(); m++) {
            if (WRITE_OFF_FLAG_FULL.equalsIgnoreCase(payLnList.get(m).getPayment_flag())) {
                payFlag++;
            }
        }

        ResponseData rd = null;
        if (payFlag == payLnList.size()) {
            rd = new ResponseData(false);
            rd.setMessage("该付款已被支付");
            return rd;
        } else {
            try {
                cshWriteOffService.updateWriteOff(createRequestContext(request), cshTransactions, session);
            } catch (Exception e) {
                responseData = new ResponseData(false);
                responseData.setMessage(e.getMessage());
                return responseData;
            }
            List<Double> list = new ArrayList<>();
            list.add(amout);
            return new ResponseData(list);
        }
    }

    public List<HlsCusCshWriteOff> getCshWriteOffs(HlsCusCshTransaction cshTransaction) throws BeyondAmountLimitException {

        List<HlsCusCshWriteOff> cshWriteOffs = new ArrayList<HlsCusCshWriteOff>();

        if (cshTransaction != null) {
            if (cshTransaction.getCreditCshWriteOffs() != null) {
                for (HlsCusCshWriteOff cshWriteOff : cshTransaction.getCreditCshWriteOffs()) {

                    cshWriteOff.setBpId(cshTransaction.getBpId());
                    cshWriteOff.setCurrencyCode(cshTransaction.getCurrencyCode());
                    cshWriteOffs.add(cshWriteOff);
                }
            }
            if (cshTransaction.getAdvanceCshWriteOffs() != null) {
                for (HlsCusCshWriteOff cshWriteOff : cshTransaction.getAdvanceCshWriteOffs()) {
                    cshWriteOff.setBpId(cshTransaction.getBpId());
                    cshWriteOff.setCurrencyCode(cshTransaction.getCurrencyCode());
                    cshWriteOffs.add(cshWriteOff);
                }
            }
            if (cshTransaction.getDepositCshWriteOffs() != null) {
                for (HlsCusCshWriteOff cshWriteOff : cshTransaction.getDepositCshWriteOffs()) {
                    cshWriteOff.setBpId(cshTransaction.getBpId());
                    cshWriteOff.setCurrencyCode(cshTransaction.getCurrencyCode());
                    cshWriteOffs.add(cshWriteOff);
                }
            }
        }

        if (cshWriteOffs != null) {
            long sumAmount = 0L;

            for (HlsCusCshWriteOff cshWriteOff : cshWriteOffs) {
                if (cshWriteOff.getSurplusAmount() != null && cshWriteOff.getWriteOffDueAmount() != null) {
                    /**
                     * 当核销金额大于预收款金额，抛出异常
                     */
                    if (cshWriteOff.getWriteOffDueAmount() > cshWriteOff.getSurplusAmount()) {
                        throw new BeyondAmountLimitException();
                    }

                    sumAmount += cshWriteOff.getWriteOffDueAmount();
                }

                cshWriteOff.setCshTransactionId(cshTransaction.getTransactionId());
                cshWriteOff.setWriteOffDate(cshTransaction.getWriteOffDate());
            }

            List<HlsCusCshTransaction> cshTransactions = transactionMapper.queryDetailById(cshTransaction.getTransactionId());
            HlsCusCshTransaction transaction = cshTransactions.get(0);

            /**
             * 当核销金额总和大于收款金额时，抛出异常
             */
            if (transaction.getWriteOffAmount() == null) {
                transaction.setWriteOffAmount(0d);
            }

            Double sum = sumAmount + transaction.getWriteOffAmount();

            if (sum > transaction.getTransactionAmount()) {
                throw new BeyondAmountLimitException();
            }
        }


        return cshWriteOffs;
    }

    @RequestMapping(value = "/csh/write/off/reversed/submit")
    @ResponseBody
    public ResponseData reversedSubmit(@RequestBody HlsCusCshWriteOff cshWriteOff, final BindingResult result, final HttpServletRequest request) throws BeyondAmountLimitException, WriteOffTypeNullException {
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestContext = createRequestContext(request);

        HlsCusCshWriteOff data = cshWriteOffService.selectByPrimaryKey(requestContext, cshWriteOff);

        data.setReversedDate(cshWriteOff.getReversedDate());
        data.setDescription(cshWriteOff.getDescription());

        cshWriteOffService.updateWriteOffReversed(requestContext, data);

        return new ResponseData();

    }


    /**
     * 付款反冲
     *
     * @param result
     * @param request
     * @return
     * @throws ChangeLimitException
     */
    @RequestMapping(value = "/csh/write/off/paymentReversed/submit")
    @ResponseBody
    public ResponseData paymentReversed(@RequestBody HlsCusCshWriteOff cshWriteOff, HttpSession session, final BindingResult result, final HttpServletRequest request) throws ChangeLimitException {
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestContext = createRequestContext(request);

        cshWriteOffService.paymentWriteOffReversed(requestContext, cshWriteOff, session);

        return new ResponseData();

    }

    /**
     * 合同现金流核销详情查询
     *
     * @param cshWriteOff
     * @param page
     * @param pagesize
     * @param request
     * @return
     */
    @RequestMapping(value = "/csh/contract/cashflow/detail/query")
    @ResponseBody
    public ResponseData cashflowDetail(final HlsCusCshWriteOff cshWriteOff, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusCshWriteOff> list = cshWriteOffService.cashflowDetail(requestContext, cshWriteOff, page, pagesize);
        return new ResponseData(list);
    }

    /**
     * 全年每月的放款和收款
     *
     * @param request
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/csh/write/off/getEveryMonthPaymentAndReceiptAmount/query")
    @ResponseBody
    public ResponseData getEveryMonthPaymentAndReceiptAmount(HttpServletRequest request, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                                             @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusCshWriteOff> list = cshWriteOffService.getEveryMonthPaymentAndReceiptAmount();
        return new ResponseData(list);
    }

    /*资金管理首页付款折线图*/
    @RequestMapping(value = "/csh/write/off/fundPaymentLineInfo/query")
    @ResponseBody
    public ResponseData fundPaymentLineInfo(@RequestBody Map map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                            @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request, final HttpSession session) {
        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        map.put("companyId", companyId);
        List<HlsCusCshWriteOff> list = cshWriteOffService.fundPaymentLineInfo(map);
        return new ResponseData(list);
    }

    /**
     * 核销事务-多行-核销明细（收付款维度）
     */
    @RequestMapping(value = "/csh/write/off/Details")
    @ResponseBody
    public ResponseData WriteOffDetails(@RequestBody(required = false) Map map, final HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        List<Map> data = cshWriteOffService.WriteOffDetail(map, iRequest);
        return new ResponseData(data);
    }

    /**
     * 核销事务-多行-核销明细（收付款维度）
     */
    @RequestMapping(value = "/csh/write/off/all/Details")
    @ResponseBody
    public ResponseData selectAllWriteOff(@RequestBody Map map, final HttpServletRequest request, HttpSession session) {
        IRequest iRequest = createRequestContext(request);
        List<Map> data = cshWriteOffService.selectAllWriteOff(map, session, iRequest);
        return new ResponseData(data);
    }

    /**
     * 核销事务-多行-核销明细（收付款维度）
     */
    @SuppressWarnings("rawtypes")
    @RequestMapping(value = "/csh/write/off/DetailsByCFL")
    @ResponseBody
    public ResponseData writeOffDetailByCFL(@RequestBody Map map, final HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        List<Map> data = cshWriteOffService.writeOffDetailByCFL(map, iRequest);
        if (map.get("contract_id") == null) {
            for (int i = 0; i < data.size(); i++) {
                if (data.get(i).get("writeOffDueAmount") == null) {
                    data.remove(i);
                }
            }
        }
        return new ResponseData(data);
    }

    /**
     * 核销事物-单行-核销明细
     *
     * @param cshWriteOff
     * @return
     */
    @RequestMapping(value = "/csh/writeOff/CancelAfterVerification/query")
    @ResponseBody
    public ResponseData CancelAfterVerification(HlsCusCshWriteOff cshWriteOff) {
        List<HlsCusCshWriteOff> list = cshWriteOffService.selectAllCancelAfterVerificationDetail(cshWriteOff);
        return new ResponseData(list);
    }

    @RequestMapping(value = "")
    @ResponseBody
    public ResponseData depositPoolMgrBySingle(@RequestBody Map map, HttpServletRequest request) {
        return new ResponseData();
    }

    @RequestMapping(value = "/csh/writeOff/detail/query")
    @ResponseBody
    public ResponseData selectDetail(HlsCusCshWriteOff cshWriteOff) {
        List<HlsCusCshWriteOff> list = cshWriteOffService.selectDetail(cshWriteOff);
        return new ResponseData(list);
    }

    /*融资还款 - 核销明细*/
    @RequestMapping(value = "/csh/writeOff/queryLonRepayWriteoffDetail")
    @ResponseBody
    public ResponseData queryLonRepayWriteoffDetail(HttpServletRequest request, HlsCusCshWriteOff dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpSession session) {
        dto.setCompanyId((Long) session.getAttribute("companyId"));
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(cshWriteOffService.queryLonRepayWriteoffDetail(requestContext, dto, page, pageSize));
    }
}
