package hls.core.csh.service;

import java.text.ParseException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpSession;

import com.hand.hap.core.IRequest;
import com.hand.hap.core.ProxySelf;
import com.hand.hap.system.service.IBaseService;

import hls.core.cont.dto.HlsCusConContractChangeReq;
import hls.core.cont.dto.HlsCusConContractStatus;
import hls.core.csh.dto.HlsCusCshConContract;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.dto.HlsCusBpMasterBankAccount;


public interface ICshConContractService extends IBaseService<HlsCusCshConContract>, ProxySelf<ICshConContractService>{

	/**
	 * 查询所有状态为签约的合同
	 * @param conContract
	 * @return
	 */
	public List<HlsCusCshConContract> selectAllConContract(HlsCusCshConContract conContract);
	/**
	 * 通过合同ID，获取该合同承租人所有银行账户
	 * @return
	 */
	public List<HlsCusBpMasterBankAccount> selectBank(HlsCusCshConContract conContract);

	List<HlsCusCshConContract> getContractsByNumberOrName(HlsCusCshConContract conContract, int page, int pagesize);
	
	public List<HlsCusCshConContract> contractDetail(IRequest requestContext, HlsCusCshConContract conContract, int page, int pagesize);
	public void save(IRequest iRequest,HlsCusCshConContract cshConContract);
	public Map<String,Double> contractAmountQuery();
	/**
	 * @param session 
	 * @return
	 */
	public HlsCusCshConContract selectAllStatistical(HttpSession session);

	public List<HlsCusCshConContract> getTotalContractCount();

	public List<HlsCusCshConContract> getDiffContractStatusCount();



	public List<HlsCusCshConContract> getTerminateContractCount();

	public List<HlsCusCshConContract> getStockContractAndMonthNewContract();

	public List<HlsCusCshConContract> getIndexContractInceptInfo(IRequest requestContext, HlsCusCshConContract conContract, int page, int pagesize);
	
	public List<HlsCusCshConContract> contractDueAmountQuery();
	/**
	 * @param conContractChangeReq
	 * @param session 
	 * @return
	 * @throws ChangeLimitException 
	 */
	List<HlsCusConContractChangeReq> selectAllReqDetailedInformation(HlsCusConContractChangeReq conContractChangeReq, HttpSession session) throws ChangeLimitException;
	//合同首页-基本信息-统计图
	public List<HlsCusConContractStatus>
	selectContractStatusInfo(HlsCusConContractStatus conContractStatus, HttpSession session);
	//合同首页-投放信息-投放金额信息百分比
	public HlsCusConContractStatus selectTotalThrowInfo(IRequest requestContext, HlsCusConContractStatus conContractStatus);
	
	public List<HlsCusCshConContract> contractHomeFirstQuery(Map<String, Object> map);
	public List<HlsCusCshConContract> contractHomeSecordQuery(Map<String, Object> map);
	public List<HlsCusCshConContract> contractHomeThirdQuery(Map<String, Object> map);
	/**
	 * @param conContract
	 * @param session 
	 * @return
	 * @throws ParseException 
	 * @throws ChangeLimitException 
	 */
	List<HlsCusCshConContract> selectTypeStatistics(HlsCusCshConContract conContract, HttpSession session) throws ParseException, ChangeLimitException;

	Map selectTwoTypeStatistics(HlsCusCshConContract conContract, HttpSession session) throws ParseException, ChangeLimitException;

	public List<HlsCusCshConContract> selectContractByID(HlsCusCshConContract conContract);
	/**
	 * @param conContract
	 * @param session 
	 * @return
	 * @throws ChangeLimitException 
	 */
	List<HlsCusCshConContract> selectContractInFo(HlsCusCshConContract conContract, HttpSession session) throws ChangeLimitException;
	/**
	 * @param conContract
	 * @param session 
	 * @return
	 * @throws ParseException 
	 * @throws ChangeLimitException 
	 */
	List<HlsCusCshConContract> selectStockCount(HlsCusCshConContract conContract, HttpSession session) throws ParseException, ChangeLimitException;
	/**
	 * @param changeReq
	 * @param session 
	 * @return
	 * @throws ParseException
	 * @throws ChangeLimitException 
	 */
	HlsCusConContractChangeReq selectAlterStatistics(HlsCusConContractChangeReq changeReq, HttpSession session) throws ParseException, ChangeLimitException;
	/**
	 * 合同首页-基本信息-合同信息查询
	 * @param conContract
	 * @return
	 */
	List<HlsCusCshConContract> selectAllCshConContractInformation(HlsCusCshConContract conContract);
	public List<HlsCusCshConContract> comprehensiveQuery(Map<String, Object> map, int page, int pagesize);

	public List<HlsCusCshConContract> selectUpToDateCshConConTract(HlsCusCshConContract cshConContract, HttpSession session);

	public List<HlsCusCshConContract> selectOnHireCshconcontract(HlsCusCshConContract cshConContract, HttpSession session);

	public List<HlsCusCshConContract> selectCshConContract (Long bpId);

	public List<HlsCusCshConContract> selectCshConContractGuarantor(Long bpId);

    List<HlsCusCshConContract> selectCshconcontractLov();

	List<Map> receivableCollection(HlsCusBpMaster hlsCusBpMaster);

	public List<HlsCusCshConContract> rentQuery(Map<String, Object> map, int page, int pagesize);
}
