package hls.core.csh.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.calc.service.HlsCalcExcelImportUtilService;
import hls.core.cont.dto.*;
import hls.core.cont.mapper.HlsCusConContractCashflowHMapper;
import hls.core.cont.mapper.HlsCusConContractChangeReqMapper;
import hls.core.cont.mapper.HlsCusConContractHMapper;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.cont.service.ConContractChangeCommon;
import hls.core.cont.service.ConContractChangeService;
import hls.core.csh.dto.*;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.service.*;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.dto.HlsCusHlsDocumentVersionRel;
import hls.core.hls.mapper.HlsCusHlsDocumentVersionRelMapper;
import hls.core.hls.service.HlsBeanRefUtilService;
import hls.core.prj.dto.HlsCusHlsDocumentVersion;
import hls.core.prj.dto.HlsCusPrjQuotation;
import hls.core.prj.dto.HlsCusPrjQuotationCashflow;
import hls.core.prj.mapper.HlsCusHlsDocumentVersionMapper;
import hls.core.prj.mapper.HlsCusPrjQuotationCashflowMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.wfl.service.IActivitiStartService;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpSession;
import java.lang.reflect.InvocationTargetException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Transactional
public class ConContractCashflowServiceImpl extends BaseServiceImpl<HlsCusConContractCashflow>
        implements IConContractCashflowService, ConContractChangeCommon {

    private static final String changeName = "HlsCusConContractCashflow";
    private static final String deleteName = "cashflowsdel";
    private static final String methodName = "getChangeCashflowFlag";
    public static final long ITEM = 11L;//提前结清
    public static final String APPROVING = "APPROVING";//审批中
    public static final String OUTFLOW = "OUTFLOW";//流出
    public static final String DEPOSIT = "DEPOSIT";//保证金
    public static final String CON_CONTRACT = "con_contract";
    public static final String CON_CONTRACT_CHANGE_REQ = "con_contract_change_req";
    public static final String HISTORY = "HISTORY";
    public static final String CHANGE_REQ = "CHANGE_REQ";

    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private HlsCusConContractHMapper conContractHMapper;
    @Autowired
    private HlsCusHlsDocumentVersionRelMapper documentVersionRelMapper;
    @Autowired
    private HlsCusHlsDocumentVersionMapper documentVersionMapper;
    @Autowired
    private HlsCusConContractCashflowMapper cashflowMapper;
    @Autowired
    private HlsCusConContractCashflowHMapper cashflowHMapper;
    @Autowired
    private ConContractChangeService changeReqService;
    @Autowired
    private ICshPaymentReqHdService paymentReqHdService;
    @Autowired
    private ICshPaymentReqLnService cshPaymentReqLnService;
    @Autowired
    private FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private HlsCusPrjQuotationCashflowMapper prjQuotationCashflowMapper;
    @Autowired
    private HlsBeanRefUtilService hlsBeanRefUtilService;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private HlsCusCshTransactionMapper transactionMapper;
    @Autowired
    private HlsCalcExcelImportUtilService hlsCalcExcelImportUtilService;
    @Autowired
    private ICshPaymentReqDtService cshPaymentReqDtService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private HlsCusConContractChangeReqMapper conContractChangeReqMapper;
    @Autowired(required = false)
    private IConPaymentApprovalCustomService iConPaymentApprovalCustomService;

    /*
     * (non-Javadoc)
     *
     * @see hls.core.csh.service.IConContractCashflowService#
     * selectAllConContractCashflow(hls.core.csh.dto.HlsCusConContractCashflow)
     */
    @Override
    public List<HlsCusConContractCashflow> selectAllConContractCashflow(HlsCusConContractCashflow cashflow) {
        return cashflowMapper.selectAllConContractCashflow(cashflow);
    }

    @Override
    public List<Map> query4depositMgr(int page, int pageSize) throws Exception {
        PageHelper.startPage(page, pageSize);
        List<Map> maps = cashflowMapper.query4depositMgr();
        return maps;
    }

    /*
     * (non-Javadoc)
     *
     * @see hls.core.csh.service.IConContractCashflowService#
     * selectConContractCashflow(java.lang.Long)
     */
    @Override
    public HlsCusConContractCashflow selectConContractCashflow(Long cashflow_id) {
        return cashflowMapper.selectConContractCashflow(cashflow_id);
    }

    @Override
    public List<Map> queryDepositDetail(Long contractId, int page, int pageSize) throws Exception {
        PageHelper.startPage(page, pageSize);
        List<Map> maps = cashflowMapper.queryDepositDetail(contractId);
        return maps;
    }

    @Override
    public List<Map> queryPoolDetail(Long bpId, int page, int pageSize) throws Exception {
        PageHelper.startPage(page, pageSize);
        List<Map> maps = cashflowMapper.queryPoolDetail(bpId);
        return maps;
    }

    @Override
    public List<Map> query4PoolDepositMgr(int page, int pageSize) throws Exception {
        PageHelper.startPage(page, pageSize);
        List<Map> maps = cashflowMapper.query4PoolDepositMgr();
        return maps;
    }

    @Override
    public List<HlsCusConContractCashflow> selectOneConContract(HlsCusConContractCashflow cashflow) {
        return cashflowMapper.selectOneConContract(cashflow);
    }

    @Override
    public void updateOne(HlsCusConContractCashflow cashflow) {
        cashflowMapper.updateOne(cashflow);
    }

    @Override
    public List<Map> query4Recall(HlsCusCshDeduction deduction, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        List<Map> maps = cashflowMapper.query4Recall(deduction);
        return maps;
    }

    @Override
    public List<HlsCusConContractCashflow> selectLovItem() {
        return cashflowMapper.selectLovItem();
    }

    @Override
    public List<Map> query4Back(HlsCusCshDeduction deduction, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        List<Map> maps = cashflowMapper.query4Back(deduction);
        return maps;
    }

    @Override
    public List<HlsCusConContractCashflow> contractCashFlowDetail(IRequest requestContext, HlsCusConContractCashflow cashflow,
                                                                  int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        Map<String, Object> params = new HashMap<String, Object>();
        List<String> list = new ArrayList<String>();
        List<String> notList = new ArrayList<String>();
        if (cashflow.getCf_item_list() != null && cashflow.getNot_cf_item_list() == null) {
            String[] cf_item = cashflow.getCf_item_list().split(",");
            for (int i = 0; i < cf_item.length; i++) {
                list.add(cf_item[i]);
            }
            params.put("list", list);
            params.put("contract_id", cashflow.getContract_id());
            params.put("contract_number", cashflow.getContract_number());
        } else if (cashflow.getCf_item_list() == null && cashflow.getNot_cf_item_list() != null) {

            String[] not_cf_item = cashflow.getNot_cf_item_list().split(",");
            for (int i = 0; i < not_cf_item.length; i++) {
                notList.add(not_cf_item[i]);
            }
            params.put("notList", notList);
            params.put("contract_id", cashflow.getContract_id());
            params.put("contract_number", cashflow.getContract_number());
        } else if (cashflow.getCf_item_list() != null && cashflow.getNot_cf_item_list() != null) {
            String[] cf_item = cashflow.getCf_item_list().split(",");
            String[] not_cf_item = cashflow.getNot_cf_item_list().split(",");

            for (int i = 0; i < cf_item.length; i++) {
                list.add(cf_item[i]);
            }

            for (int i = 0; i < not_cf_item.length; i++) {
                notList.add(not_cf_item[i]);
            }
            params.put("list", list);
            params.put("notList", notList);
            params.put("contract_id", cashflow.getContract_id());
            params.put("contract_number", cashflow.getContract_number());

        } else {
            params.put("contract_id", cashflow.getContract_id());
            params.put("contract_number", cashflow.getContract_number());
        }


        return cashflowMapper.contractCashFlowDetail(params);
    }

    @Override
    public String getChangeName() {
        return changeName;
    }

    @Override
    public Long dispose(IRequest iRequest, ConContractChangeH conContractChangeH, Long changeReqId) {
        List<HlsCusConContractCashflow> lists = conContractChangeH.getConContractCashflows();
        int count = 0;
        for (int i = 0; i < lists.size(); i++) {
            HlsCusConContractCashflow cashflow = lists.get(i);
            if (cashflow.getCashflow_id() != 0) {
                continue;
            }
            count++;
            cashflowMapper.selectConContractCashflow(lists.get(i).getCashflow_id());
        }
        if (count == 0) {
            HlsCusConContractChangeReq req = new HlsCusConContractChangeReq();
            req.setChangeReqId(changeReqId);
            req = changeReqService.selectByPrimaryKey(iRequest, req);
            req.setChangeTenantFlag("N");
            changeReqService.updateByPrimaryKey(iRequest, req);
        }
        return null;
    }

    @Override
    public void delete(IRequest iRequest, ConContractChangeH conContractChangeH) {
        Long[] code = conContractChangeH.getConContractChangeDeleteArr().getCashflowsDel();
        if (code != null && code.length > 0) {
            for (int i = 0; i < code.length; i++) {
            }
            return;
        }
    }

    @Override
    public String getDeleteName() {
        return deleteName;
    }

    /*
     * (non-Javadoc)
     *
     * @see hls.core.cont.service.ConContractChangeCommon#methodName()
     */
    @Override
    public String getMethodName() {
        return methodName;
    }

    /*
     * (non-Javadoc)
     *
     * @see hls.core.cont.service.ConContractChangeCommon#examineAndApprove()
     */
    @Override
    public void examineAndApprove(IRequest iRequest, Long changeReqId, Long contractId, String status)
            throws IllegalAccessException, InvocationTargetException {
        if ("APPROVED".equalsIgnoreCase(status)) {
            // 查询出合同现金流原表，_h备份表信息
            HlsCusConContractCashflowH cashflowH = new HlsCusConContractCashflowH();
            //cashflowH.setChangeReqId(changeReqId);
            // itemH.setContractId(contractId);
            // itemH.setVenderBpId(bpId);
            HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
            cashflow.setContract_id(contractId);

            List<HlsCusConContractCashflowH> cashflowHs = cashflowHMapper.select(cashflowH);
            List<HlsCusConContractCashflow> cashflows = cashflowMapper.select(cashflow);

            // 删除两表的数据：原表删除合同所对应的数据，_h表删除change_req_id对应的数据
            for (int i = 0; i < cashflows.size(); i++) {
                cashflowMapper.deleteByPrimaryKey(cashflows.get(i));
            }
            for (int i = 0; i < cashflowHs.size(); i++) {
                cashflowHMapper.deleteByPrimaryKey(cashflowHs.get(i));
            }
            // 将_h表的记录(conTenantList)插入到原表
            for (HlsCusConContractCashflowH i : cashflowHs) {
                HlsCusConContractCashflow cct = new HlsCusConContractCashflow();
                BeanUtils.copyProperties(cct, i);
                cct.setContract_id(contractId);
                //			cashflowMapper.insertConContractCashflow(cct);
                cct.setLastUpdatedBy((Long) iRequest.getUserId());
                cct.setLastUpdateDate(new Date());
                self().insertSelective(iRequest, cct);
            }
            // 将原表的记录(conTenantList)插入到_h表
            for (HlsCusConContractCashflow i : cashflows) {
                HlsCusConContractCashflowH ccth = new HlsCusConContractCashflowH();
                BeanUtils.copyProperties(ccth, i);
                //ccth.setChangeReqId(changeReqId);
                //cashflowHMapper.insertConContractCashflowH(ccth);
                cashflowHMapper.insertSelective(ccth);
            }
        } else {
        }
    }

    /**
     * 首页-回笼信息-应收金额、应收本金查询
     *
     * @param cashflow
     * @return
     * @throws ParseException
     * @throws ChangeLimitException
     */
    @Override
    public HlsCusConContractCashflow selectInformationStatistics(HlsCusConContractCashflow cashflow, HttpSession session) throws ParseException, ChangeLimitException {
        Long fate = cashflow.getQueryTimeSolt();
        Long company_id = (Long) session.getAttribute("companyId");
        if (company_id < 0) {
            throw new ChangeLimitException("公司ID获取失败");
        }
        if (fate != null) {
            SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");
            Date beginDate = new Date();
            Calendar date = Calendar.getInstance();
            date.setTime(beginDate);
            if (fate == 7) {
                date.set(Calendar.DATE, date.get(Calendar.DATE) - 7);
            }
            if (fate == 30) {
                date.add(Calendar.MONTH, -1);
            }
            if (fate == 90) {
                date.add(Calendar.MONTH, -3);
            }
            String endDate = dft.format(date.getTime());
            cashflow.setDue_date(dft.parse(endDate));
            cashflow.setCompany_id(company_id);
            cashflow = cashflowMapper.selectInformationStatistics(cashflow);
            if (cashflow == null) {
                return new HlsCusConContractCashflow();
            }
        }
        return cashflow;
    }
    @Override
    public Map selectInformationAllStatistics(HlsCusConContractCashflow cashflow, HttpSession session) throws ParseException, ChangeLimitException {
        Long fate = cashflow.getQueryTimeSolt();
        Map map =null;
        Long company_id = (Long) session.getAttribute("companyId");
        if (company_id < 0) {
            throw new ChangeLimitException("公司ID获取失败");
        }
        if (fate != null) {
            SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");
            Date beginDate = new Date();
            Calendar date = Calendar.getInstance();
            date.setTime(beginDate);
            if (fate == 7) {
                date.set(Calendar.DATE, date.get(Calendar.DATE) - 7);
            }
            if (fate == 30) {
                date.add(Calendar.MONTH, -1);
            }
            if (fate == 90) {
                date.add(Calendar.MONTH, -3);
            }
            String endDate = dft.format(date.getTime());
            cashflow.setDue_date(dft.parse(endDate));
            cashflow.setCompany_id(company_id);
            map = cashflowMapper.selectInformationAllStatistics(cashflow);
            if (cashflow == null) {
                return new HashMap();
            }
        }
        return map;
    }

    @Override
    public Double getCurrentWeekOverDueAmount() {
        return cashflowMapper.getCurrentWeekOverDueAmount();
    }

    @Override
    public Double getYesterdayOverDueAmount() {
        return cashflowMapper.getYesterdayOverDueAmount();
    }

    @Override
    public Double getTodayOverDueAmount() {
        return cashflowMapper.getTodayOverDueAmount();
    }

    @Override
    public Double getTotalOverDueAmount() {
        return cashflowMapper.getTotalOverDueAmount();
    }

    @Override
    public List<HlsCusConContractCashflow> getLastWeekOverDueAmount() {
        return cashflowMapper.getLastWeekOverDueAmount();
    }

    @Override
    public List<HlsCusCshPaymentReqLn> creatPayInfo(List<HlsCusConContractCashflow> list, Long companyId, String value, IRequest request) throws BeyondAmountLimitException {
        Double sumAmout = 0D;//总金额
        List<HlsCusCshPaymentReqLn> ReqLnList = new ArrayList<>();
        HlsCusCshPaymentReqHd cshPaymentReqHd = new HlsCusCshPaymentReqHd();
        HashMap<String, Object> map = new HashMap<>(5);
        map.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_PAYMENT_REQ_CREATE);
        map.put("currency", "");
        map.put("amount", "");
        for (HlsCusConContractCashflow item:list) {
            sumAmout = CalculateUtil.add(sumAmout,item.getAmount());
        }
        cshPaymentReqHd.setAmount(sumAmout);
        cshPaymentReqHd.setCompany_id(companyId);
        cshPaymentReqHd.setPayment_req_id(null);
        //cshPaymentReqHd.setCompany_id(list.get(i).getCompany_id());
        cshPaymentReqHd.setDocument_type("PAYMENT_REQ");
        cshPaymentReqHd.setDocument_category("CSH_PAYMENT_REQ");
        cshPaymentReqHd.setBusiness_type("PAYMENT_REQ");// 业务模式
        cshPaymentReqHd.setPayment_req_number(value); // 付款申请编号
        cshPaymentReqHd.setPayment_req_date(list.get(0).getApplyPayDate());
        //cshPaymentReqHd.setPayment_req_status("NEW");
        //目前没有工作流
        cshPaymentReqHd.setPayment_req_status("APPROVED");
        cshPaymentReqHd.setApply_pay_date(new Date());
        cshPaymentReqHd.setCurrency("CNY");
        cshPaymentReqHd.setLastUpdatedBy(request.getUserId());
        cshPaymentReqHd.setLastUpdateDate(new Date());
        cshPaymentReqHd = paymentReqHdService.insertSelective(request, cshPaymentReqHd);
        for (int i = 0; i < list.size(); i++) {
            Double amLong = list.get(i).getAmount(); // 申请付款金额
            Long bp_id = list.get(i).getBp_id(); // 付款对象商业伙伴ID
            HlsCusConContractCashflow cashflow = list.get(i);
            Long cashflowId = cashflow.getCashflow_id();
            HlsCusConContractCashflow conContractCashflow = cashflowMapper.selectByPrimaryKey(cashflowId);
            HlsCusCshPaymentReqLn cshPaymentReqLn = new HlsCusCshPaymentReqLn();
            cshPaymentReqLn.setPayment_req_id(cshPaymentReqHd.getPayment_req_id());
            cshPaymentReqLn.set__status("add");
            cshPaymentReqLn.setPayment_req_id(cshPaymentReqHd.getPayment_req_id());
            cshPaymentReqLn.setSource_doc_category("CON_CONTRACT");
            cshPaymentReqLn.setSource_doc_id(list.get(i).getContract_id());
            cshPaymentReqLn.setSource_doc_line_id(list.get(i).getCashflow_id());
            cshPaymentReqLn.setAmount(amLong);
            cshPaymentReqLn.setDescription("");// 输入的申请原因
            cshPaymentReqLn.setPayment_flag("NOT");
            cshPaymentReqLn.setPayment_method("");// 界面选择
            cshPaymentReqLn.setBp_id(bp_id);
            cshPaymentReqLn.setBp_bank_account_id(cashflow.getBank_accout_id());
            cshPaymentReqLn.setBp_bank_account_name(cashflow.getBankBranchName());//账户名
            cshPaymentReqLn.setBp_bank_account_num(cashflow.getBankAccountNum());
            cshPaymentReqLn.setBp_bank_branch_name(cashflow.getBank_branch_name());//银行分行
            cshPaymentReqLn.setBp_bank_name(cashflow.getBank_full_name());
            cshPaymentReqLn.setDescription(cashflow.getDescriptionLn());
            List<HlsCusCshPaymentReqDt> cshPaymentReqDts = cashflow.getCshPaymentReqDts();
            cshPaymentReqLnService.insertSelective(request,cshPaymentReqLn);//插入付款申请行
            if (conContractCashflow.getCf_item() == 0 && cshPaymentReqDts != null && !cshPaymentReqDts.isEmpty()){//只有现金流为设备款时,可以进行付款抵扣
                Double totalDeductionAmount = 0D;
                for (HlsCusCshPaymentReqDt item:cshPaymentReqDts) {
                    item.setPaymentReqLnId(cshPaymentReqLn.getPayment_req_ln_id());
                    cshPaymentReqDtService.insertSelective(request,item);//插入付款申请抵扣
                    totalDeductionAmount = CalculateUtil.add(totalDeductionAmount,item.getDeductAmount());
                }
                Double subAmount = CalculateUtil.sub(amLong, totalDeductionAmount);
                if (subAmount.compareTo(0D) < 0){
                    throw new BeyondAmountLimitException("抵扣金额超出限制");
                }
                cshPaymentReqLn.setCshPaymentReqDts(cshPaymentReqDts);//付款申请抵扣
            }
            ReqLnList.add(cshPaymentReqLn);
            Long contract_id = list.get(i).getContract_id();
            HlsCusConContract conContract = new HlsCusConContract();
            conContract.setContractId(contract_id);
            conContract = conContractMapper.selectByPrimaryKey(conContract);
            map.put("amount", list.get(i).getAmount());
            HLSCurrency hlsCurrency = new HLSCurrency();
            hlsCurrency.setCurrencyCode(conContract.getCurrency());
            hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);
            map.put("currency", hlsCurrency.getCurrencySymbol());
            sysEventService.createEvent(request, contract_id, cshPaymentReqLn.getSource_doc_category(), conContract.getDocumentType(), map);
        }
        return ReqLnList;
    }

    @Override
    public List<HlsCusConContractCashflow> selectHireConContractCashflow(HlsCusConContractCashflow conContractCashflow, HttpSession session) {
        return cashflowMapper.selectHireConContractCashflow(conContractCashflow);
    }

    @Override
    public HlsCusConContractCashflow selectOverdueCount(HlsCusConContractCashflow conContractCashflow, HttpSession session) {
        return cashflowMapper.selectOverdueCount(conContractCashflow);
    }

    @Override
    public List<HlsCusConContractCashflow> vatQuery(IRequest iRequest, Map<String, Object> map, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        return cashflowMapper.vatQuery(map);
    }

    @Override
    public List<HlsCusConContractCashflow> vatQueryList(Map<String, String> map) {
        Map<String, Object> params = new HashMap<String, Object>();

        params.put("contractNumber", map.get("contractNumber"));
        params.put("companyId", map.get("companyId"));
        params.put("contractName", map.get("contractName"));
        params.put("timesFrom", map.get("timesFrom"));
        params.put("TimesTo", map.get("timesTo"));
        params.put("dueDateFrom", map.get("dateFrom"));
        params.put("dueDateTo", map.get("dateTo"));


        params.put("contractInfo", map.get("contractInfo"));
        params.put("count", map.get("count"));

        String[] cashFlowItems = null;
        String[] status = null;
        if (map.get("cashflowItems") != null && !"".equals(map.get("cashflowItems"))) {
            cashFlowItems = map.get("cashflowItems").split("、");
            params.put("cashFlowItems", cashFlowItems);
        }
        if (map.get("status") != null && !"".equals(map.get("status"))) {
            status = map.get("status").split("、");
            params.put("status", status);
        }
        List<HlsCusConContractCashflow> conCashflowList = cashflowMapper.vatQuery(params);
        if (conCashflowList != null && conCashflowList.size() > 0) {
            Iterator<HlsCusConContractCashflow> iterator = conCashflowList.iterator();
            while (iterator.hasNext()) {
                HlsCusConContractCashflow next = iterator.next();
                // 判断每一条现金流的cfItem 是否等于1,否不处理
                if (next.getCf_item().compareTo(1L) == 0 && next.getCf_item().compareTo(next.getBsCfItem()) != 0 && next.getVatModifyFlag() != 1) {
                    //BsCfItem不等于cfItem说明配置的开票规则将租金分成了本金和利息
                    //找到对应的两条现金流,确定它们的 mergeRentalFlag 字段是否都为 Y
                    HlsCusConContractCashflow cashflow2 = null;
                    for (HlsCusConContractCashflow ccc2 : conCashflowList) {
                        if (next.getCashflow_id().compareTo(ccc2.getCashflow_id()) == 0 && next.getBsCfItem().compareTo(ccc2.getBsCfItem()) != 0) {
                            cashflow2 = ccc2;
                        }
                    }
                    if (cashflow2 != null) {
                        //mergeRentalFlag 字段都为 Y ,则从List中删除一条cashflow
                        if (cashflow2.getMergeRentalFlag().equals("Y") && next.getMergeRentalFlag().equals("Y")) {
                            next.setVatModifyFlag(1);
                            iterator.remove();
                        }
                        //否则修改两条现金流,100的改为本金,101的改为利息,修改各自的cfItemDesc 更新modifyFlag为1
                        else {
                            if (next.getBsCfItem().compareTo(100L) == 0) {
                                next.setCf_item_desc("本金");
                                next.setDue_amount(next.getPrincipal());
                                next.setInterest(0d);
                                next.setBilling_amount(next.getBilling_principal());
                                next.setBilling_interest(0d);
                                next.setVatModifyFlag(1);

                                cashflow2.setCf_item_desc("利息");
                                cashflow2.setDue_amount(cashflow2.getInterest());
                                cashflow2.setPrincipal(0d);
                                cashflow2.setBilling_amount(cashflow2.getBilling_interest());
                                cashflow2.setBilling_principal(0d);
                                cashflow2.setVatModifyFlag(1);
                            } else if (next.getBsCfItem().compareTo(101L) == 0) {
                                cashflow2.setCf_item_desc("本金");
                                cashflow2.setDue_amount(cashflow2.getPrincipal());
                                cashflow2.setInterest(0d);
                                cashflow2.setBilling_amount(cashflow2.getBilling_principal());
                                cashflow2.setBilling_interest(0d);
                                cashflow2.setVatModifyFlag(1);

                                next.setCf_item_desc("利息");
                                next.setDue_amount(next.getInterest());
                                next.setPrincipal(0d);
                                next.setBilling_amount(next.getBilling_interest());
                                next.setBilling_principal(0d);
                                next.setVatModifyFlag(1);
                            }
                        }
                    } else {
                        if (next.getBsCfItem().compareTo(100L) == 0) {
                            next.setCf_item_desc("本金");
                            next.setDue_amount(next.getPrincipal());
                            next.setInterest(0d);
                            next.setBilling_amount(next.getBilling_principal());
                            next.setBilling_interest(0d);
                            next.setVatModifyFlag(1);
                        } else if (next.getBsCfItem().compareTo(101L) == 0) {
                            next.setCf_item_desc("利息");
                            next.setDue_amount(next.getInterest());
                            next.setPrincipal(0d);
                            next.setBilling_amount(next.getBilling_interest());
                            next.setBilling_principal(0d);
                            next.setVatModifyFlag(1);
                        }
                    }
                }
            }
        }
        return conCashflowList;
    }

    /* (non-Javadoc)
     * @see hls.core.csh.service.IConContractCashflowService#selectRiskExposure(com.hand.hap.core.IRequest, hls.core.hls.bp.dto.HlsCusBpMaster)
     */
    @Override
    public HlsCusCshFundStatistics selectRiskExposure(IRequest irRequest, HlsCusBpMaster hlsCusBpMaster) {
        return cashflowMapper.selectRiskExposure(hlsCusBpMaster);
    }

    public HlsCusCshFundStatistics selectRiskGuarantorExposure(IRequest irRequest, HlsCusBpMaster hlsCusBpMaster) {
        return cashflowMapper.selectRiskGuarantorExposure(hlsCusBpMaster);
    }

    @Override
    public List<HlsCusConContractCashflow> debtQuery(IRequest iRequest, Map map, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        return cashflowMapper.debtQuery(map);
    }

    @Override
    public List<HlsCusConContractCashflow> saveCashflowFromPrjCashflow(IRequest iRequest, Long contractId, Long ProjectId) {
        HlsCusConContract c = new HlsCusConContract();
        c.setContractId(contractId);
        c = conContractMapper.selectByPrimaryKey(c);
        List<HlsCusConContractCashflow> conContractCashflowList = new ArrayList<>();
        HlsCusPrjQuotationCashflow prjQuotationCashflowParameter = new HlsCusPrjQuotationCashflow();
        prjQuotationCashflowParameter.setQuotationId(ProjectId);
        List<HlsCusPrjQuotationCashflow> prjQuotationCashflowList = prjQuotationCashflowMapper.queryPrjQuotationCashflowByProjectId(prjQuotationCashflowParameter);
        if (prjQuotationCashflowList.size() > 0) {
            c.setFirstPayDate(prjQuotationCashflowList.get(0).getDueDate());
            c.setLeaseEndDate(prjQuotationCashflowList.get(0).getDueDate());
        }
        for (int i = 0; i < prjQuotationCashflowList.size(); i++) {
            HlsCusConContractCashflow conContractCashflow = new HlsCusConContractCashflow();
            if (prjQuotationCashflowList.get(i).getDueDate() != null) {
                //新加判断 报价现金流的dueDate可能为空
                if (prjQuotationCashflowList.get(i).getCfItem() == 1 && (prjQuotationCashflowList.get(i).getDueDate().getTime() - c.getFirstPayDate().getTime()) < 0) {
                    c.setFirstPayDate(prjQuotationCashflowList.get(i).getDueDate());
                }
                if (prjQuotationCashflowList.get(i).getCfItem() == 1 && (prjQuotationCashflowList.get(i).getDueDate().getTime() - c.getLeaseEndDate().getTime()) > 0) {
                    c.setLeaseEndDate(prjQuotationCashflowList.get(i).getDueDate());
                }
            }
            //由于创建cashflowDto的人代码书写不规范，所以返回下滑线型的map,ws
            //同样这个问题，我在setFieldValue中将用到Map前将他转为驼峰发命名，zy 2017.6.27
            Map<String, String> map = hlsBeanRefUtilService.getFieldValueMap(prjQuotationCashflowList.get(i));
            hlsBeanRefUtilService.setFieldValue(conContractCashflow, map);
            conContractCashflow.setContract_id(contractId);
            conContractCashflow.setContract_status("NEW");
            conContractCashflow.setWrite_off_flag("NOT");
            conContractCashflow.setBilling_status("NOT");
            self().insertSelective(iRequest, conContractCashflow);
            conContractCashflowList.add(conContractCashflow);
        }
        conContractMapper.updateByPrimaryKey(c);
        return conContractCashflowList;
    }

    @Override
    public void paymentApproval(IRequest iRequest, HttpSession session, List<HlsCusConContractCashflow> list) throws BeyondAmountLimitException {
        if (iConPaymentApprovalCustomService != null && !iConPaymentApprovalCustomService.beforePaymentApproval(iRequest, session, list)) {
            return;
        }
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
        Long companyId = ((Long) session.getAttribute("companyId"));
        String value = getCodeValue(iRequest);

        if (list == null || list.size() < 1) {
            return;
        }
        Long contract_id = list.get(0).getContract_id();
        HlsCusConContract conContract = new HlsCusConContract();
        conContract.setContractId(contract_id);
        conContract = conContractMapper.selectByPrimaryKey(conContract);
        if (conContract == null) {
            return;
        }
        // lock con_contract
        databaseLockProvider.lock(conContract);

        //拿到所有的付款申请行
        List<HlsCusCshPaymentReqLn> ss = self().creatPayInfo(list, companyId, value, iRequest);

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "PAY_PAYMENT");
        params.put("cshPaymentReqLn", ss.get(0));
        activitiStartService.start(iRequest, list, params);

        HlsCusCshPaymentReqHd hd = new HlsCusCshPaymentReqHd();
        hd.setPayment_req_id(ss.get(0).getPayment_req_id());
        hd.setPayment_req_status("APPROVING");
        paymentReqHdService.updateByPrimaryKeySelective(iRequest, hd);
        if (iConPaymentApprovalCustomService != null) {
            iConPaymentApprovalCustomService.afterPaymentApproval(iRequest, session, list);
        }
    }

    public String getCodeValue(IRequest requestContext) {
        Map<String, String> params = new HashMap<String, String>();
        String value = codingRuleValuesService.getCodeRuleValue(requestContext, "CSH_PAYMENT_REQ", "PAYMENT_REQ", "PAYMENT_REQ", params);
        return value;
    }

    @Override
    public List<HlsCusConContractCashflow> queryConContratCashFlowById(Long cashflow_id) {
        return cashflowMapper.queryConContratCashFlowById(cashflow_id);
    }

    /**
     * 首页 资产管理 应收金额逾期、未逾期查询
     *
     * @param cashflow
     * @param session
     * @return
     * @throws ParseException
     * @throws ChangeLimitException
     */
    @Override
    public HlsCusConContractCashflow selectDueAmountInDiff(HlsCusConContractCashflow cashflow, HttpSession session) throws ParseException, ChangeLimitException {

        Long fate = cashflow.getQueryTimeSolt();

        String beginDueDate = cashflow.getDue_date_statop();
        String endDueDate = cashflow.getDue_date_out();

        SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");

        if (fate != null) {

            Date beginDate = new Date();
            Date endDate = new Date();
            Calendar date1 = Calendar.getInstance();
            ;            // 获取当前日期
            Calendar date2 = Calendar.getInstance();

            date1.setTime(beginDate);
            date2.setTime(endDate);
            if (fate == 1) {
                cashflow.setBeginDueDate(null);
                date2.set(date2.DATE, date2.get(date2.DATE));
                cashflow.setEndDueDate(dft.parse(dft.format(date2.getTime())));
            }
            if (fate == 7) {

                date1.set(date1.DATE, date1.get(date1.DATE));
                date2.add(date2.DATE, date2.get(date2.DATE) + 7);
                cashflow.setBeginDueDate(dft.parse(dft.format(date1.getTime())));
                cashflow.setEndDueDate(dft.parse(dft.format(date2.getTime())));

            }
            if (fate == 30) {

                date1.set(date1.DATE, date1.get(date1.DATE));
                date2.add(date2.MONTH, 1);
                cashflow.setBeginDueDate(dft.parse(dft.format(date1.getTime())));
                cashflow.setEndDueDate(dft.parse(dft.format(date2.getTime())));

            }
        } else {
            if (beginDueDate != null && beginDueDate != "") {
                cashflow.setBeginDueDate(dft.parse(beginDueDate));
            }
            if (endDueDate != null && endDueDate != "") {
                cashflow.setEndDueDate(dft.parse(endDueDate));
            }
        }
        cashflow = cashflowMapper.selectDueAmountInDiff(cashflow);
        if (null == cashflow) {
            return new HlsCusConContractCashflow();
        }
        return cashflow;
    }

    /**
     * 首页-资产管理-阶段内到期应收金额统计
     *
     * @return
     */
    @Override
    public ConAging SelectDueAmount(HlsCusConContractCashflow cashflow) throws ParseException {

        Long fate = cashflow.getQueryTimeSolt();

        SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");

        Date endDate = new Date();    // 获取当前日期
        Calendar date2 = Calendar.getInstance();
        date2.setTime(endDate);

        if (fate != null) {
            if (fate == 1) {
                cashflow.setBeginDueDate(null);
            } else {
                date2.set(date2.DATE, date2.get(date2.DATE));
                cashflow.setBeginDueDate(dft.parse(dft.format(date2.getTime())));
            }
        }

        ConAging conAging = cashflowMapper.SelectDueAmount(cashflow);
        if (null == conAging) {
            return new ConAging();
        }
        return conAging;
    }

    /**
     * 租金催收信息查询
     *
     * @param cashflow
     * @return
     * @throws ParseException
     */
    @Override
    public List<HlsCusConContractCashflow> selectRentBackInfo(IRequest iRequest ,HlsCusConContractCashflow cashflow, int page , int pageSize) throws ParseException {

        Long fate = cashflow.getQueryTimeSolt();

        String beginDueDate = cashflow.getDue_date_statop();
        String endDueDate = cashflow.getDue_date_out();

        SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");

        if (fate != null) {
            Date beginDate = new Date();
            Date endDate = new Date();
            Calendar date1 = Calendar.getInstance();
            ;            // 获取当前日期
            Calendar date2 = Calendar.getInstance();

            date1.setTime(beginDate);
            date2.setTime(endDate);
            if (fate == 1) {
                cashflow.setBeginDueDate(null);
                date2.set(date2.DATE, date2.get(date2.DATE));
                cashflow.setEndDueDate(dft.parse(dft.format(date2.getTime())));
            }
            if (fate == 7) {
                date1.set(date1.DATE, date1.get(date1.DATE));
                date2.add(date2.DATE, date2.get(date2.DATE) + 7);

                cashflow.setBeginDueDate(dft.parse(dft.format(date1.getTime())));
                cashflow.setEndDueDate(dft.parse(dft.format(date2.getTime())));

            }
            if (fate == 30) {
                date1.set(date1.DATE, date1.get(date1.DATE));
                date2.add(date2.MONTH, 1);
                cashflow.setBeginDueDate(dft.parse(dft.format(date1.getTime())));
                cashflow.setEndDueDate(dft.parse(dft.format(date2.getTime())));
            }

        } else {
            if (beginDueDate != null && beginDueDate != "") {
                cashflow.setBeginDueDate(dft.parse(beginDueDate));
            }
            if (endDueDate != null && endDueDate != "") {
                cashflow.setEndDueDate(dft.parse(endDueDate));
            }
        }
        PageHelper.startPage(page, pageSize);
        List<HlsCusConContractCashflow> list = cashflowMapper.selectRentBackInfo(cashflow);
        return list;
    }

    /**
     * 根据每个合同ID查询现金流
     *
     * @param contractID
     * @return
     */
    @Override
    public List<HlsCusConContractCashflow> astDetail(Long contractID) {
        return cashflowMapper.selectRentBackStatus(contractID);
    }

    /**
     * 租金催收历史记录
     *
     * @return
     */
    @Override
    public List<HlsCusConContractCollection> selectHisOfRentBackIf(IRequest iRequest, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        return cashflowMapper.selectHisOfRentBackIf();
    }

    //报价计算结束后 更新其他费用项的期数
    private HlsCusConContractCashflow updateOtherCfItemTimes(HlsCusConContractCashflow cashflow) {
        if (cashflow.getCf_item() == 0 || cashflow.getCf_item() == 2 || cashflow.getCf_item() == 3 || cashflow.getCf_item() == 5 || cashflow.getCf_item() == 6) {
            cashflow.setTimes(0L);
        } else if (cashflow.getCf_item() == 8) {
            cashflow.setTimes(cashflow.getTimes());
        }
        return cashflow;
    }

    @Override
    public List<HlsCusConContractCashflow> saveCalc2ConContractCashflow(IRequest requestContext, HlsCusPrjQuotation prjQuotation) throws Exception {
        List<Object> ObjectList = hlsCalcExcelImportUtilService.getExcelToCalcLnTable(requestContext, prjQuotation.getSheets(), prjQuotation.getPriceList(), "csh", Integer.parseInt(prjQuotation.getLeaseTimes().toString()), prjQuotation.getSourceDocumentCategory());

        List<HlsCusConContractCashflow> conContractCashflowList = new ArrayList<>();
        for (int i = 0; i < ObjectList.size(); i++) {
            HlsCusConContractCashflow conContractCashflow = (HlsCusConContractCashflow) ObjectList.get(i);
            conContractCashflow.setContract_id(prjQuotation.getSourceDocumentId());
            //下达
            conContractCashflow.setCf_status("RELEASE");
            //新增
            conContractCashflow.set__status("add");
            //conContractCashflow = updateOtherCfItemTimes(conContractCashflow);
            conContractCashflowList.add(conContractCashflow);
        }
        //删除这个合同项下现金流
        cashflowMapper.deleteByContractId(prjQuotation.getSourceDocumentId());
        //重新插入
        self().batchUpdate(requestContext, conContractCashflowList);
        return conContractCashflowList;
    }

    @Override
    public List<HlsCusConContractCashflow> selectCalculateCashflow(HlsCusConDebtExemptionReq conDebtExemptionReq) {
        return cashflowMapper.selectCalculateCashflow(conDebtExemptionReq);
    }

    /*合同变更 - 提前结清-现金流grid信息查询*/
    public Map queryCashFlowInfo(IRequest iRequest, HlsCusConContractCashflow conContractCashflow, int page, int pageSize) {
//        conContractCashflow.setSortorder("asc");
//        conContractCashflow.setSortname("cashflow_id");
        List<HlsCusConContractCashflow> conContractCashflowList = cashflowMapper.select(conContractCashflow);//查询所有现金流信息
        Long contractId = conContractCashflow.getContract_id();
        HlsCusConContractChangeReq hlsCusConContractChangeReq = new HlsCusConContractChangeReq();
        hlsCusConContractChangeReq.setContractId(contractId);
        hlsCusConContractChangeReq.setStatus(APPROVING);
        List<HlsCusConContractChangeReq> hlsCusConContractChangeReqs = conContractChangeReqMapper.select(hlsCusConContractChangeReq);
        HlsCusConContract contract = conContractMapper.selectByPrimaryKey(contractId);//查询相关合同信息
        Double rate = CalculateUtil.div(contract.getIntRate(), 360D, 6);//日利率
        Date settlementDate = conContractCashflow.getSettlementDate() != null ? conContractCashflow.getSettlementDate() : new Date();//提前结清日期
        Double remainingPrincipal = 0D;//剩余本金
        Double overDueInterest = 0D;//逾期利息
        List<Long> times = new ArrayList<>();
        for (HlsCusConContractCashflow item : conContractCashflowList) {
            if (item.getCashflow_id() != null && item.getCf_type() != 3 && item.getCf_item() != 113 && item.getCf_item() != 11 && item.getTimes().intValue() != 0) {//剩余本金
                times.add(item.getTimes());
                remainingPrincipal = CalculateUtil.add(remainingPrincipal, CalculateUtil.sub(item.getPrincipal(), item.getReceived_principal()));
            }
            if (item.getCf_item().intValue() == 1 && item.getDue_date().getTime() <= conContractCashflow.getSettlementDate().getTime()) {
                overDueInterest = CalculateUtil.add(overDueInterest, CalculateUtil.sub(item.getInterest(), item.getReceived_interest()));
            }
        }

        Map map = new HashMap<>();
        if (times != null && !times.isEmpty()) {
            Long minTimes = Collections.min(times);
            Date lastDueDate = null;
            for (HlsCusConContractCashflow item : conContractCashflowList) {
                if (item.getCashflow_id() != null && item.getTimes() != null && item.getTimes().intValue() == minTimes.intValue() - 1) {
                    lastDueDate = item.getDue_date();
                }
            }
            Long betweenDays = null;
            if (lastDueDate == null){
                map.put("message", "未找到可提前结清的现金流");
                map.put("success", false);
                return map;
            }
            try {
                betweenDays = daysBetween(lastDueDate, conContractCashflow.getSettlementDate());
            } catch (ParseException e) {
                e.printStackTrace();
            }


            //添加提前结清费用一条现金流
            HlsCusConContractCashflow terminationFee = new HlsCusConContractCashflow();
            terminationFee.setDue_date(settlementDate);
            terminationFee.setCf_item(ITEM);
            terminationFee.setTimes(Collections.max(times));
            terminationFee.setPrincipal(remainingPrincipal);//提前结清费用-应收本金
            Double earlyTerminationInterest = CalculateUtil.mul(CalculateUtil.mul(remainingPrincipal, betweenDays.doubleValue()), rate);
            terminationFee.setInterest(CalculateUtil.add(earlyTerminationInterest, overDueInterest));//提前结清费用-应收利息
            terminationFee.setDue_amount(CalculateUtil.add(terminationFee.getPrincipal(),terminationFee.getInterest()));
            if (hlsCusConContractChangeReqs == null || hlsCusConContractChangeReqs.isEmpty()) {//当在流程中时
                conContractCashflowList.add(terminationFee);
            }
            map.put("settle", CalculateUtil.mul(CalculateUtil.mul(remainingPrincipal, betweenDays.doubleValue()), rate));//提前结清利息
            map.put("success", true);
            map.put("interest", earlyTerminationInterest);
            map.put("principal", remainingPrincipal);
            map.put("maxTime",Collections.max(times));
        } else {
            map.put("message", "提前结清日期不合法");
            map.put("success", false);
        }
        map.put("list", page(conContractCashflowList, page, pageSize));//现金流信息
        map.put("total", new Long((long) conContractCashflowList.size()));//总条数
        return map;
    }

    /**
     * 合同变更 - 结清信息
     *
     * @param conContractCashflow
     * @return
     */
    public Map<String, Object> settleInformation(IRequest iRequest, HlsCusConContractCashflow conContractCashflow) {
        final Map<String, Object> result = new HashMap<>();
        HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
        HlsCusCshTransaction transaction = new HlsCusCshTransaction();
        cashflow.setContract_id(conContractCashflow.getContract_id());
        List<HlsCusConContractCashflow> conContractCashflows = cashflowMapper.select(cashflow);//获取现金流信息
        transaction.setContractId(conContractCashflow.getContract_id());
        transaction.setTransactionType(DEPOSIT);
        List<HlsCusCshTransaction> cshTransactions = transactionMapper.select(transaction);//获取保证金类型的现金事务信息
        Double overdueRent = 0D;//逾期租金
        Double overduePenalty = 0D;//逾期罚息
        Double nominalPrice = 0D;//名义货价
        Double remainingPrincipal = 0D;//剩余本金
        Double marginBalance = 0D;//保证金余额
        //Double rate = null;//利率
        Double fee = null;//提前结清费用
        Double totalInterest = 0D;//总利息
        Double afterInterest = 0D;
        Double firstTimeAmount = 0D;//第0期金额
        Double serviceCharge = 0D;//报价手续费
        Double deposit = null;
        for (HlsCusConContractCashflow item : conContractCashflows) {
            if (item.getTimes() != null && item.getTimes().intValue() == 0 && OUTFLOW.equalsIgnoreCase(item.getCf_direction()))
                firstTimeAmount = CalculateUtil.sub(0D, item.getDue_amount());
            if ("Y".equalsIgnoreCase(item.getOverdueStatus()) && item.getCf_item().intValue() == 1) {
                //当状态为逾期 现金流为租金时 计算逾期租金
                overdueRent = CalculateUtil.add(overdueRent, CalculateUtil.sub(item.getDue_amount(), item.getReceived_amount()));
            }
            if (item.getCf_item().intValue() == 9) {//逾期罚息
                overduePenalty = CalculateUtil.add(overduePenalty, CalculateUtil.sub(item.getDue_amount(), item.getReceived_amount()));
            }
            if (item.getCf_item().intValue() == 8) {//名义货价
                nominalPrice = CalculateUtil.add(nominalPrice, CalculateUtil.sub(item.getDue_amount(), item.getReceived_amount()));
            }
            //剩余本金
            if (item.getCf_type().intValue() != 3 && item.getCf_item().intValue() != 113 && item.getCf_item().intValue() != 11) {
                remainingPrincipal = CalculateUtil.add(remainingPrincipal, CalculateUtil.sub(item.getPrincipal(), item.getReceived_principal()));
            }
            if (item.getCf_type().intValue() == 5 && item.getCf_item().intValue() == 5){
                deposit = item.getDue_amount();
            }
            if ("FULL".equalsIgnoreCase(item.getWrite_off_flag())) {
                afterInterest = CalculateUtil.add(afterInterest, item.getInterest());
            }
//            if (item.getCf_type().intValue() == 3 && item.getCf_item().intValue() == 103) {//提前结清费用
//                fee = item.getDue_amount();
//            }
            if (item.getCf_item().intValue() == 3 && item.getCf_type() == 3){ //手续费
                serviceCharge = CalculateUtil.add(serviceCharge,item.getDue_amount());
            }
            totalInterest = CalculateUtil.add(totalInterest, item.getInterest());
        }
        for (HlsCusCshTransaction item : cshTransactions) {//保证金余额
            marginBalance = CalculateUtil.add(marginBalance, CalculateUtil.sub(item.getTransactionAmount(), item.getWriteOffAmount()));
        }
        Map map = queryCashFlowInfo(iRequest, conContractCashflow, 0, 0);//获取提前结清费用
        List<HlsCusConContractCashflow> conContractCashflowList = (List<HlsCusConContractCashflow>) map.get("list");
        final List<Double> list = new ArrayList<>();
        list.add(firstTimeAmount);
        conContractCashflowList.forEach(item -> list.add(item.getDue_amount()));
        list.add(conContractCashflow.getFee());//获取手续费
        result.put("overdueRent", overdueRent);//逾期租金
        result.put("overduePenalty", overduePenalty);//逾期罚息
        result.put("nominalPrice", nominalPrice);//名义货价
        result.put("remainingPrincipal", remainingPrincipal);//剩余本金
        result.put("marginBalance", marginBalance);//保证金余额
        result.put("settle", map.get("settle"));//提前结清利息
        Double calResult = CalculateUtil.add(overdueRent, overduePenalty);
        calResult = CalculateUtil.add(calResult, nominalPrice);
        calResult = CalculateUtil.add(calResult, remainingPrincipal);
        calResult = CalculateUtil.add(calResult, (Double) map.get("interest"));
        calResult = CalculateUtil.add(calResult, conContractCashflow.getFee());
        //calResult = CalculateUtil.add(calResult, deposit);
        calResult = CalculateUtil.sub(calResult, marginBalance);//保证金余额
        result.put("actualSettlement", calResult);//实际结清费用
        result.put("lossInterest", CalculateUtil.sub(CalculateUtil.sub(totalInterest, afterInterest), conContractCashflow.getFee()));
        result.put("terminationInterest", map.get("interest"));
        result.put("terminationPrincipal", map.get("principal"));
        result.put("fee", fee);
        result.put("maxTime",map.get("maxTime"));
        result.put("success", map.get("success"));
        result.put("msg", map.get("message"));
        return result;
    }

    /**
     * 对手动拼接的list进行分页
     *
     * @param list
     * @param page
     * @param pageSize
     * @return
     */
    private <T> List<T> page(List<T> list, int page, int pageSize) {
        if (list == null || list.isEmpty())
            return list;
        if (page == 0 && pageSize == 0)
            return list;
        int from = pageSize * (page - 1);
        int to = pageSize * page;
        if (list.size() < to)
            return list.subList(from, list.size());
        return list.subList(from, to);
    }

    /**
     * 比较两个日期相差的天数
     *
     * @param smdate 小的日期
     * @param bdate  大的日期
     * @return
     * @throws ParseException
     */
    public static long daysBetween(Date smdate, Date bdate) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        smdate = sdf.parse(sdf.format(smdate));
        bdate = sdf.parse(sdf.format(bdate));
        Calendar cal = Calendar.getInstance();
        cal.setTime(smdate);
        long time1 = cal.getTimeInMillis();
        cal.setTime(bdate);
        long time2 = cal.getTimeInMillis();
        long between_days = (time2 - time1) / (1000 * 3600 * 24);

        return between_days;
    }
    public Map<String, Object> settleInformationByStatus(IRequest iRequest, HlsCusConContractCashflow conContractCashflow){
        Map<String, Object> resultMap = settleInformation(iRequest,conContractCashflow);//获取新建情况的结清信息
        if("APPROVING".equals(conContractCashflow.getCheckStatus())||"REJECTED".equals(conContractCashflow.getCheckStatus())){
            Double fee=0D;
            //根据changereqid获取申请信息
            HlsCusConContractChangeReq dto = conContractChangeReqMapper.selectByPrimaryKey(conContractCashflow.getChangeReqId());
            //根据changereqid获取历史表中的现金流
            HlsCusConContractH conContractH = new HlsCusConContractH();
            conContractH.setChangeReqId(dto.getChangeReqId());
            conContractH.setVersionType("CHANGE_REQ");
            List<HlsCusConContractCashflowH> cashflowHList = cashflowHMapper.selectHistoryCashflowByContract(conContractH);
            for(HlsCusConContractCashflowH item:cashflowHList){
                if (item.getCfType().intValue() == 3 && item.getCfItem().intValue() == 113) {
                    //提前结清手续费
                    fee = item.getDueAmount();
                }
            }
            Double actualSettlement = (Double)resultMap.get("actualSettlement");
            actualSettlement = CalculateUtil.add(actualSettlement,fee);
            resultMap.put("actualSettlement",actualSettlement);
            resultMap.put("fee",fee);
            resultMap.put("terminationDate",dto.getChangeReqDate());
        } else if("APPROVED".equals(conContractCashflow.getCheckStatus())){
             //审批通过
            HlsCusConContractChangeReq changeReq = new HlsCusConContractChangeReq();
            changeReq.setChangeReqId(conContractCashflow.getChangeReqId());
            resultMap = this.settleInformationAfterTask(iRequest,changeReq);
        }
        return resultMap;
    }

    /**
     * 流程结束后的提前结清利息相关信息
     * @param request
     * @param changeReq
     * @return
     */
    public Map<String, Object> settleInformationAfterTask(IRequest request, HlsCusConContractChangeReq changeReq) {
        Map<String, Object> resultMap = new HashMap<>();
        Long changeReqId = changeReq.getChangeReqId();
        HlsCusHlsDocumentVersion documentVersion = new HlsCusHlsDocumentVersion();
        documentVersion.setSourceDocumentCategory(CON_CONTRACT_CHANGE_REQ);
        documentVersion.setSourceDocumentId(changeReqId);
        documentVersion.setVersionType(CHANGE_REQ);
        documentVersion = documentVersionMapper.selectOne(documentVersion);
        Long versionId = documentVersion.getVersionId();
        HlsCusHlsDocumentVersionRel documentVersionRel = new HlsCusHlsDocumentVersionRel();
        documentVersionRel.setVersionId(versionId);
        List<HlsCusHlsDocumentVersionRel> rels = documentVersionRelMapper.select(documentVersionRel);
        HlsCusHlsDocumentVersionRel contractRel = rels.stream().filter(item -> CON_CONTRACT.equalsIgnoreCase(item.getFromTable())).findAny().get();
        HlsCusCshTransaction transaction = new HlsCusCshTransaction();
        transaction.setContractId(contractRel.getFromPk());
        transaction.setTransactionType(DEPOSIT);
        List<HlsCusCshTransaction> cshTransactions = transactionMapper.select(transaction);//获取保证金类型的现金事务信息
        HlsCusConContractCashflowH cashflowH = new HlsCusConContractCashflowH();
        cashflowH.setContractId(contractRel.getToPk());
        List<HlsCusConContractCashflowH> contractCashflowHs = cashflowHMapper.select(cashflowH);
        HlsCusConContractH conContractH = conContractHMapper.selectByPrimaryKey(contractRel.getToPk());
        Double overdueRent = 0D;//逾期租金
        Double overduePenalty = 0D;//逾期罚息
        Double nominalPrice = 0D;//名义货价
        Double remainingPrincipal = 0D;//剩余本金
        Double marginBalance = 0D;//保证金余额
        Double fee = null;//提前结清费用
        Double totalInterest = 0D;//总利息
        Double afterInterest = 0D;
        Double serviceCharge = 0D;//报价手续费
        Date settlementDate = new Date();//提前结清日期
        Double deposit = null;
        Double overDueInterest = 0D;//逾期利息
        List<Long> times = new ArrayList<>();
        for (HlsCusConContractCashflowH item : contractCashflowHs) {
            if ("Y".equalsIgnoreCase(item.getOverdueStatus()) && item.getCfItem().intValue() == 1) {
                //当状态为逾期 现金流为租金时 计算逾期租金
                overdueRent = CalculateUtil.add(overdueRent, CalculateUtil.sub(item.getDueAmount(), item.getReceivedAmount()));
            }
            if (item.getCfItem().intValue() == 9) {//逾期罚息
                overduePenalty = CalculateUtil.add(overduePenalty, CalculateUtil.sub(item.getDueAmount(), item.getReceivedAmount()));
            }
            if (item.getCfItem().intValue() == 8) {//名义货价
                nominalPrice = CalculateUtil.add(nominalPrice, CalculateUtil.sub(item.getDueAmount(), item.getReceivedAmount()));
            }
            //剩余本金
            if ((item.getCfType().intValue() != 3 && item.getCfItem().intValue() != 113) && (item.getCfItem().intValue() != 11 && item.getCfType().intValue() != 11)) {
                remainingPrincipal = CalculateUtil.add(remainingPrincipal, CalculateUtil.sub(item.getPrincipal(), item.getReceivedPrincipal()));
            }
            if ("FULL".equalsIgnoreCase(item.getWriteOffFlag())) {
                afterInterest = CalculateUtil.add(afterInterest, item.getInterest());
            }
            if (item.getCfItem().intValue() == 3 && item.getCfType() == 3) { //手续费
                serviceCharge = CalculateUtil.add(serviceCharge, item.getDueAmount());
            }
            if (item.getCfType().intValue() == 3 && item.getCfItem().intValue() == 113) {//提前结清费用
                fee = item.getDueAmount();
                settlementDate = item.getDueDate();
            }
            if (item.getCfType().intValue() == 5 && item.getCfItem().intValue() == 5){
                deposit = item.getDueAmount();
            }
            if (item.getCfType() != 3 && item.getCfItem() != 113 && item.getCfItem() != 11 && item.getTimes().intValue() != 0) {//剩余本金
                times.add(item.getTimes());
            }
            if (item.getCfItem().intValue() == 1 && item.getDueDate().getTime() <= settlementDate.getTime()) {
                overDueInterest = CalculateUtil.add(overDueInterest, CalculateUtil.sub(item.getInterest(), item.getReceivedInterest()));
            }
            totalInterest = CalculateUtil.add(totalInterest, item.getInterest());
        }
        for (HlsCusCshTransaction item : cshTransactions) {//保证金余额
            marginBalance = CalculateUtil.add(marginBalance, CalculateUtil.sub(item.getTransactionAmount(), item.getWriteOffAmount()));
        }
        Long minTimes = Collections.min(times);
        Date lastDueDate = contractCashflowHs.stream().filter(item -> item.getCashflowId() != null && item.getTimes() != null && item.getTimes().intValue() == minTimes.intValue() - 1).findAny().get().getDueDate();
        Long betweenDays = null;
        try {
            betweenDays = daysBetween(lastDueDate, settlementDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Double rate = CalculateUtil.div(conContractH.getIntRate(), 360D, 6);//日利率
        Double earlyTerminationInterest = CalculateUtil.mul(CalculateUtil.mul(remainingPrincipal, betweenDays.doubleValue()), rate);
        resultMap.put("overdueRent", overdueRent);//逾期租金
        resultMap.put("overduePenalty", overduePenalty);//逾期罚息
        resultMap.put("nominalPrice", nominalPrice);//名义货价
        resultMap.put("remainingPrincipal", remainingPrincipal);//剩余本金
        resultMap.put("marginBalance", marginBalance);//保证金余额
        resultMap.put("settle", earlyTerminationInterest);//提前结清利息
        Double calResult = CalculateUtil.add(overdueRent, overduePenalty);
        calResult = CalculateUtil.add(calResult, nominalPrice);
        calResult = CalculateUtil.add(calResult, remainingPrincipal);
        calResult = CalculateUtil.add(calResult, CalculateUtil.add(earlyTerminationInterest, overDueInterest));
        calResult = CalculateUtil.add(calResult, fee);
        calResult = CalculateUtil.add(calResult, deposit);
        calResult = CalculateUtil.sub(calResult, marginBalance);//保证金
        resultMap.put("actualSettlement", calResult);//实际结清费用
        resultMap.put("lossInterest", CalculateUtil.sub(CalculateUtil.sub(totalInterest, afterInterest), fee));
        resultMap.put("terminationInterest", CalculateUtil.add(earlyTerminationInterest, overDueInterest));
        resultMap.put("terminationPrincipal", remainingPrincipal);
        resultMap.put("fee", fee);
        resultMap.put("terminationDate",settlementDate);
        return resultMap;
    }
}
