package hls.core.csh.service.impl;

import java.util.Date;
import java.util.List;

import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.service.ConContractService;
import hls.core.csh.service.IConContractChangeReqEventService;
import hls.core.hls.mapper.HlsUserRelationshipMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.cont.dto.HlsCusConContractChangeReq;
import hls.core.csh.service.ConContractChangeReqEventService;
import hls.core.hls.dto.HlsDocAuthority;
import hls.core.hls.dto.HlsSystemEvent;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsUserRelationship;
import hls.core.hls.service.HlsSystemEventService;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.hls.service.IHlsDocAuthorityService;
import hls.core.hls.service.IHlsUserRelationshipService;

 
@Service
@Transactional
public class ConContractChangeReqEventServiceImpl extends BaseServiceImpl<HlsCusConContractChangeReq>
		implements ConContractChangeReqEventService {
	@Autowired
	private HlsSystemEventService hlsSystemEventService;
	@Autowired
	private IHlsDocAuthorityService docAuthorityService;
	@Autowired
	private IHlsUserRelationshipService hlsUserRelationshipService;
	@Autowired
	private HlsSystemNoticeService hlsSystemNoticeService;
	@Autowired
	private HlsUserRelationshipMapper userRelationshipMapper;
	@Autowired
	private ConContractService conContractService;
	@Autowired(required = false)
	private IConContractChangeReqEventService iConContractChangeReqEventService;

	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_TENANT_CREATE = "合同变更-承租人变更-创建";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_TENANT_CONFRIM = "合同变更-承租人变更-确认";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_TENANT_CANCEL = "合同变更-承租人变更-取消";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_TENANT_REJECT = "合同变更-承租人变更-拒绝";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_CASHFLOW_CREATE = "合同变更-租金计划变更-创建";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_CASHFLOW_CONFRIM = "合同变更-租金计划变更-确认";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_CASHFLOW_CANCEL = "合同变更-租金计划变更-取消";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_CASHFLOW_REJECT = "合同变更-租金计划变更-拒绝";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_GUARANTOR_CREATE = "合同变更-担保人变更-创建";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_GUARANTOR_CONFRIM = "合同变更-担保人变更-确认";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_GUARANTOR_CANCEL = "合同变更-担保人变更-取消";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_GUARANTOR_REJECT = "合同变更-担保人变更-拒绝";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_LEASE_ITEM_CREATE = "合同变更-租赁物变更-创建";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_LEASE_ITEM_CONFRIM = "合同变更-租赁物变更-确认";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_LEASE_ITEM_CANCEL = "合同变更-租赁物变更-取消";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_LEASE_ITEM_REJECT = "合同变更-租赁物变更-拒绝";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_CREATE = "合同变更-合同文本变更-创建";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_CONFRIM = "合同变更-合同文本变更-确认";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_CANCEL = "合同变更-合同文本变更-取消";
	private static final String CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_REJECT = "合同变更-合同文本变更-拒绝";
	private static final String CON_CONTRACT_CHANGE_REQ_EARLY_TERMINATION_CREATE = "合同变更-提前结清-创建";
	private static final String CON_CONTRACT_CHANGE_REQ_EARLY_TERMINATION_CONFRIM = "合同变更-提前结清-确认";
	private static final String CON_CONTRACT_CHANGE_REQ_EARLY_TERMINATION_CANCEL = "合同变更-提前结清-取消";
	private static final String CON_CONTRACT_CHANGE_REQ_EARLY_TERMINATION_REJECT = "合同变更-提前结清-拒绝";

	private static String documentCategory = "CSH_TRANSACTION";
	private static String URL = "cont/CON100/contract_demo.view";
	private static String sourceModule = "CSH";
	private static String RelationType = "P2D";
	// private static String AuthorityTypeQuery = "1";
	private static String AuthorityTypeUpdate = "2";
	private static String documentType="STD";

	@Override
	public void conContractChangeReqEvent(IRequest iRequest, String eventCode, Long changeReqId,Long contractId) {
		String message = "";
		String noticeTitle = "";
		if (iConContractChangeReqEventService != null) {
			iConContractChangeReqEventService.beforeChange(iRequest, eventCode, changeReqId, contractId);
		}
		Long userId = iRequest.getUserId();
		HlsCusConContract conContract = conContractService.squerConContractById(contractId);
		switch (eventCode) {
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_TENANT.CREATE":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_TENANT_CREATE;
			noticeTitle = "合同【"+conContract.getContractName()+"】承租人变更创建成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_TENANT.CONFRIM":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_TENANT_CONFRIM;
			noticeTitle = "合同【"+conContract.getContractName()+"】承租人变更确认成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_TENANT.CANCEL":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_TENANT_CANCEL;
			noticeTitle = "合同【"+conContract.getContractName()+"】承租人变更已被取消";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_TENANT.REJECT":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_TENANT_REJECT;
			noticeTitle = "合同【"+conContract.getContractName()+"】承租人变更审批已被拒绝";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_CASHFLOW.CREATE":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_CASHFLOW_CREATE;
			noticeTitle = "合同【"+conContract.getContractName()+"】租金计划变更创建成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_CASHFLOW.CONFRIM":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_CASHFLOW_CONFRIM;
			noticeTitle = "合同【"+conContract.getContractName()+"】租金计划变更确认成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_CASHFLOW.CANCEL":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_CASHFLOW_CANCEL;
			noticeTitle = "合同【"+conContract.getContractName()+"】租金计划变更已被取消";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_CASHFLOW.REJECT":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_CASHFLOW_REJECT;
			noticeTitle = "合同【"+conContract.getContractName()+"】租金计划变更审批已被拒绝";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_GUARANTOR.CREATE":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_GUARANTOR_CREATE;
			noticeTitle = "合同【"+conContract.getContractName()+"】担保人变更创建成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_GUARANTOR.CONFRIM":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_GUARANTOR_CONFRIM;
			noticeTitle = "合同【"+conContract.getContractName()+"】担保人变更确认成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_GUARANTOR.CANCEL":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_GUARANTOR_CANCEL;
			noticeTitle = "合同【"+conContract.getContractName()+"】担保人变更已被取消";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_GUARANTOR.REJECT":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_GUARANTOR_REJECT;
			noticeTitle = "合同【"+conContract.getContractName()+"】担保人变更审批已被拒绝";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_LEASE_ITEM.CREATE":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_LEASE_ITEM_CREATE;
			noticeTitle = "合同【"+conContract.getContractName()+"】租赁物变更创建成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_LEASE_ITEM.CONFRIM":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_LEASE_ITEM_CONFRIM;
			noticeTitle = "合同【"+conContract.getContractName()+"】租赁物变更确认成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_LEASE_ITEM.CANCEL":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_LEASE_ITEM_CANCEL;
			noticeTitle = "合同【"+conContract.getContractName()+"】租赁物变更已被取消";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_LEASE_ITEM.REJECT":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_LEASE_ITEM_REJECT;
			noticeTitle = "合同【"+conContract.getContractName()+"】租赁物变更审批已被拒绝";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_CONTRACT_TEXT.CREATE":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_CREATE;
			noticeTitle = "合同【"+conContract.getContractName()+"】合同文本变更创建成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_CONTRACT_TEXT.CONFRIM":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_CONFRIM;
			noticeTitle = "合同【"+conContract.getContractName()+"】合同文本变更确认成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_CONTRACT_TEXT.CANCEL":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_CANCEL;
			noticeTitle = "合同【"+conContract.getContractName()+"】合同文本变更已被取消";
			break;
		case "CON_CONTRACT_CHANGE_REQ.CHANGE_CONTRACT_TEXT.REJECT":
			message = CON_CONTRACT_CHANGE_REQ_CHANGE_CONTRACT_TEXT_REJECT;
			noticeTitle = "合同【"+conContract.getContractName()+"】合同文本变更审批已被拒绝";
			break;
		case "CON_CONTRACT_CHANGE_REQ.EARLY_TERMINATION.CREATE":
			message = CON_CONTRACT_CHANGE_REQ_EARLY_TERMINATION_CREATE;
			noticeTitle = "合同【"+conContract.getContractName()+"】提前结清变更创建成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.EARLY_TERMINATION.CONFRIM":
			message = CON_CONTRACT_CHANGE_REQ_EARLY_TERMINATION_CONFRIM;
			noticeTitle = "合同【"+conContract.getContractName()+"】提前结清变更确认成功";
			break;
		case "CON_CONTRACT_CHANGE_REQ.EARLY_TERMINATION.CANCEL":
			message = CON_CONTRACT_CHANGE_REQ_EARLY_TERMINATION_CANCEL;
			noticeTitle = "合同【"+conContract.getContractName()+"】提前结清变更已被取消";
			break;
		case "CON_CONTRACT_CHANGE_REQ.EARLY_TERMINATION.REJECT":
			message = CON_CONTRACT_CHANGE_REQ_EARLY_TERMINATION_REJECT;
			noticeTitle = "合同【"+conContract.getContractName()+"】提前结清变更审批已被拒绝";
			break;
		default:
			break;
		}
		HlsSystemEvent event = new HlsSystemEvent();
		event.setEventCode(eventCode);
		event.setSourceDocumentId(changeReqId);
		event.setEventUserId(userId);
		event.setSourceDocumentCategory(documentCategory);
		event.setLastUpdatedBy(userId);
		event.setLastUpdateDate(new Date());
		// 系统通知
		HlsSystemNotice hlsSystemNotice = new HlsSystemNotice();
		hlsSystemNotice.setNotice_message(message);
		hlsSystemNotice.setNotice_title(noticeTitle);
		hlsSystemNotice.setNotice_datetime(new Date());
		hlsSystemNotice.setNotice_type("NOTICE");
		hlsSystemNotice.setNotice_url(URL);
		hlsSystemNotice.setNotice_level(2L);
		hlsSystemNotice.setSource_module(sourceModule);
		hlsSystemNotice.setSource_event(eventCode);
		hlsSystemNotice.setSource_document_category(documentCategory);
		hlsSystemNotice.setSource_document_id(changeReqId);
		hlsSystemNotice.setSource_user_id(iRequest.getUserId());
//		hlsSystemNotice.setRead_flag("Y");
		hlsSystemNotice.setLastUpdatedBy(userId);
		hlsSystemNotice.setLastUpdateDate(new Date());

		HlsDocAuthority authority = new HlsDocAuthority();
		 authority.setUserId(userId);
		authority.setDocumentCategory(documentType);// 单据类型
		authority.setDocumentId(changeReqId);
		authority.setAuthorityType(AuthorityTypeUpdate);// 1查询，2，修改
		authority.setSourceModule(sourceModule);
		authority.setSourceEvent(eventCode);
		authority.setSourceDocumentCategory(documentCategory);
		authority.setSourceDocumentId(changeReqId);
		authority.setSourceUserId(iRequest.getUserId());
		authority.setLastUpdatedBy(userId);
		authority.setLastUpdateDate(new Date());

		HlsUserRelationship hlsUserRelationship = new HlsUserRelationship();
		hlsUserRelationship.setUserId(userId);
		hlsUserRelationship.setRelationType(RelationType);
		hlsUserRelationship.setSourceModule(sourceModule);
		hlsUserRelationship.setSourceEvent(eventCode);
		hlsUserRelationship.setSourceDocumentCategory(documentCategory);
		hlsUserRelationship.setSourceDocumentId(changeReqId);
		hlsUserRelationship.setSourceUserId(iRequest.getUserId());
		hlsUserRelationship.setLastUpdatedBy(userId);
		hlsUserRelationship.setLastUpdateDate(new Date());

		if("P2P".equalsIgnoreCase(hlsUserRelationship.getRelationType()))
		{
			hlsUserRelationship.setRelationUserId(iRequest.getUserId()); // 关联用户ID

			HlsUserRelationship userRelationship = new HlsUserRelationship();
			userRelationship.setUserId(hlsUserRelationship.getUserId());
			userRelationship.setRelationType(hlsUserRelationship.getRelationType());
			userRelationship.setRelationUserId(hlsUserRelationship.getRelationUserId());
			List<HlsUserRelationship> list =  userRelationshipMapper.select(userRelationship);
			if(list.size() == 0)
			{
				hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship );
			}
		}else if("P2D".equalsIgnoreCase(hlsUserRelationship.getRelationType())){
			hlsUserRelationship.setRelationDocumentCategory(documentCategory);// 关联单据类别
			hlsUserRelationship.setRelationDocumentId(changeReqId); // 关系单据ID

			HlsUserRelationship userRelationship1 = new HlsUserRelationship();
			userRelationship1.setUserId(hlsUserRelationship.getUserId());
			userRelationship1.setRelationType(hlsUserRelationship.getRelationType());
			userRelationship1.setSourceDocumentCategory(hlsUserRelationship.getSourceDocumentCategory());
			userRelationship1.setSourceDocumentId(hlsUserRelationship.getSourceDocumentId());
			List<HlsUserRelationship> lists =  userRelationshipMapper.select(userRelationship1);
			if(lists.size() == 0)
			{
				hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship );
			}
		}

		hlsSystemEventService.insert(iRequest, event);
		docAuthorityService.insert(iRequest, authority );
		hlsSystemNoticeService.insert(iRequest, hlsSystemNotice );
		if (iConContractChangeReqEventService != null) {
			iConContractChangeReqEventService.afterChange(iRequest, eventCode, changeReqId, contractId);
		}
	}
}
