package hls.core.csh.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusConDebtExemptionReq;
import hls.core.csh.dto.HlsCusConDebtExemptionReqCf;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.csh.mapper.HlsCusConDebtExemptionReqCfMapper;
import hls.core.csh.service.ConDebtExemptionReqCfService;
import hls.core.csh.service.ConDebtExemptionReqService;
import hls.core.csh.service.IAddChangeReqCfService;
import hls.core.csh.service.IConContractCashflowService;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by haibin on 2017/7/25.
 */
@Service
@Transactional
public class ConDebtExemptionReqCfServiceImpl extends BaseServiceImpl<HlsCusConDebtExemptionReqCf> implements ConDebtExemptionReqCfService {
    @Autowired
    IConContractCashflowService cashflowService;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private ConDebtExemptionReqService conDebtExemptionReqService;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private HlsCusConDebtExemptionReqCfMapper conDebtExemptionReqCfMapper;
    @Autowired
    private HlsCusConContractCashflowMapper conContractCashflowMapper;
    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired(required = false)
    private IAddChangeReqCfService iAddChangeReqCfService;

    @Override
    public List<HlsCusConDebtExemptionReqCf> addChangeReqCf(IRequest iRequest, List<HlsCusConContractCashflow> cashflowList) {
        List<HlsCusConDebtExemptionReqCf> list = new ArrayList<>();
        Long changeReqId = 0l;
        Long contractId = 0l;
        if (iAddChangeReqCfService != null) {
            iAddChangeReqCfService.beforeChange(iRequest, cashflowList);
        }
        for (HlsCusConContractCashflow contractCashflow : cashflowList) {
            HlsCusConContractCashflow conCashflow = new HlsCusConContractCashflow();
            conCashflow.setCashflow_id(contractCashflow.getDelayId());
            HlsCusConContractCashflow cashflow = cashflowService.selectByPrimaryKey(iRequest, conCashflow);
            HlsCusConDebtExemptionReqCf cf = new HlsCusConDebtExemptionReqCf();
            cf.setChangeReqId(contractCashflow.getChangeReqId());
            changeReqId = contractCashflow.getChangeReqId();
            cf.setCashflowId(cashflow.getCashflow_id());
            cf.setContractId(cashflow.getContract_id());
            contractId = cashflow.getContract_id();

            HlsCusConContract conContract = new HlsCusConContract();
            conContract.setContractId(contractId);
            conContract = conContractMapper.selectByPrimaryKey(conContract);
            // lock con_contract
            databaseLockProvider.lock(conContract);

            cf.setCfItem(cashflow.getCf_item());
            cf.setCfType(cashflow.getCf_type());
            cf.setCfDirection(cashflow.getCf_direction());
            cf.setCfStatus(cashflow.getCf_status());
            cf.setTimes(cashflow.getTimes());
            cf.setDueDate(cashflow.getDue_date());
            cf.setDueAmount(cashflow.getDue_amount());
            cf.setPrincipal(cashflow.getPrincipal());
            cf.setInterest(cashflow.getInterest());
            cf.setExemptionAmount(cashflow.getDue_amount() - contractCashflow.getExemptionAmount());
            cf.set__status("add");
            cf = self().insertSelective(iRequest, cf);
            list.add(cf);
        }
        HlsCusConDebtExemptionReq req = new HlsCusConDebtExemptionReq();
        req.setChangeReqId(changeReqId);
        req = conDebtExemptionReqService.selectByPrimaryKey(iRequest, req);
        List<HlsCusConDebtExemptionReqCf> cfList = new ArrayList();
        HlsCusConDebtExemptionReqCf reqCf = new HlsCusConDebtExemptionReqCf();
        reqCf.setChangeReqId(req.getChangeReqId());
        reqCf.setContractId(contractId);
        reqCf.setUserId(req.getReqUserId());
        cfList.add(reqCf);
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "CON_DEBT");
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
        activitiStartService.start(iRequest, cfList, params);
        if (iAddChangeReqCfService != null) {
            iAddChangeReqCfService.afterChange(iRequest, cashflowList);
        }
        return list;

    }

    @Override
    public List<HlsCusConDebtExemptionReqCf> selectExemptionReqCf(HlsCusConDebtExemptionReqCf conDebtExemptionReqCf) {
        return conDebtExemptionReqCfMapper.selectExemptionReqCf(conDebtExemptionReqCf);
    }

    @Override
    public List<HlsCusConDebtExemptionReqCf> selectExemptionReqCfById(HlsCusConDebtExemptionReqCf conDebtExemptionReqCf) {
        return conDebtExemptionReqCfMapper.selectExemptionReqCfById(conDebtExemptionReqCf);
    }

    @Override
    public List<HlsCusConContractCashflow> selectOverdueCashflow(HlsCusConContractCashflow conContractCashflow) {
        List<HlsCusConContractCashflow> cashflowList = conContractCashflowMapper.selectOverdueCashflow(conContractCashflow);
        List<HlsCusConContractCashflow> list = new ArrayList<HlsCusConContractCashflow>();
        if (cashflowList.size() > 0) {
            for (HlsCusConContractCashflow cashflow : cashflowList) {
                cashflow.setExemptionAmount(Double.parseDouble(cashflow.getDueAmountFormat().replace(",", "")));
                list.add(cashflow);
            }
        }
        return list;
    }
}
