package hls.core.csh.service.impl;

import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpSession;

import hls.core.cont.dto.*;
import hls.core.cont.service.ConContractAttachmentService;
import hls.core.cont.service.ConContractGuarantorService;
import hls.core.cont.service.ConContractLeaseItemService;
import hls.core.cont.service.ConContractMortgageService;
import hls.core.csh.dto.HlsCusCshConContract;
import hls.core.csh.mapper.HlsCusCshConContractMapper;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.prj.service.PrjProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.service.ICshConContractService;
import hls.core.hls.bp.dto.HlsCusBpMasterBankAccount;

 
@Service
@Transactional
public class CshConContractServiceImpl extends BaseServiceImpl<HlsCusCshConContract>implements ICshConContractService {

	@Autowired
	private HlsCusCshConContractMapper mapper;
	@Autowired
	private ConContractLeaseItemService contractLeaseItemService;
	@Autowired
	private ConContractMortgageService contractMortgageService;
	@Autowired
	private ConContractAttachmentService contractAttachmentService;
	@Autowired
	private ConContractGuarantorService conContractGuarantorservice;

	@Autowired
	private PrjProjectService prjProjectService;
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * hls.core.csh.service.ICshConContractService#selectAllConContract(hls.core
	 * .csh.dto.HlsCusCshConContract)
	 */
	@Override
	public List<HlsCusCshConContract> selectAllConContract(HlsCusCshConContract conContract) {
		return mapper.selectAllConContract(conContract);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * hls.core.csh.service.ICshConContractService#selectBank(java.lang.Long)
	 */
	@Override
	public List<HlsCusBpMasterBankAccount> selectBank(HlsCusCshConContract conContract) {
		return mapper.selectBank(conContract);
	}

	@Override
	public List<HlsCusCshConContract> getContractsByNumberOrName(HlsCusCshConContract conContract, int page, int pagesize) {
		PageHelper.startPage(page, pagesize);
		return mapper.getContractsByNumberOrName(conContract);
	}

	@Override
	public List<HlsCusCshConContract> contractDetail(IRequest requestContext, HlsCusCshConContract conContract, int page,
                                                     int pagesize) {
		PageHelper.startPage(page, pagesize);

		return mapper.contractDetail(conContract);
	}

	@Override
	public void save(IRequest iRequest,HlsCusCshConContract cshConContract) {

		cshConContract.setBusiness_type(cshConContract.getBusiness_type_desc());
		cshConContract.setDocument_type(cshConContract.getDocument_type_desc());
		cshConContract.setDocument_category(cshConContract.getDocumentCategoryDesc());

		if (cshConContract.getContract_id() == null) {
			self().insertSelective(iRequest,cshConContract);
		} else {
			self().updateByPrimaryKeySelective(iRequest,cshConContract);
		}
		if (cshConContract.getConContractLeaseItems() != null) {
			for (HlsCusConContractLeaseItem contractLeaseItem : cshConContract.getConContractLeaseItems()) {
				if (contractLeaseItem.getContractLeaseItemId() == null) {
					contractLeaseItemService.insertSelective(iRequest,contractLeaseItem);
				} else {
					contractLeaseItemService.updateByPrimaryKeySelective(iRequest,contractLeaseItem);
				}
			}
		}

		if (cshConContract.getConContractMortgages() != null) {
			for (HlsCusConContractMortgage contractMortgage : cshConContract.getConContractMortgages()) {
				if (contractMortgage.getContractMortgageId() == null) {
					contractMortgageService.insertSelective(iRequest,contractMortgage);
				} else {
					contractMortgageService.updateByPrimaryKeySelective(iRequest,contractMortgage);
				}
			}
		}
		if (cshConContract.getContractAttachments() != null) {
			for (HlsCusConContractAttachment contractAttachment : cshConContract.getContractAttachments()) {
				if (contractAttachment.getContractAttachmentId() == null) {
					contractAttachmentService.insertSelective(iRequest,contractAttachment);
				} else {
					contractAttachmentService.updateByPrimaryKeySelective(iRequest,contractAttachment);
				}
			}
		}
		if (cshConContract.getContractGuarantors() != null) {
			for (HlsCusConContractGuarantor contractGuarantors : cshConContract.getContractGuarantors()) {
				if (contractGuarantors.getContractGuarantorId() == null) {
					conContractGuarantorservice.insertSelective(iRequest,contractGuarantors);
				} else {
					conContractGuarantorservice.updateByPrimaryKeySelective(iRequest,contractGuarantors);
				}
			}
		}
	}

	@Override
	public Map<String, Double> contractAmountQuery() {
		Map<String, Double> list = new HashMap<String, Double>();

		for (int index = 0; index < 7; index++) {
			List<HlsCusCshConContract> conContracts = mapper.contractAmountQuery(index);
			if (conContracts != null) {

				list.put(sdf.format(conContracts.get(0).getDay()), conContracts.get(0).getAmount());
			}
		}

		return list;
	}

	/**
	 * 统计各状态下合同数量
	 * 
	 * @return
	 */
	@Override
	public HlsCusCshConContract selectAllStatistical(HttpSession session) {
		HlsCusCshConContract conContract = new HlsCusCshConContract();
		conContract.setCompany_id((Long)session.getAttribute("companyId"));
		return mapper.selectAllStatistical(conContract);
	}

	/**
	 * 统计合同总数
	 * 
	 * @return
	 */
	@Override
	public List<HlsCusCshConContract> getTotalContractCount() {
		return mapper.getTotalContractCount();
	}

	/**
	 * 统计不同状态下合同数量
	 * 
	 * @return
	 */
	@Override
	public List<HlsCusCshConContract> getDiffContractStatusCount() {
		return mapper.getDiffContractStatusCount();
	}

	/**
	 * 统计已结清合同总数
	 * 
	 * @return
	 */
	@Override
	public List<HlsCusCshConContract> getTerminateContractCount() {
		return mapper.getTerminateContractCount();
	}

	/**
	 * 统计存量合同 和 本月新增合同
	 * 
	 * @return
	 */
	@Override
	public List<HlsCusCshConContract> getStockContractAndMonthNewContract() {
		return mapper.getStockContractAndMonthNewContract();
	}

	@Override
	public List<HlsCusCshConContract> getIndexContractInceptInfo(IRequest requestContext, HlsCusCshConContract conContract, int page, int pagesize) {
		PageHelper.startPage(page, pagesize);
		List<HlsCusCshConContract> indexContractInceptInfo = mapper.getIndexContractInceptInfo(conContract);
		Date time = new Date();
		indexContractInceptInfo.forEach(item->{
			item.setLatestHours(BigInteger.valueOf((time.getTime() - item.getInceptionOfLease().getTime())/(1000 * 60)));
		});
		return indexContractInceptInfo;
	}

	/**
	 * 变更类型统计
	 * 
	 * @param changeReq
	 * @return
	 * @throws ParseException 
	 * @throws ChangeLimitException 
	 */
	@Override
	public HlsCusConContractChangeReq selectAlterStatistics(HlsCusConContractChangeReq changeReq,HttpSession session) throws ParseException, ChangeLimitException {
		Long fate = changeReq.getQueryTimeSolt();
		long companyId = (long) session.getAttribute("companyId");
		if(companyId<0){
			throw new ChangeLimitException("公司ID获取失败");
		}
		if (fate != null) {
			Date endDate = calculationDate(fate);
			changeReq.setChangeReqDate(endDate);
		}
		if(fate == 0){
			changeReq.setChangeReqDate(null);
		}
		changeReq.setCompanyId(companyId);
		changeReq = mapper.selectAlterStatistics(changeReq);
		if(changeReq == null){
			return new HlsCusConContractChangeReq();
		}
		return changeReq;
	}

	/**
	 * 合同变更信息查询
	 * 
	 * @param
	 * @return
	 * @throws ChangeLimitException 
	 */
	@Override
	public List<HlsCusConContractChangeReq> selectAllReqDetailedInformation(HlsCusConContractChangeReq conContractChangeReq,HttpSession session) throws ChangeLimitException {
		Long fate = conContractChangeReq.getQueryTimeSolt();
		Long company_id = (Long) session.getAttribute("companyId");
		if(company_id <0){
			throw new ChangeLimitException("公司ID获取失败");
		}
		if (fate != null) {
			Date endDate = calculationDate(fate);
			conContractChangeReq.setPastTime(endDate);
		}
		if("null".equalsIgnoreCase(conContractChangeReq.getStatus())){
			conContractChangeReq.setStatus(null);
		} if("null".equalsIgnoreCase(conContractChangeReq.getChangeType())){
            conContractChangeReq.setChangeType(null);
        }
		if(conContractChangeReq.getStatus()!=null){
			String [] strs = conContractChangeReq.getStatus().split(",");
			List<String> list = new ArrayList<>();
			for (int i = 0; i < strs.length; i++) {
				list.add(strs[i]);
			}
			conContractChangeReq.setList(list);
		}
		conContractChangeReq.setCompanyId(company_id);
		//  获取查询的变更类型
		conContractChangeReq = changeType(conContractChangeReq);
		List<HlsCusConContractChangeReq> list = mapper.selectAllReqDetailedInformation(conContractChangeReq);
		//设置显示变更类型
		setDisplay(list);
		return list;
	}

	/**
	 * 设置显示变更类型
	 * @param list
	 */
	private void setDisplay(List<HlsCusConContractChangeReq> list) {
		for (int i = 0; i < list.size(); i++) {
			ConContractChangeReq c = list.get(i);
			char[] ch = c.getContractReqType().toCharArray();
			String str = "";
			if (ch[0] == 'Y') {
				str += "还款计划变更、";
			}
			if (ch[1] == 'Y') {
				str += "租赁物变更、";
			}
			if (ch[2] == 'Y') {
				str += "担保人变更、";
			}
			if (ch[3] == 'Y') {
				str += "承租人变更、";
			}
			if (str != "")
				str = str.substring(0, str.length() - 1);
			list.get(i).setContractReqType(str);
		}
	}

	/**
	 * 获取当前查询的变更类型
	 * @param conContractChangeReq
	 * @return
	 */
	public HlsCusConContractChangeReq changeType(HlsCusConContractChangeReq conContractChangeReq){
        String changeType = conContractChangeReq.getChangeType();
		if(changeType !=null){
			switch (changeType){
				case "earlyTerminationFlag":
					conContractChangeReq.setEarlyTerminationFlag("Y");
					break;
				case "changeCashflowFlag":
					conContractChangeReq.setChangeCashflowFlag("Y");
					break;
				case "changeTenantFlag":
					conContractChangeReq.setChangeTenantFlag("Y");
					break;
				case "changeGuarantorFlag":
					conContractChangeReq.setChangeGuarantorFlag("Y");
					break;
				case "changeLeaseItemFlag":
					conContractChangeReq.setChangeLeaseItemFlag("Y");
					break;
				case "changeContractTextFlag":
					conContractChangeReq.setChangeContractTextFlag("Y");
					break;
			}
		}
		return conContractChangeReq;
	}

	/**
	 * 合同首页-基本信息-合同信息
	 * @param conContract
	 * @return
	 * @throws ChangeLimitException 
	 */
	@Override
	public List<HlsCusCshConContract> selectContractInFo(HlsCusCshConContract conContract, HttpSession session) throws ChangeLimitException{
		Long companyId = (Long) session.getAttribute("companyId");
		if(companyId<0){
			throw new ChangeLimitException("公司ID获取失败");
		}
		conContract.setCompany_id(companyId);
		return mapper.selectContractInFo(conContract);
	}
	/**
	 * 合同首页-基本信息-合同总数-   存量与新增合同树
	 * @param conContract
	 * @return
	 * @throws ParseException 
	 * @throws ChangeLimitException 
	 */
	@Override
	public List<HlsCusCshConContract> selectStockCount(HlsCusCshConContract conContract, HttpSession session) throws ParseException, ChangeLimitException{
		//String dateStr = calculationDate(30L);
		SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM");
		String strDate = dft.format(new Date());
		strDate+="-01";
		conContract.setCreateDate(sdf.parse(strDate));
		Long company_id = (Long) session.getAttribute("companyId");
		if(company_id<0){
			throw new ChangeLimitException("公司Id获取失败");
		}
		conContract.setCompany_id(company_id);
		return mapper.selectStockCount(conContract);
	}
	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
	private Date calculationDate(Long fate) {
		Date beginDate = new Date();
		Calendar date = Calendar.getInstance();
		date.setTime(beginDate);
		if (fate == 7) {
			date.set(Calendar.DATE, date.get(Calendar.DATE) - 7);
		}
		if (fate == 30) {
			date.add(Calendar.MONTH, -1);
		}
		if (fate == 90) {
			date.add(Calendar.MONTH, -3);
		}
//		String endDate = sdf.format(date.getTime());
		return date.getTime();
	}
	//SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");
	@Override
	public List<HlsCusCshConContract> selectTypeStatistics(HlsCusCshConContract conContract, HttpSession session) throws ParseException, ChangeLimitException {
		Long fate = conContract.getQueryTimeSolt();
		if (fate != null) {
			Date endDate = calculationDate(fate);
			conContract.setDue_date(endDate);
		}
		Long companyId = (Long) session.getAttribute("companyId");
		if(companyId<0){
			throw new ChangeLimitException("公司ID获取失败");
		}
		conContract.setCompany_id(companyId);
		return mapper.selectTypeStatistics(conContract);
	}

	@Override
	public Map selectTwoTypeStatistics(HlsCusCshConContract conContract, HttpSession session) throws ParseException, ChangeLimitException {
		Long fate = conContract.getQueryTimeSolt();
		if (fate != null) {
			Date endDate = calculationDate(fate);
			conContract.setDue_date(endDate);
		}
		Long companyId = (Long) session.getAttribute("companyId");
		if(companyId<0){
			throw new ChangeLimitException("公司ID获取失败");
		}
		conContract.setCompany_id(companyId);
		return mapper.selectTwoTypeStatistics(conContract);
	}

    @Override
    public List<HlsCusCshConContract> selectContractByID(HlsCusCshConContract conContract) {
        return mapper.selectContractByID(conContract);
    }
    /**
     * 合同首页-基本信息-合同信息查询
     * @param conContract
     * @return
     */
    @Override
    public List<HlsCusCshConContract> selectAllCshConContractInformation(HlsCusCshConContract conContract){
    	if("null".equalsIgnoreCase(conContract.getContract_status())){
    		conContract.setContract_status(null);
    	}
    	List<HlsCusCshConContract> list = mapper.selectAllCshConContractInformation(conContract);
    	return list;
    }

    @Override
	public List<HlsCusCshConContract> contractDueAmountQuery() {

		/*
		 * Map<String,Double> mapFirst = new HashMap<String,Double>();
		 * 
		 * Map<Date,Map<String,Double>> mapTwo = new
		 * HashMap<Date,Map<String,Double>>();
		 * 
		 * 
		 * List<HlsCusCshConContract> conContracts = mapper.contractDueAmountQuery();
		 * 
		 * for(HlsCusCshConContract conContract: conContracts) {
		 * mapFirst.put("sumDueAmount", conContract.getSumDueAmount());
		 * mapFirst.put("sumPrincipal", conContract.getSumPrincipal());
		 * mapFirst.put("sumInterest", conContract.getSumInterest());
		 * 
		 * mapTwo.put(conContract.getDue_date(), mapFirst); }
		 * 
		 * return mapTwo;
		 */
		return mapper.contractDueAmountQuery();
	}

	@Override
	public List<HlsCusConContractStatus> selectContractStatusInfo(HlsCusConContractStatus onContractStatus,HttpSession session) {
		onContractStatus.setCompanyId((Long)session.getAttribute("companyId"));
		return mapper.selectContractStatusInfo(onContractStatus);
	}

	@Override
	public List<HlsCusCshConContract> contractHomeFirstQuery(Map<String, Object> map) {
		final List<String> paymentList = new ArrayList<String>();
		boolean paymentFlag = false;
		if(!"N".equals(map.get("payment1"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment1"));
		}
		if(!"N".equals(map.get("payment2"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment2"));
		}
		if(!"N".equals(map.get("payment3"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment3"));
		}
		map.put("paymentList", paymentList);
		map.put("paymentFlag", paymentFlag);
		return mapper.contractHomeFirstQuery(map);
	}

	@Override
	public List<HlsCusCshConContract> contractHomeSecordQuery(Map<String, Object> map) {
		final List<String> paymentList = new ArrayList<String>();
		boolean paymentFlag = false;
		if(!"N".equals(map.get("payment1"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment1"));
		}
		if(!"N".equals(map.get("payment2"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment2"));
		}
		if(!"N".equals(map.get("payment3"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment3"));
		}
		map.put("paymentList", paymentList);
		map.put("paymentFlag", paymentFlag);
		return mapper.contractHomeSecordQuery(map);
	}

	@Override
	public List<HlsCusCshConContract> contractHomeThirdQuery(Map<String, Object> map) {

		final List<String> paymentList = new ArrayList<String>();
		boolean paymentFlag = false;
		if(!"N".equals(map.get("payment1"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment1"));
		}
		if(!"N".equals(map.get("payment2"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment2"));
		}
		if(!"N".equals(map.get("payment3"))) {
			paymentFlag = true;
			paymentList.add((String) map.get("payment3"));
		}
		map.put("paymentList", paymentList);
		map.put("paymentFlag", paymentFlag);

		return mapper.contractHomeThirdQuery(map);
	}

	SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");
	@Override
	public HlsCusConContractStatus selectTotalThrowInfo(IRequest requestContext,HlsCusConContractStatus conContractStatus) {
		conContractStatus.setCompanyId(requestContext.getCompanyId());
		Long fate = conContractStatus.getQueryTimeSolt();
		if (fate != null) {
			Date endDate = calculationDate(fate);
			try {
				conContractStatus.setDueDate(endDate);
			}catch (Exception e){
				e.printStackTrace();
			}
			if(fate== 0){
				conContractStatus.setDueDate(null);
			}
		}
		if("null".equalsIgnoreCase(conContractStatus.getDocumentType())){
			conContractStatus.setDocumentType(null);
		}
		return mapper.selectTotalThrowInfo(conContractStatus);
	}

	@Override
	public List<HlsCusCshConContract> comprehensiveQuery(Map<String, Object> map, int page, int pagesize) {
		PageHelper.startPage(page, pagesize);
		return mapper.comprehensiveQuery(map);
	}
	@Override
	public List<HlsCusCshConContract> selectUpToDateCshConConTract(HlsCusCshConContract cshConContract, HttpSession session){
		return mapper.selectUpToDateCshConConTract(cshConContract);
	}
	@Override
	public List<HlsCusCshConContract> selectOnHireCshconcontract(HlsCusCshConContract cshConContract, HttpSession session){
		return mapper.selectOnHireCshconcontract(cshConContract);
	}
	@Override
	public List<HlsCusCshConContract> selectCshConContract (Long bpId){
		return mapper.selectCshConContract(bpId);
	}

	public List<HlsCusCshConContract> selectCshConContractGuarantor(Long bpId){
		return mapper.selectCshConContractGuarantor(bpId);
	}
	@Override
	public List<HlsCusCshConContract> selectCshconcontractLov(){
		return mapper.selectCshconcontractLov();
	}

	@Override
	public List<Map> receivableCollection(HlsCusBpMaster hlsCusBpMaster) {
		return mapper.receivableCollection(mapper.selectContractId(hlsCusBpMaster));
	}

	public List<HlsCusCshConContract> rentQuery(Map<String, Object> map, int page, int pagesize){
		PageHelper.startPage(page, pagesize);
		return mapper.rentQuery(map);
	}
}
