package hls.core.csh.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusCshDeduction;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.csh.mapper.HlsCusCshDeductionMapper;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.service.*;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.gld.service.IJeTrxDtlService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by FJM on 2016/12/5.
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class CshDeductionServiceImpl extends BaseServiceImpl<HlsCusCshDeduction> implements CshDeductionService {

    @Autowired
    private HlsCusCshDeductionMapper cshDeductionMapper;
    @Autowired
    private HlsCusConContractCashflowMapper con;
    @Autowired
    private HlsCusCshTransactionMapper ctm;
    @Autowired
    private IJeTrxDtlService jeTrxDtlService;
    @Autowired
    private CshTransactionService cshTransactionService;
    @Autowired
    private CshWriteOffService cshWriteOffService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private JeTrxCommonService jeTrxCommonService;
    @Autowired(required = false)
    private IAddCshDeductionService iAddCshDeductionService;
    @Autowired(required = false)
    private IUpdateCshFlowService iUpdateCshFlowService;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    @Override
    public List<HlsCusCshWriteOff> addCshDeduction(IRequest iRequest, List<HlsCusCshWriteOff> cshwriteoffs) {
        //   HlsCusCshTransaction cshTransaction = new HlsCusCshTransaction();
        HlsCusConContractCashflow conContractCashflow = new HlsCusConContractCashflow();
        if (iAddCshDeductionService != null) {
            iAddCshDeductionService.beforeAdd(iRequest, cshwriteoffs);
        }
        for (HlsCusCshWriteOff cshwriteoff : cshwriteoffs) {

            //传入一个自定义的rflag，如果其为N则表示为反冲，则核销的金额，本金和利息都为负数
            if (cshwriteoff.getrFlag() != null && cshwriteoff.getrFlag().equals("N")) {
                cshwriteoff.setReversedFlag("R");
                cshwriteoff.setReversedDate(new Date());
                cshwriteoff.setReversedWriteOffId(cshwriteoff.getwId());
                //反冲的时候是插入一条金额为负数的记录
                cshwriteoff.setWriteOffDueAmount(-cshwriteoff.getWriteOffDueAmount());
                cshwriteoff.setWriteOffPrincipal(-cshwriteoff.getWriteOffPrincipal());
                cshwriteoff.setWriteOffInterest(-cshwriteoff.getWriteOffInterest());
            } else {
                cshwriteoff.setReversedFlag("N");
            }
            cshwriteoff.setCshWriteOffAmount(cshwriteoff.getWriteOffDueAmount());
            //           cshTransaction.setContractId(cshDeduction.getContractId());
/*            if (ctm.select(cshTransaction)!=null && ctm.select(cshTransaction).size()>0){
                Long tId = ctm.select(cshTransaction).get(0).getTransactionId();
                cshDeduction.setCshTransactionId(tId);//现金事物ID
                }*/
            if (cshwriteoff.getTransactionId() != null) {    //反冲时把从前台传进来的事物ID set到对应的字段上
                cshwriteoff.setCshTransactionId(cshwriteoff.getTransactionId());
            }
            conContractCashflow.setCashflow_id(cshwriteoff.getContractId());
            HlsCusCshTransaction cshTransaction = new HlsCusCshTransaction();
            cshTransaction.setTransactionId(cshwriteoff.getTransactionId());
            // lock csh_transaction
            databaseLockProvider.lock(cshTransaction);
            //lock con_contract_cashflow
            databaseLockProvider.lock(conContractCashflow);
            if (con.select(conContractCashflow) != null && con.select(conContractCashflow).size() > 0) {
                Long cId = con.select(conContractCashflow).get(0).getCashflow_id();//现金流ID
                Long cItem = con.select(conContractCashflow).get(0).getCf_item();//现金流性质
                Long cType = con.select(conContractCashflow).get(0).getCf_type();//现金流类型
            }
            cshWriteOffService.self().insertSelective(iRequest, cshwriteoff);
            //如果这个flag为N则表示该记录是反冲的，然后还需要更新另外一条writeoff记录
            if (cshwriteoff.getrFlag() != null && cshwriteoff.getrFlag().equals("N")) {
                cshwriteoff.setReversedFlag("Y");
                cshWriteOffService.self().updateByPrimaryKeySelective(iRequest, cshwriteoff);
            }
            UpdateCshTransaction(cshwriteoff);
            UpdateCshFlow(cshwriteoff);
            //插入凭证事物流水表 by 17.9.1
            AbstractJeTrxService abstractJeTrxService = jeTrxCommonService.map.get("CSH_WRITE_OFF");
            Map map = new HashMap<>();
            map.put("jeTrxId", cshwriteoff.getWriteOffId());
            map.put("companyId", con.selectByPrimaryKey(cshwriteoff.getCashflowId()).getCompany_id());
            map.put("contractId", cshwriteoff.getContractId());
            map.put("sourceDoc", "CON_CONTRACT");
            abstractJeTrxService.process(iRequest, map);

        }
        if (iAddCshDeductionService != null) {
            iAddCshDeductionService.afterAdd(iRequest, cshwriteoffs);
        }
        return cshwriteoffs;
    }


    @SuppressWarnings("unused")
    private void UpdateCshTransaction(HlsCusCshWriteOff data) {
        HlsCusCshTransaction ct = new HlsCusCshTransaction();
        ct.setTransactionId(data.getCshTransactionId());
        List<HlsCusCshTransaction> datas = ctm.select(ct);
        if (data != null && datas.size() > 0) {
            Double writeOff = data.getCshWriteOffAmount();//获取本次的核销金额
            Double writeOff2 = datas.get(0).getWriteOffAmount();//获取之前的核销金额总额
            Double transactionamount = datas.get(0).getTransactionAmount();//获取应收金额
            if (writeOff == null) {
                writeOff = new Double(0);
            }
            if (writeOff2 == null) {
                writeOff2 = new Double(0);
            }
            if (transactionamount == null) {
                transactionamount = new Double(0);
            }
            //防止null值相加报出异常
            if (writeOff + writeOff2 == transactionamount) { //将csh_write_off.csh_write_off_amount累加
                data.setTransactionId(data.getCshTransactionId());
                data.setWriteOffFlag("FULL");
                Date date = new Date();
                data.setFullWriteOffDate(date);
                cshDeductionMapper.updateCshTransaction(data);
            } else if (writeOff + writeOff2 == 0) {
                data.setTransactionId(data.getCshTransactionId());
                data.setWriteOffFlag("NOT");
                cshDeductionMapper.updateCshTransaction(data);
            } else if (writeOff + writeOff2 > transactionamount) {
                //抛出异常
                throw new IllegalArgumentException("核销金额不能大于现金事务金额！");
            } else {
                data.setTransactionId(data.getCshTransactionId());
                data.setWriteOffFlag("PARTIAL");
                cshDeductionMapper.updateCshTransaction(data);
            }
        }
    }

    private void UpdateCshFlow(HlsCusCshWriteOff data) {
        HlsCusConContractCashflow csh = new HlsCusConContractCashflow();
        if (iUpdateCshFlowService != null) {
            iUpdateCshFlowService.beforeUpdate(data);
        }
        //判断cashflowId是否为空
        if (data.getCashflowId() != null) {
            csh.setCashflow_id(data.getCashflowId());
        }
        //根据上面获取的现金流ID，查询出对应的现金流数据
        List<HlsCusConContractCashflow> cashflow = con.select(csh);
        Double reciveM = null;//定义一个变量接受应收金额
        Double reciveP = null;//定义一个变量接受应收本金
        Double reciveI = null;//定义一个变量接受应收利息
        if (cashflow.size() > 0) {
            //获取应收金额，应收本金和应收利息
            reciveM = cashflow.get(0).getDue_amount();
            reciveP = cashflow.get(0).getPrincipal();
            reciveI = cashflow.get(0).getInterest();
            //判断获取的金额是否为空并且校验已收金额不大于应收金额
            if (data.getCshWriteOffAmount() != null && (data.getCshWriteOffAmount() + cashflow.get(0).getReceived_amount() <= reciveM)) {
                Double principalM = null;
                Double principalP = null;
                if (data.getWriteOffPrincipal() == null) {
                    principalM = new Double(0);
                } else {
                    principalM = data.getWriteOffPrincipal();
                }
                if (cashflow.get(0).getReceived_principal() == null) {
                    principalP = new Double(0);
                } else {
                    principalP = cashflow.get(0).getReceived_principal();
                }
                if (cashflow.get(0).getPrincipal() != null && (principalM + principalP <= reciveP)) {
                    Double interestM = null;
                    Double interestP = null;
                    if (data.getWriteOffInterest() == null) {
                        interestM = new Double(0);
                    } else {
                        interestM = data.getWriteOffInterest();
                    }
                    if (cashflow.get(0).getReceived_interest() == null) {
                        interestP = new Double(0);
                    } else {
                        interestP = cashflow.get(0).getReceived_interest();
                    }
                    if (interestM + interestP <= reciveI) {
                        Date date = new Date();
                        data.setLastReceiveDate(date);
                        if (cashflow.get(0).getDue_amount() == cashflow.get(0).getReceived_amount() + data.getCshWriteOffAmount()) {
                            data.setWriteOffFlag("FULL");
                            data.setFullWriteOffDate(date);
                        } else if (cashflow.get(0).getReceived_amount() + data.getCshWriteOffAmount() == 0) {
                            data.setWriteOffFlag("NOT");
                        } else {
                            data.setWriteOffFlag("PARTIAL");
                        }
                        cshDeductionMapper.updateCshFlow(data);
                    } else {
                        //抛出异常？
                        throw new IllegalArgumentException("已收利息不能大于应收利息");
                    }
                } else {
                    //抛出异常？
                    throw new IllegalArgumentException("已收本金不能大于应收本金");
                }
            } else {
                //抛出异常?
                throw new IllegalArgumentException("已收金额不能大于应收金额");

            }
        }
        if (iUpdateCshFlowService != null) {
            iUpdateCshFlowService.afterUpdate(data);
        }

    }

    @Override
    public List<HlsCusCshDeduction> queryLov(HlsCusCshDeduction cshDeduction) {
        if (cshDeduction.getBpId() == -1) {
            return cshDeductionMapper.queryLovContract(cshDeduction);
        } else {
            return cshDeductionMapper.queryLovPool(cshDeduction);
        }

    }

    @Override
    public List<Map> homePageDepositMgr(Map map, int page, int pageSize, IRequest iRequest) {
        //PageHelper.startPage(page,pageSize);
        return cshDeductionMapper.homePageDepositMgr(map);
    }

    @Override
    public List<Map> homePagePoolDepositMgr(Map map, int page, int pageSize, IRequest iRequest) {
        PageHelper.startPage(page, pageSize);
        return cshDeductionMapper.homePagePoolDepositMgr(map);
    }

    @Override
    public Map homePageMgrWriteOff(Map map, int page, int pageSize, IRequest iRequest) throws ParseException {
        String flag = (String) map.get("flag");
        Date beginDate = new Date();
        Calendar[] date = new Calendar[5];
        Date[] endDate = new Date[5];
        if ("M".equals(flag)) {
            for (int j=0;j<date.length;j++){
                date[j] = Calendar.getInstance();
            }
            for (int i=0;i<5;i++){
                date[i].setTime(beginDate);
                date[i].add(Calendar.MONTH,-i);
                endDate[i] = sdf.parse(sdf.format(date[i].getTime()));
                map.put("n"+i,endDate[i]);
            }
            map.put("n", 1);
        } else if ("S".equals(flag)) {
            for (int j=0;j<date.length;j++){
                date[j] = Calendar.getInstance();
            }
            for (int i=0;i<5;i++){
                date[i].setTime(beginDate);
                date[i].add(Calendar.MONTH,-3*i);
                endDate[i] = sdf.parse(sdf.format(date[i].getTime()));
                map.put("n"+i,endDate[i]);
            }
            map.put("n", 3);
        } else if ("H".equals(flag)) {
            for (int j=0;j<date.length;j++){
                date[j] = Calendar.getInstance();
            }
            for (int i=0;i<5;i++){
                date[i].setTime(beginDate);
                date[i].add(Calendar.MONTH,-6*i);
                endDate[i] = sdf.parse(sdf.format(date[i].getTime()));
                map.put("n"+i,endDate[i]);
            }
            map.put("n", 6);
        } else {
            for (int j=0;j<date.length;j++){
                date[j] = Calendar.getInstance();
            }
            for (int i=0;i<5;i++){
                date[i].setTime(beginDate);
                date[i].add(Calendar.MONTH,-12*i);
                endDate[i] = sdf.parse(sdf.format(date[i].getTime()));
                map.put("n"+i,endDate[i]);
            }
            map.put("n", 12);
        }
        return cshDeductionMapper.homePageMgrWriteOff(map);
    }

    @Override
    public Map homePagePoolMgrWriteOff(Map map, int page, int pageSize, IRequest iRequest) throws ParseException {
        String flag = (String) map.get("flag");
        Date beginDate = new Date();
        Calendar[] date = new Calendar[5];
        Date[] endDate = new Date[5];
        if ("M".equals(flag)) {
            for (int j=0;j<date.length;j++){
                date[j] = Calendar.getInstance();
            }
            for (int i=0;i<5;i++){
                date[i].setTime(beginDate);
                date[i].add(Calendar.MONTH,-i);
                endDate[i] = sdf.parse(sdf.format(date[i].getTime()));
                map.put("n"+i,endDate[i]);
            }
            map.put("n", 1);
        } else if ("S".equals(flag)) {
            for (int j=0;j<date.length;j++){
                date[j] = Calendar.getInstance();
            }
            for (int i=0;i<5;i++){
                date[i].setTime(beginDate);
                date[i].add(Calendar.MONTH,-3*i);
                endDate[i] = sdf.parse(sdf.format(date[i].getTime()));
                map.put("n"+i,endDate[i]);
            }
            map.put("n", 3);
        } else if ("H".equals(flag)) {
            for (int j=0;j<date.length;j++){
                date[j] = Calendar.getInstance();
            }
            for (int i=0;i<5;i++){
                date[i].setTime(beginDate);
                date[i].add(Calendar.MONTH,-6*i);
                endDate[i] = sdf.parse(sdf.format(date[i].getTime()));
                map.put("n"+i,endDate[i]);
            }
            map.put("n", 6);
        } else {
            for (int j=0;j<date.length;j++){
                date[j] = Calendar.getInstance();
            }
            for (int i=0;i<5;i++){
                date[i].setTime(beginDate);
                date[i].add(Calendar.MONTH,-12*i);
                endDate[i] = sdf.parse(sdf.format(date[i].getTime()));
                map.put("n"+i,endDate[i]);
            }
            map.put("n", 12);
        }
        return cshDeductionMapper.homePagePoolMgrWriteOff(map);
    }

    @Override
    public List<Map> homePageMgr(Map map, int page, int pageSize, IRequest iRequest) throws ParseException {
        String flag = (String) map.get("flag");
        List<Map> data;
        Date beginDate = new Date();
        Calendar date = Calendar.getInstance();
        date.setTime(beginDate);
        if ("M".equals(flag)) {
            date.add(Calendar.MONTH,-6);
            Date endDate = sdf.parse(sdf.format(date.getTime()));
            map.put("n", 1);
            map.put("n6",endDate);
        } else if ("S".equals(flag)) {
            date.add(Calendar.MONTH,-18);
            Date endDate = sdf.parse(sdf.format(date.getTime()));
            map.put("n", 3);
            map.put("n6",endDate);
        } else if ("H".equals(flag)) {
            date.add(Calendar.MONTH,-36);
            Date endDate = sdf.parse(sdf.format(date.getTime()));
            map.put("n", 6);
            map.put("n6",endDate);
        } else {
            date.add(Calendar.MONTH,-72);
            Date endDate = sdf.parse(sdf.format(date.getTime()));
            map.put("n", 12);
            map.put("n6",endDate);

        }
        data = cshDeductionMapper.selectDepositByRate(map);
//        if ("M".equals(flag)) {
//            Calendar cal = Calendar.getInstance();
//            cal.add(Calendar.MONTH,-6);
//            map.put("tempDate",cal.getTime());
//            data = cshDeductionMapper.depositByMonth(map);
////            data = formatResults(data,cal,1);
//        } else if ("S".equals(flag)) {
//            Calendar cal = Calendar.getInstance();
//            cal.add(Calendar.MONTH, -18);
//            cal.set(Calendar.DATE, 1);
//            map.put("lastDate",cal.getTime());
//            data = cshDeductionMapper.depositBySeason(map);
////            data = formatResults(data,cal,3);
//        } else if ("H".equals(flag)) {
//            Calendar cal = Calendar.getInstance();
//            cal.add(Calendar.YEAR, -3);
//            map.put("lastYear",cal.getTime());
//            data = cshDeductionMapper.depositByHalfYear(map);
////            data = formatResults(data,cal,6);
//        } else {
//            Calendar cal = Calendar.getInstance();
//            cal.add(Calendar.YEAR, -6);
//            map.put("last6Years",cal.getTime());
//            data = cshDeductionMapper.depositByYear(map);
////            data = formatResults(data,cal,12);
//        }
        return data;
    }

    @Override
    public List<Map> homePagePoolMgr(Map map, int page, int pageSize, IRequest iRequest) throws ParseException {
        String flag = (String) map.get("flag");
        Date beginDate = new Date();
        Calendar date = Calendar.getInstance();
        date.setTime(beginDate);
        List<Map> data;
        if ("M".equals(flag)) {
            date.add(Calendar.MONTH,-6);
            Date endDate = sdf.parse(sdf.format(date.getTime()));
            map.put("n", 1);
            map.put("n6",endDate);
        } else if ("S".equals(flag)) {
            date.add(Calendar.MONTH,-18);
            Date endDate = sdf.parse(sdf.format(date.getTime()));
            map.put("n", 3);
            map.put("n6",endDate);
        } else if ("H".equals(flag)) {
            date.add(Calendar.MONTH,-36);
            Date endDate = sdf.parse(sdf.format(date.getTime()));
            map.put("n", 6);
            map.put("n6",endDate);
        } else {
            date.add(Calendar.MONTH,-72);
            Date endDate = sdf.parse(sdf.format(date.getTime()));
            map.put("n", 12);
            map.put("n6",endDate);

        }
        data = cshDeductionMapper.selectDepositPollByRate(map);
        return data;
    }

    @Override
    public Map queryDepositPoolByBp(Map map, IRequest iRequest) {
        List<Map> maps = cshDeductionMapper.queryDepositPoolByBp(map);
        HashMap hashMap = new HashMap();
        List<Map> use = new LinkedList();
        List<Map> source = new LinkedList();
        for (Map one : maps) {
            if (one.get("transactionId") != null) {
                hashMap.put("csh_transaction_id", one.get("transactionId"));
                hashMap.put("transaction_id", one.get("transactionId"));
                hashMap.put("companyId", map.get("companyId"));
                use = cshDeductionMapper.DepositPoolWriteOffByBp(hashMap);
                source = cshDeductionMapper.DepositPoolSourceByBp(hashMap);
            }
        }
        hashMap.clear();
        hashMap.put("top", maps);
        hashMap.put("mid", use);
        hashMap.put("down", source);
        return hashMap;
    }

    @Override
    public Map queryDepositPoolByCon(Map map, IRequest iRequest) {
        List<Map> maps = cshDeductionMapper.queryDepositByCon(map);
        List<Map> use = cshDeductionMapper.queryDepositUseByCon(map);
        List<Map> source = cshDeductionMapper.queryDepositSourceByCon(map);
        HashMap hashMap = new HashMap();
        hashMap.put("top", maps);
        hashMap.put("mid", use);
        hashMap.put("down", source);
        return hashMap;
    }

    public List<Map> formatResults(List<Map> data, Calendar cal, int increment) {
        List<Map> results = new ArrayList<Map>();

        for (int i = 0; i < 6; i++) {
            Map tMap = new HashMap();
            String tYear = String.valueOf(cal.get(Calendar.YEAR));
            String tMonth = String.valueOf(cal.get(Calendar.MONTH));
            if (Integer.parseInt(tMonth) < 10) {
                tMonth = "0" + tMonth;
            }
            Boolean existFlag = false;
            for (int j = 0; j < data.size(); j++) {
                String[] arr = data.get(j).get("TIME").toString().split("-");
                if (tYear.equals(arr[0]) && tMonth.equals(arr[1])) {
                    tMap.put("TIME", data.get(j).get("TIME"));
                    tMap.put("sumMoney", data.get(j).get("sumMoney"));
                    existFlag = true;
                }
            }
            if (existFlag == false) {
                tMap.put("TIME", tYear + '-' + tMonth);
                tMap.put("sumMoney", 0);
            }
            cal.add(Calendar.MONTH, increment);
            results.add(tMap);
        }
        return results;
    }

}
