package hls.core.csh.service.impl;

import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.csh.mapper.HlsCusCshDeductionMapper;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.service.CshTransactionCommon;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWritiOffEventService;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.gld.service.IContractTrxService;
import hls.core.gld.service.IJeTrxDtlService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.dto.HlsCashflowItem;
import hls.core.hls.mapper.HlsCfItemMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hand.hap.core.IRequest;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by FJM on 2017/1/19.
 */

@Service
public class CshDepositPoolDeductionServiceImpl implements CshTransactionCommon {

    @Autowired
    private CshWritiOffEventService cshWritiOffEventService;
    @Autowired
    private HlsCusCshDeductionMapper cshDeductionMapper;
    @Autowired
    private HlsCusConContractCashflowMapper con;
    @Autowired
    private HlsCusCshTransactionMapper ctm;
    @Autowired
    private JeTrxCommonService jeTrxCommonService;
    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private HlsCusBpMasterMapper hlsCusBpMasterMapper;
    @Autowired
    private HlsCfItemMapper hlsCfItemMapper;
    @Autowired
    private IContractTrxService contractTrxService;
    @Autowired
    private CshTransactionService cshTransactionService;
    @Autowired
    private IJeTrxDtlService jeTrxDtlService;

    private static final String WriteOffType = "DEPOSIT_POOL_CREDIT";

    @Override
    public String getWriteOffType() {
        return WriteOffType;
    }

    @Override
    public void process(IRequest iRequest, HlsCusCshWriteOff cshwriteoff) throws BeyondAmountLimitException {

        HlsCusConContractCashflow conContractCashflow = new HlsCusConContractCashflow();

        if (cshwriteoff.getrFlag() != null && cshwriteoff.getrFlag().equals("N")) {
            cshwriteoff.setReversedFlag("R");
            cshwriteoff.setReversedDate(new Date());
            cshwriteoff.setReversedWriteOffId(cshwriteoff.getwId());
            //反冲的时候是插入一条金额为负数的记录
            cshwriteoff.setWriteOffDueAmount(-cshwriteoff.getWriteOffDueAmount());
            cshwriteoff.setWriteOffPrincipal(-cshwriteoff.getWriteOffPrincipal());
            cshwriteoff.setWriteOffInterest(-cshwriteoff.getWriteOffInterest());
        } else {
            cshwriteoff.setReversedFlag("N");
        }
        cshwriteoff.setCshWriteOffAmount(cshwriteoff.getWriteOffDueAmount());
        if (cshwriteoff.getTransactionId() != null) {    //反冲时把从前台传进来的事物ID set到对应的字段上
            cshwriteoff.setCshTransactionId(cshwriteoff.getTransactionId());
        }
        conContractCashflow.setCashflow_id(cshwriteoff.getContractId());
        if (con.select(conContractCashflow) != null && con.select(conContractCashflow).size() > 0) {
            Long cId = con.select(conContractCashflow).get(0).getCashflow_id();//现金流ID
            Long cItem = con.select(conContractCashflow).get(0).getCf_item();//现金流性质
            Long cType = con.select(conContractCashflow).get(0).getCf_type();//现金流类型
        }
        cshDeductionMapper.addCshWriteOff(cshwriteoff);
        //如果这个flag为N则表示该记录是反冲的，然后还需要更新另外一条writeoff记录
        if (cshwriteoff.getrFlag() != null && cshwriteoff.getrFlag().equals("N")) {
            cshDeductionMapper.updateWriteOff(cshwriteoff);
        }
        DeductionUtils.UpdateCshTransaction(iRequest, cshwriteoff, ctm, cshDeductionMapper, cshTransactionService, jeTrxCommonService.map);
        DeductionUtils.UpdateCshFlow(iRequest, cshwriteoff, con, cshDeductionMapper, cshTransactionService);

        Map<String, AbstractJeTrxService> interfaceMaps = jeTrxCommonService.map;
        AbstractJeTrxService writeOffJeTrxService = interfaceMaps.get("CSH_WRITE_OFF");
        Map writeOffParams = new HashMap<>();
        writeOffParams.put("jeTrxId", cshwriteoff.getWriteOffId());
        writeOffParams.put("companyId", cshwriteoff.getCompanyId());
        writeOffParams.put("contractId", cshwriteoff.getContractId());
        writeOffParams.put("sourceDoc", "CON_CONTRACT");
        writeOffJeTrxService.process(iRequest, writeOffParams);

        // 更新事件
        Map<String, Object> params = new HashMap<>();
        Long cashflowId = cshwriteoff.getCashflowId();
        HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
        cashflow.setCashflow_id(cashflowId);
        cashflow = con.selectByPrimaryKey(cashflow);

        Double amount = cashflow.getReceived_amount();
        Long contract_id = cashflow.getContract_id();
        HlsCusConContract conContract = new HlsCusConContract();
        conContract.setContractId(contract_id);

        conContract = conContractMapper.selectByPrimaryKey(conContract);
        String currency = conContract.getCurrency();
        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(currency);
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);

        HlsCashflowItem hlsCashflowItem = new HlsCashflowItem();
        hlsCashflowItem.setCf_type(cashflow.getCf_type().toString());
        hlsCashflowItem.setCf_item(cashflow.getCf_item().toString());
        hlsCashflowItem = hlsCfItemMapper.selectOne(hlsCashflowItem);

        Long transactionId = cshwriteoff.getTransactionId();
        HlsCusCshTransaction transaction = new HlsCusCshTransaction();
        transaction.setTransactionId(transactionId);
        HlsCusCshTransaction cshTransaction = cshTransactionService.selectByPrimaryKey(iRequest, transaction);
        //FIXME bpName参数不确定
        HlsCusBpMaster hlsCusBpMaster = new HlsCusBpMaster();
        hlsCusBpMaster.setBpId(cshTransaction.getBpId());
        hlsCusBpMaster = hlsCusBpMasterMapper.selectByPrimaryKey(hlsCusBpMaster);


        params.put("amount", amount);
        params.put("currency", hlsCurrency.getCurrencySymbol());
        params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_DEPOSIT_POOL_CREDIT);
        params.put("cfName", hlsCashflowItem.getDescription());
        params.put("bpName", hlsCusBpMaster.getBpName());
        sysEventService.createEvent(iRequest, contract_id, "CON_CONTRACT", "", params);
//        cshWritiOffEventService.cshWritiOffEvent(iRequest, "CSH_WRITE_OFF.RECEIPT_DEPOSIT_POOL", cshwriteoff.getWriteOffId());
    }

    @Override
    public void reversed(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {

    }

}
