package hls.core.csh.service.impl;

import java.util.Date;
import java.util.List;

import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.ICshTransactionEventService;
import hls.core.hls.mapper.HlsUserRelationshipMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.service.SysNoticePushService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.service.CshTransactionEventService;
import hls.core.hls.dto.HlsDocAuthority;
import hls.core.hls.dto.HlsSystemEvent;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsUserRelationship;
import hls.core.hls.service.HlsSystemEventService;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.hls.service.IHlsDocAuthorityService;
import hls.core.hls.service.IHlsUserRelationshipService;

@Service
@Transactional
public class CshTransactionEventServiceImpl extends BaseServiceImpl<HlsCusCshTransaction>
        implements CshTransactionEventService {

    @Autowired
    private HlsCusCshTransactionMapper cshTransactionMapper;
    @Autowired
    private HlsSystemEventService hlsSystemEventService;
    @Autowired
    private IHlsDocAuthorityService docAuthorityService;
    @Autowired
    private IHlsUserRelationshipService hlsUserRelationshipService;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;
    @Autowired
    private SysNoticePushService sysNoticePushService;
    @Autowired
    private HlsUserRelationshipMapper userRelationshipMapper;
    @Autowired
    private CshTransactionService cshTransactionService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired(required = false)
    private ICshTransactionEventService iCshTransactionEventService;

    private static String CSH_TRANSACTION_RECEIPT = "现金收款";//CSH_TRANSACTION.RECEIPT
    private static String CSH_TRANSACTION_PAYMENT = "现金付款";//CSH_TRANSACTION.PAYMENT

    private static String documentCategory = "CSH_TRANSACTION";
    private static String URLRECEIPT = "csh/CSH300/gathering_add2.view";
    private static String URLPAYMENT = "csh/CSH200/payment_creates.view";
    private static String sourceModule = "CSH";
    private static String RelationType = "P2D";
    // private static String AuthorityTypeQuery = "1";
    private static String AuthorityTypeUpdate = "2";
    private static String documentType = "";

    @Override
    public void CshTransactionEvent(IRequest iRequest, String eventCode, Long transactionId) {
        HlsCusCshTransaction cshTransaction = new HlsCusCshTransaction();
        if (iCshTransactionEventService != null) {
            iCshTransactionEventService.beforeUpdate(iRequest, eventCode, transactionId);
        }
        cshTransaction.setTransactionId(transactionId);
        Long userId = iRequest.getUserId();
        List<HlsCusCshTransaction> cshTransactionList = cshTransactionService.queryDetailById(transactionId);
        cshTransaction = cshTransactionMapper.selectCshTransactionById(cshTransaction).get(0);
        documentType = cshTransaction.getTransactionType();
        String message = "";
        String url = "";
        String notceTitle = "";
        switch (eventCode) {
            case "CSH_TRANSACTION.RECEIPT":
                message = "收到" + cshTransactionList.get(0).getBpName() + "￥" + cshTransaction.getTransactionAmount() + "元";
                url = URLRECEIPT;
                notceTitle = CSH_TRANSACTION_RECEIPT;
                break;
            case "CSH_TRANSACTION.PAYMENT":
                message = "支付" + cshTransactionList.get(0).getBpName() + "￥" + cshTransaction.getTransactionAmount() + "元";
                url = URLPAYMENT;
                notceTitle = CSH_TRANSACTION_PAYMENT;
                break;
            default:
                break;
        }
        HlsSystemEvent event = new HlsSystemEvent();
        event.setEventCode(eventCode);
        event.setSourceDocumentId(transactionId);
        event.setEventUserId(userId);
        event.setSourceDocumentCategory(documentCategory);
        event.setLastUpdatedBy(userId);
        event.setLastUpdateDate(new Date());
        // 系统通知
        HlsSystemNotice hlsSystemNotice = new HlsSystemNotice();
        hlsSystemNotice.setNotice_message(message);
        hlsSystemNotice.setNotice_title(notceTitle);
        hlsSystemNotice.setNotice_datetime(new Date());
        hlsSystemNotice.setNotice_type("NOTICE");
        hlsSystemNotice.setNotice_url(url);
        hlsSystemNotice.setNotice_level(2L);
        hlsSystemNotice.setSource_module(sourceModule);
        hlsSystemNotice.setSource_event(eventCode);
        hlsSystemNotice.setSource_document_category("HLS_BP_MASTER");
        hlsSystemNotice.setSource_document_id(transactionId);
        hlsSystemNotice.setSource_user_id(userId);
//		hlsSystemNotice.setRead_flag("Y");
        hlsSystemNotice.setLastUpdatedBy(userId);
        hlsSystemNotice.setLastUpdateDate(new Date());

        HlsDocAuthority authority = new HlsDocAuthority();
        authority.setUserId(userId);
        authority.setDocumentCategory(documentType);// 单据类型
        authority.setDocumentId(transactionId);
        authority.setAuthorityType(AuthorityTypeUpdate);// 1查询，2，修改
        authority.setSourceModule(sourceModule);
        authority.setSourceEvent(eventCode);
        authority.setSourceDocumentCategory(documentCategory);
        authority.setSourceDocumentId(transactionId);
        authority.setSourceUserId(userId);
        authority.setLastUpdatedBy(userId);
        authority.setLastUpdateDate(new Date());

        HlsUserRelationship hlsUserRelationship = new HlsUserRelationship();
        hlsUserRelationship.setUserId(userId);
        hlsUserRelationship.setUserId(userId);
        hlsUserRelationship.setRelationType(RelationType);
        hlsUserRelationship.setSourceModule(sourceModule);
        hlsUserRelationship.setSourceEvent(eventCode);
        hlsUserRelationship.setSourceDocumentCategory(documentCategory);
        hlsUserRelationship.setSourceDocumentId(transactionId);
        hlsUserRelationship.setSourceUserId(iRequest.getUserId());
        hlsUserRelationship.setLastUpdatedBy(userId);
        hlsUserRelationship.setLastUpdateDate(new Date());

        hlsSystemEventService.insert(iRequest, event);
        docAuthorityService.insert(iRequest, authority);

        if ("P2P".equalsIgnoreCase(hlsUserRelationship.getRelationType())) {
            hlsUserRelationship.setRelationUserId(iRequest.getUserId()); // 关联用户ID

            HlsUserRelationship userRelationship = new HlsUserRelationship();
            userRelationship.setUserId(hlsUserRelationship.getUserId());
            userRelationship.setRelationType(hlsUserRelationship.getRelationType());
            userRelationship.setRelationUserId(hlsUserRelationship.getRelationUserId());
            List<HlsUserRelationship> list = userRelationshipMapper.select(userRelationship);
            if (list.size() == 0) {
                hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship);
            }
        } else if ("P2D".equalsIgnoreCase(hlsUserRelationship.getRelationType())) {
            hlsUserRelationship.setRelationDocumentCategory(documentCategory);// 关联单据类别
            hlsUserRelationship.setRelationDocumentId(transactionId); // 关系单据ID

            HlsUserRelationship userRelationship1 = new HlsUserRelationship();
            userRelationship1.setUserId(hlsUserRelationship.getUserId());
            userRelationship1.setRelationType(hlsUserRelationship.getRelationType());
            userRelationship1.setSourceDocumentCategory(hlsUserRelationship.getSourceDocumentCategory());
            userRelationship1.setSourceDocumentId(hlsUserRelationship.getSourceDocumentId());
            List<HlsUserRelationship> lists = userRelationshipMapper.select(userRelationship1);
            if (lists.size() == 0) {
                hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship);
            }
        }
        hlsSystemNoticeService.insert(iRequest, hlsSystemNotice);
        sysEventService.setNoticeCache(hlsSystemNotice);
        if (iCshTransactionEventService != null) {
            iCshTransactionEventService.afterUpdate(iRequest, eventCode, transactionId);
        }
    }
}
