package hls.core.csh.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.csh.dto.HlsCusCshBankAccountBalance;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.exception.WriteOffTypeNullException;
import hls.core.csh.mapper.HlsCusCshBankAccountBalanceMapper;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.mapper.HlsCusCshWriteOffMapper;
import hls.core.csh.service.*;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.bp.dto.HlsCusBpMasterBankAccount;
import hls.core.hls.bp.service.HlsBpMasterBankAccountService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpSession;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;


@Service
@Transactional(rollbackFor = {Exception.class, RuntimeException.class})
public class CshTransactionServiceImpl extends BaseServiceImpl<HlsCusCshTransaction> implements CshTransactionService {

    public static final String PAYMENT = "PAYMENT";
    @Autowired
    HlsCusCshTransactionMapper mapper;
    @Autowired
    HlsBpMasterBankAccountService bankAccountService;
    @Autowired
    HlsCusCshWriteOffMapper cshWriteOffMapper;
    @Autowired
    HlsCusCshBankAccountBalanceMapper bankAccountBalanceMapper;
    @Autowired
    CshBankAccountBalanceService bankAccountBalanceService;
    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    CshWriteOffService cshWriteOffService;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private JeTrxCommonService jeTrxCommonService;
    @Autowired
    private HlsCusCshTransactionMapper transactionMapper;
    @Autowired(required = false)
    private IUpdateCshTransactionService iUpdateCshTransactionService;
    @Autowired(required = false)
    private IUpdateReversedService iUpdateReversedService;
    @Autowired(required = false)
    private IUpdateReturnService iUpdateReturnService;


    Map<String, String> params = new HashMap<String, String>();

    Calendar calendar = Calendar.getInstance();

    private int currentYear = calendar.get(Calendar.YEAR);

    private int currentMonth = calendar.get(Calendar.MONTH) + 1;

    /**
     * 收款新增
     */
    @Override
    public HlsCusCshTransaction updateCshTransaction(Long companyId, HlsCusCshTransaction cshTransaction, IRequest requestContext) {

        HlsCusBpMasterBankAccount ba = null;
        if (iUpdateCshTransactionService != null) {
            iUpdateCshTransactionService.beforeUpdate(companyId, cshTransaction, requestContext);
        }
        //当对面银行账户不存在时，新增一条商业伙伴银行账户
        if (cshTransaction.getBpBankAccountId() == null) {
            ba = new HlsCusBpMasterBankAccount();

            ba.setBpId(cshTransaction.getBpId());
            ba.setBankAccountName(cshTransaction.getBpBankAccountName());
            ba.setBankAccountNum(cshTransaction.getBpBankAccountNum());
            ba.setBankBranchName(cshTransaction.getBpBankBranchName());
            ba.setBankFullName(cshTransaction.getBpBankName());

            bankAccountService.insertSelective(requestContext, ba);

            cshTransaction.setBpBankAccountId(ba.getBankAccountId());
        }

        getTransactionNum(requestContext, companyId, cshTransaction);

        if (cshTransaction.getTransactionId() == null) {
            self().insertSelective(requestContext, cshTransaction);
        } else {
            self().updateByPrimaryKeySelective(requestContext, cshTransaction);
        }
        //收款新增事件
        HlsCusConContract conContract = new HlsCusConContract();

        HLSCurrency hlsCurrency = new HLSCurrency();

        if ("".equals(cshTransaction.getCurrencyCode())) {
            hlsCurrency.setCurrencyCode("CNY");
        } else {
            hlsCurrency.setCurrencyCode(cshTransaction.getCurrencyCode());
        }

        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);
        Map<String, Object> params = new HashMap<>();
        params.put("amount", cshTransaction.getTransactionAmount());
        params.put("currency", hlsCurrency.getCurrencySymbol());
        params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_TRANSACTION_RECEIPT);
        sysEventService.createEvent(requestContext, cshTransaction.getBpId(), "CSH_TRANSACTION", "", params);

        //cshTransactionEventService.CshTransactionEvent(requestContext, "CSH_TRANSACTION.RECEIPT", cshTransaction.getTransactionId());
        /**
         * 更新账户余额表
         * 1、判断收款时间是否在当月
         * 2、如果在当月，判断当前月是否存在账户余额记录，反之，从收款期依次跟新账户余额
         */
        receiptBankAccountBalance(requestContext, cshTransaction);

        //插入凭证事物流水 by fjm 17.9.1
        AbstractJeTrxService transactionJeTrxService = jeTrxCommonService.map.get("CSH_TRANSACTION");
        Map transactionParams = new HashMap<>();
        transactionParams.put("jeTrxId", cshTransaction.getTransactionId());
        transactionParams.put("companyId", companyId);
        if (cshTransaction.getContractId()!=null){
            transactionParams.put("jeSourceId", cshTransaction.getContractId());
            transactionParams.put("jeSourceDoc", "CON_CONTRACT");
        }else if (cshTransaction.getLonContractId()!=null){
            transactionParams.put("jeSourceId", cshTransaction.getLonContractId());
            transactionParams.put("jeSourceDoc", "LON_CONTRACT");
        }else if (cshTransaction.getWithdrawId()!=null){
            transactionParams.put("jeSourceId", cshTransaction.getWithdrawId());
            transactionParams.put("jeSourceDoc", "FCT_CONTRACT_WITHDRAW");
        }
        transactionJeTrxService.process(requestContext, transactionParams);

        if (iUpdateCshTransactionService != null) {
            iUpdateCshTransactionService.afterUpdate(companyId, cshTransaction, requestContext);
        }
        return cshTransaction;
    }

    private void receiptBankAccountBalance(IRequest requestContext, HlsCusCshTransaction cshTransaction) {
        Calendar c = Calendar.getInstance();
        c.setTime(cshTransaction.getTransactionDate());
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1;
        HlsCusCshBankAccountBalance accountBalance = null;

        if (year == currentYear && month == currentMonth) {
            HlsCusCshBankAccountBalance bankAccountBalance = bankAccountBalanceMapper.selectAccountBalance(cshTransaction.getBankAccountId(), Long.valueOf(currentYear), Long.valueOf(currentMonth));

            if (bankAccountBalance == null) {
                bankAccountBalance = new HlsCusCshBankAccountBalance();

                bankAccountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                bankAccountBalance.setPeriodYear(Long.valueOf(year));
                bankAccountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance = bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).isEmpty() ? null : bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).get(0);

                if (cshTransaction.getTransactionAmount() == null) {
                    cshTransaction.setTransactionAmount(0d);
                }

                if (accountBalance == null) {
                    bankAccountBalance.setOpeningBalance(0d);
                    bankAccountBalance.setBalance(cshTransaction.getTransactionAmount());
                } else {
                    bankAccountBalance.setOpeningBalance(accountBalance.getBalance());
                    bankAccountBalance.setBalance(accountBalance.getBalance() + cshTransaction.getTransactionAmount());
                }
                bankAccountBalance.setPeriodIncome(cshTransaction.getTransactionAmount());
                bankAccountBalance.setPeriodExpense(0d);

                bankAccountBalanceService.insertSelective(requestContext, bankAccountBalance);

            } else {
                accountBalance = new HlsCusCshBankAccountBalance();
                accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                accountBalance.setPeriodYear(Long.valueOf(year));
                accountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance.setPeriodIncome(cshTransaction.getTransactionAmount());
                accountBalance.setPeriodExpense(0d);
                bankAccountBalanceMapper.updateBankAccountBalanceByKey(accountBalance);
            }

        } else {
            HlsCusCshBankAccountBalance bankAccountBalance = bankAccountBalanceMapper.selectAccountBalance(cshTransaction.getBankAccountId(), Long.valueOf(year), Long.valueOf(month));
            if (bankAccountBalance == null) {
                bankAccountBalance = new HlsCusCshBankAccountBalance();

                bankAccountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                bankAccountBalance.setPeriodYear(Long.valueOf(year));
                bankAccountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance = bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).isEmpty() ? null : bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).get(0);

                if (cshTransaction.getTransactionAmount() == null) {
                    cshTransaction.setTransactionAmount(0d);
                }
                if (accountBalance == null) {
                    bankAccountBalance.setOpeningBalance(0d);
                    bankAccountBalance.setBalance(cshTransaction.getTransactionAmount());
                } else {
                    bankAccountBalance.setOpeningBalance(accountBalance.getBalance());
                    bankAccountBalance.setBalance(accountBalance.getBalance() + cshTransaction.getTransactionAmount());
                }
                bankAccountBalance.setPeriodIncome(cshTransaction.getTransactionAmount());
                bankAccountBalance.setPeriodExpense(0d);

                bankAccountBalanceService.insertSelective(requestContext, bankAccountBalance);

            } else {
                accountBalance = new HlsCusCshBankAccountBalance();
                accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                accountBalance.setPeriodYear(Long.valueOf(year));
                accountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance.setPeriodIncome(cshTransaction.getTransactionAmount());
                accountBalance.setPeriodExpense(0d);
                bankAccountBalanceMapper.updateBankAccountBalanceByKey(accountBalance);

            }

            accountBalance = new HlsCusCshBankAccountBalance();
            accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
            accountBalance.setPeriodYear(Long.valueOf(year));
            accountBalance.setPeriodMonth(Long.valueOf(month));
            accountBalance.setPeriodIncome(cshTransaction.getTransactionAmount());
            accountBalance.setPeriodExpense(0d);
            bankAccountBalanceMapper.updateBankAccountBalance(accountBalance);
        }

    }

    private void getTransactionNum(IRequest requestContext, Long companyId, HlsCusCshTransaction cshTransaction) {
        //现金事务编号由编码自动生成,此处为模拟生成
        cshTransaction.setTransactionCategory("CSH_TRANSACTION");
        cshTransaction.setTransactionType("RECEIPT");
        cshTransaction.setBusinessType("RECEIPT");
        cshTransaction.setTransactionNum(codingRuleValuesService.getCodeRuleValue(requestContext, cshTransaction.getTransactionCategory(),
                cshTransaction.getTransactionType(), cshTransaction.getBusinessType(), params));
        cshTransaction.setPenaltyCalcDate(cshTransaction.getTransactionDate());
        cshTransaction.setCompanyId(companyId);
        cshTransaction.setReversedFlag("N");
        cshTransaction.setPostedFlag("N");
        cshTransaction.setWriteOffFlag("NOT");

    }

    @Override
    public List<HlsCusCshTransaction> queryAll(IRequest requestContext, HlsCusCshTransaction cshTransaction, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);

        cshTransaction.setWriteOffFlag("NOT");

        return mapper.queryAll(cshTransaction);
    }

    @Override
    public List<HlsCusCshTransaction> queryDetailById(Long transactionId) {
        return mapper.queryDetailById(transactionId);
    }

    /**
     * 收款反冲
     * 1、插入一条反冲记录
     * 2、更新现金事物表
     */
    @Override
    public void updateReversed(IRequest requestContext, Long companyId, HlsCusCshTransaction cshTransaction) {
        //插入一条反冲记录
        HlsCusCshTransaction cshTr = new HlsCusCshTransaction(cshTransaction);
        if (iUpdateReversedService != null) {
            iUpdateReversedService.beforeUpdate(requestContext, companyId, cshTransaction);
        }
        if (cshTr != null) {
            Double num = 0 - cshTr.getTransactionAmount();

            cshTr.setTransactionId(null);
            cshTr.setTransactionAmount(num);
            cshTr.setTransactionDate(cshTransaction.getReversedDate());
            cshTr.setPenaltyCalcDate(cshTransaction.getReversedDate());
            cshTr.setReversedFlag("R");
            cshTr.setDescription(cshTransaction.getDescription());
            cshTr.setCompanyId(companyId);
            cshTr.setReversedTrxId(cshTransaction.getTransactionId());
            cshTr.setReversedDate(cshTransaction.getReversedDate());
            cshTr.setPostedFlag("N");
            cshTr.setContractId(null);
            cshTr.setHandlingCharge(null);
            cshTr.setWriteOffFlag("NA");
            cshTr.setWriteOffAmount(null);
            cshTr.setFullWriteOffDate(null);
            cshTr.setSourceDocCategory(null);
            cshTr.setSourceDocId(null);
            cshTr.setSourceDocLineId(null);
            cshTr.setTransactionNum(codingRuleValuesService.getCodeRuleValue(requestContext, cshTr.getTransactionCategory(), cshTr.getTransactionType(), cshTr.getBusinessType(), params));
        } else {
            return;
        }
        cshTr = self().insertSelective(requestContext, cshTr);

        //更新现金事物表
        cshTransaction.setReversedFlag("W");
        cshTransaction.setReversedTrxId(cshTr.getTransactionId());
        cshTransaction.setReversedDate(cshTr.getReversedDate());
        mapper.updateCshTrByPrimaryKey(cshTransaction);


        //更新账户余额表
        receiptBankAccountBalance(requestContext, cshTr);


        //更新合同现金流

        //插入凭证事物流水表 by fjm 17.9.1
        AbstractJeTrxService transactionJeTrxService = jeTrxCommonService.map.get("CSH_TRANSACTION");
        Map transactionParams = new HashMap<>();
        transactionParams.put("jeTrxId", cshTr.getTransactionId());
        transactionParams.put("companyId", companyId);
        transactionParams.put("contractId", cshTr.getContractId());
        transactionParams.put("sourceDoc", "CON_CONTRACT");
        transactionParams.put("reverseJeDate", cshTransaction.getReversedDate());
        transactionParams.put("reverseJeTrxId", cshTransaction.getTransactionId());
        transactionJeTrxService.process(requestContext, transactionParams);
        if (iUpdateReversedService != null) {
            iUpdateReversedService.afterUpdate(requestContext, companyId, cshTransaction);
        }

    }

    @Override
    public List<HlsCusCshTransaction> cshReturnQuery(IRequest requestContext, HlsCusCshTransaction cshTransaction, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);

        return mapper.cshReturnQuery(cshTransaction);
    }


    /**
     * 收款退款
     * 1、在核销表内新增一条数据
     * 2、在现金事物表中增加一条退款记录
     * 3、更新现金是服务中的核销金额和核销标志（判断是否核销完全，完全核销时更新完全核销日期）
     * 4、更新核销表中的后续现金事物Id
     *
     * @throws BeyondAmountLimitException
     */
    @Override
    public void updateReturn(IRequest requestContext, Long companyId, HlsCusCshTransaction cshTransaction) throws BeyondAmountLimitException {
        if (iUpdateReturnService != null) {
            iUpdateReturnService.beforeUpdate(requestContext, companyId, cshTransaction);
        }
        if (cshTransaction.getWriteOffAmount() == null) {
            cshTransaction.setWriteOffAmount(0d);
        }
        if (cshTransaction.getReturnDueAmount() == null) {
            cshTransaction.setReturnDueAmount(0d);
        }
        Double sumAmount = cshTransaction.getWriteOffAmount() + cshTransaction.getReturnDueAmount();

        if (new BigDecimal(sumAmount).compareTo(new BigDecimal(0)) == 0) {
            cshTransaction.setWriteOffFlag("NOT");
            cshTransaction.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) < 0) {
            cshTransaction.setWriteOffFlag("PARTIAL");
            cshTransaction.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) == 0) {
            cshTransaction.setWriteOffFlag("FULL");
            cshTransaction.setWriteOffAmount(sumAmount);
            cshTransaction.setFullWriteOffDate(cshTransaction.getTransactionDate());
        } else {
            throw new BeyondAmountLimitException();
        }
        //1、在核销表内新增一条数据
        HlsCusCshWriteOff cshWriteOff = new HlsCusCshWriteOff();
        cshWriteOff.setWriteOffType("REFUND");
        cshWriteOff.setWriteOffDate(cshTransaction.getTransactionDate());
        cshWriteOff.setCshTransactionId(cshTransaction.getTransactionId());
        cshWriteOff.setCshWriteOffAmount(cshTransaction.getReturnDueAmount());
        cshWriteOff.setReversedFlag("N");
        cshWriteOff.setWriteOffDueAmount(cshTransaction.getReturnDueAmount());
        cshWriteOffService.insertSelective(requestContext, cshWriteOff);
        //现金退款事件
//        HlsCusConContract conContract = new HlsCusConContract();
//        conContract.setContractId(cshTransaction.getContractId());

//        conContract = conContractMapper.selectByPrimaryKey(conContract);

//        String currency = conContract.getCurrency();
        HLSCurrency hlsCurrency = new HLSCurrency();
        HlsCusCshTransaction transaction = transactionMapper.selectByPrimaryKey(cshTransaction.getTransactionId());
        hlsCurrency.setCurrencyCode(transaction.getCurrencyCode());
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);

        Map<String, Object> param = new HashMap<>();
        param.put("amount", cshTransaction.getReturnDueAmount());
        param.put("currency", hlsCurrency.getCurrencySymbol());
        param.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_REFUND);
        sysEventService.createEvent(requestContext, cshTransaction.getBpId(), "CSH_TRANSACTION", "", param);

        //cshWritiOffEventService.cshWritiOffEvent(requestContext, "CSH_WRITE_OFF.REFUND", cshWriteOff.getWriteOffId());

        //2、在现金事物表中增加一条退款记录
        HlsCusCshTransaction returnCsh = new HlsCusCshTransaction(cshTransaction);
        returnCsh.setTransactionId(null);
        returnCsh.setTransactionCategory("CSH_TRANSACTION");
        returnCsh.setTransactionType("REFUND");
        if (!"DEPOSIT".equals(cshTransaction.getBusinessType()) && !"DEPOSIT_POOL".equals(cshTransaction.getBusinessType())) {
            //如果是保证金退款，则类型为DEPOSIT
            returnCsh.setBusinessType("PAYMENT");
        } else {
            returnCsh.setBusinessType(cshTransaction.getBusinessType());
        }
        returnCsh.setTransactionNum(codingRuleValuesService.getCodeRuleValue(requestContext, returnCsh.getTransactionCategory(), returnCsh.getTransactionType(), PAYMENT, params));
        returnCsh.setTransactionDate(cshTransaction.getTransactionDate());
        returnCsh.setPenaltyCalcDate(cshTransaction.getTransactionDate());
        returnCsh.setCompanyId(companyId);
        returnCsh.setCurrencyCode(cshTransaction.getCurrencyCode());
        returnCsh.setTransactionAmount(cshTransaction.getReturnDueAmount());
        returnCsh.setReversedFlag("N");
        returnCsh.setReversedTrxId(null);
        returnCsh.setReversedDate(null);
        returnCsh.setPostedFlag("N");
        returnCsh.setContractId(null);
        returnCsh.setHandlingCharge(null);
        returnCsh.setWriteOffFlag("NA");
        returnCsh.setWriteOffAmount(null);
        returnCsh.setFullWriteOffDate(null);
        returnCsh.setSourceDocCategory("CSH_WRITE_OFF");
        returnCsh.setSourceDocId(cshTransaction.getTransactionId());
        returnCsh.setSourceDocLineId(cshWriteOff.getWriteOffId());
        self().insertSelective(requestContext, returnCsh);

        //3、更新现金是服务中的核销金额和核销标志（判断是否核销完全，完全核销时更新完全核销日期）
        mapper.updateCshTrByPrimaryKey(cshTransaction);

        //4、更新核销表中的后续现金事物Id
        cshWriteOffMapper.updateSubCshTrxId(cshWriteOff.getWriteOffId(), returnCsh.getTransactionId(), returnCsh.getTransactionAmount());

        //更新账户余额表
        paymentBankAccountBalance(requestContext, returnCsh);

        //获取对应的凭证接口实例 by fjm 17.9.1
        Map<String, AbstractJeTrxService> interfaceMaps = jeTrxCommonService.map;
        AbstractJeTrxService writeOffJeTrxService = interfaceMaps.get("CSH_WRITE_OFF");
        AbstractJeTrxService transactionJeTrxService = interfaceMaps.get("CSH_TRANSACTION");

        /*更新凭证事物流水*/
        Map writeOffParams = new HashMap<>();
        writeOffParams.put("jeTrxId", cshWriteOff.getWriteOffId());
        writeOffParams.put("companyId", companyId);
        writeOffParams.put("contractId", cshWriteOff.getContractId());
        writeOffParams.put("sourceDoc", "CON_CONTRACT");
        writeOffJeTrxService.process(requestContext, writeOffParams);

        Map transactionParams = new HashMap<>();
        transactionParams.put("jeTrxId", returnCsh.getTransactionId());
        transactionParams.put("companyId", companyId);
        transactionParams.put("contractId", cshTransaction.getContractId());
        transactionParams.put("sourceDoc", "CON_CONTRACT");
        transactionJeTrxService.process(requestContext, transactionParams);
        if (iUpdateReturnService != null) {
            iUpdateReturnService.afterUpdate(requestContext, companyId, cshTransaction);
        }
    }

    private void paymentBankAccountBalance(IRequest requestContext, HlsCusCshTransaction cshTransaction) {
        Calendar c = Calendar.getInstance();
        c.setTime(cshTransaction.getTransactionDate());
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1;
        HlsCusCshBankAccountBalance accountBalance = null;

        if (year == currentYear && month == currentMonth) {
            HlsCusCshBankAccountBalance bankAccountBalance = bankAccountBalanceMapper.selectAccountBalance(cshTransaction.getBankAccountId(), Long.valueOf(currentYear), Long.valueOf(currentMonth));

            if (bankAccountBalance == null) {
                bankAccountBalance = new HlsCusCshBankAccountBalance();

                bankAccountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                bankAccountBalance.setPeriodYear(Long.valueOf(year));
                bankAccountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance = bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).isEmpty() ? null : bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).get(0);
                if (cshTransaction.getTransactionAmount() == null) {
                    cshTransaction.setTransactionAmount(0d);
                }

                if (accountBalance == null) {
                    bankAccountBalance.setOpeningBalance(0d);
                    bankAccountBalance.setBalance(0 - cshTransaction.getTransactionAmount());
                } else {
                    bankAccountBalance.setOpeningBalance(accountBalance.getBalance());
                    bankAccountBalance.setBalance(accountBalance.getBalance() - cshTransaction.getTransactionAmount());
                }
                bankAccountBalance.setPeriodIncome(0d);
                bankAccountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());

                bankAccountBalanceMapper.insertSelective(bankAccountBalance);

            } else {
                accountBalance = new HlsCusCshBankAccountBalance();
                accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                accountBalance.setPeriodYear(Long.valueOf(year));
                accountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance.setPeriodIncome(0d);
                accountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
                bankAccountBalanceService.updateByPrimaryKeySelective(requestContext, accountBalance);
            }

        } else {
            HlsCusCshBankAccountBalance bankAccountBalance = bankAccountBalanceMapper.selectAccountBalance(cshTransaction.getBankAccountId(), Long.valueOf(year), Long.valueOf(month));
            if (bankAccountBalance == null) {
                bankAccountBalance = new HlsCusCshBankAccountBalance();

                bankAccountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                bankAccountBalance.setPeriodYear(Long.valueOf(year));
                bankAccountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance = bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).isEmpty() ? null : bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).get(0);

                if (cshTransaction.getTransactionAmount() == null) {
                    cshTransaction.setTransactionAmount(0d);
                }
                if (accountBalance == null) {
                    bankAccountBalance.setOpeningBalance(0d);
                    bankAccountBalance.setBalance(0 - cshTransaction.getTransactionAmount());
                } else {
                    bankAccountBalance.setOpeningBalance(accountBalance.getBalance());
                    bankAccountBalance.setBalance(accountBalance.getBalance() - cshTransaction.getTransactionAmount());
                }
                bankAccountBalance.setPeriodIncome(0d);
                bankAccountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
                bankAccountBalanceMapper.insertSelective(bankAccountBalance);

            } else {
                accountBalance = new HlsCusCshBankAccountBalance();
                accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                accountBalance.setPeriodYear(Long.valueOf(year));
                accountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance.setPeriodIncome(cshTransaction.getTransactionAmount());
                accountBalance.setPeriodExpense(0d);
                bankAccountBalanceService.updateByPrimaryKeySelective(requestContext, accountBalance);

            }

            accountBalance = new HlsCusCshBankAccountBalance();
            accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
            accountBalance.setPeriodYear(Long.valueOf(year));
            accountBalance.setPeriodMonth(Long.valueOf(month));
            accountBalance.setPeriodIncome(0d);
            accountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
            bankAccountBalanceService.updateByPrimaryKeySelective(requestContext, accountBalance);
        }

    }

    /**
     * 现金事物-单行-交易明细
     *
     * @param cshTransaction
     * @return
     */
    @Override
    public List<HlsCusCshTransaction> selectAllCashThingTransactionDetails(HlsCusCshTransaction cshTransaction) {
        return mapper.selectAllCashThingTransactionDetails(cshTransaction);
    }

    @Override
    public List<HlsCusCshTransaction> cshWriteOffQuery(IRequest requestContext, HlsCusCshTransaction cshTransaction, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);

        return mapper.cshWriteOffQuery(cshTransaction);
    }

    @Override
    public List<HlsCusCshTransaction> cshReturnReversedQuery(IRequest requestContext, HlsCusCshTransaction cshTransaction, int page,
                                                             int pagesize) {
        PageHelper.startPage(page, pagesize);

        return mapper.cshReturnReversedQuery(cshTransaction);
    }


    /**
     * 支付反冲查询
     */
    @Override
    public List<HlsCusCshTransaction> paymentreversedRquery(IRequest requestContext, HlsCusCshTransaction cshTransaction, int page,
                                                            int pagesize) {
        // TODO Auto-generated method stub
        return mapper.paymentreversedRquery(cshTransaction);
    }

    @Override
    public List<HlsCusCshTransaction> paymentreversedRqueryLov() {
        return mapper.paymentreversedRqueryLov();
    }

    @Override
    public HlsCusCshTransaction fundHomePage(IRequest iRequest, Map map, int page, int pagesize) {
        final List<String> accountlist = new ArrayList<String>();
        final List<String> currencylist = new ArrayList<String>();
        boolean accountFlag = false;
        boolean currencyFlag = false;
        if (!"N".equals(map.get("accountFirst"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountFirst"));
        }
        if (!"N".equals(map.get("accountSecond"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountSecond"));
        }
        if (!"N".equals(map.get("accountThird"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountThird"));
        }
        if (!"N".equals(map.get("accountFour"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountFour"));
        }
        if (!"N".equals(map.get("currencyFirst"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencyFirst"));
        }
        if (!"N".equals(map.get("currencySecond"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencySecond"));
        }
        if (!"N".equals(map.get("currencyThird"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencyThird"));
        }
        map.put("accountlist", accountlist);
        map.put("currencylist", currencylist);
        map.put("currencyFlag", currencyFlag);
        map.put("accountFlag", accountFlag);
        return mapper.fundHomePage(map);
    }

    @Override
    public List<Map> collectionDetails(IRequest iRequest, Map map, int page, int pagesize) {
        final List<String> accountlist = new ArrayList<String>();
        final List<String> currencylist = new ArrayList<String>();
        boolean accountFlag = false;
        boolean currencyFlag = false;
        if (!"N".equals(map.get("accountFirst"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountFirst"));
        }
        if (!"N".equals(map.get("accountSecond"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountSecond"));
        }
        if (!"N".equals(map.get("accountThird"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountThird"));
        }
        if (!"N".equals(map.get("accountFour"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountFour"));
        }
        if (!"N".equals(map.get("currencyFirst"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencyFirst"));
        }
        if (!"N".equals(map.get("currencySecond"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencySecond"));
        }
        if (!"N".equals(map.get("currencyThird"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencyThird"));
        }

        map.put("accountlist", accountlist);
        map.put("currencylist", currencylist);
        map.put("currencyFlag", currencyFlag);
        map.put("accountFlag", accountFlag);
        List<Map> data = mapper.collectionDetails(map);
        for (Map d : data) {
            d.put("transactionDate", d.get("transactionDate").toString());
        }

        return data;
    }

    /*提前结清-审批页面-lov查询收款信息*/
    @Override
    public List<Map> queryCollectionInfo(IRequest iRequest, Map map) {
        final List<String> currencylist = new ArrayList<String>();
        currencylist.add("NOT");
        currencylist.add("PARTIAL");
        PageHelper.startPage(Integer.parseInt(map.get("page").toString()), Integer.parseInt(map.get("pagesize").toString()));
        map.put("accountFlag", false);//取全部类型账户
        map.put("currencyFlag", true);
        map.put("currencylist", currencylist);//只取部分核销和未核销的
        List<Map> data = mapper.collectionDetails(map);
        List<Map> dataResult = new ArrayList<>();
        //比较时间
        for (int i = 0; i < data.size(); i++) {
            Date transactionDate = (Date) data.get(i).get("transactionDate");//获取交易日期
            if (map.get("transactionDateFrom") != null && map.get("transactionDateTo") != null) {
                if (transactionDate.getTime() >= ((Date) map.get("transactionDateFrom")).getTime() && transactionDate.getTime() <= ((Date) map.get("transactionDateTo")).getTime()) {
                    dataResult.add(data.get(i));
                }
            } else if (map.get("transactionDateFrom") == null && map.get("transactionDateTo") != null) {
                if (transactionDate.getTime() <= ((Date) map.get("transactionDateTo")).getTime()) {
                    dataResult.add(data.get(i));
                }
            } else if (map.get("transactionDateFrom") != null && map.get("transactionDateTo") == null) {
                if (transactionDate.getTime() >= ((Date) map.get("transactionDateFrom")).getTime()) {
                    dataResult.add(data.get(i));
                }
            } else {
                dataResult.add(data.get(i));
            }

        }
        for (Map d : dataResult) {
            d.put("transactionDate", d.get("transactionDate").toString());
        }
        return dataResult;
    }

    @Override
    public List<Map> collectionDetailsView(IRequest iRequest, Map map, int page, int pagesize) {
        final List<String> accountlist = new ArrayList<String>();
        final List<String> currencylist = new ArrayList<String>();
        boolean accountFlag = false;
        boolean currencyFlag = false;
        if (!"N".equals(map.get("accountFirst"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountFirst"));
        }
        if (!"N".equals(map.get("accountSecond"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountSecond"));
        }
        if (!"N".equals(map.get("accountThird"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountThird"));
        }
        if (!"N".equals(map.get("accountFour"))) {
            accountFlag = true;
            accountlist.add((String) map.get("accountFour"));
        }
        if (!"N".equals(map.get("currencyFirst"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencyFirst"));
        }
        if (!"N".equals(map.get("currencySecond"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencySecond"));
        }
        if (!"N".equals(map.get("currencyThird"))) {
            currencyFlag = true;
            currencylist.add((String) map.get("currencyThird"));
        }

        map.put("accountlist", accountlist);
        map.put("currencylist", currencylist);
        map.put("currencyFlag", currencyFlag);
        map.put("accountFlag", accountFlag);

        return mapper.collectionDetailsView(map);
    }

    /**
     * 合同首页-收款信息条件查询
     *
     * @param cshTransaction
     * @return
     * @throws ParseException
     */
    @Override
    public List<HlsCusCshTransaction> selectAllCshTransaction(IRequest requestContext, HlsCusCshTransaction cshTransaction) throws ParseException {
        cshTransaction.setCompanyId(requestContext.getCompanyId());
        Long fate = cshTransaction.getQueryTimeSolt();
        if ("null".equalsIgnoreCase(cshTransaction.getBpName())) {
            cshTransaction.setBpName(null);
        }
        if ("null".equalsIgnoreCase(cshTransaction.getDocumentType())) {
            cshTransaction.setDocumentType(null);
        }
        if (fate != 0) {
            String strDate = calculationDate(fate);
            cshTransaction.setTransactionDate(sdf.parse(strDate));
        } else {
            cshTransaction.setTransactionDate(null);
        }
        return mapper.selectAllCshTransaction(cshTransaction);
    }

    @Override
    public List<Map> CashThingTransactionQueryDetail(Map map) {
        return mapper.queryByTransactionId(map);
    }

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

    private String calculationDate(Long fate) {

        Date beginDate = new Date();
        Calendar date = Calendar.getInstance();
        date.setTime(beginDate);
        if (fate == 7) {
            date.set(Calendar.DATE, date.get(Calendar.DATE) - 7);
        }
        if (fate == 30) {
            date.add(Calendar.MONTH, -1);
        }
        if (fate == 90) {
            date.add(Calendar.MONTH, -3);
        }
        String endDate = sdf.format(date.getTime());
        return endDate;
    }

    @Override
    public List<Map> csh330aCshTransactionQuery(HlsCusCshTransaction cshtransaction, IRequest iRequest, int page, int pagesize) {

        PageHelper.startPage(page, pagesize);
        List<String> list = new ArrayList<String>();
        List<String> writeOffList = new ArrayList<String>();
        if (cshtransaction.getBpType() != null) {
            String[] bptypeArray = cshtransaction.getBpType().split(",");
            for (int i = 0; i < bptypeArray.length; i++) {
                list.add(bptypeArray[i]);
            }
            cshtransaction.setQueryList(list);
        }
        if (cshtransaction.getWriteOffFlag() != null) {
            String[] writeOffFlagArray = cshtransaction.getWriteOffFlag().split(",");
            for (int i = 0; i < writeOffFlagArray.length; i++) {
                writeOffList.add(writeOffFlagArray[i]);
            }
            cshtransaction.setQueryList1(writeOffList);
        }

        List<Map> maps = mapper.csh330aCshTransactionQuery(cshtransaction);
        return maps;
    }

    @Override
    public List<Map> csh420aCshDepositPoolQuery(HlsCusCshTransaction cshtransaction, IRequest iRequest) {
        List<String> list = new ArrayList<String>();
        if (cshtransaction.getBpType() != null) {
            String[] bptypeArray = cshtransaction.getBpType().split(",");
            for (int i = 0; i < bptypeArray.length; i++) {
                list.add(bptypeArray[i]);
            }
            cshtransaction.setQueryList(list);
        }
        List<Map> maps = mapper.csh420aCshDepositPoolQuery(cshtransaction);
        return maps;
    }

    @Override
    public List<Map> csh410aConContractDepoistQuery(HlsCusCshTransaction cshtransaction, IRequest iRequest) {
        List<String> list = new ArrayList<String>();
        List<String> docStatusList = new ArrayList<String>();
        if (cshtransaction.getDocumentTypeDesc() != null) {
            String[] docTypeArray = cshtransaction.getDocumentTypeDesc().split(",");
            for (int i = 0; i < docTypeArray.length; i++) {
                list.add(docTypeArray[i]);
            }
            cshtransaction.setQueryList(list);
        }
        if (cshtransaction.getContractStatusDesc() != null) {
            String[] docStatusArray = cshtransaction.getContractStatusDesc().split(",");
            for (int i = 0; i < docStatusArray.length; i++) {
                docStatusList.add(docStatusArray[i]);
            }
            cshtransaction.setQueryList1(docStatusList);
        }
        List<Map> maps = mapper.csh410aConContractDepoistQuery(cshtransaction);
        return maps;
    }

    @Override
    public void contactAddDeposit(Long companyId, HlsCusCshTransaction cshTransaction, IRequest requestContext, HttpSession session) throws BeyondAmountLimitException, WriteOffTypeNullException, ChangeLimitException {
        //包含收款+核销操作 ， 在cshTransaction中加入了核销的相关字段

        Double writeOffAmount = cshTransaction.getWriteOffAmount();

        //这里是收款新增，同时核销保证金 ， 新增的时候 现金事务上的核销金额为0
        cshTransaction.setWriteOffAmount(0D);
        updateCshTransaction(companyId, cshTransaction, requestContext);

        HlsCusCshWriteOff cshWriteOff = new HlsCusCshWriteOff();
        cshWriteOff.set__status("add");
        cshWriteOff.setBpName(cshTransaction.getBpName());
        cshWriteOff.setCashflowId(cshTransaction.getCashflowId());
        cshWriteOff.setCfItem(cshTransaction.getCfItem());
        cshWriteOff.setContractId(cshTransaction.getContractId());
        cshWriteOff.setContractNumber(cshTransaction.getContractNumber());
        cshWriteOff.setDueAmount(cshTransaction.getDueAmount());
        cshWriteOff.setDueDate(cshTransaction.getDueDate());
        cshWriteOff.setSurplusAmount(cshTransaction.getSurplusAmount());
        cshWriteOff.setTimes(cshTransaction.getTimes());
        cshWriteOff.setWriteOffDueAmount(writeOffAmount);
        cshWriteOff.setWriteOffPrincipal(0D);
        cshWriteOff.setWriteOffInterest(0D);
        cshWriteOff.setWriteOffType("RECEIPT_DEPOSIT");

        List<HlsCusCshWriteOff> cshWriteOffs = new ArrayList<HlsCusCshWriteOff>();
        cshWriteOffs.add(cshWriteOff);


        cshTransaction.setDepositCshWriteOffs(cshWriteOffs);

        List<HlsCusCshWriteOff> cshWriteOffList = new ArrayList<HlsCusCshWriteOff>();

        try {
            cshWriteOffList = getCshWriteOffs(cshTransaction);

        } catch (BeyondAmountLimitException e) {
            e.printStackTrace();
        }
        try {
            cshWriteOffService.updateWriteOff(requestContext, cshWriteOffList, session);
        } catch (BeyondAmountLimitException e) {
            e.printStackTrace();
        } catch (WriteOffTypeNullException e) {
            e.printStackTrace();
        } catch (ChangeLimitException e) {
            e.printStackTrace();
        }

    }

    @Override
    public void updateCshTrByPrimaryKey(HlsCusCshTransaction cshTr) {
        mapper.updateCshTrByPrimaryKey(cshTr);
    }


    private List<HlsCusCshWriteOff> getCshWriteOffs(HlsCusCshTransaction cshTransaction) throws BeyondAmountLimitException {

        List<HlsCusCshWriteOff> cshWriteOffs = new ArrayList<HlsCusCshWriteOff>();

        if (cshTransaction != null) {
            if (cshTransaction.getCreditCshWriteOffs() != null) {
                for (HlsCusCshWriteOff cshWriteOff : cshTransaction.getCreditCshWriteOffs()) {
                    cshWriteOffs.add(cshWriteOff);
                }
            }
            if (cshTransaction.getAdvanceCshWriteOffs() != null) {
                for (HlsCusCshWriteOff cshWriteOff : cshTransaction.getAdvanceCshWriteOffs()) {
                    cshWriteOffs.add(cshWriteOff);
                }
            }
            if (cshTransaction.getDepositCshWriteOffs() != null) {
                for (HlsCusCshWriteOff cshWriteOff : cshTransaction.getDepositCshWriteOffs()) {
                    cshWriteOffs.add(cshWriteOff);
                }
            }
        }

        if (cshWriteOffs != null) {
            long sumAmount = 0L;

            for (HlsCusCshWriteOff cshWriteOff : cshWriteOffs) {
                if (cshWriteOff.getSurplusAmount() != null && cshWriteOff.getWriteOffDueAmount() != null) {
                    /**
                     * 当核销金额大于预收款金额，抛出异常
                     */
                    if (cshWriteOff.getWriteOffDueAmount() > cshWriteOff.getSurplusAmount()) {
                        throw new BeyondAmountLimitException();
                    }

                    sumAmount += cshWriteOff.getWriteOffDueAmount();
                }

                cshWriteOff.setCshTransactionId(cshTransaction.getTransactionId());
                cshWriteOff.setWriteOffDate(cshTransaction.getWriteOffDate());
            }

            List<HlsCusCshTransaction> cshTransactions = mapper.queryDetailById(cshTransaction.getTransactionId());
            HlsCusCshTransaction transaction = cshTransactions.get(0);

            /**
             * 当核销金额总和大于收款金额时，抛出异常
             */
            if (transaction.getWriteOffAmount() == null) {
                transaction.setWriteOffAmount(0d);
            }

            Double sum = sumAmount + transaction.getWriteOffAmount();

            if (sum > transaction.getTransactionAmount()) {
                throw new BeyondAmountLimitException();
            }
        }


        return cshWriteOffs;
    }

    //收款保证金池核销
    @Override
    public HlsCusCshTransaction cshTransactionAddDpositPool(Long companyId, HlsCusCshTransaction cshTransaction, IRequest requestContext, HttpSession session) throws BeyondAmountLimitException, WriteOffTypeNullException, ChangeLimitException {
        //获取归属核销保证金的两个字段，重置cshtransaction上的相应字段
        Double writeOffAmount = cshTransaction.getWriteOffAmount();
        Date writeOffDate = cshTransaction.getWriteOffDate();
        cshTransaction.setWriteOffDate(null);
        cshTransaction.setWriteOffAmount(null);
        HlsCusCshTransaction cshTransactionR = self().updateCshTransaction(companyId, cshTransaction, requestContext);
        cshTransactionR.setWriteOffAmount(writeOffAmount);
        cshTransactionR.setWriteOffDate(writeOffDate);
        //组装CshWriteOff
        HlsCusCshWriteOff cshWriteOff = new HlsCusCshWriteOff();
        cshWriteOff.setWriteOffDueAmount(writeOffAmount);
        cshWriteOff.setWriteOffInterest(0D);
        cshWriteOff.setWriteOffPrincipal(0D);
        cshWriteOff.setWriteOffType("RECEIPT_DEPOSIT_POOL");
        cshWriteOff.set__status("add");
        //set
        List<HlsCusCshWriteOff> depositWriteOffList = new ArrayList<HlsCusCshWriteOff>();
        depositWriteOffList.add(cshWriteOff);
        //头上字段补录
        cshTransactionR.setSumWriteOffAmount(writeOffAmount);
        cshTransactionR.setDepositCshWriteOffs(depositWriteOffList);
        List<HlsCusCshWriteOff> cshWriteOffs = new ArrayList<HlsCusCshWriteOff>();
        cshWriteOffs = getCshWriteOffs(cshTransactionR);
        cshWriteOffService.updateWriteOff(requestContext, cshWriteOffs, session);
        return cshTransactionR;
    }

    /**
     * 新增收据-收款
     *
     * @param cshTransaction
     * @param page
     * @param pageSize
     * @param iRequest
     * @return
     */
    @Override
    public List<HlsCusCshTransaction> selectAllNotInvoiced(HlsCusCshTransaction cshTransaction, int page, int pageSize, IRequest iRequest) {
        PageHelper.startPage(page, pageSize);
        List<HlsCusCshTransaction> list = mapper.selectAllNotInvoiced(cshTransaction);
        return list;
    }

    @Override
    public List<HlsCusCshTransaction> queryByCshWriteId(HlsCusCshTransaction cshTransaction) {
        return mapper.queryByCshWriteId(cshTransaction);
    }
}