package hls.core.csh.service.impl;

import com.hand.hap.core.IRequest;
import hls.core.csh.dto.HlsCusCshBankAccountBalance;
import hls.core.csh.dto.HlsCusCshPaymentReqLn;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusCshWriteOffMapper;
import hls.core.csh.service.*;
import hls.core.fct.dto.HlsCusFctContract;
import hls.core.fct.dto.HlsCusFctContractWithdraw;
import hls.core.fct.dto.HlsCusFctContractWithdrawCf;
import hls.core.fct.mapper.HlsCusFctContractMapper;
import hls.core.fct.mapper.HlsCusFctContractWithdrawCfMapper;
import hls.core.fct.mapper.HlsCusFctContractWithdrawMapper;
import hls.core.fct.service.FctContractWithdrawCfService;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.components.CalculateUtil;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * Created by syj on 2017/12/06.
 */
@Service
public class CshWriteOffFctPaymentServiceImpl implements CshTransactionCommon {
    private static final String writeOffType = "FCT_PAYMENT_DEBT";
    public static final String FCT_CONTRACT_WITHDRAW = "FCT_CONTRACT_WITHDRAW";
    public Map<Long, Double> amountMap;
    public static Double WRITE_OFF_AMOUNT = 0d;
    static HlsCusCshTransaction tempCshTransaction = null;

    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;


    @Autowired
    private HlsCusFctContractWithdrawMapper hlsCusFctContractWithdrawMapper;
    @Autowired
    private HlsCusFctContractWithdrawCfMapper hlsCusFctContractWithdrawCfMapper;
    @Autowired
    private HlsCusFctContractMapper hlsCusFctContractMapper;
    @Autowired
    CshTransactionService cshTransactionService;
    @Autowired
    CshWriteOffService cshWriteOffService;
    @Autowired
    ICshPaymentReqLnService cshPaymentReqLnService;
    @Autowired
    private FctContractWithdrawCfService fctContractWithdrawCfService;

    @Autowired
    private SysEventService sysEventService;

    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private JeTrxCommonService commonService;

    @Autowired
    HlsCusCshWriteOffMapper cshWriteOffMapper;

    @Autowired
    CshBankAccountBalanceService bankAccountBalanceService;


    Map<String, String> params = new HashMap<String, String>();

    public static Double transactionDueAmount = 0d;
    public static List<Long> list = new ArrayList<Long>();
    static HlsCusCshTransaction temp_cshTransaction = null;
    public static Double amount_paid = 0D;

    Calendar calendar = Calendar.getInstance();

    private int currentYear = calendar.get(Calendar.YEAR);

    private int currentMonth = calendar.get(Calendar.MONTH) + 1;


    @Override
    public String getWriteOffType() {

        return writeOffType;
    }

    /**
     * 生成编码
     *
     * @param iRequest
     * @param
     * @return
     */
    public String getCodeValue(IRequest iRequest, HlsCusCshTransaction tempCshTransaction) {
        Map<String, String> params = new HashMap<String, String>();
        String value = codingRuleValuesService.getCodeRuleValue(iRequest, tempCshTransaction.getTransactionCategory(), tempCshTransaction.getTransactionType(), tempCshTransaction.getBusinessType(), params);
        return value;
    }

    /*核销主方法*/
    @Override
    public void process(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        Long company_id = CshWriteOffServiceImpl.company_id;//公司ID
        amountMap = CshWriteOffServiceImpl.amountMap;

        Map<String, Object> param = new HashMap<>();
        Long cf = cshWriteOff.getWithdrawCfId();

        HlsCusFctContractWithdraw hlsCusFctContractWithdraw = hlsCusFctContractWithdrawMapper.selectByPrimaryKey(cshWriteOff.getWithdrawId());
        HlsCusFctContractWithdrawCf hlsCusFctContractWithdrawCf = hlsCusFctContractWithdrawCfMapper.selectByPrimaryKey(cf);
        HlsCusFctContract hlsCusFctContract = hlsCusFctContractMapper.selectByPrimaryKey(hlsCusFctContractWithdrawCf.getContractId());

        String hlsCusFctContractCurrency = hlsCusFctContract.getCurrency();
        HLSCurrency hlsTransactionCurrency = new HLSCurrency();
        hlsTransactionCurrency.setCurrencyCode(hlsCusFctContractCurrency);
        hlsTransactionCurrency = hlsCurrencyMapper.selectOne(hlsTransactionCurrency);

        param.put("amount", cshWriteOff.getCshWriteOffAmount());
        param.put("currency", hlsTransactionCurrency.getCurrencySymbol());


        if (cshWriteOff.getCshTransaction() != null) {
            WRITE_OFF_AMOUNT = 0D;
            //插入现金事务表
            tempCshTransaction = cshWriteOff.getCshTransaction();
            if (company_id == null) {
                throw new IllegalArgumentException("公司ID获取失败!");
            } else {
                tempCshTransaction.setCompanyId(company_id);
            }
            tempCshTransaction.setTransactionNum(getCodeValue(iRequest, tempCshTransaction));
            cshTransactionService.insertSelective(iRequest, tempCshTransaction);

            //此处应插入凭证事务
            Map trxMap = new HashMap<>();
            trxMap.put("jeTrxId", tempCshTransaction.getTransactionId());
            trxMap.put("companyId", cshWriteOff.getCompanyId());
            trxMap.put("jeSourceId", hlsCusFctContractWithdraw.getWithdrawId());
            trxMap.put("jeSourceDoc", FCT_CONTRACT_WITHDRAW);
            AbstractJeTrxService transactionJeTrx = commonService.map.get("CSH_TRANSACTION");
            transactionJeTrx.process(iRequest, trxMap);

            param.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_TRANSACTION_PAYMENT);
            sysEventService.createEvent(iRequest, tempCshTransaction.getBpId(), "CSH_TRANSACTION", "", param);
        }
        if (tempCshTransaction.getTransactionId() != null) {
            cshWriteOff.setCshTransactionId(tempCshTransaction.getTransactionId());//给核销事务添加现金事务ID
        }
        WRITE_OFF_AMOUNT = CalculateUtil.add(WRITE_OFF_AMOUNT, cshWriteOff.getCshWriteOffAmount());//记录当前核销金额
        //插入核销表
        cshWriteOff.setWriteOffDocCategory("FCT_CONTRACT_WITHDRAW");
        cshWriteOff.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());
        cshWriteOffService.insertSelective(iRequest, cshWriteOff);

        //此处应添加付款核销事件
        //修改申请行表的amount_paid和payment_flag
        HlsCusCshPaymentReqLn cshPaymentReqLn = (HlsCusCshPaymentReqLn) cshPaymentReqLnService
                .selectCshPaymentReqLnDetailByLnID(Arrays.asList(cshWriteOff.getPaymentReqLineId().toString())).get(0);//获取当前申请行信息
        if (cshPaymentReqLn.getAmount_paid() == null) {
            cshPaymentReqLn.setAmount_paid(0d);
        }
        Double amountPaid = CalculateUtil.add(cshPaymentReqLn.getAmount_paid(), cshWriteOff.getCshWriteOffAmount());
        cshPaymentReqLn.setAmount_paid(amountPaid);
        if (new BigDecimal(amountPaid).compareTo(new BigDecimal(cshPaymentReqLn.getAmount())) == 0) {
            cshPaymentReqLn.setPayment_flag("FULL");
            cshPaymentReqLn.setPayment_completed_date(cshWriteOff.getWriteOffDate());
        } else if (new BigDecimal(amountPaid).compareTo(new BigDecimal(cshPaymentReqLn.getAmount())) < 0) {
            cshPaymentReqLn.setPayment_flag("PARTIAL");
        } else {
            throw new BeyondAmountLimitException();
        }

        cshPaymentReqLnService.updateAmountPaid(cshPaymentReqLn);
        //修改还款表 writeoffamount和核销标志
        HlsCusFctContractWithdrawCf withdrawCf = new HlsCusFctContractWithdrawCf();
        withdrawCf.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());
        withdrawCf.setCfItem(40L);
        withdrawCf.setTimes(0L);
        withdrawCf = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(withdrawCf).get(0);

        Double repayWriteOffAmount = withdrawCf.getWriteOffAmount();
        repayWriteOffAmount = CalculateUtil.add(repayWriteOffAmount, cshWriteOff.getCshWriteOffAmount());
        withdrawCf.setWriteOffAmount(repayWriteOffAmount);//核销金额变为原核销金额加上本次核销金额
        if (new BigDecimal(repayWriteOffAmount).compareTo(new BigDecimal(withdrawCf.getDueAmount())) == 0) {
            withdrawCf.setWriteOffFlag("FULL");
        } else if (new BigDecimal(repayWriteOffAmount).compareTo(new BigDecimal(withdrawCf.getDueAmount())) < 0) {
            withdrawCf.setWriteOffFlag("PARTIAL");
        } else {
            throw new BeyondAmountLimitException();
        }
        fctContractWithdrawCfService.updateByPrimaryKeySelective(iRequest, withdrawCf);


        //此处应插入凭证事务
        Map map = new HashMap<>();
        map.put("jeTrxId", cshWriteOff.getWriteOffId());
        map.put("companyId", cshWriteOff.getCompanyId());
        map.put("jeSourceId", hlsCusFctContractWithdraw.getWithdrawId());
        map.put("jeSourceDoc", FCT_CONTRACT_WITHDRAW);
        AbstractJeTrxService writeOffJeTrx = commonService.map.get("CSH_WRITE_OFF");
        writeOffJeTrx.process(iRequest, map);
    }

    /*反冲方法*/
    @Override
    public void reversed(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        // 1、核销表插入一条记录
        if (cshWriteOff.getWriteOffDueAmount() != null) {
            cshWriteOff.setCshWriteOffAmount(0 - cshWriteOff.getWriteOffDueAmount());
        }
        if (cshWriteOff.getWriteOffDueAmount() != null) {
            cshWriteOff.setWriteOffDueAmount(0 - cshWriteOff.getWriteOffDueAmount());
        }
        if (cshWriteOff.getWriteOffPrincipal() != null) {
            cshWriteOff.setWriteOffPrincipal(0 - cshWriteOff.getWriteOffPrincipal());
        }
        if (cshWriteOff.getWriteOffInterest() != null) {
            cshWriteOff.setWriteOffInterest(0 - cshWriteOff.getWriteOffInterest());
        }

        HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);
        writeOff.setReversedFlag("R");
        writeOff.setReversedWriteOffId(cshWriteOff.getWriteOffId());
        writeOff.setReversedDate(cshWriteOff.getReversedDate());
        cshWriteOffService.insertSelective(iRequest, writeOff);
        // 2、更新现金事物表
        HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
        cshTransactionService.updateCshTrByPrimaryKey(cshTran);
        // 3、更新原核销表数据
        cshWriteOffMapper.updateCshWriteOffbyId(cshWriteOff.getWriteOffId(), "W", writeOff.getWriteOffId(),
                cshWriteOff.getReversedDate(), iRequest.getUserId(), new Date());
        // 4、插入一条现金事物表
        HlsCusCshTransaction cshTransaction = getCshTransaction(iRequest, cshWriteOff, "REFUND");
        cshTransaction.setTransactionAmount(cshWriteOff.getWriteOffDueAmount());
        cshTransaction.setReversedFlag("R");
        cshTransaction.setReversedTrxId(cshWriteOff.getSubsequentCshTrxId());
        cshTransaction.setReversedDate(cshWriteOff.getReversedDate());
        cshTransaction.setSourceDocLineId(writeOff.getWriteOffId());
        cshTransactionService.insertSelective(iRequest, cshTransaction);
        // 5、更新后续现金事物表
        HlsCusCshTransaction tcx = new HlsCusCshTransaction();
        tcx.setTransactionId(cshWriteOff.getSubsequentCshTrxId());
        tcx.setReversedFlag("W");
        tcx.setReversedTrxId(cshTransaction.getTransactionId());
        tcx.setReversedDate(cshTransaction.getReversedDate());
        cshTransactionService.updateCshTrByPrimaryKey(tcx);
        // 6、更新核销插入核销数据的关联事物Id
        cshWriteOffMapper.updateSubCshTrxId(writeOff.getWriteOffId(), cshTransaction.getTransactionId(),
                cshTransaction.getTransactionAmount());
        // 更新账户余额表

        paymentBankAccountBalance(iRequest, cshTransaction);
    }

    private HlsCusCshWriteOff getReceiptCredit(HlsCusCshWriteOff cshWriteOff) {
        HlsCusCshWriteOff wo = new HlsCusCshWriteOff();

        wo.setWriteOffType(cshWriteOff.getWriteOffType());
        wo.setWriteOffDate(cshWriteOff.getWriteOffDate());
        wo.setCshTransactionId(cshWriteOff.getCshTransactionId());
        wo.setCshWriteOffAmount(cshWriteOff.getWriteOffDueAmount());
        wo.setReversedFlag("N");
        wo.setDescription(cshWriteOff.getDescription());
        wo.setWithdrawCfId(cshWriteOff.getWithdrawCfId());
        wo.setWithdrawId(cshWriteOff.getWithdrawId());
        wo.setPaymentReqId(cshWriteOff.getPaymentReqId());
        wo.setPaymentReqLineId(cshWriteOff.getPaymentReqLineId());
        wo.setTimes(cshWriteOff.getTimes());
        wo.setCfItem(cshWriteOff.getCfItem());
        wo.setCfType(cshWriteOff.getCfType());
        wo.setWriteOffDueAmount(cshWriteOff.getWriteOffDueAmount());
        wo.setWriteOffPrincipal(cshWriteOff.getWriteOffPrincipal());
        wo.setWriteOffInterest(cshWriteOff.getWriteOffInterest());

        return wo;
    }


    private HlsCusCshTransaction updateCshTrxAfterWriteoff(HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        HlsCusCshTransaction ct = new HlsCusCshTransaction();

        List<HlsCusCshTransaction> cshTransactions = cshTransactionService.queryDetailById(cshWriteOff.getCshTransactionId());

        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

        if (cshTransaction.getWriteOffAmount() == null) {
            cshTransaction.setWriteOffAmount(0d);
        }
        if (cshWriteOff.getWriteOffDueAmount() == null) {
            cshWriteOff.setWriteOffDueAmount(0d);
        }

        Double sumAmount = cshTransaction.getWriteOffAmount() + cshWriteOff.getWriteOffDueAmount();

        if (new BigDecimal(sumAmount).compareTo(new BigDecimal(0)) == 0) {
            ct.setWriteOffFlag("NOT");
            ct.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) < 0) {
            ct.setWriteOffFlag("PARTIAL");
            ct.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) == 0) {
            ct.setWriteOffFlag("FULL");
            ct.setWriteOffAmount(sumAmount);
            ct.setFullWriteOffDate(cshWriteOff.getWriteOffDate());
        } else {
            throw new BeyondAmountLimitException();
        }

        ct.setTransactionId(cshWriteOff.getCshTransactionId());

        return ct;
    }

    private HlsCusCshTransaction getCshTransaction(IRequest iRequest, HlsCusCshWriteOff cshWriteOff, String writeOffType) {
        List<HlsCusCshTransaction> cshTransactions = cshTransactionService.queryDetailById(cshWriteOff.getCshTransactionId());

        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

        cshTransaction.setTransactionId(null);
        cshTransaction.setTransactionCategory("CSH_TRANSACTION");
        if ("RECEIPT_ADVANCE_RECEIPT".equals(writeOffType)) {
            cshTransaction.setTransactionType("ADVANCE_RECEIPT");
            cshTransaction.setBusinessType("ADVANCE_RECEIPT");
        } else if ("RECEIPT_DEPOSIT".equals(writeOffType)) {

            cshTransaction.setTransactionType("DEPOSIT");
            cshTransaction.setBusinessType("DEPOSIT");
        } else if ("RECEIPT_DEPOSIT_POOL".equals(writeOffType)) {
            cshTransaction.setTransactionType("DEPOSIT_POOL");
            cshTransaction.setBusinessType("DEPOSIT");
        } else if ("REFUND".equals(writeOffType)) {
            cshTransaction.setTransactionType("REFUND");
            cshTransaction.setBusinessType("PAYMENT");
        }
        if (cshWriteOff.getContractId() != null) {
            cshTransaction.setContractId(cshWriteOff.getContractId());
        }
        cshTransaction.setTransactionNum(codingRuleValuesService.getCodeRuleValue(iRequest, cshTransaction.getTransactionCategory(), cshTransaction.getTransactionType(), cshTransaction.getBusinessType(), params));
        cshTransaction.setTransactionAmount(cshWriteOff.getWriteOffDueAmount());
        cshTransaction.setReversedFlag("N");
        cshTransaction.setReversedTrxId(null);
        cshTransaction.setReversedDate(null);
        cshTransaction.setPostedFlag("N");
        cshTransaction.setHandlingCharge(null);
        cshTransaction.setDescription(cshWriteOff.getDescription());
        cshTransaction.setWriteOffFlag("NOT");
        cshTransaction.setWriteOffAmount(null);
        cshTransaction.setFullWriteOffDate(null);
        cshTransaction.setSourceDocCategory("CSH_WRITE_OFF");

        return cshTransaction;
    }


    private void paymentBankAccountBalance(IRequest iRequest, HlsCusCshTransaction cshTransaction) {
        Calendar c = Calendar.getInstance();
        c.setTime(cshTransaction.getTransactionDate());
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1;
        HlsCusCshBankAccountBalance accountBalance = null;

        if (year == currentYear && month == currentMonth) {
            HlsCusCshBankAccountBalance bankAccountBalance = bankAccountBalanceService.selectAccountBalance(
                    cshTransaction.getBankAccountId(), Long.valueOf(currentYear), Long.valueOf(currentMonth));

            if (bankAccountBalance == null) {
                bankAccountBalance = new HlsCusCshBankAccountBalance();

                bankAccountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                bankAccountBalance.setPeriodYear(Long.valueOf(year));
                bankAccountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance = bankAccountBalanceService.getCloseBankAccountBalance(bankAccountBalance);
                if (cshTransaction.getTransactionAmount() == null) {
                    cshTransaction.setTransactionAmount(0d);
                }
                if (accountBalance == null) {
                    bankAccountBalance.setOpeningBalance(0d);
                    bankAccountBalance.setBalance(0 - cshTransaction.getTransactionAmount());
                } else {
                    bankAccountBalance.setOpeningBalance(accountBalance.getBalance());
                    bankAccountBalance.setBalance(accountBalance.getBalance() - cshTransaction.getTransactionAmount());
                }
                bankAccountBalance.setPeriodIncome(0d);
                bankAccountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
                bankAccountBalanceService.insertSelective(iRequest, bankAccountBalance);

            } else {
                accountBalance = new HlsCusCshBankAccountBalance();
                accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                accountBalance.setPeriodYear(Long.valueOf(year));
                accountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance.setPeriodIncome(0d);
                accountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
                bankAccountBalanceService.updateByPrimaryKeySelective(iRequest, accountBalance);
            }

        } else {
            HlsCusCshBankAccountBalance bankAccountBalance = bankAccountBalanceService
                    .selectAccountBalance(cshTransaction.getBankAccountId(), Long.valueOf(year), Long.valueOf(month));
            if (bankAccountBalance == null) {
                bankAccountBalance = new HlsCusCshBankAccountBalance();

                bankAccountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                bankAccountBalance.setPeriodYear(Long.valueOf(year));
                bankAccountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance = bankAccountBalanceService.getCloseBankAccountBalance(bankAccountBalance);
                if (cshTransaction.getTransactionAmount() == null) {
                    cshTransaction.setTransactionAmount(0d);
                }
                if (accountBalance == null) {
                    bankAccountBalance.setOpeningBalance(0d);
                    bankAccountBalance.setBalance(0 - cshTransaction.getTransactionAmount());
                } else {
                    bankAccountBalance.setOpeningBalance(accountBalance.getBalance());
                    bankAccountBalance.setBalance(accountBalance.getBalance() - cshTransaction.getTransactionAmount());
                }
                bankAccountBalance.setPeriodIncome(0d);
                bankAccountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
                bankAccountBalanceService.insertSelective(iRequest, bankAccountBalance);

            } else {
                accountBalance = new HlsCusCshBankAccountBalance();
                accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                accountBalance.setPeriodYear(Long.valueOf(year));
                accountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance.setPeriodIncome(cshTransaction.getTransactionAmount());
                accountBalance.setPeriodExpense(0d);
                bankAccountBalanceService.updateByPrimaryKeySelective(iRequest, accountBalance);

            }

            accountBalance = new HlsCusCshBankAccountBalance();
            accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
            accountBalance.setPeriodYear(Long.valueOf(year));
            accountBalance.setPeriodMonth(Long.valueOf(month));
            accountBalance.setPeriodIncome(0d);
            accountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
            bankAccountBalanceService.updateByPrimaryKeySelective(iRequest, accountBalance);
        }

    }

}
