package hls.core.csh.service.impl;

import com.hand.hap.core.IRequest;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.service.CshTransactionCommon;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWriteOffService;
import hls.core.fct.dto.HlsCusFctContract;
import hls.core.fct.dto.HlsCusFctContractWithdrawCf;
import hls.core.fct.mapper.HlsCusFctContractMapper;
import hls.core.fct.mapper.HlsCusFctContractWithdrawCfMapper;
import hls.core.fct.mapper.HlsCusFctContractWithdrawMapper;
import hls.core.fct.service.FctContractWithdrawCfService;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.dto.HlsCashflowItem;
import hls.core.hls.mapper.HlsCfItemMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by syj on 2017/12/18.
 */
@Service
public class CshWriteOffFctReceiptServiceImpl implements CshTransactionCommon {
    private static final String WRITE_OFF_TYPE = "FCT_RECEIPT_CREDIT";
    public static final String FULL = "FULL";
    public static final String PARTIAL = "PARTIAL";
    public static final String NOT = "NOT";
    public static final String FCT_CONTRACT_WITHDRAW = "FCT_CONTRACT_WITHDRAW";
    public static final String FCT_CONTRACT = "FCT_CONTRACT";

    @Autowired
    private CshTransactionService cshTransactionService;
    @Autowired
    private HlsCusCshTransactionMapper cshTransactionMapper;
    @Autowired
    private CshWriteOffService cshWriteOffService;
    @Autowired
    private HlsCusFctContractMapper fctContractMapper;
    @Autowired
    private HlsCusFctContractWithdrawMapper fctContractWithdrawMapper;


    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private HlsCusBpMasterMapper hlsCusBpMasterMapper;
    @Autowired
    private HlsCfItemMapper hlsCfItemMapper;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;

    @Autowired
    private HlsCusFctContractWithdrawCfMapper fctContractWithdrawCfMapper;

    @Autowired
    private FctContractWithdrawCfService fctContractWithdrawCfservice;


    @Autowired
    private JeTrxCommonService commonService;

    @Override
    public String getWriteOffType() {
        return WRITE_OFF_TYPE;
    }

    /*核销主方法*/
    @Override
    public void process(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        // 校验
        // 1.校验核销金额是否超出现金事物最大可核销金额
        // 2.校验核销金额是否超出现金流应收金额
        Long transactionId = cshWriteOff.getCshTransactionId();
        HlsCusCshTransaction transaction = cshTransactionMapper.selectByPrimaryKey(transactionId);
        Double transactionAmount = transaction.getTransactionAmount();//现金事物上的金额
        Double writeOffAmount = transaction.getWriteOffAmount()!=null ? transaction.getWriteOffAmount() : 0d;//现金事物上已经核销的金额
        Double leftAmount = CalculateUtil.sub(transactionAmount, writeOffAmount);//剩余金额
        Double cshWriteOffAmount = cshWriteOff.getWriteOffDueAmount();//本次核销金额
        if (cshWriteOffAmount.compareTo(leftAmount) < 0) {
            transaction.setWriteOffFlag(PARTIAL);
        } else if (cshWriteOffAmount.compareTo(leftAmount) == 0) {
            transaction.setWriteOffFlag(FULL);
            transaction.setFullWriteOffDate(new Date());
        } else {
            throw new BeyondAmountLimitException();
        }

        HlsCusFctContractWithdrawCf fctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
        fctContractWithdrawCf.setWithdrawCfId(cshWriteOff.getCashflowId());
        fctContractWithdrawCf = fctContractWithdrawCfMapper.selectByPrimaryKey(fctContractWithdrawCf);
        Double cfLeftAmount = CalculateUtil.sub(fctContractWithdrawCf.getDueAmount(),fctContractWithdrawCf.getWriteOffAmount()); //现金流上的剩余未收金额
        if (cshWriteOffAmount.compareTo(cfLeftAmount) < 0) {
            fctContractWithdrawCf.setWriteOffFlag(PARTIAL);
        } else if (cshWriteOffAmount.compareTo(cfLeftAmount) == 0) {
            fctContractWithdrawCf.setWriteOffFlag(FULL);
        } else {
            throw new BeyondAmountLimitException();
        }
        fctContractWithdrawCf.setWriteOffAmount(CalculateUtil.add(cshWriteOffAmount,fctContractWithdrawCf.getWriteOffAmount()));

        // 插入核销表
        cshWriteOff.setWithdrawId(fctContractWithdrawCf.getWithdrawId());
        cshWriteOff.setWithdrawCfId(fctContractWithdrawCf.getWithdrawCfId());
        cshWriteOff.setWriteOffType(WRITE_OFF_TYPE);
        cshWriteOff.setWriteOffDocCategory("FCT_CONTRACT_WITHDRAW");
        cshWriteOffService.insertSelective(iRequest, cshWriteOff);

        // 修改现金流表
        fctContractWithdrawCfMapper.updateByPrimaryKey(fctContractWithdrawCf);


        // 更新现金事物表
        transaction.setWriteOffAmount(CalculateUtil.add(cshWriteOffAmount, writeOffAmount));//更新现金事物上的已核销金额
        cshTransactionService.updateByPrimaryKeySelective(iRequest, transaction);

        //插入事件
        Map<String, Object> params = new HashMap<>();
        HlsCusFctContract fctContract = fctContractMapper.selectByPrimaryKey(cshWriteOff.getContractId());
        String currency = fctContract.getCurrency();
        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(currency);
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);
        HlsCashflowItem hlsCashflowItem = new HlsCashflowItem();
        hlsCashflowItem.setCf_type(cshWriteOff.getCfType().toString());
        hlsCashflowItem.setCf_item(cshWriteOff.getCfItem().toString());
        hlsCashflowItem = hlsCfItemMapper.selectOne(hlsCashflowItem);
        //FIXME bpName参数不确定
        HlsCusBpMaster hlsCusBpMaster = hlsCusBpMasterMapper.selectByPrimaryKey(cshWriteOff.getBpId());
        params.put("amount", cshWriteOffAmount);
        params.put("currency", hlsCurrency.getCurrencySymbol());
        params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_LON_RECEIPT_CREDIT);
        params.put("cfName", hlsCashflowItem.getDescription());
        params.put("bpName", hlsCusBpMaster.getBpName());
        params.put("contractName", fctContract.getContractNumber());
        sysEventService.createEvent(iRequest, fctContractWithdrawCf.getWithdrawId(), "FCT_CONTRACT", "", params);


        //此处应插入凭证事务
        Map map = new HashMap<>();
        map.put("jeTrxId", cshWriteOff.getWriteOffId());
        map.put("companyId", cshWriteOff.getCompanyId());
        map.put("jeSourceId", fctContractWithdrawCf.getWithdrawId());
        map.put("jeSourceDoc", FCT_CONTRACT_WITHDRAW);
        AbstractJeTrxService writeOffJeTrx = commonService.map.get("CSH_WRITE_OFF");
        writeOffJeTrx.process(iRequest, map);
    }

    /*反冲方法*/
    @Override
    public void reversed(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        //预留方法
        Long originWriteOffId = cshWriteOff.getWriteOffId();//原始的核销记录ID
        Long transactionId = cshWriteOff.getCshTransactionId();//现金事物ID
        Date reversedDate = cshWriteOff.getReversedDate();//反冲日期
        String description = cshWriteOff.getDescription();//反冲描述
        HlsCusCshTransaction transaction = cshTransactionMapper.selectByPrimaryKey(transactionId);//现金事物对象
        Double transactionAmount = transaction.getTransactionAmount();
        Double transactionWriteOffAmount = transaction.getWriteOffAmount();
        HlsCusCshWriteOff originWriteOff = new HlsCusCshWriteOff();
        originWriteOff.setWriteOffId(originWriteOffId);
        originWriteOff = cshWriteOffService.selectByPrimaryKey(iRequest, originWriteOff);//找到原始记录的具体数据


        Long withdrawCfId = originWriteOff.getWithdrawCfId();

        Double writeOffDueAmount = originWriteOff.getWriteOffDueAmount();//核销金额
        Double cshWriteOffAmount = originWriteOff.getCshWriteOffAmount();//核销金额
        Double negativeWriteOffDueAmount = CalculateUtil.sub(0D, writeOffDueAmount);//得到负数金额
        Double negativeCshWriteOffAmount = CalculateUtil.sub(0D, cshWriteOffAmount);//得到负数金额
        HlsCusCshWriteOff reverseWriteOff = null;
        try {
            reverseWriteOff = (HlsCusCshWriteOff) BeanUtils.cloneBean(originWriteOff);//复制原始核销对象
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }
        reverseWriteOff = cleanStdWhoAndId(reverseWriteOff);
        reverseWriteOff.setReversedWriteOffId(originWriteOffId);
        reverseWriteOff.setReversedFlag("R");
        reverseWriteOff.setReversedDate(reversedDate);
        reverseWriteOff.setCshWriteOffAmount(negativeCshWriteOffAmount);
        reverseWriteOff.setWriteOffDueAmount(negativeWriteOffDueAmount);
        reverseWriteOff.setDescription(description);
        HlsCusCshWriteOff reverseResult = cshWriteOffService.insertSelective(iRequest, reverseWriteOff);

        Double newTransactionWriteOffAmount = CalculateUtil.add(transactionWriteOffAmount, negativeCshWriteOffAmount);
        if (newTransactionWriteOffAmount.compareTo(transactionAmount) < 0 && newTransactionWriteOffAmount.compareTo(0D) > 0) {
            transaction.setWriteOffFlag(PARTIAL);
        } else if (newTransactionWriteOffAmount.compareTo(0D) == 0) {
            transaction.setWriteOffFlag(NOT);
        } else if (newTransactionWriteOffAmount.compareTo(transactionAmount) == 0) {
            transaction.setWriteOffFlag(FULL);
            transaction.setFullWriteOffDate(new Date());
        } else {
            throw new BeyondAmountLimitException();
        }
        transaction.setWriteOffAmount(newTransactionWriteOffAmount);
        cshTransactionService.updateByPrimaryKeySelective(iRequest, transaction);

        originWriteOff.setReversedFlag("W");
        originWriteOff.setReversedDate(reversedDate);
        originWriteOff.setReversedWriteOffId(reverseResult.getWriteOffId());
        cshWriteOffService.updateByPrimaryKeySelective(iRequest, originWriteOff);

        HlsCusFctContractWithdrawCf fctContractWithdrawCf = fctContractWithdrawCfMapper.selectByPrimaryKey(withdrawCfId);


        Double dueAmount = fctContractWithdrawCf.getDueAmount();
        Double writeOffAmount = fctContractWithdrawCf.getWriteOffAmount();
        Double resultAmount = CalculateUtil.add(negativeCshWriteOffAmount, writeOffAmount);

        if (resultAmount.compareTo(0D) == 0) {
            fctContractWithdrawCf.setWriteOffFlag(NOT);
        } else if (resultAmount.compareTo(dueAmount) == 0) {
            fctContractWithdrawCf.setWriteOffFlag(FULL);
        } else if (resultAmount.compareTo(dueAmount) < 0 && resultAmount.compareTo(0D) > 0) {
            fctContractWithdrawCf.setWriteOffFlag(PARTIAL);
        } else {
            throw new BeyondAmountLimitException();
        }

        fctContractWithdrawCf.setWriteOffAmount(resultAmount);
        fctContractWithdrawCfservice.updateByPrimaryKeySelective(iRequest, fctContractWithdrawCf);


        //此处应插入凭证事务
        Map map = new HashMap<>();
        map.put("jeTrxId", reverseWriteOff.getWriteOffId());
        map.put("companyId", iRequest.getCompanyId());
        map.put("jeSourceId", fctContractWithdrawCf.getWithdrawId());
        map.put("jeSourceDoc", FCT_CONTRACT_WITHDRAW);
        map.put("reverseJeDate", originWriteOff.getReversedDate());
        map.put("reverseJeTrxId", originWriteOff.getWriteOffId());
        AbstractJeTrxService writeOffJeTrx = commonService.map.get("CSH_WRITE_OFF");
        writeOffJeTrx.process(iRequest, map);


    }

    /**
     * 清除who字段和id
     *
     * @param writeOff
     * @return
     */
    private static HlsCusCshWriteOff cleanStdWhoAndId(HlsCusCshWriteOff writeOff) {
        writeOff.setWriteOffId(null);
        writeOff.setObjectVersionNumber(null);
        writeOff.setCreatedBy(null);
        writeOff.setCreationDate(null);
        writeOff.setLastUpdateDate(null);
        writeOff.setLastUpdatedBy(null);
        writeOff.setLastUpdateLogin(null);
        return writeOff;
    }

}
