package hls.core.csh.service.impl;

import com.hand.hap.core.IRequest;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusCshPaymentReqLn;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.service.CshTransactionCommon;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWriteOffService;
import hls.core.csh.service.ICshPaymentReqLnService;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.dto.HlsCashflowItem;
import hls.core.hls.mapper.HlsCfItemMapper;
import hls.core.lon.dto.HlsCusLonContract;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.dto.HlsCusLonContractWithdraw;
import hls.core.lon.mapper.HlsCusLonContractMapper;
import hls.core.lon.mapper.HlsCusLonContractRepaymentMapper;
import hls.core.lon.service.ILonContractRepaymentService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by wangyan on 2017/11/17.
 */
@Service
public class CshWriteOffLonPaymentServiceImpl implements CshTransactionCommon {
    private static final String writeOffType = "LON_PAYMENT_DEBT";
    public static final String LON_CONTRACT = "LON_CONTRACT";
    public Map<Long, Double> amountMap;
    public static Double WRITE_OFF_AMOUNT = 0d;
    static HlsCusCshTransaction tempCshTransaction = null;

    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private HlsCusLonContractMapper lonContractMapper;
    @Autowired
    CshTransactionService cshTransactionService;
    @Autowired
    CshWriteOffService cshWriteOffService;
    @Autowired
    ICshPaymentReqLnService cshPaymentReqLnService;
    @Autowired
    ILonContractRepaymentService repaymentService;
    @Autowired
    private HlsCusLonContractRepaymentMapper lonContractRepaymentMapper;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private HlsCusBpMasterMapper hlsCusBpMasterMapper;
    @Autowired
    private HlsCfItemMapper hlsCfItemMapper;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private JeTrxCommonService commonService;


    @Override
    public String getWriteOffType() {

        return writeOffType;
    }

    /**
     * 生成编码
     *
     * @param iRequest
     * @param
     * @return
     */
    public String getCodeValue(IRequest iRequest, HlsCusCshTransaction tempCshTransaction) {
        Map<String, String> params = new HashMap<String, String>();
        String value = codingRuleValuesService.getCodeRuleValue(iRequest, tempCshTransaction.getTransactionCategory(), tempCshTransaction.getTransactionType(), tempCshTransaction.getBusinessType(), params);
        return value;
    }

    /*核销主方法*/
    @Override
    public void process(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        Long company_id = CshWriteOffServiceImpl.company_id;//公司ID
        amountMap = CshWriteOffServiceImpl.amountMap;
        //  HlsCusCshTransaction tempCshTransaction = new HlsCusCshTransaction();
        Map<String, Object> param = new HashMap<>();
        Long repaymentId = cshWriteOff.getCashflowId();
        HlsCusLonContractRepayment contractRepayment = lonContractRepaymentMapper.selectByPrimaryKey(repaymentId);
        HlsCusLonContract hlsCusLonContract = lonContractMapper.selectByPrimaryKey(contractRepayment.getContractId());
        String hlsCusLonContractCurrency = hlsCusLonContract.getCurrency();
        HLSCurrency hlsTransactionCurrency = new HLSCurrency();
        hlsTransactionCurrency.setCurrencyCode(hlsCusLonContractCurrency);
        hlsTransactionCurrency = hlsCurrencyMapper.selectOne(hlsTransactionCurrency);

        param.put("amount", cshWriteOff.getCshWriteOffAmount());
        param.put("currency", hlsTransactionCurrency.getCurrencySymbol());
        if (cshWriteOff.getCshTransaction() != null) {
            WRITE_OFF_AMOUNT = 0D;
            //插入现金事务表
            tempCshTransaction = cshWriteOff.getCshTransaction();
            if (company_id == null) {
                throw new IllegalArgumentException("公司ID获取失败!");
            } else {
                tempCshTransaction.setCompanyId(company_id);
            }
            tempCshTransaction.setTransactionNum(getCodeValue(iRequest, tempCshTransaction));
            cshTransactionService.insertSelective(iRequest, tempCshTransaction);

            //此处应插入凭证事务
            Map trxMap = new HashMap<>();
            trxMap.put("jeTrxId",tempCshTransaction.getTransactionId());
            trxMap.put("companyId",cshWriteOff.getCompanyId());
            trxMap.put("jeSourceId",contractRepayment.getContractId());
            trxMap.put("jeSourceDoc", LON_CONTRACT);
            AbstractJeTrxService transactionJeTrx = commonService.map.get("CSH_TRANSACTION");
            transactionJeTrx.process(iRequest,trxMap);

            param.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_TRANSACTION_PAYMENT);
            sysEventService.createEvent(iRequest, tempCshTransaction.getBpId(), "CSH_TRANSACTION", "", param);
        }
        if (tempCshTransaction.getTransactionId() != null) {
            cshWriteOff.setCshTransactionId(tempCshTransaction.getTransactionId());//给核销事务添加现金事务ID
        }
        WRITE_OFF_AMOUNT = CalculateUtil.add(WRITE_OFF_AMOUNT, cshWriteOff.getCshWriteOffAmount());//记录当前核销金额
        //插入核销表
        cshWriteOff.setWriteOffDocCategory("LON_CONTRACT");
        cshWriteOff.setRepaymentId(cshWriteOff.getCashflowId());
        cshWriteOff.setCashflowId(null);
        //根据还款id获取合同id
        HlsCusLonContractRepayment dto = new HlsCusLonContractRepayment();
        dto.setRepaymentId(cshWriteOff.getRepaymentId());
        Long contractId = lonContractRepaymentMapper.select(dto).stream().findAny().orElseGet(HlsCusLonContractRepayment::new).getContractId();
        cshWriteOff.setContractId(contractId);
        cshWriteOffService.insertSelective(iRequest, cshWriteOff);
        //此处应添加付款核销事件
        //修改申请行表的amount_paid和payment_flag
        HlsCusCshPaymentReqLn cshPaymentReqLn = (HlsCusCshPaymentReqLn) cshPaymentReqLnService
                .selectCshPaymentReqLnDetailByLnID(Arrays.asList(cshWriteOff.getPaymentReqLineId().toString())).get(0);//获取当前申请行信息
        if (cshPaymentReqLn.getAmount_paid() == null) {
            cshPaymentReqLn.setAmount_paid(0d);
        }
        Double amountPaid = CalculateUtil.add(cshPaymentReqLn.getAmount_paid(), cshWriteOff.getCshWriteOffAmount());
        cshPaymentReqLn.setAmount_paid(amountPaid);
        if (new BigDecimal(amountPaid).compareTo(new BigDecimal(cshPaymentReqLn.getAmount())) == 0) {
            cshPaymentReqLn.setPayment_flag("FULL");
            cshPaymentReqLn.setPayment_completed_date(cshWriteOff.getWriteOffDate());
        } else if (new BigDecimal(amountPaid).compareTo(new BigDecimal(cshPaymentReqLn.getAmount())) < 0) {
            cshPaymentReqLn.setPayment_flag("PARTIAL");
        } else {
            throw new BeyondAmountLimitException();
        }

        cshPaymentReqLnService.updateAmountPaid(cshPaymentReqLn);
        //修改还款表 writeoffamount和核销标志
        HlsCusLonContractRepayment repayment = new HlsCusLonContractRepayment();
        repayment.setRepaymentId(cshWriteOff.getRepaymentId());
        repayment = repaymentService.selectByPrimaryKey(iRequest, repayment);//获取还款信息
        Double repayWriteOffAmount = repayment.getWriteOffAmount();
        repayWriteOffAmount = CalculateUtil.add(repayWriteOffAmount, cshWriteOff.getCshWriteOffAmount());
        repayment.setWriteOffAmount(repayWriteOffAmount);//核销金额变为原核销金额加上本次核销金额
        if (new BigDecimal(repayWriteOffAmount).compareTo(new BigDecimal(repayment.getDueAmount())) == 0) {
            repayment.setWriteOffFlag("FULL");
        } else if (new BigDecimal(repayWriteOffAmount).compareTo(new BigDecimal(repayment.getDueAmount())) < 0) {
            repayment.setWriteOffFlag("PARTIAL");
        } else {
            throw new BeyondAmountLimitException();
        }
        repaymentService.updateByPrimaryKeySelective(iRequest, repayment);

        //插入事件
        Map<String, Object> params = new HashMap<>();
        HlsCusLonContractRepayment lonContractRepayment = lonContractRepaymentMapper.selectByPrimaryKey(cshWriteOff.getRepaymentId());
        HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(lonContractRepayment.getContractId());
        String currency = lonContract.getCurrency();
        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(currency);
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);

        HlsCashflowItem hlsCashflowItem = new HlsCashflowItem();
        hlsCashflowItem.setCf_type(cshWriteOff.getCfType().toString());
        hlsCashflowItem.setCf_item(cshWriteOff.getCfItem().toString());
        hlsCashflowItem = hlsCfItemMapper.selectOne(hlsCashflowItem);

        //FIXME bpName参数不确定
        HlsCusBpMaster hlsCusBpMaster = hlsCusBpMasterMapper.selectByPrimaryKey(lonContract.getCreditBpId());

        params.put("amount", repayWriteOffAmount);
        params.put("currency", hlsCurrency.getCurrencySymbol());
        params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_LON_PAYMENT_DEBT);
        params.put("cfName", hlsCashflowItem.getDescription());
        params.put("bpName", hlsCusBpMaster.getBpName());
        params.put("contractName", lonContract.getContractNumber());
        sysEventService.createEvent(iRequest, cshWriteOff.getWithdrawId(), "LON_CONTRACT", "", params);
        //此处应插入凭证事务
        Map map = new HashMap<>();
        map.put("jeTrxId",cshWriteOff.getWriteOffId());
        map.put("companyId",cshWriteOff.getCompanyId());
        map.put("jeSourceId",lonContract.getContractId());
        map.put("jeSourceDoc", LON_CONTRACT);
        AbstractJeTrxService writeOffJeTrx = commonService.map.get("CSH_WRITE_OFF");
        writeOffJeTrx.process(iRequest,map);
    }
    /*反冲方法*/
    @Override
    public void reversed(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) {
        //预留方法
    }


}
