package hls.core.csh.service.impl;

import com.hand.hap.core.IRequest;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.service.CshTransactionCommon;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWriteOffService;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.HlsCashflowItem;
import hls.core.hls.dto.HlsCusHlsCreditLineTrx;
import hls.core.hls.mapper.HlsCfItemMapper;
import hls.core.hls.service.IHlsCreditLineTrxService;
import hls.core.lon.dto.HlsCusLonContract;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.dto.HlsCusLonContractWithdraw;
import hls.core.lon.mapper.HlsCusLonContractMapper;
import hls.core.lon.mapper.HlsCusLonContractRepaymentMapper;
import hls.core.lon.mapper.HlsCusLonContractWithdrawMapper;
import hls.core.lon.service.ILonContractWithdrawService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by fjm on 2017/11/20.
 */
@Service
public class CshWriteOffLonReceiptServiceImpl implements CshTransactionCommon {
    private static final String WRITE_OFF_TYPE = "LON_RECEIPT_CREDIT";
    public static final String FULL = "FULL";
    public static final String PARTIAL = "PARTIAL";
    public static final String NOT = "NOT";
    public static final String LON_CONTRACT_WITHDRAW = "LON_CONTRACT_WITHDRAW";
    public static final String LON_CONTRACT = "LON_CONTRACT";

    @Autowired
    private CshTransactionService cshTransactionService;
    @Autowired
    private HlsCusCshTransactionMapper cshTransactionMapper;
    @Autowired
    private CshWriteOffService cshWriteOffService;
    @Autowired
    private HlsCusLonContractMapper lonContractMapper;
    @Autowired
    private HlsCusLonContractWithdrawMapper lonContractWithdrawMapper;
    @Autowired
    private ILonContractWithdrawService lonContractWithdrawService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private HlsCusBpMasterMapper hlsCusBpMasterMapper;
    @Autowired
    private HlsCfItemMapper hlsCfItemMapper;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private IHlsCreditLineTrxService hlsCreditLineTrxService;
    @Autowired
    private HlsCusLonContractRepaymentMapper lonContractRepaymentMapper;
    @Autowired
    private JeTrxCommonService commonService;

    @Override
    public String getWriteOffType() {
        return WRITE_OFF_TYPE;
    }

    /*核销主方法*/
    @Override
    public void process(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {

        cshWriteOff.setWriteOffType(WRITE_OFF_TYPE);
        cshWriteOffService.insertSelective(iRequest, cshWriteOff);

        Long transactionId = cshWriteOff.getTransactionId();
        HlsCusCshTransaction transaction = cshTransactionMapper.selectByPrimaryKey(transactionId);
        Double transactionAmount = transaction.getTransactionAmount();//现金事物上的金额
        Double writeOffAmount = transaction.getWriteOffAmount();//现金事物上已经核销的金额
        Double leftAmount = CalculateUtil.sub(transactionAmount, writeOffAmount);//剩下的金额
        Double cshWriteOffAmount = cshWriteOff.getCshWriteOffAmount();//writeOff记录上的核销金额

        if (cshWriteOffAmount.compareTo(leftAmount) < 0) {
            transaction.setWriteOffFlag(PARTIAL);
        } else if (cshWriteOffAmount.compareTo(leftAmount) == 0) {
            transaction.setWriteOffFlag(FULL);
            transaction.setFullWriteOffDate(new Date());
        } else {
            throw new BeyondAmountLimitException();
        }
        transaction.setWriteOffAmount(CalculateUtil.add(cshWriteOffAmount, writeOffAmount));//更新现金事物上的已核销金额

        cshTransactionService.updateByPrimaryKeySelective(iRequest, transaction);

        //插入事件
        Map<String, Object> params = new HashMap<>();
        HlsCusLonContractWithdraw lonContractWithdraw = lonContractWithdrawMapper.selectByPrimaryKey(cshWriteOff.getWithdrawId());
        HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(lonContractWithdraw.getContractId());
        String currency = lonContract.getCurrency();
        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(currency);
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);

        HlsCashflowItem hlsCashflowItem = new HlsCashflowItem();
        hlsCashflowItem.setCf_type(cshWriteOff.getCfType().toString());
        hlsCashflowItem.setCf_item(cshWriteOff.getCfItem().toString());
        hlsCashflowItem = hlsCfItemMapper.selectOne(hlsCashflowItem);

        //FIXME bpName参数不确定
        HlsCusBpMaster hlsCusBpMaster = hlsCusBpMasterMapper.selectByPrimaryKey(lonContract.getCreditBpId());

        params.put("amount", cshWriteOffAmount);
        params.put("currency", hlsCurrency.getCurrencySymbol());
        params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_LON_RECEIPT_CREDIT);
        params.put("cfName", hlsCashflowItem.getDescription());
        params.put("bpName", hlsCusBpMaster.getBpName());
        params.put("contractName", lonContract.getContractNumber());
        sysEventService.createEvent(iRequest, cshWriteOff.getWithdrawId(), "LON_CONTRACT", "", params);

        //插入额度事物表
        HlsCusHlsCreditLineTrx hlsCreditLineTrx = new HlsCusHlsCreditLineTrx();
        hlsCreditLineTrx.setCompanyId(iRequest.getCompanyId());
        hlsCreditLineTrx.setTrxCode(LON_CONTRACT_WITHDRAW);
        hlsCreditLineTrx.setTrxAmount(cshWriteOffAmount);
        hlsCreditLineTrx.setSourceDocumentId(cshWriteOff.getWithdrawId());
        hlsCreditLineTrx.setSourceDocumentCategory(DocumentChangeCommon.getTableName(HlsCusLonContractWithdraw.class));
        hlsCreditLineTrx.setCreditLineId(lonContract.getCreditLineId());
        hlsCreditLineTrx.setTrxDate(cshWriteOff.getWriteOffDate());

        hlsCreditLineTrx = hlsCreditLineTrxService.insertSelective(iRequest, hlsCreditLineTrx);

        //更新额度主数据上对应的已占用额度
        hlsCreditLineTrxService.calcCreditMethod(iRequest, hlsCreditLineTrx.getTrxId());

        //此处应插入凭证事务
        Map map = new HashMap<>();
        map.put("jeTrxId", cshWriteOff.getWriteOffId());
        map.put("companyId", cshWriteOff.getCompanyId());
        map.put("jeSourceId", lonContract.getContractId());
        map.put("jeSourceDoc", LON_CONTRACT);
        AbstractJeTrxService writeOffJeTrx = commonService.map.get("CSH_WRITE_OFF");
        writeOffJeTrx.process(iRequest, map);
    }

    /*反冲方法*/
    @Override
    public void reversed(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        //预留方法
        Long originWriteOffId = cshWriteOff.getWriteOffId();//原始的核销记录ID
        Long transactionId = cshWriteOff.getCshTransactionId();//现金事物ID
        Date reversedDate = cshWriteOff.getReversedDate();//反冲日期
        String description = cshWriteOff.getDescription();//反冲描述
        HlsCusCshTransaction transaction = cshTransactionMapper.selectByPrimaryKey(transactionId);//现金事物对象
        Double transactionAmount = transaction.getTransactionAmount();
        Double transactionWriteOffAmount = transaction.getWriteOffAmount();
        HlsCusCshWriteOff originWriteOff = new HlsCusCshWriteOff();
        originWriteOff.setWriteOffId(originWriteOffId);
        originWriteOff = cshWriteOffService.selectByPrimaryKey(iRequest, originWriteOff);//找到原始记录的具体数据
        Long withdrawId = originWriteOff.getWithdrawId();//还款id
        HlsCusLonContractRepayment repayment = new HlsCusLonContractRepayment();
        repayment.setWithdrawId(withdrawId);
        List<HlsCusLonContractRepayment> repayments = lonContractRepaymentMapper.select(repayment);
        for (HlsCusLonContractRepayment item : repayments) {
            Double amount = item.getWriteOffAmount() != null ? item.getWriteOffAmount() : 0D;
            if (new Double(0).compareTo(amount) != 0) {
                throw new IllegalArgumentException("该笔提款已发生还款,不可直接反冲");
            }
        }
        Double writeOffDueAmount = originWriteOff.getWriteOffDueAmount();//核销金额
        Double cshWriteOffAmount = originWriteOff.getCshWriteOffAmount();//核销金额
        Double negativeWriteOffDueAmount = CalculateUtil.sub(0D, writeOffDueAmount);//得到负数金额
        Double negativeCshWriteOffAmount = CalculateUtil.sub(0D, cshWriteOffAmount);//得到负数金额
        HlsCusCshWriteOff reverseWriteOff = null;
        try {
            reverseWriteOff = (HlsCusCshWriteOff) BeanUtils.cloneBean(originWriteOff);//复制原始核销对象
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }
        reverseWriteOff = cleanStdWhoAndId(reverseWriteOff);
        reverseWriteOff.setReversedWriteOffId(originWriteOffId);
        reverseWriteOff.setReversedFlag("R");
        reverseWriteOff.setReversedDate(reversedDate);
        reverseWriteOff.setCshWriteOffAmount(negativeCshWriteOffAmount);
        reverseWriteOff.setWriteOffDueAmount(negativeWriteOffDueAmount);
        reverseWriteOff.setDescription(description);
        HlsCusCshWriteOff reverseResult = cshWriteOffService.insertSelective(iRequest, reverseWriteOff);

        Double newTransactionWriteOffAmount = CalculateUtil.add(transactionWriteOffAmount, negativeCshWriteOffAmount);
        if (newTransactionWriteOffAmount.compareTo(transactionAmount) < 0 && newTransactionWriteOffAmount.compareTo(0D) > 0) {
            transaction.setWriteOffFlag(PARTIAL);
        } else if (newTransactionWriteOffAmount.compareTo(0D) == 0) {
            transaction.setWriteOffFlag(NOT);
        } else if (newTransactionWriteOffAmount.compareTo(transactionAmount) == 0) {
            transaction.setWriteOffFlag(FULL);
            transaction.setFullWriteOffDate(new Date());
        } else {
            throw new BeyondAmountLimitException();
        }
        transaction.setWriteOffAmount(newTransactionWriteOffAmount);
        cshTransactionService.updateByPrimaryKeySelective(iRequest, transaction);

        originWriteOff.setReversedFlag("W");
        originWriteOff.setReversedDate(reversedDate);
        originWriteOff.setReversedWriteOffId(reverseResult.getWriteOffId());
        cshWriteOffService.updateByPrimaryKeySelective(iRequest, originWriteOff);

        HlsCusLonContractWithdraw lonContractWithdraw = lonContractWithdrawMapper.selectByPrimaryKey(withdrawId);
        Double dueAmount = lonContractWithdraw.getDueAmount();
        Double writeOffAmount = lonContractWithdraw.getWriteOffAmount();
        Double resultAmount = CalculateUtil.add(negativeCshWriteOffAmount, writeOffAmount);

        if (resultAmount.compareTo(0D) == 0) {
            lonContractWithdraw.setWriteOffFlag(NOT);
        } else if (resultAmount.compareTo(dueAmount) == 0) {
            lonContractWithdraw.setWriteOffFlag(FULL);
        } else if (resultAmount.compareTo(dueAmount) < 0 && resultAmount.compareTo(0D) > 0) {
            lonContractWithdraw.setWriteOffFlag(PARTIAL);
        } else {
            throw new BeyondAmountLimitException();
        }

        lonContractWithdraw.setWriteOffAmount(resultAmount);
        lonContractWithdrawService.updateByPrimaryKeySelective(iRequest, lonContractWithdraw);
        Long contractId = lonContractWithdraw.getContractId();
        HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(contractId);
        //插入额度事物表
        HlsCusHlsCreditLineTrx hlsCreditLineTrx = new HlsCusHlsCreditLineTrx();
        hlsCreditLineTrx.setCompanyId(iRequest.getCompanyId());
        hlsCreditLineTrx.setTrxCode(LON_CONTRACT_WITHDRAW);
        hlsCreditLineTrx.setTrxAmount(negativeWriteOffDueAmount);
        hlsCreditLineTrx.setSourceDocumentId(withdrawId);
        hlsCreditLineTrx.setSourceDocumentCategory(DocumentChangeCommon.getTableName(HlsCusLonContractWithdraw.class));
        hlsCreditLineTrx.setCreditLineId(lonContract.getCreditLineId());
        hlsCreditLineTrx.setTrxDate(lonContractWithdraw.getDueDate());
        hlsCreditLineTrx = hlsCreditLineTrxService.insertSelective(iRequest, hlsCreditLineTrx);
        //更新额度主数据上对应的已占用额度
        hlsCreditLineTrxService.calcCreditMethod(iRequest, hlsCreditLineTrx.getTrxId());

        //此处应插入凭证事务
        Map map = new HashMap<>();
        map.put("jeTrxId", reverseWriteOff.getWriteOffId());
        map.put("companyId", iRequest.getCompanyId());
        map.put("jeSourceId", lonContract.getContractId());
        map.put("jeSourceDoc", LON_CONTRACT);
        map.put("reverseJeDate", originWriteOff.getReversedDate());
        map.put("reverseJeTrxId", originWriteOff.getWriteOffId());
        AbstractJeTrxService writeOffJeTrx = commonService.map.get("CSH_WRITE_OFF");
        writeOffJeTrx.process(iRequest, map);


    }

    /**
     * 清除who字段和id
     *
     * @param writeOff
     * @return
     */
    private static HlsCusCshWriteOff cleanStdWhoAndId(HlsCusCshWriteOff writeOff) {
        writeOff.setWriteOffId(null);
        writeOff.setObjectVersionNumber(null);
        writeOff.setCreatedBy(null);
        writeOff.setCreationDate(null);
        writeOff.setLastUpdateDate(null);
        writeOff.setLastUpdatedBy(null);
        writeOff.setLastUpdateLogin(null);
        return writeOff;
    }

}
