package hls.core.csh.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.ConContractMapper;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hand.hap.core.IRequest;

import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.mapper.HlsCusCshWriteOffMapper;
import hls.core.csh.service.CshTransactionCommon;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWriteOffService;
import hls.core.fnd.service.FndCodingRuleValuesService;

@Service
public class CshWriteOffReceiptAdvanceReceiptServiceImpl implements CshTransactionCommon {

    private static final String WriteOffType = "RECEIPT_ADVANCE_RECEIPT";

    @Autowired
    HlsCusCshWriteOffMapper cshWriteOffMapper;
    @Autowired
    CshWriteOffService cshWriteOffService;
    @Autowired
    HlsCusCshTransactionMapper cshTransactionMapper;
    @Autowired
    CshTransactionService cshTransactionService;
    @Autowired
    HlsCusConContractCashflowMapper cashflowMapper;
    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private ConContractMapper conContractMapper;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private JeTrxCommonService jeTrxCommonService;

    Map<String, String> params = new HashMap<String, String>();

    @Override
    public String getWriteOffType() {
        return WriteOffType;
    }

    @Override
    public void process(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        /**
         * 收款核销为预收款类型 1、核销表插入一条记录 2、更新现金事物表 3、插入一条新的现金事物记录
         * 4、更新后续事务Id
         */
        // 1、核销表插入一条记录
        HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);
        cshWriteOffService.insertSelective(iRequest, writeOff);
        // 2、更新现金事物表
        HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
        cshTransactionService.updateByPrimaryKeySelective(iRequest, cshTran);
        // 3、插入一条新的现金事物记录
        HlsCusCshTransaction cshTransaction = getCshTransaction(iRequest, cshWriteOff, "RECEIPT_ADVANCE_RECEIPT");
        cshTransaction.setSourceDocLineId(writeOff.getWriteOffId());
        cshTransactionService.insertSelective(iRequest, cshTransaction);
        // 4、更新后续事务Id
        cshWriteOffMapper.updateSubCshTrxId(writeOff.getWriteOffId(), cshTransaction.getTransactionId(),
                cshTransaction.getTransactionAmount());

        //添加凭证事物
        AbstractJeTrxService JeWriteOffService = jeTrxCommonService.map.get("CSH_WRITE_OFF");
        Map map = new HashMap<>();
        map.put("jeTrxId",writeOff.getWriteOffId());
        map.put("companyId",cshWriteOff.getCompanyId());
        map.put("contractId",writeOff.getContractId());
        map.put("sourceDoc","CON_CONTRACT");
        JeWriteOffService.process(iRequest,map);

        // 添加事件
        Map<String, Object> params = new HashMap<>();

        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(cshWriteOff.getCurrencyCode());
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);

        params.put("amount", cshWriteOff.getWriteOffDueAmount());
        params.put("currency", hlsCurrency.getCurrencySymbol());
        params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_RECEIPT_ADVANCE_RECEIPT);

        sysEventService.createEvent(iRequest, cshWriteOff.getBpId(), "CSH_TRANSACTION", "", params);

    }

    private HlsCusCshWriteOff getReceiptCredit(HlsCusCshWriteOff cshWriteOff) {
        HlsCusCshWriteOff wo = new HlsCusCshWriteOff();

        wo.setWriteOffType(cshWriteOff.getWriteOffType());
        wo.setWriteOffDate(cshWriteOff.getWriteOffDate());
        wo.setCshTransactionId(cshWriteOff.getCshTransactionId());
        wo.setCshWriteOffAmount(cshWriteOff.getWriteOffDueAmount());
        wo.setReversedFlag("N");
        wo.setDescription(cshWriteOff.getDescription());
        wo.setCashflowId(cshWriteOff.getCashflowId());
        wo.setContractId(cshWriteOff.getContractId());
        wo.setTimes(cshWriteOff.getTimes());
        wo.setCfItem(cshWriteOff.getCfItem());
        wo.setCfType(cshWriteOff.getCfType());
        wo.setWriteOffDueAmount(cshWriteOff.getWriteOffDueAmount());
        wo.setWriteOffPrincipal(cshWriteOff.getWriteOffPrincipal());
        wo.setWriteOffInterest(cshWriteOff.getWriteOffInterest());
        wo.setWriteOffDocCategory(DocumentChangeCommon.getTableName(HlsCusConContract.class));
        return wo;
    }

    private HlsCusCshTransaction getCshTransaction(IRequest iRequest, HlsCusCshWriteOff cshWriteOff, String writeOffType) {
        List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper.queryDetailById(cshWriteOff.getCshTransactionId());

        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

        cshTransaction.setTransactionId(null);
        cshTransaction.setTransactionCategory("CSH_TRANSACTION");
        if ("RECEIPT_ADVANCE_RECEIPT".equals(writeOffType)) {
            cshTransaction.setTransactionType("ADVANCE_RECEIPT");
            cshTransaction.setBusinessType("ADVANCE_RECEIPT");
        } else if ("RECEIPT_DEPOSIT".equals(writeOffType)) {

            cshTransaction.setTransactionType("DEPOSIT");
            cshTransaction.setBusinessType("DEPOSIT");
        } else if ("RECEIPT_DEPOSIT_POOL".equals(writeOffType)) {
            cshTransaction.setTransactionType("DEPOSIT_POOL");
            cshTransaction.setBusinessType("DEPOSIT");
        } else if ("REFUND".equals(writeOffType)) {
            cshTransaction.setTransactionType("REFUND");
            cshTransaction.setBusinessType("PAYMENT");
        }
        if (cshWriteOff.getContractId() != null) {
            cshTransaction.setContractId(cshWriteOff.getContractId());
        }
        cshTransaction.setTransactionAmount(cshWriteOff.getWriteOffDueAmount());
        cshTransaction.setReversedFlag("N");
        cshTransaction.setReversedTrxId(null);
        cshTransaction.setReversedDate(null);
        cshTransaction.setPostedFlag("N");
        cshTransaction.setHandlingCharge(null);
        cshTransaction.setDescription(cshWriteOff.getDescription());
        cshTransaction.setWriteOffFlag("NOT");
        cshTransaction.setWriteOffAmount(null);
        cshTransaction.setFullWriteOffDate(null);
        cshTransaction.setSourceDocCategory("CSH_WRITE_OFF");
        cshTransaction.setTransactionNum(codingRuleValuesService.getCodeRuleValue(iRequest,
                cshTransaction.getTransactionCategory(), cshTransaction.getTransactionType(), cshTransaction.getBusinessType(), params));

        return cshTransaction;
    }

    private HlsCusCshTransaction updateCshTrxAfterWriteoff(HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        HlsCusCshTransaction ct = new HlsCusCshTransaction();

        List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper.queryDetailById(cshWriteOff.getCshTransactionId());

        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

        if (cshTransaction.getWriteOffAmount() == null) {
            cshTransaction.setWriteOffAmount(0d);
        }
        if (cshWriteOff.getWriteOffDueAmount() == null) {
            cshWriteOff.setWriteOffDueAmount(0d);
        }

        Double sumAmount = cshTransaction.getWriteOffAmount() + cshWriteOff.getWriteOffDueAmount();

        if (new BigDecimal(sumAmount).compareTo(new BigDecimal(0)) == 0) {
            ct.setWriteOffFlag("NOT");
            ct.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) < 0) {
            ct.setWriteOffFlag("PARTIAL");
            ct.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) == 0) {
            ct.setWriteOffFlag("FULL");
            ct.setWriteOffAmount(sumAmount);
            ct.setFullWriteOffDate(cshWriteOff.getWriteOffDate());
        } else {
            throw new BeyondAmountLimitException();
        }

        ct.setTransactionId(cshWriteOff.getCshTransactionId());

        return ct;
    }

    @Override
    public void reversed(IRequest requestContext, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        // 1、核销表插入一条记录
        if (cshWriteOff.getWriteOffDueAmount() != null) {
            cshWriteOff.setCshWriteOffAmount(0 - cshWriteOff.getWriteOffDueAmount());
        }
        if (cshWriteOff.getWriteOffDueAmount() != null) {
            cshWriteOff.setWriteOffDueAmount(0 - cshWriteOff.getWriteOffDueAmount());
        }
        if (cshWriteOff.getWriteOffPrincipal() != null) {
            cshWriteOff.setWriteOffPrincipal(0 - cshWriteOff.getWriteOffPrincipal());
        }
        if (cshWriteOff.getWriteOffInterest() != null) {
            cshWriteOff.setWriteOffInterest(0 - cshWriteOff.getWriteOffInterest());
        }

        HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);
        writeOff.setReversedFlag("R");
        writeOff.setReversedWriteOffId(cshWriteOff.getWriteOffId());
        writeOff.setReversedDate(cshWriteOff.getReversedDate());
        cshWriteOffService.insertSelective(requestContext, writeOff);
        // 2、更新现金事物表
        HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
        cshTransactionService.updateByPrimaryKeySelective(requestContext, cshTran);
        // 3、更新原核销表数据
        cshWriteOffMapper.updateCshWriteOffbyId(cshWriteOff.getWriteOffId(), "W", writeOff.getWriteOffId(),
                cshWriteOff.getReversedDate(), requestContext.getUserId(), new Date());
        // 4、插入一条现金事物表
        HlsCusCshTransaction cshTransaction = getCshTransaction(requestContext, cshWriteOff, "RECEIPT_ADVANCE_RECEIPT");
        cshTransaction.setTransactionAmount(cshWriteOff.getWriteOffDueAmount());
        cshTransaction.setReversedFlag("R");
        cshTransaction.setReversedTrxId(cshWriteOff.getSubsequentCshTrxId());
        cshTransaction.setReversedDate(cshWriteOff.getReversedDate());
        cshTransaction.setSourceDocLineId(writeOff.getWriteOffId());
        cshTransactionService.insertSelective(requestContext, cshTransaction);
        // 5、更新后续现金事物表
        HlsCusCshTransaction tcx = new HlsCusCshTransaction();
        tcx.setTransactionId(cshWriteOff.getSubsequentCshTrxId());
        tcx.setReversedFlag("W");
        tcx.setReversedTrxId(cshTransaction.getTransactionId());
        tcx.setReversedDate(cshTransaction.getReversedDate());
        cshTransactionMapper.updateCshTrByPrimaryKey(tcx);
        // 6、更新核销插入核销数据的关联事物Id
        cshWriteOffMapper.updateSubCshTrxId(writeOff.getWriteOffId(), cshTransaction.getTransactionId(),
                cshTransaction.getTransactionAmount());
    }


}
