package hls.core.csh.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import hls.core.cont.dto.*;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.cont.mapper.HlsCusConContractTenantMapper;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.HlsCashflowItem;
import hls.core.hls.mapper.HlsCfItemMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hand.hap.core.IRequest;

import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.mapper.HlsCusCshWriteOffMapper;
import hls.core.csh.service.CshTransactionCommon;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWriteOffService;
import hls.core.csh.service.CshWritiOffEventService;

@Service
public class CshWriteOffReceiptCreditServiceImpl implements CshTransactionCommon{
    
	private static final String WriteOffType = "RECEIPT_CREDIT";

	@Autowired
	private JeTrxCommonService commonService;
	@Autowired
	HlsCusCshWriteOffMapper cshWriteOffMapper;
	@Autowired
	CshWriteOffService cshWriteOffService;
	@Autowired
	HlsCusCshTransactionMapper cshTransactionMapper;
	@Autowired
	CshTransactionService cshTransactionService;
	@Autowired
	HlsCusConContractCashflowMapper cashflowMapper;
	@Autowired
	HlsCusConContractMapper conContractMapper;
	@Autowired
	private CshWritiOffEventService cshWritiOffEventService;
	@Autowired
	private HLSCurrencyMapper hlsCurrencyMapper;
	@Autowired
	private SysEventService sysEventService;
	@Autowired
	HlsCfItemMapper hlsCfItemMapper;
	@Autowired
	HlsCusConContractCashflowMapper contractCashflowMapper ;
	@Autowired
	private HlsCusBpMasterMapper hlsCusBpMasterMapper;
	@Autowired
	private HlsCusConContractTenantMapper cusConContractTenantMapper;

	@Override
	public String getWriteOffType() {
		return WriteOffType;
	}

	@Override
	public void process(IRequest iRequest,HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
		/**
		 * 收款核销债权类型 1、核销表插入一条记录 2、更新现金事物表 3、更新现金合同流表
		 */
		// 1、核销表插入一条记录
		HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);

		cshWriteOffService.insertSelective(iRequest,writeOff);

		//插入凭证事物流水
		Map map = new HashMap<>();
		map.put("jeTrxId",writeOff.getWriteOffId());
		map.put("companyId",writeOff.getCompanyId());
		map.put("contractId",writeOff.getContractId());
		map.put("sourceDoc","CON_CONTRACT");
		AbstractJeTrxService writeOffJeTrx = commonService.map.get("CSH_WRITE_OFF");
		writeOffJeTrx.process(iRequest,map);
		//收款核销债权
		// 更新事件
		Map<String, Object> params = new HashMap<>();
		Long cashflowId = cshWriteOff.getCashflowId();
		HlsCusConContractCashflow conContractCashflow = new HlsCusConContractCashflow();
		conContractCashflow.setCashflow_id(cashflowId);
		conContractCashflow = contractCashflowMapper.selectByPrimaryKey(conContractCashflow);

		Double amount = conContractCashflow.getReceived_amount();
		Long contract_id = conContractCashflow.getContract_id();
		HlsCusConContract conContract = new HlsCusConContract();
		conContract.setContractId(contract_id);

		conContract = conContractMapper.selectByPrimaryKey(conContract);

		HlsCusConContractTenant hlsCusConContractTenant = new HlsCusConContractTenant();
		hlsCusConContractTenant.setContractId(contract_id);
		List<HlsCusConContractTenant> hlsCusConContractTenants = cusConContractTenantMapper.select(hlsCusConContractTenant);

		String currency = conContract.getCurrency();
		HLSCurrency hlsCurrency = new HLSCurrency();
		hlsCurrency.setCurrencyCode(currency);
		hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);

		HlsCashflowItem hlsCashflowItem = new HlsCashflowItem();
		hlsCashflowItem.setCf_type(conContractCashflow.getCf_type().toString());
		hlsCashflowItem.setCf_item(conContractCashflow.getCf_item().toString());
//		hlsCashflowItem = hlsCfItemMapper.selectOne(hlsCashflowItem);
		hlsCashflowItem = hlsCfItemMapper.selectByPrimaryKey(hlsCashflowItem);
		//FIXME bpName参数不确定
		HlsCusBpMaster hlsCusBpMaster = new HlsCusBpMaster();
		hlsCusBpMaster.setBpId(cshWriteOff.getBpId());
		hlsCusBpMaster = hlsCusBpMasterMapper.selectByPrimaryKey(hlsCusConContractTenants.get(0).getTenantBpId());

		params.put("amount",amount);
		params.put("currency",hlsCurrency.getCurrencySymbol());
		params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_RECEIPT_CREDIT);
		params.put("cfName",hlsCashflowItem.getDescription());
		params.put("bpName", hlsCusBpMaster.getBpName());
		sysEventService.createEvent(iRequest,contract_id,"CON_CONTRACT","",params);

//		cshWritiOffEventService.cshWritiOffEvent(iRequest, "CSH_WRITE_OFF.RECEIPT_CREDIT", writeOff.getWriteOffId());
		// 2、更新现金事物表
		HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
		cshTransactionService.updateByPrimaryKeySelective(iRequest, cshTran);
		// 3、更新现金合同流表
		HlsCusConContractCashflow conCashflow = updateConCashflowAfterWriteOff(cshWriteOff);
		cashflowMapper.updateOne(conCashflow);

		//更新合同
		cshWriteOffService.updateContractReceivedStatus(conCashflow);
	}
	
	private HlsCusCshWriteOff getReceiptCredit(HlsCusCshWriteOff cshWriteOff) {
		HlsCusCshWriteOff wo = new HlsCusCshWriteOff();

		wo.setWriteOffType(cshWriteOff.getWriteOffType());
		wo.setWriteOffDate(cshWriteOff.getWriteOffDate());
		wo.setCshTransactionId(cshWriteOff.getCshTransactionId());
		wo.setCshWriteOffAmount(cshWriteOff.getWriteOffDueAmount());
		wo.setReversedFlag("N");
		wo.setDescription(cshWriteOff.getDescription());
		wo.setCashflowId(cshWriteOff.getCashflowId());
		wo.setContractId(cshWriteOff.getContractId());
		wo.setTimes(cshWriteOff.getTimes());
		wo.setCfItem(cshWriteOff.getCfItem());
		wo.setCfType(cshWriteOff.getCfType());
		wo.setWriteOffDueAmount(cshWriteOff.getWriteOffDueAmount());
		wo.setWriteOffPrincipal(cshWriteOff.getWriteOffPrincipal());
		wo.setWriteOffInterest(cshWriteOff.getWriteOffInterest());
		wo.setCompanyId(cshWriteOff.getCompanyId());
		wo.setWriteOffDocCategory(DocumentChangeCommon.getTableName(HlsCusConContract.class));

		return wo;
	}
	
	private HlsCusCshTransaction updateCshTrxAfterWriteoff(HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
		HlsCusCshTransaction ct = new HlsCusCshTransaction();

		List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper.queryDetailById(cshWriteOff.getCshTransactionId());

		HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

		if (cshTransaction.getWriteOffAmount() == null) {
			cshTransaction.setWriteOffAmount(0d);
		}
		if (cshWriteOff.getWriteOffDueAmount() == null) {
			cshWriteOff.setWriteOffDueAmount(0d);
		}

		Double sumAmount = CalculateUtil.add(cshTransaction.getWriteOffAmount(),cshWriteOff.getWriteOffDueAmount());

		if (new BigDecimal(sumAmount).compareTo(new BigDecimal(0))==0) {
			ct.setWriteOffFlag("NOT");
			ct.setWriteOffAmount(sumAmount);
		} else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) < 0 ) {
			ct.setWriteOffFlag("PARTIAL");
			ct.setWriteOffAmount(sumAmount);
		} else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount()))==0 ) {
			ct.setWriteOffFlag("FULL");
			ct.setWriteOffAmount(sumAmount);
			ct.setFullWriteOffDate(cshWriteOff.getWriteOffDate());
		} else {
			throw new BeyondAmountLimitException();
		}

		ct.setTransactionId(cshWriteOff.getCshTransactionId());

		return ct;
	}
	
	private HlsCusConContractCashflow updateConCashflowAfterWriteOff(HlsCusCshWriteOff cshWriteOff) {
		HlsCusConContractCashflow cf = cashflowMapper.selectConContractCashflow(cshWriteOff.getCashflowId());

		if (cf.getReceived_amount() == null) {
			cf.setReceived_amount(0d);
		}

		if (cf.getReceived_principal() == null) {
			cf.setReceived_principal(cshWriteOff.getWriteOffPrincipal());
		} else {
			cf.setReceived_principal(CalculateUtil.add(cf.getReceived_principal(),cshWriteOff.getWriteOffPrincipal()));
		}

		if (cf.getReceived_interest() == null) {
			cf.setReceived_interest(0d);
		} else {
			cf.setReceived_interest(CalculateUtil.add(cf.getReceived_interest(),cshWriteOff.getWriteOffInterest()));
		}

		Double receivedAmount = CalculateUtil.add(cf.getReceived_amount(),cshWriteOff.getWriteOffDueAmount());
		if (receivedAmount == 0) {
			cf.setReceived_amount(receivedAmount);
			cf.setWrite_off_flag("NOT");
		} else if (receivedAmount < cf.getDue_amount()) {
			cf.setReceived_amount(receivedAmount);
			cf.setWrite_off_flag("PARTIAL");
		} else if (receivedAmount.equals(cf.getDue_amount())) {
			cf.setReceived_amount(receivedAmount);
			cf.setWrite_off_flag("FULL");
			cf.setFull_write_off_date(cshWriteOff.getWriteOffDate());
		}

		cf.setLast_received_date(cshWriteOff.getWriteOffDate());

		return cf;
	}

	@Override
	public void reversed(IRequest requestContext,HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
		// 1、核销表插入一条记录
		if (cshWriteOff.getWriteOffDueAmount() != null) {
			cshWriteOff.setCshWriteOffAmount(0 - cshWriteOff.getWriteOffDueAmount());
		}
		if (cshWriteOff.getWriteOffDueAmount() != null) {
			cshWriteOff.setWriteOffDueAmount(0 - cshWriteOff.getWriteOffDueAmount());
		}
		if (cshWriteOff.getWriteOffPrincipal() != null) {
			cshWriteOff.setWriteOffPrincipal(0 - cshWriteOff.getWriteOffPrincipal());
		}
		if (cshWriteOff.getWriteOffInterest() != null) {
			cshWriteOff.setWriteOffInterest(0 - cshWriteOff.getWriteOffInterest());
		}

		HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);
		writeOff.setReversedFlag("R");
		writeOff.setReversedWriteOffId(cshWriteOff.getWriteOffId());
		writeOff.setReversedDate(cshWriteOff.getReversedDate());
		cshWriteOffService.insertSelective(requestContext,writeOff);
		// 2、更新现金事物表
		HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
		cshTransactionService.updateByPrimaryKeySelective(requestContext, cshTran);
		// 3、更新原核销表数据
		cshWriteOffMapper.updateCshWriteOffbyId(cshWriteOff.getWriteOffId(), "W", writeOff.getWriteOffId(),
				cshWriteOff.getReversedDate(),requestContext.getUserId(),new Date());
		// 4、更新现金流表数据
		HlsCusConContractCashflow conCashflow = updateConCashflowAfterWriteOff(cshWriteOff);
		cashflowMapper.updateOne(conCashflow);

		//更新合同
		cshWriteOffService.updateContractReceivedStatus(conCashflow);

		//插入凭证事物流水表 by fjm 17.9.1
		AbstractJeTrxService writeOffService = commonService.map.get("CSH_WRITE_OFF");
		Map params = new HashMap<>();
		params.put("jeTrxId", writeOff.getWriteOffId());
		params.put("companyId", requestContext.getCompanyId());
		params.put("contractId", cshWriteOff.getContractId());
		params.put("sourceDoc", "CON_CONTRACT");
		params.put("reverseJeDate",cshWriteOff.getReversedDate());
		params.put("reverseJeTrxId",cshWriteOff.getWriteOffId());
		writeOffService.process(requestContext, params);

	}

	
}
