package hls.core.csh.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import hls.core.cont.dto.HlsCusConContract;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hand.hap.core.IRequest;

import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.mapper.HlsCusCshWriteOffMapper;
import hls.core.csh.service.CshTransactionCommon;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWriteOffService;
import hls.core.fnd.service.FndCodingRuleValuesService;

@Service
public class CshWriteOffReceiptDepositPoolServiceImpl implements CshTransactionCommon {

    private static final String WriteOffType = "RECEIPT_DEPOSIT_POOL";

    @Autowired
    HlsCusCshWriteOffMapper cshWriteOffMapper;
    @Autowired
    CshWriteOffService cshWriteOffService;
    @Autowired
    HlsCusCshTransactionMapper cshTransactionMapper;
    @Autowired
    CshTransactionService cshTransactionService;
    @Autowired
    HlsCusConContractCashflowMapper cashflowMapper;
    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private JeTrxCommonService jeTrxCommonService;

    Map<String, String> params = new HashMap<String, String>();

    @Override
    public String getWriteOffType() {
        return WriteOffType;
    }

    @Override
    public void process(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        /**
         * 收款核销为保证金池 1、核销表插入一条记录 2、更新现金事物表 3、现金事务表中,更新或者新增保证金池记录
         * 4、更新后续事务Id
         */
        // 1、核销表插入一条记录
        HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);
        cshWriteOffService.insertSelective(iRequest, writeOff);
        // 2、更新现金事物表
        HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
        cshTransactionService.updateByPrimaryKeySelective(iRequest, cshTran);
        // 3、现金事务表中,更新或者新增保证金池记录
        List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper
                .queryDetailById(cshWriteOff.getCshTransactionId());
        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);
        List<HlsCusCshTransaction> cst = cshTransactionMapper.queryReceDepositPool("DEPOSIT_POOL", "DEPOSIT",
                cshTransaction.getCompanyId(), cshTransaction.getBpId());
        Long jeTrxTransactionId = null;


        if (cst.size() == 0) {
            HlsCusCshTransaction insertCt = getInsertCshTran(iRequest, cshWriteOff);
            cshTransactionService.insertSelective(iRequest, insertCt);
            // 4、更新后续事务Id
            cshWriteOffMapper.updateSubCshTrxId(writeOff.getWriteOffId(), insertCt.getTransactionId(),
                    insertCt.getTransactionAmount());
            jeTrxTransactionId = insertCt.getTransactionId();
        } else {
            HlsCusCshTransaction ct = cst.get(0);
            if (ct.getTransactionAmount() == null) {
                ct.setTransactionAmount(0d);
            }
            if (cshWriteOff.getWriteOffDueAmount() == null) {
                cshWriteOff.setWriteOffDueAmount(0d);
            }
            if (ct.getWriteOffAmount() == null) {
                ct.setWriteOffAmount(0d);
            }

            Double sumAmountValue = ct.getTransactionAmount() + cshWriteOff.getWriteOffDueAmount();

            if (ct.getWriteOffAmount() == null) {
                ct.setWriteOffAmount(0d);
            }

            if (new BigDecimal(sumAmountValue).compareTo(new BigDecimal(0)) == 0) {
                ct.setWriteOffFlag("NOT");
                ct.setTransactionAmount(sumAmountValue);
            } else if (new BigDecimal(ct.getWriteOffAmount()).compareTo(new BigDecimal(sumAmountValue)) < 0) {
                ct.setWriteOffFlag("PARTIAL");
                ct.setTransactionAmount(sumAmountValue);
            } else if (new BigDecimal(ct.getWriteOffAmount()).compareTo(new BigDecimal(sumAmountValue)) == 0) {
                ct.setWriteOffFlag("FULL");
                ct.setTransactionAmount(sumAmountValue);
                ct.setFullWriteOffDate(new Date());
            } else {
                throw new BeyondAmountLimitException();
            }
            cshTransactionMapper.updateCshTrByPrimaryKey(ct);
            // 4、更新后续事务Id
            cshWriteOffMapper.updateSubCshTrxId(writeOff.getWriteOffId(), ct.getTransactionId(), ct.getTransactionAmount());
            jeTrxTransactionId = ct.getTransactionId();
        }

        AbstractJeTrxService transactionService = jeTrxCommonService.map.get("CSH_TRANSACTION");
        Map transactionParams = new HashMap<>();
        transactionParams.put("jeTrxId", jeTrxTransactionId);
        transactionParams.put("companyId", iRequest.getCompanyId());
        transactionParams.put("contractId", cshWriteOff.getContractId());
        transactionParams.put("sourceDoc", "CON_CONTRACT");
        transactionService.process(iRequest, transactionParams);

        AbstractJeTrxService JeWriteOffService = jeTrxCommonService.map.get("CSH_WRITE_OFF");
        Map map = new HashMap<>();
        map.put("jeTrxId",writeOff.getWriteOffId());
        map.put("companyId",iRequest.getCompanyId());
        map.put("contractId",cshWriteOff.getContractId());
        map.put("sourceDoc","CON_CONTRACT");
        JeWriteOffService.process(iRequest,map);



        // 更新事件

        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(cshTransaction.getCurrencyCode());
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);
        Map<String, Object> params = new HashMap<>();
        params.put("amount", writeOff.getWriteOffDueAmount());
        params.put("currency", hlsCurrency.getCurrencySymbol());
        params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_RECEIPT_DEPOSIT_POOL);
        sysEventService.createEvent(iRequest, cshTransactionMapper.selectByPrimaryKey(jeTrxTransactionId).getBpId(), "CSH_TRANSACTION", "", params);

    }

    private HlsCusCshTransaction getInsertCshTran(IRequest requestContext, HlsCusCshWriteOff cshWriteOff) {
        List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper.queryDetailById(cshWriteOff.getCshTransactionId());

        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);
        cshTransaction.setTransactionId(null);
        cshTransaction.setTransactionCategory("CSH_TRANSACTION");
        cshTransaction.setTransactionType("DEPOSIT_POOL");
        cshTransaction.setBusinessType("DEPOSIT");
        cshTransaction.setTransactionAmount(cshWriteOff.getWriteOffDueAmount());
        cshTransaction.setBankAccountId(null);
        cshTransaction.setBankSlipNum(null);
        cshTransaction.setPaymentMethod(null);
        cshTransaction.setBpBankAccountId(null);
        cshTransaction.setBpBankAccountName(null);
        cshTransaction.setBpBankAccountNum(null);
        cshTransaction.setBpBankBranchName(null);
        cshTransaction.setBpBankName(null);
        cshTransaction.setReversedFlag("N");
        cshTransaction.setReversedTrxId(null);
        cshTransaction.setReversedDate(null);
        cshTransaction.setPostedFlag("N");
        cshTransaction.setContractId(null);
        cshTransaction.setHandlingCharge(null);
        cshTransaction.setWriteOffFlag("NOT");
        cshTransaction.setWriteOffAmount(null);
        cshTransaction.setFullWriteOffDate(null);
        cshTransaction.setSourceDocCategory(null);
        cshTransaction.setSourceDocId(null);
        cshTransaction.setSourceDocLineId(null);
        cshTransaction.setTransactionNum(codingRuleValuesService.getCodeRuleValue(requestContext,
                cshTransaction.getTransactionCategory(), cshTransaction.getTransactionType(), cshTransaction.getBusinessType(), params));
        return cshTransaction;
    }

    private HlsCusCshWriteOff getReceiptCredit(HlsCusCshWriteOff cshWriteOff) {
        HlsCusCshWriteOff wo = new HlsCusCshWriteOff();

        wo.setWriteOffType(cshWriteOff.getWriteOffType());
        wo.setWriteOffDate(cshWriteOff.getWriteOffDate());
        wo.setCshTransactionId(cshWriteOff.getCshTransactionId());
        wo.setCshWriteOffAmount(cshWriteOff.getWriteOffDueAmount());
        wo.setReversedFlag("N");
        wo.setDescription(cshWriteOff.getDescription());
        wo.setCashflowId(cshWriteOff.getCashflowId());
        wo.setContractId(cshWriteOff.getContractId());
        wo.setTimes(cshWriteOff.getTimes());
        wo.setCfItem(cshWriteOff.getCfItem());
        wo.setCfType(cshWriteOff.getCfType());
        wo.setWriteOffDueAmount(cshWriteOff.getWriteOffDueAmount());
        wo.setWriteOffPrincipal(cshWriteOff.getWriteOffPrincipal());
        wo.setWriteOffInterest(cshWriteOff.getWriteOffInterest());
        wo.setWriteOffDocCategory(DocumentChangeCommon.getTableName(HlsCusConContract.class));

        return wo;
    }

    private HlsCusCshTransaction updateCshTrxAfterWriteoff(HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        HlsCusCshTransaction ct = new HlsCusCshTransaction();

        List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper.queryDetailById(cshWriteOff.getCshTransactionId());

        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

        if (cshTransaction.getWriteOffAmount() == null) {
            cshTransaction.setWriteOffAmount(0d);
        }
        if (cshWriteOff.getWriteOffDueAmount() == null) {
            cshWriteOff.setWriteOffDueAmount(0d);
        }

        Double sumAmount = cshTransaction.getWriteOffAmount() + cshWriteOff.getWriteOffDueAmount();

        if (new BigDecimal(sumAmount).compareTo(new BigDecimal(0)) == 0) {
            ct.setWriteOffFlag("NOT");
            ct.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) < 0) {
            ct.setWriteOffFlag("PARTIAL");
            ct.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) == 0) {
            ct.setWriteOffFlag("FULL");
            ct.setWriteOffAmount(sumAmount);
            ct.setFullWriteOffDate(cshWriteOff.getWriteOffDate());
        } else {
            throw new BeyondAmountLimitException();
        }

        ct.setTransactionId(cshWriteOff.getCshTransactionId());

        return ct;
    }

    @Override
    public void reversed(IRequest requestContext, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        // 1、核销表插入一条记录
        if (cshWriteOff.getWriteOffDueAmount() != null) {
            cshWriteOff.setCshWriteOffAmount(0 - cshWriteOff.getWriteOffDueAmount());
        }
        if (cshWriteOff.getWriteOffDueAmount() != null) {
            cshWriteOff.setWriteOffDueAmount(0 - cshWriteOff.getWriteOffDueAmount());
        }
        if (cshWriteOff.getWriteOffPrincipal() != null) {
            cshWriteOff.setWriteOffPrincipal(0 - cshWriteOff.getWriteOffPrincipal());
        }
        if (cshWriteOff.getWriteOffInterest() != null) {
            cshWriteOff.setWriteOffInterest(0 - cshWriteOff.getWriteOffInterest());
        }

        HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);
        writeOff.setReversedFlag("R");
        writeOff.setReversedWriteOffId(cshWriteOff.getWriteOffId());
        writeOff.setReversedDate(cshWriteOff.getReversedDate());
        cshWriteOffService.insertSelective(requestContext, writeOff);
        // 2、更新现金事物表
        HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
        cshTransactionService.updateByPrimaryKeySelective(requestContext, cshTran);
        // 3、更新原核销表数据
        cshWriteOffMapper.updateCshWriteOffbyId(cshWriteOff.getWriteOffId(), "W", writeOff.getWriteOffId(),
                cshWriteOff.getReversedDate(), requestContext.getUserId(), new Date());
        // 4、更新保证金池的现金事物数据
        List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper
                .queryDetailById(cshWriteOff.getCshTransactionId());
        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

        List<HlsCusCshTransaction> cst = cshTransactionMapper.queryReceDepositPool("DEPOSIT_POOL", "DEPOSIT",
                cshTransaction.getCompanyId(), cshTransaction.getBpId());
        HlsCusCshTransaction ct = cst.get(0);

        HlsCusCshTransaction ctRequest = new HlsCusCshTransaction();

        if (ct.getTransactionAmount() == null) {
            ct.setTransactionAmount(0d);
        }
        if (cshWriteOff.getWriteOffDueAmount() == null) {
            cshWriteOff.setWriteOffDueAmount(0d);
        }

        Double sumAmount = ct.getWriteOffAmount() + cshWriteOff.getWriteOffDueAmount();

        Double sumAmountValue = ct.getTransactionAmount() + cshWriteOff.getWriteOffDueAmount();

        if (new BigDecimal(sumAmount).compareTo(new BigDecimal(0)) == 0) {
            ctRequest.setWriteOffFlag("NOT");
            ctRequest.setTransactionAmount(sumAmountValue);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) < 0) {
            ctRequest.setWriteOffFlag("PARTIAL");
            ctRequest.setTransactionAmount(sumAmountValue);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) == 0) {
            ctRequest.setWriteOffFlag("FULL");
            ctRequest.setTransactionAmount(sumAmountValue);
            ctRequest.setFullWriteOffDate(new Date());
        } else {
            throw new BeyondAmountLimitException();
        }
        cshTransactionMapper.updateCshTrByPrimaryKey(ctRequest);
        // 5、更新后续事务Id
        cshWriteOffMapper.updateSubCshTrxId(writeOff.getWriteOffId(), writeOff.getSubsequentCshTrxId(),
                (0 - writeOff.getSubseqCshWriteOffAmount()));
    }

}
