package hls.core.csh.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.HlsCashflowItem;
import hls.core.hls.mapper.HlsCfItemMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.hand.hap.core.IRequest;

import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusCshTransactionMapper;
import hls.core.csh.mapper.HlsCusCshWriteOffMapper;
import hls.core.csh.service.CshTransactionCommon;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.CshWriteOffService;
import hls.core.fnd.service.FndCodingRuleValuesService;

@Service
public class CshWriteOffReceiptDepositServiceImpl implements CshTransactionCommon {

    private static final String WriteOffType = "RECEIPT_DEPOSIT";

    @Autowired
    HlsCusCshWriteOffMapper cshWriteOffMapper;
    @Autowired
    HlsCusConContractMapper conContractMapper;
    @Autowired
    CshWriteOffService cshWriteOffService;
    @Autowired
    HlsCusCshTransactionMapper cshTransactionMapper;
    @Autowired
    CshTransactionService cshTransactionService;
    @Autowired
    HlsCusConContractCashflowMapper cashflowMapper;
    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private HlsCfItemMapper hlsCfItemMapper;
    @Autowired
    private HlsCusConContractCashflowMapper contractCashflowMapper;
    @Autowired
    private HlsCusBpMasterMapper hlsCusBpMasterMapper;
    @Autowired
    private JeTrxCommonService jeTrxCommonService;

    Map<String, String> params = new HashMap<String, String>();

    @Override
    public String getWriteOffType() {
        return WriteOffType;
    }

    @Override
    public void process(IRequest iRequest, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        /**
         * 收款核销为保证金 1、核销表插入一条记录 2、更新现金事物表 3、更新现金合同流表 4、插入一条新的现金事物记录
         * 5、更新后续事务Id
         */
        // 1、核销表插入一条记录
        HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);
        cshWriteOffService.insertSelective(iRequest, writeOff);
        // 2、更新现金事物表
        HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
        cshTransactionService.updateByPrimaryKeySelective(iRequest, cshTran);
        // 3、更新现金合同流表
        HlsCusConContractCashflow conCashflow = updateConCashflowAfterWriteOff(cshWriteOff);
        cashflowMapper.updateOne(conCashflow);

        //更新合同
        cshWriteOffService.updateContractReceivedStatus(conCashflow);

        // 4、插入一条新的现金事物记录
        HlsCusCshTransaction cshTransaction = getCshTransaction(iRequest, cshWriteOff, "RECEIPT_DEPOSIT");
        cshTransaction.setSourceDocLineId(writeOff.getWriteOffId());
        cshTransactionService.insertSelective(iRequest, cshTransaction);
        // 5、更新后续事务Id
        cshWriteOffMapper.updateSubCshTrxId(writeOff.getWriteOffId(), cshTransaction.getTransactionId(),
                cshTransaction.getTransactionAmount());

        // 更新事件
        Map<String, Object> params = new HashMap<>();
        Long cashflowId = writeOff.getCashflowId();
        HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
        cashflow.setCashflow_id(cashflowId);
        cashflow = contractCashflowMapper.selectByPrimaryKey(cashflow);

        Double amount = cashflow.getReceived_amount();
        Long contract_id = cashflow.getContract_id();
        HlsCusConContract conContract = new HlsCusConContract();
        conContract.setContractId(contract_id);

        conContract = conContractMapper.selectByPrimaryKey(conContract);
        String currency = conContract.getCurrency();
        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(currency);
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);

        HlsCashflowItem hlsCashflowItem = new HlsCashflowItem();
        hlsCashflowItem.setCf_type(cashflow.getCf_type().toString());
        hlsCashflowItem.setCf_item(cashflow.getCf_item().toString());
        hlsCashflowItem = hlsCfItemMapper.selectOne(hlsCashflowItem);

        //插入凭证事物,17-09-28 by fjm
        AbstractJeTrxService transactionService = jeTrxCommonService.map.get("CSH_TRANSACTION");
        Map transactionParams = new HashMap<>();
        transactionParams.put("jeTrxId", cshTransaction.getTransactionId());
        transactionParams.put("companyId", iRequest.getCompanyId());
        transactionParams.put("contractId", cshWriteOff.getContractId());
        transactionParams.put("sourceDoc", "CON_CONTRACT");
        transactionService.process(iRequest, transactionParams);

        AbstractJeTrxService JeWriteOffService = jeTrxCommonService.map.get("CSH_WRITE_OFF");
        Map map = new HashMap<>();
        map.put("jeTrxId",writeOff.getWriteOffId());
        map.put("companyId",iRequest.getCompanyId());
        map.put("contractId",cshWriteOff.getContractId());
        map.put("sourceDoc","CON_CONTRACT");
        JeWriteOffService.process(iRequest,map);


        //FIXME bpName参数不确定
        HlsCusBpMaster hlsCusBpMaster = new HlsCusBpMaster();
        hlsCusBpMaster.setBpId(cshTransaction.getBpId());
        hlsCusBpMaster = hlsCusBpMasterMapper.selectByPrimaryKey(hlsCusBpMaster);

        params.put("amount", amount);
        params.put("currency", hlsCurrency.getCurrencySymbol());
        params.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_WRITE_OFF_RECEIPT_DEPOSIT);
        params.put("cfName", hlsCashflowItem.getDescription());
        params.put("bpName", hlsCusBpMaster.getBpName());
        sysEventService.createEvent(iRequest, contract_id, "CON_CONTRACT", "", params);
    }

    private HlsCusCshWriteOff getReceiptCredit(HlsCusCshWriteOff cshWriteOff) {
        HlsCusCshWriteOff wo = new HlsCusCshWriteOff();

        wo.setWriteOffType(cshWriteOff.getWriteOffType());
        wo.setWriteOffDate(cshWriteOff.getWriteOffDate());
        wo.setCshTransactionId(cshWriteOff.getCshTransactionId());
        wo.setCshWriteOffAmount(cshWriteOff.getWriteOffDueAmount());
        wo.setReversedFlag("N");
        wo.setDescription(cshWriteOff.getDescription());
        wo.setCashflowId(cshWriteOff.getCashflowId());
        wo.setContractId(cshWriteOff.getContractId());
        wo.setTimes(cshWriteOff.getTimes());
        wo.setCfItem(cshWriteOff.getCfItem());
        wo.setCfType(cshWriteOff.getCfType());
        wo.setWriteOffDueAmount(cshWriteOff.getWriteOffDueAmount());
        wo.setWriteOffPrincipal(cshWriteOff.getWriteOffPrincipal());
        wo.setWriteOffInterest(cshWriteOff.getWriteOffInterest());
        wo.setBpId(cshWriteOff.getBpId());
        wo.setWriteOffDocCategory(DocumentChangeCommon.getTableName(HlsCusConContract.class));

        return wo;
    }

    private HlsCusCshTransaction updateCshTrxAfterWriteoff(HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        HlsCusCshTransaction ct = new HlsCusCshTransaction();

        List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper.queryDetailById(cshWriteOff.getCshTransactionId());

        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

        if (cshTransaction.getWriteOffAmount() == null) {
            cshTransaction.setWriteOffAmount(0d);
        }
        if (cshWriteOff.getWriteOffDueAmount() == null) {
            cshWriteOff.setWriteOffDueAmount(0d);
        }

        Double sumAmount = cshTransaction.getWriteOffAmount() + cshWriteOff.getWriteOffDueAmount();

        if (new BigDecimal(sumAmount).compareTo(new BigDecimal(0)) == 0) {
            ct.setWriteOffFlag("NOT");
            ct.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) < 0) {
            ct.setWriteOffFlag("PARTIAL");
            ct.setWriteOffAmount(sumAmount);
        } else if (new BigDecimal(sumAmount).compareTo(new BigDecimal(cshTransaction.getTransactionAmount())) == 0) {
            ct.setWriteOffFlag("FULL");
            ct.setWriteOffAmount(sumAmount);
            ct.setFullWriteOffDate(cshWriteOff.getWriteOffDate());
        } else {
            throw new BeyondAmountLimitException();
        }

        ct.setTransactionId(cshWriteOff.getCshTransactionId());

        return ct;
    }

    private HlsCusConContractCashflow updateConCashflowAfterWriteOff(HlsCusCshWriteOff cshWriteOff) {
        HlsCusConContractCashflow cf = cashflowMapper.selectConContractCashflow(cshWriteOff.getCashflowId());
        if (cf.getReceived_amount() == null) {
            cf.setReceived_amount(0d);
        }

        if (cf.getReceived_principal() == null) {
            cf.setReceived_principal(cshWriteOff.getWriteOffPrincipal());
        } else {
            cf.setReceived_principal(cf.getReceived_principal() + cshWriteOff.getWriteOffPrincipal());
        }

        if (cf.getReceived_interest() == null) {
            cf.setReceived_interest(0d);
        } else {
            cf.setReceived_interest(cf.getReceived_interest() + cshWriteOff.getWriteOffInterest());
        }

        Double receivedAmount = cf.getReceived_amount() + cshWriteOff.getWriteOffDueAmount();
        if (receivedAmount == 0) {
            cf.setReceived_amount(receivedAmount);
            cf.setWrite_off_flag("NOT");
        } else if (receivedAmount < cf.getDue_amount()) {
            cf.setReceived_amount(receivedAmount);
            cf.setWrite_off_flag("PARTIAL");
        } else if (receivedAmount.equals(cf.getDue_amount())) {
            cf.setReceived_amount(receivedAmount);
            cf.setWrite_off_flag("FULL");
            cf.setFull_write_off_date(cshWriteOff.getWriteOffDate());
        }

        cf.setLast_received_date(cshWriteOff.getWriteOffDate());

        return cf;
    }

    private HlsCusCshTransaction getCshTransaction(IRequest iRequest, HlsCusCshWriteOff cshWriteOff, String writeOffType) {
        List<HlsCusCshTransaction> cshTransactions = cshTransactionMapper.queryDetailById(cshWriteOff.getCshTransactionId());

        HlsCusCshTransaction cshTransaction = cshTransactions.get(0);

        cshTransaction.setTransactionId(null);
        cshTransaction.setTransactionCategory("CSH_TRANSACTION");
        if ("RECEIPT_ADVANCE_RECEIPT".equals(writeOffType)) {
            cshTransaction.setTransactionType("ADVANCE_RECEIPT");
            cshTransaction.setBusinessType("ADVANCE_RECEIPT");
        } else if ("RECEIPT_DEPOSIT".equals(writeOffType)) {

            cshTransaction.setTransactionType("DEPOSIT");
            cshTransaction.setBusinessType("DEPOSIT");
        } else if ("RECEIPT_DEPOSIT_POOL".equals(writeOffType)) {
            cshTransaction.setTransactionType("DEPOSIT_POOL");
            cshTransaction.setBusinessType("DEPOSIT");
        } else if ("REFUND".equals(writeOffType)) {
            cshTransaction.setTransactionType("REFUND");
            cshTransaction.setBusinessType("PAYMENT");
        }
        if (cshWriteOff.getContractId() != null) {
            cshTransaction.setContractId(cshWriteOff.getContractId());
        }
        cshTransaction.setTransactionAmount(cshWriteOff.getWriteOffDueAmount());
        cshTransaction.setReversedFlag("N");
        cshTransaction.setReversedTrxId(null);
        cshTransaction.setReversedDate(null);
        cshTransaction.setPostedFlag("N");
        cshTransaction.setHandlingCharge(null);
        cshTransaction.setDescription(cshWriteOff.getDescription());
        cshTransaction.setWriteOffFlag("NOT");
        cshTransaction.setWriteOffAmount(null);
        cshTransaction.setFullWriteOffDate(null);
        cshTransaction.setSourceDocCategory("CSH_WRITE_OFF");
        cshTransaction.setTransactionNum(codingRuleValuesService.getCodeRuleValue(iRequest,
                cshTransaction.getTransactionCategory(), cshTransaction.getTransactionType(), cshTransaction.getBusinessType(), params));
        return cshTransaction;
    }

    @Override
    public void reversed(IRequest requestContext, HlsCusCshWriteOff cshWriteOff) throws BeyondAmountLimitException {
        // 1、核销表插入一条记录
        if (cshWriteOff.getWriteOffDueAmount() != null) {
            cshWriteOff.setCshWriteOffAmount(0 - cshWriteOff.getWriteOffDueAmount());
        }
        if (cshWriteOff.getWriteOffDueAmount() != null) {
            cshWriteOff.setWriteOffDueAmount(0 - cshWriteOff.getWriteOffDueAmount());
        }
        if (cshWriteOff.getWriteOffPrincipal() != null) {
            cshWriteOff.setWriteOffPrincipal(0 - cshWriteOff.getWriteOffPrincipal());
        }
        if (cshWriteOff.getWriteOffInterest() != null) {
            cshWriteOff.setWriteOffInterest(0 - cshWriteOff.getWriteOffInterest());
        }

        HlsCusCshWriteOff writeOff = getReceiptCredit(cshWriteOff);
        writeOff.setReversedFlag("R");
        writeOff.setReversedWriteOffId(cshWriteOff.getWriteOffId());
        writeOff.setReversedDate(cshWriteOff.getReversedDate());
        cshWriteOffService.insertSelective(requestContext, writeOff);
        // 2、更新现金事物表
        HlsCusCshTransaction cshTran = updateCshTrxAfterWriteoff(cshWriteOff);
        cshTransactionService.updateByPrimaryKeySelective(requestContext, cshTran);
        // 3、更新原核销表数据
        cshWriteOffMapper.updateCshWriteOffbyId(cshWriteOff.getWriteOffId(), "W", writeOff.getWriteOffId(),
                cshWriteOff.getReversedDate(), requestContext.getUserId(), new Date());
        // 4、更新现金流表数据
        HlsCusConContractCashflow conCashflow = updateConCashflowAfterWriteOff(cshWriteOff);
        cashflowMapper.updateOne(conCashflow);
        // 5、插入一条现金事物表
        HlsCusCshTransaction cshTransaction = getCshTransaction(requestContext, cshWriteOff, "RECEIPT_DEPOSIT");
        cshTransaction.setTransactionAmount(cshWriteOff.getWriteOffDueAmount());
        cshTransaction.setReversedFlag("R");
        cshTransaction.setReversedTrxId(cshWriteOff.getSubsequentCshTrxId());
        cshTransaction.setReversedDate(cshWriteOff.getReversedDate());
        cshTransaction.setSourceDocLineId(writeOff.getWriteOffId());
        cshTransactionService.insertSelective(requestContext, cshTransaction);
        // 6、更新后续现金事物表
        HlsCusCshTransaction tcx = new HlsCusCshTransaction();
        tcx.setTransactionId(cshWriteOff.getSubsequentCshTrxId());
        tcx.setReversedFlag("W");
        tcx.setReversedTrxId(cshTransaction.getTransactionId());
        tcx.setReversedDate(cshTransaction.getReversedDate());
        cshTransactionMapper.updateCshTrByPrimaryKey(tcx);
        // 7、更新核销插入核销数据的关联事物Id
        cshWriteOffMapper.updateSubCshTrxId(writeOff.getWriteOffId(), cshTransaction.getTransactionId(),
                cshTransaction.getTransactionAmount());

    }


}
