package hls.core.csh.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.AppContextInitListener;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import de.odysseus.el.tree.impl.Parser.ParseException;
import hls.core.cont.dto.ConAging;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.csh.dto.*;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.exception.ChangeLimitException;
import hls.core.csh.exception.WriteOffTypeNullException;
import hls.core.csh.mapper.*;
import hls.core.csh.service.*;
import hls.core.fct.dto.HlsCusFctContractWithdrawCf;
import hls.core.fct.mapper.HlsCusFctContractWithdrawCfMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.HlsCusHlsCreditLineTrx;
import hls.core.hls.service.IHlsCreditLineTrxService;
import hls.core.lon.dto.HlsCusLonContract;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.mapper.HlsCusLonContractMapper;
import hls.core.lon.mapper.HlsCusLonContractRepaymentMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpSession;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = {Exception.class, RuntimeException.class})
public class CshWriteOffServiceImpl extends BaseServiceImpl<HlsCusCshWriteOff>
        implements AppContextInitListener, CshWriteOffService {

    public static final String WRITE_OFF_FLAG_FULL = "FULL";//完全核销标志
    public static final String WRITE_OFF_FLAG_NOT = "NOT";//未核销标志
    public static final String WRITE_OFF_FLAG_PARTIAL = "PARTIAL";//部分核销标志
    public static final String DEDUCTION = "DEDUCTION";
    public static final String REVERSE_FLAG_N = "N";
    public static final String POSTED_FLAG_N = "N";
    public static final String CSH_PAYMENT_REQ = "CSH_PAYMENT_REQ";
    public static final String CON_CONTRACT_CASHFLOW = "con_contract_cashflow";
    public static final String WRITE_OFF_TYPE_PAYMENT_DEBT = "PAYMENT_DEBT";
    public static final String WRITE_OFF_TYPE_RECEIPT_CREDIT = "RECEIPT_CREDIT";
    public static final String WRITE_OFF_TYPE_RECEIPT_DEPOSIT = "RECEIPT_DEPOSIT";
    public static final String PAYMENT_FLAG_FULL = "FULL";
    public static final String PAYMENT_FLAG_NOT = "NOT";
    public static final String PAYMENT_FLAG_PARTIAL = "PARTIAL";
    public static final String DEPOSIT = "DEPOSIT";
    public static final String CSH_TRANSACTION = "CSH_TRANSACTION";
    public static final String CSH_WRITE_OFF = "CSH_WRITE_OFF";

    @Autowired
    HlsCusConContractMapper conContractMapper;
    @Autowired
    HlsCusCshWriteOffMapper cshWriteOffMapper;
    @Autowired
    HlsCusCshTransactionMapper cshTransactionMapper;
    @Autowired
    CshTransactionService cshTransactionService;
    @Autowired
    HlsCusConContractCashflowMapper cashflowMapper;
    @Autowired
    HlsCusCshPaymentReqLnMapper cshPaymentReqLnMapper;
    @Autowired
    private ICshPaymentReqLnService cshPaymentReqLnService;
    @Autowired
    HlsCusCshBankAccountBalanceMapper bankAccountBalanceMapper;
    @Autowired
    CshBankAccountBalanceService cshBankAccountBalanceService;
    @Autowired
    private FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private HlsCusLonContractMapper lonContractMapper;
    @Autowired
    private IHlsCreditLineTrxService hlsCreditLineTrxService;
    @Autowired
    private JeTrxCommonService jeTrxCommonService;
    @Autowired
    private HlsCusCshPaymentReqDtMapper cshPaymentReqDtMapper;
    @Autowired(required = false)
    private IPaymentWriteOffReversedService iPaymentWriteOffReversedService;
    @Autowired
    private HlsCusLonContractRepaymentMapper repaymentMapper;
    @Autowired
    private HlsCusFctContractWithdrawCfMapper hlsCusFctContractWithdrawCfMapper;
    @Autowired
    private IConContractCashflowService contractCashflowService;


    Calendar calendar = Calendar.getInstance();

    private int currentYear = calendar.get(Calendar.YEAR);

    private int currentMonth = calendar.get(Calendar.MONTH) + 1;

    private Map<String, CshTransactionCommon> writeOffRegistion = new HashMap<>();

    public static Long company_id = 0L;
    public static Map<Long, Double> amountMap;

    @Override
    public boolean querySize(HlsCusCshWriteOff cshWriteOff, HttpSession session) {
        HlsCusCshBankAccountBalance accountBalance = new HlsCusCshBankAccountBalance();
        accountBalance.setBankAccountId(cshWriteOff.getBankAccountId());
        List<HlsCusCshBankAccountBalance> list = bankAccountBalanceMapper.selectCshBankAccountBalanceById(accountBalance);
        accountBalance = list.get(0);
        if (accountBalance.getBalance().compareTo(cshWriteOff.getCshWriteOffAmount()) < 0) {
            return false;
        }
        return true;
    }

    @Override
    public void updateWriteOff(IRequest iRequest, List<HlsCusCshWriteOff> cshWriteOffs, HttpSession session)
            throws BeyondAmountLimitException, IllegalArgumentException, WriteOffTypeNullException, ChangeLimitException {
        company_id = (Long) session.getAttribute("companyId");
        amountMap = new HashMap<>();
        if (cshWriteOffs != null) {
            for (HlsCusCshWriteOff cshWriteOff : cshWriteOffs) {
                String writeOffType = cshWriteOff.getWriteOffType();
                CshTransactionCommon trans = writeOffRegistion.get(writeOffType);
                if (trans == null) {
                    throw new WriteOffTypeNullException();
                } else {
                    cshWriteOff.setCompanyId(company_id);
                    //根据核销类型走不同的方法
                    trans.process(iRequest, cshWriteOff);
                }
            }
            try {
                String writeOffType = cshWriteOffs.stream().findFirst().orElse(new HlsCusCshWriteOff()).getWriteOffType();
                if ("PAYMENT_DEBT".equalsIgnoreCase(writeOffType)) {
                    saveWriteOffType(iRequest, cshWriteOffs, session);
                } else if ("LON_PAYMENT_DEBT".equalsIgnoreCase(writeOffType)) {
                    //融资还款  更新现金事务表的核销标志  更新账户余额表
                    updateLonPayment(iRequest, cshWriteOffs);
                } else if ("FCT_PAYMENT_DEBT".equalsIgnoreCase(writeOffType)) {
                    //保理  更新现金事务表的核销标志  更新账户余额表
                    updateFctPayment(iRequest, cshWriteOffs);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void updateFctPayment(IRequest iRequest, List<HlsCusCshWriteOff> cshWriteOffs) {
        Double sumWriteOffAmount = CshWriteOffFctPaymentServiceImpl.WRITE_OFF_AMOUNT;//申请总金额
        HlsCusCshTransaction tempCshTransaction = cshWriteOffs.get(0).getCshTransaction();
        if (new BigDecimal(sumWriteOffAmount).compareTo(new BigDecimal(tempCshTransaction.getTransactionAmount())) == 0) {
            //如果现金事务表中的transaction_amount值与核销总金额相等
            tempCshTransaction.setWriteOffAmount(sumWriteOffAmount);
            tempCshTransaction.setWriteOffFlag("FULL");
            tempCshTransaction.setFullWriteOffDate(tempCshTransaction.getTransactionDate());

        } else {
            tempCshTransaction.setWriteOffAmount(sumWriteOffAmount);
            tempCshTransaction.setWriteOffFlag("PARTIAL");
        }
        cshTransactionService.updateByPrimaryKeySelective(iRequest, tempCshTransaction);
        //更新账户余额表
        paymentBankAccountBalance(iRequest, cshWriteOffs.get(0).getCshTransaction());
    }

    /*融资还款  更新现金事务表的核销标志  更新账户余额表*/
    public void updateLonPayment(IRequest iRequest, List<HlsCusCshWriteOff> cshWriteOffs) {
        //更新现金事务表
        Double sumWriteOffAmount = CshWriteOffLonPaymentServiceImpl.WRITE_OFF_AMOUNT;//申请总金额
        HlsCusCshTransaction tempCshTransaction = cshWriteOffs.get(0).getCshTransaction();
        if (new BigDecimal(sumWriteOffAmount).compareTo(new BigDecimal(tempCshTransaction.getTransactionAmount())) == 0) {
            //如果现金事务表中的transaction_amount值与核销总金额相等
            tempCshTransaction.setWriteOffAmount(sumWriteOffAmount);
            tempCshTransaction.setWriteOffFlag("FULL");
            tempCshTransaction.setFullWriteOffDate(tempCshTransaction.getTransactionDate());

        } else {
            tempCshTransaction.setWriteOffAmount(sumWriteOffAmount);
            tempCshTransaction.setWriteOffFlag("PARTIAL");
        }
        cshTransactionService.updateByPrimaryKeySelective(iRequest, tempCshTransaction);
        //更新账户余额表
        paymentBankAccountBalance(iRequest, cshWriteOffs.get(0).getCshTransaction());
    }

    @Override
    public void updateWriteOff(IRequest iRequest, List<HlsCusCshWriteOff> cshWriteOffs, Long companyId)
            throws BeyondAmountLimitException, IllegalArgumentException, WriteOffTypeNullException, ChangeLimitException {
        company_id = companyId;
        amountMap = new HashMap<>();
        if (cshWriteOffs != null) {
            for (HlsCusCshWriteOff cshWriteOff : cshWriteOffs) {
                String writeOffType = cshWriteOff.getWriteOffType();
                CshTransactionCommon trans = writeOffRegistion.get(writeOffType);
                if (trans == null) {
                    throw new WriteOffTypeNullException();
                } else {
                    cshWriteOff.setCompanyId(company_id);
                    trans.process(iRequest, cshWriteOff);
                }
            }
            try {
                String writeOffType = cshWriteOffs.stream().findFirst().orElse(new HlsCusCshWriteOff()).getWriteOffType();
                if ("PAYMENT_DEBT".equalsIgnoreCase(writeOffType)) {
                    saveWriteOffType(iRequest, cshWriteOffs, null);
                } else if ("LON_PAYMENT_DEBT".equalsIgnoreCase(writeOffType)) {
                    //融资还款  更新现金事务表的核销标志  更新账户余额表
                    updateLonPayment(iRequest, cshWriteOffs);
                } else if ("FCT_PAYMENT_DEBT".equalsIgnoreCase(writeOffType)) {
                    //保理  更新现金事务表的核销标志  更新账户余额表
                    updateFctPayment(iRequest, cshWriteOffs);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 更新现金事物和账户余额
     *
     * @param iRequest
     * @param cshWriteOffs
     * @throws ChangeLimitException
     */
    private void saveWriteOffType(IRequest iRequest, List<HlsCusCshWriteOff> cshWriteOffs, HttpSession session) throws ChangeLimitException, BeyondAmountLimitException {
        Double temp_TransactionAmount = 0D;
        HlsCusCshTransaction temp_cshTransaction = cshWriteOffs.get(0).getCshTransaction();
        Double transactionDueAmount = CshWriteOffPaymentServiceImpl.transactionDueAmount;//申请行上的申请金额总和
        Double amount_paid = CshWriteOffPaymentServiceImpl.amount_paid;//所有申请行上的已支付金额
        for (int i = 0; i < cshWriteOffs.size(); i++) {
            temp_TransactionAmount = temp_TransactionAmount + cshWriteOffs.get(i).getCshWriteOffAmount();//真实的申请金额总和
        }
        temp_cshTransaction.setTransactionAmount(temp_TransactionAmount);
        temp_cshTransaction.setWriteOffAmount(temp_TransactionAmount);
        if (new BigDecimal(temp_TransactionAmount + amount_paid).compareTo(new BigDecimal(transactionDueAmount)) == 0) {
            temp_cshTransaction.setWriteOffFlag("FULL");
        } else {
            temp_cshTransaction.setWriteOffFlag("PARTIAL");
        }
        if (transactionDueAmount.compareTo(0D) != 0) {
            cshTransactionService.updateByPrimaryKeySelective(iRequest, temp_cshTransaction);
        }
        // 更新账户余额表
        paymentBankAccountBalance(iRequest, temp_cshTransaction);

        Map<String, Object> map = new HashMap<>();
        HlsCusCshWriteOff randomWriteOff = cshWriteOffs.stream().findAny().orElseGet(HlsCusCshWriteOff::new);
        Date transactionDate = randomWriteOff.getCshTransaction().getTransactionDate();
        Long paymentReqId = randomWriteOff.getPaymentReqId();
        map.put("transactionDate", transactionDate);
        map.put("paymentReqId", paymentReqId);
        for (HlsCusCshWriteOff item : cshWriteOffs) {
            if (item.getCfItem() != 0) {
                continue;
            }
            Long paymentReqLineId = item.getPaymentReqLineId();//获取还款申请行id
            HlsCusCshPaymentReqDt cshPaymentReqDt = new HlsCusCshPaymentReqDt();
            cshPaymentReqDt.setPaymentReqLnId(paymentReqLineId);
            List<HlsCusCshPaymentReqDt> cshPaymentReqDts = cshPaymentReqDtMapper.select(cshPaymentReqDt);
            if (!cshPaymentReqDts.isEmpty()) {
                this.paymentDeduction(iRequest, cshPaymentReqDts, map);
            }
        }
        // validate(list);
        //validate(CshWriteOffPaymentServiceImpl.list,   session);
    }

    /**
     * 付款抵扣
     *
     * @param request
     * @param cshPaymentReqDts
     * @param map
     */
    private void paymentDeduction(IRequest request, List<HlsCusCshPaymentReqDt> cshPaymentReqDts, Map<String, Object> map) throws BeyondAmountLimitException {
        HlsCusCshTransaction cshTransaction = new HlsCusCshTransaction();
        //cshTransaction.setWriteOffAmount(0D);
        cshTransaction.setTransactionAmount(0D);
        cshTransaction.setWriteOffFlag(WRITE_OFF_FLAG_FULL);
        cshTransaction.setCompanyId(request.getCompanyId());
        cshTransaction.setTransactionCategory(CSH_TRANSACTION);
        cshTransaction.setTransactionType(DEDUCTION);
        cshTransaction.setBusinessType(DEDUCTION);
        Date date = (Date) map.get("transactionDate");
        cshTransaction.setTransactionDate(date);
        Map<String, String> params = new HashMap<>();
        String value = codingRuleValuesService.getCodeRuleValue(request, CSH_TRANSACTION, DEDUCTION, DEDUCTION, params);
        cshTransaction.setTransactionNum(value);
        cshTransaction.setReversedFlag(REVERSE_FLAG_N);
        cshTransaction.setPostedFlag(POSTED_FLAG_N);
        cshTransaction.setSourceDocCategory(CSH_PAYMENT_REQ);
        Long paymentReqId = (Long) map.get("paymentReqId");
        cshTransaction.setSourceDocId(paymentReqId);
        cshTransactionService.insertSelective(request, cshTransaction);
        Long transactionId = cshTransaction.getTransactionId();
        Map<String, Object> param = new HashMap<>();
        param.put("transactionDate", date);
        param.put("paymentReqId", paymentReqId);
        for (HlsCusCshPaymentReqDt cshPaymentReqDt : cshPaymentReqDts) {
            this.doWriteOffDetail(request, transactionId, cshPaymentReqDt, param);
        }
    }

    /**
     * 抵扣核销
     *
     * @param request
     * @param transactionId
     * @param transactionId
     */
    private void doWriteOffDetail(IRequest request, Long transactionId, HlsCusCshPaymentReqDt cshPaymentReqDt, Map<String, Object> param) throws BeyondAmountLimitException {
        Long sourceDocId = cshPaymentReqDt.getSourceDocId();
        String sourceDocCategory = cshPaymentReqDt.getSourceDocCategory();
        Long paymentReqLnId = cshPaymentReqDt.getPaymentReqLnId();
        HlsCusCshPaymentReqLn hlsCusCshPaymentReqLn = cshPaymentReqLnMapper.selectByPrimaryKey(paymentReqLnId);
        Long originCashflowId = hlsCusCshPaymentReqLn.getSource_doc_line_id();
        HlsCusConContractCashflow requestCashflow = cashflowMapper.selectByPrimaryKey(originCashflowId);
        if (!CON_CONTRACT_CASHFLOW.equalsIgnoreCase(sourceDocCategory))
            return;
        HlsCusConContractCashflow conContractCashflow = cashflowMapper.selectByPrimaryKey(sourceDocId);
        Long cfItem = conContractCashflow.getCf_item();
        Long cfType = conContractCashflow.getCf_type();
        if (cfItem == 5 && cfType == 5) {//表示为保证金类型
            HlsCusCshWriteOff payment = getWriteOffObject(request, requestCashflow, cshPaymentReqDt, param, WRITE_OFF_TYPE_PAYMENT_DEBT, transactionId);
            HlsCusCshWriteOff deposit = getWriteOffObject(request, conContractCashflow, cshPaymentReqDt, param, WRITE_OFF_TYPE_RECEIPT_DEPOSIT, transactionId);
            this.createDepositTransaction(request, payment, param, conContractCashflow);
        } else {
            HlsCusCshWriteOff payment = getWriteOffObject(request, requestCashflow, cshPaymentReqDt, param, WRITE_OFF_TYPE_PAYMENT_DEBT, transactionId);
            HlsCusCshWriteOff credit = getWriteOffObject(request, conContractCashflow, cshPaymentReqDt, param, WRITE_OFF_TYPE_RECEIPT_CREDIT, transactionId);
        }
        this.updateDeductionCashflow(request, requestCashflow, cshPaymentReqDt);
        this.updateDeductionCashflow(request, conContractCashflow, cshPaymentReqDt);
        Double amount = hlsCusCshPaymentReqLn.getAmount();
        Double amountPaid = hlsCusCshPaymentReqLn.getAmount_paid();
        Double totalPaid = CalculateUtil.add(amountPaid, cshPaymentReqDt.getDeductAmount());
        hlsCusCshPaymentReqLn.setAmount_paid(totalPaid);
        if (amount.compareTo(totalPaid) == 0) {
            hlsCusCshPaymentReqLn.setPayment_flag(PAYMENT_FLAG_FULL);
            hlsCusCshPaymentReqLn.setPayment_completed_date((Date) param.get("transactionDate"));
        } else if (amount.compareTo(totalPaid) > 0 && totalPaid.compareTo(0D) == 0) {
            hlsCusCshPaymentReqLn.setPayment_flag(PAYMENT_FLAG_NOT);
        } else if (amount.compareTo(totalPaid) > 0 && totalPaid.compareTo(0D) != 0) {
            hlsCusCshPaymentReqLn.setPayment_flag(PAYMENT_FLAG_PARTIAL);
        } else {
            throw new BeyondAmountLimitException("核销金额不应当大于应收金额");
        }
        cshPaymentReqLnService.updateByPrimaryKeySelective(request, hlsCusCshPaymentReqLn);
    }

    /**
     * 创建保证金的现金事物
     *
     * @param request
     * @param cshWriteOff
     * @param map
     * @param conContractCashflow
     */
    private void createDepositTransaction(IRequest request, HlsCusCshWriteOff cshWriteOff, Map<String, Object> map, HlsCusConContractCashflow conContractCashflow) {
        HlsCusCshTransaction cshTransaction = new HlsCusCshTransaction();
        cshTransaction.setTransactionAmount(cshWriteOff.getCshWriteOffAmount());
        cshTransaction.setWriteOffFlag(WRITE_OFF_FLAG_FULL);
        cshTransaction.setCompanyId(request.getCompanyId());
        cshTransaction.setTransactionCategory(CSH_TRANSACTION);
        cshTransaction.setTransactionType(DEPOSIT);
        cshTransaction.setBusinessType(DEPOSIT);
        Date date = (Date) map.get("transactionDate");
        cshTransaction.setTransactionDate(date);
        Map<String, String> params = new HashMap<>();
        String value = codingRuleValuesService.getCodeRuleValue(request, CSH_TRANSACTION, DEPOSIT, DEPOSIT, params);
        cshTransaction.setTransactionNum(value);
        cshTransaction.setReversedFlag(REVERSE_FLAG_N);
        cshTransaction.setPostedFlag(POSTED_FLAG_N);
        cshTransaction.setContractId(conContractCashflow.getContract_id());
        cshTransaction.setSourceDocCategory(CSH_WRITE_OFF);
        cshTransaction.setSourceDocId(cshWriteOff.getWriteOffId());
        cshTransaction.setSourceDocLineId(cshWriteOff.getTransactionId());

        cshTransactionService.insertSelective(request, cshTransaction);
    }

    /**
     * 更新保证金现金流
     *
     * @param request
     * @param conContractCashflow
     * @param cshPaymentReqDt
     */
    private void updateDeductionCashflow(IRequest request, HlsCusConContractCashflow conContractCashflow, HlsCusCshPaymentReqDt cshPaymentReqDt) throws BeyondAmountLimitException {
        Double dueAmount = conContractCashflow.getDue_amount();
        Double principal = conContractCashflow.getPrincipal();
        Double interest = conContractCashflow.getInterest();
        Double receivedAmount = conContractCashflow.getReceived_amount();
        Double receivedInterest = conContractCashflow.getReceived_interest();
        Double receivedPrincipal = conContractCashflow.getReceived_principal();
        Double deductAmount = cshPaymentReqDt.getDeductAmount();
        Double deductPrincipal = cshPaymentReqDt.getDeductPrincipal();
        Double deductInterest = cshPaymentReqDt.getDeductInterest();

        Double totalAmount = CalculateUtil.add(receivedAmount, deductAmount);
        Double totalInterest = CalculateUtil.add(receivedInterest, deductInterest);
        Double totalPrincipal = CalculateUtil.add(receivedPrincipal, deductPrincipal);

        conContractCashflow.setReceived_amount(totalAmount);
        conContractCashflow.setReceived_interest(totalInterest);
        conContractCashflow.setReceived_principal(totalPrincipal);

        if (principal.compareTo(totalPrincipal) == 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_FULL);
        } else if (principal.compareTo(totalPrincipal) > 0 && totalPrincipal.compareTo(0D) != 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_PARTIAL);
        } else if (principal.compareTo(totalPrincipal) > 0 && totalPrincipal.compareTo(0D) == 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_NOT);
        } else {
            throw new BeyondAmountLimitException("已收本金不应当大于应收本金");
        }

        if (interest.compareTo(totalInterest) == 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_FULL);
        } else if (interest.compareTo(totalInterest) > 0 && totalInterest.compareTo(0D) != 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_PARTIAL);
        } else if (interest.compareTo(totalInterest) > 0 && totalInterest.compareTo(0D) == 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_NOT);
        } else {
            throw new BeyondAmountLimitException("已收利息不应当大于应收利息");
        }

        if (dueAmount.compareTo(totalAmount) == 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_FULL);
        } else if (dueAmount.compareTo(totalAmount) > 0 && totalAmount.compareTo(0D) != 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_PARTIAL);
        } else if (dueAmount.compareTo(totalAmount) > 0 && totalAmount.compareTo(0D) == 0) {
            conContractCashflow.setWrite_off_flag(WRITE_OFF_FLAG_NOT);
        } else {
            throw new BeyondAmountLimitException("已收金额不应当大于应收金额");
        }

        contractCashflowService.updateByPrimaryKeySelective(request, conContractCashflow);
    }

    /**
     * 获取核销对象
     *
     * @param request
     * @param conContractCashflow
     * @param cshPaymentReqDt
     * @param param
     * @param writeOffType
     * @param transactionId
     * @return
     */
    private HlsCusCshWriteOff getWriteOffObject(IRequest request, HlsCusConContractCashflow conContractCashflow, HlsCusCshPaymentReqDt cshPaymentReqDt, Map<String, Object> param, String writeOffType, Long transactionId) {
        Date transactionDate = (Date) param.get("transactionDate");
        Long paymentReqId = (Long) param.get("paymentReqId");
        Long cfItem = conContractCashflow.getCf_item();
        Long cfType = conContractCashflow.getCf_type();
        Double deductAmount = cshPaymentReqDt.getDeductAmount();
        Long cashflowId = conContractCashflow.getCashflow_id();
        Long contractId = conContractCashflow.getContract_id();
        Long times = conContractCashflow.getTimes();
        HlsCusCshWriteOff result = new HlsCusCshWriteOff();
        result.setWriteOffType(writeOffType);
        result.setWriteOffDate(transactionDate);
        result.setCshTransactionId(transactionId);
        result.setCshWriteOffAmount(deductAmount);
        result.setReversedFlag(REVERSE_FLAG_N);
        result.setCashflowId(cashflowId);
        result.setContractId(contractId);
        result.setTimes(times);
        result.setCfItem(cfItem);
        result.setCfType(cfType);
        result.setWriteOffDueAmount(deductAmount);
        result.setPaymentReqId(paymentReqId);
        result.setPaymentReqLineId(cshPaymentReqDt.getPaymentReqLnId());
        self().insertSelective(request, result);
        return result;
    }


//
//	public void validate(List<Long> lists, HttpSession session) throws ChangeLimitException {
//		HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
//		cashflow.setCashflow_id(9L);
//		List<HlsCusConContractCashflow> listFlow = cashflowMapper.selectAllConContractCashflow(cashflow);
//		int index = 0;
//		for (int i = 0; i < lists.size(); i++) {
//			for (int j = 0; j < listFlow.size(); j++) {
//				if (listFlow.get(j).getCashflow_id() == lists.get(i)) {
//					index = j;
//					break;
//				}
//			}
//			HlsCusConContractCashflow contractCashflow = listFlow.get(index);
//			if (contractCashflow.getUnpaid_amount() < 0.000001 || contractCashflow.getUnpaid_amount() == null) {
//				contractCashflow.setWrite_off_flag("FULL");
//			} else {
//				contractCashflow.setWrite_off_flag("PARTIAL");
//			}
//			cashflowMapper.updateConContractCashflow(contractCashflow);
//		}
//		finalInterpolation(listFlow,   session);
//	}
//	/**
//	 * 回写合同现金流表
//	 * @param listFlow
//	 * @return
//	 * @throws ChangeLimitException 
//	 */
//	private boolean finalInterpolation(List<HlsCusConContractCashflow> listFlow, HttpSession session) throws ChangeLimitException{
//		//Map<Long,Double> amountMap =  amountMap;
//		for (HlsCusConContractCashflow conContractCashflow : listFlow) {
//			Double amount =amountMap.get(conContractCashflow.getCashflow_id());
//			if(amount==null){
//				continue;
//			}
//			
//			administer(session, conContractCashflow, amount,0);
//			//cashflowMapper.updateByPrimaryKey(conContractCashflow);
//		}
//		return true;
//	}

    /**
     * 执行修改
     *
     * @param session
     * @param conContractCashflow
     * @param amount
     * @param identifying
     * @throws ChangeLimitException
     */
    private void administer(HttpSession session, HlsCusConContractCashflow conContractCashflow, Double amount, int identifying) throws ChangeLimitException {
        Double receivedAmount = conContractCashflow.getReceived_amount();
        Double dueAmount = conContractCashflow.getDue_amount();

        conContractCashflow.setLast_received_date(new Date());//最后核销日
        if (identifying == 0) {
            receivedAmount = receivedAmount + amount;
            if (receivedAmount == dueAmount) {
                conContractCashflow.setFull_write_off_date(new Date());//完全核销日
            }
        } else {
            receivedAmount = receivedAmount - amount;

        }
        conContractCashflow.setLast_updated_by((Long) session.getAttribute("userId"));
        conContractCashflow.setLast_update_date(new Date());
        conContractCashflow.setReceived_amount(receivedAmount);
        cashflowMapper.updateOneConContractCashflow(conContractCashflow);
    }

/*	@Override
    public void updateSubCshTrxId(@Param("writeOffId") Long writeOffId, @Param("transactionId") Long transactionId, @Param("transactionAmount") Double transactionAmount) {
		cshWriteOffMapper.updateSubCshTrxId(writeOffId,transactionId,transactionAmount);
	}*/


    @Override
    public List<HlsCusCshWriteOff> queryDetail(IRequest requestContext, HlsCusCshWriteOff cshWriteOff, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);

        return cshWriteOffMapper.queryDetail(cshWriteOff);
    }

    @Override
    public List<HlsCusCshWriteOff> queryReversed(IRequest requestContext, HlsCusCshWriteOff cshWriteOff, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);

        return cshWriteOffMapper.queryReversed(cshWriteOff);
    }

    /**
     * 核销反冲 1、插入一条核销反冲记录 2、更改原核销记录核销标志
     *
     * @throws BeyondAmountLimitException
     * @throws WriteOffTypeNullException
     */
    @Override
    public void updateWriteOffReversed(IRequest requestContext, HlsCusCshWriteOff cshWriteOff)
            throws BeyondAmountLimitException, WriteOffTypeNullException {
        if (cshWriteOff != null) {
            CshTransactionCommon trans = writeOffRegistion.get(cshWriteOff.getWriteOffType());
            if (trans == null) {
                throw new WriteOffTypeNullException();
            } else {
                trans.reversed(requestContext, cshWriteOff);
            }
        }

    }

    private void paymentBankAccountBalance(IRequest iRequest, HlsCusCshTransaction cshTransaction) {
        Calendar c = Calendar.getInstance();
        c.setTime(cshTransaction.getTransactionDate());
        int year = calendar.get(Calendar.YEAR);
        int month = calendar.get(Calendar.MONTH) + 1;
        HlsCusCshBankAccountBalance accountBalance = null;

        if (year == currentYear && month == currentMonth) {
            HlsCusCshBankAccountBalance bankAccountBalance = bankAccountBalanceMapper.selectAccountBalance(
                    cshTransaction.getBankAccountId(), Long.valueOf(currentYear), Long.valueOf(currentMonth));

            if (bankAccountBalance == null) {
                bankAccountBalance = new HlsCusCshBankAccountBalance();

                bankAccountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                bankAccountBalance.setPeriodYear(Long.valueOf(year));
                bankAccountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance = bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).isEmpty() ? null : bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).get(0);
                if (cshTransaction.getTransactionAmount() == null) {
                    cshTransaction.setTransactionAmount(0d);
                }
                if (accountBalance == null) {
                    bankAccountBalance.setOpeningBalance(0d);
                    bankAccountBalance.setBalance(0 - cshTransaction.getTransactionAmount());
                } else {
                    bankAccountBalance.setOpeningBalance(accountBalance.getBalance());
                    bankAccountBalance.setBalance(accountBalance.getBalance() - cshTransaction.getTransactionAmount());
                }
                bankAccountBalance.setPeriodIncome(0d);
                bankAccountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());

                cshBankAccountBalanceService.insertSelective(iRequest, bankAccountBalance);

            } else {
                accountBalance = new HlsCusCshBankAccountBalance();
                accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                accountBalance.setPeriodYear(Long.valueOf(year));
                accountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance.setPeriodIncome(0d);
                accountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
                accountBalance.setLastUpdatedBy(iRequest.getUserId());
                accountBalance.setLastUpdateDate(new Date());
                cshBankAccountBalanceService.updateByPrimaryKeySelective(iRequest, accountBalance);
            }

        } else {
            HlsCusCshBankAccountBalance bankAccountBalance = bankAccountBalanceMapper
                    .selectAccountBalance(cshTransaction.getBankAccountId(), Long.valueOf(year), Long.valueOf(month));
            if (bankAccountBalance == null) {
                bankAccountBalance = new HlsCusCshBankAccountBalance();

                bankAccountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                bankAccountBalance.setPeriodYear(Long.valueOf(year));
                bankAccountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance = bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).isEmpty() ? null : bankAccountBalanceMapper.getCloseBankAccountBalance(bankAccountBalance).get(0);
                if (cshTransaction.getTransactionAmount() == null) {
                    cshTransaction.setTransactionAmount(0d);
                }
                if (accountBalance == null) {
                    bankAccountBalance.setOpeningBalance(0d);
                    bankAccountBalance.setBalance(0 - cshTransaction.getTransactionAmount());
                } else {
                    bankAccountBalance.setOpeningBalance(accountBalance.getBalance());
                    bankAccountBalance.setBalance(accountBalance.getBalance() - cshTransaction.getTransactionAmount());
                }
                bankAccountBalance.setPeriodIncome(0d);
                bankAccountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());

                cshBankAccountBalanceService.insertSelective(iRequest, bankAccountBalance);

            } else {
                accountBalance = new HlsCusCshBankAccountBalance();
                accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
                accountBalance.setPeriodYear(Long.valueOf(year));
                accountBalance.setPeriodMonth(Long.valueOf(month));
                accountBalance.setPeriodIncome(cshTransaction.getTransactionAmount());
                accountBalance.setPeriodExpense(0d);
                accountBalance.setLastUpdatedBy(iRequest.getUserId());
                accountBalance.setLastUpdateDate(new Date());
                cshBankAccountBalanceService.updateByPrimaryKeySelective(iRequest, accountBalance);

            }

            accountBalance = new HlsCusCshBankAccountBalance();
            accountBalance.setBankAccountId(cshTransaction.getBankAccountId());
            accountBalance.setPeriodYear(Long.valueOf(year));
            accountBalance.setPeriodMonth(Long.valueOf(month));
            accountBalance.setPeriodIncome(0d);
            accountBalance.setPeriodExpense(cshTransaction.getTransactionAmount());
            accountBalance.setLastUpdatedBy(iRequest.getUserId());
            accountBalance.setLastUpdateDate(new Date());
            cshBankAccountBalanceService.updateByPrimaryKeySelective(iRequest, accountBalance);
        }

    }

    /**
     * 付款反冲 1、插入核销反冲记录 csh_write_off 可能为多条 2、更改csh_transaction原核销记录核销标志、日期
     *
     * @throws ChangeLimitException
     */
    @Override
    public void paymentWriteOffReversed(IRequest iRequest, HlsCusCshWriteOff cshWriteOff, HttpSession session) throws ChangeLimitException {
        // 查询现金事务对应的所有核销记录
        HlsCusCshWriteOff cshw = new HlsCusCshWriteOff();
        if (iPaymentWriteOffReversedService != null) {
            iPaymentWriteOffReversedService.beforeReverse(iRequest, cshWriteOff, session);
        }
        // cshw.setWriteOffType("PAYMENT_DEBT"); 由于有多种类型的核销事务  去掉这个限制 只使用id查询
        cshw.setCshTransactionId(cshWriteOff.getCshTransactionId());
        //List<HlsCusCshWriteOff> list = cshWriteOffMapper.select(cshw);
        List<HlsCusCshWriteOff> list = cshWriteOffMapper.selectAllCshWriteOff(cshw);
        List<HlsCusCshWriteOff> clist = list;
        HlsCusCshTransaction sss = new HlsCusCshTransaction();
        sss.setTransactionId(cshWriteOff.getCshTransactionId());
        // 更新原CshTransaction
//		HlsCusCshTransaction cshTransaction1 = cshTransactionMapper.selectByPrimaryKey(sss);
        HlsCusCshTransaction cshTransaction1 = cshTransactionMapper.selectCshTransactionById(sss).get(0);
        cshTransaction1.setReversedFlag("W");
        cshTransaction1.setReversedDate(cshWriteOff.getReversedDate());
        if (cshTransaction1.getWriteOffFlag().equals("PARTIAL")) {
            cshTransaction1.setWriteOffFlag("NOT");
        } else if (cshTransaction1.getWriteOffFlag().equals("FULL")) {
            cshTransaction1.setWriteOffFlag("NOT");
        }
        cshTransactionService.updateByPrimaryKeySelective(iRequest, cshTransaction1);
        //插入新的CshTransaction
        Long temp_tid = cshTransaction1.getTransactionId();
        if (cshWriteOff.getRemarks() != null) {
            cshTransaction1.setDescription(cshWriteOff.getRemarks());
        }
        cshTransaction1.setReversedFlag("R");
        cshTransaction1.setReversedTrxId(cshTransaction1.getTransactionId());
        cshTransaction1.setReversedDate(cshWriteOff.getReversedDate());
        cshTransaction1.setWriteOffAmount(CalculateUtil.mul(cshTransaction1.getWriteOffAmount(), -1D));

        cshTransaction1.setTransactionAmount(CalculateUtil.mul(cshTransaction1.getTransactionAmount(), -1D));
        cshTransactionService.insertSelective(iRequest, cshTransaction1);
        // 更新账户余额表
        if (cshTransaction1.getTransactionAmount() != null && cshTransaction1.getTransactionAmount().compareTo(0D) != 0)
            paymentBankAccountBalance(iRequest, cshTransaction1);
        //原CshTransaction记录新CshTransaction的id
        cshTransaction1.setReversedTrxId(cshTransaction1.getTransactionId());
        cshTransaction1.setTransactionId(temp_tid);
        cshTransactionMapper.updateReversedTrxId(cshTransaction1);
        // 更新原先的CshTransaction
        // cshTransaction1.setTransactionId(temp_tid);
        // cshTransaction1.setReversedFlag("W");
        /*
         * cshTransaction1.setReversedTrxId(cshTransaction1.getTransactionId());
		 */
        // cshTransaction1.setReversedDate(null);
        // cshTransaction1.setWriteOffAmount(cshTransaction1.getWriteOffAmount()*(-1));
        // cshTransaction1.setTransactionAmount(cshTransaction1.getTransactionAmount()*(-1));
        // cshTransactionMapper.updateByPrimaryKeySelective(cshTransaction1);
        //插入凭证事物流水表 by fjm 17.9.1
        AbstractJeTrxService transactionJeTrxService = jeTrxCommonService.map.get("CSH_TRANSACTION");
        AbstractJeTrxService writeOffJetrxService = jeTrxCommonService.map.get("CSH_WRITE_OFF");

        Map transactionParams = new HashMap<>();
        transactionParams.put("jeTrxId", temp_tid);
        transactionParams.put("companyId", iRequest.getCompanyId());
        transactionParams.put("contractId", cshWriteOff.getContractId());
        String writeOffType = list.stream().findFirst().orElse(new HlsCusCshWriteOff()).getWriteOffType();
        if ("PAYMENT_DEBT".equals(writeOffType)) {
            transactionParams.put("sourceDoc", "CON_CONTRACT");
        } else if ("LON_PAYMENT_DEBT".equals(writeOffType)) {
            transactionParams.put("sourceDoc", "LON_CONTRACT");
        } else if ("FCT_PAYMENT_DEBT".equals(writeOffType)) {
            transactionParams.put("sourceDoc", "FCT_CONTRACT");
        }
        transactionParams.put("reverseJeDate", cshWriteOff.getReversedDate());
        transactionParams.put("reverseJeTrxId", cshTransaction1.getTransactionId());
        transactionJeTrxService.process(iRequest, transactionParams);
        String desc = "";
        for (HlsCusCshWriteOff c : list) {
            String writeOffTypeInside = c.getWriteOffType();
            if ("RECEIPT_CREDIT".equalsIgnoreCase(writeOffTypeInside) || "RECEIPT_DEPOSIT".equalsIgnoreCase(writeOffTypeInside)){
                CshTransactionCommon cshTransactionCommon = writeOffRegistion.get(writeOffTypeInside);
                try {
                    cshTransactionCommon.reversed(iRequest,c);
                } catch (BeyondAmountLimitException e) {
                    e.printStackTrace();
                }
                continue;
            }
            Map writeOffMap = new HashMap<>();
            Long temp_wfid = c.getWriteOffId();
            desc = c.getDescription();
            // 插入反冲核销记录，不同的是核销金额为负数
            c.setReversedFlag("R");
            c.setReversedWriteOffId(c.getWriteOffId());
            c.setReversedDate(cshWriteOff.getReversedDate());
            c.setDescription(cshWriteOff.getRemarks());
            c.setWriteOffDueAmount(CalculateUtil.mul(c.getWriteOffDueAmount(), -1D));
            c.setCshWriteOffAmount(CalculateUtil.mul(c.getCshWriteOffAmount(), -1D));
            self().insertSelective(iRequest, c);

            writeOffMap.put("jeTrxId", c.getWriteOffId());
            writeOffMap.put("companyId", iRequest.getCompanyId());
            writeOffMap.put("contractId", cshWriteOff.getContractId());
            if ("PAYMENT_DEBT".equals(c.getWriteOffType()) || "RECEIPT_CREDIT".equalsIgnoreCase(c.getWriteOffType()) || "RECEIPT_DEPOSIT".equalsIgnoreCase(c.getWriteOffType())) {
                writeOffMap.put("sourceDoc", "CON_CONTRACT");
            } else if ("LON_PAYMENT_DEBT".equals(c.getWriteOffType())) {
                writeOffMap.put("sourceDoc", "LON_CONTRACT");
            } else if ("FCT_PAYMENT_DEBT".equals(c.getWriteOffType())) {
                writeOffMap.put("sourceDoc", "FCT_CONTRACT");
            }
            writeOffMap.put("reverseJeDate", cshWriteOff.getReversedDate());
            writeOffMap.put("reverseJeTrxId", temp_wfid);

            // 修改原先的核销记录
            HlsCusCshWriteOff cshWriteOffOrigin = new HlsCusCshWriteOff();
            cshWriteOffOrigin.setReversedWriteOffId(c.getWriteOffId());
            cshWriteOffOrigin.setWriteOffId(temp_wfid);
            cshWriteOffOrigin.setDescription(desc);
            cshWriteOffOrigin.setReversedFlag("W");
            cshWriteOffOrigin.setWriteOffDueAmount(c.getWriteOffDueAmount() * (-1));
            cshWriteOffOrigin.setCshWriteOffAmount(c.getCshWriteOffAmount() * (-1));
            cshWriteOffOrigin.setReversedDate(cshWriteOff.getReversedDate());
            self().updateByPrimaryKeySelective(iRequest, cshWriteOffOrigin);

            writeOffJetrxService.process(iRequest, writeOffMap);

            // 更新付款申请行的已支付金额
            HlsCusCshPaymentReqLn cshPaymentReqLn = (HlsCusCshPaymentReqLn) cshPaymentReqLnMapper
                    .selectCshPaymentReqLnDetailByLnID(Arrays.asList(c.getPaymentReqLineId().toString())).get(0);

            if (CalculateUtil.add(c.getCshWriteOffAmount(), cshPaymentReqLn.getAmount_paid()).compareTo(0D) != 0) {
                // 回写申请行表已支付标志
                cshPaymentReqLn.setPayment_flag("PARTIAL");
                cshPaymentReqLnMapper.updatePaymentFlag(cshPaymentReqLn);
            } else {
                cshPaymentReqLn.setPayment_flag("NOT");
                cshPaymentReqLnMapper.updatePaymentFlag(cshPaymentReqLn);
            }

            // 更新行表已经支付标志
            cshPaymentReqLn.setAmount_paid(CalculateUtil.add(cshPaymentReqLn.getAmount_paid(), c.getCshWriteOffAmount()));
            cshPaymentReqLnMapper.updateAmountPaid(cshPaymentReqLn);

        }
        //如果是付款 更新现金流表上的已收金额和核销标志
        if ("PAYMENT_DEBT".equals(writeOffType)) {
            finalInterpolation(clist, session);
        } else if ("LON_PAYMENT_DEBT".equals(writeOffType)) {
            //如果是融资还款  更新还款表上的已收金额和核销标志
            changeLonRepayment(iRequest, clist);
        } else if ("FCT_PAYMENT_DEBT".equalsIgnoreCase(writeOffType)) {
            //保理    更新账户余额表
            changeFctrRePayment(iRequest, clist);
        }



        if (iPaymentWriteOffReversedService != null) {
            iPaymentWriteOffReversedService.afterReverse(iRequest, cshWriteOff, session);
        }
    }

    public void changeFctrRePayment(IRequest iRequest, List<HlsCusCshWriteOff> clist) {
        for (int i = 0; i < clist.size(); i++) {
            HlsCusCshWriteOff writeOff = clist.get(i);
            double reverseAmount = writeOff.getWriteOffDueAmount();//反冲金额（负数）
            Long withdrawCfId = writeOff.getWithdrawCfId();//获取还款id
            HlsCusFctContractWithdrawCf cf = new HlsCusFctContractWithdrawCf();
            cf.setWithdrawCfId(withdrawCfId);
            cf = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(cf).get(0);

            double receivedAmount = 0D;
            if (cf.getWriteOffAmount() != null) {
                receivedAmount = cf.getWriteOffAmount();
            }
            receivedAmount = CalculateUtil.add(receivedAmount, reverseAmount);//新还款金额
            if (new BigDecimal(receivedAmount).compareTo(new BigDecimal(0)) == 0) {
                cf.setWriteOffFlag("NOT");
            } else if (new BigDecimal(receivedAmount).compareTo(new BigDecimal(0)) > 0) {
                cf.setWriteOffFlag("PARTIAL");
            }
            cf.setWriteOffAmount(receivedAmount);
            hlsCusFctContractWithdrawCfMapper.updateByPrimaryKeySelective(cf);
        }
    }

    /**
     * 付款反冲，融资还款回写还款计划表
     **/
    public void changeLonRepayment(IRequest iRequest, List<HlsCusCshWriteOff> clist) {
        for (int i = 0; i < clist.size(); i++) {
            HlsCusCshWriteOff writeOff = clist.get(i);
            double reverseAmount = writeOff.getWriteOffDueAmount();//反冲金额（负数）
            Long repaymentId = writeOff.getRepaymentId();//获取还款id
            HlsCusLonContractRepayment repayment = new HlsCusLonContractRepayment();
            repayment.setRepaymentId(repaymentId);
            repayment = repaymentMapper.selectByPrimaryKey(repayment);
            Long contractId = repayment.getContractId();//融资合同id
            double receivedAmount = 0D;
            if (repayment.getWriteOffAmount() != null) {
                receivedAmount = repayment.getWriteOffAmount();//还款计划上的还款金额
            }
            receivedAmount = CalculateUtil.add(receivedAmount, reverseAmount);//新还款金额
            if (new BigDecimal(receivedAmount).compareTo(new BigDecimal(0)) == 0) {
                repayment.setWriteOffFlag("NOT");
            } else if (new BigDecimal(receivedAmount).compareTo(new BigDecimal(0)) > 0) {
                repayment.setWriteOffFlag("PARTIAL");
            }
            repayment.setWriteOffAmount(receivedAmount);
            repaymentMapper.updateByPrimaryKeySelective(repayment);
            HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(contractId);
            //插入额度事物表
            HlsCusHlsCreditLineTrx hlsCreditLineTrx = new HlsCusHlsCreditLineTrx();
            hlsCreditLineTrx.setCompanyId(iRequest.getCompanyId());
            hlsCreditLineTrx.setTrxCode("LON_CONTRACT_REPAYMENT");
            hlsCreditLineTrx.setTrxAmount(reverseAmount);
            hlsCreditLineTrx.setSourceDocumentId(repaymentId);
            hlsCreditLineTrx.setSourceDocumentCategory(DocumentChangeCommon.getTableName(HlsCusLonContractRepayment.class));
            hlsCreditLineTrx.setCreditLineId(lonContract.getCreditLineId());
            hlsCreditLineTrx.setTrxDate(repayment.getDueDate());
            hlsCreditLineTrx = hlsCreditLineTrxService.insertSelective(iRequest, hlsCreditLineTrx);
            //更新额度主数据上对应的已占用额度
            hlsCreditLineTrxService.calcCreditMethod(iRequest, hlsCreditLineTrx.getTrxId());

        }
    }

    /**
     * 付款反冲，回写现金流表已收的金额
     *
     * @throws ChangeLimitException
     */
    public void finalInterpolation(List<HlsCusCshWriteOff> clist, HttpSession session) throws ChangeLimitException {
        double reverseAmount = 0.00;
        for (int i = 0; i < clist.size(); i++) {
            reverseAmount = reverseAmount + clist.get(i).getWriteOffDueAmount();
        }
        Long cashFlowId = clist.get(0).getCashflowId();
        HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
        cashflow.setCashflow_id(cashFlowId);
        HlsCusConContractCashflow contractCashflow = cashflowMapper.selectByPrimaryKey(cashflow);
        Double receivedAmount = contractCashflow.getReceived_amount();
        contractCashflow.setLast_received_date(new Date());//最后核销日
        receivedAmount = CalculateUtil.add(receivedAmount, reverseAmount);
        if (new BigDecimal(receivedAmount).compareTo(new BigDecimal(0)) == 0) {
            contractCashflow.setWrite_off_flag("NOT");
        } else {
            contractCashflow.setWrite_off_flag("PARTIAL");
        }
        contractCashflow.setLast_updated_by((Long) session.getAttribute("userId"));
        contractCashflow.setLast_update_date(new Date());
        contractCashflow.setReceived_amount(receivedAmount);

        cashflowMapper.updateOneConContractCashflow(contractCashflow);
    }

    @Override
    public List<HlsCusCshWriteOff> cashflowDetail(IRequest requestContext, HlsCusCshWriteOff cshWriteOff, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);

        return cshWriteOffMapper.cashflowDetail(cshWriteOff);
    }

    @Override
    public void contextInitialized(ApplicationContext appCtx) {
        Map<String, CshTransactionCommon> map = appCtx.getBeansOfType(CshTransactionCommon.class);

        map.forEach((k, v) -> {
            writeOffRegistion.put(v.getWriteOffType(), v);
        });

    }

    private String calculationDate(Long fate) {
        //SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");
        Date beginDate = new Date();
        Calendar date = Calendar.getInstance();
        date.setTime(beginDate);
        if (fate == 7) {
            date.set(Calendar.DATE, date.get(Calendar.DATE) - 7);
        }
        if (fate == 30) {
            date.add(Calendar.MONTH, -1);
        }
        if (fate == 90) {
            date.add(Calendar.MONTH, -3);
        }
        String endDate = dft.format(date.getTime());
        return endDate;
    }

    /**
     * 合同首页投放情况查询 -- 金额部分
     *
     * @param cshWriteOff
     * @return
     * @throws java.text.ParseException
     */
    @Override
    public List<HlsCusCshWriteOff> selectAllPutIn(IRequest requestContext, HlsCusCshWriteOff cshWriteOff, int page, int pagesize) throws java.text.ParseException {
        PageHelper.startPage(page, pagesize);
        cshWriteOff.setCompanyId(requestContext.getCompanyId());
        Long fate = cshWriteOff.getQueryTimeSolt();
        if (fate != null) {
            String endDate = calculationDate(fate);
            cshWriteOff.setWriteOffDate(dft.parse(endDate));
        }
        if (fate == 0) {
            cshWriteOff.setWriteOffDate(null);
        }
        if ("null".equalsIgnoreCase(cshWriteOff.getDivision())) {
            cshWriteOff.setDivision(null);
        }
        if ("null".equalsIgnoreCase(cshWriteOff.getDocumentType())) {
            cshWriteOff.setDocumentType(null);
        }
        List<HlsCusCshWriteOff> list = cshWriteOffMapper.selectAllPutIn(cshWriteOff);
        return list;
    }

    SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");

    /**
     * 合同首页投放详情查询
     *
     * @param cshWriteOff
     * @return
     * @throws ParseException
     * @throws java.text.ParseException
     */
    @Override
    public List<HlsCusCshWriteOff> selectAlldetails(IRequest requestContext, HlsCusCshWriteOff cshWriteOff, int page, int
            pagesize) throws ParseException, java.text.ParseException {
        //PageHelper.startPage(page, pagesize);
        cshWriteOff.setCompanyId(requestContext.getCompanyId());

        Long fate = cshWriteOff.getQueryTimeSolt();
        if (fate != null) {
            String endDate = calculationDate(fate);
            try {
                cshWriteOff.setWriteOffDate(dft.parse(endDate));
            } catch (java.text.ParseException e) {
                e.printStackTrace();
            }
            if (fate == 0) {
                cshWriteOff.setWriteOffDate(null);
            }
        }
        if (cshWriteOff.getStrDate() != null) {
            cshWriteOff.setWriteOffDate(dft.parse(cshWriteOff.getStrDate()));
        }
        if ("null".equalsIgnoreCase(cshWriteOff.getDivision())) {
            cshWriteOff.setDivision(null);
        }
        if ("null".equalsIgnoreCase(cshWriteOff.getDocumentType())) {
            cshWriteOff.setDocumentType(null);
        }
        List<HlsCusCshWriteOff> list = cshWriteOffMapper.selectAlldetails(cshWriteOff);
        return list;
    }

    /**
     * 合同首页 -回笼信息-账龄
     */
    @Override
    public ConAging SelectAgingCount(IRequest requestContext) {
        Long companyId = requestContext.getCompanyId();
        return cshWriteOffMapper.SelectAgingCount(companyId);
    }

    @Override
    public List<Map> selectAllWriteOff(Map map, HttpSession session, IRequest iRequest) {
        List<Map> list = cshWriteOffMapper.selectAllWriteOff(map);
        return list;
    }


    /**
     * 全年每月的放款和收款
     *
     * @return
     */
    @Override
    public List<HlsCusCshWriteOff> getEveryMonthPaymentAndReceiptAmount() {
        //PageHelper.startPage(page, pagesize);
        List<HlsCusCshWriteOff> list = cshWriteOffMapper.getEveryMonthPaymentAndReceiptAmount();
        return list;
    }

    @Override
    public List<HlsCusCshWriteOff> fundPaymentLineInfo(Map map) {
        // TODO Auto-generated method stub
        int year = (int) map.get("year");
        int month = (int) map.get("month");
        List<HlsCusCshWriteOff> list = new ArrayList<HlsCusCshWriteOff>();
        Map m = new HashMap();
        final List<String> paymentList = new ArrayList<String>();
        boolean paymentFlag = false;
        if (!"N".equals(map.get("payment1"))) {
            paymentFlag = true;
            paymentList.add((String) map.get("payment1"));
        }
        if (!"N".equals(map.get("payment2"))) {
            paymentFlag = true;
            paymentList.add((String) map.get("payment2"));
        }
        if (!"N".equals(map.get("payment3"))) {
            paymentFlag = true;
            paymentList.add((String) map.get("payment3"));
        }
        m.put("year", year);
        if (map.get("payInfo") == null) {
            m.put("payInfo", "");
        } else {
            m.put("payInfo", map.get("payInfo"));
        }
        if (map.get("bankInfo") == null) {
            m.put("bankInfo", "");
        } else {
            m.put("bankInfo", map.get("bankInfo"));
        }
        if (map.get("contractInfo") == null) {
            m.put("contractInfo", "");
        } else {
            m.put("contractInfo", map.get("contractInfo"));
        }
        m.put("companyId", map.get("companyId"));
        m.put("paymentList", paymentList);
        m.put("paymentFlag", paymentFlag);
        for (int i = 1; i <= 12; i++) {
            m.put("month", i);
            HlsCusCshWriteOff cshWriteOff = new HlsCusCshWriteOff();
            cshWriteOff = cshWriteOffMapper.fundPaymentLineInfo(m).get(0);
            list.add(cshWriteOff);
        }
        return list;
    }

    /**
     * 核销事物-单行-核销明细
     *
     * @param cshWriteOff
     * @return
     */
    @Override
    public List<HlsCusCshWriteOff> selectAllCancelAfterVerificationDetail(HlsCusCshWriteOff cshWriteOff) {
        return cshWriteOffMapper.selectAllCancelAfterVerificationDetail(cshWriteOff);
    }

    @Override
    public List<HlsCusCshWriteOff> selectDetail(HlsCusCshWriteOff cshWriteOff) {
        return cshWriteOffMapper.selectDetail(cshWriteOff);
    }

    @Override
    public List<Map> WriteOffDetail(Map map, IRequest iRequest) {

        return cshWriteOffMapper.writeOffDetail(map);
    }

    @Override
    public List<Map> writeOffDetailByCFL(Map map, IRequest iRequest) {
        return cshWriteOffMapper.writeOffDetailByCFL(map);
    }

    @Override
    public List<HlsCusCshWriteOff> vatDetail(Long cashflowId) {

        return cshWriteOffMapper.vatDetail(cashflowId);
    }

    @Override
    public void updateContractReceivedStatus(HlsCusConContractCashflow conCashflow) {
        if (conCashflow.getCf_item() != null && conCashflow.getCf_item() == 1) {
            HlsCusConContract c = new HlsCusConContract();
            c.setContractId(conCashflow.getContract_id());
            c = conContractMapper.selectByPrimaryKey(c);
            List<HlsCusConContractCashflow> cccList = cashflowMapper.selectCashFlowByContractId(conCashflow.getContract_id());
            boolean allWriteOffFlag = true;
            if (cccList != null && cccList.size() > 0) {
                for (HlsCusConContractCashflow ccc : cccList) {
                    if (!WRITE_OFF_FLAG_FULL.equalsIgnoreCase(ccc.getWrite_off_flag())) {
                        allWriteOffFlag = false;
                    }
                }
            }
            if (allWriteOffFlag) {
                c.setReceivedStatus("FULL");
            } else {
                c.setReceivedStatus("PARTIAL");
            }
            conContractMapper.updateByPrimaryKey(c);
        }
    }

    /*融资还款-核销明细*/
    public List<HlsCusCshWriteOff> queryLonRepayWriteoffDetail(IRequest request, HlsCusCshWriteOff dto, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        List<HlsCusCshWriteOff> list = cshWriteOffMapper.queryLonRepayWriteoffDetail(dto);
        list.stream().map(item -> {
            item.setLastUpdateDate(item.getLAST_UPDATE_DATE());//处理大写last_update_date
            return item;
        }).collect(Collectors.toList());
        return list;
    }


}
