package hls.core.csh.service.impl;

import java.util.Date;
import java.util.List;

import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.service.CshTransactionService;
import hls.core.csh.service.ICshWritiOffEventService;
import hls.core.hls.mapper.HlsUserRelationshipMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.csh.service.CshWriteOffService;
import hls.core.csh.service.CshWritiOffEventService;
import hls.core.hls.dto.HlsDocAuthority;
import hls.core.hls.dto.HlsSystemEvent;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsUserRelationship;
import hls.core.hls.service.HlsSystemEventService;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.hls.service.IHlsDocAuthorityService;
import hls.core.hls.service.IHlsUserRelationshipService;


@Service
@Transactional
public class CshWritiOffEventServiceImpl extends BaseServiceImpl<HlsCusCshWriteOff> implements CshWritiOffEventService {
    @Autowired
    private HlsSystemEventService hlsSystemEventService;
    @Autowired
    private IHlsDocAuthorityService docAuthorityService;
    @Autowired
    private IHlsUserRelationshipService hlsUserRelationshipService;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;
    @Autowired
    private CshWriteOffService cshWriteOffService;
    @Autowired
    private HlsUserRelationshipMapper userRelationshipMapper;
    @Autowired
    private CshTransactionService cshTransactionService;
    @Autowired(required = false)
    private ICshWritiOffEventService iCshWritiOffEventService;

    private static String CSH_WRITE_OFF_RECEIPT_CREDIT = "收款核销债权";//CSH_WRITE_OFF.RECEIPT_CREDIT
    private static String CSH_WRITE_OFF_PAYMENT_DEBT = "付款核销债务";//CSH_WRITE_OFF.PAYMENT_DEBT
    private static String CSH_WRITE_OFF_RECEIPT_ADVANCE_RECEIPT = "收款核销预收款";//CSH_WRITE_OFF.RECEIPT_ADVANCE_RECEIPT
    private static String CSH_WRITE_OFF_ADVANCE_RECEIPT_CREDIT = "预收款核销债权";//CSH_WRITE_OFF.ADVANCE_RECEIPT_CREDIT
    private static String CSH_WRITE_OFF_REFUND = "现金退款";//CSH_WRITE_OFF.REFUND
    private static String CSH_WRITE_OFF_PAYMENT_PREPAYMENT = "付款核销预付款";//CSH_WRITE_OFF.PAYMENT_PREPAYMENT
    private static String CSH_WRITE_OFF_PREPAYMENT_DEBT = "预付款核销债务";//CSH_WRITE_OFF.PREPAYMENT_DEBT
    private static String CSH_WRITE_OFF_RECEIPT_DEPOSIT = "收款核销保证金";//CSH_WRITE_OFF.RECEIPT_DEPOSIT
    private static String CSH_WRITE_OFF_RECEIPT_DEPOSIT_POOL = "收款核销保证金池";//CSH_WRITE_OFF.RECEIPT_DEPOSIT_POOL
    private static String CSH_WRITE_OFF_DEPOSIT_CREDIT = "保证金核销债权";//CSH_WRITE_OFF_DEPOSIT.CREDIT

    private static String documentCategory = "CSH_WRITE_OFF";
    private static String URL = "csh/CSH320/gathering_cancel.view";
    private static String sourceModule = "CSH";
    private static String RelationType = "P2D";
    // private static String AuthorityTypeQuery = "1";
    private static String AuthorityTypeUpdate = "2";
    //private static String documentType="";

    @Override
    public void cshWritiOffEvent(IRequest iRequest, String eventCode, Long writeOffId) {
        if (iCshWritiOffEventService != null) {
            iCshWritiOffEventService.beforeUpdate(iRequest, eventCode, writeOffId);
        }
        HlsCusCshWriteOff cshWriteOff = new HlsCusCshWriteOff();
        cshWriteOff.setWriteOffId(writeOffId);
        HlsCusCshTransaction cshTransaction = new HlsCusCshTransaction();
        cshTransaction.setWriteOffId(writeOffId);
        cshWriteOff = cshWriteOffService.selectByPrimaryKey(iRequest, cshWriteOff);
        List<HlsCusCshTransaction> cshTransactionList = cshTransactionService.queryByCshWriteId(cshTransaction);
        String message = "";
        String noticeTitle = "";
        Long userId = iRequest.getUserId();
        switch (eventCode) {
            case "CSH_WRITE_OFF.RECEIPT_CREDIT":
                message = "收款" + cshTransactionList.get(0).getTransactionNum() + "被核销成债权";
                noticeTitle = CSH_WRITE_OFF_RECEIPT_CREDIT;
                break;
            case "CSH_WRITE_OFF.PAYMENT_DEBT":
                message = "付款" + cshTransactionList.get(0).getTransactionNum() + "被核销成债权";
                noticeTitle = CSH_WRITE_OFF_PAYMENT_DEBT;
                break;
            case "CSH_WRITE_OFF.RECEIPT_ADVANCE_RECEIPT":
                message = "收款" + cshTransactionList.get(0).getTransactionNum() + "被核销成预收款";
                noticeTitle = CSH_WRITE_OFF_RECEIPT_ADVANCE_RECEIPT;
                break;
            case "CSH_WRITE_OFF.ADVANCE_RECEIPT_CREDIT":
                message = "预收款" + cshTransactionList.get(0).getTransactionNum() + "被核销成债权";
                noticeTitle = CSH_WRITE_OFF_ADVANCE_RECEIPT_CREDIT;
                break;
            case "CSH_WRITE_OFF.REFUND":
                message = "收款" + cshTransactionList.get(0).getTransactionNum() + "退款成功";
                noticeTitle = CSH_WRITE_OFF_REFUND;
                break;
            case "CSH_WRITE_OFF.PAYMENT_PREPAYMENT":
                message = "付款" + cshTransactionList.get(0).getTransactionNum() + "被核销成预付款";
                noticeTitle = CSH_WRITE_OFF_PAYMENT_PREPAYMENT;
                break;
            case "CSH_WRITE_OFF.PREPAYMENT_DEBT":
                message = "预付款" + cshTransactionList.get(0).getTransactionNum() + "被核销成债权";
                noticeTitle = CSH_WRITE_OFF_PREPAYMENT_DEBT;
                break;
            case "CSH_WRITE_OFF.RECEIPT_DEPOSIT":
                message = "收款" + cshTransactionList.get(0).getTransactionNum() + "被核销成保证金";
                noticeTitle = CSH_WRITE_OFF_RECEIPT_DEPOSIT;
                break;
            case "CSH_WRITE_OFF.RECEIPT_DEPOSIT_POOL":
                message = "收款" + cshTransactionList.get(0).getTransactionNum() + "被核销成保证金池";
                noticeTitle = CSH_WRITE_OFF_RECEIPT_DEPOSIT_POOL;
                break;
            case "CSH_WRITE_OFF.DEPOSIT_CREDIT":
                message = "保证金" + cshTransactionList.get(0).getTransactionNum() + "被核销成债权";
                noticeTitle = CSH_WRITE_OFF_DEPOSIT_CREDIT;
                break;
            default:
                break;
        }
        HlsSystemEvent event = new HlsSystemEvent();
        event.setEventCode(eventCode);
        event.setSourceDocumentId(writeOffId);
        event.setEventUserId(userId);
        event.setSourceDocumentCategory(documentCategory);
        event.setLastUpdatedBy(userId);
        event.setLastUpdateDate(new Date());

        // 系统通知
        HlsSystemNotice hlsSystemNotice = new HlsSystemNotice();
        hlsSystemNotice.setNotice_message(message);
        hlsSystemNotice.setNotice_title(noticeTitle);
        hlsSystemNotice.setNotice_datetime(new Date());
        hlsSystemNotice.setNotice_type("NOTICE");
        hlsSystemNotice.setNotice_url(URL);
        hlsSystemNotice.setNotice_level(2L);
        hlsSystemNotice.setSource_module(sourceModule);
        hlsSystemNotice.setSource_event(eventCode);
        hlsSystemNotice.setSource_document_category(documentCategory);
        hlsSystemNotice.setSource_document_id(writeOffId);
        hlsSystemNotice.setSource_user_id(iRequest.getUserId());
//		hlsSystemNotice.setRead_flag("Y");
        hlsSystemNotice.setLastUpdatedBy(userId);
        hlsSystemNotice.setLastUpdateDate(new Date());

        HlsDocAuthority authority = new HlsDocAuthority();
        authority.setUserId(userId);
        authority.setDocumentCategory("documentCategory");// 单据类型
        authority.setDocumentId(writeOffId);
        authority.setAuthorityType(AuthorityTypeUpdate);// 1查询，2，修改
        authority.setSourceModule(sourceModule);
        authority.setSourceEvent(eventCode);
        authority.setSourceDocumentCategory(documentCategory);
        authority.setSourceDocumentId(writeOffId);
        authority.setSourceUserId(iRequest.getUserId());
        authority.setLastUpdatedBy(userId);
        authority.setLastUpdateDate(new Date());

        HlsUserRelationship hlsUserRelationship = new HlsUserRelationship();
        hlsUserRelationship.setUserId(userId);
        hlsUserRelationship.setRelationType(RelationType);
        hlsUserRelationship.setSourceModule(sourceModule);
        hlsUserRelationship.setSourceEvent(eventCode);
        hlsUserRelationship.setSourceDocumentCategory(documentCategory);
        hlsUserRelationship.setSourceDocumentId(writeOffId);
        hlsUserRelationship.setSourceUserId(iRequest.getUserId());
        hlsUserRelationship.setLastUpdatedBy(userId);
        hlsUserRelationship.setLastUpdateDate(new Date());

        hlsSystemEventService.insert(iRequest, event);
        docAuthorityService.insert(iRequest, authority);

        if ("P2P".equalsIgnoreCase(hlsUserRelationship.getRelationType())) {
            hlsUserRelationship.setRelationUserId(iRequest.getUserId()); // 关联用户ID

            HlsUserRelationship userRelationship = new HlsUserRelationship();
            userRelationship.setUserId(hlsUserRelationship.getUserId());
            userRelationship.setRelationType(hlsUserRelationship.getRelationType());
            userRelationship.setRelationUserId(hlsUserRelationship.getRelationUserId());
            List<HlsUserRelationship> list = userRelationshipMapper.select(userRelationship);
            if (list.size() == 0) {
                hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship);
            }
        } else if ("P2D".equalsIgnoreCase(hlsUserRelationship.getRelationType())) {
            hlsUserRelationship.setRelationDocumentCategory(documentCategory);// 关联单据类别
            hlsUserRelationship.setRelationDocumentId(writeOffId); // 关系单据ID

            HlsUserRelationship userRelationship1 = new HlsUserRelationship();
            userRelationship1.setUserId(hlsUserRelationship.getUserId());
            userRelationship1.setRelationType(hlsUserRelationship.getRelationType());
            userRelationship1.setSourceDocumentCategory(hlsUserRelationship.getSourceDocumentCategory());
            userRelationship1.setSourceDocumentId(hlsUserRelationship.getSourceDocumentId());
            List<HlsUserRelationship> lists = userRelationshipMapper.select(userRelationship1);
            if (lists.size() == 0) {
                hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship);
            }
        }
        hlsSystemNoticeService.insert(iRequest, hlsSystemNotice);
        if (iCshWritiOffEventService != null) {
            iCshWritiOffEventService.afterUpdate(iRequest, eventCode, writeOffId);
        }
    }
}
