package hls.core.csh.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.csh.mapper.HlsCusGatheringMapper;
import hls.core.rpt.dto.RptReportHd;
import hls.core.rpt.service.IRptReportGetDataService;
import org.apache.poi.hssf.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.csh.dto.HlsCusGathering;
import hls.core.csh.service.IGatheringService;
import org.springframework.transaction.annotation.Transactional;

import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.List;

@Service
@Transactional
public class GatheringServiceImpl extends BaseServiceImpl<HlsCusGathering> implements IGatheringService , IRptReportGetDataService {

    @Autowired
    private HlsCusGatheringMapper gatheringMapper;

    @Autowired
    private IGatheringService iGatheringService;

    @Override
    public String getCode() {
        return "hls.core.csh.service.impl.GatheringServiceImpl";
    }
    @Override
    public void tempData(IRequest iRequest, RptReportHd rptReportHd) {
        HlsCusGathering gathering = new HlsCusGathering();
        gathering.setCompanyId(rptReportHd.getCompanyId());
        //查询需要的数据并做插入
        List<HlsCusGathering> gatherings = gatheringMapper.queryData(gathering);

        if(gatherings != null || gatherings.size()>0){
            for(int i=0;i<gatherings.size();i++){
                gatherings.get(i).setReportDtlId(rptReportHd.getReportDtlId());
                gatherings.get(i).set__status("add");
            }
            iGatheringService.batchUpdate(iRequest,gatherings);
        }
    }
    @Override
    public List<HlsCusGathering> queryResult(HlsCusGathering dto, int page, int pageSize){
        return gatheringMapper.queryResult(dto);
    }

    @Override
    public boolean doExcel(List<HlsCusGathering> gatherings){
        try {
            createExcel(gatherings);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }
    //创建收款查询表格、文件目录？(先放在本地，部署的时候名字生成要不同)，操作一些相关的表
    public void createExcel(List<HlsCusGathering> gatherings){
        try {
            //创建一行
            HSSFWorkbook wb = new HSSFWorkbook();
            //创建sheet
            HSSFSheet sheet = wb.createSheet("收款查询");
            // 创建一行
            HSSFRow rowTitle = sheet.createRow(0);
            // 创建标题栏样式
            HSSFCellStyle styleTitle = wb.createCellStyle();
            styleTitle.setAlignment(HSSFCellStyle.ALIGN_CENTER);// 居中
            HSSFFont fontTitle = wb.createFont();
            // 宋体加粗
            fontTitle.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
            fontTitle.setFontName("宋体");
            fontTitle.setFontHeight((short) 200);
            styleTitle.setFont(fontTitle);

            // 在新建的那行写一个单元格，并列标题及样式
            HSSFCell cellTitle = rowTitle.createCell(0);
            cellTitle.setCellValue("事务日期");
            cellTitle.setCellStyle(styleTitle);

            cellTitle = rowTitle.createCell(1);
            cellTitle.setCellValue("租金");
            cellTitle.setCellStyle(styleTitle);

            cellTitle = rowTitle.createCell(2);
            cellTitle.setCellValue("保证金");
            cellTitle.setCellStyle(styleTitle);

            cellTitle = rowTitle.createCell(3);
            cellTitle.setCellValue("手续费");
            cellTitle.setCellStyle(styleTitle);

            //取数据
            for(int i=0;i<gatherings.size();i++){

                HlsCusGathering dto = gatherings.get(i);
                //创建一行
                HSSFRow row = sheet.createRow(i + 1);
                //创建一个cell
                HSSFCell cell = row.createCell(0);
                //将日期对象转字符串，否则只是导入数字
                SimpleDateFormat simpleDateFormat=new SimpleDateFormat("yyyy-MM-dd");
                cell.setCellValue(simpleDateFormat.format(dto.getTranTime()));

                cell = row.createCell(1);
                cell.setCellValue(dto.getRent());

                cell = row.createCell(2);
                cell.setCellValue(dto.getMargin());

                cell = row.createCell(3);
                cell.setCellValue(dto.getHandCharge());
            }
            //定义输出流
            FileOutputStream fout = new FileOutputStream("D:/收款查询.xls");
            wb.write(fout);//HSSFWorkbook
            fout.close();
            wb.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}