package hls.core.db.interceptor;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.ibatis.executor.statement.RoutingStatementHandler;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.plugin.*;
import org.springframework.beans.factory.annotation.Value;

import java.lang.reflect.Field;
import java.sql.Connection;
import java.util.Properties;


/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2017-09-08
 * Time: 9:49
 */
@Intercepts({
        @Signature(method = "prepare", type = StatementHandler.class, args = {Connection.class})})
public class SqlIfNullInterceptor implements Interceptor {
    @Value("${db.type}")
    private String databaseType;
    private boolean isOracle;

    public void init() {
        if ("oracle".equalsIgnoreCase(databaseType)) {
            this.isOracle = true;
        }
    }

    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        if (!isOracle) {
            if ("oracle".equalsIgnoreCase(databaseType)) {
                isOracle = true;
            }
            return invocation.proceed();
        }
        RoutingStatementHandler handler = (RoutingStatementHandler) invocation.getTarget();
        StatementHandler delegate = (StatementHandler) getObject(handler, "delegate");
        BoundSql boundSql = delegate.getBoundSql();
        String sql = boundSql.getSql();
        String newSql = StringUtils.replaceChars(translate(sql), '`', '"');

        setObject(boundSql, "sql", newSql);
        return invocation.proceed();
    }

    public Object getObject(Object object, String name) {
        Field delegate = null;
        try {
            delegate = object.getClass().getDeclaredField(name);
            if (delegate != null) {
                delegate.setAccessible(true);
                Object o = delegate.get(object);
                delegate.setAccessible(false);
                return o;
            }
        } catch (NoSuchFieldException e) {
            //pass
        } catch (IllegalAccessException e) {
            //pass
        }
        return null;

    }

    public void setObject(Object object, String name, Object value) {
        try {
            Field declaredField = object.getClass().getDeclaredField(name);
            declaredField.setAccessible(true);
            declaredField.set(object, value);
            declaredField.setAccessible(false);
        } catch (Exception e) {
            //pass
        }
        return;
    }

    private String translate(String oldSql) {
        if (StringUtils.isEmpty(oldSql)) {
            return oldSql;
        }
        oldSql = oldSql
                .replaceAll("\\W*=\\W*\"(.*?)\"", "='$1'")
                .replaceAll("(?i)\\bifnull\\b", "NVL")
                .replaceAll("(?i)\\bgroup_concat\\(\\b", "group_concat(")
                .replaceAll("(?i)\\bdate\\(", "HDATE(");
        int index = -1;
        int lastIndex = -1;
        while ((index = oldSql.indexOf("group_concat(", index)) > 0) {
            index = index + "group_concat(".length();
            int count = 1;
            boolean isOk = false;
            for (int i = index + 1; i < oldSql.length() && !isOk; i++) {
                char c = oldSql.charAt(i);
                if (c == '(') {
                    count++;
                    continue;
                } else if (c == ')') {
                    count--;
                    if (count == 0) {
                        // 匹配到右括号
                        StringBuffer stringBuffer = new StringBuffer(oldSql);
                        stringBuffer.insert(i, ")");
                        oldSql = stringBuffer.toString();
                        isOk = true;
                        continue;
                    }
                }
            }
        }
        return oldSql.replaceAll("(?i)\\bgroup_concat\\(", "to_char(wmsys.wm_concat(");
    }

    @Override
    public Object plugin(Object o) {
        return Plugin.wrap(o, this);
    }

    @Override
    public void setProperties(Properties properties) {
        this.databaseType = properties.getProperty("databaseType");
        if ("oracle".equalsIgnoreCase(databaseType)) {
            this.isOracle = true;
        }
    }
}
