package hls.core.document.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import com.hand.hap.system.controllers.BaseController;
import hls.core.document.components.EntranceComponent;
import hls.core.document.utils.DocumentUtil;
import hls.core.sys.dto.HlsCusSysUserVisitHistory;
import hls.core.sys.service.ISysUserVisitHistoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by fjm on 2018/1/17.
 */
@Controller
public class DocumentEntranceController extends BaseController {

    private static final String REDIRECT = "redirect:";
    private static final String DOCUMENT_ID = "documentId";
    private static final String DOCUMENT_CATEGORY = "documentCategory";
    private static final String VISIT_TYPE = "visitType";
    private static final String VISIT_TITLE = "visitTitle";
    private static final String CREATE_VIEW = "create.view";
    public static final String VISIT_TYPE_SOURCE = "visitTypeSource";

    @Autowired
    private EntranceComponent entranceComponent;

    @Autowired
    private ISysUserVisitHistoryService sysUserVisitHistoryService;

    @RequestMapping(value = "/document_entrance")
    public ModelAndView redirect2Document(HttpServletRequest request) {
        IRequest currentRequest = RequestHelper.getCurrentRequest();
        Map<String, Object> parameterMap = getRequestParamMap(request);
        DocumentUtil.removeNullParameterAndSort(parameterMap);
        String entranceUrlByDocument = entranceComponent.getEntranceUrlByDocument(parameterMap);
        StringBuilder redirectUrl = new StringBuilder(entranceUrlByDocument);
        StringBuilder originUrl = new StringBuilder(request.getRequestURI() + "?");//请求url
        parameterMap.forEach((k, v) -> {
            originUrl.append(k + "=" + v + "&");
            if (!DOCUMENT_ID.equalsIgnoreCase(k) && !DOCUMENT_CATEGORY.equalsIgnoreCase(k)) {
                redirectUrl.append(k + "=" + v + "&");//拼接除了documentId和documentCategory之外原有的参数 exp:name=张三&
            }
        });
        if (originUrl.toString().endsWith("&"))
            originUrl.delete(originUrl.length() - 1, originUrl.length());
        if (redirectUrl.toString().endsWith("&"))
            redirectUrl.delete(redirectUrl.length() - 1, redirectUrl.length());
        if (!redirectUrl.toString().contains(CREATE_VIEW)) {
            createVisitObject(currentRequest, parameterMap, originUrl.toString());
        }
        ModelAndView mv = new ModelAndView();
        mv.setViewName(REDIRECT + redirectUrl);
        return mv;
    }

    @RequestMapping(value = "/module_entrance")
    public ModelAndView redirect2Module(HttpServletRequest request) {
        IRequest currentRequest = RequestHelper.getCurrentRequest();
        Map<String, Object> parameterMap = getRequestParamMap(request);
        StringBuilder originUrl = new StringBuilder(request.getRequestURI() + "?");//请求url
        String entranceUrlByModule = entranceComponent.getEntranceUrlByModule(parameterMap);
        parameterMap.forEach((k, v) -> {
            originUrl.append(k + "=" + v + "&");
        });
        if (originUrl.toString().endsWith("&"))
            originUrl.delete(originUrl.length() - 1, originUrl.length());
        HlsCusSysUserVisitHistory sysUserVisitHistory = new HlsCusSysUserVisitHistory();
        sysUserVisitHistory.setVisitType("MODULE");
        sysUserVisitHistory.setVisitUrl(originUrl.toString());
        sysUserVisitHistory.setVisitTitle((String) parameterMap.get("menu_name"));
        sysUserVisitHistory.setVisitTypeSource((String) parameterMap.get("menu_code"));
        if (sysUserVisitHistory.getVisitTypeSource() == null) {
            sysUserVisitHistory.setVisitTypeSource((String) parameterMap.get("code"));
        }
        sysUserVisitHistoryService.saveHistory(currentRequest, sysUserVisitHistory);
        sysUserVisitHistory.setVisitTypeSourceId(sysUserVisitHistory.getHistoryId());
        sysUserVisitHistory.setObjectVersionNumber(null);//避免乐观锁报错
        sysUserVisitHistoryService.updateByPrimaryKeySelective(currentRequest, sysUserVisitHistory);
        ModelAndView mv = new ModelAndView(REDIRECT + entranceUrlByModule);
        return mv;
    }

    /**
     * 插入visit数据
     *
     * @param request
     * @param parameterMap
     * @param url
     * @return
     */
    private HlsCusSysUserVisitHistory createVisitObject(IRequest request, Map<String, Object> parameterMap, String url) {
        HlsCusSysUserVisitHistory sysUserVisitHistory = new HlsCusSysUserVisitHistory();
        String visitType = (String) parameterMap.get(VISIT_TYPE);
        String visitTitle = (String) parameterMap.get(VISIT_TITLE);
        String visitTypeSource = (String) parameterMap.get(VISIT_TYPE_SOURCE);
        Long visitTypeSourceId = Long.parseLong(parameterMap.get(DOCUMENT_ID).toString());
        sysUserVisitHistory.setVisitUrl(url);
        sysUserVisitHistory.setVisitTitle(visitTitle);
        sysUserVisitHistory.setVisitType(visitType);
        sysUserVisitHistory.setVisitTypeSourceId(visitTypeSourceId);
        sysUserVisitHistory.setVisitTypeSource(visitTypeSource);
        sysUserVisitHistoryService.saveHistory(request, sysUserVisitHistory);
        return sysUserVisitHistory;
    }

    /**
     * 将请求中的参数转换为Map
     *
     * @param request
     * @return
     */
    private Map<String, Object> getRequestParamMap(HttpServletRequest request) {
        Enumeration<String> parameterNames = request.getParameterNames();
        final ArrayList<String> parameterNameList = Collections.list(parameterNames);
        Map<String, Object> parameterMap = new HashMap<>();
        parameterNameList.forEach(item -> {
            parameterMap.put(item, request.getParameter(item));
        });
        return parameterMap;
    }

}
