package hls.core.docx4J.component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.attachment.dto.Attachment;
import com.hand.hap.attachment.dto.SysFile;
import com.hand.hap.attachment.service.IAttachmentService;
import com.hand.hap.attachment.service.ISysFileService;
import com.hand.hap.core.IRequest;
import hls.core.cont.service.CustomSqlService;
import hls.core.docx4J.utils.Docx4jUtils;
import hls.core.fnd.dto.HlsDocFileParaTable;
import hls.core.fnd.dto.HlsDocFileTemplet;
import hls.core.fnd.dto.HlsDocFileTempletPara;
import hls.core.fnd.dto.HlsDocFileTmpParaLink;
import hls.core.fnd.service.HlsDocFileParaTableService;
import hls.core.fnd.service.HlsDocFileTempletParaService;
import hls.core.fnd.service.HlsDocFileTempletService;
import hls.core.fnd.service.HlsDocFileTmpParaLinkService;
import hls.core.hls.components.Datasource2Json;
import org.docx4j.jaxb.Context;
import org.docx4j.model.fields.merge.DataFieldName;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.openpackaging.parts.Part;
import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
import org.docx4j.wml.Body;
import org.docx4j.wml.Document;
import org.docx4j.wml.ObjectFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;


/**
 * by FJM
 * 所有类型书签替换都要经过此处，以此处为起点
 */
@Component
public class BookMarkReplaceComponent {


    @Autowired
    private CustomSqlService customSqlService;

    @Autowired
    private AddWatermarkAndDeleteComments addWatermarkAndDeleteComments;

    @Autowired
    private TableBookmarksReplaceWithContent tableBookmarksReplaceWithContent;

    @Autowired
    private BlockBookmarkReplaceWithContent blockBookmarkReplaceWithContent;

    @Autowired
    private InsertTableNextRow insertTableNextRow;

    @Autowired
    private TextBookmarkReplaceWithContents textBookmarkReplaceWithContents;

    @Autowired
    private HlsDocFileTempletService hlsDocFileTempletService;

    @Autowired
    private HlsDocFileTmpParaLinkService hlsDocFileTmpParaLinkService;

    @Autowired
    private HlsDocFileTempletParaService hlsDocFileTempletParaService;

    @Autowired
    private HlsDocFileParaTableService hlsDocFileParaTableService;

    @Autowired
    private IAttachmentService attachmentService;

    @Autowired
    private ISysFileService sysFileService;

    @Autowired
    private Datasource2Json datasource2Json;

    protected static Logger log = LoggerFactory.getLogger(BookMarkReplaceComponent.class);
    private static boolean DELETE_BOOKMARK = true;
    private static ObjectFactory factory = Context.getWmlObjectFactory();


    private static String bookmarkTypeTable = "TABLE";
    private static String bookmarkTypeFormTable = "FORM_TABLE";
    private static String bookmarkTypeDynamicTable = "DYNAMIC_TABLE";
    private static String bookmarkTypeText = "TEXT";
    private static String bookmarkTypeTextArea = "TEXT_AREA";
    private static String bookmarkTypeWatermark = "WATERMARK";

    public WordprocessingMLPackage docxCreateBookMarkReplaceWithText(WordprocessingMLPackage wordMLPackage, IRequest iRequest, Map<String, Object> params) throws Exception {

        Long templetId = Long.parseLong(params.get("templetId").toString());
        // Long contractId = Long.parseLong(params.get("contractId").toString());

        HlsDocFileTemplet hlsDocFileTemplet = new HlsDocFileTemplet();
        hlsDocFileTemplet.setTempletId(templetId);
        hlsDocFileTemplet = hlsDocFileTempletService.selectByPrimaryKey(iRequest, hlsDocFileTemplet);//此处查询的结果为文件模板对应的数据


        MainDocumentPart documentPart = wordMLPackage.getMainDocumentPart();
        Document wmlDocumentEl = documentPart.getJaxbElement();
        Body body = wmlDocumentEl.getBody();//拿到docx文件中的body


        if (hlsDocFileTemplet != null) {

            HlsDocFileTmpParaLink hlsDocFileTmpParaLink = new HlsDocFileTmpParaLink();
            hlsDocFileTmpParaLink.setTempletId(hlsDocFileTemplet.getTempletId());
            hlsDocFileTmpParaLink.setEnabledFlag("Y");
            List<HlsDocFileTmpParaLink> paraLinks = hlsDocFileTmpParaLinkService.selectListByObject(hlsDocFileTmpParaLink);//获取文件模板与参数的对应关系

            List<HlsDocFileTempletPara> bookMarkValueList = new ArrayList<>();//当前文档模板对应的参数的集合
            HlsDocFileTempletPara parameter = new HlsDocFileTempletPara();

            for (HlsDocFileTmpParaLink link : paraLinks) {
                parameter.setTempletParaId(link.getTempletParaId());
                parameter.setEnabledFlag("Y");
                bookMarkValueList.add(hlsDocFileTempletParaService.selectByPrimaryKey(iRequest, parameter));//将根据模板ID查询到的参数加入到参数集合中
            }


            Map<DataFieldName, String> map = new HashMap<DataFieldName, String>();
            Map<DataFieldName, String> mapType = new HashMap<DataFieldName, String>();
            Map<DataFieldName, String> mapFontFamily = new HashMap<DataFieldName, String>();
            Map<DataFieldName, String> mapFontSize = new HashMap<DataFieldName, String>();
            Map<DataFieldName, String> mapUnderLine = new HashMap<DataFieldName, String>();
            Map<DataFieldName, String> mapBold = new HashMap<DataFieldName, String>();

            List<Long> dsIds = new ArrayList<Long>();
            Map<String, Object> dsMap = new HashMap<String, Object>();
            for (int m = 0; m < bookMarkValueList.size(); m++) {
                if (bookMarkValueList.get(m).getDataSourceId() != null) {
                    dsIds.add(bookMarkValueList.get(m).getDataSourceId());
                    dsMap.put(bookMarkValueList.get(m).getDataSourceId().toString(), params);
                }

            }

            String datas = datasource2Json.executeSQLs4Json(dsIds, dsMap);
            JSONObject jsonObjects = JSON.parseObject(datas);

            if (bookMarkValueList != null && bookMarkValueList.size() != 0) {
                for (HlsDocFileTempletPara child : bookMarkValueList) {
                    String bookmark = child.getBookmark();
                    String bookmarkType = child.getBookmarkType();
                    HashMap param = new HashMap<>();//param为excuteSql方法所需要的参数。其中sql是必要的
                    if (child.getDataSourceId() == null) {
                        continue;
                    }
//					param.put("sql",child.getSqlSource());//获取文档模板参数中书签对应的sql
//					param.put("contractId",contractId);//先假设sql中的参数为合同ID且为值2
                    //FIX BY fjm(表格只能生成一行)  11.27 17   begin ------------
                    JSONObject result = JSON.parseObject(jsonObjects.getString(child.getDataSourceId().toString()));
                    JSONObject jsonObject = new JSONObject();
                    JSONArray jsonArray = new JSONArray();
                    boolean isDefault = false;
                    boolean isTable = false;
                    if (result.containsKey("default")) {
                        jsonObject = JSON.parseObject(JSON.toJSONString(result.get("default")));
                        isDefault = true;
                    } else if (result.containsKey("table")) {
                        jsonArray = JSONArray.parseArray(JSON.toJSONString(result.get("table")));
                        isTable = true;
                    } else {
                        continue;
                    }
//                  JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(jsonObject.get("default")));
//					List<HashMap> data = customSqlService.excuteSql(iRequest,param);//执行对应的动态sql，返回的值为一个list
//					if(data.size() == 0)
//					{
//						continue;
//					}
                    List<HashMap> data = new ArrayList<HashMap>();
                    if (isDefault) {
                        for (Iterator<String> iterator = jsonObject.keySet().iterator(); iterator.hasNext(); ) {
                            String key = iterator.next();
                            param.put(key, jsonObject.get(key));
                        }
                        data.add(param);
                    }
                    if (isTable) {
                        Iterator<Object> jsonArrayIterator = jsonArray.iterator();
                        while (jsonArrayIterator.hasNext()) {
                            JSONObject jsonArrayObject = (JSONObject) jsonArrayIterator.next();
                            param = new HashMap<>();
                            for (Iterator<String> iterator = jsonArrayObject.keySet().iterator(); iterator.hasNext(); ) {
                                String key = iterator.next();
                                param.put(key, jsonArrayObject.get(key));
                            }
                            data.add(param);
                        }
                    }
                    //FIX BY fjm  11.27 17   end ------------
                    HashMap bookmarkValueMap = data.stream().findFirst().orElseGet(HashMap::new);//若为替换书签的类型普通的文本或者文本域，则此处返回值为一个HashMap
                    if (bookmarkValueMap.isEmpty() && !bookmarkTypeWatermark.equals(bookmarkType)) {
                        continue;
                    }

                    if ("TEXT".equals(bookmarkType) || "TEXT_AREA".equals(bookmarkType)) {
                        Iterator<Map.Entry<String, Object>> iterator = bookmarkValueMap.entrySet().iterator();
                        while (iterator.hasNext()) {
                            Object obj = iterator.next().getValue();
                            map.put(new DataFieldName(bookmark), (obj == null) ? null : obj.toString());//将map中的值取出，该值为要替换的书签的值
                        }
                    } else if (bookmarkTypeWatermark.equals(bookmarkType)){
                        Iterator<Map.Entry<String, Object>> iterator = bookmarkValueMap.entrySet().iterator();
                        while (iterator.hasNext()) {
                            Object obj = iterator.next().getValue();
                            map.put(new DataFieldName(bookmark), (obj == null) ? null : obj.toString());//将map中的值取出，该值为要替换的书签的值
                        }
                    } else {
                        map.put(new DataFieldName(bookmark), "NO_DEAL");//除了文本和文本域之外的书签本身都不处理
                    }

                    mapType.put(new DataFieldName(bookmark), child.getBookmarkType());
                    mapFontFamily.put(new DataFieldName(bookmark), child.getFontFamily());
                    mapFontSize.put(new DataFieldName(bookmark), (child.getFontSize() == null) ? null : child.getFontSize().toString());
                    mapUnderLine.put(new DataFieldName(bookmark), child.getUnderline());
                    mapBold.put(new DataFieldName(bookmark), child.getBlod());

                    HlsDocFileParaTable tableChild = new HlsDocFileParaTable();
                    tableChild.setTempletParaId(child.getTempletParaId());

                    if (bookmarkTypeTable.equals(bookmarkType)) {
                        List<HlsDocFileParaTable> tableResultList = hlsDocFileParaTableService.selectList(tableChild);//此处代表书签处替换的为一个表单

                        tableBookmarksReplaceWithContent.replaceTableBookmarkContents(wordMLPackage, tableResultList, data, bookmark);

                    } else if (bookmarkTypeFormTable.equals(bookmarkType)) {

                        List<HlsDocFileParaTable> tableResultList = hlsDocFileParaTableService.selectList(tableChild);//此处代表书签处替换的为一个表单

                        blockBookmarkReplaceWithContent.replaceBlockWithBookmark(wordMLPackage, tableResultList, data);

                    } else if (bookmarkTypeDynamicTable.equals(bookmarkType)) {

                        List<HlsDocFileParaTable> tableResultList = hlsDocFileParaTableService.selectList(tableChild);

                        insertTableNextRow.replaceTableWithBookmark(wordMLPackage, tableResultList, data, bookmark);

                    } else if (bookmarkTypeWatermark.equals(bookmarkType)) {
                        List<SysFile> imgFile = sysFileService.queryFilesByTypeAndKey(iRequest, "DOC_WATER_MARK", child.getTempletParaId().toString());
                        String imgPath = imgFile.stream().findFirst().map(item -> item.getFilePath()).orElse("");
                        Map paramMap = new HashMap<>();
                        paramMap.put("bookmark", bookmark);
                        paramMap.put("bookmarkValue", map.get(new DataFieldName(bookmark)));
                        paramMap.put("fontFamily", child.getFontFamily());
                        paramMap.put("fontSize", child.getFontSize());
                        paramMap.put("bookmarkType", child.getBookmarkType());
                        paramMap.put("imgPath", imgPath);
                        addWatermarkAndDeleteComments.replaceBookmarkFromContent(wordMLPackage, paramMap);
                    }
                }
            }

            textBookmarkReplaceWithContents.replaceBookmarkContents(body.getContent(), map, mapType,
                    mapFontFamily, mapFontSize, mapUnderLine, mapBold);

            // 替换页眉以及页脚的书签
            Part header1 = Docx4jUtils.getNamedPart(wordMLPackage, "/word/header1.xml");
            Part footer1 = Docx4jUtils.getNamedPart(wordMLPackage, "/word/footer1.xml");
            Part header2 = Docx4jUtils.getNamedPart(wordMLPackage, "/word/header2.xml");
            Part footer2 = Docx4jUtils.getNamedPart(wordMLPackage, "/word/footer2.xml");
            List<Object> headContent1 = Docx4jUtils.getPartContent(header1);
            List<Object> footContent1 = Docx4jUtils.getPartContent(footer1);
            List<Object> headContent2 = Docx4jUtils.getPartContent(header2);
            List<Object> footContent2 = Docx4jUtils.getPartContent(footer2);

            if (headContent1 != null && headContent1.size() > 0) {
                textBookmarkReplaceWithContents.replaceBookmarkContents(headContent1, map, mapType, mapFontFamily,
                        mapFontSize, mapUnderLine, mapBold);
            }
            if (footContent1 != null && footContent1.size() > 0) {
                textBookmarkReplaceWithContents.replaceBookmarkContents(footContent1, map, mapType, mapFontFamily,
                        mapFontSize, mapUnderLine, mapBold);
            }
            if (headContent2 != null && headContent2.size() > 0) {
                textBookmarkReplaceWithContents.replaceBookmarkContents(headContent2, map, mapType, mapFontFamily,
                        mapFontSize, mapUnderLine, mapBold);
            }
            if (footContent2 != null && footContent2.size() > 0) {
                textBookmarkReplaceWithContents.replaceBookmarkContents(footContent2, map, mapType, mapFontFamily,
                        mapFontSize, mapUnderLine, mapBold);
            }
        }

        return wordMLPackage;

    }

}
