package hls.core.docx4J.utils;

import org.docx4j.openpackaging.exceptions.InvalidFormatException;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.openpackaging.parts.Part;
import org.docx4j.openpackaging.parts.PartName;
import org.docx4j.openpackaging.parts.Parts;
import org.docx4j.wml.*;
import org.springframework.util.StringUtils;

import java.math.BigInteger;
import java.util.List;

/**
 * Created by FJM on 2017/4/17.
 */
public class Docx4jUtils {
    /**
     * @param partName <b>文档某部分xml的名字</b>
     *                 <p>
     *                 <p>
     *                 word文档一般有以下几种：<br/>
     *                 /docProps/app.xml<br/>
     *                 /docProps/core.xml<br/>
     *                 /word/document.xml: 文档主体内容<br/>
     *                 /word/header1.xml /word/header2.xml /word/header3.xml ： 文档页眉内容
     *                 <br/>
     *                 /word/footer1.xml /word/footer2.xml /word/footer3.xml ：文档页脚内容<br/>
     *                 /word/fontTable.xml<br/>
     *                 /word/settings.xml<br/>
     *                 /word/styles.xml<br/>
     *                 /word/endnotes.xml<br/>
     *                 /word/webSettings.xml<br/>
     *                 /word/theme/theme1.xml<br/>
     *                 </p>
     * @return partName有效且该part存在时返回指定part，否则返回null
     */

    public static Part getNamedPart(WordprocessingMLPackage workPackage, String partName) {
        Parts parts = workPackage.getParts();
        if (parts != null) {
            Part part = null;
            try {
                part = parts.get(new PartName(partName));
            } catch (InvalidFormatException e) {
                throw new RuntimeException(partName + " is not valid format.");
            }
            if (part != null) {
                return part;
            }
        }
        return null;
    }

    /**
     * 获取Part的内容部分
     *
     * @return 仅当Part为ContentAccessor子类时有效，否则返回null
     */
    public static List<Object> getPartContent(Part part) {
        if (part != null && part instanceof ContentAccessor) {
            return ((ContentAccessor) part).getContent();
        }
        return null;
    }

    /**
     * 创建R标签
     *
     * @param value      指定R标签中的value
     * @param fontFamily 字体
     * @param fontSize   字体大小
     * @param underLine  下划线
     * @return
     */
    public static R makeRWithContent(String value, String fontFamily, String fontSize, String underLine, String bold) {
        R run = new R();
        Text t = new Text();
        RPr rPr = new RPr();
        RFonts RFonts = new RFonts();
        RFonts.setHint(STHint.EAST_ASIA);

        RFonts.setEastAsia(fontFamily);
        RFonts.setAscii(fontFamily);
        RFonts.setHAnsi(fontFamily);


        if (!StringUtils.isEmpty(fontSize)) {           //避免空指针异常
            HpsMeasure sz = new HpsMeasure();
            sz.setVal(new BigInteger(fontSize));
            rPr.setSz(sz);
            rPr.setSzCs(sz);
        }

        if (!StringUtils.isEmpty(underLine)) {
            U U = new U();
            rPr.setU(U);
            if ("SINGLE".equalsIgnoreCase(underLine)) {
                U.setVal(UnderlineEnumeration.SINGLE);
            } else {
                U.setVal(UnderlineEnumeration.DOUBLE);
            }
        }
        if ("Y".equalsIgnoreCase(bold)) {
            BooleanDefaultTrue b = new BooleanDefaultTrue();
            b.setVal(true);
            rPr.setB(b);
        }
        if ("R".equals(value) && "Wingdings 2".equals(fontFamily)) {
            R.Sym sym = new R.Sym();
            sym.setChar("F052");
            sym.setFont(fontFamily);
            run.setRPr(rPr);
            run.getContent().add(sym);
        } else {
            rPr.setRFonts(RFonts);
            run.getContent().add(t);
            run.setRPr(rPr);
            t.setValue(value);
        }
        return run;
    }


}
