package hls.core.ease.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.ease.service.IEaseService;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsUserRelationship;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by fjm on 2018/1/4.
 */
@Controller
@RequestMapping(value = "/ease")
public class EaseController extends BaseController {

    @Autowired
    private IEaseService easeService;

    @RequestMapping(value = "/system/noticeMessage/query")
    @ResponseBody
    public ResponseData getNoticeMessage(HttpServletRequest request, HlsSystemNotice hlsSystemNotice, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                         @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
        IRequest requestContext = RequestHelper.getCurrentRequest();
        ResponseData responseData = null;
        if (hlsSystemNotice == null)
            hlsSystemNotice = new HlsSystemNotice();
        hlsSystemNotice.setUserId(requestContext.getUserId());
        List<HlsSystemNotice> hlsSystemNotices = easeService.selectNoticeMessageForEase(requestContext, hlsSystemNotice, page, pagesize);
        if (hlsSystemNotices != null){
            responseData = new ResponseData(hlsSystemNotices);
        }else {
            responseData = new ResponseData();
        }
        return responseData;
    }
    /*查询我的收藏*/
    @RequestMapping(value = "/system/collection/query")
    @ResponseBody
    public ResponseData getCollections(HttpServletRequest request, HlsUserRelationship userRelationship) {
        IRequest requestContext = RequestHelper.getCurrentRequest();
        ResponseData responseData = null;
        if (userRelationship == null)
            userRelationship = new HlsUserRelationship();
        userRelationship.setUserId(requestContext.getUserId());
        List<HlsUserRelationship> resultList = easeService.getCollections(requestContext,userRelationship);
        if (resultList != null){
            responseData = new ResponseData(resultList);
        }else {
            responseData = new ResponseData();
            responseData.setSuccess(false);
        }
        return responseData;
    }
    /*查询收藏单据的消息动态*/
    @RequestMapping(value = "/system/collection/message/query")
    @ResponseBody
    public ResponseData getCollectionMessage(HttpServletRequest request, HlsSystemNotice hlsSystemNotice, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                             @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
        IRequest requestContext = RequestHelper.getCurrentRequest();
        ResponseData responseData = null;
        List<HlsSystemNotice> resultList = easeService.getCollectNoticeMessage(requestContext,hlsSystemNotice,page,pagesize);
        if (resultList != null){
            responseData = new ResponseData(resultList);
        }else {
            responseData = new ResponseData();
            responseData.setSuccess(false);
        }
        return responseData;
    }
    /*查询我的申请*/
    @RequestMapping(value = "/my/application/query")
    @ResponseBody
    public ResponseData queryMyApplication(HttpServletRequest request,@RequestParam(required = false) String description,@RequestParam(required = false) String approveStatus,@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                             @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
        IRequest requestContext = RequestHelper.getCurrentRequest();
        Map map = new HashMap();
        map.put("description",description);
        map.put("approveStatus",approveStatus);
        map.put("employeeCode",requestContext.getAttribute("employeeCode"));
        return new ResponseData(easeService.queryMyApplication(map,page,pagesize));
    }
    /*查询我的审批*/
    @RequestMapping(value = "/my/approve/query")
    @ResponseBody
    public ResponseData queryMyApprove(HttpServletRequest request,@RequestParam(required = false) String description,@RequestParam(required = false) String approveStatus,@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                           @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        IRequest requestContext = RequestHelper.getCurrentRequest();
        Map map = new HashMap();
        map.put("approveStatus",approveStatus);
        map.put("description",description);
        map.put("employeeCode",requestContext.getAttribute("employeeCode"));
        return new ResponseData(easeService.queryMyApprove(map,page,pageSize));
    }
    /*查询我的待办申请审批条数*/
    @RequestMapping(value = "/my/list/queryMyListCount")
    @ResponseBody
    public ResponseData queryMyListCount(HttpServletRequest request) {
        IRequest requestContext = RequestHelper.getCurrentRequest();
        Map map = new HashMap();
        map.put("userId",requestContext.getUserId());
        map.put("employeeCode",requestContext.getAttribute("employeeCode"));
        return new ResponseData(easeService.queryMyListCount(map));
    }
    /*查询事件屏蔽情况*/
    @RequestMapping(value = "/my/event/bother/query")
    @ResponseBody
    public ResponseData queryEventCondition(HttpServletRequest request,@RequestParam(required = false) String filterFlag,@RequestParam(required = false) String eventName,@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                            @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        IRequest requestContext = RequestHelper.getCurrentRequest();
        Map map = new HashMap();
        map.put("userId",requestContext.getUserId());
        map.put("filterFlag",filterFlag);
        map.put("eventName",eventName);
        return new ResponseData(easeService.queryEventCondition(map,page,pageSize));
    }
}
