package hls.core.fct.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fct.dto.HlsCusFctContract;
import hls.core.fct.service.FctContractService;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.service.HlsSystemNoticeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Controller
public class FctContractController extends BaseController {

    @Autowired
    private FctContractService service;

    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;


    @RequestMapping(value = "/fct/contract/query")
    @ResponseBody
    public ResponseData query(HlsCusFctContract dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }


    @RequestMapping(value = "/fct/contract/withdraw/detail/query")
    @ResponseBody
    public ResponseData fctContractWithdrawQuery(HlsCusFctContract dto) {
        return new ResponseData(service.fctContractWithdrawQuery(dto));
    }

    // 保理合同首页grid查询
    @RequestMapping(value = "/fct/contract/detail/query")
    @ResponseBody
    public ResponseData detailQuery(HlsCusFctContract dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.fctContractDetailQuery(requestContext, dto, page, pageSize));
    }

    // 保理合同首页动态查询
    @RequestMapping(value = "/fct/contract/dynamicInfoQuery")
    @ResponseBody
    public ResponseData contractDynamicInfoQuery(HlsSystemNotice hlsSystemNotice, final HttpServletRequest request, @RequestParam(defaultValue = DEFAULT_PAGE) int page, @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        IRequest requestContext = createRequestContext(request);
        hlsSystemNotice.setCompanyId(requestContext.getCompanyId());
        List<HlsSystemNotice> hlist = hlsSystemNoticeService.selectFctContractManagerNotice(hlsSystemNotice, page, pageSize);
        for(int i=0;i<hlist.size();i++){
            if(hlist.get(i).getNotice_message().length()>25){
                hlist.get(i).setNotice_message(hlist.get(i).getNotice_message().substring(0,25)+"....");
            }
        }

        return new ResponseData(hlist);
    }

    // 保理合同首页合同状态统计图
    @RequestMapping(value = "/fct/contract/contractCountByStatus")
    @ResponseBody
    public ResponseData contractStatusChart(HlsCusFctContract dto, final HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        dto.setCompanyId(requestContext.getCompanyId());

        List<HlsCusFctContract> hList = service.selectFctContractCountByStatus(dto);
        return new ResponseData(hList);
    }


    @RequestMapping(value = "/fct/contract/detail/line/query")
    @ResponseBody
    public ResponseData datailQuery(HlsCusFctContract dto, HttpServletRequest request,@RequestParam Long contractId ) {
        IRequest requestContext = createRequestContext(request);
        if(contractId == null){
            contractId = 0L;
        }
        dto.setContractId(contractId);
        dto.setCompanyId(requestContext.getCompanyId());
        return new ResponseData(service.fctContractDetailLineQuery(dto));
    }

    @RequestMapping(value = "/fct/contract/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusFctContract> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/fct/contract/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusFctContract> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }



    @RequestMapping(value = "/fct/contract/withdraw/request/status")
    @ResponseBody
    public String withdrawRequestStatus(@RequestParam Long contractId) {
        return service.withdrawRequestStatus(contractId);
    }

    @RequestMapping(value = "/fct/contract/advances/received/status")
    @ResponseBody
    public String advancesReceivedStatus(@RequestParam Long contractId) {
        return service.advancesReceivedStatus(contractId);
    }

    @RequestMapping(value = "/fct/contract/special/payment/status")
    @ResponseBody
    public String specialRequestAndPaymentRequestStatus(@RequestParam Long contractId) {
        return service.specialRequestAndPaymentRequestStatus(contractId);
    }
    @RequestMapping(value = "/fct/contract/collection/cashflow/status")
    @ResponseBody
    public String collectionCashflowStatus(@RequestParam Long contractId) {
        return service.collectionCashflowStatus(contractId);
    }

    @RequestMapping(value = "/fct/contract/et/status")
    @ResponseBody
    public String etStatus(@RequestParam Long contractId) {
        return service.earlyTerminateStatus(contractId);
    }

    @RequestMapping(value = "/fct/contract/terminate/status")
    @ResponseBody
    public String contactTerminateStatus(@RequestParam Long contractId) {
        return service.contactTerminateStatus(contractId);
    }

    @RequestMapping(value = "/fct/contract/detail/info")
    @ResponseBody
    public ResponseData selectContractDetailInfo(Long contractId) {
        HlsCusFctContract fctContract=new HlsCusFctContract();
        fctContract.setContractId(contractId);
        return new ResponseData(service.selectContractDetailInfo(fctContract));
    }
    @RequestMapping(value = "/fct/contract/end/submit")
    @ResponseBody
    public ResponseData contractEndSubmit(HttpServletRequest request,Long contractId) {
        IRequest requestCtx = createRequestContext(request);
        HlsCusFctContract fctContract=new HlsCusFctContract();
        fctContract.setContractId(contractId);
        fctContract=service.selectByPrimaryKey(requestCtx,fctContract);
        return new ResponseData(service.contractEndSubmit(requestCtx,fctContract));
    }
}