package hls.core.fct.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.csh.dto.HlsCusCshPaymentReqHd;
import hls.core.csh.dto.HlsCusCshPaymentReqLn;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.mapper.HlsCusCshPaymentReqHdMapper;
import hls.core.csh.service.CshWriteOffService;
import hls.core.csh.service.ICshPaymentReqLnService;
import hls.core.fct.dto.HlsCusFctContractWithdraw;
import hls.core.fct.mapper.HlsCusFctContractWithdrawMapper;
import hls.core.fct.service.FctContractPaymentService;
import hls.core.fct.service.FctContractWithdrawService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author: syj
 * @Description:
 * @Date: Created in 2017/12/6 15:18
 * @Modified By:
 */
@Controller
public class FctContractPaymentController extends BaseController {
    @Autowired
    private FctContractWithdrawService fctContractWithdrawService;
    @Autowired
    private HlsCusFctContractWithdrawMapper hlsCusFctContractWithdrawMapper;
    @Autowired
    private FctContractPaymentService fctContractPaymentService;
    @Autowired
    private ICshPaymentReqLnService iCshPaymentReqLnService;

    @Autowired
    private CshWriteOffService cshWriteOffService;
    @Autowired
    private HlsCusCshPaymentReqHdMapper hlsCusCshPaymentReqHdMapper;


    /**
     * 付款申请保存
     *
     * @return
     */
    @RequestMapping(value = "/fct/csh/payment/save")
    @ResponseBody
    public ResponseData save(@RequestBody HlsCusFctContractWithdraw withdraw, final HttpServletRequest request) {
        try {
            IRequest requestContext = createRequestContext(request);
            HlsCusCshPaymentReqHd hd = fctContractPaymentService.payRequestSave(withdraw, requestContext);
            List<HlsCusCshPaymentReqHd> hdList = new ArrayList<HlsCusCshPaymentReqHd>();
            hdList.add(hd);
            return new ResponseData(hdList);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        } catch (ParseException e) {
            e.printStackTrace();
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;

        }

    }


    /**
     * 付款申请提交
     *
     * @return
     */
    @RequestMapping(value = "/fct/csh/payment/submit")
    @ResponseBody
    public ResponseData submit(@RequestBody HlsCusFctContractWithdraw withdraw, final HttpServletRequest request, HttpSession session) {
        try {
            IRequest requestContext = createRequestContext(request);


            withdraw.setDocumentCategory("FCT_CONTRACT_WITHDRAW");
            withdraw.setBusinessType("FCT_CONTRACT_WITHDRAW");
            withdraw.setDocumentType("STD");
//            withdraw=fctContractWithdrawService.selectByPrimaryKey(requestContext,withdraw);
            HlsCusCshPaymentReqHd hd = fctContractPaymentService.payRequestSave(withdraw, requestContext);
            withdraw.setWithdrawId(hd.getWithdrawId());
            withdraw.setPaymentReqId(hd.getPayment_req_id());
            withdraw.setPaymentReqLnId(hd.getPaymentReqLnId());
//            withdraw.setDocumentType("FCT_PAYMENT_DEBT");
//            withdraw.setDocumentCategory("CSH_PAYMENT_REQ");

            fctContractPaymentService.cshPaymentReqSubmit(requestContext, withdraw);
            return new ResponseData();
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        } catch (ParseException e) {
            e.printStackTrace();
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }

    }

    /**
     * 支付
     *
     * @param cshTransactions
     * @param result
     * @param request
     * @param session
     * @return
     * @throws BeyondAmountLimitException
     */
    @RequestMapping(value = "/fct/csh/payment/pay")
    @ResponseBody
    public ResponseData pay(@RequestBody List<HlsCusCshWriteOff> cshTransactions, final BindingResult result, final HttpServletRequest request, HttpSession session) throws BeyondAmountLimitException {
        Double amount = 0d;
        int payFlag = 0;
        ResponseData responseData = null;
        if (result.hasErrors()) {
            responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }

        HlsCusCshPaymentReqLn cshPaymentReqLn = new HlsCusCshPaymentReqLn();
        List<HlsCusCshPaymentReqLn> payLnList = new ArrayList<>();
        HlsCusCshPaymentReqHd cshPaymentReqHd = new HlsCusCshPaymentReqHd();

        for (int i = 0; i < cshTransactions.size(); i++) {
            Double OffAmount = cshTransactions.get(i).getCshWriteOffAmount();
            cshPaymentReqLn.setPaymentReqId(cshTransactions.get(i).getPaymentReqId());
            List<HlsCusCshPaymentReqLn> cshPLnList = iCshPaymentReqLnService.queryByHnId(cshPaymentReqLn);
            cshTransactions.get(i).setWriteOffType("FCT_PAYMENT_DEBT");

            if (cshPLnList.size() > 0) {
                for (int j = 0; j < cshPLnList.size(); j++) {
                    payLnList.add(cshPLnList.get(j));
                }
            }
            if (OffAmount != null) {
                amount += OffAmount;
            }
        }
        if (payLnList.size() > 0) {
            for (int m = 0; m < payLnList.size(); m++) {
                if (payLnList.get(m).getPayment_flag() == "FULL") {
                    payFlag++;
                }
            }
        }
        ResponseData rd = null;
        if (payFlag == payLnList.size()) {
            rd = new ResponseData(false);
            rd.setMessage("该付款已被支付");
            return rd;
        } else {
            try {
                cshWriteOffService.updateWriteOff(createRequestContext(request), cshTransactions, session);
            } catch (Exception e) {
                responseData = new ResponseData(false);
                responseData.setMessage(e.getMessage());
                return responseData;
            }
            List<Double> list = new ArrayList<>();
            list.add(amount);
            return new ResponseData(list);
        }
    }

    /**
     * 保理付款申请查询
     */
    @RequestMapping(value = "/fct/csh/payment/req/history")
    @ResponseBody
    public ResponseData detailQuery(HlsCusCshPaymentReqHd dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(fctContractPaymentService.fctPaymentReqHistorySelect(dto));
    }


}
