package hls.core.fct.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fct.dto.HlsCusFctContractWithdraw;
import hls.core.fct.dto.HlsCusFctContractWithdrawCf;
import hls.core.fct.service.FctContractService;
import hls.core.fct.service.FctContractWithdrawCfService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.util.List;

@Controller
public class FctContractWithdrawCfController extends BaseController {

    @Autowired
    private FctContractWithdrawCfService service;
    @Autowired
    private FctContractService fctContractService;


    @RequestMapping(value = "/fct/contract/withdraw/cf/query")
    @ResponseBody
    public ResponseData query(HlsCusFctContractWithdrawCf dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/fct/contract/withdraw/cf/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusFctContractWithdrawCf> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/fct/contract/withdraw/cf/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusFctContractWithdrawCf> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/fct/contract/withdraw/cf/calcAdvance")
    @ResponseBody
    public ResponseData createContractWithdrawCfAdvance(@RequestBody HlsCusFctContractWithdraw dto, BindingResult result, HttpServletRequest request) throws ParseException {
        getValidator().validate(dto, result);
        /*if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }*/
        try {
            IRequest requestCtx = createRequestContext(request);
            return new ResponseData(fctContractService.createContractWithdrawCfAdvance(requestCtx, dto));
        }catch(Exception e){
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

    @RequestMapping(value = "/fct/contract/withdraw/cf/calcPayment")
    @ResponseBody
    public ResponseData createContractWithdrawCfPayment(@RequestBody HlsCusFctContractWithdraw dto, BindingResult result, HttpServletRequest request) throws ParseException {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(fctContractService.createContractWithdrawCfPayment(requestCtx, dto));
    }


    @RequestMapping(value = "/fct/contract/withdraw/cf/reCalc")
    @ResponseBody
    public ResponseData reCalc(@RequestBody List<HlsCusFctContractWithdrawCf> dto, BindingResult result, HttpServletRequest request) throws ParseException {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(fctContractService.reCalcContractWithdrawCf(requestCtx, dto));
    }


    /**
     * （预计）收款安排制作，计算完成后的grid查询
     */
    @RequestMapping(value = "/fct/contract/withdraw/arrange/make/query")
    @ResponseBody
    public ResponseData withdrawArrangeMakeQuery(HlsCusFctContractWithdrawCf dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.fctContractWithdrawCfQuery(requestContext, dto, page, pageSize));
    }
}