package hls.core.fct.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fct.dto.HlsCusFctContractWithdraw;
import hls.core.fct.service.FctContractWithdrawService;
import hls.core.fnd.service.FndCodingRuleValuesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

@Controller
public class FctContractWithdrawController extends BaseController {

    @Autowired
    private FctContractWithdrawService service;
    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;


    @RequestMapping(value = "/fct/contract/withdraw/query")
    @ResponseBody
    public ResponseData query(HlsCusFctContractWithdraw dto, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        dto.setCompanyId(requestContext.getCompanyId());
        return new ResponseData(service.fctContractWithdrawAprlDetail(dto));
    }


    @RequestMapping(value = "/fct/contract/payment/detail/query")
    @ResponseBody
    public ResponseData paymentDetailQuery(HlsCusFctContractWithdraw dto, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        dto.setCompanyId(requestContext.getCompanyId());
        return new ResponseData(service.paymentDetailQuery(dto));
    }



    @RequestMapping(value = "/fct/contract/withdraw/history/query")
    @ResponseBody
    public ResponseData withdrawHistoryQuery(Long contractId, HttpServletRequest request) {
        HlsCusFctContractWithdraw dto = new HlsCusFctContractWithdraw();
        dto.setContractId(contractId);
        IRequest requestContext = createRequestContext(request);
        dto.setCompanyId(requestContext.getCompanyId());
        return new ResponseData(service.fctContractWithdrawDetailQuery(dto));
    }

    @RequestMapping(value = "/fct/contract/history/query")
    @ResponseBody
    public ResponseData historyQuery(@RequestParam Long contractId,@RequestParam String withdrawApprovalType, HttpServletRequest request) {
        HlsCusFctContractWithdraw dto = new HlsCusFctContractWithdraw();
        dto.setContractId(contractId);
        IRequest requestContext = createRequestContext(request);
        dto.setCompanyId(requestContext.getCompanyId());
        String withdrawApprovalTypeStr = "";
        switch (withdrawApprovalType){

            // 预计收款安排制作
            case "PRE_RECEIPT":
                withdrawApprovalTypeStr = "CREATE_ADVANCES_RECEIVED";
                break;
            // 付款申请
            case "PAYMENT_APPLY":
                withdrawApprovalTypeStr = "PAYMENT_REQ";
                break;
            // 收款安排制作
            case "RECEIPT":

                withdrawApprovalTypeStr = "CREATE_COLLECTION_CASHFLOW";
                break;
        }
        dto.setWithdrawApprovalTypeArr(withdrawApprovalTypeStr.split("-"));
        return new ResponseData(service.fctContractHistoryQuery(dto));
    }



    @RequestMapping(value = "/fct/contract/withdraw/payment/history/query")
    @ResponseBody
    public ResponseData paymentHistoryQuery(@RequestParam Long contractId, HttpServletRequest request) {
        HlsCusFctContractWithdraw dto = new HlsCusFctContractWithdraw();
        dto.setContractId(contractId);
        IRequest requestContext = createRequestContext(request);
        dto.setCompanyId(requestContext.getCompanyId());
        return new ResponseData(service.withdrawPaymentHistoryQuery(dto));
    }



    @RequestMapping(value = "/fct/contract/withdrawSpcl/query")
    @ResponseBody
    public ResponseData querySpcl(Long withdrawId, HttpServletRequest request) {
        HlsCusFctContractWithdraw fctContractWithdraw = new HlsCusFctContractWithdraw();
        fctContractWithdraw.setWithdrawId(withdrawId);
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.fctContractWithdrawDetailForSpcl(fctContractWithdraw));
    }
    /**
     * 用款需求申请 提交审批
     *
     * @param dto
     * @param result
     * @param request
     * @return
     * @throws ParseException
     */
    @RequestMapping(value = "/fct/contract/withdraw/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusFctContractWithdraw> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        try {
            IRequest requestCtx = createRequestContext(request);
            List<HlsCusFctContractWithdraw> results =service.cashDemandSubmit(requestCtx, dto);
            return new ResponseData(results);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

    /**
     * 计算之前保存修改的提款信息
     * @param dto
     * @param request
     * @return
     */
    @RequestMapping(value = "/fct/contract/withdrawInfo/update")
    @ResponseBody
    public ResponseData updateWithdrawInfo(@RequestBody List<HlsCusFctContractWithdraw> dto, HttpServletRequest request) {
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.fctContractWithdrawSave(requestCtx,dto));
    }


    @RequestMapping(value = "/fct/contract/withdraw/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusFctContractWithdraw> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/fct/contract/withdraw/lov")
    @ResponseBody
    public ResponseData withdrawInfoLov(HlsCusFctContractWithdraw dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                        @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.fctContractWithdrawInfoLov(requestContext, dto, page, pageSize));
    }


    @RequestMapping(value = "/fct/contract/withdraw/writeoff/lov")
    @ResponseBody
    public ResponseData withdrawInfoWriteOffLov(HlsCusFctContractWithdraw dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                        @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.fctContractWithdrawInfoWriteOffLov(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/fct/contract/withdraw/et/lov")
    @ResponseBody
    public ResponseData withdrawInfoETLov(HlsCusFctContractWithdraw dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.fctContractWithdrawInfoETLov(requestContext, dto, page, pageSize));
    }

    /**
     * 预计收款安排制作 提交审批
     *
     * @param dto
     * @param result
     * @param request
     * @return
     * @throws ParseException
     */
    @RequestMapping(value = "/fct/contract/withdraw/cf/wfl")
    @ResponseBody
    public ResponseData repaymentSubmit(@RequestBody HlsCusFctContractWithdraw dto, BindingResult result, HttpServletRequest request) throws ParseException {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        List<HlsCusFctContractWithdraw> withdraws =  new ArrayList<>();
        withdraws.add(dto);
        service.repaymentSubmit(requestCtx, withdraws);
        return new ResponseData();
    }

    /**
     * 收款安排制作 提交审批
     *
     * @param dto
     * @param result
     * @param request
     * @return
     * @throws ParseException
     */
    @RequestMapping(value = "/fct/contract/withdraw/cf/re/wfl")
    @ResponseBody
    public ResponseData collectionCashflowSubmit(@RequestBody HlsCusFctContractWithdraw dto, BindingResult result, HttpServletRequest request) throws ParseException {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        List<HlsCusFctContractWithdraw> withdraws =  new ArrayList<>();
        withdraws.add(dto);
        service.collectionCashflowSubmit(requestCtx, withdraws);
        return new ResponseData();
    }



/*    @RequestMapping(value = "/fct/contract/withdraw/fcd/wfl")
    @ResponseBody
    public ResponseData cashDemandSubmit(@RequestBody HlsCusFctContractWithdraw dto, BindingResult result, HttpServletRequest request) throws ParseException {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        List<HlsCusFctContractWithdraw> withdraws =  new ArrayList<>();
        withdraws.add(dto);
        service.cashDemandSubmit(requestCtx, withdraws);
        return new ResponseData();
    }*/


    /**
     * 提前还款 提交审批
     *
     * @param dto
     * @param result
     * @param request
     * @return
     * @throws ParseException
     */
    @RequestMapping(value = "/fct/contract/withdraw/et/submit")
    @ResponseBody
    public ResponseData et_submit(@RequestBody List<HlsCusFctContractWithdraw> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        service.earlyTerminateSubmit(requestCtx, dto);
        return new ResponseData();
    }

    //生成合同文本记录
    @RequestMapping(value = "/fct/contract/withdraw/content/create")
    @ResponseBody
    public ResponseData fctContentCreate(HttpServletRequest request, @RequestBody final HlsCusFctContractWithdraw fctContractWithdraw) {
        IRequest iRequest = createRequestContext(request);
        service.createWithdrawContent(iRequest,fctContractWithdraw);
        return new ResponseData();
    }
}