package hls.core.fct.controllers;

import com.hand.hap.attachment.dto.SysFile;
import com.hand.hap.core.IRequest;
import com.hand.hap.security.TokenUtils;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fct.dto.HlsCusFctProjectAttachment;
import hls.core.fct.service.FctProjectAttachmentService;
import hls.core.hls.bp.dto.HlsCusSysFile;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

@Controller
public class FctProjectAttachmentController extends BaseController {

    @Autowired
    private FctProjectAttachmentService service;


   @RequestMapping(value = "/fct/project/attachment/other/query")
    @ResponseBody
    public ResponseData query(Long sourceId, HttpServletRequest request) {
       List<HlsCusFctProjectAttachment> list=new ArrayList<>();
       list=service.fctProjectAttQuery(sourceId);
       //生成file_token
       String securityKey = TokenUtils.getSecurityKey(request.getSession());
       SysFile file = new SysFile();
       for (HlsCusFctProjectAttachment f : list) {
           file.setFileId(f.getFileId());
           f.set_token(TokenUtils.generateToken(securityKey, file));
       }
        return new ResponseData(list);
    }

    @RequestMapping(value = "/fct/project/attachment/query")
    @ResponseBody
    public ResponseData creditLineDetailQuery(HlsCusFctProjectAttachment dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        Long projectId = dto.getProjectId();
        if (projectId == null) {
            projectId = 0L;
            dto.setProjectId(projectId);
        }
        //dto.setCreditLineId(creditLineId);
        return new ResponseData(service.fctProjectAttachmentQuery(dto, page, pageSize));
    }

    @RequestMapping(value = "/fct/cre/project/attachment/query")
    @ResponseBody
    public ResponseData fctCreAttQuery(HlsCusFctProjectAttachment dto,@RequestParam(defaultValue = DEFAULT_PAGE) int page,
    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        return new ResponseData(service.fctCreAttQuery(dto));
    }

    @RequestMapping(value = "/fct/project/attachment/sign/submit")
    @ResponseBody
    public ResponseData signUpdate(@RequestBody List<HlsCusFctProjectAttachment> fctProjectAttachmentList, BindingResult result, HttpServletRequest request) {
        getValidator().validate(fctProjectAttachmentList, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
         }
        IRequest iRequest = createRequestContext(request);
        if(fctProjectAttachmentList!=null){
            for (HlsCusFctProjectAttachment dt : fctProjectAttachmentList) {
                if(dt.getProjectAttachmentId()== null || dt.getProjectAttachmentId()== 0){
                    dt.set__status("add");
                }
                else{
                    if(dt.getStatus().equals("GENERATED")) {
                        dt.setStatus("UN_SIGN");
                    }else if(dt.getStatus().equals("UN_SIGN")) {
                        dt.setStatus("GENERATED");
                    }
                    dt.set__status("update");
                }
            }
        }
        return new ResponseData(service.batchUpdate(iRequest,fctProjectAttachmentList));
    }

    @RequestMapping(value = "/fct/project/attachment/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusFctProjectAttachment> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        if (dto != null) {
            for (HlsCusFctProjectAttachment dt : dto) {
                if(dt.getProjectAttachmentId()== null || dt.getProjectAttachmentId()== 0)
                {
                    dt.set__status("add");

                }else
                {
                    dt.set__status("update");
                }
            }
        }
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }




    @RequestMapping(value = "/fct/project/attachment/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusFctProjectAttachment> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }


    @RequestMapping(value = "/fct/contract/content/query")
    @ResponseBody
    public ResponseData contentDetailQuery(@RequestBody HlsCusFctProjectAttachment dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        return new ResponseData(service.fctContractContentQuery(dto, page, pageSize));
    }

    @RequestMapping(value = "/fct/contract/content/history/query")
    @ResponseBody
    public ResponseData contentHistoryDetailQuery(@RequestBody HlsCusFctProjectAttachment dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                           @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        return new ResponseData(service.fctContractContentHistoryQuery(dto, page, pageSize));
    }

    @RequestMapping(value = "/fct/contract/content/att/query")
    @ResponseBody
    public ResponseData contractContentDetailQuery(@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                           @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize,Long contractId) {
        if(contractId == null){
            contractId = 0L;
        }
        return new ResponseData(service.fctContractQuery(contractId, page, pageSize));
    }

    @RequestMapping(value = "/fct/attachment/history/query")
    @ResponseBody
    public ResponseData attachmentHistoryDetailQuery(@RequestBody HlsCusFctProjectAttachment dto) {
        return new ResponseData(service.fctProjectAttachmentForSignProcessQuery(dto));
    }

    @RequestMapping(value = "/fct/contract/management/content/query")
    @ResponseBody
    public ResponseData fctContractManagementContentQuery(HlsCusFctProjectAttachment dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                           @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize,HttpServletRequest request) {
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.fctContractManagementContentQuery(dto, page, pageSize));
    }

    @RequestMapping(value = "/hls/fct/attachment/risk/report/query")
    @ResponseBody
    public ResponseData riskReportQuery(HlsCusFctProjectAttachment dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                        @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        String[] fileTypeArr=dto.getSourceType().split("-");
        dto.setFileTypeArr(fileTypeArr);
        List<HlsCusFctProjectAttachment> list = service.riskAssessmentReportFctQuery(dto);
        SysFile file = new SysFile();
        String securityKey = TokenUtils.getSecurityKey(request.getSession());
        for (HlsCusFctProjectAttachment f : list) {
            if(f.getFileId()==null||"".equals(f.getFileId())){

            }else{
                file.setFileId(Long.valueOf(f.getFileId()));
                f.set_token(TokenUtils.generateToken(securityKey, file));
            }
        }
        return new ResponseData(list);
    }

}