package hls.core.fct.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fct.dto.HlsCusFctProjectBasicTrx;
import hls.core.fct.service.FctProjectBasicTrxService;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.service.HlsSystemNoticeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@Controller
    public class FctProjectBasicTrxController extends BaseController{
        public static String DOCUMENT_CATEGORY = "FCT_PROJECT_BASIC_TRX";
        public static String DOCUMENT_TYPE = "BASIC_TRX";
        public static String BUSINESS_TYPE = "BASIC_TRX";

    @Autowired
    private FctProjectBasicTrxService service;
        @Autowired
        private FndCodingRuleValuesService fndCodingRuleValuesService;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;


    @RequestMapping(value = "/fct/project/basic/trx/query")
    @ResponseBody
    public ResponseData query(HlsCusFctProjectBasicTrx dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext,dto,page,pageSize));
    }
    @RequestMapping(value = "/fct/project/basic/trx/queryAll")
    @ResponseBody
    public ResponseData queryAllTrx(HlsCusFctProjectBasicTrx dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
            IRequest requestContext = createRequestContext(request);
            return new ResponseData(service.queryAllTrxInfo(requestContext,dto,page,pageSize));
    }

    @RequestMapping(value = "/fct/project/basic/trx/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusFctProjectBasicTrx> dto, BindingResult result, HttpServletRequest request){
    getValidator().validate(dto, result);
    if (result.hasErrors()) {
    ResponseData responseData = new ResponseData(false);
    responseData.setMessage(getErrorMessage(result, request));
    return responseData;
    }
        IRequest requestCtx = createRequestContext(request);
        if(dto !=null){
            for(HlsCusFctProjectBasicTrx dt:dto){
                if(dt.getBasicTrxId()==null||dt.getBasicTrxId()==0){
                    dt.set__status("add");
                    dt.setBasicDealsStatus("NEW");
                    dt.setDocumentType("BASIC_TRX");
                    dt.setDocumentCategory("FCT_PROJECT_BASIC_TRX");
                    dt.setBusinessType("BASIC_TRX");
                    dt.setObjectVersionNumber(1L);
                    Map<String, String> params = new HashMap<String, String>();
                    dt.setBasicDealsNumber(fndCodingRuleValuesService.getCodeRuleValue(requestCtx, DOCUMENT_CATEGORY, DOCUMENT_TYPE, BUSINESS_TYPE, params));
                }else{
                    dt.set__status("update");
                    dt.setObjectVersionNumber(dt.getObjectVersionNumber());
                }
            }
        }
        List<HlsCusFctProjectBasicTrx> hlsCusFctProjectBasicTrxList = new ArrayList<>();
        hlsCusFctProjectBasicTrxList = service.batchUpdate(requestCtx, dto);
        if(hlsCusFctProjectBasicTrxList.get(0).get__status() == "add"){
            service.addTrxEvent(requestCtx,hlsCusFctProjectBasicTrxList.get(0));
        }else if(hlsCusFctProjectBasicTrxList.get(0).get__status() == "update"){
            service.updateTrxEvent(requestCtx,hlsCusFctProjectBasicTrxList.get(0));
        }
        return new ResponseData(hlsCusFctProjectBasicTrxList);
    }

    @RequestMapping(value = "/fct/project/basic/trx/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request,@RequestBody List<HlsCusFctProjectBasicTrx> dto){
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/fct/project/basic/trx/aprl/submit")
    @ResponseBody
    public ResponseData trxchangeSubmit(HttpServletRequest request, @RequestBody List<HlsCusFctProjectBasicTrx> dto) {
        try {
            IRequest requestCtx = createRequestContext(request);
            List<HlsCusFctProjectBasicTrx> result =service.submitApproval(requestCtx,dto);
            return new ResponseData(result);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }
    @RequestMapping(value = "/fct/project/dynamicInfo/query")
    @ResponseBody
    public ResponseData dynamicInfo(final HttpServletRequest request, final HlsSystemNotice hlsSystemNotice, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize){
        IRequest iRequest = createRequestContext(request);
        hlsSystemNotice.setCompanyId(iRequest.getCompanyId());
        List<HlsSystemNotice> hlist = hlsSystemNoticeService.selectFctNotice(hlsSystemNotice, page, pageSize);
        for(int i=0;i<hlist.size();i++){
            Date prjDate = hlist.get(i).getNotice_datetime();
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
            String dateString = formatter.format(prjDate);
            hlist.get(i).setNotice_datetime_differ(dateString);
            if(hlist.get(i).getNotice_message().length()>16){
                hlist.get(i).setNotice_message(hlist.get(i).getNotice_message().substring(0,16)+"....");
            }
        }
        return new ResponseData(hlist);
    }
    }