package hls.core.fct.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fct.dto.HlsCusFctProjectConSignAprl;
import hls.core.fct.service.FctProjectConSignAprlService;
import hls.core.user.service.LoginUserInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

@Controller
public class FctProjectConSignAprlController extends BaseController {

    @Autowired
    private FctProjectConSignAprlService service;

    @Autowired
    private LoginUserInfoService loginUserInfoService;


    @RequestMapping(value = "/fct/project/con/sign/aprl/query")
    @ResponseBody
    public ResponseData query(HlsCusFctProjectConSignAprl dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/fct/project/con/sign/aprl/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusFctProjectConSignAprl> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/fct/project/con/sign/aprl/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusFctProjectConSignAprl> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }


    @RequestMapping(value = "/fct/contract/sign/query")
    @ResponseBody
    public ResponseData signQuery(HlsCusFctProjectConSignAprl dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);

        List<HlsCusFctProjectConSignAprl> hlsCusFctProjectConSignAprlList =  service.fctProjectConSignAprlDetailQuery(dto);
        List<HlsCusFctProjectConSignAprl> fctProjectConSignAprlList = new ArrayList<HlsCusFctProjectConSignAprl>();
        if(hlsCusFctProjectConSignAprlList.size()>0) {
            fctProjectConSignAprlList.add(hlsCusFctProjectConSignAprlList.get(0));
        }
        return new ResponseData(fctProjectConSignAprlList);
    }

    @RequestMapping(value = "/fct/contract/sign/history/query")
    @ResponseBody
    public ResponseData signHistoryQuery(HlsCusFctProjectConSignAprl dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize,HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        //String applyName = loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).get(0).getUserName();
        String applyName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            applyName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }

        List<HlsCusFctProjectConSignAprl> hlsCusFctProjectConSignAprlList =  service.fctProjectConSignAprlDetailQuery(dto);
        List<HlsCusFctProjectConSignAprl> fctProjectConSignAprlList = new ArrayList<HlsCusFctProjectConSignAprl>();
        for(int i=0;i<hlsCusFctProjectConSignAprlList.size();i++) {
            hlsCusFctProjectConSignAprlList.get(i).setApplyName(applyName);
            fctProjectConSignAprlList.add(hlsCusFctProjectConSignAprlList.get(i));
        }
        return new ResponseData(fctProjectConSignAprlList);
    }
}