package hls.core.fct.controllers;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.hand.hap.attachment.exception.FileReadIOException;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.exception.TokenException;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fct.dto.*;
import hls.core.fct.service.*;
import hls.core.fnd.dto.DocFileTempletRule;
import hls.core.fnd.dto.HlsDocFileTemplet;
import hls.core.fnd.mapper.DocFileTempletRuleMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.fnd.service.HlsDocFileTempletService;
import hls.core.hls.components.Datasource2Json;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.service.HlsCreditLineService;
import hls.core.ruleengine.service.IHLSRuleEngineInitService;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Controller
public class FctProjectController extends BaseController {
    public static String DOCUMENT_CATEGORY = "FCT_PROJECT";
    public static String DOCUMENT_TYPE = "STD";
    public static String BUSINESS_TYPE = "FACTORING";

    @Autowired
    private FctProjectService service;
    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;
    @Autowired
    private FctProjectQuotationService fctProjectQuotationService;
    @Autowired
    private FctProjectFinStatementService fctProjectFinStatementService;
    @Autowired
    private FctProjectBpService fctProjectBpService;

    @Autowired
    private FctContractService fctContractService;
    @Autowired
    private DocFileTempletRuleMapper docFileTempletRuleMapper;
    @Autowired
    private Datasource2Json datasource2Json;

    @Autowired
    private IHLSRuleEngineInitService hLSRuleEngineInitService;
    @Autowired
    private HlsDocFileTempletService hlsDocFileTempletService;

    @Autowired
    private FctContractDocxService contractDocxService;
    @Autowired
    private FctProjectService fctProjectService;
    @Autowired
    private HlsCreditLineService hlsCreditLineService;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;


    @RequestMapping(value = "/fct/setup/project/detail/query")
    @ResponseBody
    public ResponseData creditLineDetailQuery(FctProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, @RequestParam Long projectId, final HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        if (projectId == null) {
            projectId = 0L;
        }
        dto.setProjectId(projectId);
        dto.setCompanyId(requestContext.getCompanyId());
        return new ResponseData(service.selectSetupProjectDetail(dto, page, pageSize));
    }


    @RequestMapping(value = "/fct/setup/project/query")
    @ResponseBody
    public ResponseData currentGetAmountInfo(final FctProject fctProject, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                             @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, final HttpServletRequest request) throws ParseException {
        IRequest requestContext = createRequestContext(request);
        fctProject.setCompanyId(requestContext.getCompanyId());
        List<FctProject> list = service.selectAllSetupProject(requestContext, fctProject, page, pageSize);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/hls/prj/statisticalChart/query")
    @ResponseBody
    public List<FctProject> statisticalChart(final FctProject fctProject, HttpSession session) {
        fctProject.setCompanyId((Long) session.getAttribute("companyId"));
        List<FctProject> clist = service.selectPrjStatusInfo(fctProject, session);
        return clist;
    }

    @RequestMapping(value = "/fct/project/query")
    @ResponseBody
    public ResponseData query(HlsCusFctProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        dto.setCompanyId(requestContext.getCompanyId());
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/fct/projectCredit/query")
    @ResponseBody
    public ResponseData queryProjectCreditInfo(HlsCusFctProject dto, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        dto.setCompanyId(requestContext.getCompanyId());
        return new ResponseData(service.queryFctCreditInfo(dto));
    }


    private void fctProjectValidate(HlsCusFctProjectInfo fctProjectInfo) {
        HlsCusFctProject hlsCusFctProject = fctProjectInfo.getHlsCusFctProject();
        if ("APPROVING".equals(hlsCusFctProject.getProjectStatus()) || "APPROVED".equals(hlsCusFctProject.getProjectStatus())) {
            throw new IllegalArgumentException("状态错误，无法提交！");
        }
        //TODO 必填校验
    }


    @RequestMapping(value = "/fct/project/submit")
    @ResponseBody
    public List<HlsCusFctProject> update(@RequestBody(required = false) HlsCusFctProjectInfo dto, HttpServletRequest request) {
        //状态校验
        fctProjectValidate(dto);
        HlsCusHlsCreditLine hlsCusHlsCreditLine = dto.getHlsCusHlsCreditLine();
        HlsCusFctProject fctProject = dto.getHlsCusFctProject();
        HlsCusFctProjectQuotation fctProjectQuotation = dto.getHlsCusFctProjectQuotation();
        HlsCusFctProjectFinStatement fctProjectFinStatement = dto.getHlsCusFctProjectFinStatement();
        List<HlsCusFctProjectBp> fctProjectBpList = dto.getHlsCusFctProjectBpList();
        IRequest requestCtx = createRequestContext(request);
        List<HlsCusHlsCreditLine> hlsCreditLineList = new ArrayList<>();
        hlsCreditLineList.add(hlsCusHlsCreditLine);
        List<HlsCusFctProject> fctProjectList = new ArrayList<HlsCusFctProject>();
        fctProjectList.add(fctProject);
        List<HlsCusFctProjectQuotation> fctProjectQuotationList = new ArrayList<HlsCusFctProjectQuotation>();
        fctProjectQuotationList.add(fctProjectQuotation);
        List<HlsCusFctProjectFinStatement> fctProjectFinStatementList = new ArrayList<HlsCusFctProjectFinStatement>();
        fctProjectFinStatementList.add(fctProjectFinStatement);
        if (dto != null) {
            if (hlsCreditLineList.get(0).getCreditLineId() == 0) {
                for (HlsCusHlsCreditLine dt : hlsCreditLineList) {
                    dt.set__status("add");
                    dt.setCreditLineStatus("NEW");
                    dt.setCompanyId(requestCtx.getCompanyId());
                    Map<String, String> params = new HashMap<String, String>();
                    dt.setCreditLineNumber(fndCodingRuleValuesService.getCodeRuleValue(requestCtx, dt.getDocumentCategory(), dt.getDocumentType(), dt.getBusinessType(), params));
                }
                hlsCreditLineList = hlsCreditLineService.batchUpdate(requestCtx, hlsCreditLineList);
            } else if (hlsCreditLineList.get(0).getCreditLineId() != 0 && hlsCreditLineList.get(0).getBusinessType().equals("FACTORING")) {
                for (HlsCusHlsCreditLine dt : hlsCreditLineList) {
                    dt.set__status("update");
                }
                hlsCreditLineList = hlsCreditLineService.batchUpdate(requestCtx, hlsCreditLineList);
            }
            if (fctProjectList.size() != 0) {
                for (HlsCusFctProject dt : fctProjectList) {
                    dt.setCreditLineId(hlsCreditLineList.get(0).getCreditLineId());
                    if (dt.getProjectId() == null || dt.getProjectId() == 0) {
                        //add by syj 创建时 赋初始值
                        dt.setContractStatus("NEW");
                        dt.setCreditStatus("NEW");
                        //end add
                        dt.set__status("add");
                        dt.setProjectStatus("NEW");
                        dt.setCurrency(hlsCreditLineList.get(0).getCurrencyCode());
                        BUSINESS_TYPE = dt.getBusinessType();
                        DOCUMENT_TYPE = dt.getDocumentType();
                        dt.setCompanyId(requestCtx.getCompanyId());
                        dt.setDocumentCategory(DOCUMENT_CATEGORY);
                        Map<String, String> params = new HashMap<String, String>();
                        dt.setProjectNumber(fndCodingRuleValuesService.getCodeRuleValue(requestCtx, DOCUMENT_CATEGORY, DOCUMENT_TYPE, BUSINESS_TYPE, params));
                        //插入附件清单
                        service.hlsCreditAttachmentCreate(requestCtx, dto.getHlsCusFctProject());
                    } else {
                        dt.set__status("update");
                    }
                }
            }
            boolean isAddQuotation = true;
            boolean isAddStatement = true;
            if (fctProject.getProjectId() == null || fctProject.getProjectId() == 0) {
                isAddQuotation = true;
                isAddStatement = true;
            } else {
                isAddQuotation = false;
                isAddStatement = false;
            }
            fctProjectList = service.batchUpdate(requestCtx, fctProjectList);
            if (fctProjectList.get(0).get__status() == "add") {
                fctProjectList.get(0).setDocumentCategory(DOCUMENT_CATEGORY);
                service.addFctProjectEvent(requestCtx, fctProjectList.get(0));
            } else if (fctProjectList.get(0).get__status() == "update") {
                fctProjectList.get(0).setDocumentCategory(DOCUMENT_CATEGORY);
                service.updateFctProjectEvent(requestCtx, fctProjectList.get(0));
            }
            if (fctProjectQuotation != null) {
                for (HlsCusFctProjectQuotation dt : fctProjectQuotationList) {
                    if (dt != null) {
                        if ((dt.getQuotataionId() == null || dt.getQuotataionId() == 0) && (isAddQuotation == true)) {
                            dt.set__status("add");
                            dt.setProjectId(fctProjectList.get(0).getProjectId());
                        } else {
                            dt.set__status("update");
                        }
                    }
                }
                fctProjectQuotationList = fctProjectQuotationService.batchUpdate(requestCtx, fctProjectQuotationList);
            }
            if (fctProjectFinStatement != null) {
                for (HlsCusFctProjectFinStatement dt : fctProjectFinStatementList) {
                    if (dt != null) {
                        if ((dt.getFinStatementId() == null || dt.getFinStatementId() == 0) && (isAddStatement == true)) {
                            dt.set__status("add");
                            dt.setProjectId(fctProjectList.get(0).getProjectId());
                        } else {
                            dt.set__status("update");
                        }
                    }
                }
                fctProjectFinStatementList = fctProjectFinStatementService.batchUpdate(requestCtx, fctProjectFinStatementList);
            }

            if (fctProjectBpList != null) {
                for (HlsCusFctProjectBp bp : fctProjectBpList) {
                    if (bp != null) {
                        if ((bp.getProjectBpId() == null || bp.getProjectBpId() == 0)) {
                            bp.set__status("add");
                            bp.setProjectId(fctProjectList.get(0).getProjectId());
                        } else {
                            bp.set__status("update");
                        }
                    }
                }
                fctProjectBpService.batchUpdate(requestCtx, fctProjectBpList);
            }

        }
        for (HlsCusFctProject dt : fctProjectList) {
            dt.setQuotationId(fctProjectQuotationList.get(0).getQuotataionId());
            dt.setFinStatementId(fctProjectFinStatementList.get(0).getFinStatementId());
        }


        return fctProjectList;
    }


    @RequestMapping(value = "/fct/project/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusFctProject> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }


    @RequestMapping(value = "/fct/credit/detail/query")
    @ResponseBody
    public ResponseData creditQuery(HttpServletRequest request, HlsCusFctProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page, @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
        PageHelper.startPage(page, pagesize);
        dto.setCompanyId(createRequestContext(request).getCompanyId());
        return new ResponseData(service.fctProjectDetailQuery(dto));
    }

    @RequestMapping(value = "/fct/credit/currencyMonthAddQuery")
    @ResponseBody
    public Double currencyMonthAddQuery() {
        return service.currencyMonthAddQuery();
    }


    @RequestMapping(value = "/fct/credit/statusQuery")
    @ResponseBody
    public Map<String, Integer> statusQuery(final HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = requestContext.getCompanyId();
        return service.creditStatusQuery(companyId);
    }

    @RequestMapping(value = "/fct/credit/dynamicInfoQuery")
    @ResponseBody
    public ResponseData dynamicInfoQuery(HlsSystemNotice hlsSystemNotice, final HttpServletRequest request, @RequestParam(defaultValue = DEFAULT_PAGE) int page, @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
        PageHelper.startPage(page, pagesize);
        IRequest requestContext = createRequestContext(request);
        hlsSystemNotice.setCompanyId(requestContext.getCompanyId());
        List<HlsSystemNotice> hlist = hlsSystemNoticeService.selectFctProjectCreditNotice(hlsSystemNotice, page, pagesize);
        return new ResponseData(hlist);
    }

    @RequestMapping(value = "/fct/credit/save")
    @ResponseBody
    public ResponseData fctCreditSave(HttpServletRequest request, @RequestBody final HlsCusFctProject hlsCusFctProject) {
        try {
            IRequest iRequest = createRequestContext(request);
            service.fctCreditSave(iRequest, hlsCusFctProject);
            List<HlsCusFctProject> hlsCusFctProjectList = new ArrayList<HlsCusFctProject>();
            hlsCusFctProjectList.add(hlsCusFctProject);
            return new ResponseData(hlsCusFctProjectList);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

    @RequestMapping(value = "/fct/credit/submit")
    @ResponseBody

    public ResponseData fctCreditSubmit(HttpServletRequest request, @RequestBody final HlsCusFctProject hlsCusFctProject) {
        try {
            IRequest iRequest = createRequestContext(request);
            List<HlsCusFctProject> hlsCusFctProjectList = new ArrayList<HlsCusFctProject>();
            hlsCusFctProjectList.add(hlsCusFctProject);
            service.fctCreditSubmit(iRequest, hlsCusFctProjectList);
            return new ResponseData(hlsCusFctProjectList);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

    @RequestMapping(value = "/fct/credit/attachment")
    @ResponseBody

    public ResponseData test(HttpServletRequest request, @RequestBody final HlsCusFctProject hlsCusFctProject) {
        IRequest iRequest = createRequestContext(request);
        service.hlsCreditAttachmentCreate(iRequest, hlsCusFctProject);
        return new ResponseData();
    }


    @RequestMapping(value = "/fct/content/dynamicInfoQuery")
    @ResponseBody
    public ResponseData ContentDynamicInfoQuery(HlsSystemNotice hlsSystemNotice, final HttpServletRequest request, @RequestParam(defaultValue = DEFAULT_PAGE) int page, @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        IRequest requestContext = createRequestContext(request);
        hlsSystemNotice.setCompanyId(requestContext.getCompanyId());
        List<HlsSystemNotice> list = hlsSystemNoticeService.selectFctProjectSignNotice(hlsSystemNotice, page, pageSize);
        for(int i=0;i<list.size();i++){
            Date fctDate = list.get(i).getNotice_datetime();
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
            String dateString = formatter.format(fctDate);
            list.get(i).setNotice_datetime_differ(dateString);
            if(list.get(i).getNotice_message().length()>=17){
                list.get(i).setNotice_message(list.get(i).getNotice_message().substring(0,16)+"....");
            }
        }
        return new ResponseData(list);
    }


    @RequestMapping(value = "/fct/content/save")
    @ResponseBody
    public ResponseData fctContentSave(HttpServletRequest request, @RequestBody final HlsCusFctProject hlsCusFctProject) {
        try {
            IRequest iRequest = createRequestContext(request);
            service.fctContentSave(iRequest, hlsCusFctProject);
            List<HlsCusFctProject> hlsCusFctProjectList = new ArrayList<HlsCusFctProject>();
            hlsCusFctProjectList.add(hlsCusFctProject);
            return new ResponseData(hlsCusFctProjectList);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

    @RequestMapping(value = "/fct/content/submit")
    @ResponseBody
    public ResponseData fctContentSubmit(HttpServletRequest request, @RequestBody HlsCusFctProject hlsCusFctProject) {
        try {
            IRequest iRequest = createRequestContext(request);
            List<HlsCusFctProject> hlsCusFctProjectList = new ArrayList<HlsCusFctProject>();
            Double principalPenaltyRate = hlsCusFctProject.getPrincipalPenaltyRate();
            Double penaltyRate = hlsCusFctProject.getPenaltyRate();
            hlsCusFctProject = fctProjectService.selectByPrimaryKey(iRequest, hlsCusFctProject);
            hlsCusFctProject.setPrincipalPenaltyRate(principalPenaltyRate);
            hlsCusFctProject.setPenaltyRate(penaltyRate);
            hlsCusFctProjectList.add(hlsCusFctProject);
            service.fctContentSubmit(iRequest, hlsCusFctProjectList);
            return new ResponseData(hlsCusFctProjectList);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

    @RequestMapping(value = "/fct/project/approve/submit")
    @ResponseBody
    public ResponseData fctProjectSubmit(HttpServletRequest request, @RequestBody List<HlsCusFctProject> list) {
        try {
            IRequest requestCtx = createRequestContext(request);
            //获取界面信息先进行保存
            HlsCusFctProjectInfo fctProjectInfo = list.get(0).getFctProjectInfo();
            update(fctProjectInfo, request);
            List<HlsCusFctProject> resultList = service.submitApproval(requestCtx, list);
            return new ResponseData(resultList);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

    //生成合同文本记录
    @RequestMapping(value = "/fct/content/create")
    @ResponseBody
    public ResponseData fctContentCreate(HttpServletRequest request, @RequestBody final HlsCusFctProject hlsCusFctProject) {
        IRequest iRequest = createRequestContext(request);
        fctContractService.createContractContent(iRequest, hlsCusFctProject);
        return new ResponseData();
    }

    //生成合同文本文件
    @RequestMapping(value = "/fct/content/export")
    @ResponseBody
    public ResponseData fctContentExport(@RequestBody HlsCusFctProjectAttachment hlsCusFctProjectAttachment, final HttpServletRequest request, HttpServletResponse response, HttpSession session) throws TokenException, FileReadIOException, Docx4JException {
        IRequest requestContext = createRequestContext(request);
        String templetType = "FCT_CONTRACT_SIGN";
        Long companyId = (Long) session.getAttribute("companyId");
        DocFileTempletRule docFileTempletRule = new DocFileTempletRule();
        docFileTempletRule.setCompanyId(companyId);
        docFileTempletRule.setTempletType(templetType);
        docFileTempletRule = docFileTempletRuleMapper.selectOne(docFileTempletRule);

        HlsCusFctProject hlsCusFctProject = new HlsCusFctProject();
        hlsCusFctProject.setProjectId(hlsCusFctProjectAttachment.getProjectId());
        hlsCusFctProject = fctProjectService.selectByPrimaryKey(requestContext, hlsCusFctProject);


        String json = null;
        try {
            Map<String, Object> pMap = new HashMap<String, Object>();
            pMap.put("projectId", hlsCusFctProjectAttachment.getProjectId());
            pMap.put("projectAttachmentId", hlsCusFctProjectAttachment.getProjectAttachmentId());
            pMap.put("bpId", hlsCusFctProjectAttachment.getBpId());
            pMap.put("documentType", hlsCusFctProject.getDocumentType());
            json = datasource2Json.executeSQL4Json(docFileTempletRule.getDataSourceId(), pMap);
        } catch (IOException e) {
            e.printStackTrace();
        }
        JSONObject jsonObject0 = JSON.parseObject(json);
        JSONObject jsonObject1 = JSON.parseObject(JSON.toJSONString(jsonObject0.get(docFileTempletRule.getDataSourceId().toString())));
        JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(jsonObject1.get("default")));
        jsonObject.put("ruleEngineId", docFileTempletRule.getRuleEngineId());
        String[] arr = hLSRuleEngineInitService.ruleEngineInit(requestContext, jsonObject);

        HlsDocFileTemplet hlsDocFileTemplet = null;
        HlsCusFctProjectAttachment attachment = null;

        for (int i = 0; i < arr.length; i++) {

            //contractAttachmentService.deleteContentById(CshConContract.getContract_id());

            hlsDocFileTemplet = new HlsDocFileTemplet();
            hlsDocFileTemplet.setTempletId(Long.parseLong(arr[i]));
            hlsDocFileTemplet = hlsDocFileTempletService.selectByPrimaryKey(requestContext, hlsDocFileTemplet);

            Map<String, Object> params = new HashMap<String, Object>();

            params.put("projectId", hlsCusFctProjectAttachment.getProjectId());
            params.put("templetId", hlsDocFileTemplet.getTempletId());
            params.put("bpId", hlsCusFctProjectAttachment.getBpId());
            params.put("projectAttachmentId", hlsCusFctProjectAttachment.getProjectAttachmentId());
            contractDocxService.process(requestContext, params, jsonObject);
        }


        return new ResponseData();
    }


    @RequestMapping(value = "/fct/sign/submit")
    @ResponseBody
    public ResponseData fctSignSubmit(HttpServletRequest request, @RequestBody final HlsCusFctProject hlsCusFctProject) {
        try {
            IRequest iRequest = createRequestContext(request);
            List<HlsCusFctProject> hlsCusFctProjectList = new ArrayList<HlsCusFctProject>();
            hlsCusFctProjectList.add(hlsCusFctProject);
            service.fctSignSubmit(iRequest, hlsCusFctProjectList);
            return new ResponseData(hlsCusFctProjectList);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

}