package hls.core.fct.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fct.dto.HlsCusHlsCreditLineH;
import hls.core.fct.service.HlsCreditLineHService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

@Controller
public class HlsCreditLineHController extends BaseController {

    @Autowired
    private HlsCreditLineHService service;


    @RequestMapping(value = "/hls/credit/line/h/query")
    @ResponseBody
    public ResponseData query(HlsCusHlsCreditLineH dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }
    @RequestMapping(value = "/hls/credit/line/history/query")
    @ResponseBody
    public ResponseData queryCreditLineHistory(Long creditLineId) {
        HlsCusHlsCreditLineH hlsCreditLineH=new HlsCusHlsCreditLineH();
        hlsCreditLineH.setCreditLineId(creditLineId);
        return new ResponseData(service.queryCreditLineHistory(hlsCreditLineH));
    }

    @RequestMapping(value = "/hls/credit/line/h/query/info")
    @ResponseBody
    public ResponseData queryCreditLineH(HlsCusHlsCreditLineH dto) {
        return new ResponseData(service.queryCreditLineH(dto));
    }

    @RequestMapping(value = "/hls/credit/line/h/queryVersion")
    @ResponseBody
    public ResponseData queryVersion(Long creditLineId, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.queryVersion(creditLineId));
    }

    @RequestMapping(value = "/hls/credit/line/h/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusHlsCreditLineH> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        if (dto != null) {
            for (HlsCusHlsCreditLineH dt : dto) {
                if (dt.getCreditLineId() == null || dt.getCreditLineId() == 0) {
                    dt.set__status("add");
                    dt.setCreditLineStatus("APPROVING");
                } else {
                    dt.set__status("update");
                }
            }
        }
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/hls/credit/line/h/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusHlsCreditLineH> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/hls/credit/line/h/change/submit")
    @ResponseBody
    public ResponseData changeSubmit(HttpServletRequest request, @RequestBody List<HlsCusHlsCreditLineH> dto) {
        try {
            IRequest requestCtx = createRequestContext(request);
            List<HlsCusHlsCreditLineH> result = service.submitApproval(requestCtx,dto);
            return new ResponseData(result);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }

    @RequestMapping(value = "/hls/credit/line/h/change/history/query")
    @ResponseBody
    public ResponseData changeHistoryQuery(HttpServletRequest request, HlsCusHlsCreditLineH dto) {
        return new ResponseData(service.queryHistoryByStatus(dto));
    }
    @RequestMapping(value = "/hls/credit/line/h/change/seq/query")
    @ResponseBody
    public ResponseData  getChangeSeq(Long creditLineId) {
        HlsCusHlsCreditLineH hlsCreditLineH=new HlsCusHlsCreditLineH();
        hlsCreditLineH.setCreditLineId(service.getChangeSeq(creditLineId));
        List<HlsCusHlsCreditLineH> list=new ArrayList<>();
        list.add(hlsCreditLineH);
        return new ResponseData(list);
    }
    @RequestMapping(value = "/hls/credit/line/h/change/data/query")
    @ResponseBody
    public ResponseData  queryChangeData(HttpServletRequest request,Long creditLineId) {
        IRequest requestCtx = createRequestContext(request);
        HlsCusHlsCreditLineH hlsCreditLineH=new HlsCusHlsCreditLineH();
        hlsCreditLineH.setCreditLineId(creditLineId);
        return new ResponseData(service.queryChangeData(requestCtx,hlsCreditLineH));
    }

}