package hls.core.fct.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.csh.dto.HlsCusCshPaymentReqHd;
import hls.core.csh.dto.HlsCusCshPaymentReqLn;
import hls.core.csh.mapper.HlsCusCshPaymentReqHdMapper;
import hls.core.csh.service.ICshPaymentReqHdService;
import hls.core.csh.service.ICshPaymentReqLnService;
import hls.core.fct.dto.HlsCusFctContract;
import hls.core.fct.dto.HlsCusFctContractWithdraw;
import hls.core.fct.dto.HlsCusFctContractWithdrawCf;
import hls.core.fct.mapper.HlsCusFctContractMapper;
import hls.core.fct.mapper.HlsCusFctContractWithdrawCfMapper;
import hls.core.fct.mapper.HlsCusFctContractWithdrawMapper;
import hls.core.fct.service.FctContractPaymentService;
import hls.core.fct.service.FctContractWithdrawService;
import hls.core.fnd.dto.HLSCurrency;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.*;

/**
 * @Author: syj
 * @Description:
 * @Date: Created in 2017/12/6 11:20
 * @Modified By:
 */
@Service
@Transactional
public class FctContractPaymentServiceImpl extends BaseServiceImpl<HlsCusFctContractWithdraw> implements FctContractPaymentService {
    /**
     * 生成付款申请
     */

    @Autowired
    private FndCodingRuleValuesService codingRuleValuesService;

    @Autowired
    private ICshPaymentReqHdService paymentReqHdService;
    @Autowired
    private ICshPaymentReqLnService cshPaymentReqLnService;

    @Autowired
    private HlsCusFctContractMapper hlsCusFctContractMapper;

    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;

    @Autowired
    private SysEventService sysEventService;

    @Autowired
    private HlsCusCshPaymentReqHdMapper hlsCusCshPaymentReqHdMapper;

    @Autowired
    private HlsCusFctContractWithdrawMapper hlsCusFctContractWithdrawMapper;

    @Autowired
    private HlsCusFctContractWithdrawCfMapper hlsCusFctContractWithdrawCfMapper;

    @Autowired
    private IActivitiStartService activitiStartService;

    @Autowired
    private FctContractWithdrawService fctContractWithdrawService;

    public static final String PROPERTY_FCT_PAYMENT_REQUEST_SUBMIT = "FCT_PAYMENT_REQUEST.SUBMIT";


    public String getCodeValue(IRequest requestContext) {
        Map<String, String> params = new HashMap<String, String>();
        String value = codingRuleValuesService.getCodeRuleValue(requestContext, "CSH_PAYMENT_REQ", "PAYMENT_REQ", "PAYMENT_REQ", params);
        return value;
    }


    private HlsCusCshPaymentReqHd addCshPaymentReq(HlsCusFctContractWithdraw withdraw, Long companyId, String value, IRequest request) {

        HlsCusCshPaymentReqHd cshPaymentReqHd = new HlsCusCshPaymentReqHd();
        HashMap<String, Object> map = new HashMap<>(5);
        map.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_PAYMENT_REQ_CREATE);
        map.put("currency", "");
        map.put("amount", "");

        Double amLong = withdraw.getWriteOffAmount(); // 申请付款金额
        Long bp_id = withdraw.getBpId(); // 付款对象商业伙伴ID


        cshPaymentReqHd.setCompany_id(companyId);
        cshPaymentReqHd.setPayment_req_id(null);

        cshPaymentReqHd.setDocument_type("FCT_PAYMENT_REQ");
        cshPaymentReqHd.setDocument_category("CSH_PAYMENT_REQ");
        cshPaymentReqHd.setBusiness_type("PAYMENT_REQ");// 业务模式
        cshPaymentReqHd.setPayment_req_number(value); // 付款申请编号
        cshPaymentReqHd.setPayment_req_date(withdraw.getApplyPayDate());
        cshPaymentReqHd.setPayment_req_status("NEW");
        cshPaymentReqHd.setApply_pay_date(new Date());
        cshPaymentReqHd.setCurrency("CNY");
        cshPaymentReqHd.setObjectVersionNumber(new Long("1"));
        cshPaymentReqHd.setLastUpdatedBy(request.getUserId());
        cshPaymentReqHd.setLastUpdateDate(new Date());
        cshPaymentReqHd.setAmount(amLong);
        paymentReqHdService.insertSelective(request, cshPaymentReqHd);


        HlsCusCshPaymentReqLn cshPaymentReqLn = new HlsCusCshPaymentReqLn();

        cshPaymentReqLn.setPayment_req_id(cshPaymentReqHd.getPayment_req_id());
        cshPaymentReqLn.setSource_doc_category("FCT_CONTRACT_WITHDRAW");
        cshPaymentReqLn.setSource_doc_id(withdraw.getWithdrawId());
        cshPaymentReqLn.setSource_doc_line_id(null);//审批通过生成outflow现金流 回写这个字段

        cshPaymentReqLn.setAmount(amLong);
        cshPaymentReqLn.setDescription("");
        cshPaymentReqLn.setPayment_flag("NOT");
//        cshPaymentReqLn.setPayment_method(withdraw.getPaymentMethod());
        cshPaymentReqLn.setBp_id(bp_id);
        cshPaymentReqLn.setBp_bank_account_id(withdraw.getBankAccountId());
        cshPaymentReqLn.setBp_bank_account_name(withdraw.getBankBranchName());
        cshPaymentReqLn.setBp_bank_account_num(withdraw.getBankAccountNum());
        cshPaymentReqLn.setBp_bank_branch_name(withdraw.getBankBranchName());
        cshPaymentReqLn.setBp_bank_name(withdraw.getBankFullName());
        cshPaymentReqLn.setDescription(withdraw.getDescription());

        map.put("amount", amLong);
        Long contract_id = withdraw.getContractId();
        HlsCusFctContract conContract = new HlsCusFctContract();
        conContract.setContractId(contract_id);
        conContract = hlsCusFctContractMapper.selectByPrimaryKey(conContract);
        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(conContract.getCurrency());
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);
        map.put("currency", hlsCurrency.getCurrencySymbol());

        //sysEventService.createEvent(request, contract_id, cshPaymentReqLn.getSource_doc_category(), cshPaymentReqLn.getDocumentType(), map);
        cshPaymentReqLnService.insertSelective(request, cshPaymentReqLn);
        cshPaymentReqHd.setPaymentReqLnId(cshPaymentReqLn.getPayment_req_ln_id());
        cshPaymentReqHd.setWithdrawId(withdraw.getWithdrawId());
        return cshPaymentReqHd;
    }


    private HlsCusCshPaymentReqHd updateCshPaymentReq(HlsCusFctContractWithdraw withdraw, Long companyId, IRequest request, Long hdId, Long lnId) {
        HlsCusCshPaymentReqHd cshPaymentReqHd = new HlsCusCshPaymentReqHd();
        HashMap<String, Object> map = new HashMap<>(5);
        map.put("eventCode", SysEventCodeUtil.PROPERTY_CSH_PAYMENT_REQ_CREATE);
        map.put("currency", "");
        map.put("amount", "");

        Double amLong = withdraw.getWriteOffAmount(); // 申请付款金额
        Long bp_id = withdraw.getBpId(); // 付款对象商业伙伴ID

        cshPaymentReqHd.setPayment_req_id(hdId);
        cshPaymentReqHd = hlsCusCshPaymentReqHdMapper.selectByPrimaryKey(cshPaymentReqHd);

        cshPaymentReqHd.setCompany_id(companyId);
        cshPaymentReqHd.setPayment_req_date(withdraw.getApplyPayDate());
        cshPaymentReqHd.setPayment_req_status("NEW");
        cshPaymentReqHd.setApply_pay_date(new Date());
        cshPaymentReqHd.setCurrency("CNY");
        cshPaymentReqHd.setLastUpdatedBy(request.getUserId());
        cshPaymentReqHd.setLastUpdateDate(new Date());
        cshPaymentReqHd.setAmount(amLong);
        paymentReqHdService.updateByPrimaryKeySelective(request, cshPaymentReqHd);

        HlsCusCshPaymentReqLn cshPaymentReqLn = new HlsCusCshPaymentReqLn();
        cshPaymentReqLn.setPayment_req_ln_id(lnId);
        cshPaymentReqLn.setPayment_req_id(cshPaymentReqHd.getPayment_req_id());
        cshPaymentReqLn.setSource_doc_category("FCT_CONTRACT_WITHDRAW");
        cshPaymentReqLn.setSource_doc_id(withdraw.getWithdrawId());
        cshPaymentReqLn.setSource_doc_line_id(null);
        cshPaymentReqLn.setAmount(amLong);
        cshPaymentReqLn.setDescription("");// 输入的申请原因
        cshPaymentReqLn.setPayment_flag("NOT");
//        cshPaymentReqLn.setPayment_method(withdraw.getPaymentMethod());
        cshPaymentReqLn.setBp_id(bp_id);
        cshPaymentReqLn.setBp_bank_account_id(withdraw.getBankAccountId());
        cshPaymentReqLn.setBp_bank_account_name(withdraw.getBankBranchName());//账户名
        cshPaymentReqLn.setBp_bank_account_num(withdraw.getBankAccountNum());
        cshPaymentReqLn.setBp_bank_branch_name(withdraw.getBankBranchName());//银行分行
        cshPaymentReqLn.setBp_bank_name(withdraw.getBankFullName());
        cshPaymentReqLn.setDescription(withdraw.getDescription());
        cshPaymentReqLn.setPayment_method(withdraw.getPaymentMethod());


        map.put("amount", amLong);
        Long contract_id = withdraw.getContractId();
        HlsCusFctContract conContract = new HlsCusFctContract();
        conContract.setContractId(contract_id);
        conContract = hlsCusFctContractMapper.selectByPrimaryKey(conContract);
        HLSCurrency hlsCurrency = new HLSCurrency();
        hlsCurrency.setCurrencyCode(conContract.getCurrency());
        hlsCurrency = hlsCurrencyMapper.selectOne(hlsCurrency);
        map.put("currency", hlsCurrency.getCurrencySymbol());

       // sysEventService.createEvent(request, contract_id, cshPaymentReqLn.getSource_doc_category(), cshPaymentReqLn.getDocumentType(), map);
        cshPaymentReqLnService.updateByPrimaryKeySelective(request, cshPaymentReqLn);
        cshPaymentReqHd.setPaymentReqLnId(cshPaymentReqLn.getPayment_req_ln_id());
        cshPaymentReqHd.setWithdrawId(withdraw.getWithdrawId());
        return cshPaymentReqHd;
    }

    @Override
    public HlsCusCshPaymentReqHd payRequestSave(HlsCusFctContractWithdraw hlsCusFctContractWithdraw, IRequest iRequest) throws ParseException {
        HlsCusCshPaymentReqHd cshPaymentReqHd = new HlsCusCshPaymentReqHd();

        String value = getCodeValue(iRequest);

        if (hlsCusFctContractWithdraw == null) {
            return null;
        }

        if (hlsCusFctContractWithdraw.getPaymentReqId() == null) {
            cshPaymentReqHd = addCshPaymentReq(hlsCusFctContractWithdraw, iRequest.getCompanyId(), value, iRequest);

        } else {
            cshPaymentReqHd = updateCshPaymentReq(hlsCusFctContractWithdraw, iRequest.getCompanyId(), iRequest, hlsCusFctContractWithdraw.getPaymentReqId(), hlsCusFctContractWithdraw.getPaymentReqLnId());
        }

        return cshPaymentReqHd;
    }


    public void validate(HlsCusFctContractWithdraw hlsCusFctContractWithdraw) {


        //收息方式
        /**
         COLLECT_TOTAL_INTEREST_FIRST     全部先收（先付）
         COLLECT_PARTIAL_INTEREST_FIRST   部分先收（先付）
         PER_MONTH                        每月日（后付）
         PER_QUARTER                      每季日（后付）
         ALL_REPAY_CAPITAL_WITH_INTEREST_AT_MATURITY  到期一次还本付息（后付）
         */
        String interestCollectionMethod = hlsCusFctContractWithdraw.getInterestCollectionMethod();
        //先付，现将第0期的利息全部核销，才能付款申请
        if ("COLLECT_TOTAL_INTEREST_FIRST".equals(interestCollectionMethod) || "COLLECT_PARTIAL_INTEREST_FIRST".equals(interestCollectionMethod)) {
            HlsCusFctContractWithdrawCf cf = new HlsCusFctContractWithdrawCf();
            cf.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());
            cf.setCfItem(501L);
            cf.setTimes(0L);
            cf = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(cf).get(0);
            if (!"FULL".equals(cf.getWriteOffFlag())) {
                throw new IllegalArgumentException("先付时，第0期利息需全部核销才能发起付款申请。");
            }
        }

    }

    @Override
    public void cshPaymentReqSubmit(IRequest iRequest, HlsCusFctContractWithdraw hlsCusFctContractWithdraw) throws ParseException {
        validate(hlsCusFctContractWithdraw);
        Long paymentReqId= hlsCusFctContractWithdraw.getPaymentReqId();
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_PAYMENT_REQUEST");
        params.put("fctContractWithdraw", hlsCusFctContractWithdraw);
        params.put("paymentReqId", hlsCusFctContractWithdraw.getPaymentReqId());
        params.put("paymentReqLnId", hlsCusFctContractWithdraw.getPaymentReqLnId());
        List<HlsCusFctContractWithdraw> hlsCusFctContractWithdrawList = new ArrayList<>();
        hlsCusFctContractWithdrawList.add(hlsCusFctContractWithdraw);

        hlsCusFctContractWithdraw = hlsCusFctContractWithdrawMapper.selectByPrimaryKey(hlsCusFctContractWithdraw.getWithdrawId());

        //2.插入事件
        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "保理提款" + hlsCusFctContractWithdraw.getWithdrawName() + "提交付款申请");
        evenParams.put("noticeTitle", "保理付款申请审批");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(iRequest, hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getDocumentCategory(), hlsCusFctContractWithdraw.getDocumentType(), "FCT", PROPERTY_FCT_PAYMENT_REQUEST_SUBMIT, "P2D", evenParams);

        activitiStartService.start(iRequest, hlsCusFctContractWithdrawList, params);
        HlsCusCshPaymentReqHd hd = new HlsCusCshPaymentReqHd();
        hd.setPayment_req_id(paymentReqId);
        hd = paymentReqHdService.selectByPrimaryKey(iRequest, hd);
        hd.setPayment_req_status("APPROVING");
        paymentReqHdService.updateByPrimaryKeySelective(iRequest, hd);

        hlsCusFctContractWithdraw.setWithdrawStatus("PROCESS_IN_PAYMENT_REQUEST");
        fctContractWithdrawService.updateByPrimaryKeySelective(iRequest,hlsCusFctContractWithdraw);


    }

    @Override
    public List<HlsCusCshPaymentReqHd> fctPaymentReqHistorySelect(HlsCusCshPaymentReqHd cshPaymentReqHd) {
        return hlsCusCshPaymentReqHdMapper.fctPaymentReqHistorySelect(cshPaymentReqHd);
    }
}

