package hls.core.fct.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.mapper.HlsCusCshWriteOffMapper;
import hls.core.fct.dto.*;
import hls.core.fct.mapper.*;
import hls.core.fct.service.*;
import hls.core.fnd.dto.FndCompany;
import hls.core.fnd.mapper.FndCompanyMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.service.HlsBeanRefUtilService;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.service.HlsCreditLineService;
import hls.core.sys.event.service.SysEventService;
import hls.core.user.service.LoginUserInfoService;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.temporal.ChronoUnit;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class FctContractServiceImpl extends BaseServiceImpl<HlsCusFctContract> implements FctContractService {
    @Autowired
    private FctProjectService fctProjectService;
    @Autowired
    IActivitiStartService activitiStartService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private HlsBeanRefUtilService hlsBeanRefUtilService;
    @Autowired
    private FctProjectQuotationService fctProjectQuotationService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private FctContractQuotationService fctContractQuotationService;
    @Autowired
    private FctProjectBpService fctProjectBpService;
    @Autowired
    private FctProjectAttachmentService fctProjectAttachmentService;
    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;
    @Autowired
    private FndCompanyMapper fndCompanyMapper;
    @Autowired
    private FctContractBpService fctContractBpService;
    @Autowired
    private HlsCreditLineService hlsCreditLineService;

    @Autowired
    private HlsCusFctContractMapper hlsCusFctContractMapper;
    @Autowired
    private HlsCusFctContractWithdrawMapper hlsCusFctContractWithdrawMapper;
    @Autowired
    private HlsCusFctContractWithdrawCfMapper hlsCusFctContractWithdrawCfMapper;

    @Autowired
    private HlsCusFctContractQuotationMapper hlsCusFctContractQuotationMapper;

    @Autowired
    private FctContractWithdrawCfService fctContractWithdrawCfService;
    @Autowired
    private HlsCusFctContractWithdrawAprlMapper hlsCusFctContractWithdrawAprlMapper;
    @Autowired
    private HlsCusFctProjectAttachmentMapper fctProjectAttachmentMapper;
    @Autowired
    private HlsCusCshWriteOffMapper hlsCusCshWriteOffMapper;
    @Autowired
    private LoginUserInfoService loginUserInfoService;


    public static final String PROPERTY_FCT_CONTRACT_CREATE = "FCT_CONTRACT.CREATE";
    public static final String PROPERTY_FCT_CONTRACT_END_SUBMIT = "FCT_CONTRACT_END.SUBMIT";

    public int getMonth(Date start, Date end) {
        if (start.after(end)) {
            Date t = start;
            start = end;
            end = t;
        }
        Calendar startCalendar = Calendar.getInstance();
        startCalendar.setTime(start);
        Calendar endCalendar = Calendar.getInstance();
        endCalendar.setTime(end);
        Calendar temp = Calendar.getInstance();
        temp.setTime(end);
        temp.add(Calendar.DATE, 1);

        int year = endCalendar.get(Calendar.YEAR) - startCalendar.get(Calendar.YEAR);
        int month = endCalendar.get(Calendar.MONTH) - startCalendar.get(Calendar.MONTH);

        if ((startCalendar.get(Calendar.DATE) == 1) && (temp.get(Calendar.DATE) == 1)) {
            return year * 12 + month + 1;
        } else if ((startCalendar.get(Calendar.DATE) != 1) && (temp.get(Calendar.DATE) == 1)) {
            return year * 12 + month;
        } else if ((startCalendar.get(Calendar.DATE) == 1) && (temp.get(Calendar.DATE) != 1)) {
            return year * 12 + month;
        } else {
            return (year * 12 + month - 1) < 0 ? 0 : (year * 12 + month);
        }
    }

    /**
     * 合同签约 点击合同文本生成
     *
     * @param iRequest
     * @param hlsCusFctProject
     */
    @Override
    public void createContractContent(IRequest iRequest, HlsCusFctProject hlsCusFctProject) {
        Long projectId = hlsCusFctProject.getProjectId();

        List<HlsCusFctProject> hlsCusFctProjectList = fctProjectService.fctProjectDetailQuery(hlsCusFctProject);
        hlsCusFctProject = hlsCusFctProjectList.get(0);

        HlsCusFctProjectQuotation fctProjectQuotation = new HlsCusFctProjectQuotation();
        fctProjectQuotation.setProjectId(projectId);
        fctProjectQuotation = fctProjectQuotationService.fctProjectQuotationDetailQuery(fctProjectQuotation).get(0);

        //根据 卖方和业务类型 生成  保理合同
        String documentType = hlsCusFctProject.getDocumentType();
        HlsCusFctProjectBp fctProjectBp = new HlsCusFctProjectBp();
        fctProjectBp.setProjectId(projectId);
        List<HlsCusFctProjectBp> fctProjectBpList = fctProjectBpService.fctProjectBpQuery(fctProjectBp, 1, 999999);

        String documentNumber = "";
        String documentName = "";

        HlsCusFctProjectAttachment projectAttachment = new HlsCusFctProjectAttachment();
        projectAttachment.setProjectId(hlsCusFctProject.getProjectId());
        projectAttachment.setSourceType("FCT_CONTRACT");
        List<HlsCusFctProjectAttachment> projectAttachmentList1 = fctProjectAttachmentService.fctProjectAttachmentQuery(projectAttachment, 1, 999999);
        projectAttachment.setSourceType("FCT_GUARANTOR");
        List<HlsCusFctProjectAttachment> projectAttachmentList2 = fctProjectAttachmentService.fctProjectAttachmentQuery(projectAttachment, 1, 999999);
        projectAttachment.setSourceType("FCT_REPURCHASE");
        List<HlsCusFctProjectAttachment> projectAttachmentList3 = fctProjectAttachmentService.fctProjectAttachmentQuery(projectAttachment, 1, 999999);
        List<HlsCusFctProjectAttachment> list = new ArrayList<HlsCusFctProjectAttachment>();

        list.addAll(projectAttachmentList1);
        list.addAll(projectAttachmentList2);
        list.addAll(projectAttachmentList3);
        fctProjectAttachmentService.batchDelete(list);
        //一旦生成不在重复生成
        /*projectAttachment.setProjectId(hlsCusFctProject.getProjectId());
        projectAttachment.setSourceType("FCT_CONTRACT");
        List<HlsCusFctProjectAttachment> projectAttachmentList = fctProjectAttachmentService.fctProjectAttachmentQuery(projectAttachment, 1, 999999);

        if (projectAttachmentList.size() == 0) {*/

        int factoring = 0;
        int k = 0;
        int l = 0;

        //如果是正向保理 只取一个卖方

        if ("FACTORING".equals(hlsCusFctProject.getBusinessType())) {
            HlsCusFctProjectBp bp = new HlsCusFctProjectBp();
            factoring = 1;
            for (int j = fctProjectBpList.size() - 1; j >= 0; j--) {
                if ("SELLER".equals(fctProjectBpList.get(j).getBpRoleType())) {
                    bp = fctProjectBpList.get(j);
                    fctProjectBpList.remove(j);
                }
            }
            fctProjectBpList.add(bp);
        }

        for (int i = 0; i < fctProjectBpList.size(); i++) {

            HlsCusFctProjectAttachment fctProjectAttachment = new HlsCusFctProjectAttachment();

            fctProjectAttachment.setProjectId(projectId);
            //已生成
            fctProjectAttachment.setStatus("GENERATED");

            //根据卖方个数生成保理主合同
            if ("SELLER".equals(fctProjectBpList.get(i).getBpRoleType())) {
                if ("HIDE_RECOURSE".equals(documentType) || "OPEN_RECOURSE".equals(documentType)) {
                    documentName = "有追索权国内商业保理合同";
                } else {
                    documentName = "无追索权国内商业保理合同";
                }
                k = k + 1;
                if (factoring == 1) {
                    documentNumber = hlsCusFctProject.getProjectNumber();
                } else {
                    documentNumber = hlsCusFctProject.getProjectNumber() + "-" + (k);
                }
                fctProjectAttachment.setSourceId(-1 * i + 0L);//置为- i 真正创建合同之后改为合同ID
                fctProjectAttachment.setSourceType("FCT_CONTRACT");
                fctProjectAttachment.setBpCategory("SELLER");
                fctProjectAttachment.setBpId(fctProjectBpList.get(i).getBpId());
                fctProjectAttachment.setDocumentName(documentName);
                fctProjectAttachment.setDocumentNumber(documentNumber);
                fctProjectAttachment.setCreditAmount(fctProjectQuotation.getFinanceAmount());
                fctProjectAttachment.setPerformanceTerm(getMonth(hlsCusFctProject.getValidFrom(), hlsCusFctProject.getValidTo()) + 0D);
                fctProjectAttachment.setSeller(fctProjectBpList.get(i).getBpName());
                fctProjectAttachment.setIntRate(fctProjectQuotation.getIntRate());
                fctProjectAttachment.setCurrency(hlsCusFctProject.getCurrency());

                fctProjectAttachmentService.insertSelective(iRequest, fctProjectAttachment);

                l = 0;
                //针对每一个卖方 有多少个保证人 就有多少个承诺函
                for (int j = 0; j < fctProjectBpList.size(); j++) {
                    if ("GUARANTOR".equals(fctProjectBpList.get(j).getBpRoleType())) {
                        l = l + 1;
                        if ("HIDE_RECOURSE".equals(documentType) || "OPEN_RECOURSE".equals(documentType)) {
                            documentName = "编号为" + documentNumber + "的《有追索权国内商业保理合同》的担保承诺函";
                        } else {
                            documentName = "编号为" + documentNumber + "的《无追索权国内商业保理合同》的担保承诺函";
                        }
                        if (j < 10) {
                            documentNumber = documentNumber + "-G-0" + (l);
                        } else {
                            documentNumber = documentNumber + "-G-" + (l);
                        }
                        fctProjectAttachment.setSourceType("FCT_GUARANTOR");
                        fctProjectAttachment.setBpCategory("GUARANTOR");
                        fctProjectAttachment.setSourceId(-1 * i + 0L);//置为-i 真正创建合同之后改为合同ID
                        fctProjectAttachment.setBpId(fctProjectBpList.get(j).getBpId());
                        fctProjectAttachment.setDocumentName(documentName);
                        fctProjectAttachment.setDocumentNumber(documentNumber);
                        documentNumber = hlsCusFctProject.getProjectNumber() + "-" + (k);
                        fctProjectAttachment.setSeller(fctProjectBpList.get(i).getBpName());
                        fctProjectAttachment.setIntRate(fctProjectQuotation.getIntRate());
                        fctProjectAttachment.setCurrency(hlsCusFctProject.getCurrency());
                        fctProjectAttachmentService.insertSelective(iRequest, fctProjectAttachment);
                    }
                }
                //针对无追索保理 每一个卖方
                if ("HIDE_WITHOUT_RECOURSE".equals(documentType) || "OPEN_WITHOUT_RECOURSE".equals(documentType)) {
                    documentName = "编号为" + documentNumber + "的《无追索权国内商业保理合同》的回购承诺函";
                    documentNumber = documentNumber + "-R-01";
                    fctProjectAttachment.setSourceType("FCT_REPURCHASE");
                    fctProjectAttachment.setBpCategory("SELLER");
                    fctProjectAttachment.setSourceId(-1 * i + 0L);//暂时将souceid置为 - i 真正创建合同之后改为合同ID
                    fctProjectAttachment.setBpId(fctProjectBpList.get(i).getBpId());
                    fctProjectAttachment.setDocumentName(documentName);
                    fctProjectAttachment.setDocumentNumber(documentNumber);
                    documentNumber = hlsCusFctProject.getProjectNumber() + "-" + (k);
                    fctProjectAttachment.setSeller(fctProjectBpList.get(i).getBpName());
                    fctProjectAttachment.setIntRate(fctProjectQuotation.getIntRate());
                    fctProjectAttachment.setCurrency(hlsCusFctProject.getCurrency());
                    fctProjectAttachmentService.insertSelective(iRequest, fctProjectAttachment);
                }
            }

        }
        //}

    }

    //合保理同明细
    @Override
    public List<HlsCusFctContract> fctContractDetailLineQuery(HlsCusFctContract hlsCusFctContract) {
        return hlsCusFctContractMapper.fctContractDetailLineQuery(hlsCusFctContract);
    }

    public void setAuthorityRuleString(IRequest requestContext, HlsCusFctContract fctContract , Long companyId){
        FndCompany fndCompany = new FndCompany();
        fndCompany.setCompany_id(companyId);
        fndCompany.setCompanyId(companyId);
        fndCompany = fndCompanyMapper.selectByPrimaryKey(fndCompany);
        String userName = requestContext.getUserName();
        String positionCode = requestContext.getAttribute("positionCode")==null?"":requestContext.getAttribute("positionCode");
        String unitCode = requestContext.getAttribute("unitCode")==null?"":requestContext.getAttribute("unitCode");
        fctContract.setAuthorityRuleString('"' + fndCompany.getCompany_code() + '"' + "." + '"'+unitCode+ '"' + "." + '"' + '"' + "." + '"'+'"' + "." + '"' + '"' + "." + '"' + positionCode + '"' + "." + '"' + userName + '"');
    }

    @Override
    public List<HlsCusFctContract> createContractFromProject(IRequest iRequest, HlsCusFctProject hlsCusFctProject) {

        List<HlsCusFctContract> hlsCusFctContractList = new ArrayList<HlsCusFctContract>();
        Long projectId = hlsCusFctProject.getProjectId();


        HlsCusFctProjectAttachment fctProjectAttachment = new HlsCusFctProjectAttachment();
        fctProjectAttachment.setProjectId(hlsCusFctProject.getProjectId());
        fctProjectAttachment.setSourceType("FCT_CONTRACT");
        fctProjectAttachment.setSignApprovalId(hlsCusFctProject.getSignApprovalId());
        List<HlsCusFctProjectAttachment> fctProjectAttachmentList = fctProjectAttachmentMapper.fctContractContentHistoryQuery(fctProjectAttachment);


        for (int i = 0; i < fctProjectAttachmentList.size(); i++) {

            //复制合同主表 fct_contract
            HlsCusFctProject fctProject = fctProjectService.selectByPrimaryKey(iRequest, hlsCusFctProject);
            if (fctProject == null) {
                throw new IllegalArgumentException("当前项目不存在！");
            }
            if (!"APPROVED".equals(fctProject.getCreditStatus())) {
                throw new IllegalArgumentException("当前授信未审批通过！");
            }

            databaseLockProvider.lock(fctProject);
            Map<String, String> map = hlsBeanRefUtilService.getFieldValueMap(fctProject);
            HlsCusFctContract fctContract = new HlsCusFctContract();
            hlsBeanRefUtilService.setFieldValue(fctContract, map);

            //获取编码规则
            Map<String, String> params = new HashMap<String, String>();
            fctContract.setContractStatus("SIGN");
            fctContract.setDocumentCategory("FCT_CONTRACT");
            //暂时用项目名称赋值给合同名称
            fctContract.setSignDate(hlsCusFctProject.getSignDate());
            fctContract.setSignDescription(hlsCusFctProject.getSignDescription());
            fctContract.setSignApprovalId(hlsCusFctProject.getSignApprovalId());
            fctContract.setContractName(hlsCusFctProject.getProjectName());
            fctContract.setContractNumber(fctProjectAttachmentList.get(i).getDocumentNumber());
            /*fctContract.setContractNumber(fndCodingRuleValuesService.getCodeRuleValue(iRequest, fctContract.getDocumentCategory(), fctContract.getDocumentType(), fctContract.getBusinessType(), params));*/
            FndCompany fndCompany = new FndCompany();
            fndCompany.setCompany_id(fctContract.getCompanyId());
            fndCompany.setCompanyId(fctContract.getCompanyId());
            fndCompany = fndCompanyMapper.selectByPrimaryKey(fndCompany);
            //fctContract.setAuthorityRuleString('"' + fndCompany.getCompany_code() + '"' + "." + '"' + '"' + "." + '"' + '"' + "." + '"' + fctContract.getDocumentType() + '"' + "." + '"' + fctContract.getBusinessType() + '"' + "." + '"' + "" + '"' + "." + '"' + '"');
            setAuthorityRuleString(iRequest,fctContract,fctContract.getCompanyId());

            self().insertSelective(iRequest, fctContract);
            fctContract.setProjectAttachmentId(fctProjectAttachmentList.get(i).getProjectAttachmentId());

            hlsCusFctContractList.add(fctContract);

            Long contractId = fctContract.getContractId();

            Map<String, Object> evenParams = new HashMap<>();
            evenParams.put("message", "合同创建");
            evenParams.put("noticeTitle", "合同创建");
            evenParams.put("url", "");
            evenParams.put("level", 1L);
            evenParams.put("noticeType", "NOTICE");

            sysEventService.eventSave(iRequest, fctContract.getContractId(), fctContract.getDocumentCategory(), fctContract.getDocumentType(), "FCT", PROPERTY_FCT_CONTRACT_CREATE, "P2D", evenParams);


            //复制报价表
            HlsCusFctProjectQuotation fctProjectQuotation = new HlsCusFctProjectQuotation();
            fctProjectQuotation.setProjectId(projectId);
            fctProjectQuotation = fctProjectQuotationService.fctProjectQuotationDetailQuery(fctProjectQuotation).get(0);
            if (fctProjectQuotation != null) {
                Map<String, String> quotationMap = hlsBeanRefUtilService.getFieldValueMap(fctProjectQuotation);
                HlsCusFctContractQuotation fctContractQuotation = new HlsCusFctContractQuotation();
                hlsBeanRefUtilService.setFieldValue(fctContractQuotation, quotationMap);
                fctContractQuotation.setContractId(contractId);
                fctContractQuotationService.insertSelective(iRequest, fctContractQuotation);
            }

            //复制商业伙伴表
            HlsCusFctProjectBp fctProjectBp = new HlsCusFctProjectBp();
            fctProjectBp.setProjectId(projectId);
            List<HlsCusFctProjectBp> hlsCusFctProjectBpList = fctProjectBpService.fctProjectBpQuery(fctProjectBp, 1, 9999);


            if (hlsCusFctProjectBpList.size() > 0) {
                for (int j = 0; j < hlsCusFctProjectBpList.size(); j++) {

                    if ("SELLER".equals(hlsCusFctProjectBpList.get(j).getBpRoleType())) {
                        if (hlsCusFctProjectBpList.get(j).getBpId().compareTo(fctProjectAttachmentList.get(i).getBpId()) == 0) {
                            insertContractBp(hlsCusFctProjectBpList.get(j), contractId, iRequest);
                        }
                    } else {
                        insertContractBp(hlsCusFctProjectBpList.get(j), contractId, iRequest);
                    }
                }

            }
            //回写
            HlsCusFctProjectAttachment attachment = new HlsCusFctProjectAttachment();
            attachment.setProjectId(projectId);
            attachment.setSourceId(fctProjectAttachmentList.get(i).getSourceId());
            List<HlsCusFctProjectAttachment> attachments = fctProjectAttachmentMapper.fctProjectAttachmentForSignProcessQuery(attachment);

            for (HlsCusFctProjectAttachment projectAttachment : attachments) {
                projectAttachment.setSourceId(contractId);
                fctProjectAttachmentService.updateByPrimaryKeySelective(iRequest, projectAttachment);

            }
            //fctProjectAttachmentList.get(i).setSourceId(contractId);
            //fctProjectAttachmentService.updateByPrimaryKeySelective(iRequest, fctProjectAttachmentList.get(i));

        }


        return hlsCusFctContractList;


    }

    public void insertContractBp(HlsCusFctProjectBp bp, Long contractId, IRequest iRequest) {
        Map<String, String> bpMap = hlsBeanRefUtilService.getFieldValueMap(bp);
        HlsCusFctContractBp fctContractBp = new HlsCusFctContractBp();
        hlsBeanRefUtilService.setFieldValue(fctContractBp, bpMap);
        fctContractBp.setContractId(contractId);
        fctContractBpService.insertSelective(iRequest, fctContractBp);
    }

    /**
     * 保理合同管理首页 grid 查询
     *
     * @param iRequest
     * @param hlsCusFctContract
     * @param page
     * @param pageSize
     * @return List<HlsCusFctContract>
     */
    @Override
    public List<HlsCusFctContract> fctContractDetailQuery(IRequest iRequest, HlsCusFctContract hlsCusFctContract, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        hlsCusFctContract.setCompanyId(iRequest.getCompanyId());
        return hlsCusFctContractMapper.fctContractDetailQuery(hlsCusFctContract);
    }

    @Override
    public List<HlsCusFctContract> selectFctContractCountByStatus(HlsCusFctContract hlsCusFctContract) {
        return hlsCusFctContractMapper.selectFctContractCountByStatus(hlsCusFctContract);
    }

    @Override
    public void creditOccupation(IRequest iRequest, HlsCusFctContract hlsCusFctContract) {
        HlsCusHlsCreditLine hlsCusHlsCreditLine = new HlsCusHlsCreditLine();
        hlsCusHlsCreditLine.setCreditLineId(hlsCusFctContract.getCreditLineId());
        hlsCusHlsCreditLine = hlsCreditLineService.selectByPrimaryKey(iRequest, hlsCusHlsCreditLine);


        HlsCusFctProjectAttachment fctProjectAttachment = new HlsCusFctProjectAttachment();
        fctProjectAttachment.setSourceId(hlsCusFctContract.getContractId());
        fctProjectAttachment.setSourceType("FCT_CONTRACT");

        fctProjectAttachment = fctProjectAttachmentMapper.fctContractContentQuery(fctProjectAttachment).get(0);


        //额度类型
        HlsCusFctContract fctContract = new HlsCusFctContract();
        //fctContract.setCreditLineId(hlsCusFctContract.getCreditLineId());
        //当前授信下所有的合同
        //List<HlsCusFctContract> fctContractList = hlsCusFctContractMapper.fctContractDetailQuery(fctContract);
        fctContract.setContractId(hlsCusFctContract.getContractId());
        fctContract = hlsCusFctContractMapper.fctContractDetailQuery(fctContract).get(0);


        //当前合同下所有的提款
        Double withdrawAmount = 0D;
        //当前合同的融资额
        Double financeAmount = 0D;
        Double repaymentAmount = 0D;
        // for (int i = 0; i < fctContractList.size(); i++) {
        HlsCusFctContractWithdraw fctContractWithdraw = new HlsCusFctContractWithdraw();
        fctContractWithdraw.setContractId(fctContract.getContractId());
        fctContractWithdraw.setNotStatus("REJECT");
        List<HlsCusFctContractWithdraw> hlsCusFctContractWithdrawList = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(fctContractWithdraw);
        //求和 全部的提款金额
        withdrawAmount = CalculateUtil.add(withdrawAmount, hlsCusFctContractWithdrawList.stream().mapToDouble(HlsCusFctContractWithdraw::getWithdrawAmount).sum());
        financeAmount = CalculateUtil.add(financeAmount, fctContract.getFinanceAmount());

        for (int j = 0; j < hlsCusFctContractWithdrawList.size(); j++) {
            HlsCusFctContractWithdrawCf fctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
            fctContractWithdrawCf.setWithdrawId(hlsCusFctContractWithdrawList.get(j).getWithdrawId());
            List<HlsCusFctContractWithdrawCf> fctContractWithdrawCfList = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(fctContractWithdrawCf);
            //全部的还款金额
            repaymentAmount = CalculateUtil.add(repaymentAmount, fctContractWithdrawCfList.stream().mapToDouble(HlsCusFctContractWithdrawCf::getWriteOffAmount).sum());
        }

        //}


        //不可循环
        if ("NON_REVOLVING".equals(hlsCusHlsCreditLine.getDocumentType())) {
            //查询之前所有的占用
            //if (hlsCusHlsCreditLine.getCreditLineAmt().compareTo(withdrawAmount) == -1) {
            if (fctProjectAttachment.getCreditAmount().compareTo(withdrawAmount) == -1
                //hlsCusHlsCreditLine.getCreditLineAmt().compareTo(financeAmount) == -1
                    ) {
                throw new IllegalArgumentException("占用额度超过授信额度，请先做授信变更。");
            } else {
                //更新授信占用
                //hlsCusHlsCreditLine.setCreditExposureAmt(withdrawAmount);
                hlsCusHlsCreditLine.setCreditExposureAmt(withdrawAmount);
                hlsCusHlsCreditLine.setCreditExposureQty(hlsCusFctContractWithdrawList.size() + 0L);
                hlsCreditLineService.updateByPrimaryKeySelective(iRequest, hlsCusHlsCreditLine);
            }
        }
        //可循环
        if ("REVOLVING".equals(hlsCusHlsCreditLine.getDocumentType())) {
            //if (hlsCusHlsCreditLine.getCreditLineAmt().compareTo(CalculateUtil.sub(withdrawAmount, repaymentAmount)) == -1) {
            if (//hlsCusHlsCreditLine.getCreditLineAmt().compareTo(CalculateUtil.sub(financeAmount, repaymentAmount)) == -1
                    fctProjectAttachment.getCreditAmount().compareTo(CalculateUtil.sub(withdrawAmount, repaymentAmount)) == -1
                    ) {
                throw new IllegalArgumentException("占用额度超过授信额度，请先做授信变更。");
            } else {
                //hlsCusHlsCreditLine.setCreditExposureAmt(CalculateUtil.sub(withdrawAmount, repaymentAmount));
                hlsCusHlsCreditLine.setCreditExposureAmt(CalculateUtil.sub(withdrawAmount, repaymentAmount));
                hlsCusHlsCreditLine.setCreditExposureQty(hlsCusFctContractWithdrawList.size() + 0L);
                hlsCreditLineService.updateByPrimaryKeySelective(iRequest, hlsCusHlsCreditLine);
            }
        }
        //一次性
        /*if ("ONCE".equals(hlsCusHlsCreditLine.getDocumentType())) {
            if (hlsCusFctContractWithdrawList.size() > 1) {
                throw new IllegalArgumentException("该授信为一次性授信，只能使用一次");
            } else {
                //hlsCusHlsCreditLine.setCreditExposureAmt(withdrawAmount);
                hlsCusHlsCreditLine.setCreditExposureAmt(withdrawAmount);
                hlsCusHlsCreditLine.setCreditExposureQty(hlsCusFctContractWithdrawList.size() + 0L);
                hlsCreditLineService.updateByPrimaryKeySelective(iRequest, hlsCusHlsCreditLine);
            }

        }*/
    }

    //插入流出的现金流 (提款审批通过)
    @Override
    public HlsCusFctContractWithdrawCf insertOutFlowCf(IRequest iRequest, HlsCusFctContractWithdraw hlsCusFctContractWithdraw, Date applyPayDate) {
        HlsCusFctContractWithdrawCf hlsCusFctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
        hlsCusFctContractWithdrawCf.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());
        hlsCusFctContractWithdrawCf.setContractId(hlsCusFctContractWithdraw.getContractId());
        hlsCusFctContractWithdrawCf.setTimes(0L);
        //保理-融资额
        hlsCusFctContractWithdrawCf.setCfItem(40L);
        hlsCusFctContractWithdrawCf.setCfType(40L);
        hlsCusFctContractWithdrawCf.setCfDirection("OUTFLOW");
        hlsCusFctContractWithdrawCf.setCfStatus("RELEASE");
        hlsCusFctContractWithdrawCf.setDueDate(applyPayDate);
        hlsCusFctContractWithdrawCf.setDueAmount(hlsCusFctContractWithdraw.getWithdrawAmount());
        hlsCusFctContractWithdrawCf.setWriteOffFlag("NOT");
        hlsCusFctContractWithdrawCf.setWriteOffAmount(0D);
        hlsCusFctContractWithdrawCf.setInterestAccrualBalance(hlsCusFctContractWithdraw.getWithdrawAmount());
        fctContractWithdrawCfService.insertSelective(iRequest, hlsCusFctContractWithdrawCf);
        return hlsCusFctContractWithdrawCf;
    }


    //插入流入的现金流
    private void insertInflowCf(IRequest iRequest, HlsCusFctContractWithdrawCf hlsCusFctContractWithdrawCf) {
        fctContractWithdrawCfService.insertSelective(iRequest, hlsCusFctContractWithdrawCf);
    }


    //创建利息的现金流
    private HlsCusFctContractWithdrawCf createInterestCf(Long withdrawId, Long contractId, Long times, Date dueDate, Double DueAmount, Double interestAccrualBalance, Long interestPeriodDays, Long writeOffId) {
        HlsCusFctContractWithdrawCf hlsCusFctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
        hlsCusFctContractWithdrawCf.setWithdrawId(withdrawId);
        hlsCusFctContractWithdrawCf.setContractId(contractId);
        hlsCusFctContractWithdrawCf.setTimes(times);
        //保理-利息
        hlsCusFctContractWithdrawCf.setCfItem(502L);
        hlsCusFctContractWithdrawCf.setCfType(50L);
        hlsCusFctContractWithdrawCf.setCfDirection("INFLOW");
        hlsCusFctContractWithdrawCf.setCfStatus("RELEASE");
        hlsCusFctContractWithdrawCf.setDueDate(dueDate);
        hlsCusFctContractWithdrawCf.setDueAmount(DueAmount);
        hlsCusFctContractWithdrawCf.setWriteOffFlag("NOT");
        hlsCusFctContractWithdrawCf.setWriteOffAmount(0D);
        hlsCusFctContractWithdrawCf.setInterestPeriodDays(interestPeriodDays);
        hlsCusFctContractWithdrawCf.setInterestAccrualBalance(interestAccrualBalance);
        hlsCusFctContractWithdrawCf.setWriteOffId(writeOffId);
        return hlsCusFctContractWithdrawCf;
    }

    //创建本金的现金流
    private HlsCusFctContractWithdrawCf createPrincipalCf(Long withdrawId, Long contractId, Long times, Date dueDate, Double DueAmount, Double interestAccrualBalance, Long interestPeriodDays, Long writeOffId) {
        HlsCusFctContractWithdrawCf hlsCusFctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
        hlsCusFctContractWithdrawCf.setWithdrawId(withdrawId);
        hlsCusFctContractWithdrawCf.setContractId(contractId);
        hlsCusFctContractWithdrawCf.setTimes(times);
        //保理-本金
        hlsCusFctContractWithdrawCf.setCfItem(501L);
        hlsCusFctContractWithdrawCf.setCfType(50L);
        hlsCusFctContractWithdrawCf.setCfDirection("INFLOW");
        hlsCusFctContractWithdrawCf.setCfStatus("RELEASE");
        hlsCusFctContractWithdrawCf.setDueDate(dueDate);
        hlsCusFctContractWithdrawCf.setDueAmount(DueAmount);
        hlsCusFctContractWithdrawCf.setWriteOffFlag("NOT");
        hlsCusFctContractWithdrawCf.setWriteOffAmount(0D);
        hlsCusFctContractWithdrawCf.setInterestPeriodDays(interestPeriodDays);
        hlsCusFctContractWithdrawCf.setInterestAccrualBalance(interestAccrualBalance);
        hlsCusFctContractWithdrawCf.setWriteOffId(writeOffId);
        return hlsCusFctContractWithdrawCf;
    }

    //折扣型 金额计算
    private Double disCountMethod(Double outstandingPrincipal, Double intRate) {
        BigDecimal bg = new BigDecimal(CalculateUtil.mul(outstandingPrincipal, intRate) * 2);
        return bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    //比例预付（先付）
    private Double proportionalPrepaymentFirstMethod(Double outstandingPrincipal, Double intRate, Long interestPeriodDays) {
        BigDecimal bg = new BigDecimal(CalculateUtil.div(CalculateUtil.mul(outstandingPrincipal, intRate) * interestPeriodDays, 360D));
        return bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    // 比例预付（后付）
    private Double proportionalPrepaymentNextFMethod(Double outstandingPrincipal, Double principal, Double intRate, Long interestPeriodDays) {
        BigDecimal bg = new BigDecimal(CalculateUtil.add(CalculateUtil.div(CalculateUtil.mul(outstandingPrincipal, intRate) * interestPeriodDays, 360D),
                CalculateUtil.div(CalculateUtil.mul(principal, intRate) * (interestPeriodDays - 1), 360D)));
        return bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    //部分先收 ，利息差额补到第二期利息上（在收款安排制作点击计算时）
    @Override
    public List<HlsCusFctContractWithdrawCf> createContractWithdrawCfPayment(IRequest iRequest, HlsCusFctContractWithdraw hlsCusFctContractWithdraw) throws ParseException {

        Double writeOffAmount = hlsCusFctContractWithdraw.getWriteOffAmount();
        Long writeOffId = hlsCusFctContractWithdraw.getWriteOffId();
        Date interestEndDate = hlsCusFctContractWithdraw.getInterestEndDate();
        hlsCusFctContractWithdraw = hlsCusFctContractWithdrawMapper.selectByPrimaryKey(hlsCusFctContractWithdraw);
        Long withdrawId = hlsCusFctContractWithdraw.getWithdrawId();
        hlsCusFctContractWithdraw.setWriteOffAmount(writeOffAmount);
        hlsCusFctContractWithdraw.setWriteOffId(writeOffId);
        hlsCusFctContractWithdraw.setInterestEndDate(interestEndDate);


        if ("COLLECT_PARTIAL_INTEREST_FIRST".equals(hlsCusFctContractWithdraw.getInterestCollectionMethod())) {
            //取实际放款支付时间
            HlsCusFctContractWithdraw withdraw = new HlsCusFctContractWithdraw();
            withdraw.setWithdrawId(withdrawId);
            withdraw = hlsCusFctContractWithdrawMapper.getWriteOffDateByWithdrawId(withdraw).get(0);

            //计息天数差
            HlsCusFctContractWithdrawCf withdrawCf = new HlsCusFctContractWithdrawCf();
            withdrawCf.setWithdrawId(withdrawId);
            withdrawCf.setCfItem(40L);
            withdrawCf = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(withdrawCf).get(0);

            Calendar calendar1 = Calendar.getInstance();
            Calendar calendar2 = Calendar.getInstance();
            calendar1.setTime(withdraw.getWriteOffDate());
            calendar2.setTime(withdrawCf.getDueDate());

            int days = ((int) calendar2.getTime().getTime() / 1000 - (int) calendar1.getTime().getTime() / 1000) / 3600 / 24;
            BigDecimal bg = new BigDecimal((days * withdrawCf.getDueAmount() * hlsCusFctContractWithdraw.getIntRate() / 360));
            Double interest = bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();

            HlsCusFctContractWithdrawCf withdrawCf1 = new HlsCusFctContractWithdrawCf();
            withdrawCf1.setWithdrawId(withdrawId);
            withdrawCf1.setCfItem(502L);
            withdrawCf1.setTimes(1L);
            withdrawCf1 = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(withdrawCf1).get(0);
            withdrawCf1.setDueAmount(withdrawCf.getDueAmount() + interest);
            fctContractWithdrawCfService.updateByPrimaryKeySelective(iRequest, withdrawCf1);

            HlsCusFctContractWithdrawCf cf = new HlsCusFctContractWithdrawCf();
            cf.setWithdrawId(withdrawId);
            //将合同状态置为 INCEPT
            HlsCusFctContract fctContract = new HlsCusFctContract();
            fctContract.setContractId(hlsCusFctContractWithdraw.getContractId());
            fctContract = self().selectByPrimaryKey(iRequest, fctContract);
            fctContract.setContractStatus("INCEPT");
            self().updateByPrimaryKeySelective(iRequest, fctContract);

            return hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(cf);
        } else if ("COLLECT_TOTAL_INTEREST_FIRST".equals(hlsCusFctContractWithdraw.getInterestCollectionMethod())) {
            HlsCusFctContractWithdrawCf cf = new HlsCusFctContractWithdrawCf();
            cf.setWithdrawId(withdrawId);
            return hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(cf);
        } else {
            //将合同状态置为 INCEPT
            HlsCusFctContract fctContract = new HlsCusFctContract();
            fctContract.setContractId(hlsCusFctContractWithdraw.getContractId());
            fctContract = self().selectByPrimaryKey(iRequest, fctContract);
            fctContract.setContractStatus("INCEPT");
            self().updateByPrimaryKeySelective(iRequest, fctContract);
            return createContractWithdrawCfAdvance(iRequest, hlsCusFctContractWithdraw);
        }
    }

    //根据提款信息 生成现金流
    @Override
    public List<HlsCusFctContractWithdrawCf> createContractWithdrawCfAdvance(IRequest iRequest, HlsCusFctContractWithdraw hlsCusFctContractWithdraw) throws ParseException {


        HlsCusFctContractWithdrawCf cf = new HlsCusFctContractWithdrawCf();
        cf.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());


        //outflow的会先生成
        /*if (hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(cf).size() > 1) {
            throw new IllegalArgumentException("该收款已经生成过现金流！");
        }*/
        //删除一开始计算的现金流
        List<HlsCusFctContractWithdrawCf> cfs = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(cf);
        for (int i = 0; i < cfs.size(); i++) {
            if (cfs.get(i).getCfItem() != 40) {
                fctContractWithdrawCfService.deleteByPrimaryKey(cfs.get(i));

            }
        }


        Double writeOffAmount = hlsCusFctContractWithdraw.getWriteOffAmount();
        Long writeOffId = hlsCusFctContractWithdraw.getWriteOffId();
        Date interestEndDate = hlsCusFctContractWithdraw.getInterestEndDate();
        hlsCusFctContractWithdraw = hlsCusFctContractWithdrawMapper.selectByPrimaryKey(hlsCusFctContractWithdraw);
        hlsCusFctContractWithdraw.setWriteOffAmount(writeOffAmount);
        hlsCusFctContractWithdraw.setWriteOffId(writeOffId);
        hlsCusFctContractWithdraw.setInterestEndDate(interestEndDate);
        //收息方式
        /**
         COLLECT_TOTAL_INTEREST_FIRST     全部先收（先付）
         COLLECT_PARTIAL_INTEREST_FIRST   部分先收（先付）
         PER_MONTH                        每月日（后付）
         PER_QUARTER                      每季日（后付）
         ALL_REPAY_CAPITAL_WITH_INTEREST_AT_MATURITY  到期一次还本付息（后付）
         */
        String interestCollectionMethod = hlsCusFctContractWithdraw.getInterestCollectionMethod();

        //计息方式
        /**
         PROPORTIONAL_PREPAYMENT	比例预付
         DISCOUNT	折扣
         */
        String interestCalcMethod = hlsCusFctContractWithdraw.getInterestCalcMethod();


        //校验到息日期 填写是否正确
        if ("PER_MONTH".equals(interestCollectionMethod) || "PER_QUARTER".equals(interestCollectionMethod)) {
            Calendar c = Calendar.getInstance();
            c.setTime(hlsCusFctContractWithdraw.getWithdrawDate());
            c.add(Calendar.MONTH, hlsCusFctContractWithdraw.getLeaseTerm().intValue() - 1);

            SimpleDateFormat format = new SimpleDateFormat("yyyy-mm-dd");

            if (c.getTime().compareTo(interestEndDate) == 1) {
                throw new IllegalArgumentException("到息日期填写不正确，不能小于" + format.format(c.getTime()));
            }
        }

        //到期一次还本付息（后付）
        if ("ALL_REPAY_CAPITAL_WITH_INTEREST_AT_MATURITY".equals(interestCollectionMethod)) {

            //插入流出现金流
            //insertOutFlowCf(iRequest, hlsCusFctContractWithdraw);
            Double interestAmount = 0D;
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(hlsCusFctContractWithdraw.getWithdrawDate());
            calendar.add(Calendar.MONTH, hlsCusFctContractWithdraw.getLeaseTerm().intValue());
            Date dueDate = calendar.getTime();

            //已填写的到息日为准
            dueDate = interestEndDate;

            //计息天数
            Long interestPeriodDays = ChronoUnit.DAYS.between(hlsCusFctContractWithdraw.getWithdrawDate().toInstant(), dueDate.toInstant());
            //利息金额
            if ("DISCOUNT".equals(interestCalcMethod)) {
                interestAmount = disCountMethod(hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getIntRate());
            } else if ("PROPORTIONAL_PREPAYMENT".equals(interestCalcMethod)) {
                interestAmount = proportionalPrepaymentNextFMethod(hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getIntRate(), interestPeriodDays);
            }
            insertInflowCf(iRequest, createInterestCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), 1L, hlsCusFctContractWithdraw.getWithdrawDate(), interestAmount, hlsCusFctContractWithdraw.getWriteOffAmount(), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
            insertInflowCf(iRequest, createPrincipalCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), 2L, dueDate, hlsCusFctContractWithdraw.getWriteOffAmount(), 0D, interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
        }
        //全部先收（先付）
        if ("COLLECT_TOTAL_INTEREST_FIRST".equals(interestCollectionMethod)) {
            //插入流出现金流
            //insertOutFlowCf(iRequest, hlsCusFctContractWithdraw);

            //本金还款时间
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(hlsCusFctContractWithdraw.getWithdrawDate());
            calendar.add(Calendar.MONTH, hlsCusFctContractWithdraw.getLeaseTerm().intValue());
            Date dueDate = calendar.getTime();

            //已填写的到息日为准
            dueDate = interestEndDate;

            //计息天数
            Long interestPeriodDays = ChronoUnit.DAYS.between(hlsCusFctContractWithdraw.getWithdrawDate().toInstant(), dueDate.toInstant());
            Double interestAmount = 0D;
            if ("DISCOUNT".equals(interestCalcMethod)) {
                //利息金额
                interestAmount = disCountMethod(hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getIntRate());
            } else if ("PROPORTIONAL_PREPAYMENT".equals(interestCalcMethod)) {
                interestAmount = proportionalPrepaymentNextFMethod(hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getIntRate(), interestPeriodDays);
            }
            //插入流入现金流
            insertInflowCf(iRequest, createInterestCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), 0L, hlsCusFctContractWithdraw.getWithdrawDate(), interestAmount, hlsCusFctContractWithdraw.getWriteOffAmount(), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
            insertInflowCf(iRequest, createPrincipalCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), 1L, dueDate, hlsCusFctContractWithdraw.getWriteOffAmount(), 0D, interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));

        }
        //部分先收（先付） 没有折扣型
        if ("COLLECT_PARTIAL_INTEREST_FIRST".equals(interestCollectionMethod)) {
            //第二次利息支付日默认6个月后
            //插入流出现金流
            //insertOutFlowCf(iRequest, hlsCusFctContractWithdraw);

            //本金还款时间
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(hlsCusFctContractWithdraw.getWithdrawDate());
            calendar.add(Calendar.MONTH, hlsCusFctContractWithdraw.getLeaseTerm().intValue());
            Date dueDate = calendar.getTime();

            //已填写的到息日为准
            dueDate = interestEndDate;


            //计息天数
            Calendar calendar2 = Calendar.getInstance();
            calendar2.setTime(hlsCusFctContractWithdraw.getWithdrawDate());
            calendar2.add(Calendar.MONTH, 6);
            Date dueDate2 = calendar2.getTime();
            if (interestEndDate.compareTo(dueDate2) <= 0) {
                throw new IllegalArgumentException("到息日期要在放款日期6个月之后");
            }

            Long interestPeriodDays = ChronoUnit.DAYS.between(hlsCusFctContractWithdraw.getWithdrawDate().toInstant(), dueDate2.toInstant());
            Long interestPeriodDays2 = ChronoUnit.DAYS.between(dueDate2.toInstant(), dueDate.toInstant());


            Double interestAmount = proportionalPrepaymentFirstMethod(hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getIntRate(), interestPeriodDays);
            Double interestAmount2 = proportionalPrepaymentFirstMethod(hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getIntRate(), interestPeriodDays2);

            insertInflowCf(iRequest, createInterestCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), 0L, hlsCusFctContractWithdraw.getWithdrawDate(), interestAmount, hlsCusFctContractWithdraw.getWriteOffAmount(), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
            insertInflowCf(iRequest, createInterestCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), 1L, dueDate2, interestAmount2, hlsCusFctContractWithdraw.getWriteOffAmount(), interestPeriodDays2, hlsCusFctContractWithdraw.getWriteOffId()));
            insertInflowCf(iRequest, createPrincipalCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), 2L, dueDate, hlsCusFctContractWithdraw.getWriteOffAmount(), 0D, interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));

        }

        List<Date> dueDateList = new ArrayList<Date>();
        List<Double> outstandingPrincipalList = new ArrayList<Double>();


        //每月日（后付） 没有折扣型
        if ("PER_MONTH".equals(interestCollectionMethod)) {


            //初始化List
            for (int i = 0; i < hlsCusFctContractWithdraw.getLeaseTerm(); i++) {
                dueDateList.add(null);
                outstandingPrincipalList.add(null);
            }

            //插入流出现金流
            //insertOutFlowCf(iRequest, hlsCusFctContractWithdraw);
            //每月20号还款
            for (int i = 0; i < hlsCusFctContractWithdraw.getLeaseTerm(); i++) {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(hlsCusFctContractWithdraw.getWithdrawDate());
                calendar.add(Calendar.MONTH, i + 1);
                SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM");
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
                //下一个月 20号
                String formatDate = formatter.format(calendar.getTime()) + "-20";
                Date dueDate = format.parse(formatDate);
                //收款日期
                if (i == hlsCusFctContractWithdraw.getLeaseTerm() - 1) {
                    //已填写的到息日为准
                    dueDate = interestEndDate;
                }
                dueDateList.set(i, dueDate);

                //本金
                Double principal = 0D;
                //最后一期倒减
                if (i == hlsCusFctContractWithdraw.getLeaseTerm() - 1) {
                    //principal = CalculateUtil.sub(hlsCusFctContractWithdraw.getWriteOffAmount(), CalculateUtil.mul(CalculateUtil.div(hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getLeaseTerm(), 2), hlsCusFctContractWithdraw.getLeaseTerm() - 1));
                    principal = hlsCusFctContractWithdraw.getWriteOffAmount();
                } else {
                    //principal = CalculateUtil.div(hlsCusFctContractWithdraw.getWriteOffAmount(), hlsCusFctContractWithdraw.getLeaseTerm(), 2);
                    principal = 0D;
                }
                Long interestPeriodDays = 0L;
                //计息天数 和 剩余本金
                if (i == 0) {
                    interestPeriodDays = ChronoUnit.DAYS.between(hlsCusFctContractWithdraw.getWithdrawDate().toInstant(), dueDate.toInstant()) + 1;
                    outstandingPrincipalList.set(0, CalculateUtil.sub(hlsCusFctContractWithdraw.getWriteOffAmount(), principal));
                } else if (i == hlsCusFctContractWithdraw.getLeaseTerm() - 1) {
                    interestPeriodDays = ChronoUnit.DAYS.between(dueDateList.get(i - 1).toInstant(), dueDateList.get(i).toInstant()) - 1;
                    outstandingPrincipalList.set(i, outstandingPrincipalList.get(i - 1) - principal);
                } else {
                    interestPeriodDays = ChronoUnit.DAYS.between(dueDateList.get(i - 1).toInstant(), dueDateList.get(i).toInstant());
                    outstandingPrincipalList.set(i, outstandingPrincipalList.get(i - 1) - principal);
                }
                //利息
                Double interestAmount = proportionalPrepaymentNextFMethod(outstandingPrincipalList.get(i), principal, hlsCusFctContractWithdraw.getIntRate(), interestPeriodDays);

                insertInflowCf(iRequest, createInterestCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), (1L + i), dueDateList.get(i), interestAmount, outstandingPrincipalList.get(i), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
                insertInflowCf(iRequest, createPrincipalCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), (1L + i), dueDateList.get(i), principal, outstandingPrincipalList.get(i), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
            }
        }
        if ("PER_QUARTER".equals(interestCollectionMethod)) {

            //插入流出现金流
            //(iRequest, hlsCusFctContractWithdraw);

            //根据 租赁期限月 算出 最后一个 日期
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(hlsCusFctContractWithdraw.getWithdrawDate());
            calendar.add(Calendar.MONTH, hlsCusFctContractWithdraw.getLeaseTerm().intValue());
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM");
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            String formatDate = formatter.format(calendar.getTime()) + "-20";
            Date dueDate = format.parse(formatDate);

            //已填写的到息日为准
            dueDate = interestEndDate;
            //期数
            Integer times = CalculateUtil.div(hlsCusFctContractWithdraw.getLeaseTerm(), 4D, 0).intValue();

            //初始化List
            for (int i = 0; i < times; i++) {
                dueDateList.add(null);
                outstandingPrincipalList.add(null);
            }


            dueDateList.set(times - 1, dueDate);
            //根据 提款日 算出 第一个收款日
            //每个 3,6,9,12 月 的 20号作为 还款日
            SimpleDateFormat monthFormat = new SimpleDateFormat("MM");
            SimpleDateFormat yearFormat = new SimpleDateFormat("yyyy");
            String month = monthFormat.format(hlsCusFctContractWithdraw.getWithdrawDate());
            String year = yearFormat.format(hlsCusFctContractWithdraw.getWithdrawDate());
            //如果是12月份 就要 到下一年的3月
            if ("12".equals(month)) {
                calendar.set(Integer.parseInt(year) + 1, 3, 20);
            } else {
                if (Integer.parseInt(month) < 3) {
                    calendar.set(Integer.parseInt(year), 3, 20);
                } else if (Integer.parseInt(month) >= 3 && Integer.parseInt(month) < 6) {
                    calendar.set(Integer.parseInt(year), 6, 20);
                } else if (Integer.parseInt(month) >= 6 && Integer.parseInt(month) < 9) {
                    calendar.set(Integer.parseInt(year), 9, 20);
                } else if (Integer.parseInt(month) >= 9 && Integer.parseInt(month) < 12) {
                    calendar.set(Integer.parseInt(year), 12, 20);
                }
            }
            dueDate = calendar.getTime();
            dueDateList.set(0, dueDate);

            for (int j = 1; j < times - 1; j++) {
                calendar.add(Calendar.MONTH, 3);
                dueDate = calendar.getTime();
                dueDateList.set(j, dueDate);
            }

            for (int i = 0; i < times; i++) {
                //本金
                Double principal = 0D;
                //最后一期倒减
                if (i == times - 1) {
                    //principal = CalculateUtil.sub(hlsCusFctContractWithdraw.getWriteOffAmount(), CalculateUtil.mul(CalculateUtil.div(hlsCusFctContractWithdraw.getWriteOffAmount(), times + 0D, 2), times - 1 + 0D));
                    principal = hlsCusFctContractWithdraw.getWriteOffAmount();
                } else {
                    //principal = CalculateUtil.div(hlsCusFctContractWithdraw.getWriteOffAmount(), times + 0D, 2);
                    principal = 0D;
                }
                Long interestPeriodDays = 0L;
                //计息天数 和 剩余本金
                if (i == 0) {
                    interestPeriodDays = ChronoUnit.DAYS.between(hlsCusFctContractWithdraw.getWithdrawDate().toInstant(), dueDateList.get(i).toInstant()) + 1;
                    outstandingPrincipalList.set(0, CalculateUtil.sub(hlsCusFctContractWithdraw.getWriteOffAmount(), principal));
                } else if (i == times - 1) {
                    interestPeriodDays = ChronoUnit.DAYS.between(dueDateList.get(i - 1).toInstant(), dueDateList.get(i).toInstant()) - 1;
                    outstandingPrincipalList.set(i, outstandingPrincipalList.get(i - 1) - principal);
                } else {
                    interestPeriodDays = ChronoUnit.DAYS.between(dueDateList.get(i - 1).toInstant(), dueDateList.get(i).toInstant());
                    outstandingPrincipalList.set(i, outstandingPrincipalList.get(i - 1) - principal);
                }

                //利息
                Double interestAmount = proportionalPrepaymentNextFMethod(outstandingPrincipalList.get(i), principal, hlsCusFctContractWithdraw.getIntRate(), interestPeriodDays);
                insertInflowCf(iRequest, createInterestCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), (1L + i), dueDateList.get(i), interestAmount, outstandingPrincipalList.get(i), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
                insertInflowCf(iRequest, createPrincipalCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), (1L + i), dueDateList.get(i), principal, outstandingPrincipalList.get(i), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
            }

        }

        HlsCusFctContractWithdrawCf hlsCusFctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
        hlsCusFctContractWithdrawCf.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());
        return hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(hlsCusFctContractWithdrawCf);
    }

    @Override
    public List<HlsCusFctContract> fctContractWithdrawQuery(HlsCusFctContract hlsCusFctContract) {
        return hlsCusFctContractMapper.fctContractWithdrawQuery(hlsCusFctContract);
    }

    @Override
    public String withdrawRequestStatus(Long contractId) {

        //绿色 todo 蓝色 read

        // 合同
        HlsCusFctContract fctContract = new HlsCusFctContract();
        fctContract.setContractId(contractId);
         List<HlsCusFctContract> fctContractList = hlsCusFctContractMapper.fctContractDetailQuery(fctContract);
        if(fctContractList.size()>0 ){
            fctContract = fctContractList.get(0);
            // 合同已结清
            if("TERMINATE".equals(fctContract.getContractStatus())){
                return "read";
            }
            // 融资期限到期
            if(new Date().getTime() >= fctContract.getSignDate().getTime() + fctContract.getLeaseTerm()*30*24*60*60*1000){
                return "read";
            }

        }

        HlsCusFctContractWithdraw hlsCusFctContractWithdraw = new HlsCusFctContractWithdraw();
        hlsCusFctContractWithdraw.setContractId(contractId);
        List<HlsCusFctContractWithdraw> list = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(hlsCusFctContractWithdraw);
        if (list.size() < 1) {
            return "todo";
        } else {
            Double exposureAmt = list.get(0).getCreditExposureAmt();
            Double lineAmt = list.get(0).getCreditLineAmt();
            if (exposureAmt == null) {
                exposureAmt = 0D;
            }
            if (lineAmt == null) {
                lineAmt = 0D;
            }
            if (exposureAmt.compareTo(lineAmt) == 0) {
                return "read";
            }
        }
        return "todo";
    }


    @Override
    public String advancesReceivedStatus(Long contractId) {
        //绿色 todo 灰色 undo 蓝色 read
        HlsCusFctContract fctContract = hlsCusFctContractMapper.selectByPrimaryKey(contractId);
        if(fctContract!=null && "TERMINATE".equals(fctContract.getContractStatus())){
            return "read";
        }

        HlsCusFctContractQuotation hlsCusFctContractQuotation = new HlsCusFctContractQuotation();
        hlsCusFctContractQuotation.setContractId(contractId);
        List<HlsCusFctContractQuotation> list = hlsCusFctContractQuotationMapper.fctContractQuotationDetailLineQuery(hlsCusFctContractQuotation);

        /**
         *
         COLLECT_TOTAL_INTEREST_FIRST     全部先收（先付）
         COLLECT_PARTIAL_INTEREST_FIRST   部分先收（先付）
         PER_MONTH                        每月日（后付）
         PER_QUARTER                      每季日（后付）
         ALL_REPAY_CAPITAL_WITH_INTEREST_AT_MATURITY  到期一次还本付息（后付）
         */

        HlsCusFctContractWithdraw hlsCusFctContractWithdraw = new HlsCusFctContractWithdraw();
        hlsCusFctContractWithdraw.setContractId(contractId);
        hlsCusFctContractWithdraw.setWithdrawStatus("WITHDRAW_REQUEST");
        List<HlsCusFctContractWithdraw> withdraws = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(hlsCusFctContractWithdraw);
        int k = 0;
        for (int i = 0; i < withdraws.size(); i++) {
            if ("COLLECT_TOTAL_INTEREST_FIRST".equals(withdraws.get(i).getInterestCollectionMethod()) || "COLLECT_PARTIAL_INTEREST_FIRST".equals(withdraws.get(i).getInterestCollectionMethod())) {
                k = k + 1;
            }
        }

        if (withdraws.size() > 0 && k > 0) {
            return "todo";
        }

        HlsCusFctContractWithdraw fctContractWithdraw = new HlsCusFctContractWithdraw();
        fctContractWithdraw.setContractId(contractId);
        List<HlsCusFctContractWithdraw> withdrawList = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(fctContractWithdraw);

        Long size = 0L;
        for (HlsCusFctContractWithdraw withdraw : withdrawList) {
            HlsCusFctContractWithdrawAprl hlsCusFctContractWithdrawAprl = new HlsCusFctContractWithdrawAprl();
            hlsCusFctContractWithdrawAprl.setWithdrawId(withdraw.getWithdrawId());
            hlsCusFctContractWithdrawAprl.setWithdrawApprovalType("CREATE_ADVANCES_RECEIVED");
            hlsCusFctContractWithdrawAprl.setStatus("APPROVED");
            List<HlsCusFctContractWithdrawAprl> aprls = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);
            hlsCusFctContractWithdrawAprl.setStatus("APPROVING");
            List<HlsCusFctContractWithdrawAprl> aprls1 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);
            size = size + aprls.size() + aprls1.size();
        }

        if (size > 0) {
            return "read";
        }

        if (k == 0 && size == 0) {
            return "undo";
        }
        return "undo";
    }

    @Override
    public String specialRequestAndPaymentRequestStatus(Long contractId) {
        //绿色 todo 蓝色 read 灰色  undo

        HlsCusFctContract fctContract = hlsCusFctContractMapper.selectByPrimaryKey(contractId);
        if(fctContract!=null && "TERMINATE".equals(fctContract.getContractStatus())){
            return "read";
        }
        HlsCusFctContractWithdraw hlsCusFctContractWithdraw = new HlsCusFctContractWithdraw();
        hlsCusFctContractWithdraw.setContractId(contractId);
        List<HlsCusFctContractWithdraw> withdraws = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(hlsCusFctContractWithdraw);

        HlsCusFctContractQuotation hlsCusFctContractQuotation = new HlsCusFctContractQuotation();
        hlsCusFctContractQuotation.setContractId(contractId);
        List<HlsCusFctContractQuotation> list = hlsCusFctContractQuotationMapper.fctContractQuotationDetailLineQuery(hlsCusFctContractQuotation);


        for (int i = 0; i < withdraws.size(); i++) {
            //先付 做了预付款申请 绿色
            if ("COLLECT_TOTAL_INTEREST_FIRST".equals(withdraws.get(i).getInterestCollectionMethod()) || "COLLECT_PARTIAL_INTEREST_FIRST".equals(withdraws.get(i).getInterestCollectionMethod())) {
                HlsCusFctContractWithdraw fctContractWithdraw = new HlsCusFctContractWithdraw();
                fctContractWithdraw.setContractId(contractId);
                fctContractWithdraw.setWithdrawId(withdraws.get(i).getWithdrawId());
                fctContractWithdraw.setWithdrawStatus("ADVANCES_RECEIVED");
                HlsCusCshWriteOff cshWriteOff = new HlsCusCshWriteOff();
                cshWriteOff.setWithdrawId(withdraws.get(i).getWithdrawId());
                cshWriteOff.setCfItem(502L);
                cshWriteOff.setReversedFlag("N");

                List<HlsCusFctContractWithdraw> fctContractWithdraws = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(fctContractWithdraw);
                if (fctContractWithdraws.size() > 0 && hlsCusCshWriteOffMapper.selectAllCshWriteOff(cshWriteOff).size() > 0) {
                    return "todo";
                }
            }
            //后付 做了提款申请 绿色
            if ("PER_MONTH".equals(withdraws.get(i).getInterestCollectionMethod()) || "PER_QUARTER".equals(withdraws.get(i).getInterestCollectionMethod()) || "ALL_REPAY_CAPITAL_WITH_INTEREST_AT_MATURITY".equals(withdraws.get(i).getInterestCollectionMethod())) {
                HlsCusFctContractWithdraw fctContractWithdraw = new HlsCusFctContractWithdraw();
                fctContractWithdraw.setContractId(contractId);
                fctContractWithdraw.setWithdrawId(withdraws.get(i).getWithdrawId());
                fctContractWithdraw.setWithdrawStatus("WITHDRAW_REQUEST");
                List<HlsCusFctContractWithdraw> fctContractWithdraws = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(fctContractWithdraw);
                if (fctContractWithdraws.size() > 0) {
                    return "todo";
                }
            }
        }


        //如果是先付，没有做预付款申请 就是灰色
        /*int k =0;
        int m =0 ;
        for (int i = 0; i < withdraws.size(); i++) {
            if ("COLLECT_TOTAL_INTEREST_FIRST".equals(withdraws.get(i).getInterestCollectionMethod()) || "COLLECT_PARTIAL_INTEREST_FIRST".equals(withdraws.get(i).getInterestCollectionMethod())) {
                 k++;
                HlsCusFctContractWithdraw fctContractWithdraw = new HlsCusFctContractWithdraw();
                fctContractWithdraw.setWithdrawId(withdraws.get(i).getWithdrawId());
                fctContractWithdraw.setWithdrawStatus("ADVANCES_RECEIVED");
                List<HlsCusFctContractWithdraw> fctContractWithdraws = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(fctContractWithdraw);
                m = m+fctContractWithdraws.size();
            }
        }

        if(k == withdraws.size() && m == 0){
            //全部都是先付
            return "undo";
        }*/
        Long size = 0L;

        HlsCusFctContractWithdraw fctContractWithdraw = new HlsCusFctContractWithdraw();
        fctContractWithdraw.setContractId(contractId);
        List<HlsCusFctContractWithdraw> fctContractWithdraws = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(fctContractWithdraw);

        for (HlsCusFctContractWithdraw withdraw : fctContractWithdraws) {
            HlsCusFctContractWithdrawAprl hlsCusFctContractWithdrawAprl = new HlsCusFctContractWithdrawAprl();
            hlsCusFctContractWithdrawAprl.setWithdrawId(withdraw.getWithdrawId());
            hlsCusFctContractWithdrawAprl.setWithdrawApprovalType("CREATE_ADVANCES_RECEIVED");
            hlsCusFctContractWithdrawAprl.setStatus("APPROVED");
            List<HlsCusFctContractWithdrawAprl> aprls = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);
            hlsCusFctContractWithdrawAprl.setStatus("APPROVING");
            List<HlsCusFctContractWithdrawAprl> aprls1 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);

            hlsCusFctContractWithdrawAprl.setWithdrawApprovalType("WITHDRAW_REQUEST");
            hlsCusFctContractWithdrawAprl.setStatus("APPROVED");
            List<HlsCusFctContractWithdrawAprl> aprls2 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);
            hlsCusFctContractWithdrawAprl.setStatus("APPROVING");
            List<HlsCusFctContractWithdrawAprl> aprls3 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);


            size = size + aprls.size() + aprls1.size() + aprls2.size() + aprls3.size();
        }
        if (size > 0) {
            return "read";
        }


        return "undo";
    }

    @Override
    public String collectionCashflowStatus(Long contractId) {

        //绿色 todo 蓝色 read

        HlsCusFctContract fctContract = hlsCusFctContractMapper.selectByPrimaryKey(contractId);
        if(fctContract!=null && "TERMINATE".equals(fctContract.getContractStatus())){
            return "read";
        }
        HlsCusFctContractWithdraw hlsCusFctContractWithdraw = new HlsCusFctContractWithdraw();
        hlsCusFctContractWithdraw.setContractId(contractId);
        List<HlsCusFctContractWithdraw> withdraws = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(hlsCusFctContractWithdraw);

        hlsCusFctContractWithdraw.setWithdrawStatus("PAYMENT_REQUEST");
        List<HlsCusFctContractWithdraw> withdrawList = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(hlsCusFctContractWithdraw);

        Long size = 0L;
        Long size2 = 0L;

        for (HlsCusFctContractWithdraw fctContractWithdraw : withdrawList) {

            HlsCusCshWriteOff writeOff = new HlsCusCshWriteOff();
            writeOff.setWithdrawId(fctContractWithdraw.getWithdrawId());
            writeOff.setCfItem(40L);
            writeOff.setReversedFlag("N");
            size = size + hlsCusCshWriteOffMapper.selectAllCshWriteOff(writeOff).size();
        }

        if (size > 0) {
            return "todo";
        }


        for (HlsCusFctContractWithdraw withdraw : withdraws) {
            HlsCusFctContractWithdrawAprl hlsCusFctContractWithdrawAprl = new HlsCusFctContractWithdrawAprl();
            hlsCusFctContractWithdrawAprl.setWithdrawId(withdraw.getWithdrawId());
            hlsCusFctContractWithdrawAprl.setWithdrawApprovalType("CREATE_COLLECTION_CASHFLOW");
            hlsCusFctContractWithdrawAprl.setStatus("APPROVED");
            List<HlsCusFctContractWithdrawAprl> aprls1 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);

            hlsCusFctContractWithdrawAprl.setStatus("APPROVING");
            List<HlsCusFctContractWithdrawAprl> aprls2 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);
            size2 = size2 + aprls1.size() + aprls2.size();
        }

        if (size == 0 && size2 > 0) {
            return "read";
        }
        return "undo";
    }

    @Override
    public String earlyTerminateStatus(Long contractId) {

        HlsCusFctContract fctContract = hlsCusFctContractMapper.selectByPrimaryKey(contractId);
        if(fctContract!=null && "TERMINATE".equals(fctContract.getContractStatus())){
            return "read";
        }

        HlsCusFctContractWithdraw hlsCusFctContractWithdraw = new HlsCusFctContractWithdraw();
        hlsCusFctContractWithdraw.setContractId(contractId);
        List<HlsCusFctContractWithdraw> withdraws = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(hlsCusFctContractWithdraw);
        Long size = 0L;
        Long size2 = 0L;
        Long size3 = 0L;
        for (HlsCusFctContractWithdraw withdraw : withdraws) {
            HlsCusFctContractWithdrawAprl hlsCusFctContractWithdrawAprl = new HlsCusFctContractWithdrawAprl();
            hlsCusFctContractWithdrawAprl.setWithdrawId(withdraw.getWithdrawId());
            hlsCusFctContractWithdrawAprl.setWithdrawApprovalType("CREATE_COLLECTION_CASHFLOW");
            hlsCusFctContractWithdrawAprl.setStatus("APPROVED");
            List<HlsCusFctContractWithdrawAprl> aprls1 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);

            hlsCusFctContractWithdrawAprl.setWithdrawApprovalType("ET");
            hlsCusFctContractWithdrawAprl.setStatus("APPROVED");
            List<HlsCusFctContractWithdrawAprl> aprls2 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);
            hlsCusFctContractWithdrawAprl.setStatus("APPROVING");
            List<HlsCusFctContractWithdrawAprl> aprls3 = hlsCusFctContractWithdrawAprlMapper.fctContractWithdrawAprlDetailQuery(hlsCusFctContractWithdrawAprl);

            size = size + aprls1.size();
            size2 = size2 + aprls1.size();
            size3 = size3 + aprls1.size();
        }

        if (size.compareTo((size2 + size3)) == 0 && size > 0) {
            return "read";
        }

        if (size.compareTo((size2 + size3)) == 1 && size > 0) {
            return "todo";
        }
        return "undo";
    }

    @Override
    public String contactTerminateStatus(Long contractId) {
        HlsCusFctContract fctContract = new HlsCusFctContract();
        fctContract.setContractId(contractId);
        List<HlsCusFctContract> fctContractList = hlsCusFctContractMapper.fctContractDetailQuery(fctContract);
        if(fctContractList.size()>0 ){
            fctContract = fctContractList.get(0);
            // 合同已结清
            if("TERMINATE".equals(fctContract.getContractStatus())){
                return "read";
            }
            // 融资期限到期
            if(new Date().getTime() >= fctContract.getSignDate().getTime() + fctContract.getLeaseTerm()*30*24*60*60*1000){
                return "todo";
            }

        }



        //合同签约日
        HlsCusFctContract hlsCusFctContract = new HlsCusFctContract();
        hlsCusFctContract.setContractId(contractId);
        hlsCusFctContract = hlsCusFctContractMapper.fctContractDetailQuery(hlsCusFctContract).get(0);
        Date signDate = hlsCusFctContract.getSignDate();
        Double leaseTerm = hlsCusFctContract.getLeaseTerm();
        Calendar c = Calendar.getInstance();
        c.setTime(signDate);
        c.add(Calendar.MONTH, leaseTerm.intValue());

        HlsCusFctContractWithdrawCf hlsCusFctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
        hlsCusFctContractWithdrawCf.setContractId(contractId);
        List<HlsCusFctContractWithdrawCf> list = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(hlsCusFctContractWithdrawCf);

        //如果这个合同有发生过提款 蓝色
        HlsCusFctContractWithdraw hlsCusFctContractWithdraw = new HlsCusFctContractWithdraw();
        hlsCusFctContractWithdraw.setContractId(contractId);
        int size = hlsCusFctContractWithdrawMapper.fctContractWithdrawDetailQuery(hlsCusFctContractWithdraw).size();
        if (size > 0 && new Date().compareTo(c.getTime()) < 1) {
            return "read";
        }


        int k = 0;
        //合同所有的债权全部核销 绿色
        for (int i = 0; i < list.size(); i++) {
            if ("FULL".equals(list.get(0).getWriteOffFlag())) {
                k++;
            }
        }
        if (new Date().compareTo(c.getTime()) == 1 && k == list.size() && !"TERMINATE".equals(hlsCusFctContract.getContractStatus())) {
            return "todo";
        }

        if ("TERMINATE".equals(hlsCusFctContract.getContractStatus())) {
            return "read";
        }
        return "undo";
    }


    @Override
    public List<HlsCusFctContractWithdrawCf> reCalcContractWithdrawCf(IRequest iRequest, List<HlsCusFctContractWithdrawCf> hlsCusFctContractWithdrawCfList) throws ParseException {
        if (hlsCusFctContractWithdrawCfList.size() < 1) {
            throw new IllegalArgumentException("请先点击计算，再重算！");
        }

        HlsCusFctContractWithdrawCf cf = new HlsCusFctContractWithdrawCf();
        cf.setWithdrawId(hlsCusFctContractWithdrawCfList.get(0).getWithdrawId());
        HlsCusFctContractWithdraw hlsCusFctContractWithdraw = hlsCusFctContractWithdrawMapper.selectByPrimaryKey(cf);
        /**
         *
         COLLECT_TOTAL_INTEREST_FIRST     全部先收（先付）
         COLLECT_PARTIAL_INTEREST_FIRST   部分先收（先付）
         PER_MONTH                        每月日（后付）
         PER_QUARTER                      每季日（后付）
         ALL_REPAY_CAPITAL_WITH_INTEREST_AT_MATURITY  到期一次还本付息（后付）
         */
        //先付不能重算
        if ("COLLECT_TOTAL_INTEREST_FIRST".equals(hlsCusFctContractWithdraw.getInterestCollectionMethod()) || "COLLECT_PARTIAL_INTEREST_FIRST".equals(hlsCusFctContractWithdraw.getInterestCollectionMethod())) {
            throw new IllegalArgumentException("先付不能重算！");
        }


        //校验到息日期 填写是否正确
        if ("PER_MONTH".equals(hlsCusFctContractWithdraw.getInterestCollectionMethod()) || "PER_QUARTER".equals(hlsCusFctContractWithdraw.getInterestCollectionMethod())) {
            Calendar c = Calendar.getInstance();
            c.setTime(hlsCusFctContractWithdraw.getWithdrawDate());
            c.add(Calendar.MONTH, hlsCusFctContractWithdraw.getLeaseTerm().intValue() - 1);

            SimpleDateFormat format = new SimpleDateFormat("yyyy-mm-dd");

            if (c.getTime().compareTo(hlsCusFctContractWithdrawCfList.get(0).getInterestEndDate()) == 1) {
                throw new IllegalArgumentException("到息日期填写不正确，不能小于" + format.format(c.getTime()));
            }
        }
        //删除一开始计算的现金流
        List<HlsCusFctContractWithdrawCf> cfs = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(cf);
        if (cfs.size() <= 1) {
            throw new IllegalArgumentException("请先点击计算，再重算！");
        }
        for (int i = 0; i < cfs.size(); i++) {
            if (cfs.get(i).getCfItem() != 40) {
                fctContractWithdrawCfService.deleteByPrimaryKey(cfs.get(i));
            }
        }

        Date paymentDate = new Date();
        paymentDate = hlsCusFctContractWithdrawCfList.get(0).getWithdrawDate();

        Collections.sort(hlsCusFctContractWithdrawCfList, new Comparator<HlsCusFctContractWithdrawCf>() {
            @Override
            public int compare(HlsCusFctContractWithdrawCf o1, HlsCusFctContractWithdrawCf o2) {
                return o1.getDueDate().compareTo(o2.getDueDate());
            }
        });
        //填写的本金之和 == 提款金额
        if (hlsCusFctContractWithdrawCfList.get(0).getWriteOffAmount().compareTo(hlsCusFctContractWithdrawCfList.stream().mapToDouble(HlsCusFctContractWithdrawCf::getCurrentPeriodPrincipal).sum()) != 0) {
            throw new IllegalArgumentException("填写的本金不等于提款金额！");
        }


        Date dueDate = new Date();
        List<Date> dueDateList = new ArrayList<>();
        List<Double> outstandingPrincipalList = new ArrayList<Double>();
        //初始化List
        for (int i = 0; i < hlsCusFctContractWithdrawCfList.size(); i++) {
            dueDateList.add(null);
            outstandingPrincipalList.add(null);
        }

        if ("PER_MONTH".equals(hlsCusFctContractWithdraw.getInterestCollectionMethod()) || "PER_QUARTER".equals(hlsCusFctContractWithdraw.getInterestCollectionMethod())) {

            for (int i = 0; i < hlsCusFctContractWithdrawCfList.size(); i++) {
                if (i == hlsCusFctContractWithdrawCfList.size() - 1) {
                    //已填写的到息日为准
                    dueDate = hlsCusFctContractWithdrawCfList.get(0).getInterestEndDate();
                }
                dueDate = hlsCusFctContractWithdrawCfList.get(i).getDueDate();
                dueDateList.set(i, dueDate);
                //本金
                Double principal = 0D;
                if (hlsCusFctContractWithdrawCfList.get(i).getCurrentPeriodPrincipal() == null) {
                    principal = 0D;
                } else {
                    principal = hlsCusFctContractWithdrawCfList.get(i).getCurrentPeriodPrincipal();
                }
                Long interestPeriodDays = 0L;
                //计息天数 和 剩余本金
                //考虑中间提前还本的情况

                if (i == 0) {
                    interestPeriodDays = ChronoUnit.DAYS.between(paymentDate.toInstant(), dueDate.toInstant()) + 1;
                    outstandingPrincipalList.set(0, CalculateUtil.sub(hlsCusFctContractWithdrawCfList.get(0).getWriteOffAmount(), principal));
                } else if (i == hlsCusFctContractWithdrawCfList.size() - 1) {
                    interestPeriodDays = ChronoUnit.DAYS.between(dueDateList.get(i - 1).toInstant(), dueDateList.get(i).toInstant()) - 1;
                    outstandingPrincipalList.set(i, outstandingPrincipalList.get(i - 1) - principal);
                } else {
                    interestPeriodDays = ChronoUnit.DAYS.between(dueDateList.get(i - 1).toInstant(), dueDateList.get(i).toInstant());
                    outstandingPrincipalList.set(i, outstandingPrincipalList.get(i - 1) - principal);
                }


                //发生了还本
                if (i > 0) {
                    if (hlsCusFctContractWithdrawCfList.get(i - 1).getCurrentPeriodPrincipal().compareTo(0D) != 0 && hlsCusFctContractWithdrawCfList.get(i - 1).getCurrentPeriodPrincipal() != null) {
                        if (i == hlsCusFctContractWithdrawCfList.size() - 1) {
                            interestPeriodDays = ChronoUnit.DAYS.between(dueDateList.get(i - 2).toInstant(), dueDateList.get(i).toInstant()) - 1;
                            outstandingPrincipalList.set(i + 1, outstandingPrincipalList.get(i - 1) - principal);
                        } else {
                            interestPeriodDays = ChronoUnit.DAYS.between(dueDateList.get(i - 2).toInstant(), dueDateList.get(i).toInstant());
                            outstandingPrincipalList.set(i + 1, outstandingPrincipalList.get(i - 1) - principal);
                        }
                    }
                }


                //利息
                Double interestAmount = proportionalPrepaymentNextFMethod(outstandingPrincipalList.get(i), principal, hlsCusFctContractWithdraw.getIntRate(), interestPeriodDays);
                insertInflowCf(iRequest, createInterestCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), (1L + i), dueDateList.get(i), interestAmount, outstandingPrincipalList.get(i), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
                insertInflowCf(iRequest, createPrincipalCf(hlsCusFctContractWithdraw.getWithdrawId(), hlsCusFctContractWithdraw.getContractId(), (1L + i), dueDateList.get(i), principal, outstandingPrincipalList.get(i), interestPeriodDays, hlsCusFctContractWithdraw.getWriteOffId()));
            }
        }

        HlsCusFctContractWithdrawCf hlsCusFctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
        hlsCusFctContractWithdrawCf.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());
        return hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfQuery(hlsCusFctContractWithdrawCf);
    }


    //查询合同结束界面的合同信息
    @Override
    public List<HlsCusFctContract> selectContractDetailInfo(HlsCusFctContract hlsCusFctContract) {
        return hlsCusFctContractMapper.selectContractDetailInfo(hlsCusFctContract);
    }

    //合同结束提交
    @Override
    public List<HlsCusFctContract> contractEndSubmit(IRequest iRequest, HlsCusFctContract fctContract) {
        List<HlsCusFctContract> fctContractList = new ArrayList<>();
        fctContractList.add(fctContract);
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        //String userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }

        String msg = userName + "提交了一条保理合同" + fctContract.getContractName() + fctContract.getContractNumber() + "的合同结束申请";
        evenParams.put("message", msg);
        evenParams.put("noticeTitle", "保理合同结束提交");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");

        sysEventService.eventSave(iRequest, fctContract.getContractId(), fctContract.getDocumentCategory(), fctContract.getDocumentType(), "FCT", PROPERTY_FCT_CONTRACT_END_SUBMIT, "P2D", evenParams);
        //启动工作流
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_PROJECT_WRITEOFF");
        params.put("contractId", fctContract.getContractId());
        activitiStartService.start(iRequest, fctContractList, params);
        //更新状态 合同结束申请
        return fctContractList;
    }
}