package hls.core.fct.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fct.dto.HlsCusFctContractWithdraw;
import hls.core.fct.dto.HlsCusFctContractWithdrawPt;
import hls.core.fct.mapper.HlsCusFctContractWithdrawPtMapper;
import hls.core.fct.service.FctContractWithdrawService;
import hls.core.fct.service.IFctContractWithdrawPtService;
import hls.core.sys.event.service.SysEventService;
import hls.core.user.service.LoginUserInfoService;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional(rollbackFor = Exception.class)
public class FctContractWithdrawPtServiceImpl extends BaseServiceImpl<HlsCusFctContractWithdrawPt> implements IFctContractWithdrawPtService{



    @Autowired
    private HlsCusFctContractWithdrawPtMapper mapper;
    @Autowired
    private FctContractWithdrawService fctContractWithdrawService;


    @Autowired
    DatabaseLockProvider databaseLockProvider;
    @Autowired
    private LoginUserInfoService loginUserInfoService;
    @Autowired
    SysEventService sysEventService;
    @Autowired
    IActivitiStartService activitiStartService;


    //特殊情况申请 提交
    @Override
    public void fctContractWithdrawPtSubmit(IRequest iRequest, List<HlsCusFctContractWithdrawPt> hlsCusFctContractWithdrawPt) {
        //获取对应提款信息
        HlsCusFctContractWithdraw fctContractWithdraw=new HlsCusFctContractWithdraw();
        fctContractWithdraw.setWithdrawId(hlsCusFctContractWithdrawPt.get(0).getWithdrawId());
        fctContractWithdraw=fctContractWithdrawService.selectByPrimaryKey(iRequest,fctContractWithdraw);
        List<HlsCusFctContractWithdraw> fctContractWithdrawList=new ArrayList<>();
        fctContractWithdrawList.add(fctContractWithdraw);
        //提交前先执行保存
        HlsCusFctContractWithdrawPt fctContractWithdrawPt = hlsCusFctContractWithdrawPt.get(0);
        //锁表
        databaseLockProvider.lock(fctContractWithdraw);
//        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        //String userName=(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg=userName+"提交了一条提款申请编号为"+fctContractWithdraw.getWithdrawNumber()+"的特殊情况申请";
        evenParams.put("message", msg);
        evenParams.put("noticeTitle", "特殊情况申请审批提交");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");

        sysEventService.eventSave(iRequest, fctContractWithdraw.getWithdrawId(), fctContractWithdraw.getDocumentCategory(), fctContractWithdraw.getDocumentType(), "FCT","FCT_SPECIAL_REQUEST", "P2D", evenParams);
        //启动工作流
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_PAYMENT_REQUEST_EXCEPTION");
        params.put("withdrawId",  fctContractWithdraw.getWithdrawId());
        activitiStartService.start(iRequest, fctContractWithdrawList, params);
        //更新状态
        fctContractWithdraw.setWithdrawStatus("PROCESS_IN_SPECIAL_REQUEST");
        fctContractWithdrawService.updateByPrimaryKey(iRequest,fctContractWithdraw);
    }


    //拒绝回滚
    @Override
    public void updateByWithdrawId(HlsCusFctContractWithdrawPt fctContractWithdrawPt){
        mapper.updateByWithdrawId(fctContractWithdrawPt);
    }


}