package hls.core.fct.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fct.dto.*;
import hls.core.fct.mapper.*;
import hls.core.fct.service.*;
import hls.core.fnd.dto.DocFileTempletRule;
import hls.core.fnd.dto.HlsDocFileTemplet;
import hls.core.fnd.mapper.DocFileTempletRuleMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.fnd.service.HlsDocFileTempletService;
import hls.core.hls.components.Datasource2Json;
import hls.core.ruleengine.service.IHLSRuleEngineInitService;
import hls.core.sys.event.service.SysEventService;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class FctContractWithdrawServiceImpl extends BaseServiceImpl<HlsCusFctContractWithdraw> implements FctContractWithdrawService {


    @Autowired
    private HlsCusFctContractWithdrawMapper mapper;
    @Autowired
    private HlsCusFctContractWithdrawCfMapper hlsCusFctContractWithdrawCfMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private FctContractService fctContractService;
    @Autowired
    private HlsCusFctContractMapper hlsCusFctContractMapper;
    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;

    @Autowired
    private HlsCusFctContractQuotationMapper hlsCusFctContractQuotationMapper;
    @Autowired
    private HlsCusFctContractWithdrawAttMapper fctContractWithdrawAttMapper;
    @Autowired
    private FctContractWithdrawAttService fctContractWithdrawAttService;
    @Autowired(required = false)
    private HlsCusFctWflCustomService fctWflCostomService;

    @Autowired
    private DocFileTempletRuleMapper docFileTempletRuleMapper;

    @Autowired
    private Datasource2Json datasource2Json;

    @Autowired
    private IHLSRuleEngineInitService hLSRuleEngineInitService;
    @Autowired
    private HlsDocFileTempletService hlsDocFileTempletService;

    @Autowired
    private FctWithdrawDocxService withdrawDocxService;


    public static final String PROPERTY_FCT_CASH_DEMAND_SUBMIT = "FCT_WITHDRAW_REQUEST.SUBMIT";

    public static final String PROPERTY_FCT_ADVANCES_RECEIVED_SUBMIT = "FCT_ADVANCES_RECEIVED.SUBMIT";

    public static final String PROPERTY_FCT_COLLECTION_CASHFLOW_SUBMIT = "FCT_COLLECTION_CASHFLOW.SUBMIT";

    @Override
    public List<HlsCusFctContractWithdraw> fctContractWithdrawDetailForSpcl(HlsCusFctContractWithdraw hlsCusFctContractWithdraw) {
        return mapper.fctContractWithdrawDetailForSpcl(hlsCusFctContractWithdraw);
    }

    @Override
    public List<HlsCusFctContractWithdraw> fctContractWithdrawInfoLov(IRequest request, HlsCusFctContractWithdraw hlsCusFctContractWithdraw, int page, int pageSize) {
        hlsCusFctContractWithdraw.setCompanyId(request.getCompanyId());
        PageHelper.startPage(page, pageSize);
        return mapper.fctContractWithdrawInfoLov(hlsCusFctContractWithdraw);
    }

    @Override
    public List<HlsCusFctContractWithdraw> fctContractWithdrawInfoWriteOffLov(IRequest request, HlsCusFctContractWithdraw hlsCusFctContractWithdraw, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        hlsCusFctContractWithdraw.setCompanyId(request.getCompanyId());
        return mapper.fctContractWithdrawInfoWriteOffLov(hlsCusFctContractWithdraw);
    }

    @Override
    public List<HlsCusFctContractWithdraw> fctContractWithdrawInfoETLov(IRequest request, HlsCusFctContractWithdraw hlsCusFctContractWithdraw, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        hlsCusFctContractWithdraw.setCompanyId(request.getCompanyId());
        return mapper.fctContractWithdrawInfoETLov(hlsCusFctContractWithdraw);
    }

    @Override
    public List<HlsCusFctContractWithdraw> fctContractWithdrawInfoAdvancesLov(IRequest request, HlsCusFctContractWithdraw hlsCusFctContractWithdraw, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        hlsCusFctContractWithdraw.setCompanyId(request.getCompanyId());
        return mapper.fctContractWithdrawInfoAdvancesLov(hlsCusFctContractWithdraw);
    }

    @Override
    public List<HlsCusFctContractWithdraw> fctContractWithdrawAprlDetail(HlsCusFctContractWithdraw hlsCusFctContractWithdraw) {
        return mapper.fctContractWithdrawAprlDetail(hlsCusFctContractWithdraw);
    }

    @Override
    public List<HlsCusFctContractWithdraw> fctContractWithdrawDetailQuery(HlsCusFctContractWithdraw hlsCusFctContractWithdraw) {
        return mapper.fctContractWithdrawDetailQuery(hlsCusFctContractWithdraw);
    }

    @Override
    public List<Map> fctContractHistoryQuery(HlsCusFctContractWithdraw hlsCusFctContractWithdraw) {
        return mapper.fctContractHistoryQuery(hlsCusFctContractWithdraw);
    }

    @Override
    public List<HlsCusFctContractWithdraw> withdrawPaymentHistoryQuery(HlsCusFctContractWithdraw hlsCusFctContractWithdraw) {
        return mapper.withdrawPaymentHistoryQuery(hlsCusFctContractWithdraw);
    }

    @Override
    public List<Map> paymentDetailQuery(HlsCusFctContractWithdraw hlsCusFctContractWithdraw) {
        return mapper.paymentDetailQuery(hlsCusFctContractWithdraw);
    }


    private void validate(HlsCusFctContractWithdraw hlsCusFctContractWithdraw) {
        HlsCusFctContractWithdrawCf cf = new HlsCusFctContractWithdrawCf();
        cf.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());
        cf.setContractId(hlsCusFctContractWithdraw.getContractId());
        if (hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(cf).size() < 1) {
            throw new IllegalArgumentException("请先点击计算，在提交。");
        }


    }

    //预计收款安排
    @Override
    public void repaymentSubmit(IRequest request, List<HlsCusFctContractWithdraw> hlsCusFctContractWithdraws) {
        updateWithdrawAtt(request,hlsCusFctContractWithdraws);
        //1.校验
        HlsCusFctContractWithdraw fctContractWithdraw = hlsCusFctContractWithdraws.get(0);
        fctContractWithdraw = mapper.fctContractWithdrawDetailQuery(fctContractWithdraw).get(0);
       // validate(fctContractWithdraw);
        databaseLockProvider.lock(fctContractWithdraw);
        //2.插入事件
        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "保理提款" + fctContractWithdraw.getWithdrawName() + "提交预计收款安排制作");
        evenParams.put("noticeTitle", "预计收款安排制作审批");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(request, fctContractWithdraw.getWithdrawId(), fctContractWithdraw.getDocumentCategory(), fctContractWithdraw.getDocumentType(), "FCT", PROPERTY_FCT_ADVANCES_RECEIVED_SUBMIT, "P2D", evenParams);

        //3.启动工作流
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_ADVANCES_RECEIVED");
        params.put("withdrawId", fctContractWithdraw.getWithdrawId());
        activitiStartService.start(request, hlsCusFctContractWithdraws, params);
        //4.更新状态
        mapper.updateWithdrawStatus(fctContractWithdraw.getWithdrawId(), "PROCESS_IN_ADVANCES_RECEIVED", request.getUserId(), new Date());

    }

    @Override
    public List<HlsCusFctContractWithdraw> cashDemandSubmit(IRequest request, List<HlsCusFctContractWithdraw> hlsCusFctContractWithdraws) {
        //先取附件ID信息
        String [] attachmentArr = hlsCusFctContractWithdraws.get(0).getWithdrawAttachmentArr();
        //保存提款信息
        hlsCusFctContractWithdraws = self().fctContractWithdrawSave(request,hlsCusFctContractWithdraws);
        //重新赋值
        hlsCusFctContractWithdraws.get(0).setWithdrawAttachmentArr(attachmentArr);
        //更新附件信息
        updateWithdrawAtt(request,hlsCusFctContractWithdraws);
        List<HlsCusFctContractWithdraw> withdraws =  new ArrayList<HlsCusFctContractWithdraw>();
        withdraws.add(hlsCusFctContractWithdraws.get(0));

        //1.校验
            HlsCusFctContractWithdraw fctContractWithdraw = hlsCusFctContractWithdraws.get(0);
        //fctContractWithdraw = mapper.fctContractWithdrawDetailQuery(fctContractWithdraw).get(0);
        fctContractWithdraw = mapper.selectByPrimaryKey(fctContractWithdraw);
        //授信占用校验
        fctContractService.creditOccupation(request, hlsCusFctContractMapper.selectByPrimaryKey(fctContractWithdraw.getContractId()));

        //提款时的融资期限要不大于单笔提款最长期限（月）
        HlsCusFctContractQuotation quotation = new HlsCusFctContractQuotation();
        quotation.setContractId(fctContractWithdraw.getContractId());
        quotation= hlsCusFctContractQuotationMapper.fctContractQuotationDetailQuery(quotation).get(0);
        if(quotation.getWithdrawMaximumTerm().compareTo(fctContractWithdraw.getLeaseTerm()) == -1){
            throw new IllegalArgumentException("融资期限大于单笔提款最长期限（月）");
        }

        //validate(fctContractWithdraw);
        databaseLockProvider.lock(fctContractWithdraw);
        //2.插入事件
        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "保理提款" + fctContractWithdraw.getWithdrawName() + "提交用款需求申请");
        evenParams.put("noticeTitle", "用款需求申请审批");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(request, fctContractWithdraw.getWithdrawId(), fctContractWithdraw.getDocumentCategory(), fctContractWithdraw.getDocumentType(), "FCT", PROPERTY_FCT_CASH_DEMAND_SUBMIT, "P2D", evenParams);

        //3.启动工作流
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_DRAWING");
        params.put("withdrawId", fctContractWithdraw.getWithdrawId());
        activitiStartService.start(request, hlsCusFctContractWithdraws, params);

        //4.更新状态
        mapper.updateWithdrawStatus(fctContractWithdraw.getWithdrawId(), "PROCESS_IN_WITHDRAW_REQUEST", request.getUserId(), new Date());

        return hlsCusFctContractWithdraws;
    }


    //收款安排制作
    @Override
    public void collectionCashflowSubmit(IRequest request, List<HlsCusFctContractWithdraw> hlsCusFctContractWithdraws) {

        //1.校验
        HlsCusFctContractWithdraw fctContractWithdraw = hlsCusFctContractWithdraws.get(0);
        fctContractWithdraw = mapper.fctContractWithdrawDetailQuery(fctContractWithdraw).get(0);
        databaseLockProvider.lock(fctContractWithdraw);
        //2.插入事件
        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "保理提款" + fctContractWithdraw.getWithdrawName() + "提交收款安排制作");
        evenParams.put("noticeTitle", "实际收款安排制作审批");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(request, fctContractWithdraw.getWithdrawId(), fctContractWithdraw.getDocumentCategory(), fctContractWithdraw.getDocumentType(), "FCT", PROPERTY_FCT_COLLECTION_CASHFLOW_SUBMIT, "P2D", evenParams);

        //3.启动工作流
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_GATHERING_PRODUCE");
        params.put("withdrawId", fctContractWithdraw.getWithdrawId());
        activitiStartService.start(request, hlsCusFctContractWithdraws, params);
        //4.更新状态
        mapper.updateWithdrawStatus(fctContractWithdraw.getWithdrawId(), "PROCESS_IN_COLLECTION_CASHFLOW", request.getUserId(), new Date());

    }


    @Override
   public  List<HlsCusFctContractWithdraw> fctContractWithdrawSave(IRequest request,List<HlsCusFctContractWithdraw> hlsCusFctContractWithdraw){
        for (HlsCusFctContractWithdraw dt : hlsCusFctContractWithdraw) {
            if (dt.getWithdrawId() == null || dt.getWithdrawId() == 0) {
                dt.set__status("add");
                dt.setDocumentCategory("FCT_CONTRACT_WITHDRAW");
                dt.setBusinessType("FCT_CONTRACT_WITHDRAW");
                dt.setDocumentType("STD");
                Map<String, String> params = new HashMap<String, String>();
                dt.setObjectVersionNumber(1L);
                dt.setWithdrawNumber(createWithdrawNumber(request,dt.getContractId()));
               // dt.setWithdrawNumber(fndCodingRuleValuesService.getCodeRuleValue(request,dt.getDocumentCategory(), dt.getDocumentType(), dt.getBusinessType(), params));
                dt.setWithdrawName(dt.getWithdrawNumber());
            } else {
                dt.set__status("update");
            }
        }
        if (fctWflCostomService != null) {
            fctWflCostomService.afterCollectCalculateSubmit(request,hlsCusFctContractWithdraw.get(0));
        }
        return self().batchUpdate(request,hlsCusFctContractWithdraw);
    }
    //修改FCT_CONTRACT_WITHDRAW_ATT表记录
    @Override
    public void updateWithdrawAtt(IRequest request,List<HlsCusFctContractWithdraw> hlsCusFctContractWithdraws){
        String [] attachmentArr = hlsCusFctContractWithdraws.get(0).getWithdrawAttachmentArr();
        Long widthDrawId = hlsCusFctContractWithdraws.get(0).getWithdrawId();
        if(attachmentArr != null && attachmentArr.length > 0 ){
            HlsCusFctContractWithdrawAtt fctContractWithdrawAtt = new HlsCusFctContractWithdrawAtt();
            for(int i=0; i<attachmentArr.length; i++){
                fctContractWithdrawAtt.setWithdrawAttachmentId(Long.parseLong(attachmentArr[i].split("、")[0]));
                HlsCusFctContractWithdrawAtt fctContractWithdrawAttTmp =  fctContractWithdrawAttMapper.selectByPrimaryKey(fctContractWithdrawAtt);
                if(fctContractWithdrawAttTmp != null){
                    fctContractWithdrawAttTmp.setWithdrawId(widthDrawId);
                    fctContractWithdrawAttTmp.setDescription(attachmentArr[i].split("、")[1]);
                    fctContractWithdrawAttMapper.updateByPrimaryKey(fctContractWithdrawAttTmp);
                }
            }
        }
    }

    @Override
    public void earlyTerminateSubmit(IRequest request, List<HlsCusFctContractWithdraw> hlsCusFctContractWithdraws) {
        //1.校验
        HlsCusFctContractWithdraw fctContractWithdraw = hlsCusFctContractWithdraws.get(0);
        fctContractWithdraw = mapper.fctContractWithdrawDetailQuery(fctContractWithdraw).get(0);
        databaseLockProvider.lock(fctContractWithdraw);
        //2.插入事件
        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "保理提款" + fctContractWithdraw.getWithdrawName() + "提交提前还款");
        evenParams.put("noticeTitle", "提前还款审批");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(request, fctContractWithdraw.getWithdrawId(), fctContractWithdraw.getDocumentCategory(), fctContractWithdraw.getDocumentType(), "FCT", PROPERTY_FCT_COLLECTION_CASHFLOW_SUBMIT, "P2D", evenParams);

        //3.启动工作流
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_EARLY_TERMINATE");
        params.put("withdrawId", fctContractWithdraw.getWithdrawId());
        activitiStartService.start(request, hlsCusFctContractWithdraws, params);
        //4.更新状态
        mapper.updateWithdrawStatus(fctContractWithdraw.getWithdrawId(), "PROCESS_IN_ET", request.getUserId(), new Date());
    }
    @Override
    public void createWithdrawContent(IRequest request,HlsCusFctContractWithdraw hlsCusFctContractWithdraws){

        hlsCusFctContractWithdraws=mapper.selectByPrimaryKey(hlsCusFctContractWithdraws);
        String[] strName={"应收账款债权转让通知书","应收账款债权转让通知书回执","应收账款转让申请审核意见（适用于受让）","应收账款转让申请书","保理融资凭证"};
        String[] strType={"WITHDRAW_TRANS_NOTIC","WITHDRAW_TRANS_NOTIC_RECEIPT","WITHDRAW_TRANS_APPR_SUGG","WITHDRAW_TRANS_APPR","WITHDRAW_FINANCING_VOUNCHER"};
        //防止重复生成
        HlsCusFctContractWithdrawAtt fctContractWithdrawAttOld=new HlsCusFctContractWithdrawAtt();
        fctContractWithdrawAttOld.setWithdrawId(hlsCusFctContractWithdraws.getWithdrawId());
        String sourceType="'WITHDRAW_TRANS_NOTIC,WITHDRAW_TRANS_NOTIC_RECEIPT,WITHDRAW_TRANS_APPR_SUGG,WITHDRAW_TRANS_APPR,WITHDRAW_FINANCING_VOUNCHER'";
        fctContractWithdrawAttOld.setSourceType(sourceType);
        fctContractWithdrawAttService.deleteWithdrawContent(fctContractWithdrawAttOld);

        //生成
        for(int i=0;i<strName.length;i++){
            HlsCusFctContractWithdrawAtt fctContractWithdrawAtt=new HlsCusFctContractWithdrawAtt();
            fctContractWithdrawAtt.setWithdrawId(hlsCusFctContractWithdraws.getWithdrawId());
            fctContractWithdrawAtt.setDocumentName(strName[i]);
            fctContractWithdrawAtt.setSourceType(strType[i]);
            fctContractWithdrawAtt =fctContractWithdrawAttService.insertSelective(request,fctContractWithdrawAtt);

            //生成文件
            String templetType = "FCT_WITHDRAW_DOCUMENT";
            Long companyId = request.getCompanyId();
            DocFileTempletRule docFileTempletRule = new DocFileTempletRule();
            docFileTempletRule.setCompanyId(companyId);
            docFileTempletRule.setTempletType(templetType);
            docFileTempletRule = docFileTempletRuleMapper.selectOne(docFileTempletRule);
            String json = null;
            try {
                Map<String, Object> pMap = new HashMap<String, Object>();
                pMap.put("withdrawId", hlsCusFctContractWithdraws.getWithdrawId());
                pMap.put("withdrawAttachmentId", fctContractWithdrawAtt.getWithdrawAttachmentId());
                json = datasource2Json.executeSQL4Json(docFileTempletRule.getDataSourceId(), pMap);
            } catch (IOException e) {
                e.printStackTrace();
            }
            JSONObject jsonObject0 = JSON.parseObject(json);
            JSONObject jsonObject1 = JSON.parseObject(JSON.toJSONString(jsonObject0.get(docFileTempletRule.getDataSourceId().toString())));
            JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(jsonObject1.get("default")));
            jsonObject.put("ruleEngineId", docFileTempletRule.getRuleEngineId());
            String[] arr = hLSRuleEngineInitService.ruleEngineInit(request, jsonObject);

            HlsDocFileTemplet hlsDocFileTemplet = null;
            HlsCusFctProjectAttachment attachment = null;

            for (int j = 0; j < arr.length; j++) {
                hlsDocFileTemplet = new HlsDocFileTemplet();
                hlsDocFileTemplet.setTempletId(Long.parseLong(arr[j]));
                hlsDocFileTemplet = hlsDocFileTempletService.selectByPrimaryKey(request, hlsDocFileTemplet);
                Map<String, Object> params = new HashMap<String, Object>();
                params.put("withdrawId", hlsCusFctContractWithdraws.getWithdrawId());
                params.put("withdrawAttachmentId", fctContractWithdrawAtt.getWithdrawAttachmentId());
                params.put("templetId", hlsDocFileTemplet.getTempletId());
                withdrawDocxService.process(request, params, jsonObject);
            }
        }
    }

    @Override
    public Long getWithdrawCount(Long contractId){
        return mapper.getWithdrawCount(contractId);
    }

    public String createWithdrawNumber(IRequest request,Long contractId){
        HlsCusFctContract fctContract=new HlsCusFctContract();
        fctContract.setContractId(contractId);
        fctContract=fctContractService.selectByPrimaryKey(request,fctContract);
        Long withdrawCount=self().getWithdrawCount(contractId)+1;
        String witdhrawNumber="";
        if(withdrawCount>=1&&withdrawCount<10){
            witdhrawNumber=fctContract.getContractNumber()+"-000"+withdrawCount.toString();
        }else if(withdrawCount>=10&&withdrawCount<100){
            witdhrawNumber=fctContract.getContractNumber()+"-00"+withdrawCount.toString();
        }else if(withdrawCount>=100&&withdrawCount<1000){
            witdhrawNumber=fctContract.getContractNumber()+"-0"+withdrawCount.toString();
        }else if(withdrawCount>=1000&&withdrawCount<9999){
            witdhrawNumber=fctContract.getContractNumber()+"-"+withdrawCount.toString();
        }
        return witdhrawNumber;
    }
}