package hls.core.fct.service.impl;

import hls.core.csh.dto.HlsCusConDebtExemptionReqCf;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.mapper.HlsCusConDebtExemptionReqCfMapper;
import hls.core.csh.mapper.HlsCusCshWriteOffMapper;
import hls.core.fct.dto.*;
import hls.core.fct.mapper.*;
import hls.core.fnd.mapper.HlsPenaltyProfileDtlMapper;
import hls.core.hls.service.HlsDayEndCommon;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class FctPenaltyCalServiceImpl implements HlsDayEndCommon {

    @Autowired
    HlsCusFctContractWithdrawMapper hlsCusFctContractWithdrawMapper;
    @Autowired
    HlsCusFctContractMapper hlsCusFctContractMapper;
    @Autowired
    HlsCusFctContractWithdrawCfMapper hlsCusFctContractWithdrawCfMapper;
    @Autowired
    HlsCusFctProjectMapper hlsCusFctProjectMapper;
    @Autowired
    HlsCusFctContractQuotationMapper hlsCusFctContractQuotationMapper;
    @Autowired
    HlsCusCshWriteOffMapper cshWriteOffMapper;
    @Autowired
    HlsPenaltyProfileDtlMapper hlsPenaltyProfileDtlMapper;
    @Autowired
    HlsCusConDebtExemptionReqCfMapper conDebtExemptionReqCfMapper;

    private static final String DayEndType = "FCT_PENALTY";

    @Override
    public String getDayEndType() {
        return DayEndType;
    }

    @Override
    public void process(Map map) {
        if (map.get("withdrawId") != null && map.get("withdrawId") != "") { //对特定合同计算罚息
            Long withdrawId = (Long) map.get("withdrawId");
            HlsCusFctContractWithdraw withdraw = new HlsCusFctContractWithdraw();
            withdraw.setWithdrawId(withdrawId);
            withdraw = hlsCusFctContractWithdrawMapper.selectByPrimaryKey(withdraw);
            HlsCusFctContract contract = new HlsCusFctContract();
            contract.setContractId(withdraw.getContractId());
            contract = hlsCusFctContractMapper.selectByPrimaryKey(contract);

            if (contract == null || contract.getContractStatus().equals("INCEPT")) {
                calculatePenalty(withdraw, (Date) map.get("dayEndDate"));

            } else {
                //抛出异常合同状态错误不能计算罚息！
                throw new IllegalArgumentException("合同状态错误不能计算罚息！");
            }
        } else { //对所有合同计算罚息
            List<HlsCusFctContractWithdraw> list = hlsCusFctContractWithdrawMapper.selectNeedCalcPenaltyWithdraw(Long.parseLong(map.get("companyId").toString()));

            if(list!= null && list.size() > 0){
                for (HlsCusFctContractWithdraw con : list) {
                    //计算罚息
                    calculatePenalty(con, (Date) map.get("dayEndDate"));
                }
            }

        }
    }

    //计算一条合同的罚息
    public void calculatePenalty(HlsCusFctContractWithdraw withdraw, Date dayEndDate) {
        //找出该合同对应的现金流
        HlsCusFctContractWithdrawCf contractWithdrawCf = new HlsCusFctContractWithdrawCf();
        contractWithdrawCf.setDayEndDate(dayEndDate);
        contractWithdrawCf.setWithdrawId(withdraw.getWithdrawId());
        List<HlsCusFctContractWithdrawCf> withdrawCfList = hlsCusFctContractWithdrawCfMapper.selectCalcPenaltyCashFlowByWithdrawId(contractWithdrawCf);

        if (withdrawCfList != null && withdrawCfList.size() > 0) {
            for (HlsCusFctContractWithdrawCf ccc : withdrawCfList) {

                ccc.setOverdueStatus("Y");
                hlsCusFctContractWithdrawCfMapper.updateByPrimaryKeySelective(ccc);


                HlsCusFctContract contract = new HlsCusFctContract();
                contract.setContractId(withdraw.getContractId());
                contract = hlsCusFctContractMapper.selectByPrimaryKey(contract);
                HlsCusFctProject project = new HlsCusFctProject();
                project.setProjectId(contract.getProjectId());
                project = hlsCusFctProjectMapper.selectByPrimaryKey(project);
                //利息罚息利率
                Double penaltyRate = project.getPenaltyRate();
                //本金罚息利率
                Double principalPenaltyRate = project.getPrincipalPenaltyRate();

                HlsCusFctContractQuotation quotation = new HlsCusFctContractQuotation();
                quotation.setContractId(withdraw.getContractId());
                quotation = hlsCusFctContractQuotationMapper.fctContractQuotationDetailLineQuery(quotation).get(0);

                Double exemptionPeriod = quotation.getExemptionPeriod();
                if (exemptionPeriod == null || exemptionPeriod.equals("")) {
                    exemptionPeriod = 0.0d;
                }

                //计算已收部分
                HlsCusCshWriteOff cshWriteOff = new HlsCusCshWriteOff();
                cshWriteOff.setWithdrawCfId(ccc.getWithdrawCfId());
                cshWriteOff.setDueDate(ccc.getDueDate());
                List<HlsCusCshWriteOff> cshWriteOffList = cshWriteOffMapper.selectPenaltyWriteOffFct(cshWriteOff);


                double received = 0.0d;

                double writeOffDueAmount = 0.0d;

                for (HlsCusCshWriteOff cwo : cshWriteOffList) {
                    if (cwo.getCfItem().compareTo(501L) == 0) {
                        writeOffDueAmount = writeOffDueAmount + cwo.getWriteOffDueAmount();
                        received = received + cwo.getWriteOffDueAmount() * (int) (((cwo.getPenaltyCalcDate().getTime() - ccc.getDueDate().getTime()) / (24 * 3600 * 1000)) - exemptionPeriod) * principalPenaltyRate;
                    } else if (cwo.getCfItem().compareTo(502L) == 0) {
                        writeOffDueAmount = writeOffDueAmount + cwo.getWriteOffDueAmount();
                        received = received + cwo.getWriteOffDueAmount() * (int) (((cwo.getPenaltyCalcDate().getTime() - ccc.getDueDate().getTime()) / (24 * 3600 * 1000)) - exemptionPeriod) * penaltyRate;
                    }
                }
                //查询罚息减免部分
                double derateAmount = 0.0D;
                HlsCusConDebtExemptionReqCf conDebtExemptionReqCf = new HlsCusConDebtExemptionReqCf();
                conDebtExemptionReqCf.setWithdrawCfId(ccc.getWithdrawCfId());
                List<HlsCusConDebtExemptionReqCf> list = conDebtExemptionReqCfMapper.selectReqCfByCashflowId(conDebtExemptionReqCf);
                for (HlsCusConDebtExemptionReqCf cf : list) {
                    if (cf.getExemptionAmount() != null) {
                        derateAmount += cf.getExemptionAmount();
                    }
                }
                double unreceived = 0.0D;
                //计算未收部分 (未核销罚息)
                if (ccc.getCfItem().compareTo(501L) == 0) {
                    unreceived = (ccc.getDueAmount() - writeOffDueAmount) * ((int) ((dayEndDate.getTime() - ccc.getDueDate().getTime()) / (24 * 3600 * 1000) - exemptionPeriod)) * penaltyRate;
                } else if (ccc.getCfItem().compareTo(502L) == 0) {
                    unreceived = (ccc.getDueAmount() - writeOffDueAmount) * ((int) ((dayEndDate.getTime() - ccc.getDueDate().getTime()) / (24 * 3600 * 1000) - exemptionPeriod)) * penaltyRate;
                }

                double profileTotalAmount = unreceived + received - derateAmount;

                //查询该现金流是否存在罚息现金流，有就修改，没有就添加
                HlsCusFctContractWithdrawCf withdrawCf = new HlsCusFctContractWithdrawCf();
                withdrawCf.setGeneratedSourceDocId(ccc.getWithdrawCfId());
                withdrawCf.setGeneratedSource("DAYEND");
                withdrawCf.setOverdueStatus("N");

                List<HlsCusFctContractWithdrawCf> withdrawCfs = hlsCusFctContractWithdrawCfMapper.selectCalcPenaltyCashFlowBySourceId(withdrawCf);
                if (withdrawCfs.size() > 0) {
                    for (HlsCusFctContractWithdrawCf c : withdrawCfs) {
                        c.setDueDate(dayEndDate);
                        c.setDueAmount(profileTotalAmount);
                        if (new BigDecimal(c.getWriteOffAmount()).compareTo(new BigDecimal(0)) == 0) {
                            c.setWriteOffFlag("NOT");
                        } else if (new BigDecimal(c.getWriteOffAmount()).compareTo(new BigDecimal(c.getDueAmount())) >= 0) {
                            c.setWriteOffFlag("FULL");
                        } else {
                            c.setWriteOffFlag("PARTIAL");
                        }
                        hlsCusFctContractWithdrawCfMapper.updateByPrimaryKeySelective(c);
                    }

                } else {
                    HlsCusFctContractWithdrawCf fctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
                    fctContractWithdrawCf.setWithdrawId(ccc.getWithdrawId());
                    fctContractWithdrawCf.setContractId(ccc.getContractId());
                    fctContractWithdrawCf.setTimes(ccc.getTimes());
                    fctContractWithdrawCf.setCfItem(509L);
                    fctContractWithdrawCf.setCfType(9L);
                    fctContractWithdrawCf.setCfDirection("INFLOW");
                    fctContractWithdrawCf.setCfStatus("RELEASE");
                    fctContractWithdrawCf.setDueDate(dayEndDate);
                    fctContractWithdrawCf.setDueAmount(profileTotalAmount);
                    fctContractWithdrawCf.setWriteOffFlag("NOT");
                    fctContractWithdrawCf.setOverdueStatus("N");
                    fctContractWithdrawCf.setGeneratedSource("DAYEND");
                    fctContractWithdrawCf.setGeneratedSourceDocId(ccc.getWithdrawCfId());
                    hlsCusFctContractWithdrawCfMapper.insertSelective(fctContractWithdrawCf);
                }
            }
        }

    }
}
