package hls.core.fct.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fct.dto.HlsCusFctProject;
import hls.core.fct.dto.HlsCusFctProjectBasicTrx;
import hls.core.fct.mapper.HlsCusFctProjectBasicTrxMapper;
import hls.core.fct.mapper.HlsCusFctProjectMapper;
import hls.core.fct.service.FctProjectBasicTrxService;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.user.service.LoginUserInfoService;
import hls.core.wfl.service.IActivitiStartService;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional(rollbackFor = Exception.class)
public class FctProjectBasicTrxServiceImpl extends BaseServiceImpl<HlsCusFctProjectBasicTrx> implements FctProjectBasicTrxService{

    @Autowired
    private HlsCusFctProjectBasicTrxMapper hlsCusFctProjectBasicTrxMapper;
    @Autowired
    private HlsCusFctProjectMapper hlsCusFctProjectMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private HlsEmployeeMapper employeeMapper;

    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private LoginUserInfoService loginUserInfoService;

    public List<HlsCusFctProjectBasicTrx> queryAllTrxInfo(IRequest requestContext, HlsCusFctProjectBasicTrx hlsCusFctProjectBasicTrx, int page, int pageSize){
        PageHelper.startPage(page, pageSize);
        Map<String, Object> map = new HashedMap();
        map.put("projectName",hlsCusFctProjectBasicTrx.getProjectName());
        map.put("projectNumber",hlsCusFctProjectBasicTrx.getProjectNumber());
        map.put("trxAmountFrom",hlsCusFctProjectBasicTrx.getTrxAmountFrom());
        map.put("trxAmountTo",hlsCusFctProjectBasicTrx.getTrxAmountTo());
        map.put("basicTrxId",hlsCusFctProjectBasicTrx.getBasicTrxId());
        return hlsCusFctProjectBasicTrxMapper.queryAllTrxInfo(map);
    }
    //插入事件表
    public void addTrxEvent(IRequest request,HlsCusFctProjectBasicTrx hlsCusFctProjectBasicTrx){
        Map<String,Object> paramsEvent = new HashMap<String,Object>();
        String userName=loginUserInfoService.queryUserInfo(request.getEmployeeCode()).get(0).getUserName();
        String meg = userName +"创建了"+hlsCusFctProjectBasicTrx.getProjectName()+"项目的基础交易:"+hlsCusFctProjectBasicTrx.getBasicDealsNumber();
        paramsEvent.put("message",meg);
        paramsEvent.put("noticeTitle","基础交易审核创建");
        paramsEvent.put("noticeType","NOTICE");
        paramsEvent.put("url","");
        paramsEvent.put("level",1L);
        paramsEvent.put("sourceUserId",request.getUserId());
        sysEventService.eventSave(request,hlsCusFctProjectBasicTrx.getBasicTrxId(),hlsCusFctProjectBasicTrx.getDocumentCategory(),hlsCusFctProjectBasicTrx.getDocumentType(),"FCT","FCT_PROJECT_BASIC_TRX.CREATE","P2D",paramsEvent);
    }

    public void updateTrxEvent(IRequest request,HlsCusFctProjectBasicTrx hlsCusFctProjectBasicTrx){
        Map<String,Object> paramsEvent = new HashMap<String,Object>();
        String userName=loginUserInfoService.queryUserInfo(request.getEmployeeCode()).get(0).getUserName();
        String meg = userName +"更新了"+hlsCusFctProjectBasicTrx.getProjectName()+"项目的基础交易:"+hlsCusFctProjectBasicTrx.getBasicDealsNumber();
        paramsEvent.put("message",meg);
        paramsEvent.put("noticeTitle","基础交易审核更新");
        paramsEvent.put("noticeType","NOTICE");
        paramsEvent.put("url","");
        paramsEvent.put("level",1L);
        paramsEvent.put("sourceUserId",request.getUserId());
        sysEventService.eventSave(request,hlsCusFctProjectBasicTrx.getBasicTrxId(),hlsCusFctProjectBasicTrx.getDocumentCategory(),hlsCusFctProjectBasicTrx.getDocumentType(),"FCT","FCT_PROJECT_BASIC_TRX.UPDATE","P2D",paramsEvent);
    }
    //提交审批
    public List<HlsCusFctProjectBasicTrx> submitApproval(IRequest iRequest,List<HlsCusFctProjectBasicTrx> list){
        //获取需要审批的额度信息
        Long basicTrxId = list.get(0).getBasicTrxId();
        Long projectId = list.get(0).getProjectId();
        HlsCusFctProjectBasicTrx fctProjectBasicTrx=hlsCusFctProjectBasicTrxMapper.selectByPrimaryKey(basicTrxId);
        HlsCusFctProject fctProject = hlsCusFctProjectMapper.selectByPrimaryKey(projectId);
        List<HlsCusFctProjectBasicTrx> fctProjectBasicTrxList=new ArrayList<>();
        fctProjectBasicTrxList.add(fctProjectBasicTrx);
        databaseLockProvider.lock(fctProjectBasicTrx);
        //获取申请人
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
        //开始流程
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_PROJECT_BASIC");
        activitiStartService.start(iRequest, fctProjectBasicTrxList, params);
        //修改审核状态
        fctProjectBasicTrx.setBasicDealsStatus("APPROVING");
        self().updateByPrimaryKeySelective(iRequest,fctProjectBasicTrx);

        //插入事件表
        Map<String,Object> paramsEvent = new HashMap<String,Object>();
        //String userName=(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg=userName+"创建了"+fctProject.getProjectName()+"项目的基础交易审核"+fctProject.getProjectNumber();
        paramsEvent.put("message",msg);
        paramsEvent.put("noticeTitle","基础交易审核审批");
        paramsEvent.put("noticeType","NOTICE");
        paramsEvent.put("url","");
        paramsEvent.put("level",1L);
        sysEventService.eventSave(iRequest,fctProjectBasicTrx.getBasicTrxId(),fctProjectBasicTrx.getDocumentCategory(),fctProjectBasicTrx.getDocumentType(),"BAC","FCT_PROJECT_BASIC_TRX","P2D",paramsEvent);
        return fctProjectBasicTrxList;
    }

}