package hls.core.fct.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fct.dto.*;
import hls.core.fct.mapper.HlsCusFctProjectAttachmentMapper;
import hls.core.fct.mapper.HlsCusFctProjectBpMapper;
import hls.core.fct.mapper.HlsCusFctProjectMapper;
import hls.core.fct.mapper.HlsCusFctProjectQuotationMapper;
import hls.core.fct.service.FctProjectAttachmentService;
import hls.core.fct.service.HlsCusFctProjectCustomService;
import hls.core.fct.service.FctProjectService;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.hls.components.CalculateUtil;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.mapper.HlsCusHlsCreditLineMapper;
import hls.core.sys.dto.SysDocumentList;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.mapper.SysDocumentListMapper;
import hls.core.user.dto.LoginUserInfo;
import hls.core.user.service.LoginUserInfoService;
import hls.core.wfl.service.IActivitiStartService;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpSession;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class FctProjectServiceImpl extends BaseServiceImpl<HlsCusFctProject> implements FctProjectService {

    @Autowired
    private HlsCusFctProjectMapper mapper;
    @Autowired
    SysEventService sysEventService;
    @Autowired(required = false)
    private HlsCusFctProjectCustomService hlsCusFctProjectCustomService;
    @Autowired
    DatabaseLockProvider databaseLockProvider;
    @Autowired
    IActivitiStartService activitiStartService;
    @Autowired
    SysDocumentListMapper sysDocumentListMapper;
    @Autowired
    FctProjectAttachmentService fctProjectAttachmentService;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private LoginUserInfoService loginUserInfoService;

    @Autowired
    private HlsCusHlsCreditLineMapper hlsCusHlsCreditLineMapper;

    @Autowired
    private HlsCusFctProjectAttachmentMapper fctProjectAttachmentMapper;
    @Autowired
    private HlsCusFctProjectQuotationMapper fctProjectQuotationMapper;

    @Autowired
    private HlsCusFctProjectBpMapper hlsCusFctProjectBpMapper;

    public static final String PROPERTY_FCT_PROJECT_CREDIT_MODIFY = "FCT_PROJECT_CREDIT.MODIFY";
    public static final String PROPERTY_FCT_PROJECT_CREDIT_SUBMIT = "FCT_PROJECT_CREDIT.SUBMIT";

    public static final String PROPERTY_FCT_CONTRACT_CONTENT_MODIFY = "FCT_CONTRACT_CONTENT.MODIFY";
    public static final String PROPERTY_FCT_CONTRACT_CONTENT_SUBMIT = "FCT_CONTRACT_CONTENT.SUBMIT";

    public static final String PROPERTY_FCT_CONTRACT_SIGN_SUBMIT = "FCT_CONTRACT_SIGN.SUBMIT";

    @Override
    public List<FctProject> selectSetupProjectDetail(FctProject fctProject, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        return mapper.selectSetupProjectDetail(fctProject);
    }

    @Override
    public List<FctProject> selectPrjStatusInfo(FctProject fctProject, HttpSession session) {
        Map<String, Object> map = new HashedMap();
        fctProject.setManagerId((Long) session.getAttribute("userId"));
        String[] paramBusStatus = null;
        String[] paramFctStatus = null;
        if (fctProject.getBusTypeStatusList() != null) {
            paramBusStatus = fctProject.getBusTypeStatusList().split("、");
        }
        if (fctProject.getFactorTypeStatusList() != null) {
            paramFctStatus = fctProject.getFactorTypeStatusList().split("、");
        }
        map.put("busTypeStatusList", paramBusStatus);
        map.put("factorTypeStatusList", paramFctStatus);
        map.put("companyId", session.getAttribute("companyId"));
        List<FctProject> list = mapper.selectPrjStatusInfo(map);
        Long countPercent = 0L;
        for (int i = 0; i <= list.size() - 2; i++) {
            countPercent = list.get(i).getPrjCountPercent() + countPercent;
        }
        list.get(list.size() - 1).setPrjCountPercent(100 - countPercent);
        return list;
    }

    @Override
    public List<FctProject> selectAllSetupProject(IRequest requestContext, FctProject fctProject, int page, int pageSize) throws ParseException {
        PageHelper.startPage(page, pageSize);
        Map<String, Object> map = new HashedMap();
        map.put("companyId", fctProject.getCompanyId());
        map.put("managerId", requestContext.getUserId());
        String[] paramBusStatus = null;
        String[] paramFctStatus = null;
        if (fctProject.getBusTypeStatusList() != null) {
            paramBusStatus = fctProject.getBusTypeStatusList().split("、");
        }
        if (fctProject.getFactorTypeStatusList() != null) {
            paramFctStatus = fctProject.getFactorTypeStatusList().split("、");
        }
        map.put("busTypeStatusList", paramBusStatus);
        map.put("factorTypeStatusList", paramFctStatus);
        if (fctProject.getBpNameCode() != null) {
            map.put("bpNameCode", fctProject.getBpNameCode());
        }
        if (fctProject.getPrjNameCode() != null) {
            map.put("prjNameCode", fctProject.getPrjNameCode());
        }
        if (fctProject.getProjectStatus() != null) {
            map.put("projectStatus", fctProject.getProjectStatus());
        }
        if (fctProject.getProjectTrxStatus() != null) {
            map.put("projectTrxStatus", fctProject.getProjectTrxStatus());
        }
        return mapper.selectAllSetupProject(map);
    }

    //明细查询
    @Override
    public List<HlsCusFctProject> fctProjectDetailQuery(HlsCusFctProject fctProject) {

        return mapper.fctProjectDetailQuery(fctProject);
    }

    @Override
    public List<HlsCusFctProject> queryFctCreditInfo(HlsCusFctProject fctProject) {
        if(fctProject.getProjectId()!=null&&fctProject.getProjectId()!=0){
            return mapper.queryFctCreditInfoProjectId(fctProject);
        }else{
            return mapper.queryFctCreditInfoTrxId(fctProject);
        }

    }

    //本月新增
    @Override
    public Double currencyMonthAddQuery() {
        //查出所有的fct_project
        HlsCusFctProject fctProject = new HlsCusFctProject();
        fctProject.setProjectStatus("APPROVED");
        List<HlsCusFctProject> fctProjectList = self().fctProjectDetailQuery(fctProject);

        //所有的授信（立项通过）
        Integer totalCount = fctProjectList.size();

        //本月新增授信
        Calendar sysDate = Calendar.getInstance();
        sysDate.set(Calendar.DAY_OF_MONTH, 1);//设置为1号,当前日期既为本月第一天
        Date currencyMonthFirstDay = sysDate.getTime();
        Integer currencyMonthAddCount = 0;

        for (int i = 0; i < fctProjectList.size(); i++) {
            if (fctProjectList.get(i).getCreationDate().getTime() >= currencyMonthFirstDay.getTime() && fctProjectList.get(i).getCreationDate().getTime() <= new Date().getTime()) {
                currencyMonthAddCount++;
            }
        }
        Double rate = 0D;
        if (totalCount != 0) {
            rate = CalculateUtil.div(currencyMonthAddCount.doubleValue(), totalCount.doubleValue(), 2);
        }

        return rate;
    }

    //授信 总数，状态，数量
    @Override
    public Map<String, Integer> creditStatusQuery(Long companyId) {
        HlsCusFctProject fctProject = new HlsCusFctProject();
        fctProject.setProjectStatus("APPROVED");
        fctProject.setCompanyId(companyId);
        List<HlsCusFctProject> fctProjectList = self().fctProjectDetailQuery(fctProject);
        Integer totalCount = fctProjectList.size();

        Map<String, Integer> statusMap = new HashMap<String, Integer>();
        statusMap.put("totalCount", totalCount);
        for (FctProject project : fctProjectList) {
            if (statusMap.containsKey(project.getCreditStatus())) {
                statusMap.put(project.getCreditStatus(), statusMap.get(project.getCreditStatus()) + 1);
            } else {
                statusMap.put(project.getCreditStatus(), 1);
            }
        }
        return statusMap;
    }


    private void fctCreditValidate(HlsCusFctProject hlsCusFctProject) {
        if ("APPROVING".equals(hlsCusFctProject.getCreditStatus()) || "APPROVED".equals(hlsCusFctProject.getCreditStatus())) {
            throw new IllegalArgumentException("授信状态错误，无法保存！");
        }
        //
    }

    //授信保存
    @Override
    public void fctCreditSave(IRequest iRequest, HlsCusFctProject hlsCusFctProject) {
        if (hlsCusFctProjectCustomService != null && !hlsCusFctProjectCustomService.beforeSave(iRequest, hlsCusFctProject)) {
            return;
        }
        //状态校验
        fctCreditValidate(hlsCusFctProject);
        hlsCusFctProject = self().updateByPrimaryKey(iRequest, hlsCusFctProject);
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        //String userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg = userName + "修改了一条保理项目" + hlsCusFctProject.getProjectName() + hlsCusFctProject.getProjectNumber();
        evenParams.put("message", msg);
        evenParams.put("noticeTitle", "保理项目修改");
        evenParams.put("url", "");
        evenParams.put("level", 2L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(iRequest, hlsCusFctProject.getProjectId(), hlsCusFctProject.getDocumentCategory(), hlsCusFctProject.getDocumentType(), "FCT", PROPERTY_FCT_PROJECT_CREDIT_MODIFY, "P2D", evenParams);
        if (hlsCusFctProjectCustomService != null) {
            hlsCusFctProjectCustomService.afterSave(iRequest, hlsCusFctProject);
        }
    }

    //授信提交
    @Override
    public void fctCreditSubmit(IRequest iRequest, List<HlsCusFctProject> hlsCusFctProjectList) {
        if (hlsCusFctProjectCustomService != null && !hlsCusFctProjectCustomService.beforeSubmit(iRequest, hlsCusFctProjectList)) {
            return;
        }
        try {
            //提交前先执行保存
            HlsCusFctProject hlsCusFctProject = hlsCusFctProjectList.get(0);
            fctCreditSave(iRequest, hlsCusFctProject);
            //锁表
            databaseLockProvider.lock(hlsCusFctProject);
            //插入事件
            Map<String, Object> evenParams = new HashMap<>();
            //String userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
            String userName ="";
            if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
                userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
            }
            String msg = userName + "提交了一条保理项目" + hlsCusFctProject.getProjectName() + hlsCusFctProject.getProjectNumber();
            evenParams.put("message", msg);
            evenParams.put("noticeTitle", "授信审批提交");
            evenParams.put("url", "");
            evenParams.put("level", 1L);
            evenParams.put("noticeType", "NOTICE");

            sysEventService.eventSave(iRequest, hlsCusFctProject.getProjectId(), hlsCusFctProject.getDocumentCategory(), hlsCusFctProject.getDocumentType(), "FCT", PROPERTY_FCT_PROJECT_CREDIT_SUBMIT, "P2D", evenParams);
            //启动工作流
            Map<String, Object> params = new HashMap<String, Object>();
            params.put("workFlowType", "FCT_PROJECT_CREDIT");
            params.put("projectId", hlsCusFctProject.getProjectId());
            activitiStartService.start(iRequest, hlsCusFctProjectList, params);
            //更新状态
            mapper.updateCreditStatus(hlsCusFctProject.getProjectId(), "APPROVING", iRequest.getUserId(), new Date());

            if (hlsCusFctProjectCustomService != null) {
                hlsCusFctProjectCustomService.afterSubmit(iRequest, hlsCusFctProjectList);
            }
        } catch (Exception e) {
            throw e;
        }

    }


    //立项创建之后 插入附件资料清单
    @Override
    public void hlsCreditAttachmentCreate(IRequest iRequest, HlsCusFctProject hlsCusFctProject) {
        //通过类型 创建对应的资料模版
        //插入通用的

        HlsCusFctProject fctProject = new HlsCusFctProject();
        fctProject.setProjectId(hlsCusFctProject.getProjectId());
        List<HlsCusFctProject> fctProjectList = mapper.fctProjectDetailQuery(fctProject);

        if (fctProjectList.size() > 0) {
            fctProject = fctProjectList.get(0);

            List<SysDocumentList> sysDocumentLists = new ArrayList<SysDocumentList>();

            SysDocumentList sysDocumentList = new SysDocumentList();
            sysDocumentList.setDocumentCategory("FCT_PROJECT");
            sysDocumentList.setDocumentType("NORMAL");
            List<SysDocumentList> sysDocumentLists1 = sysDocumentListMapper.sysDocumentListQuery(sysDocumentList);

            sysDocumentList.setDocumentCategory("FCT_PROJECT");
            sysDocumentList.setDocumentType("GUARANTOR");
            List<SysDocumentList> sysDocumentLists2 = sysDocumentListMapper.sysDocumentListQuery(sysDocumentList);


            sysDocumentLists.addAll(sysDocumentLists1);
            sysDocumentLists.addAll(sysDocumentLists2);
            if ("OPEN_WITHOUT_RECOURSE".equals(fctProject.getDocumentType()) || "OPEN_RECOURSE".equals(fctProject.getDocumentType())) {
                sysDocumentList.setDocumentCategory("FCT_PROJECT");
                sysDocumentList.setDocumentType("OPEN_FCT");
                List<SysDocumentList> sysDocumentLists3 = sysDocumentListMapper.sysDocumentListQuery(sysDocumentList);
                sysDocumentLists.addAll(sysDocumentLists3);
            }

            for (SysDocumentList documentList : sysDocumentLists) {
                HlsCusFctProjectAttachment fctProjectAttachment = new HlsCusFctProjectAttachment();
                fctProjectAttachment.setProjectId(fctProject.getProjectId());
                fctProjectAttachment.setSourceType("FCT_PROJECT");
                fctProjectAttachment.setSourceId(documentList.getDocumentId());
                fctProjectAttachment.setBpId(fctProject.getBpId());
                fctProjectAttachment.setDocumentName(documentList.getDocumentListName());
                fctProjectAttachmentService.insertSelective(iRequest, fctProjectAttachment);
            }

        }
    }


    private void ContentValidate(HlsCusFctProject hlsCusFctProject) {
        if ("APPROVING".equals(hlsCusFctProject.getContractStatus()) || "APPROVED".equals(hlsCusFctProject.getContractStatus())) {
            throw new IllegalArgumentException("合同状态错误，无法保存！");
        }
    }

    @Override
    public void fctContentSave(IRequest iRequest, HlsCusFctProject hlsCusFctProject) {
        if (hlsCusFctProjectCustomService != null && !hlsCusFctProjectCustomService.beforeContentSave(iRequest, hlsCusFctProject)) {
            return;
        }
        //状态校验
        ContentValidate(hlsCusFctProject);
        hlsCusFctProject = self().updateByPrimaryKey(iRequest, hlsCusFctProject);
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        //String userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String meg = userName + "更新了" + hlsCusFctProject.getProjectNumber() + hlsCusFctProject.getProjectName() + "的合同文本";
        /*evenParams.put("message", "合同文本审批" + hlsCusFctProject.getProjectNumber() + "更新");*/
        evenParams.put("message", meg);
        evenParams.put("noticeTitle", "合同文本审批更新");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(iRequest, hlsCusFctProject.getProjectId(), hlsCusFctProject.getDocumentCategory(), hlsCusFctProject.getDocumentType(), "FCT", PROPERTY_FCT_CONTRACT_CONTENT_MODIFY, "P2D", evenParams);
        if (hlsCusFctProjectCustomService != null) {
            hlsCusFctProjectCustomService.afterContentSave(iRequest, hlsCusFctProject);
        }
    }

    @Override
    public void fctContentSubmit(IRequest iRequest, List<HlsCusFctProject> hlsCusFctProjectList) {
        if (hlsCusFctProjectCustomService != null && !hlsCusFctProjectCustomService.beforeContentSubmit(iRequest, hlsCusFctProjectList)) {
            return;
        }

        //提交前先执行保存
        HlsCusFctProject hlsCusFctProject = hlsCusFctProjectList.get(0);
        fctContentSave(iRequest, hlsCusFctProject);
        //锁表
        //判断合同文本是否生成
        HlsCusFctProjectAttachment hlsCusFctProjectAttachment = new HlsCusFctProjectAttachment();
        hlsCusFctProjectAttachment.setProjectId(hlsCusFctProject.getProjectId());
        hlsCusFctProjectAttachment.setSourceType("FCT_CONTRACT");

        if (fctProjectAttachmentMapper.fctContractContentQuery(hlsCusFctProjectAttachment).size() < 1) {
            throw new IllegalArgumentException("请先点击合同文本生成，再提交审批！");
        }

        //额度校验
        HlsCusHlsCreditLine hlsCusHlsCreditLine = new HlsCusHlsCreditLine();
        hlsCusHlsCreditLine.setCreditLineId(hlsCusFctProject.getCreditLineId());
        hlsCusHlsCreditLine = hlsCusHlsCreditLineMapper.selectByPrimaryKey(hlsCusHlsCreditLine);

        List<HlsCusFctProjectAttachment> list = fctProjectAttachmentMapper.fctContractContentQuery(hlsCusFctProjectAttachment);

        HlsCusFctProjectQuotation fctProjectQuotation = new HlsCusFctProjectQuotation();
        fctProjectQuotation.setProjectId(hlsCusFctProject.getProjectId());
        List<HlsCusFctProjectQuotation> fctProjectQuotationList = fctProjectQuotationMapper.fctProjectQuotationDetailQuery(fctProjectQuotation);

        if (fctProjectQuotationList.get(0).getFinanceAmount().compareTo(list.stream().mapToDouble(FctProjectAttachment::getCreditAmount).sum()) == -1) {
            Number amount = fctProjectQuotationList.get(0).getFinanceAmount();
            String financeAmount =  amount.toString();

            BigDecimal b = new BigDecimal(financeAmount);
            throw new IllegalArgumentException("合同授信超过项目授信额度，当前项目授信额度为："+b.toPlainString());
        }


        databaseLockProvider.lock(hlsCusFctProject);
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "合同文本审批" + hlsCusFctProject.getProjectNumber() + "提交");
        evenParams.put("noticeTitle", "合同文本审批提交");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");

        sysEventService.eventSave(iRequest, hlsCusFctProject.getProjectId(), hlsCusFctProject.getDocumentCategory(), hlsCusFctProject.getDocumentType(), "FCT", PROPERTY_FCT_CONTRACT_CONTENT_SUBMIT, "P2D", evenParams);
        //启动工作流
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_CONTRACT_CONTENT");
        params.put("projectId", hlsCusFctProject.getProjectId());
        activitiStartService.start(iRequest, hlsCusFctProjectList, params);
        //更新状态
        mapper.updateContractStatus(hlsCusFctProject.getProjectId(), "PROCESS_IN_FILE_APPROVAL", iRequest.getUserId(), new Date());

        if (hlsCusFctProjectCustomService != null) {
            hlsCusFctProjectCustomService.afterContentSubmit(iRequest, hlsCusFctProjectList);
        }

    }


    private void signValidate(HlsCusFctProject hlsCusFctProject) {
        if (!"FILE_APPROVED".equals(hlsCusFctProject.getContractStatus())) {
            throw new IllegalArgumentException("状态错误，无法提交！");
        }
    }

    @Override
    public void fctSignSubmit(IRequest iRequest, List<HlsCusFctProject> hlsCusFctProjectList) {
        if (hlsCusFctProjectCustomService != null && !hlsCusFctProjectCustomService.beforeSignSubmit(iRequest, hlsCusFctProjectList)) {
            return;
        }
        HlsCusFctProject hlsCusFctProject = hlsCusFctProjectList.get(0);
        List<HlsCusFctProject> hlsCusFctProjects = self().fctProjectDetailQuery(hlsCusFctProject);
        hlsCusFctProject = hlsCusFctProjects.get(0);
        /*signValidate(hlsCusFctProject);*/
        //fctCreditSave(iRequest, hlsCusFctProject);
        databaseLockProvider.lock(hlsCusFctProject);
        //插入事件


        hlsCusFctProject = self().selectByPrimaryKey(iRequest, hlsCusFctProject);

        Map<String, Object> evenParams = new HashMap<>();
        evenParams.put("message", "签约审批" + hlsCusFctProject.getProjectNumber() + "提交");
        evenParams.put("noticeTitle", "签约审批提交");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(iRequest, hlsCusFctProject.getProjectId(), hlsCusFctProject.getDocumentCategory(), hlsCusFctProject.getDocumentType(), "FCT", PROPERTY_FCT_CONTRACT_SIGN_SUBMIT, "P2D", evenParams);

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_CONTRACT_SIGN");
        params.put("projectId", hlsCusFctProject.getProjectId());
        mapper.updateContractStatus(hlsCusFctProject.getProjectId(), "PROCESS_IN_SIGN", iRequest.getUserId(), new Date());

        activitiStartService.start(iRequest, hlsCusFctProjectList, params);
        if (hlsCusFctProjectCustomService != null) {
            hlsCusFctProjectCustomService.afterSignSubmit(iRequest, hlsCusFctProjectList);
        }
       /* *//*修改合同状态*//*
        HlsCusFctProjectAttachment dto = new HlsCusFctProjectAttachment();
        Long projectId = hlsCusFctProject.getProjectId();
        dto.setProjectId(projectId);
        dto.setStatus("UN_SIGN");
        List<HlsCusFctProjectAttachment> hlsCusFctProjectAttachmentList = new ArrayList<>();
        hlsCusFctProjectAttachmentList = fctProjectAttachmentMapper.fctProjectAttachmentForSignProcessQuery(dto);
        for(int i=0;i<hlsCusFctProjectAttachmentList.size();i++){
            hlsCusFctProjectAttachmentList.get(i).setStatus("SIGNING");
            hlsCusFctProjectAttachmentList.get(i).set__status("update");
        }
        fctProjectAttachmentService.batchUpdate(iRequest,hlsCusFctProjectAttachmentList);*/
    }

    @Override
    public List<HlsCusFctProject> submitApproval(IRequest iRequest, List<HlsCusFctProject> hlsCusFctProjectList) {

        //获取需要的额度
        Long projectId = hlsCusFctProjectList.get(0).getProjectId();
        HlsCusFctProject fctProject = mapper.selectByPrimaryKey(projectId);
        List<HlsCusFctProject> fctProjectParam = new ArrayList<>();
        fctProjectParam.add(fctProject);
        databaseLockProvider.lock(fctProject);

        //校验 授信主体要和授信方保持一致
        HlsCusHlsCreditLine hlsCusHlsCreditLine = new HlsCusHlsCreditLine();
        hlsCusHlsCreditLine.setCreditLineId(fctProject.getCreditLineId());
        hlsCusHlsCreditLine=   hlsCusHlsCreditLineMapper.selectByPrimaryKey(hlsCusHlsCreditLine);

        HlsCusFctProjectBp hlsCusFctProjectBp = new HlsCusFctProjectBp();
        hlsCusFctProjectBp.setProjectId(projectId);
        hlsCusFctProjectBp.setCreditGrantorParty("Y");

        List<HlsCusFctProjectBp> bpList =  hlsCusFctProjectBpMapper.fctProjectBpQuery(hlsCusFctProjectBp);
        if(bpList.size()<1){
            throw new IllegalArgumentException("至少有一个授信方");
        }
        if(bpList.size()>1){
            throw new IllegalArgumentException("只能有有一个授信方");
        }
        Long k =0L;
//        for(int i = 0 ; i<bpList.size() ; i++){
//           if("SELLER".equals(bpList.get(i).getBpRoleType())){
//                k=k+1;
//            }
//        }
        HlsCusFctProjectBp fctProjectBp = new HlsCusFctProjectBp();
        fctProjectBp.setProjectId(projectId);
        List<HlsCusFctProjectBp> bpLists =  hlsCusFctProjectBpMapper.fctProjectBpQuery(fctProjectBp);
        for(int i = 0 ; i<bpLists.size() ; i++){
            if("SELLER".equals(bpLists.get(i).getBpRoleType())){
                k=k+1;
            }
        }
        if("FACTORING".equals( fctProject.getBusinessType())){
            if(k!=1){
                throw new IllegalArgumentException("正向保理有且仅有一个卖方");
            }
        }else{
            if(k<1){
                throw new IllegalArgumentException("反向保理必须有卖方");
            }
        }
        if(!hlsCusHlsCreditLine.getBpId().equals(bpList.get(0).getBpId())){
            throw new IllegalArgumentException("授信主体要和授信方保持一致");
        }


        //获取申请人
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        //String userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg = userName + "提交了一条保理项目:" + fctProject.getProjectName() + fctProject.getProjectNumber();
        evenParams.put("message", msg);
        evenParams.put("noticeTitle", "立项审批");
        evenParams.put("url", "");
        evenParams.put("level", 1L);
        evenParams.put("noticeType", "NOTICE");
        sysEventService.eventSave(iRequest, fctProject.getProjectId(), fctProject.getDocumentCategory(), fctProject.getDocumentType(), "FCT", "FCT_PROJECT", "P2D", evenParams);
        //开始流程
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "FCT_PROJECT_HLSCUS");
        activitiStartService.start(iRequest, fctProjectParam, params);
        //修改合同状态
        fctProject.setProjectStatus("APPROVING");
        self().updateByPrimaryKeySelective(iRequest, fctProject);


        return fctProjectParam;
    }

    //立项创建事件
    public void addFctProjectEvent(IRequest iRequest, HlsCusFctProject hlsCusFctProject) {
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        LoginUserInfo dto = new LoginUserInfo();
        //String userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg = userName + "创建了一条保理项目:" + hlsCusFctProject.getProjectName() + hlsCusFctProject.getProjectNumber();
        evenParams.put("message", msg);
        evenParams.put("noticeTitle", "保理项目新建");
        evenParams.put("url", "");
        evenParams.put("level", 2L);
        evenParams.put("noticeType", "NOTICE");
        evenParams.put("sourceUserId", iRequest.getUserId());
        sysEventService.eventSave(iRequest, hlsCusFctProject.getProjectId(), hlsCusFctProject.getDocumentCategory(), hlsCusFctProject.getDocumentType(), "FCT", "FCT_PROJECT.CREATE", "P2D", evenParams);
    }

    //立项更新事件
    public void updateFctProjectEvent(IRequest iRequest, HlsCusFctProject hlsCusFctProject) {
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        //String userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg = userName + "更新了一条保理项目:" + hlsCusFctProject.getProjectName() + hlsCusFctProject.getProjectNumber();
        evenParams.put("message", msg);
        evenParams.put("noticeTitle", "保理项目更新");
        evenParams.put("url", "");
        evenParams.put("level", 2L);
        evenParams.put("noticeType", "NOTICE");
        evenParams.put("sourceUserId", iRequest.getUserId());
        sysEventService.eventSave(iRequest, hlsCusFctProject.getProjectId(), hlsCusFctProject.getDocumentCategory(), hlsCusFctProject.getDocumentType(), "FCT", "FCT_PROJECT.UPDATE", "P2D", evenParams);
    }
}