package hls.core.fct.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hand.hap.attachment.dto.AttachCategory;
import com.hand.hap.attachment.dto.Attachment;
import com.hand.hap.attachment.exception.FileReadIOException;
import com.hand.hap.attachment.service.IAttachCategoryService;
import com.hand.hap.attachment.service.IAttachmentService;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.exception.TokenException;
import hls.core.docx4J.component.BookMarkReplaceComponent;
import hls.core.fct.dto.HlsCusFctContractWithdrawAtt;
import hls.core.fct.service.FctContractWithdrawAttService;
import hls.core.fct.service.FctWithdrawDocxService;
import hls.core.hls.bp.dto.HlsCusSysAttachment;
import hls.core.hls.bp.dto.HlsCusSysFile;
import hls.core.hls.bp.service.HlsSysAttachmentService;
import hls.core.hls.bp.service.HlsSysFileService;
import org.apache.commons.lang.StringUtils;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * @Author: syj
 * @Description:
 * @Date: Created in 2018/1/19 14:51
 * @Modified By:
 */
@Service
@Transactional
public class FctWithdrawDocxServiceImpl implements FctWithdrawDocxService {

    /**
     * 文件不存在提示.
     */
    private static final String FILE_NOT_EXSIT = "未找到文件模板";

    /**
     * 文件下载默认编码.
     */
    private static final String ENC = "UTF-8";


    @Autowired
    private IAttachmentService attachmentService;
    @Autowired
    private HlsSysFileService fileService;
    @Autowired
    private BookMarkReplaceComponent bookMarkReplaceComponent;

    @Autowired
    private FctContractWithdrawAttService fctContractWithdrawAttService;
    @Autowired
    private IAttachCategoryService attachCategoryService;
    @Autowired
    private HlsSysAttachmentService hlsSysAttachmentService;


    public void createDocx(IRequest requestContext, InputStream is, File file, Map<String, Object> params) throws Exception {

        WordprocessingMLPackage wordMLPackage = WordprocessingMLPackage.load(is);//通过输入流构建WordprocessingMLPackage对象

        wordMLPackage = bookMarkReplaceComponent.docxCreateBookMarkReplaceWithText(wordMLPackage, requestContext, params);//将构建的wordMLPackage对象传入方法中

        wordMLPackage.save(file);//将替换后的合同文本保存到服务器上作为备份


    }

    @Override
    public void process(IRequest requestContext, Map<String, Object> params, JSONObject jsonObject) {
        try {
            docxCreateMethod(requestContext, params);
        } catch (TokenException e) {
            e.printStackTrace();
        } catch (FileReadIOException e) {
            e.printStackTrace();
        } catch (Docx4JException e) {
            e.printStackTrace();
        }
    }

    public HlsCusSysFile docxCreateMethod(IRequest requestContext, Map<String, Object> params) throws TokenException, FileReadIOException, Docx4JException {
        Long templetId = Long.parseLong(params.get("templetId").toString());
        Long withdrawId = Long.parseLong(params.get("withdrawId").toString());
        Long withdrawAttachmentId = Long.parseLong(params.get("withdrawAttachmentId").toString());

        String SourceType = "HLS_DOC_FILE_TEMPLET";
        String backSourceType = "";
        Long SourceKey = templetId;

        Attachment attachment = new Attachment();
        HlsCusSysFile returnSysFile = null;
        try {
            attachment = attachmentService.selectAttachByCodeAndKey(requestContext, SourceType, SourceKey.toString());//根据sys_attachment表上的记录获取附件的信息
            HlsCusSysFile fileParam = new HlsCusSysFile();
            HlsCusSysFile sysFile = fileService.queryByAttachmentId(attachment.getAttachmentId());

            if (sysFile != null && StringUtils.isNotBlank(sysFile.getFilePath())) {
                File file = new File(sysFile.getFilePath());
                if (file.exists()) {
                    int fileLength = (int) file.length();//先获取模板文件的大小
                    int fileBackLength = 0;//定义备份文件的大小
                    if (fileLength > 0) {
                        InputStream inStream = new FileInputStream(file);//用inputStram输入流读取本地的docx文件

                        String copyPath = sysFile.getFilePath().concat("_back_").concat(UUID.randomUUID().toString());//定义复制模板的filepath,使用uuid拼接于文件末尾，避免备份文件重名覆盖

                        this.copyModel(copyPath, inStream);//复制模板

                        InputStream modelIs = new FileInputStream(copyPath);//用输入流读取复制后的模板

                        createDocx(requestContext, modelIs, new File(copyPath), params);//生成合同文本

                        fileBackLength = (int) new File(copyPath).length();

                        HlsCusFctContractWithdrawAtt att = new HlsCusFctContractWithdrawAtt();
                        att.setWithdrawAttachmentId(withdrawAttachmentId);
                        att = fctContractWithdrawAttService.selectByPrimaryKey(requestContext, att);

                        if (att.getDocumentName() == null) {
                            att.setDocumentName(sysFile.getFileName());
                            fctContractWithdrawAttService.updateByPrimaryKeySelective(requestContext, att);
                        }


                        HlsCusSysAttachment hsa = new HlsCusSysAttachment();
                        hsa.setSourceKey(withdrawAttachmentId.toString());

                        AttachCategory attachCategory = new AttachCategory();
                        attachCategory.setSourceType("FCT_WITHDRAW_DOCUMENT");
                        List<AttachCategory> lists = attachCategoryService.selectCategories(requestContext, attachCategory);

                        hsa.setCategoryId(lists.get(0).getCategoryId());
                        hsa.setSourceType("FCT_WITHDRAW_DOCUMENT");
                        hsa.setName("FCT_WITHDRAW_DOCUMENT");
                        hsa.setStatus("1");


                        //如果有就更新
                        if (hlsSysAttachmentService.sysAttachmentQuery(hsa).size() < 1) {
                            hsa = hlsSysAttachmentService.insertSelective(requestContext, hsa);
                        } else {
                            hsa = hlsSysAttachmentService.sysAttachmentQuery(hsa).get(0);
                        }

                        List<HlsCusSysFile> hlsCusSysFileList = fileService.selectByAttachmentId(hsa.getAttachmentId());
                        for (int i = 0; i < hlsCusSysFileList.size(); i++) {
                            fileService.deleteByPrimaryKey(hlsCusSysFileList.get(i));
                        }
                        HlsCusSysFile backSysFile = new HlsCusSysFile();
                        String fileBak = sysFile.getFileName().substring(0, sysFile.getFileName().length() - 5);
                        backSysFile.setFileName(fileBak.concat(".docx"));
                        backSysFile.setAttachmentId(hsa.getAttachmentId());
                        backSysFile.setFilePath(copyPath);
                        backSysFile.setFileSize(new BigDecimal(fileBackLength));
                        backSysFile.setFileType(sysFile.getFileType());
                        backSysFile.setUploadDate(new Date());
                        returnSysFile = fileService.insertSelective(requestContext, backSysFile);

                    }
                }
            }
        } catch (IOException e) {
            throw new FileReadIOException();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return returnSysFile;
    }


    /**
     * 复制文件模板,将读取源文件的输入流复制文件存入一个备用的模板文件
     *
     * @param filePath
     * @param is
     * @throws IOException
     */
    private static synchronized void copyModel(String filePath, InputStream is) throws IOException {

        FileOutputStream fos = new FileOutputStream(filePath); //复制出一个模板
        int readData;
        byte[] b = new byte[1024];

        while ((readData = is.read(b)) != -1) {
            fos.write(b, 0, readData);
        }

        fos.flush();
        is.close();
        fos.close();
    }

}
