package hls.core.fct.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fct.dto.FctContractWithdrawCf;
import hls.core.fct.dto.HlsCusFctContractWithdrawCf;
import hls.core.fct.dto.HlsCusGldFctContractFinIncome;
import hls.core.fct.mapper.HlsCusFctContractWithdrawCfMapper;
import hls.core.fct.service.GldFctContractFinIncomeService;
import hls.core.hls.components.CalculateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = Exception.class)
public class GldFctContractFinIncomeServiceImpl extends BaseServiceImpl<HlsCusGldFctContractFinIncome> implements GldFctContractFinIncomeService {
    @Autowired
    HlsCusFctContractWithdrawCfMapper hlsCusFctContractWithdrawCfMapper;


    public Double getFinIncome(Long days, Double intRate, Double interestAccrualBalance) {
        return CalculateUtil.div(CalculateUtil.mul(interestAccrualBalance, intRate) * days, 360D, 2);
    }


    public int getMonth(Date start, Date end) {
        if (start.after(end)) {
            Date t = start;
            start = end;
            end = t;
        }
        Calendar startCalendar = Calendar.getInstance();
        startCalendar.setTime(start);
        Calendar endCalendar = Calendar.getInstance();
        endCalendar.setTime(end);
        Calendar temp = Calendar.getInstance();
        temp.setTime(end);
        temp.add(Calendar.DATE, 1);

        int year = endCalendar.get(Calendar.YEAR) - startCalendar.get(Calendar.YEAR);
        int month = endCalendar.get(Calendar.MONTH) - startCalendar.get(Calendar.MONTH);

        if ((startCalendar.get(Calendar.DATE) == 1) && (temp.get(Calendar.DATE) == 1)) {
            return year * 12 + month + 1;
        } else if ((startCalendar.get(Calendar.DATE) != 1) && (temp.get(Calendar.DATE) == 1)) {
            return year * 12 + month;
        } else if ((startCalendar.get(Calendar.DATE) == 1) && (temp.get(Calendar.DATE) != 1)) {
            return year * 12 + month;
        } else {
            return (year * 12 + month - 1) < 0 ? 0 : (year * 12 + month);
        }
    }


    @Override
    public List<HlsCusGldFctContractFinIncome> fctContractFinIncome(IRequest iRequest, Long withdrawId) {
        HlsCusFctContractWithdrawCf fctContractWithdrawCf = new HlsCusFctContractWithdrawCf();
        fctContractWithdrawCf.setWithdrawId(withdrawId);
        List<HlsCusFctContractWithdrawCf> cfs = hlsCusFctContractWithdrawCfMapper.fctContractWithdrawCfDetailQuery(fctContractWithdrawCf);
        Double intRate = cfs.get(0).getIntRate();
        //收益分摊开始日
        Date startDate = cfs.stream().min(Comparator.comparing(FctContractWithdrawCf::getDueDate)).get().getDueDate();
        //收益分摊结束日
        Date endDate = cfs.stream().max(Comparator.comparing(FctContractWithdrawCf::getDueDate)).get().getDueDate();

        List<HlsCusGldFctContractFinIncome> gldFctContractFinCostList = new ArrayList<>();

        List<Date> dates = new ArrayList<>();
        List<Date> dateList = new ArrayList<>();
        List<Date> receiptDates = new ArrayList<>();
        List<Double> receiptAmounts = new ArrayList<>();
        //分组
        cfs.stream().collect(Collectors.groupingBy(HlsCusFctContractWithdrawCf::getDueDate)).forEach((k, v) -> {
            //先将放款日期 提前还款的日期 排除，之后 会在加回来
            if (v.get(0).getCfItem().compareTo(40L) != 0 && (v.get(0).getCurrentPeriodPrincipal().compareTo(0D) == 0 || v.get(0).getCurrentPeriodPrincipal() == null)) {
                dates.add(v.get(0).getDueDate());
            }
        });
        //排序
        Collections.sort(dates, new Comparator<Date>() {
            @Override
            public int compare(Date o1, Date o2) {
                return o1.compareTo(o2);
            }
        });
        //构建每一段的计提开始日
        int monthBetween = 0;
        //放款日期
        dateList.add(startDate);
        for (int i = 0; i < dates.size() - 1; i++) {
            monthBetween = getMonth(dates.get(i + 1), dates.get(i));
            for (int j = 0; j < monthBetween; j++) {
                //每个月的1号
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(dates.get(i));
                calendar.add(Calendar.MONTH, j);
                calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), 1);
                dateList.add(calendar.getTime());
            }
        }
        //分组
        cfs.stream().collect(Collectors.groupingBy(HlsCusFctContractWithdrawCf::getDueDate)).forEach((k, v) -> {
            //提前还款的日期
            if (v.get(0).getCurrentPeriodPrincipal().compareTo(0D) != 0 && v.get(0).getCurrentPeriodPrincipal() != null) {
                dateList.add(v.get(0).getDueDate());
                receiptDates.add(v.get(0).getDueDate());
                receiptAmounts.add(v.get(0).getCurrentPeriodPrincipal());

            }
        });
        //排序
        Collections.sort(dateList, new Comparator<Date>() {
            @Override
            public int compare(Date o1, Date o2) {
                return o1.compareTo(o2);
            }
        });
        //结束日
        dateList.add(endDate);

        for (int i = 0; i < dateList.size(); i++) {
            String receipt_flag = "N";
            Date periodStartDate = new Date();
            Date periodEndDate = new Date();
            Double interestAccrualBalance = 0D;
            HlsCusGldFctContractFinIncome finCost = new HlsCusGldFctContractFinIncome();
            finCost.setCompanyId(iRequest.getCompanyId());
            finCost.setContractId(cfs.get(0).getContractId());
            finCost.setWithdrawId(cfs.get(0).getWithdrawId());
            finCost.setSourceType("FCT_CONTRACT");
            finCost.setSourceId(cfs.get(0).getContractId());
            finCost.setPostFlag("N");

            Calendar calendar = Calendar.getInstance();

            finCost.set__status("add");
            finCost.setStartDate(dateList.get(i));
            if (i == dateList.size() - 1) {
                calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), 1);
                periodStartDate = calendar.getTime();
                finCost.setStartDate(periodStartDate);
                finCost.setEndDate(endDate);
            } else {
                calendar.setTime(dateList.get(i + 1));
                calendar.add(Calendar.DATE, -1);
                periodEndDate = calendar.getTime();
                finCost.setEndDate(periodEndDate);
            }
            for (int j = 0; j < cfs.size(); j++) {
                if (cfs.get(j).getDueDate().compareTo(finCost.getStartDate()) > -1 && cfs.get(j).getDueDate().compareTo(finCost.getEndDate()) < 1) {
                    finCost.setWithdrawCfId(cfs.get(j).getWithdrawCfId());
                    if (i == 0) {
                        interestAccrualBalance = cfs.get(j).getWriteOffAmount();
                    }
                    interestAccrualBalance = cfs.get(j).getInterestAccrualBalance();
                }
            }

            finCost.setDays(ChronoUnit.DAYS.between(finCost.getStartDate().toInstant(), finCost.getEndDate().toInstant()) + 1);
            finCost.setFinanceIncome(getFinIncome(finCost.getDays(), intRate, interestAccrualBalance));
            for (int k = 0; k < receiptDates.size() - 1; k++) {
                if (receiptDates.get(k).equals(finCost.getStartDate())) {
                    finCost.setDays(ChronoUnit.DAYS.between(finCost.getStartDate().toInstant(), finCost.getEndDate().toInstant()) + 2);
                    finCost.setFinanceIncome(getFinIncome(finCost.getDays(), intRate, interestAccrualBalance) - CalculateUtil.div(CalculateUtil.mul(receiptAmounts.get(k), intRate), 360D, 2));
                }

                if (receiptDates.get(k).equals(gldFctContractFinCostList.get(i - 1).getStartDate())) {
                    finCost.setDays(ChronoUnit.DAYS.between(finCost.getStartDate().toInstant(), finCost.getEndDate().toInstant()));
                    finCost.setFinanceIncome(getFinIncome(finCost.getDays(), intRate, interestAccrualBalance));
                }
            }
            if (i == dateList.size() - 1) {
                finCost.setFinanceIncome(getFinIncome(finCost.getDays(), intRate, interestAccrualBalance) - CalculateUtil.div(CalculateUtil.mul(receiptAmounts.get(receiptDates.size() - 2), intRate), 360D, 2));
            }

            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");
            finCost.setPeriodName(format.format(finCost.getStartDate()));

            gldFctContractFinCostList.add(finCost);
        }


        self().batchUpdate(iRequest, gldFctContractFinCostList);


        return gldFctContractFinCostList;
    }
}