package hls.core.fct.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fct.dto.HlsCreditLineH;
import hls.core.fct.dto.HlsCusHlsCreditLineH;
import hls.core.fct.mapper.HlsCusHlsCreditLineHMapper;
import hls.core.fct.service.HlsCreditLineHService;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.mapper.HlsCusHlsCreditLineMapper;
import hls.core.prj.service.HlsCreditLineService;
import hls.core.sys.event.service.SysEventService;
import hls.core.user.dto.LoginUserInfo;
import hls.core.user.service.LoginUserInfoService;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional(rollbackFor = Exception.class)
public class HlsCreditLineHServiceImpl extends BaseServiceImpl<HlsCusHlsCreditLineH> implements HlsCreditLineHService{
    @Autowired
    private HlsCusHlsCreditLineHMapper hlsCreditLineHMapper;
    @Autowired
    private HlsCusHlsCreditLineMapper hlsCreditLineMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private HlsEmployeeMapper employeeMapper;

    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private LoginUserInfoService loginUserInfoService;
    @Autowired
    private HlsCreditLineService hlsCreditLineService;

    @Override
    public List<HlsCusHlsCreditLineH> updateCreditLineHInfo(HlsCusHlsCreditLineH hlsCusHlsCreditLineH){
        return hlsCreditLineHMapper.updateCreditLineHInfo(hlsCusHlsCreditLineH);
    }

    @Override
    public Long queryCount(Long creditLineId){
        return hlsCreditLineHMapper.queryCount(creditLineId);
    }


    @Override
    public List<HlsCreditLineH> queryVersion(Long creditLineId){
        return hlsCreditLineHMapper.queryVersion(creditLineId);
    }
    @Override
    public List<HlsCusHlsCreditLineH> queryVersionByPk(HlsCusHlsCreditLineH hlsCreditLineH){
        return hlsCreditLineHMapper.queryVersionByPk(hlsCreditLineH);
    }

    @Override
    public List<HlsCusHlsCreditLineH> queryHistoryByStatus(HlsCusHlsCreditLineH hlsCreditLineH){
        return hlsCreditLineHMapper.queryHistoryByStatus(hlsCreditLineH);
    }
    //提交审批
    @Override
    public List<HlsCusHlsCreditLineH> submitApproval(IRequest iRequest,List<HlsCusHlsCreditLineH> list){
        //获取需要审批的额度信息
        Long creditLineHId = list.get(0).getCreditLineHId();
        HlsCusHlsCreditLineH creditLineH=hlsCreditLineHMapper.selectByPrimaryKey(creditLineHId);
        Long creditLineId = list.get(0).getCreditLineId();
        HlsCusHlsCreditLine hlsCreditLine=hlsCreditLineMapper.selectByPrimaryKey(creditLineId);
        hlsCreditLine.setCreditLineHId(creditLineHId);
        List<HlsCusHlsCreditLineH> hlsCreditLineHList=new ArrayList<>();
        List<HlsCusHlsCreditLine> hlsCusHlsCreditLineList=new ArrayList<>();
        hlsCreditLineHList.add(creditLineH);
        hlsCusHlsCreditLineList.add(hlsCreditLine);
        databaseLockProvider.lock(creditLineH);
        //获取申请人
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
        //开始流程
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "CREDIT_LINE_CHANGE");
        activitiStartService.start(iRequest, hlsCusHlsCreditLineList, params);
        //修改额度状态
        hlsCreditLine.setCreditLineStatus("PENDING");
        hlsCreditLineMapper.updateByPrimaryKeySelective(hlsCreditLine);
       // self().updateByPrimaryKeySelective(iRequest,hlsCreditLine);
        //插入事件表
        Map<String,Object> paramsEvent = new HashMap<String,Object>();
        LoginUserInfo dto=new LoginUserInfo();
        //String userName=(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg=userName+"创建了"+creditLineH.getCreditLineName()+"的一条综合授信变更申请";
        paramsEvent.put("message", msg);
        paramsEvent.put("noticeTitle","综合授信变更申请");
        paramsEvent.put("noticeType","NOTICE");
        paramsEvent.put("url","");
        paramsEvent.put("level",1L);
        sysEventService.eventSave(iRequest,creditLineH.getCreditLineId(),creditLineH.getDocumentCategory(),creditLineH.getDocumentType(),"CREH","HLS_CREDIT_LINE_H","P2D",paramsEvent);
        return hlsCreditLineHList;
    }

    //申请状态下查询授信信息
    @Override
    public List<HlsCusHlsCreditLineH> queryCreditLineH(HlsCusHlsCreditLineH hlsCreditLineH){
        return hlsCreditLineHMapper.queryCreditLineH(hlsCreditLineH);
    }

    //授信变更历史
    @Override
    public List<HlsCusHlsCreditLineH> queryCreditLineHistory(HlsCusHlsCreditLineH hlsCreditLineH){
        return hlsCreditLineHMapper.queryCreditLineHistory(hlsCreditLineH);
    }

    //获取变更序列
    @Override
    public Long getChangeSeq(Long creditLineId){
        return hlsCreditLineHMapper.getChangeSeq(creditLineId);
    }

    //获取变更信息
    @Override
    public List<HlsCusHlsCreditLineH> queryChangeData(IRequest iRequest,HlsCusHlsCreditLineH hlsCreditLineH){
        List<HlsCusHlsCreditLineH> list=new ArrayList<>();
        if(hlsCreditLineHMapper.queryChangeAfter(hlsCreditLineH).size()==1){
            list.add(hlsCreditLineHMapper.queryChangeAfter(hlsCreditLineH).get(0));
            list.add(hlsCreditLineHMapper.queryVersionByCrePk(hlsCreditLineH).get(0));
        }else{
            list.add(hlsCreditLineHMapper.queryChangeAfter(hlsCreditLineH).get(0));
            list.add(hlsCreditLineHMapper.queryChangeAdvance(hlsCreditLineH).get(0));
        }
//        if(hlsCreditLineHMapper.queryChangeAdvance(hlsCreditLineH).size()==0){
//            list.add(hlsCreditLineHMapper.queryVersionByPk(hlsCreditLineH).get(0));
//        }else{
//            list.add(hlsCreditLineHMapper.queryChangeAfter(hlsCreditLineH).get(0));
//        }
        return list;
    }

}