package hls.core.fnd.controllers;

import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fnd.dto.DocFileTempletRule;
import hls.core.fnd.service.IDocFileTempletRuleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.List;

@Controller
public class DocFileTempletRuleController extends BaseController {

    @Autowired
    private IDocFileTempletRuleService service;


    @RequestMapping(value = "/hls/doc/file/templet/rule/query")
    @ResponseBody
    public ResponseData query(DocFileTempletRule dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        dto.setCompanyId(companyId);
        return new ResponseData(service.selectByFontCondition(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/hls/doc/file/templet/rule/default/query")
    @ResponseBody
    public ResponseData defaultQuery(DocFileTempletRule dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        dto.setCompanyId(companyId);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/hls/doc/file/templet/rule/submit")
    @ResponseBody
    public ResponseData update(HttpServletRequest request, HttpSession session, @RequestBody List<DocFileTempletRule> dto, BindingResult result) {
        IRequest requestCtx = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        dto.forEach(item -> {
            item.setCompanyId(companyId);
        });
        ResponseData rd;
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            rd = new ResponseData(false);
            rd.setMessage(getErrorMessage(result, request));
            return rd;
        }

        try{
            List<DocFileTempletRule> docFileTempletRules = service.batchUpdate(requestCtx, dto);
            rd = new ResponseData(docFileTempletRules);
        }catch (Exception e){
            rd = new ResponseData(false);
            rd.setMessage("插入数据重复,请核对数据");
        }
        return rd;
    }

    @RequestMapping(value = "/hls/doc/file/templet/rule/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<DocFileTempletRule> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }
}