package hls.core.fnd.controllers;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.excel.dto.ColumnInfo;
import com.hand.hap.excel.dto.ExportConfig;
import com.hand.hap.excel.service.IExportService;
import com.hand.hap.function.dto.Function;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fnd.dto.*;
import hls.core.fnd.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
public class FirmDefineController extends BaseController {

	@Autowired
	IFirmDefineService firmDefineService;
	@Autowired
	HLSIOrgUnitService service;// 部门
	@Autowired
	HLSIPositionService positionService;// 岗位
	@Autowired
	private HLSIEmployeeService employeeService;// 员工
	@Autowired
	private IHlsEmployeeAssignsService assignService;// 员工分配
	@Autowired
	IExportService excelService;

	@Autowired
	ObjectMapper objectMapper;

	// 岗位复制时保存源岗位ID和复制出来的新岗位ID
	private Map<Long, Long> phoneMap = new HashMap<>();
	// 部门复制时保存源岗位ID和复制出来的新部门ID
	private Map<Long, Long> unitMap = new HashMap<>();
	// 员工复制时保存源岗位ID和复制出来的新员工ID
	private Map<Long, Long> employeeMap = new HashMap<>();

	// 部门控制器
	private HLSOrgUnitController orgUnitController = new HLSOrgUnitController();

	private Map<String, String> map = new HashMap<>();
	
	public IFirmDefineService getFirmDefineService(){
		return firmDefineService;
	}

	@RequestMapping(value = "/fnd/fndCompany/export")
	public void createXLS(HttpServletRequest request, @RequestParam String config,
						  HttpServletResponse httpServletResponse) {
		IRequest requestContext = createRequestContext(request);
		try {
			JavaType type = objectMapper.getTypeFactory().constructParametrizedType(ExportConfig.class,
					ExportConfig.class, FndCompany.class, ColumnInfo.class);
			ExportConfig<Function, ColumnInfo> exportConfig = objectMapper.readValue(config, type);
			excelService.exportAndDownloadExcel("hls.core.fnd.mapper.FirmDefineMapper.select",
					exportConfig, request, httpServletResponse, requestContext);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	/**
	 * 查询所有公司信息
	 * @param request
	 * @param fndCompany
	 * @param page
	 * @param pageSize
	 * @return
	 */
	@RequestMapping(value = "/firm/query")
	@ResponseBody
	public ResponseData selectList(HttpServletRequest request,  FndCompany fndCompany,
			@RequestParam(defaultValue = DEFAULT_PAGE, name="page") int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE, name="pageSize") int pageSize) {
		PageHelper.startPage(page, pageSize);
		if("".equals(fndCompany.getCompany_code())){
			fndCompany.setCompany_code(null);
		}
		if("".equals(fndCompany.getCompany_full_name())){
				fndCompany.setCompany_full_name(null);
		}
		List<FndCompany> datas = firmDefineService.selectList(fndCompany);
		datas = transitionMap(datas, 1);
		return new ResponseData(datas);
	}
	
	/**
	 * 组织架构首页，公司信息查询，并且不进行分页
	 * @param request
	 * @param fndCompany
	 * @param page
	 * @param pagesize
	 * @return
	 */
	@RequestMapping(value = "/firm/homePage/query")
	@ResponseBody
	public ResponseData selectLists(HttpServletRequest request, final FndCompany fndCompany,
			@RequestParam(defaultValue = DEFAULT_PAGE) final int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
		List<FndCompany> datas = firmDefineService.selectList(fndCompany);
		datas = transitionMap(datas, 1);
		return new ResponseData(datas);
	}

	/**
	 * 公司新增，下拉列表数据查询
	 * 
	 * @param request
	 * @param fndCompany
	 * @param page
	 * @param pagesize
	 * @return
	 */
	@RequestMapping(value = "/firm/parentCompany/query", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData selectAllList(HttpServletRequest request, final FndCompany fndCompany,
			@RequestParam(defaultValue = DEFAULT_PAGE) final int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
		PageHelper.startPage(page, pagesize);
		List<FndCompany> datas = firmDefineService.select(createRequestContext(request), fndCompany, page, pagesize);
		datas = transitionMap(datas, 1);
		return new ResponseData(datas);
	}

	/*-------------查询所有公司信息，无分页-------------*/
	@RequestMapping(value = "/firm/parentCompany/queryAllCompany", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData queryAllCompany(HttpServletRequest request, final FndCompany fndCompany) {
		List<FndCompany> datas = firmDefineService.selectAll(createRequestContext(request));
		datas = transitionMap(datas, 1);
		return new ResponseData(datas);
	}

	/**
	 * 上级公司lov数据查询
	 * 
	 * @param request
	 * @param fndCompany
	 * @param page
	 * @param pagesize
	 * @return
	 */
	@RequestMapping(value = "/firm/lov/query", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData selectLovList(HttpServletRequest request, final FndCompany fndCompany,
			@RequestParam(defaultValue = DEFAULT_PAGE) final int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize) {
		PageHelper.startPage(page, pagesize);
		List<FndCompany> datas = firmDefineService.selectLovList(fndCompany);
		datas = transitionMap(datas, 1);
		return new ResponseData(datas);
	}

	public List<FndCompany> transitionMap(List<FndCompany> list, int i) {
		for (int j = 0; j < list.size(); j++) {
			FndCompany fnd = list.get(j);
			map.put(fnd.getCompany_code(), fnd.getCompany_full_name());
		}
		list = transition(list, 1);
		return list;
	}

	public List<FndCompany> transition(List<FndCompany> list, int i) {
		for (int j = 0; j < list.size(); j++) {
			FndCompany fnd = list.get(j);
			String id = fnd.getParent_company_id();
			if (null == id) {
				continue;
			}
			if (id.trim().length() > 0) {
				if (i == 1) {
					list.get(j).setParent_company_id(map.get(id));
				} else {
					for (Map.Entry<String, String> m : map.entrySet()) {
						if (m.getValue().equals(id))
							list.get(j).setParent_company_id(m.getKey());
					}
				}
			}
		}
		return list;
	}

	/**
	 * 公司提交保存
	 * @param request
	 * @param companyList
	 * @return
	 */
	@RequestMapping(value = "/firm/submit", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData submitAdd(HttpServletRequest request, @RequestBody List<FndCompany> companyList) {
		ResponseData rd = null;
		try{
			IRequest iRequest = createRequestContext(request);
			companyList = transition(companyList, 2);
			List<FndCompany> datas = firmDefineService.batchUpdate(iRequest, companyList);
			datas = transition(datas, 1);
			rd = new ResponseData(datas);

		}catch (Exception e){
			rd = new ResponseData(false);
			rd.setMessage("公司代码不可重复，请重新入");
		}
		return rd;
	}

	/**
	 * 公司编辑时数据提交保存，修改
	 * @param request
	 * @param companyList
	 * @return
	 */
	@RequestMapping(value = "/firm/Add/submit", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData submit(HttpServletRequest request, @RequestBody FndCompany companyList) {
		IRequest iRequest = createRequestContext(request);
		firmDefineService.updateChannel(iRequest,companyList);
		return new ResponseData();
	}

	/**
	 * 公司复制时提交保存
	 * @param request
	 * @param company
	 * @return
	 */
	@RequestMapping(value = "/firm/copy/Add/submit", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData submitCopy(HttpServletRequest request, @RequestBody FndCompany company) {
		IRequest iRequest = createRequestContext(request);
		Long company_id = company.getCompany_id();
		FndCompany fnd = new FndCompany();
		fnd.setCompany_id(company_id);
		FndCompany fndCompany = firmDefineService.selectList(fnd).get(0);
		fndCompany.setCompany_id(null);
		fndCompany.setCompany_code(company.getNewCompany_code());
		fndCompany.setCompany_full_name(company.getNewCompany_name());
		fndCompany.setParent_company_id(company.getParent_company_id());
		// 保存 copy公司信息
		Long newComapny_id = firmDefineService.insert(iRequest, fndCompany).getCompany_id();
		// 复制部门信息
		if (company.getUnit().equals("Y") || company.getQuarters().equals("Y")) {
			copyOrgUnit(company_id, newComapny_id, company);
		}
		if (company.getEmployee().equals("Y")) {
			// 员工复制
			copyEmployee(company_id, newComapny_id);
		}
		return new ResponseData();
	}

	/**
	 * 复制员工岗位分配表
	 * 
	 * @param company_id
	 *            源公司ID
	 * @param newCompany_id
	 *            新公司ID
	 */
	public void copyEmployeeAssign(Long company_id, Long newCompany_id) {
		HlsEmployeeAssigns employeeAssigns = new HlsEmployeeAssigns();
		employeeAssigns.setCompany_id(company_id);
		List<HlsEmployeeAssigns> list = assignService.selectAllEmployeeAssigns(employeeAssigns);
		for (int i = 0; i < list.size(); i++) {
			HlsEmployeeAssigns assigns = list.get(i);
			assigns.setCompany_id(newCompany_id);
			assigns.setEmployee_assign_id(null);
			assigns.setEmployee_id(employeeMap.get(assigns.getEmployee_id()));
			if(assigns.getUnit_id()==null){
				System.out.println("21231");
			}
			assigns.setUnit_id(unitMap.get(assigns.getUnit_id()));
			assigns.setPosition_id(phoneMap.get(assigns.getPosition_id()));
			assignService.insert(null, assigns);
		}

	}

	/**
	 * 复制员工
	 * @param company_id
	 * @param newCompany_id
	 */
	private void copyEmployee(Long company_id, Long newCompany_id) {
		HlsEmployee hlsEmployee = new HlsEmployee();
		hlsEmployee.setCompanyId(company_id);
		List<HlsEmployee> list = employeeService.selectAllEmployee(hlsEmployee);
		for (int i = 0; i < list.size(); i++) {
			HlsEmployee employee = list.get(i);
			Long employeeId = employee.getEmployeeId();
			employee.setCompanyId(newCompany_id);
			employee.setEmployeeId(null);
			HlsEmployee newHls = employeeService.insert(null, employee);
			employeeMap.put(employeeId, newHls.getEmployeeId());
		}
		// 复制员工分配信息表
		copyEmployeeAssign(company_id, newCompany_id);
	}

	/**
	 * 复制部门信息
	 * 
	 * @param company_id
	 * @param newCompany_id
	 */
	public void copyOrgUnit(Long company_id, Long newCompany_id, FndCompany company) {
		HLSOrgUnit hlsOrgUnit = new HLSOrgUnit();
		hlsOrgUnit.setCompanyId(company_id);
		List<HLSOrgUnit> orgUnits = service.queryAll(hlsOrgUnit);
		phoneMap = new HashMap<>();
		for (int i = 0; i < orgUnits.size(); i++) {
			HLSOrgUnit orgUnit = orgUnits.get(i);
			Long unitId = orgUnit.getUnitId();
			orgUnit.setCompanyId(newCompany_id);
			orgUnit.setUnitId(null);
			// 保存一行部门信息
			Long newUnitId = service.insert(null, orgUnit).getUnitId();
			unitMap.put(unitId, newUnitId);
			// 岗位复制
			if (company.getQuarters().equals("Y")) {

				copyPhone(unitId, newUnitId);

			}
		}
		// if (company.getQuarters().equals("Y")) {
		// // 复制员工分配信息表
		// copyEmployeeAssign(company_id, newCompany_id);
		// }
	}

	/**
	 * 岗位复制
	 * 
	 * @param unitId
	 *            源部门ID
	 * @param newUnitId
	 *            新部门ID
	 */
	public void copyPhone(Long unitId, Long newUnitId) {

		HLSPosition hlsPosition = new HLSPosition();
		hlsPosition.setUnitId(unitId);
		List<HLSPosition> list = positionService.queryAll(hlsPosition);
		for (int i = 0; i < list.size(); i++) {
			HLSPosition position = list.get(i);
			Long positionId = position.getPositionId();
			position.setPositionId(null);
			position.setUnitId(newUnitId);
			HLSPosition newHls = positionService.insert(null, position);
			phoneMap.put(positionId, newHls.getPositionId());
		}
	}

	/**
	 * 公司删除
	 * @param request
	 * @param companyList
	 * @return
	 */
	@RequestMapping(value = "/firm/remove", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData delete(HttpServletRequest request, @RequestBody List<FndCompany> companyList) {
		// 联动删除部门信息
		deleteOrgUnit(companyList);
		// 联动删除商业模式信息
		deleteChannel(companyList);
		deleteEmployee(companyList);
		firmDefineService.batchDelete(companyList);
		return new ResponseData();
	}

	/**
	 * 删除公司ID对应的员工信息
	 * 
	 */
	public void deleteEmployee(List<FndCompany> companyList) {
		for (int i = 0; i < companyList.size(); i++) {
			HlsEmployee employee = new HlsEmployee();
			employee.setCompanyId(companyList.get(i).getCompany_id());
			List<HlsEmployee> list = employeeService.selectAllEmployee(employee);
			employeeService.batchDelete(list);
		}

	}

	/**
	 * 删除商业模式信息
	 * 
	 * @param companyList
	 * @return
	 */
	public boolean deleteChannel(List<FndCompany> companyList) {
		List<String> compantCode = new ArrayList<>();
		for (int i = 0; i < companyList.size(); i++) {
			compantCode.add(companyList.get(i).getCompany_id()+"");
		}
		for (int i = 0; i < compantCode.size(); i++) {
			firmDefineService.deleteChannel(compantCode.get(i));
		}
		return true;
	}

	/**
	 * 联动删除部门表信息
	 * 
	 * @param companyList
	 */
	public void deleteOrgUnit(List<FndCompany> companyList) {
		if (companyList.size() == 0) {
			return;
		}
		for (int i = 0; i < companyList.size(); i++) {
			HLSOrgUnit hlsOrgUnit = new HLSOrgUnit();
			hlsOrgUnit.setCompanyId(companyList.get(i).getCompany_id());
			List<HLSOrgUnit> orgUnits = service.queryAll(hlsOrgUnit);
			if (orgUnits.size() == 0) {
				continue;
			}
			for (int j = 0; j < orgUnits.size(); j++) {
				HLSOrgUnit orgunit = orgUnits.get(j);
				orgUnitController.linkageDelete(orgunit.getUnitId(), service, positionService, assignService);
			}
			// 删除员工分配表信息
			linkageDelete(companyList.get(i).getCompany_id(), assignService);
			service.batchDelete(orgUnits);
		}
	}

	/**
	 * 联动删除公司ID对应的员工分配表中的信息
	 * 
	 */
	public void linkageDelete(Long company_id, IHlsEmployeeAssignsService assignService) {
		if (company_id > 0 || company_id == null) {
			HlsEmployeeAssigns assigns = new HlsEmployeeAssigns();
			assigns.setCompany_id(company_id);
			List<HlsEmployeeAssigns> hlsList = assignService.selectAllEmployeeAssigns(assigns);
			if (hlsList.size() == 0) {
				return;
			}
			assignService.batchDelete(hlsList);
		}
	}
}
