package hls.core.fnd.controllers;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.fnd.dto.HLSOrgUnit;
import hls.core.fnd.dto.HLSPosition;
import hls.core.fnd.dto.HlsEmployeeAssigns;
import hls.core.fnd.service.HLSIEmployeeService;
import hls.core.fnd.service.HLSIOrgUnitService;
import hls.core.fnd.service.HLSIPositionService;
import hls.core.fnd.service.IHlsEmployeeAssignsService;

 
@Controller
@RequestMapping(value = "/fnd")
public class HLSOrgUnitController extends BaseController{

    @Autowired
    HLSIOrgUnitService service;
    @Autowired
    HLSIEmployeeService employeeService ;
    @Autowired
	private HLSIPositionService positionService;
    @Autowired
	private IHlsEmployeeAssignsService assignService;
    
    @RequestMapping(value = "/org/unit/query")
    @ResponseBody
    public ResponseData selectList(HttpServletRequest request, HLSOrgUnit unit, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
            @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize){
    	
    	PageHelper.startPage(page, pagesize);
    	IRequest iRequest = createRequestContext(request);
    	List<HLSOrgUnit> list = service.select(iRequest, unit, page, pagesize);
    	return new ResponseData(list);
    }

    /**
     * 查询所有部门信息
     * @param request
     * @param unit
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/unit/query")
    @ResponseBody
    public ResponseData query(HttpServletRequest request, HLSOrgUnit unit, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize){
                    IRequest requestCtx = createRequestContext(request);
                    PageHelper.startPage(page, pagesize);
                    return new ResponseData(service.queryAll(requestCtx,unit,page,pagesize));
    }

    /**
     * 查询所有部门信息
     * @param request
     * @param unit
     * @param page
     * @param pagesize
     * @return
     */
    @RequestMapping(value = "/unit/query/all" ,method = RequestMethod.POST)
    @ResponseBody
    public ResponseData queryAllUnit(HttpServletRequest request, final HLSOrgUnit unit,
                                     @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize){
        IRequest requestCtx = createRequestContext(request);
        List<HLSOrgUnit> list = service.queryAll(requestCtx,unit,page,pagesize);
        return new ResponseData(list);
    }
    /**
     * 查询所有部门名称
     * @param unit
     * @return
     */
    @RequestMapping(value = "/unitName/query")
    @ResponseBody
    public ResponseData queryName(HLSOrgUnit unit,HttpSession session){
    	List<String> list = service.queryAllName(unit);
    	return new ResponseData(list);
    }
    /**
     * 查询等人用户所在的所有部门名称
     * @param unit
     * @return
     */
    @RequestMapping(value = "/unitName/All/query")
    @ResponseBody
    public ResponseData queryNames(HLSOrgUnit unit,HttpSession session){
    	Long companyId = (Long) session.getAttribute("companyId");
    	unit.setCompanyId(companyId);
    	List<String> list = service.queryAllName(unit);
    	return new ResponseData(list);
    }
    /**
     * 部门lov数据查询
     * @param lovCode
     * @return
     */
    @RequestMapping(value = "/lov/query")
    @ResponseBody
    public ResponseData queryLov(String lovCode){
    	    	
    	return new ResponseData(service.queryLov(lovCode));
    	
    }

    /**
     * 部门信息提交保存
     * @param request
     * @param units
     * @return
     */
    @RequestMapping(value = "/unit/submit")
    @ResponseBody
    public ResponseData update(HttpServletRequest request,@RequestBody List<HLSOrgUnit> units){
        IRequest requestCtx = createRequestContext(request);
        for(int i=0 ;i<units.size();i++){
    		Long parentUnitId = service.selectById(units.get(i));
    		units.get(i).setParentUnitId(parentUnitId);
        }
        return new ResponseData(service.batchUpdate(requestCtx, units));
    }
    /**
     * 部门删除
     * @param request
     * @param units
     * @return
     */
    @RequestMapping(value = "/unit/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request,@RequestBody List<HLSOrgUnit> units){
    	for(int i=0 ;i<units.size();i++){
    		HlsEmployeeAssigns hlsEmployeeAssigns = new HlsEmployeeAssigns();
        	Long Unit_id = units.get(i).getUnitId();
        	hlsEmployeeAssigns.setUnit_id(Unit_id);
        	List<HlsEmployeeAssigns> listAssigns = assignService.selectAllEmployeeAssigns(hlsEmployeeAssigns);
        	assignService.batchDelete(listAssigns);
        	linkageDelete(units.get(i).getUnitId(),service,positionService,assignService);
        	service.deletePosition(units.get(i).getUnitId());
		}
        service.batchDelete(units);
        return new ResponseData();
    }
    /**
     * 联动删除岗位表对应信息
     * @param unitId
     */
    public void linkageDelete(Long unitId,HLSIOrgUnitService service,HLSIPositionService positionService,IHlsEmployeeAssignsService assignService){
    	if(unitId==null){
    		return;
    	}
    	HLSOrgUnit unit = new HLSOrgUnit();
    	unit.setUnitId(unitId);
    	List<HLSOrgUnit> list = service.queryAll(unit);
    	if(list.size()==0){
    		return ;
    	}
    	for (int i = 0; i < list.size(); i++) {
    		HLSPosition hlsPosition = new HLSPosition();
    		hlsPosition.setUnitId(list.get(i).getUnitId());
    		List<HLSPosition> lists = positionService.queryAll(hlsPosition);
    		if(lists.size()==0){
    			continue;
    		}
    		positionService.batchDelete(lists);
		}
    }
  
}
