package hls.core.fnd.controllers;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.exception.TokenException;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.fnd.dto.HLSOrgUnit;
import hls.core.fnd.dto.HLSPosition;
import hls.core.fnd.dto.HlsEmployeeAssigns;
import hls.core.fnd.service.HLSIOrgUnitService;
import hls.core.fnd.service.HLSIPositionService;
import hls.core.fnd.service.IHlsEmployeeAssignsService;

/**
 * 岗位控制器
 *  
 *
 */
@Controller
public class HLSPositionController extends BaseController {
	@Autowired
	private HLSIPositionService positionService;
	@Autowired
	private IHlsEmployeeAssignsService assignService;



	public static List<HLSPosition> hlsPositionList;
	
	private HLSPosition position;

	/**
	 * 员工岗位分配信息lov查询
	 * 
	 * @param request
	 * @param position
	 * @param page
	 * @param pagesize
	 * @return
	 */
	@RequestMapping(value = "/fnd/position/assign/lov/query")
	@ResponseBody
	public ResponseData SelectAll(HttpServletRequest request, HLSPosition position,
			@RequestParam(defaultValue = DEFAULT_PAGE) int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
		PageHelper.startPage(page, pagesize);
		List<HLSPosition> list = positionService.selectAll(position);
		return new ResponseData(list);
	}

	/**
	 * 岗位信息查询.
	 * 
	 * @param position
	 *            position
	 * @param page
	 *            起始页
	 * @param pagesize
	 *            分页大小
	 * @return ResponseData
	 */
	@RequestMapping(value = "/fnd/position/assign/query")
	@ResponseBody
	public ResponseData getPosition(HttpServletRequest request, HLSPosition position,
			@RequestParam(defaultValue = DEFAULT_PAGE) int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
		PageHelper.startPage(page, pagesize);
		List<HLSPosition> list = positionService.queryAll(position);
		List<HLSPosition> selectList = positionService.selectAll(createRequestContext(request));
		for (int i = 0; i < list.size(); i++) {
			if (list.get(i).getParentPositionId() == null) {
				continue;
			}
			for (int j = 0; j < selectList.size(); j++) {
				Long ParentPositionId = list.get(i).getParentPositionId();
				Long PositionId = selectList.get(j).getPositionId();
				if (ParentPositionId.equals(PositionId)) {
					list.get(i).setParentPositionName(selectList.get(j).getPositionName());
				}
			}
		}
		hlsPositionList = list;
		return new ResponseData(list);
	}
	
	/**
	 * 加载岗位信息查询.
	 * 
	 * @param position
	 *            position
	 * @param page
	 *            起始页
	 * @param pagesize
	 *            分页大小
	 * @return ResponseData
	 */
	@RequestMapping(value = "/fnd/position/assign/load/query")
	@ResponseBody
	public ResponseData getPositions(HttpServletRequest request, HLSPosition position,
			@RequestParam(defaultValue = DEFAULT_PAGE) int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {
		//List<HLSPosition> list = positionService.select(createRequestContext(request), position, page, pagesize);
		List<HLSPosition> list = positionService.selectAllHLSPosition(position);
		List<HLSPosition> selectList = positionService.selectAll(createRequestContext(request));
		for (int i = 0; i < list.size(); i++) {
			if (list.get(i).getParentPositionId() == null) {
				continue;
			}
			for (int j = 0; j < selectList.size(); j++) {
				Long ParentPositionId = list.get(i).getParentPositionId();
				Long PositionId = selectList.get(j).getPositionId();
				if (ParentPositionId.equals(PositionId)) {
					list.get(i).setParentPositionName(selectList.get(j).getPositionName());
				}
			}
		}
		hlsPositionList = list;
		return new ResponseData(list);
	}

	/**
	 * 员工分配岗位时LOV数据查询
	 * 
	 * @param request
	 * @param position
	 * @param page
	 * @param pagesize
	 * @return
	 */
	@RequestMapping(value = "/fnd/position/assign/Lov/query")
	@ResponseBody
	public ResponseData getLovPosition(HttpServletRequest request,HLSPosition hlsPosition ,
			@RequestParam(defaultValue = DEFAULT_PAGE) int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {// int company_id
//		List<HLSPosition> list = positionService.queryLovAll(Long.parseLong(company_id + ""));
		List<HLSPosition> list = positionService.queryLovAll(hlsPosition);
		hlsPositionList = list;
		return new ResponseData(list);
	}

	/**
	 * 岗位信息保存.
	 * 
	 * @param positions
	 *            positions
	 * @param result
	 *            BindingResult
	 * @param request
	 *            HttpServletRequest
	 * @return ResponseData
	 * @throws TokenException
	 */
	@RequestMapping(value = "/fnd/position/assign/submit", method = RequestMethod.POST)
	public ResponseData submitPosition(@RequestBody List<HLSPosition> positions, BindingResult result,
			HttpServletRequest request) throws TokenException {
		checkToken(request, positions);
		getValidator().validate(positions, result);
		if (result.hasErrors()) {
			ResponseData rd = new ResponseData(false);
			rd.setMessage(getErrorMessage(result, request));
			return rd;
		}
		IRequest requestCtx = createRequestContext(request);

		return new ResponseData(positionService.updatePostion(requestCtx, positions));
	}

	/**
	 * 岗位信息删除.
	 * 
	 * @param positions
	 *            positions
	 * @param result
	 *            BindingResult
	 * @param request
	 *            HttpServletRequest
	 * @return ResponseData
	 * @throws TokenException
	 */
	@RequestMapping(value = "/fnd/position/assign/remove")
	@ResponseBody
	public ResponseData delete(HttpServletRequest request, @RequestBody List<HLSPosition> positions) {
		for (int i = 0; i < positions.size(); i++) {
			linkageDelete(positions.get(i).getPositionId());
		}
		positionService.batchDelete(positions);
		return new ResponseData();
	}

	/**
	 * 联动删除岗位ID对应的员工分配表中的信息
	 * 
	 * @param positionId
	 */
	public void linkageDelete(Long positionId) {
		if (positionId < 0 || positionId != null) {
			HlsEmployeeAssigns assigns = new HlsEmployeeAssigns();
			assigns.setPosition_id(positionId);
			List<HlsEmployeeAssigns> hlsList = assignService.selectAllEmployeeAssigns(assigns);
			if (hlsList.size() == 0) {
				return;
			}
			assignService.batchDelete(hlsList);
		}
	}

	@RequestMapping(value = "/fnd/position/assign/queryName")
	@ResponseBody
	public ResponseData queryPositionName(HLSPosition position) {
		this.position = position;
		List<String> list = positionService.queryPositionName(position);
		/*List<String> listStr = new ArrayList<>();
		for (int i = 0; i < list.size(); i++) {
			listStr.add(list.get(i).getPositionName());
		}*/
		return new ResponseData(list);
	}

	/**
	 * 查询所有岗位信息
	 * @param request
	 * @param position
	 * @return
	 */
	@RequestMapping(value = "/position/query" , method = RequestMethod.POST)
	@ResponseBody
	public ResponseData queryAllByCompany(HttpServletRequest request,final HLSPosition position){
		IRequest requestCtx = createRequestContext(request);
		List<HLSPosition> list = positionService.queryAll(position);
		return new ResponseData(list);
	}
}
