package hls.core.fnd.controllers;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fnd.dto.HLSPosition;
import hls.core.fnd.dto.HlsEmployeeAssigns;
import hls.core.fnd.service.IHlsEmployeeAssignsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Objects;

 
@Controller
public class HlsEmployeeAssignController extends BaseController {

	@Autowired
	private IHlsEmployeeAssignsService assignService;
	
	/**
	 * 员工分配表查询
	 * @param assigns 
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/hls/mployeeAssign/query")
	@ResponseBody
	public ResponseData query(final HlsEmployeeAssigns assigns,
			@RequestParam(defaultValue = DEFAULT_PAGE) final int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
		PageHelper.startPage(page, pagesize); 
		List<HlsEmployeeAssigns> list = assignService.selectAllEmployeeAssigns(assigns);
		return new ResponseData(list);
	}
	/**
	 * 查找出新建是所需要的默认值
	 * @param assigns
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/hls/mployeeAssign/default/query")
	@ResponseBody
	public ResponseData findDefault(final HlsEmployeeAssigns assigns,
			@RequestParam(defaultValue = DEFAULT_PAGE) final int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		
		PageHelper.startPage(page, pagesize);	
		List<HlsEmployeeAssigns> list =assignService.selectFindDefault(assigns);
		
		return new ResponseData(list);
	}
	/**
	 * 员工分配表保存
	 * @param assigns
	 * @param result
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/hls/mployeeAssign/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody final List<HlsEmployeeAssigns> assigns, final BindingResult result,
			final HttpServletRequest request) {
		
		List<HLSPosition> hlsPostList = HLSPositionController.hlsPositionList;
		for (int i = 0; i < assigns.size(); i++) {
			for (int j = 0; j < hlsPostList.size(); j++) {
					HLSPosition hlsPosition = hlsPostList.get(j);
					if( assigns.get(i).getPosition_code().equals(hlsPosition.getPositionCode()) && Objects.equals(assigns.get(i).getUnit_code(), hlsPosition.getUnit_code())){
						if(hlsPosition.getCompany_id()!=null){
							assigns.get(i).setCompany_id(hlsPosition.getCompany_id());
						}
						assigns.get(i).setUnit_id(hlsPosition.getUnitId());
						Long ProgramId = hlsPosition.getPositionId();
						assigns.get(i).setPosition_id(ProgramId);
					}
			}
		}
		List<HlsEmployeeAssigns> list = saveEmployeeAssign(assigns, request);
		return new ResponseData(list);
	}
	/**
	 * 员工岗位分配信息保存
	 * @param assigns
	 * @param request
	 * @return
	 */
	public List<HlsEmployeeAssigns> saveEmployeeAssign(final List<HlsEmployeeAssigns> assigns,
			final HttpServletRequest request) {
		IRequest requestContext = createRequestContext(request);
		List<HlsEmployeeAssigns> list = assignService.batchUpdate(requestContext, assigns);
		return list;
	}
	/**
	 * 员工分配表删除
	 * @param assigns
	 * @return
	 */
	@RequestMapping(value = "/hls/mployeeAssign/remove")
	@ResponseBody
	public ResponseData delete(@RequestBody final List<HlsEmployeeAssigns> assigns) {
		assignService.batchDelete(assigns);
		return new ResponseData();
	}

}
