package hls.core.fnd.controllers;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import hls.core.fnd.dto.HlsEmployee;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.fnd.dto.HlsEmployeeAssigns;
import hls.core.fnd.dto.HlsEmployeeRegister;
import hls.core.fnd.service.HLSIEmployeeService;
import hls.core.fnd.service.IHlsEmployeeAssignsService;

 
@Controller
public class HlsEmployeeController extends BaseController {

	@Autowired
	private HLSIEmployeeService employeeService;
	@Autowired
	private IHlsEmployeeAssignsService iHlsEmployeeAssignsService;

	/**
	 * 查询所有员工信息
	 * 
	 * @param employee
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/fnd/employee/query")
	@ResponseBody
	public ResponseData query(final HlsEmployee employee, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
							  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
		PageHelper.startPage(page, pagesize);

//		if(employee.getName()!=null &&employee.getName().indexOf(",")>0){
//			employee.setName(employee.getName().split(",")[1]);
//		}
		List<HlsEmployee> list = employeeService.selectAllEmployee(employee);
		return new ResponseData(list);
	}

	/**
	 * 提交保存员工信息
	 * 
	 * @param employees
	 * @param result
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/fnd/employee/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody final List<HlsEmployee> employees, final BindingResult result,
                               final HttpServletRequest request) {
		IRequest requestContext = createRequestContext(request);
		ResponseData rd = null;
		try {
			getValidator().validate(employees, result);
			for (int i = 0; i < employees.size(); i++) {
				if (employees.get(i).getEmployeeCode() == null || employees.get(i).getName() == null) {
					employees.remove(i);
				}
			}
			if (result.hasErrors()) {
				ResponseData responseData = new ResponseData(false);
				responseData.setMessage(getErrorMessage(result, request));
				return responseData;
			}
			List<HlsEmployee> list = employeeService.batchUpdate(requestContext, employees);
			rd = new ResponseData(list);
		}catch (Exception e){
			rd = new ResponseData(false);

			rd.setMessage("员工编码不可重复！");
		}

		return rd;
	}

	/**
	 * 公司首页，员工保存数据提交
	 * 
	 * @param result
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/fnd/employee/unit/submit")
	@ResponseBody
	public ResponseData employee_unit_submit(@RequestBody final List<HlsEmployeeRegister> hlsEmployeeRegisters,
			final BindingResult result, final HttpServletRequest request) {
		IRequest requestContext = createRequestContext(request);
		HlsEmployee hList =null;
		List<HlsEmployeeAssigns> heaList = new ArrayList<HlsEmployeeAssigns>();
		for (int i = 0; i < hlsEmployeeRegisters.size(); i++) {
			HlsEmployeeRegister hlsEmployeeRegister = hlsEmployeeRegisters.get(i);
			if (result.hasErrors()) {
				ResponseData responseData = new ResponseData(false);
				responseData.setMessage(getErrorMessage(result, request));
				return responseData;
			}
			List<HlsEmployee> employeesList = new ArrayList<HlsEmployee>();
			if(i==0){
				HlsEmployee employees = new HlsEmployee();
				employees.set__status("add");
				employees.setEmployeeCode(hlsEmployeeRegister.getEmployeeCode());
				employees.setCompanyId(hlsEmployeeRegister.getCompanyId());
				employees.setName(hlsEmployeeRegister.getName());
				employees.setEmail(hlsEmployeeRegister.getEmail());
				employees.setMobil(hlsEmployeeRegister.getMobil());
				employees.setPhone(hlsEmployeeRegister.getPhone());
				employees.setIdType(hlsEmployeeRegister.getIdType());
				employees.setIdCardNo(hlsEmployeeRegister.getIdCardNo());
				employees.setEnabledFlag(hlsEmployeeRegister.getEnabledFlag());
				employeesList.add(employees);
				hList = employeeService.insertSelective(requestContext, employees);//   batchUpdate(requestContext, employeesList);
			}
			if(hlsEmployeeRegister.getPosition_id()!=null) {
				HlsEmployeeAssigns hlsEmployeeAssigns = new HlsEmployeeAssigns();
				hlsEmployeeAssigns.set__status("add");
				hlsEmployeeAssigns.setCompany_id(hlsEmployeeRegister.getCompanyId());
				hlsEmployeeAssigns.setEmployee_id(hlsEmployeeRegister.getEmployee_id());
				hlsEmployeeAssigns.setUnit_id(hlsEmployeeRegister.getUnit_id());
				hlsEmployeeAssigns.setPosition_id(hlsEmployeeRegister.getPosition_id());
				hlsEmployeeAssigns.setEnabled_flag(hlsEmployeeRegister.getEnabled_flag());
				hlsEmployeeAssigns.setEmployee_id(hList.getEmployeeId());
				heaList.add(hlsEmployeeAssigns);
			}
		}
		iHlsEmployeeAssignsService.batchUpdate(requestContext, heaList);
		return new ResponseData();
	}
	
	/**
	 * 员工删除
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/fnd/employee/remove", method = RequestMethod.POST)
	@ResponseBody
	public ResponseData delete(HttpServletRequest request, @RequestBody  List<HlsEmployee> employees) {
		employeeService.batchDelete(employees);
		return new ResponseData();
	}
}
