package hls.core.fnd.service.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.fnd.dto.FndBaseRate;
import hls.core.fnd.dto.FndBaseRateSet;
import hls.core.fnd.dto.FndDistrict;
import hls.core.fnd.mapper.FndBaseRateMapper;
import hls.core.fnd.mapper.FndBaseRateSetMapper;
import hls.core.fnd.mapper.FndDistrictMapper;
import hls.core.fnd.service.FndBaseRateService;
import hls.core.fnd.service.FndBaseRateSetService;

@Service
@Transactional
public class FndBaseRateSetServiceImpl extends BaseServiceImpl<FndBaseRateSet> implements FndBaseRateSetService{

	@Autowired
	FndBaseRateSetMapper mapper;

	@Autowired
	FndBaseRateMapper fndBaseRateMapper;

	@Autowired
	FndBaseRateService fndBaseRateService;

	public List<FndBaseRateSet> getCount(){
		return mapper.getCount();
	}

	@Override
	public List<FndBaseRateSet> queryAll() {
		return mapper.selectAll();
	}

	public List<FndBaseRateSet> selectByQuery(FndBaseRateSet fndBaseRateSet,int page,int pagesize){
		 PageHelper.startPage(page, pagesize);
		return mapper.selectByQuery(fndBaseRateSet);
	}

	@Override
	public void batchChildUpdate(IRequest iRequest, FndBaseRateSet fndBaseRateSet, List<FndBaseRate> fndBaseRate, List<FndBaseRateSet> datas) {
		FndBaseRateSet oldFndBaseRateSet = new FndBaseRateSet();
		if(fndBaseRateSet != null)
		{
			//更新和插入头表
			boolean check = false;

			for(FndBaseRateSet data:datas)
			{
				if(data.getBase_rate_set().equals(fndBaseRateSet.getBase_rate_set()))
				{
					check = true;
				}
				else if(data.getBase_rate_type().equals(fndBaseRateSet.getBase_rate_type()))
				{
					if(data.getValid_to() == null){
							Date beginDate = fndBaseRateSet.getValid_from();

							SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");

							Calendar date1 = Calendar.getInstance();
							date1.setTime(beginDate);
							date1.set(Calendar.DATE, date1.get(Calendar.DATE) - 1);
							Date endDate = null;
							try {
								endDate = dft.parse(dft.format(date1.getTime()));
							} catch (ParseException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}

							data.setValid_to(endDate);
							data.set__status("update");
						    self().updateByPrimaryKeySelective(iRequest, data);

					}
				}
				if(data.getBase_rate_set().equals(fndBaseRateSet.getBaseRateSetOld())){
					//如果是修改的话  获取原旧数据
					oldFndBaseRateSet = data;
				}
			}

			if(!check)
			{
				//按照原本的逻辑 无论新增还是修改都是插入操作  所以修改时应该删除旧数据
				fndBaseRateSet.setEnabled_flag("Y");
				self().insertSelective(iRequest, fndBaseRateSet);
				if(oldFndBaseRateSet.getBase_rate_set()!=null){
					self().deleteByPrimaryKey(oldFndBaseRateSet);
				}

			}else
			{
				self().updateByPrimaryKeySelective(iRequest, fndBaseRateSet);
			}

			//更新和插入行表
			if(fndBaseRateSet.getFndBaseRate() != null)
			{
				for(FndBaseRate fndbaserate :fndBaseRate)
				{
					if(fndbaserate.getBase_rate_id() == null)
					{
						fndbaserate.setBase_rate_set(fndBaseRateSet.getBase_rate_set());
						fndBaseRateService.insertSelective(iRequest, fndbaserate);
					}else
					{
						fndbaserate.setBase_rate_set(fndBaseRateSet.getBase_rate_set());
						fndBaseRateService.updateByPrimaryKey(iRequest, fndbaserate);
					}
				}
			}


		}


	}
	//融资合同-根据基础利率类型和融资期限查询利率值
	public List<FndBaseRateSet> queryForFinanceContract(FndBaseRateSet fndBaseRateSet){
		return mapper.queryForFinanceContract(fndBaseRateSet);
	}


}
