package hls.core.fnd.service.impl;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fnd.dto.FndCodingRule;
import hls.core.fnd.dto.FndCodingRuleDetails;
import hls.core.fnd.dto.FndCodingRuleValues;
import hls.core.fnd.mapper.FndCodingRuleDetailsMapper;
import hls.core.fnd.mapper.FndCodingRuleMapper;
import hls.core.fnd.mapper.FndCodingRuleValuesMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.dto.DocumentType;
import hls.core.hls.mapper.DocumentTypeMapper;

@Service
@Transactional
public class FndCodingRuleValuesServiceImpl extends BaseServiceImpl<FndCodingRuleValues> implements FndCodingRuleValuesService{

	@Autowired
	FndCodingRuleMapper  codingRuleMapper;
	@Autowired
	FndCodingRuleDetailsMapper  codingRuleDetailsMapper;
	@Autowired
	FndCodingRuleValuesMapper codingRuleValuesMapper;
	@Autowired
	DocumentTypeMapper documentTypeMapper;
	@Override
	public String getCodeRuleValue(IRequest requestContext,String documentCategory,String documentType,String businessType,Map<String,String> params) {
		
		DocumentType type = new DocumentType();
		type.setDocumentCategory(documentCategory);
		type.setDocumentType(documentType);
		type.setBusinessType(businessType);
		
		DocumentType document = documentTypeMapper.selectByPrimaryKey(type);
		
		Long codingRuleId = document.getCodingRuleId();
		
		String value = null ;
		
		String segmentTypeValue = null;
		//获取编码规则
		FndCodingRule codingRule = codingRuleMapper.selectByPrimaryKey(codingRuleId);
		
		if(codingRule == null)
		{
			return null;
		}
		//获取编码规则详情
		List<FndCodingRuleDetails> codingRuleDetails = codingRuleDetailsMapper.selectByCodingRuleId(codingRule.getCodingRuleId());
		
		Date date = new Date();
		
		for(FndCodingRuleDetails codingRuleDetail : codingRuleDetails)
		{
			segmentTypeValue = getSegmentTypeValue(requestContext,documentCategory,documentType,businessType,date,codingRuleDetail.getSegmentType(),codingRule,params);
			
			if(value == null)
			{
				value = segmentTypeValue;
			}else
			{
				StringBuffer sb = new StringBuffer(value);
				
				value = sb.append(segmentTypeValue).toString();
			}
		}
		
		return value;
	}
	
	@Override
	public String getCodeRuleValue(IRequest requestContext, String documentCategory, String documentType,
			String businessType, Date date, Map<String, String> params) {
		
		DocumentType type = new DocumentType();
		type.setDocumentCategory(documentCategory);
		type.setDocumentType(documentType);
		type.setBusinessType(businessType);
		
		DocumentType document = documentTypeMapper.selectByPrimaryKey(type);
		
		Long codingRuleId = document.getCodingRuleId();
		
		String value = null ;
		
		String segmentTypeValue = null;
		//获取编码规则
		FndCodingRule codingRule = codingRuleMapper.selectByPrimaryKey(codingRuleId);
		
		if(codingRule == null)
		{
			return null;
		}
		//获取编码规则详情
		List<FndCodingRuleDetails> codingRuleDetails = codingRuleDetailsMapper.selectByCodingRuleId(codingRule.getCodingRuleId());
		
		for(FndCodingRuleDetails codingRuleDetail : codingRuleDetails)
		{
			segmentTypeValue = getSegmentTypeValue(requestContext,documentCategory,documentType,businessType,date,codingRuleDetail.getSegmentType(),codingRule,params);
			
			if(value == null)
			{
				value = segmentTypeValue;
			}else
			{
				StringBuffer sb = new StringBuffer(value);
				
				value = sb.append(segmentTypeValue).toString();
			}
		}
		
		return value;
	}
	
	private String getSegmentTypeValue(IRequest requestContext, String documentCategory,String documentType,String businessType,Date date,String segmentType,FndCodingRule codingRule, Map<String, String> params)
	{
		FndCodingRuleDetails codingRuleDetails = null;
		
		if("SYS_01".equals(segmentType))
		{
			codingRuleDetails = codingRuleDetailsMapper.selectByUk(segmentType,codingRule.getCodingRuleId());
			
			return codingRuleDetails.getSegmentValue();
		}else if("SYS_02".equals(segmentType))
		{
			codingRuleDetails = codingRuleDetailsMapper.selectByUk(segmentType,codingRule.getCodingRuleId());
			
			Date currentTime = new Date();
			
			String format = codingRuleDetails.getDateFormat().replace("D", "d");
			
			SimpleDateFormat formatter = new SimpleDateFormat(format);
			
			return formatter.format(currentTime);
		}
		else if("SYS_03".equals(segmentType))
		{
			String value  = getSeqValue(requestContext,segmentType,codingRule,date);
			
			return value;
		}
		else if("SYS_04".equals(segmentType))
		{
			
			return documentCategory;
		}
		else if("SYS_05".equals(segmentType))
		{
			
			return documentType;
		}
		else if("SYS_06".equals(segmentType))
		{
			
			return businessType;
		}
		else
		{
			return params.get(segmentType);
		}
	}

	private String getSeqValue(IRequest requestContext, String segmentType, FndCodingRule codingRule,Date date) {
		FndCodingRuleValues codingRulevalue = null;
		
		Calendar cal = Calendar.getInstance();
		if(date != null)
		{
			cal.setTime(date);
		}
		int month = cal.get(Calendar.MONTH) + 1;
		int year = cal.get(Calendar.YEAR);
		
		FndCodingRuleDetails codingRuleDetails = codingRuleDetailsMapper.selectByUk(segmentType,codingRule.getCodingRuleId());
		
		if("MONTH".equals(codingRule.getResetFrequence()))
		{
			codingRulevalue = codingRuleValuesMapper.selectByUyKey(codingRule.getCodingRuleId(),Integer.toString(month),new Long(year));
		}else if("YEAR".equals(codingRule.getResetFrequence()))
		{
			codingRulevalue = codingRuleValuesMapper.selectByUyKey(codingRule.getCodingRuleId(),null,new Long(year));
		}else
		{
			codingRulevalue = codingRuleValuesMapper.selectByUyKey(codingRule.getCodingRuleId(),null,null);
		}
		
		if(codingRulevalue == null)
		{
			Long seq = codingRuleDetails.getStartValue() + codingRuleDetails.getIncremental();
			
			String value = seq.toString();
			
			while(value.length() < codingRuleDetails.getLength())
			{
				value =  "0"+ value;
			}
			
			FndCodingRuleValues codingRuleValues = new FndCodingRuleValues();
			
			if("MONTH".equals(codingRule.getResetFrequence()))
			{
				codingRuleValues.setCodingRuleId(codingRule.getCodingRuleId());
				codingRuleValues.setMonth(Integer.toString(month));
				codingRuleValues.setYear(new Long(year));
				codingRuleValues.setCurrentValue(seq);
			}else if("YEAR".equals(codingRule.getResetFrequence()))
			{
				codingRuleValues.setCodingRuleId(codingRule.getCodingRuleId());
				codingRuleValues.setYear(new Long(year));
				codingRuleValues.setCurrentValue(seq);
			}else
			{
				codingRuleValues.setCodingRuleId(codingRule.getCodingRuleId());
				codingRuleValues.setCurrentValue(seq);			
			}
			self().insertSelective(requestContext,codingRuleValues);
			
			return value;
		}else
		{
			Long seqValue = codingRulevalue.getCurrentValue() + codingRuleDetails.getIncremental();
			
			String value = seqValue.toString();
			
			while(value.length() < codingRuleDetails.getLength())
			{
				value =  "0"+ value;
			}
			
			if("MONTH".equals(codingRule.getResetFrequence()))
			{
				codingRuleValuesMapper.updateByUyKey(codingRule.getCodingRuleId(),Integer.toString(month),new Long(year),seqValue);
			}else if("YEAR".equals(codingRule.getResetFrequence()))
			{
				codingRuleValuesMapper.updateByUyKey(codingRule.getCodingRuleId(),null,new Long(year),seqValue);
			}else
			{
				codingRuleValuesMapper.updateByUyKey(codingRule.getCodingRuleId(),null,null,seqValue);
			}
			
			return value;
		}
		
	}

}
