package hls.core.fnd.service.impl;

import java.util.List;

import hls.core.fnd.dto.HLSOrgUnit;
import hls.core.fnd.mapper.HLSOrgUnitMapper;
import hls.core.fnd.service.HLSIOrgUnitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.message.IMessagePublisher;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.fnd.dto.HLSPosition;
import hls.core.fnd.mapper.HlsPositionMapper;
import hls.core.fnd.service.HLSIPositionService;

/**
 * 功能服务接口实现.
 * 
 * @author hailin.xu@hand-china.com
 */
@Service
@Transactional
public class HLSPositionServiceImpl extends BaseServiceImpl<HLSPosition> implements HLSIPositionService {
    @Autowired
    HlsPositionMapper positionMapper;

    @Autowired
    private IMessagePublisher messagePublisher;


	@Autowired
	HLSOrgUnitMapper orgUnitMapper;


	@Override
    public List<HLSPosition> selectAllHLSPosition(HLSPosition hlsPosition){
    	return positionMapper.selectAllHLSPosition(hlsPosition);
    }



	@Override
	public List<HLSPosition> updatePostion(IRequest request, List<HLSPosition> hlsPositionList) {
		// 修改 fnd_org_unit 表的 chief_position_id 字段为positionId
		for (int i = 0; i < hlsPositionList.size(); i++) {
			HLSPosition hlsPosition = hlsPositionList.get(i);
			if ("Y".equals(hlsPosition.getChiefPositionFlag())) {
				HLSOrgUnit hLSOrgUnit = new  HLSOrgUnit();
				hLSOrgUnit.setUnitId(hlsPosition.getUnitId());
				hLSOrgUnit = orgUnitMapper.selectByPrimaryKey(hLSOrgUnit);
				hLSOrgUnit.setChiefPositionId(hlsPosition.getPositionId());
				orgUnitMapper.updateByPrimaryKey(hLSOrgUnit);
			}
		}

    	// 更新岗位信息
    	List<HLSPosition> results =  self().batchUpdate(request,hlsPositionList);


		return results;
	}




	/**
     * 查询所有岗位信息
     */
    @Override
    public List<HLSPosition> queryAll(IRequest requestContext, HLSPosition position, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        return positionMapper.queryAll(position);
    }


    @Override
    public List<HLSPosition> batchUpdate(IRequest request, List<HLSPosition> list) {
        super.batchUpdate(request, list);
        for (HLSPosition position : list) {
            messagePublisher.publish("position.change", position);
        }
        return list;
    }


	@Override
	public Long selectById(HLSPosition hlsPosition) {
		
		return positionMapper.selectById(hlsPosition);
	}


	@Override
	public List<String> queryPositionName( HLSPosition position) {
		return positionMapper.queryPositionName(position);
	}


	/* (non-Javadoc)
	 * @see hls.core.fnd.service.HLSIPositionService#selectAll(hls.core.fnd.dto.HLSPosition)
	 */
	@Override
	public List<HLSPosition> selectAll(HLSPosition position) {
		return positionMapper.selectAll(position);
	}


	/* (non-Javadoc)
	 * @see hls.core.fnd.service.HLSIPositionService#queryAll(hls.core.fnd.dto.HLSPosition)
	 */
	@Override
	public List<HLSPosition> queryAll(HLSPosition position) {
		return positionMapper.queryAll(position);
	}
	@Override
	public List<HLSPosition> queryLovAll(HLSPosition hlsPosition){
		return positionMapper.queryLovAll(hlsPosition);
	}

}
