package hls.core.fnd.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fnd.dto.*;
import hls.core.fnd.mapper.*;
import hls.core.fnd.service.HlsFinStatementHdService;
import hls.core.fnd.service.HlsFinStatementLnService;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.service.HlsBeanRefUtilService;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * Created by haibin on 2017/6/27.
 */
@Service
@Transactional
public class HlsFinStatementHdServiceImpl extends BaseServiceImpl<HlsFinStatementHd> implements HlsFinStatementHdService {
    public static final String BALANCE_SHEET = "BALANCE_SHEET";
    public static final String PROFIT_STATEMENT = "PROFIT_STATEMENT";
    public static final String CASH_FLOW_STATEMENT = "CASH_FLOW_STATEMENT";
    public static final String FINANCIAL_INDEX = "FINANCIAL_INDEX";
    ThreadLocal<Long> message = new ThreadLocal<>();
    @Autowired
    private FndInterfaceLinesMapper fndInterfaceLinesMapper;
    @Autowired
    private HlsFinStatementTmpltLnMapper hlsFinStatementTmpltLnMapper;
    @Autowired
    private HlsBeanRefUtilService hlsBeanRefUtilService;
    @Autowired
    private HlsFinStatementLnService hlsFinStatementLnService;
    @Autowired
    private HlsFinStatementHdMapper hlsFinStatementHdMapper;
    @Autowired
    private HlsFinStatementTmpltHdMapper hlsFinStatementTmpltHdMapper;
    @Autowired
    private HlsFinStatementLnMapper hlsFinStatementLnMapper;

    @Override
    public List getList(HlsFinStatementTmpltLn hlsFinStatementTmpltLn, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        FndInterfaceLines fndInterfaceLines = new FndInterfaceLines();
        fndInterfaceLines.setHeaderId(hlsFinStatementTmpltLn.getHeaderId());
        fndInterfaceLines.setSheetName(hlsFinStatementTmpltLn.getSheetName());


        //List<FndInterfaceLines> interfaceLinesList = fndInterfaceLinesMapper.fndInterfeLacinesQuery(fndInterfaceLines);
        List<FndInterfaceLines> interfaceLinesList = fndInterfaceLinesMapper.fndInterfaceByHeaderIdAndSheetName(fndInterfaceLines);


        for (int i = 0; i < interfaceLinesList.size(); i++) {
            FndInterfaceLines interfaceLines = new FndInterfaceLines();
            interfaceLines = interfaceLinesList.get(i);
            int k = 3;
            try {
                String attr;
                while ((attr = BeanUtils.getProperty(interfaceLines, "attributes_" + k)) != null && !attr.equals(" ")) {

                    if (isNumber(attr) || isBigNumber(attr)) {
                        String amount;
                        BigDecimal db;
                        try {
                            db = new BigDecimal(attr);
                            amount = db.toString();
                        } catch (Exception e) {
                            amount = attr;
                        }
                        Map map = hlsBeanRefUtilService.getFieldValueMap(interfaceLines);
                        map.put("attributes_" + k, amount);
                        hlsBeanRefUtilService.setFieldValues(interfaceLines, map);
                    } else {
                        Map map = hlsBeanRefUtilService.getFieldValueMap(interfaceLines);
                        map.put("attributes_" + k, " ");
                        hlsBeanRefUtilService.setFieldValues(interfaceLines, map);
                    }
                    k++;
                }
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            }
            interfaceLinesList.set(i, interfaceLines);
        }
        return interfaceLinesList;
    }

    public List<Map<String, String>> getClounm(HlsFinStatementTmpltLn hlsFinStatementTmpltLn) {
        FndInterfaceLines fndInterfaceLines = new FndInterfaceLines();
        fndInterfaceLines.setHeaderId(hlsFinStatementTmpltLn.getHeaderId());
        fndInterfaceLines.setSheetName(hlsFinStatementTmpltLn.getSheetName());
        List<FndInterfaceLines> interfaceLinesList = fndInterfaceLinesMapper.fndInterColumnQuery(fndInterfaceLines);
        List<Map<String, String>> list = new ArrayList<Map<String, String>>();
        FndInterfaceLines interfaceLines = new FndInterfaceLines();
        interfaceLines = interfaceLinesList.get(0);
        String attributes_2 = interfaceLines.getAttributes_2();
        String s = replaceBlank(attributes_2);
        if ("行次".equalsIgnoreCase(s)) {
            int k = 1;
            Map<String, String> column = new HashMap<String, String>();
            column.clear();
            column = hlsBeanRefUtilService.getFieldValueMap(interfaceLines);
            while (column.get("attributes_" + k) != null) {
                Map<String, String> map = new HashMap<String, String>();
                if (k < 3) {
                    map.put("attributes_" + k, column.get("attributes_" + k).trim());
                } else {
                    String data = column.get("attributes_" + k).trim();
                    String datas[] = data.split("-");
                    String str = datas[0] + "-" + datas[1];
                    map.put("attributes_" + k, str);
                }
                list.add(map);
                k++;
            }
        }
        return list;
    }

    @Override
    public boolean statementSubmit(IRequest iRequest, HlsFinStatementTmpltLn hlsFinStatementTmpltLn) {
        FndInterfaceLines lines = new FndInterfaceLines();
        lines.setHeaderId(hlsFinStatementTmpltLn.getHeaderId());
        lines.setSheetName(hlsFinStatementTmpltLn.getSheetName());
        //从接口表中取出对应的财报数据
        List<FndInterfaceLines> interfaceLinesList = fndInterfaceLinesMapper.fndInterColumnQuery(lines);
        FndInterfaceLines interfaceLines1 = new FndInterfaceLines();
        //当没有获取到数据时,直接返回
        if (interfaceLinesList.size() == 0) {
            return false;
        }
        //获取excel的表头(第一行)
        interfaceLines1 = interfaceLinesList.get(0);

        FndInterfaceLines fndInterfaceLines = new FndInterfaceLines();
        fndInterfaceLines.setHeaderId(hlsFinStatementTmpltLn.getHeaderId());
        List<FndInterfaceLines> interfaceLinesList1 = new ArrayList<>();
        List<FndInterfaceLines> interfaceLinesList2 = new ArrayList<>();
        List<FndInterfaceLines> interfaceLinesList3 = new ArrayList<>();
        List<FndInterfaceLines> interfaceLinesList4 = new ArrayList<>();
        //找到对应的行表的数据
        List<FndInterfaceLines> interfaceLines = fndInterfaceLinesMapper.fndInterfeLacinesQuery(fndInterfaceLines);
        //将数据添加到对应的list中
        interfaceLines.forEach(item -> {
            String sheetName = replaceBlank(item.getSheetName());
            if ("资产负债表".equalsIgnoreCase(sheetName)) {
                interfaceLinesList1.add(item);
                return;
            } else if ("利润表".equalsIgnoreCase(sheetName)) {
                interfaceLinesList2.add(item);
                return;
            } else if ("现金流量表".equalsIgnoreCase(sheetName)) {
                interfaceLinesList3.add(item);
                return;
            } else if ("财务指标表".equalsIgnoreCase(sheetName)) {
                interfaceLinesList4.add(item);
                return;
            }
        });
        HlsFinStatementHd hlsFinStatementHd = new HlsFinStatementHd();
        hlsFinStatementHd.setFinStatementTempletHdId(hlsFinStatementTmpltLn.getFinStatementTempletHdId());
        hlsFinStatementHd.setBpId(hlsFinStatementTmpltLn.getBpId());
        hlsFinStatementHd.setCompanyId(hlsFinStatementTmpltLn.getCompanyId());
        HlsFinStatementTmpltHd hlsFinStatementTmpltHd = new HlsFinStatementTmpltHd();
        hlsFinStatementTmpltHd.setFinStatementTempletHdId(hlsFinStatementTmpltLn.getFinStatementTempletHdId());
        //获取财报模板头表
        List<HlsFinStatementTmpltHd> hlsFinStatementTmpltHdList = hlsFinStatementTmpltHdMapper.queryById(hlsFinStatementTmpltHd);
        hlsFinStatementTmpltHd = hlsFinStatementTmpltHdList.get(0);
        hlsFinStatementHd.setCurrencyCode(hlsFinStatementTmpltHd.getCurrencyCode());
        Long id1 = 0L;
        Long id2 = 0L;
        Long id3 = 0L;
        Long id4 = 0L;
        int k = 3;
        if (null != interfaceLines1) {
            Map<String, String> column = new HashMap<String, String>();
            column.clear();
            column = hlsBeanRefUtilService.getFieldValueMap(interfaceLines1);
            while (column.get("attributes_" + k) != null) {
                String data = column.get("attributes_" + k);
                //取出表头上的日期
                String datas[] = replaceBlank(data).trim().split("-");
                HlsFinStatementHd hlsHd = new HlsFinStatementHd();
                hlsHd.setBpId(hlsFinStatementTmpltLn.getBpId());
                hlsHd.setFiscalYear(Long.parseLong(datas[0]));
                hlsHd.setFiscalMonth(Long.parseLong(datas[1]));
                //根据bpId以及头id和年月,找到对应的财报头数据
                List<HlsFinStatementHd> hdList = hlsFinStatementHdMapper.hdQuery(hlsHd);
                HlsFinStatementHd hd = new HlsFinStatementHd();
                if (hdList.size() > 0) {
                    hd = hdList.get(0);
                    HlsFinStatementLn ln = new HlsFinStatementLn();
                    ln.setFinStatementHdId(hd.getFinStatementHdId());
                    //找到财报头对应的行表数据
                    List<HlsFinStatementLn> hlsFinStatementLns = hlsFinStatementLnMapper.lnQuery(ln);
                    //先遍历数据,如果行表存在对应年月的数据,则根据类型删除
                    hlsFinStatementLns.forEach(item -> {
                        String lineType = item.getLineType();
                        if (BALANCE_SHEET.equalsIgnoreCase(lineType) && interfaceLinesList1.size() > 0) {
                            hlsFinStatementLnService.deleteByPrimaryKey(item);
                        } else if (PROFIT_STATEMENT.equalsIgnoreCase(lineType) && interfaceLinesList2.size() > 0) {
                            hlsFinStatementLnService.deleteByPrimaryKey(item);
                        } else if (CASH_FLOW_STATEMENT.equalsIgnoreCase(lineType) && interfaceLinesList3.size() > 0) {
                            hlsFinStatementLnService.deleteByPrimaryKey(item);
                        } else if (FINANCIAL_INDEX.equalsIgnoreCase(lineType) && interfaceLinesList4.size() > 0) {
                            hlsFinStatementLnService.deleteByPrimaryKey(item);
                        }
                    });

                } else {
                    //如果头表不存在对应的财报信息,则将数据插入表中
                    hlsFinStatementHd.setFiscalYear(Long.parseLong(datas[0]));
                    hlsFinStatementHd.setFiscalMonth(Long.parseLong(datas[1]));
                    hlsFinStatementHd.set__status("add");
                    hd = self().insertSelective(iRequest, hlsFinStatementHd);
                }
                //赋值头表id
                if (3 == k) {
                    id1 = hd.getFinStatementHdId();
                } else if (4 == k) {
                    id2 = hd.getFinStatementHdId();
                } else if (5 == k) {
                    id3 = hd.getFinStatementHdId();
                } else if (6 == k) {
                    id4 = hd.getFinStatementHdId();
                }
                k++;
            }
        }
        if (id1 == 0 && id2 == 0 && id3 == 0 && id4 == 0) {
            return false;
        }
        List<HlsFinStatementLn> hlsLnList = new ArrayList<HlsFinStatementLn>();
        List<HlsFinStatementTmpltLn> list1 = new ArrayList<>();
        List<HlsFinStatementTmpltLn> list2 = new ArrayList<>();
        List<HlsFinStatementTmpltLn> list3 = new ArrayList<>();
        List<HlsFinStatementTmpltLn> list4 = new ArrayList<>();
        //找到模板行的值
        List<HlsFinStatementTmpltLn> templateLnList = hlsFinStatementTmpltLnMapper.query(hlsFinStatementTmpltLn);
        templateLnList.forEach(item -> {
            String lineType = replaceBlank(item.getLineType());
            if (BALANCE_SHEET.equalsIgnoreCase(lineType)) {
                list1.add(item);
                return;
            } else if (PROFIT_STATEMENT.equalsIgnoreCase(lineType)) {
                list2.add(item);
                return;
            } else if (CASH_FLOW_STATEMENT.equalsIgnoreCase(lineType)) {
                list3.add(item);
                return;
            } else if (FINANCIAL_INDEX.equalsIgnoreCase(lineType)) {
                list4.add(item);
                return;
            }
        });

        if (interfaceLinesList1.size() > 0) {
            List<HlsFinStatementLn> list = dealAmount(list1, interfaceLinesList1, k, id1, id2, id3, id4);
            hlsLnList.addAll(list);
        }
        if (interfaceLinesList2.size() > 0) {
            List<HlsFinStatementLn> list = dealAmount(list2, interfaceLinesList2, k, id1, id2, id3, id4);
            hlsLnList.addAll(list);
        }
        if (interfaceLinesList3.size() > 0) {
            List<HlsFinStatementLn> list = dealAmount(list3, interfaceLinesList3, k, id1, id2, id3, id4);
            hlsLnList.addAll(list);
        }
        if (interfaceLinesList4.size() > 0) {
            List<HlsFinStatementLn> list = dealAmount(list4, interfaceLinesList4, k, id1, id2, id3, id4);
            hlsLnList.addAll(list);
        }
        List<HlsFinStatementLn> lnList = new ArrayList<HlsFinStatementLn>();
        if (hlsLnList.size() == 0) {
            return false;
        }
        for (int j = 0; j < hlsLnList.size(); j++) {
            HlsFinStatementLn finStatementLn = hlsLnList.get(j);
            if (finStatementLn.getAmount() != null) {//导入时,先根据excel上对应的字段是否有amount值存储
                BigDecimal db = new BigDecimal(finStatementLn.getAmount());
                String amount = db.toPlainString();
                finStatementLn.setAmount(Double.parseDouble(amount));
                finStatementLn.setFinalAmount(Double.parseDouble(amount));
            }
            hlsFinStatementLnService.insertSelective(iRequest, finStatementLn);
        }
        Map<String, Long> map = new HashMap<>();
        map.put("id1", id1);
        map.put("id2", id2);
        map.put("id3", id3);
        map.put("id4", id4);
        boolean flag = statementLnUpdate(iRequest, map, hlsFinStatementTmpltLn.getFinStatementTempletHdId());
        if (!flag) {
            return false;
        }
        return true;
    }

    public boolean statementLnUpdate(IRequest iRequest, Map<String, Long> map, Long finStatementTempletHdId) {
        Long id1 = map.get("id1");
        Long id2 = map.get("id2");
        Long id3 = map.get("id3");
        Long id4 = map.get("id4");
        HlsFinStatementTmpltLn hlsFinStatementTmpltLn = new HlsFinStatementTmpltLn();
        hlsFinStatementTmpltLn.setFinStatementTempletHdId(finStatementTempletHdId);
        hlsFinStatementTmpltLn.setLineType(BALANCE_SHEET);
        List<HlsFinStatementTmpltLn> list1 = hlsFinStatementTmpltLnMapper.queryByFormula(hlsFinStatementTmpltLn);
        hlsFinStatementTmpltLn.setLineType(PROFIT_STATEMENT);
        List<HlsFinStatementTmpltLn> list2 = hlsFinStatementTmpltLnMapper.queryByFormula(hlsFinStatementTmpltLn);
        hlsFinStatementTmpltLn.setLineType(CASH_FLOW_STATEMENT);
        List<HlsFinStatementTmpltLn> list3 = hlsFinStatementTmpltLnMapper.queryByFormula(hlsFinStatementTmpltLn);
        hlsFinStatementTmpltLn.setLineType(FINANCIAL_INDEX);
        List<HlsFinStatementTmpltLn> list4 = hlsFinStatementTmpltLnMapper.queryByFormula(hlsFinStatementTmpltLn);
        if (id1 != 0) {
            boolean flag = judgeIds(iRequest, list1, list2, list3, list4, id1);
            if (!flag) {
                return false;
            }
        }
        if (id2 != 0) {
            boolean flag = judgeIds(iRequest, list1, list2, list3, list4, id2);
            if (!flag) {
                return false;
            }
        }
        if (id3 != 0) {
            boolean flag = judgeIds(iRequest, list1, list2, list3, list4, id3);
            if (!flag) {
                return false;
            }
        }
        if (id4 != 0) {
            boolean flag = judgeIds(iRequest, list1, list2, list3, list4, id4);
            if (!flag) {
                return false;
            }
        }
        return true;
    }

    public boolean judgeIds(IRequest iRequest, List<HlsFinStatementTmpltLn> list1, List<HlsFinStatementTmpltLn> list2, List<HlsFinStatementTmpltLn> list3, List<HlsFinStatementTmpltLn> list4, Long ids) {
        HlsFinStatementLn hlsFinStatementLn = new HlsFinStatementLn();
        hlsFinStatementLn.setFinStatementHdId(ids);
        hlsFinStatementLn.setLineType(BALANCE_SHEET);
        List<HlsFinStatementLn> lnList1 = hlsFinStatementLnMapper.lnQuery(hlsFinStatementLn);
        Map<String, List<HlsFinStatementLn>> totalList = new HashMap<>();
        message.set(ids);
        if (lnList1.size() > 0) {
            totalList.put(BALANCE_SHEET, lnList1);
            boolean flag = calculateAmount(iRequest, list1, lnList1, totalList);
            if (!flag) {
                return false;
            }
        }
        hlsFinStatementLn.setLineType(PROFIT_STATEMENT);
        List<HlsFinStatementLn> lnList2 = hlsFinStatementLnMapper.lnQuery(hlsFinStatementLn);
        if (lnList2.size() > 0) {
            totalList.put(PROFIT_STATEMENT, lnList2);
            boolean flag = calculateAmount(iRequest, list2, lnList2, totalList);
            if (!flag) {
                return false;
            }
        }
        hlsFinStatementLn.setLineType(CASH_FLOW_STATEMENT);
        List<HlsFinStatementLn> lnList3 = hlsFinStatementLnMapper.lnQuery(hlsFinStatementLn);
        if (lnList3.size() > 0) {
            totalList.put(CASH_FLOW_STATEMENT, lnList3);
            boolean flag = calculateAmount(iRequest, list3, lnList3, totalList);
            if (!flag) {
                return false;
            }
        }
        hlsFinStatementLn.setLineType(FINANCIAL_INDEX);
        List<HlsFinStatementLn> lnList4 = hlsFinStatementLnMapper.lnQuery(hlsFinStatementLn);
        if (lnList4.size() > 0) {
            totalList.put(FINANCIAL_INDEX, lnList4);
            boolean flag = calculateAmount(iRequest, list4, lnList4, totalList);
            if (!flag) {
                return false;
            }
        }
        return true;
    }

    public boolean calculateAmount(IRequest iRequest, List<HlsFinStatementTmpltLn> list, List<HlsFinStatementLn> lnList, Map<String, List<HlsFinStatementLn>> totalList) {
        list = list.stream().sorted((a, b) -> Long.compare(a.getFormulaPriority(), b.getFormulaPriority())).collect(Collectors.toList());//根据计算优先值从小到大排序
        for (int i = 0; i < list.size(); i++) {
            HlsFinStatementTmpltLn hlsFinStatementTmpltLn = list.get(i);
            for (int j = 0; j < lnList.size(); j++) {
                HlsFinStatementLn hlsFinStatementLn = lnList.get(j);
                if (hlsFinStatementLn == null)//当获取的元素为空时,直接进入下次循环
                    continue;
                if (hlsFinStatementTmpltLn.getLineSeqNumber() == hlsFinStatementLn.getLineSeqNumber()) {
                    if (hlsFinStatementLn.getAmount() == null ) {
                        if (-1 != hlsFinStatementTmpltLn.getFormulaText().indexOf("[") && -1 != hlsFinStatementTmpltLn.getFormulaText().indexOf("]")) {
//                            List<String> elements = replaceByRegular(hlsFinStatementTmpltLn.getFormulaText());//获取对应的元素
//                            List<Double> strList = getEleValueByRegular(elements, lnList, totalList);
//                            //List<String> strList = extractMessage(hlsFinStatementTmpltLn.getFormulaText());
//                            List<String> signList = extractMessageByRegular(hlsFinStatementTmpltLn.getFormulaText());
//                            Double sum = 0d;
//                            for (int a = 0; a < strList.size(); a++) {
//                                if (a == 0) {
//                                    sum = strList.get(0);
//                                } else {
//                                    if (signList.get(a - 1) != null) {
//                                        Double param = strList.get(a);
//                                        if (signList.get(a - 1).equals("+")) {
//                                            sum = CalculateUtil.add(sum, param);
//                                        }
//                                        if (signList.get(a - 1).equals("-")) {
//                                            sum = CalculateUtil.sub(sum, param);
//                                        }
//                                    }
//                                }
//                            }
                            //元素运算改为js引擎
                            Double sum = getCalculateResult(hlsFinStatementTmpltLn.getFormulaText(), lnList, totalList);
                            hlsFinStatementLn.setAmount(sum);
                            hlsFinStatementLn.setFinalAmount(sum);
                            lnList.set(j, hlsFinStatementLn);
                            hlsFinStatementLn.set__status("update");
                            HlsFinStatementLn ln = hlsFinStatementLnService.updateByPrimaryKeySelective(iRequest, hlsFinStatementLn);
                            if (ln.getAmount() != sum) {
                                return false;
                            }
                        }
                    }
                    break;
                }
            }
        }
        return true;
    }

    /**
     * 获取公式计算的结果
     *
     * @param formulaText
     * @param lnList
     * @param totalList
     * @return
     */
    private Double getCalculateResult(String formulaText, List<HlsFinStatementLn> lnList, Map<String, List<HlsFinStatementLn>> totalList){
        StringBuilder sb = new StringBuilder(formulaText);
        String element;
        String elementValue;
        while (sb.lastIndexOf("[") >= 0 && sb.lastIndexOf("]") > 0) {
            element = sb.subSequence(sb.lastIndexOf("[") + 1, sb.lastIndexOf("]")).toString();
            List<String> list = new ArrayList<>();
            list.add(element);
            List<Double> eleValueByRegular = getEleValueByRegular(list, lnList, totalList);
            elementValue = String.valueOf(eleValueByRegular.stream().findAny().orElse(0D));
            sb.replace(sb.lastIndexOf("["), sb.lastIndexOf("]")+1, String.valueOf(elementValue));
        }
        Double result = null;
        try {
            result = excuetExpressionByJs(sb.toString());
        } catch (ScriptException e) {
            e.printStackTrace();
        }
        //当计算到无穷值时
        if (new Double("Infinity").compareTo(result) == 0){
            return null;
        }
        if (new Double("-Infinity").compareTo(result) == 0){
            return null;
        }
        return result;
    }

    /**
     * @param elements
     * @param lnList
     * @param totalList
     * @return
     */
    private List<Double> getEleValueByRegular(List<String> elements, List<HlsFinStatementLn> lnList, Map<String, List<HlsFinStatementLn>> totalList) {
        List<Double> result = new ArrayList<>();
        elements.forEach(item -> {
            String[] splitArr = item.split("\\.");
            List<String> splitList = Arrays.asList(splitArr);
            if (splitList.size() == 3) {//当根据.标志截取的字符串数组的大小为3时,表示层级为3
                Long hdId = message.get();
                HlsFinStatementHd hlsFinStatementHd = hlsFinStatementHdMapper.selectByPrimaryKey(hdId);
                HlsFinStatementHd queryParam = new HlsFinStatementHd();
                queryParam.setBpId(hlsFinStatementHd.getBpId());
                queryParam.setCompanyId(hlsFinStatementHd.getCompanyId());
                List<HlsFinStatementHd> hlsFinStatementHds = hlsFinStatementHdMapper.hdQueryByTime(queryParam);
                String location = splitList.get(0).substring(1, splitList.get(0).length());
                String key = splitList.get(1);
                String position = splitList.get(2).substring(0, splitList.get(2).length());
                Integer locationIndex = Integer.parseInt(location.substring(0, location.length()));
                if (!location.startsWith("a")) {
                    locationIndex = -locationIndex;
                }
                Integer index = null;
                for (int i = 0; i < hlsFinStatementHds.size(); i++) {
                    if (hlsFinStatementHds.get(i).getFinStatementHdId().compareTo(hlsFinStatementHd.getFinStatementHdId()) == 0) {
                        index = i + locationIndex;
                        break;
                    }
                }
                if (index >= 0 && index < hlsFinStatementHds.size()) {
                    HlsFinStatementHd finStatementHd = hlsFinStatementHds.get(index);
                    Long finStatementHdId = finStatementHd.getFinStatementHdId();
                    HlsFinStatementLn finStatementLn = new HlsFinStatementLn();
                    finStatementLn.setFinStatementHdId(finStatementHdId);
                    finStatementLn.setLineType(key.toUpperCase());
                    List<HlsFinStatementLn> select = hlsFinStatementLnMapper.select(finStatementLn);
                    List<HlsFinStatementLn> collect = select.stream().sorted((a, b) -> Long.compare(a.getLineSeqNumber(), b.getLineSeqNumber())).collect(Collectors.toList());
                    String amountStr = getAmount(collect, position);
                    Double amount = Double.parseDouble(amountStr);
                    result.add(amount);
                }

            } else if (splitList.size() == 2) {//当根据.标志截取的字符串数组的大小为2时,表示层级为2
                String key = splitList.get(0);
                String position = splitList.get(1);
                List<HlsFinStatementLn> hlsFinStatementLns = totalList.get(key.toUpperCase());
                String amountStr = getAmount(hlsFinStatementLns, position);
                Double amount = Double.parseDouble(amountStr);
                result.add(amount);

            } else if (splitList.size() == 1) {//当根据.标志截取的字符串数组的大小为1时,表示层级为1
                String s = splitList.stream().findAny().get();
                String amount = getAmount(lnList, s);
                result.add(Double.parseDouble(amount));
//                int start = 0;
//                int end = 0;
//                for (int i = 0; i < s.length(); i++) {
//                    if (s.charAt(i) == '[') {
//                        start = i;
//                    } else if (s.charAt(i) == ']') {
//                        end = i;
//                        Double amount = Double.parseDouble(getAmount(lnList, s.substring(start + 1, end)));
//                        result.add(amount);
//                        break;
//                    }
//                }
            } else {
                return;
            }

        });
        return result;
    }

    public List<String> extractMessage(String msg) {
        List<String> list = new ArrayList<String>();
        int start = 0;
        int startFlag = 0;
        int endFlag = 0;

        for (int i = 0; i < msg.length(); i++) {
            if (msg.charAt(i) == '[') {
                startFlag++;
                if (startFlag == endFlag + 1) {
                    start = i;
                }
            } else if (msg.charAt(i) == ']') {
                endFlag++;
                if (endFlag == startFlag) {
                    list.add(msg.substring(start + 1, i));
                }
            } else {

            }
        }
        return list;
    }

    /**
     * @param list               模板行
     * @param interfaceLinesList 接口表中存储的值
     * @param k                  字段
     * @param id1
     * @param id2
     * @param id3
     * @param id4
     * @return
     */
    public List<HlsFinStatementLn> dealAmount(List<HlsFinStatementTmpltLn> list, List<FndInterfaceLines> interfaceLinesList, int k, Long id1, Long id2, Long id3, Long id4) {
        List<HlsFinStatementLn> lnList = new ArrayList<HlsFinStatementLn>();
        //i从1开始,因为第一行的数据存储的是表头
        for (int i = 1; i < interfaceLinesList.size(); i++) {
            FndInterfaceLines interFace = interfaceLinesList.get(i);
            String attributes_2 = interFace.getAttributes_2();
            String lineSeq = replaceBlank(attributes_2);
            List temp = new ArrayList<>();
            for (int j = 0; j < list.size(); j++) {
                HlsFinStatementTmpltLn item = list.get(j);
                if (lineSeq.equalsIgnoreCase(Long.toString(item.getLineSeqNumber()))) {
                    Map map = hlsBeanRefUtilService.getFieldValueMap(item);
                    //循环保存sheet页中对应的4列
                    for (int b = 3; b < k; b++) {
                        HlsFinStatementLn result = new HlsFinStatementLn();
                        String attr = null;
                        try {
                            attr = BeanUtils.getProperty(interFace, "attributes_" + b);
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        }
                        if ("".equals(attr.trim())) {
                            map.put("sheetName", interFace.getSheetName());
                        } else {
                            if (isNumber(attr) || isBigNumber(attr)) {
                                String num = attr;
                                BigDecimal db = new BigDecimal(num.trim());
                                String amount = db.toPlainString();
                                map.put("amount", amount);
                            } else {
                                map.put("amount", null);
                            }
                            map.put("sheetName", interFace.getSheetName().trim());
                        }
                        hlsBeanRefUtilService.setFieldValues(result, map);
                        Double amount = result.getAmount() != null ? result.getAmount() : 0D;
                        if (3 == b) {
                            result.setFinStatementHdId(id1);
                            interfaceLinesList.get(i).setAttributes_3(amount.toString());
                        } else if (4 == b) {
                            result.setFinStatementHdId(id2);
                            interfaceLinesList.get(i).setAttributes_4(amount.toString());
                        } else if (5 == b) {
                            result.setFinStatementHdId(id3);
                            interfaceLinesList.get(i).setAttributes_5(amount.toString());
                        } else if (6 == b) {
                            result.setFinStatementHdId(id4);
                            interfaceLinesList.get(i).setAttributes_6(amount.toString());
                        }
                        temp.add(result);
                        lnList.add(result);
                        map.put("amount", null);
                    }
                }
                if (temp.isEmpty() && j == list.size() - 1) {
                    Map map = hlsBeanRefUtilService.getFieldValueMap(item);
                    //循环保存sheet页中对应的4列
                    for (int b = 3; b < k; b++) {
                        HlsFinStatementLn result = new HlsFinStatementLn();
                        String attr = null;
                        try {
                            attr = BeanUtils.getProperty(interFace, "attributes_" + b);
                        } catch (IllegalAccessException e) {
                            e.printStackTrace();
                        } catch (InvocationTargetException e) {
                            e.printStackTrace();
                        } catch (NoSuchMethodException e) {
                            e.printStackTrace();
                        }
                        if ("".equals(attr.trim())) {
                            map.put("sheetName", interFace.getSheetName());
                        } else {
                            if (isNumber(attr) || isBigNumber(attr)) {
                                String num = attr;
                                BigDecimal db = new BigDecimal(num.trim());
                                String amount = db.toPlainString();
                                map.put("amount", amount);
                            } else {
                                map.put("amount", null);
                            }
                            map.put("sheetName", interFace.getSheetName().trim());
                        }
                        Double resultAmount = (Double) map.get("amount");
                        result.setAmount(resultAmount);
                        result.setFinalAmount(resultAmount);
                        result.setLineSeqNumber(Long.parseLong(interFace.getAttributes_2().trim()));
                        result.setFinStatementItemName(interFace.getAttributes_1());
                        //result.set
                        result.setLineType((String) map.get("lineType"));
                        String amount = "";
                        if (resultAmount != null)
                            amount = resultAmount.toString();
                        if (3 == b) {
                            result.setFinStatementHdId(id1);
                            interfaceLinesList.get(i).setAttributes_3(amount);
                        } else if (4 == b) {
                            result.setFinStatementHdId(id2);
                            interfaceLinesList.get(i).setAttributes_4(amount);
                        } else if (5 == b) {
                            result.setFinStatementHdId(id3);
                            interfaceLinesList.get(i).setAttributes_5(amount);
                        } else if (6 == b) {
                            result.setFinStatementHdId(id4);
                            interfaceLinesList.get(i).setAttributes_6(amount);
                        }
                        lnList.add(result);
                        map.put("amount", null);
                    }
                }
            }
        }

//        list.parallelStream().forEach(hlsFinStatementTmpltLn1 -> {
//            for (int i = 0; i < interfaceLinesList.size(); i++) {
//                FndInterfaceLines interFace = interfaceLinesList.get(i);
//                String attributes_2 = interFace.getAttributes_2();
//                String s = replaceBlank(attributes_2);
//                if (Long.toString(hlsFinStatementTmpltLn1.getLineSeqNumber()).equals(attributes_2.trim())) {
//                    //Map<String, String> fieldValueMap = hlsBeanRefUtilService.getFieldValueMap(interFace);
//                    Map map = hlsBeanRefUtilService.getFieldValueMap(hlsFinStatementTmpltLn1);
//                    for (int b = 3; b < k; b++) {
//                        HlsFinStatementTmpltLn hlsFinStatementTmpltLn2 = new HlsFinStatementTmpltLn();
//                        String attr = null;
//                        try {
//                            attr = BeanUtils.getProperty(interFace, "attributes_" + b);
//                        } catch (IllegalAccessException e) {
//                            e.printStackTrace();
//                        } catch (InvocationTargetException e) {
//                            e.printStackTrace();
//                        } catch (NoSuchMethodException e) {
//                            e.printStackTrace();
//                        }
//                        if ("".equals(attr.trim())) {
//                            map.put("sheetName", interFace.getSheetName());
//                        } else {
//                            if (isNumber(attr) || isBigNumber(attr)) {
//                                String num = attr;
//                                BigDecimal db = new BigDecimal(num.trim());
//                                String amount = db.toPlainString();
//                                map.put("amount", amount);
//                            } else {
//                                map.put("amount", null);
//                            }
//                            map.put("sheetName", interFace.getSheetName().trim());
//                        }
//                        hlsBeanRefUtilService.setFieldValues(hlsFinStatementTmpltLn2, map);
//                        if (3 == b) {
//                            hlsFinStatementTmpltLn2.setFinStatementHdId(id1);
//                            interfaceLinesList.get(i).setAttributes_3(hlsFinStatementTmpltLn2.getAmount());
//                        } else if (4 == b) {
//                            hlsFinStatementTmpltLn2.setFinStatementHdId(id2);
//                            interfaceLinesList.get(i).setAttributes_4(hlsFinStatementTmpltLn2.getAmount());
//                        } else if (5 == b) {
//                            hlsFinStatementTmpltLn2.setFinStatementHdId(id3);
//                            interfaceLinesList.get(i).setAttributes_5(hlsFinStatementTmpltLn2.getAmount());
//                        } else if (6 == b) {
//                            hlsFinStatementTmpltLn2.setFinStatementHdId(id4);
//                            interfaceLinesList.get(i).setAttributes_6(hlsFinStatementTmpltLn2.getAmount());
//                        }
//                        lnList.add(hlsFinStatementTmpltLn2);
//                    }
//                    break;
//                }
//            }
//        });
        return lnList;
    }

    public boolean isNumber(String num) {
        Pattern pattern = Pattern.compile("^(-)?\\d+(\\.\\d+)?$");
        Matcher m = pattern.matcher(num);
        boolean flag = m.find();
        return flag;
    }

    public boolean isBigNumber(String num) {
        Pattern pattern = Pattern.compile("[0-9]+[a-zA-Z]+[0-9a-zA-Z]*");
        Matcher m = pattern.matcher(num);
        boolean flag = m.find();
        return flag;
    }

    public String getAmount(List<HlsFinStatementLn> list, String numSeq) {
        String amount = "0";
        for (int i = 0; i < list.size(); i++) {
            HlsFinStatementLn hlsFinStatementLn = new HlsFinStatementLn();
            hlsFinStatementLn = list.get(i);
            if (hlsFinStatementLn.getLineSeqNumber() == (Long.parseLong(numSeq))) {
                if (hlsFinStatementLn.getAmount() == null) {
                    return amount;
                } else {
                    BigDecimal db = new BigDecimal(hlsFinStatementLn.getAmount());
                    amount = db.toPlainString();
                    return amount;
                }
            }
        }
        return amount;
    }


    public List<String> extractMessageByRegular(String msg) {

        List<String> list = new ArrayList<String>();
        Pattern p = Pattern.compile("\\+|\\-");
        Matcher m = p.matcher(msg);
        while (m.find()) {
            list.add(m.group());
        }
        return list;
    }

    /**
     * 根据—,+分割出对应的元素
     *
     * @param msg
     * @return
     */
    public List<String> replaceByRegular(String msg) {
        if (msg == null)
            return new ArrayList<>();
        String[] split = msg.split("\\+|-");
        return Arrays.asList(split);
    }

    public Double add(Double sum, Double amount) {
        return sum + amount;
    }

    public Double minus(Double sum, Double amount) {
        return sum - amount;
    }

    @Override
    public List<HlsFinStatementHd> hdQuery(HlsFinStatementHd hlsFinStatementHd, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        List<HlsFinStatementHd> list = hlsFinStatementHdMapper.hdQuery(hlsFinStatementHd);
        return list;
    }

    @Override
    public List<HlsFinStatementHd> hdGuaranQuery(HlsFinStatementHd hlsFinStatementHd, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        List<HlsFinStatementHd> list = hlsFinStatementHdMapper.hdGuaranQuery(hlsFinStatementHd);
        return list;
    }

    @Override
    public List<HlsFinStatementHd> hdColumnQuery(HlsFinStatementHd hlsFinStatementHd) {
        List<HlsFinStatementHd> list = hlsFinStatementHdMapper.hdColumnQuery(hlsFinStatementHd);
        return list;
    }

    public static String replaceBlank(String str) {
        String dest = "";
        if (str != null) {
            Pattern p = Pattern.compile("\\s*|\t|\r|\n");
            Matcher m = p.matcher(str);
            dest = m.replaceAll("");
        }
        return dest;
    }

    /**
     * 使用js引擎执行字符串中的计算表达式
     *
     * @param expression
     * @return
     */
    private static Double excuetExpressionByJs(String expression) throws ScriptException {
        ScriptEngineManager sem = new ScriptEngineManager();
        ScriptEngine se = sem.getEngineByName("javascript");
        String eval = se.eval(expression).toString();
        return Double.parseDouble(eval);
    }
}