package hls.core.gld.components;

import com.hand.hap.core.IRequest;
import hls.core.gld.dto.JeTrxDtl;
import hls.core.gld.service.IJeTrxDtlService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;
import java.util.Map;

/**
 * 凭证事物接口
 * Created by fjm on 2017/8/31.
 */
public abstract class AbstractJeTrxService {

    protected final static String STATUS = "NEW";//新增凭证状态
    protected final static String OPERATE_TYPE_CREATE = "CREATE";//制证
    protected final static String OPERATE_TYPE_REVERSE = "REVERSE";//反冲
    private ThreadLocal<String> operateType = new ThreadLocal<>();
    //private String operateType = OPERATE_TYPE_CREATE;

    @Autowired
    private IJeTrxDtlService jeTrxDtlService;

    public abstract String getJeTrx();

    public Long getJeTrxId(Map param) {
        return param.get("jeTrxId") != null ? (Long) param.get("jeTrxId") : null;
    }

    public Long getCompanyId(Map param) {
        return param.get("companyId") != null ? (Long) param.get("companyId") : null;
    }

    public Long getContractId(Map param) {
        return param.get("contractId") != null ? (Long) param.get("contractId") : null;
    }

    public String getSourceDoc(Map param) {
        return param.get("sourceDoc") != null ? (String) param.get("sourceDoc") : null;
    }

    //生成凭证事物时建议直接使用jeSourceId字段 by fjm 17.09.21
    public Long getJeSourceId(Map param) {
        return param.get("jeSourceId") != null ? (Long) param.get("jeSourceId") : null;
    }

    //在生成凭证事物的时候也可以使用jeSourceDoc字段 by fjm 17.09.21
    public String getJeSourceDoc(Map param) {
        return param.get("jeSourceDoc") != null ? (String) param.get("jeSourceDoc") : null;
    }

    //生成反冲事物时,所取的反冲事物ID
    public Long getReverseJeTrxId(Map param) {
        Object reverseJeTrxId = param.get("reverseJeTrxId");
        if (reverseJeTrxId != null) {
            operateType.set(OPERATE_TYPE_REVERSE);
            return (Long) reverseJeTrxId;
        } else {
            operateType.set(OPERATE_TYPE_CREATE);
            return null;
        }
    }

    //生成反冲事物时,所取的反冲日期
    public Date getReverseJeDate(Map param) {
        return param.get("reverseJeDate") != null ? (Date) param.get("reverseJeDate") : null;
    }

    /**
     * 执行方法
     *
     * @param request
     * @param param
     */
    public void process(IRequest request, Map param) {
        operateType.set(OPERATE_TYPE_CREATE);
        if (!before(request, param)) {
            return;
        }
        JeTrxDtl jeTrxDtl = new JeTrxDtl();
        jeTrxDtl.setJeTrx(getJeTrx());
        jeTrxDtl.setJeTrxId(getJeTrxId(param));
        jeTrxDtl.setCompanyId(getCompanyId(param));
        //生成凭证事物时,参数中应该直接使用jeSourceId by fjm 17.09.21
        if (getJeSourceId(param) != null)
            jeTrxDtl.setJeSourceId(getJeSourceId(param));
        else
            jeTrxDtl.setJeSourceId(getContractId(param));
        //增加判断如果getSourceDoc取不到对应的值,则使用getJeSourceDoc by fjm 17.09.21
        if (getSourceDoc(param) != null)
            jeTrxDtl.setJeSourceDoc(getSourceDoc(param));
        else
            jeTrxDtl.setJeSourceDoc(getJeSourceDoc(param));
        jeTrxDtl.setStatus(STATUS);

        //反冲
        jeTrxDtl.setReverseJeTrxId(getReverseJeTrxId(param));
        jeTrxDtl.setReverseJeDate(getReverseJeDate(param));
        jeTrxDtl.setOperateType(operateType.get());

        JeTrxDtl result = jeTrxDtlService.insertSelective(request, jeTrxDtl);
        after(request, param, result);
    }


    /**
     * 生成凭证事务流水之前的拓展操作
     * 为true时继续执行,否则直接返回,不执行after
     *
     * @param request
     * @param param
     */
    protected abstract boolean before(IRequest request, Map param);

    /**
     * 生成凭证事务流水之后的拓展操作
     *
     * @param request
     * @param param
     * @param result
     */
    protected abstract void after(IRequest request, Map param, JeTrxDtl result);


}
