package hls.core.gld.controllers;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.hand.hap.excel.dto.ColumnInfo;
import com.hand.hap.excel.dto.ExportConfig;
import com.hand.hap.excel.service.IExportService;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.gld.dto.ContractFinanceIncome;
import hls.core.gld.service.IContractFinanceIncomeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.List;

@Controller
public class ContractFinanceIncomeController extends BaseController {

    @Autowired
    private IContractFinanceIncomeService service;

    @Autowired
    ObjectMapper objectMapper;

    @Autowired
    IExportService excelService;
    @RequestMapping(value = "/gld/contract/finance/income/query")
    @ResponseBody
    public ResponseData query(ContractFinanceIncome dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/gld/contract/finance/income/submit")
    @ResponseBody
    public ResponseData update(HttpServletRequest request, @RequestBody List<ContractFinanceIncome> dto) {
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/gld/contract/finance/income/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<ContractFinanceIncome> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/gld/contract/finance/income/post")
    @ResponseBody
    public ResponseData post(HttpSession session,HttpServletRequest request, @RequestBody List<ContractFinanceIncome> dto) {
        IRequest requestCtx = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        dto.forEach(item->item.setCompanyId(companyId));
        return new ResponseData(service.post(requestCtx, dto));
    }



    /**
     * 分摊
     *
     * @param request
     * @param contractIds
     * @return
     */
    @RequestMapping(value = "/gld/finance/income/apportioned")
    @ResponseBody
    public ResponseData apportioned(HttpServletRequest request,@RequestParam(name = "contractIds") Long[] contractIds) {
        IRequest requestCtx = createRequestContext(request);
        for (Long id :
                contractIds) {
            service.createGldContract(requestCtx, id);
        }
        return new ResponseData();
    }

    /*财务首页计提查询清单*/
    @RequestMapping(value="/gld/finance/income/queryList")
    @ResponseBody
    public ResponseData queryList(@RequestBody(required = false) ContractFinanceIncome dto, HttpServletRequest request,HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        dto.setCompanyId(companyId);
        return new ResponseData(service.queryList(dto));
    }
    /*计提查询清单导出*/
    @RequestMapping(value="/gld/finance/income/exportInfo")
    public void  exportInfo(ContractFinanceIncome dto, HttpServletRequest request, HttpServletResponse httpServletResponse, @RequestParam String config){
        IRequest requestContext = createRequestContext(request);
        try {
			com.fasterxml.jackson.databind.JavaType type = objectMapper.getTypeFactory().constructParametrizedType(ExportConfig.class,
					ExportConfig.class, ContractFinanceIncome.class, ColumnInfo.class);
			ExportConfig<ContractFinanceIncome, ColumnInfo> exportConfig = objectMapper.readValue(config, type);
			excelService.exportAndDownloadExcel("hls.core.gld.mapper.ContractFinanceIncomeMapper.queryList",
					exportConfig, request, httpServletResponse, requestContext);
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
}