package hls.core.gld.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.gld.dto.HlsCusGldLonContractFinCost;
import hls.core.gld.dto.HlsCusGldLonContractHd;
import hls.core.gld.dto.PeriodStatus;
import hls.core.gld.mapper.PeriodStatusMapper;
import hls.core.gld.service.IGldLonContractFinCostService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Controller
public class GldLonContractFinCostController extends BaseController {

    @Autowired
    private IGldLonContractFinCostService service;

    @Autowired
    private PeriodStatusMapper periodStatusMapper;


    @RequestMapping(value = "/gld/lon/contract/fin/cost/query")
    @ResponseBody
    public ResponseData query(HlsCusGldLonContractFinCost dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/gld/lon/contract/fin/cost/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusGldLonContractFinCost> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        PeriodStatus periodStatus = new PeriodStatus();
        periodStatus.setCompanyId(requestCtx.getCompanyId());
        periodStatus.setStatus("OPEN");
        List<PeriodStatus> select = periodStatusMapper.select(periodStatus);
        Map<String, PeriodStatus> collect = select.stream().collect(Collectors.toMap(PeriodStatus::getPeriodName, item -> item));
        HlsCusGldLonContractFinCost finCost = new HlsCusGldLonContractFinCost();
        for (HlsCusGldLonContractFinCost item:dto) {
            Long financeCostId = item.getFinanceCostId();
            finCost.setFinanceCostId(financeCostId);
            String periodName = service.selectByPrimaryKey(requestCtx, finCost).getPeriodName();
            if (!collect.containsKey(periodName)){
                ResponseData responseData = new ResponseData(false);
                responseData.setMessage("期间 "+periodName+" 未打开");
                return responseData;
            }
        }
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/gld/lon/contract/fin/cost/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusGldLonContractFinCost> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/gld/lon/contract/fin/cost/create")
    @ResponseBody
    public ResponseData createLonFinCost(HttpServletRequest request,@RequestParam(name = "trxIds") Long[] trxIds) {
        IRequest currentRequest = RequestHelper.getCurrentRequest();
        List<Long> list = Arrays.asList(trxIds);
        service.createLonContractFinCost(currentRequest,list);
        return new ResponseData();
    }

    /*计提查询*/
    @RequestMapping(value = "/gld/lon/contract/fin/cost/countDrawQuery")
    @ResponseBody
    public ResponseData countDrawQuery(@RequestBody(required = false) HlsCusGldLonContractFinCost dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        dto.setCompanyId(companyId);
        return new ResponseData(service.countDrawQuery(requestContext, dto, page, pageSize));
    }
}