package hls.core.gld.controllers;

import com.hand.hap.lock.components.DatabaseLockProvider;
import hls.core.gld.dto.JeLine;
import hls.core.gld.service.IJeLineService;
import org.apache.bcel.generic.NEW;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.gld.dto.JeHead;
import hls.core.gld.service.IJeHeadService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.validation.BindingResult;

import java.util.*;

@Controller
public class JeHeadController extends BaseController {

    @Autowired
    private IJeHeadService service;


    @RequestMapping(value = "/gld/je/head/query")
    @ResponseBody
    public ResponseData query(JeHead dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/gld/je/head/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<JeHead> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/gld/je/head/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<JeHead> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    /*凭证头表查询*/
    @RequestMapping(value = "/gld/je/head/jeListQuery")
    @ResponseBody
    public ResponseData jeListQuery(@RequestBody(required = false)JeHead dto,
                                    @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize,
                                    HttpServletRequest request, HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("companyId",companyId);
        map.put("jeHeadId",dto.getJeHeadId());
        map.put("periodName",dto.getPeriodName());
        map.put("dateFrom",dto.getDateFrom());
        map.put("dateTo",dto.getDateTo());
        map.put("description",dto.getDescription());
        map.put("companyId",companyId);
        map.put("amountFrom",dto.getAmountFrom());
        map.put("amountTo",dto.getAmountTo());
        map.put("jeTrxDescription",dto.getJeTrxDescription());
        map.put("amountValue",dto.getAmountValue());
        map.put("contractNumberFrom",dto.getContractNumberFrom());
        map.put("contractNumberTo",dto.getContractNumberTo());
        map.put("jeStatusDescription",dto.getJeStatusDescription());
        map.put("jeHeadId",dto.getJeHeadId());
        map.put("jeNumber",dto.getJeNumber());
        String[] paramStatus = null;
        String[] paramJeTrx = null;
        String[] paramPeriodName = null;
        if(dto.getJeStatus()!=null){
            paramStatus=dto.getJeStatus().split("、");
        }
        if(dto.getJeTrx()!=null){
            paramJeTrx=dto.getJeTrx().split("、");
        }
        if(dto.getPeriodName()!=null){
            paramPeriodName=dto.getPeriodName().split("、");
        }
        map.put("paramStatus",paramStatus);
        map.put("paramJeTrx",paramJeTrx);
        map.put("paramPeriodName",paramPeriodName);
        return new ResponseData(service.jeListQuery(map,page,pageSize));
    }

    /*进行凭证确认*/
    @RequestMapping(value = "/gld/je/head/jeConfirm")
    @ResponseBody
    public ResponseData jeConfirm(@RequestBody List<JeHead> list, HttpServletRequest request, HttpSession session) {
        IRequest requestCtx = createRequestContext(request);
        Long userId = (Long) session.getAttribute("userId");
        Long companyId = (Long) session.getAttribute("companyId");
        ResponseData dt = new ResponseData();
        List<JeLine> jeLines = null;
        try {
            jeLines = service.jeConfirm(list, requestCtx, companyId, userId);
        }catch (IllegalArgumentException e){
            dt.setMessage(e.getMessage());
            dt.setSuccess(false);
            return dt;
        }
        dt.setRows(jeLines);
        dt.setTotal(Long.valueOf(jeLines!=null?jeLines.size():0));
        dt.setMessage("确认成功");
        dt.setSuccess(true);
        return dt;
    }
}