package hls.core.gld.controllers;

import com.mysql.jdbc.exceptions.MySQLIntegrityConstraintViolationException;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.gld.dto.JeRuleEngine;
import hls.core.gld.service.IJeRuleEngineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.sql.SQLException;
import java.util.List;

@Controller
public class JeRuleEngineController extends BaseController {

    @Autowired
    private IJeRuleEngineService service;


    @RequestMapping(value = "/gld/je/rule/engine/query")
    @ResponseBody
    public ResponseData query(JeRuleEngine dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        dto.setCompanyId(companyId);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/gld/je/rule/engine/submit")
    @ResponseBody
    public ResponseData update(HttpServletRequest request, @RequestBody List<JeRuleEngine> dto, BindingResult result, HttpSession session) {
        IRequest requestCtx = createRequestContext(request);
        ResponseData rd;
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            rd = new ResponseData(false);
            rd.setMessage(getErrorMessage(result, request));
            return rd;
        }
        Long companyId = (Long) session.getAttribute("companyId");
        dto.forEach(item -> {
            item.setCompanyId(companyId);
        });
        try {
            List<JeRuleEngine> jeRuleEngines = service.batchUpdate(requestCtx, dto);
            rd = new ResponseData(jeRuleEngines);
        } catch (Exception e) {
            rd = new ResponseData(false);
            rd.setMessage("同一凭证事务只能创建一条数据");
        }
        return rd;
    }

    @RequestMapping(value = "/gld/je/rule/engine/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<JeRuleEngine> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/gld/je/ruleEngine/lovData")
    @ResponseBody
    public ResponseData jeRuleEngineerLovData(JeRuleEngine jeRuleEngine, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HttpSession session){
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        jeRuleEngine.setCompanyId(companyId);
        return new ResponseData(service.jeRuleEngineerLovData(requestContext,jeRuleEngine,page,pageSize));
    }
}