package hls.core.gld.job;

import com.hand.hap.core.IRequest;
import hls.core.gld.dto.ContractTrx;
import hls.core.gld.mapper.ContractTrxMapper;
import hls.core.gld.service.IContractFinanceIncomeService;
import hls.core.gld.service.IContractTrxService;
import hls.core.job.AbstractJobWithIRequest;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.quartz.PersistJobDataAfterExecution;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * Created by fjm on 2017/8/24.
 */
@DisallowConcurrentExecution
@PersistJobDataAfterExecution
public class ContractFinacnceIncomeJob extends AbstractJobWithIRequest {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private Exception exception = null;

    @Autowired
    private ContractTrxMapper contractTrxMapper;
    @Autowired
    private IContractFinanceIncomeService contractFinanceIncomeService;

    @Override
    public void safeExecuteWithIRequest(JobExecutionContext jobExecutionContext, IRequest iRequest) {
        ContractTrx contractTrx = new ContractTrx();
        //contractTrx.setStatus("NEW");
        List<ContractTrx> contractTrxes = contractTrxMapper.selectNewOrError(contractTrx);
        try {
            contractTrxes.forEach(item->{
                    contractFinanceIncomeService.createGldContract(iRequest,item.getContractId());
            });
        }catch (Exception e){
            exception = e;
            logger.error(e.getMessage());
        }

        if (exception != null) {
            setExecutionSummary(exception.getClass().getName() + ":" + exception.getMessage());
        } else {
            setExecutionSummary("success");
        }
    }

    /**
     * 任务发生异常的时候挂起
     *
     * @return
     */
    @Override
    protected boolean isRefireImmediatelyWhenException() {
        return false;
    }

}
