package hls.core.gld.job;

import com.hand.hap.core.IRequest;
import hls.core.gld.dto.JeTrxDtl;
import hls.core.gld.service.IJeTrxDtlService;
import hls.core.job.AbstractJobWithIRequest;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.quartz.PersistJobDataAfterExecution;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;

/**
 * DisallowConcurrentExecution该job不允许并发
 * Created by fjm on 2017/8/17.
 */
@DisallowConcurrentExecution
@PersistJobDataAfterExecution
public class JeTrxLineJob extends AbstractJobWithIRequest {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    private Exception exception = null;

    @Autowired
    private IJeTrxDtlService jeTrxDtlService;

    @Override
    public void safeExecuteWithIRequest(JobExecutionContext jobExecutionContext, IRequest iRequest) {
        List<JeTrxDtl> jeTrxDtls = jeTrxDtlService.queryJeTrxDtlStatus();
        final List<Long> ids = new ArrayList<>();
        try {
            jeTrxDtls.forEach(item->ids.add(item.getJeTrxDtlId()));
            jeTrxDtlService.createJeLine(iRequest,ids);
        } catch (Exception e) {
            exception = e;
            logger.error(e.getMessage(), e);
            throw e;
        }
        if (exception != null) {
            setExecutionSummary(exception.getClass().getName() + ":" + exception.getMessage());
        } else {
            setExecutionSummary("success");
        }
    }

    /**
     * 任务发生异常的时候挂起
     *
     * @return
     */
    @Override
    protected boolean isRefireImmediatelyWhenException() {
        return false;
    }


}
