package hls.core.gld.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.gld.dto.Account;
import hls.core.gld.dto.AccountHierarchy;
import hls.core.gld.dto.AccountHierarchyDtl;
import hls.core.gld.mapper.AccountHierarchyDtlMapper;
import hls.core.gld.mapper.AccountHierarchyMapper;
import hls.core.gld.mapper.AccountMapper;
import hls.core.gld.service.IAccountHierarchyDtlService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.gld.dto.AccountSet;
import hls.core.gld.service.IAccountSetService;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class AccountSetServiceImpl extends BaseServiceImpl<AccountSet> implements IAccountSetService{

    @Autowired
    private AccountMapper accountMapper;

    @Autowired
    private AccountHierarchyMapper accountHierarchyMapper;

    @Autowired
    private AccountHierarchyDtlMapper accountHierarchyDtlMapper;

    @Autowired
    private IAccountHierarchyDtlService dtlService;

    @Override
    public int batchDelete(List<AccountSet> list) {
        list.forEach(accountItem->{
            Account account = new Account();
            account.setAccountSetId(accountItem.getAccountSetId());

            List<Account> accountList = accountMapper.select(account);

            accountList.forEach(item->{
                AccountHierarchyDtl accountHierarchyDtl = new AccountHierarchyDtl();
                accountHierarchyDtl.setParentAccountId(item.getAccountId());
                accountHierarchyDtlMapper.delete(accountHierarchyDtl);
                AccountHierarchy accountHierarchy = new AccountHierarchy();
                accountHierarchy.setAccountId(item.getAccountId());
                accountHierarchyMapper.delete(accountHierarchy);
            });

            accountMapper.delete(account);
        });
        return super.batchDelete(list);
    }

    /**
     * 生成树的关系
     * @param accountSet
     * @param request
     */
    @Override
    public void createRelation(AccountSet accountSet, IRequest request) {
        Account account = new Account();
        account.setAccountSetId(accountSet.getAccountSetId());
        account.setEnabledFlag("Y");
        account.setSummaryFlag("Y");
        List<Account> list = accountMapper.select(account);//先找出科目明细表中,启用且为汇总的,非汇总不可能作为父层,所以不考虑
        list.forEach(accountItem->{
            AccountHierarchy accountHierarchy = new AccountHierarchy();
            accountHierarchy.setAccountId(accountItem.getAccountId());
            List<AccountHierarchy> accountHierarchyList = accountHierarchyMapper.select(accountHierarchy);//科目下的所有科目层次
            accountHierarchyList.forEach(accountHierarchyItem->{
                accountHierarchyItem.setAccountSetId(accountSet.getAccountSetId());
                List<Account> childList = accountHierarchyMapper.selectChildAccount(accountHierarchyItem);//找出符合匹配规则的子集合,要先排除自身
                childList.forEach(item->{
                    AccountHierarchyDtl accountHierarchyDtl = new AccountHierarchyDtl();
                    accountHierarchyDtl.setAccountId(item.getAccountId());
                    accountHierarchyDtl.setParentAccountId(accountHierarchyItem.getAccountId());
                    if (!exits(accountHierarchyDtl))
                        dtlService.self().insert(request,accountHierarchyDtl);
                });
            });
        });
    }

    private boolean exits(AccountHierarchyDtl dtl){
        List list = accountHierarchyDtlMapper.select(dtl);
        if (list.size()>0)
            return true;
        else
            return false;
    }
}