package hls.core.gld.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.gld.dto.*;
import hls.core.gld.mapper.*;
import hls.core.gld.service.*;
import hls.core.gld.utils.IrrUtil;
import hls.core.hls.components.CalculateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Transactional    /* 此类中所有自行处理的异常都为IllegalArgumentException */
public class ContractFinanceIncomeServiceImpl extends BaseServiceImpl<ContractFinanceIncome> implements IContractFinanceIncomeService {

    protected final static Long CF_ITEM = new Long(1);
    /*支付频率*/
    protected final static int PAY_TIMES_YEAR = 1;
    protected final static int PAY_TIMES_HALF_YEAR = 2;
    protected final static int PAY_TIMES_SEASON = 4;
    protected final static int PAY_TIMES_MONTH = 12;
    protected final static String CLOSE_STATUS = "CLOSE";
    public static final String RECEIVABLE_INTEREST = "RECEIVABLE_INTEREST";//应收利息
    public static final String REAL_INTEREST_RATE = "REAL_INTEREST_RATE";//实率法分摊
    public static final String ONGOING = "ONGOING";

    @Autowired
    private HlsCusConContractMapper conContractMapper;
    @Autowired
    private HlsCusConContractCashflowMapper conContractCashflowMapper;
    @Autowired
    private FinancialAttributeMapper financialAttributeMapper;
    @Autowired
    private SetOfBooksMapper setOfBooksMapper;
    @Autowired
    private PeriodMapper periodMapper;
    @Autowired
    private ContractFinanceIncomeMapper mapper;
    @Autowired
    private ContractHdMapper contractHdMapper;
    @Autowired
    private IContractHdService contractHdService;
    @Autowired
    private ContractCashflowMapper contractCashflowMapper;
    @Autowired
    private IContractCashflowService contractCashflowService;
    @Autowired
    private PeriodStatusMapper periodStatusMapper;
    @Autowired
    private ContractTrxMapper contractTrxMapper;
    @Autowired
    private IContractTrxService contractTrxService;
    @Autowired
    private IJeTrxDtlService jeTrxDtlService;
    @Autowired
    private JeTrxCommonService jeTrxCommonService;

    private Calendar calendar = Calendar.getInstance();//获取一个日历的实例


    public void createGldContract(IRequest request, Long contractId) {

        HlsCusConContract contract = conContractMapper.selectByPrimaryKey(contractId);
        Long companyId = contract.getCompanyId();
        ContractHd contractHd = contractHdMapper.selectByPrimaryKey(contractId);
        ContractTrx contractTrx = new ContractTrx();
        contractTrx.setCompanyId(companyId);
        contractTrx.setContractId(contractId);
        contractTrx = contractTrxMapper.selectOne(contractTrx);

        try {
            List<Map> result = this.calculate(contractId, companyId);//现根据合同id计算
            if (contractHd == null || contractHd.getStatus() == null) {
                contractHd = new ContractHd();
                contractHd.setCompanyId(companyId);
                contractHd.setContractId(contractId);
                contractHd.setStatus(ONGOING);
                contractHdService.self().insertSelective(request, contractHd);

                //如果租赁合同收入头表是ongoing状态,则会处理result,否则直接把计算到的result插入到租赁合同现金流表
                result.forEach(item -> {
                    ContractCashflow contractCashflow = new ContractCashflow();
                    contractCashflow.setContractId(contractId);
                    contractCashflow.setCashflowId((Long) item.get("cashflowId"));
                    contractCashflow.setTimes((Long) item.get("times"));
                    contractCashflow.setFinIncomeDate((Date) item.get("dueDate"));
                    contractCashflow.setFinanceIncome((Double) item.get("erInterest"));
                    contractCashflowService.self().insertSelective(request, contractCashflow);
                });
            } else {
                if (ONGOING.equalsIgnoreCase(contractHd.getStatus())) {//Ongoing状态处理,其他状态直接返回
                    List<ContractCashflow> confirm = new ArrayList<>();
                    FinancialAttribute financialAttribute = new FinancialAttribute();
                    financialAttribute.setCompanyId(companyId);
                    financialAttribute = financialAttributeMapper.selectOne(financialAttribute);//根据合同上的公司id找到对应的公司财务属性
                    Long setOfBooksId = financialAttribute.getSetOfBooksId();
                    SetOfBooks setOfBooks = setOfBooksMapper.selectByPrimaryKey(setOfBooksId);//根据公司财务属性上对应的账套id找到对应的账套
                    Long periodSetId = setOfBooks.getPeriodSetId();

                    for (int i = 0; i < result.size(); i++) {
                        String periodName = null;
                        Map item = result.get(i);
                        ContractFinanceIncome income = new ContractFinanceIncome();
                        ContractCashflow cashflow = new ContractCashflow();
                        cashflow.setContractId(contractId);
                        cashflow.setTimes((Long) item.get("times"));
                        cashflow.setCfItem((String) item.get("cfItem"));
                        cashflow = contractCashflowMapper.selectOne(cashflow);
                        income.setGldCashflowId(cashflow.getGldCashflowId());

                        List<ContractFinanceIncome> select = mapper.select(income);
                        List<ContractFinanceIncome> incomes = mapper.selectPostFlag(cashflow.getGldCashflowId());
                        if (incomes != null && incomes.size() > 0) {
                            ContractFinanceIncome last = incomes.get(incomes.size() - 1);
                            Calendar c = Calendar.getInstance();
                            c.setTime(last.getEndDate());
                            c.add(Calendar.DATE, 1);
                            ContractCashflow contractCashflow = contractCashflowMapper.selectByPrimaryKey(cashflow.getGldCashflowId());
                            Date finIncomeDate = contractCashflow.getFinIncomeDate();
                            Period p = new Period();
                            p.setPeriodSetId(periodSetId);
                            List<Period> periods = periodMapper.select(p);
                            //找到finIncomeDate对应的区间
                            for (Period period : periods) {
                                if (period.getEndDate().getTime() >= finIncomeDate.getTime() && period.getStartDate().getTime() <= finIncomeDate.getTime()) {
                                    periodName = period.getPeriodName();
                                    break;
                                }

                            }

                            PeriodStatus periodStatus = new PeriodStatus();
                            periodStatus.setCompanyId(companyId);
                            periodStatus.setPeriodName(periodName);

                            try {
                                periodStatus = periodStatusMapper.selectOne(periodStatus);
                            } catch (Exception e) {
                                throw new IllegalArgumentException("periodName:" + periodName + "对应凭证状态取值出错,请核对记录");
                            }

                            //完全确认且期间关闭
                            if (select.size() == incomes.size() && c.getTime().getTime() == ((Date) item.get("dueDate")).getTime() && "CLOSE".equalsIgnoreCase(periodStatus.getStatus())) {
                                confirm.add(contractCashflow);
                            }
                        }
                    }
                    this.reCalculate(confirm, result);//重新分摊之后,result中的值会变化
                } else {
                    return;
                }
                //如果租赁合同收入头表是ongoing状态,则会处理result,否则直接把计算到的result插入到租赁合同现金流表
                result.forEach(item -> {
                    ContractCashflow contractCashflow = new ContractCashflow();
                    contractCashflow.setContractId(contractId);
                    contractCashflow.setCashflowId((Long) item.get("cashflowId"));
                    contractCashflow.setTimes((Long) item.get("times"));
                    contractCashflow.setFinIncomeDate((Date) item.get("dueDate"));
                    contractCashflow.setFinanceIncome((Double) item.get("erInterest"));
                    contractCashflowService.self().updateByPrimaryKeySelective(request, contractCashflow);
                });
            }
            //把租息收入分摊计算的结果插入租赁合同收入分摊表
            try {
                this.contractFinanceIncome(request, contractId);
            } catch (IllegalArgumentException e) {
                throw e;
            }
            contractTrx.setMsgLog("计算完成");
            contractTrx.setStatus("POST");
        } catch (Exception e) {
            if (e instanceof IllegalArgumentException) {
                contractTrx.setMsgLog(e.getMessage());
                contractTrx.setStatus("ERROR");
            } else {
                throw e;
            }
        }
        contractTrxService.updateByPrimaryKeySelective(request, contractTrx);
    }


    /**
     * 重新计算租赁合同现金流表上的值
     *
     * @param confirm
     * @param result
     */
    private void reCalculate(List<ContractCashflow> confirm, List<Map> result) {
        if (confirm == null || confirm.size() == 0) {
            return;
        }
        Double subAmount = 0D;//原表上的ER利息与调息后的差值
        for (int i = 0; i < confirm.size(); i++) {
            Double erInterest = null;
            ContractCashflow item = confirm.get(i);
            Double financeIncome = item.getFinanceIncome() != null ? item.getFinanceIncome() : 0D;

            Iterator<Map> iterator = result.iterator();
            while (iterator.hasNext()) {
                Map next = iterator.next();
                if (item.getTimes().compareTo((Long) next.get("times")) == 0 && item.getCfItem().toString().equalsIgnoreCase((String) next.get("cfItem")) && item.getContractId().compareTo((Long) next.get("contractId")) == 0) {
                    erInterest = (Double) next.get("erInterest");
                    iterator.remove();
                    break;
                }
            }

            subAmount = CalculateUtil.add(subAmount, CalculateUtil.sub(erInterest, financeIncome));
        }

        Double totalAmount = 0D;

        for (Map map : result) {
            totalAmount = CalculateUtil.add((Double) map.get("erInterest"), totalAmount);
        }

        Double medium = 0D;//中间变量,用于记录除最后一次之外的每一次重算金额的总和
        for (int i = 0; i < result.size(); i++) {
            Map map = result.get(i);
            Double erInterest = (Double) map.get("erInterest");
            if (i == result.size() - 1) {
                erInterest = CalculateUtil.add(erInterest, CalculateUtil.sub(subAmount, medium));
            } else {
                Double percent = CalculateUtil.div(erInterest, totalAmount, 6);
                Double changeAmount = CalculateUtil.mul(subAmount, percent);
                medium = CalculateUtil.add(medium, changeAmount);
                erInterest = CalculateUtil.add(erInterest, changeAmount);
            }
            map.put("erInterest", erInterest);
        }

    }

    /**
     * 租赁合同现金流重算
     *
     * @param contractId
     * @param companyId
     */
    private List<Map> calculate(Long contractId, Long companyId) {
        final List<Map> total = new ArrayList<>();
        ContractFinanceIncome income = new ContractFinanceIncome();
        HlsCusConContract contract = new HlsCusConContract();
        HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
        FinancialAttribute financialAttribute = financialAttributeMapper.selectByPrimaryKey(companyId);
        String conIncomeRule = financialAttribute.getConIncomeRule();//租息分摊规则

        if (conIncomeRule == null)
            throw new IllegalArgumentException("请在公司属性中,设置财务分摊规则");

        income.setContractId(contractId);
        contract.setContractId(contractId);
        cashflow.setCf_item(CF_ITEM);
        cashflow.setContract_id(contractId);
        cashflow.setSortname("times");
        cashflow.setSortorder("asc");
        contract = conContractMapper.selectOne(contract);
        List<HlsCusConContractCashflow> cashflowList = conContractCashflowMapper.select(cashflow);

        if (cashflowList == null || cashflowList.size() == 0) {
            throw new IllegalArgumentException("未找到对应的现金流");
        }

        if (RECEIVABLE_INTEREST.equalsIgnoreCase(conIncomeRule)) {//按应收利息
            cashflowList.forEach(item -> {
                Map map = new HashMap<>();
                map.put("erInterest", item.getInterest());
                map.put("times", item.getTimes());
                map.put("cashflowId", item.getCashflow_id());
                map.put("dueDate", item.getDue_date());
                map.put("cfItem", item.getCf_item());
                map.put("contractId", contractId);
                total.add(map);
            });
        } else if (REAL_INTEREST_RATE.equalsIgnoreCase(conIncomeRule)) {//按实率分摊
            Long payTimes = contract.getAnnualPayTimes();
            Double finaceAmount = contract.getFinanceAmount();
            Double vatRateParam = CalculateUtil.add(contract.getVatRate() != null ? contract.getVatRate() : 0D, 1D);
            final List<Double> irrArray = new ArrayList<>();//数组的第0个元素一定是-finaceAmount
            irrArray.add(CalculateUtil.sub(0D, finaceAmount));
            cashflowList.forEach(item -> {
                irrArray.add(item.getDue_amount());
            });
            Double irr = IrrUtil.irr(irrArray);//irr利率
            irr = CalculateUtil.mul(irr, payTimes.doubleValue());//乘以支付频率
            Double financeAmountCopy = new Double(finaceAmount.doubleValue());
            for (int i = 0; i < cashflowList.size(); i++) {
                HlsCusConContractCashflow item = cashflowList.get(i);
                Map map = new HashMap<>();
                Double erInterest = null;
                Double erPrincipal = null;
                Double interestBalance = null;
                Long times = item.getTimes();
                Long cashflowId = item.getCashflow_id();
                if (i + 1 == cashflowList.size()) {
                    erInterest = CalculateUtil.div(CalculateUtil.div(CalculateUtil.mul(irr, financeAmountCopy), payTimes.doubleValue(), 6), vatRateParam, 6);//ER利息,除税
                    if (cashflowList.size() == 1)
                        erPrincipal = finaceAmount;
                    else
                        erPrincipal = (Double) total.get(i - 1).get("interestBalance");//最后一期的ER本金取上一期的计息余额
                    interestBalance = 0D;//最后一期计息余额为0
                    financeAmountCopy = interestBalance;
                } else {
                    erInterest = CalculateUtil.div(CalculateUtil.div(CalculateUtil.mul(irr, financeAmountCopy), payTimes.doubleValue(), 6), vatRateParam, 6);//ER利息,除税
                    erPrincipal = CalculateUtil.sub(item.getDue_amount(), erInterest);//ER本金
                    interestBalance = CalculateUtil.sub(financeAmountCopy, erPrincipal);//计息余额
                    financeAmountCopy = interestBalance;
                }
                map.put("erInterest", erInterest);
                map.put("erPrincipal", erPrincipal);
                map.put("interestBalance", interestBalance);
                map.put("times", times);
                map.put("cfItem", cashflowList.get(i).getCf_item());
                map.put("cashflowId", cashflowId);
                map.put("dueDate", item.getDue_date());
                map.put("contractId", contractId);
                total.add(map);
            }
        }
        return total;
    }

    /**
     * 租息收入分摊计算
     *
     * @param request
     * @param contractId
     */
    @Override
    public void contractFinanceIncome(IRequest request, Long contractId) {
        HlsCusConContract contract = new HlsCusConContract();
        ContractCashflow cashflow = new ContractCashflow();
        FinancialAttribute financialAttribute = new FinancialAttribute();
        contract.setContractId(contractId);
        contract = conContractMapper.selectOne(contract);//获取对应的合同的数据

        Long companyId = contract.getCompanyId();//公司ID

        cashflow.setContractId(contractId);

        List<ContractCashflow> contractCashflowList = contractCashflowMapper.select(cashflow);//合同对应的每一期现金流

        financialAttribute.setCompanyId(companyId);
        financialAttribute = financialAttributeMapper.selectOne(financialAttribute);//根据合同上的公司id找到对应的公司财务属性
        Long setOfBooksId = financialAttribute.getSetOfBooksId();

        SetOfBooks setOfBooks = setOfBooksMapper.selectByPrimaryKey(setOfBooksId);//根据公司财务属性上对应的账套id找到对应的账套
        Long periodSetId = setOfBooks.getPeriodSetId();

        Date inceptionOfLease = contract.getInceptionOfLease();//起租日期

        if (inceptionOfLease == null)
            throw new IllegalArgumentException("合同" + contractId + "inceptionOfLease字段为空");

        Calendar start = (Calendar) calendar.clone();
        Calendar end = (Calendar) calendar.clone();

        contractCashflowList.forEach(item -> {
            mapper.deleteByFlag(item.getGldCashflowId());//删除未确认的数据
        });

        //先以现金流分组,再以期间分组
        for (int i = 0; i < contractCashflowList.size(); i++) {
            ContractCashflow contractCashflow = contractCashflowList.get(i);
            Date from;//日期从
            Date to;//日期到
            List<ContractFinanceIncome> list = new ArrayList<>();//每次循环都新建一个list用于保存本条现金流生成的incomes
            list.clear();
            //第一条的开始日期去值为inceptionOfLease,之后的取值为上一条的dueDate
            if (i == 0) {
                start.clear();
                end.clear();
                from = inceptionOfLease;
                to = contractCashflow.getFinIncomeDate();
                start.setTime(from);//起始日期为起租日期
                end.setTime(to);
                end.add(Calendar.DATE, -1);//结束日期为dueDate的前一天
            } else {
                start.clear();
                end.clear();
                from = contractCashflowList.get(i - 1).getFinIncomeDate();
                to = contractCashflow.getFinIncomeDate();
                start.setTime(from);
                end.setTime(to);
                end.add(Calendar.DATE, -1);//结束日期为dueDate的前一天
            }

            getInCome(list, start, end, periodSetId, companyId, contractId, contractCashflow);

            list.forEach(item -> {
                self().insertSelective(request, item);
            });


        }

    }

    /**
     * 收入确认
     *
     * @param request
     * @param collection
     * @return
     */
    @Override
    public List<ContractFinanceIncome> post(IRequest request, List<ContractFinanceIncome> collection) {
        AbstractJeTrxService finIncomeRecognitionTrxService = jeTrxCommonService.map.get("FIN_INCOME_RECOGNITION");
        collection.forEach(item -> {
            Long financeIncomeId = item.getFinanceIncomeId();
            ContractFinanceIncome income = mapper.selectByPrimaryKey(financeIncomeId);
            Date startDate = income.getStartDate();
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(startDate);
            int year = calendar.get(GregorianCalendar.YEAR);
            int month = calendar.get(GregorianCalendar.MONTH) + 1;
            String periodName;
            if (month < 10){
                periodName = year + "-0" + month;
            }else {
                periodName = year + "-" +month;
            }
            Long companyId = income.getCompanyId();
            PeriodStatus status = new PeriodStatus();
            status.setCompanyId(companyId);
            status.setPeriodName(periodName);
            status = periodStatusMapper.selectOne(status);
            if (status == null || CLOSE_STATUS.equalsIgnoreCase(status.getStatus()))
                return;
            self().updateByPrimaryKeySelective(request, item);
            //插入凭证事物流水表 by fjm 17.9.1
            Map transactionParams = new HashMap<>();
            transactionParams.put("jeTrxId", item.getFinanceIncomeId());
            transactionParams.put("companyId", item.getCompanyId());
            transactionParams.put("contractId", item.getContractId());
            transactionParams.put("sourceDoc", "CON_CONTRACT");
            finIncomeRecognitionTrxService.process(request, transactionParams);

        });
        return collection;
    }

    /**
     * 获取生成income的明细
     *
     * @param start
     * @param end
     * @param periodSetId
     * @return
     */
    private void getInCome(List<ContractFinanceIncome> list, Calendar start, Calendar end, Long periodSetId, Long companyId, Long contractId, ContractCashflow cashflow) {

        List<ContractFinanceIncome> incomes = mapper.selectPostFlag(cashflow.getGldCashflowId());//找出已经确认的数据集合
        if (incomes != null && incomes.size() > 0) {//当存在已确认的数据时
            ContractFinanceIncome incomeItem = incomes.get(incomes.size() - 1);
            Date newStartDate = incomeItem.getEndDate();
            start.clear();
            start.setTime(newStartDate);
            start.add(Calendar.DATE, 1);//取上一个结束日期的后一天
        }
        periodGrouping(list, start, end, periodSetId, companyId, contractId, cashflow);//根据期间分组
        calculateGrouping(list, cashflow, incomes, companyId);//计算分组中的金额
    }


    /**
     * 把现金流中的记录根据期间表中的期间分组
     *
     * @param list
     * @param start
     * @param end
     * @param periodSetId
     * @param companyId
     * @param contractId
     * @param cashflow
     */
    private void periodGrouping(List<ContractFinanceIncome> list, Calendar start, Calendar end, Long periodSetId, Long companyId, Long contractId, ContractCashflow cashflow) {

        Map paraMap = new HashMap<>();
        //从Calendar取月份的值必须要加一
        int startMonth = start.get(Calendar.MONTH) + 1;
        int startYear = start.get(Calendar.YEAR);
        int endMonth = end.get(Calendar.MONTH) + 1;
        int endYear = end.get(Calendar.YEAR);

        paraMap.put("periodSetId", periodSetId);
        paraMap.put("startMonth", startMonth);
        paraMap.put("startYear", startYear);
        paraMap.put("endMonth", endMonth);
        paraMap.put("endYear", endYear);

        List<Period> periodBetween = periodMapper.getPeriodBetween(paraMap);//获取现金流所包含的区间
        for (Period item : periodBetween) {
            ContractFinanceIncome income = new ContractFinanceIncome();
            income.setCompanyId(companyId);
            income.setContractId(contractId);
            income.setGldCashflowId(cashflow.getGldCashflowId());
            income.setPeriodName(item.getPeriodName());
            if (startYear == item.getPeriodYear() && startMonth == item.getPeriodMonth()) {
                income.setEndDate(item.getEndDate());
                income.setStartDate(start.getTime());
            } else if (endYear == item.getPeriodYear() && endMonth == item.getPeriodMonth()) {
                income.setEndDate(end.getTime());
                income.setStartDate(item.getStartDate());
            } else {
                income.setEndDate(item.getEndDate());
                income.setStartDate(item.getStartDate());
            }
            income.setDays(Long.valueOf(compareDays(income.getStartDate(), income.getEndDate())));
            list.add(income);
        }
    }


    /**
     * 计算每组数据的分摊金额
     *
     * @param list
     * @param contractCashflow
     * @param incomes
     * @param companyId
     */
    private void calculateGrouping(List<ContractFinanceIncome> list, ContractCashflow contractCashflow, List<ContractFinanceIncome> incomes, Long companyId) {

        long allDays = 0;
        Double total = 0D;
        Double amount = contractCashflow.getFinanceIncome();//总税后利息
        Double postAmount = 0D;//确认过的金额
        for (ContractFinanceIncome child : incomes) {
            postAmount = CalculateUtil.add(postAmount, child.getFinanceIncome());
        }


        Calendar c = Calendar.getInstance();
        c.setTime(contractCashflow.getFinIncomeDate());
        c.add(Calendar.DATE, -1);
        if (incomes.size() > 0 && incomes.get(incomes.size() - 1).getEndDate().getTime() >= c.getTime().getTime()) {
            list.clear();
            if (amount.compareTo(postAmount) != 0) {
                ContractFinanceIncome income = new ContractFinanceIncome();
                income.setCompanyId(companyId);
                income.setContractId(contractCashflow.getContractId());
                income.setGldCashflowId(contractCashflow.getGldCashflowId());
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(contractCashflow.getFinIncomeDate());
                calendar.add(Calendar.DATE, -1);//取dueDate的前一天
                income.setStartDate(calendar.getTime());
                income.setEndDate(calendar.getTime());
                income.setDays(1L);
                income.setPeriodName(incomes.get(incomes.size() - 1).getPeriodName());
                income.setFinanceIncome(CalculateUtil.sub(amount, postAmount));
                list.add(income);
                return;
            }
        }

        amount = CalculateUtil.sub(amount, postAmount);//减去确认过的金额,才是要拆分的金额
        for (ContractFinanceIncome child : list) {
            allDays += child.getDays();
        }

        for (int i = 0; i < list.size(); i++) {
            ContractFinanceIncome item = list.get(i);
            Double percent = CalculateUtil.div(Double.valueOf(item.getDays().doubleValue()), Double.valueOf(allDays), 6);//所占权重
            if (i + 1 == list.size()) {
                Double incomeAmount = CalculateUtil.sub(amount, total);
                item.setFinanceIncome(incomeAmount);
            } else {
                Double incomeAmount = CalculateUtil.mul(amount, percent);
                total = CalculateUtil.add(total, incomeAmount);
                item.setFinanceIncome(incomeAmount);
            }
        }


    }

    /**
     * 比较日期相差的天数
     *
     * @param start
     * @param from
     * @return
     */
    private static int compareDays(Date start, Date from) {
        if (null == start || null == from) {

            return -1;

        }

        long intervalMilli = Math.abs(start.getTime() - from.getTime());//取绝对值

        return (int) (intervalMilli / (24 * 60 * 60 * 1000)) + 1;
    }


    /*财务首页计提清单查询*/
    public List<ContractFinanceIncome> queryList(ContractFinanceIncome dto) {
        List<ContractFinanceIncome> incomes = mapper.queryList(dto);
        //incomes.forEach(item->item.setInterest(CalculateUtil.div(item.getInterest()!=null?item.getInterest():0D,CalculateUtil.add(item.getVatRate()!=null?item.getVatRate():0D,1D))));
        return incomes;
    }

}