package hls.core.gld.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.DTOStatus;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.gld.dto.*;
import hls.core.gld.mapper.*;
import hls.core.gld.service.IGldLonContractFinCostService;
import hls.core.gld.service.IGldLonContractHdService;
import hls.core.gld.service.IGldLonContractRepaymentService;
import hls.core.gld.service.IGldLonContractTrxService;
import hls.core.hls.components.CalculateUtil;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.dto.HlsCusLonContractWithdraw;
import hls.core.lon.mapper.HlsCusLonContractRepaymentMapper;
import hls.core.lon.mapper.HlsCusLonContractWithdrawMapper;
import hls.core.lon.utils.LocalDateUtils;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.InvocationTargetException;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service
@Transactional(rollbackFor = Exception.class)
public class GldLonContractFinCostServiceImpl extends BaseServiceImpl<HlsCusGldLonContractFinCost> implements IGldLonContractFinCostService {

    protected static final String CLOSED = "CLOSED";
    protected static final String ONGOING = "ONGOING";
    protected static final String LOAN_INCEPT = "LOAN_INCEPT";
    protected static final String LOAN_CHANGE = "LOAN_CHANGE";
    protected static final String POST_FLAG_N = "N";
    protected static final String POST_FLAG_Y = "Y";
    protected static final String POST = "POST";
    protected static final String NEW = "NEW";
    @Autowired
    private HlsCusLonContractRepaymentMapper lonContractRepaymentMapper;
    @Autowired
    private IGldLonContractRepaymentService gldLonContractRepaymentService;
    @Autowired
    private HlsCusGldLonContractRepaymentMapper gldLonContractRepaymentMapper;
    @Autowired
    private HlsCusLonContractWithdrawMapper lonContractWithdrawMapper;
    @Autowired
    private HlsCusGldLonContractFinCostMapper gldLonContractFinCostMapper;
    @Autowired
    private HlsCusGldLonContractTrxMapper gldLonContractTrxMapper;
    @Autowired
    private IGldLonContractTrxService gldLonContractTrxService;
    @Autowired
    private HlsCusGldLonContractHdMapper gldLonContractHdMapper;
    @Autowired
    private IGldLonContractHdService gldLonContractHdService;
    @Autowired
    private FinancialAttributeMapper financialAttributeMapper;
    @Autowired
    private SetOfBooksMapper setOfBooksMapper;
    @Autowired
    private PeriodMapper periodMapper;

    /**
     * @param request
     * @param trxId
     */
    @Override
    public void createLonContractFinCost(IRequest request, List<Long> trxId) {
        HlsCusLonContractRepayment lonContractRepayment = new HlsCusLonContractRepayment();
        trxId.forEach(item -> {
            HlsCusGldLonContractTrx gldLonContractTrx = gldLonContractTrxMapper.selectByPrimaryKey(item);
            if (gldLonContractTrx == null || POST.equalsIgnoreCase(gldLonContractTrx.getStatus()))//状态为POST的记录直接跳过不处理
                return;
            if (LOAN_CHANGE.equalsIgnoreCase(gldLonContractTrx.getSourceType())) {
                HlsCusGldLonContractTrx param = new HlsCusGldLonContractTrx();
                param.setCompanyId(gldLonContractTrx.getCompanyId());
                param.setContractId(gldLonContractTrx.getContractId());
                param.setSourceType(LOAN_INCEPT);
                param.setStatus(NEW);
                param.setSourceId(gldLonContractTrx.getSourceId());
                gldLonContractTrxMapper.delete(param);
            }
            Long sourceId = gldLonContractTrx.getSourceId();
            HlsCusGldLonContractHd gldLonContractHd = gldLonContractHdMapper.selectByPrimaryKey(sourceId);
            Long companyId = gldLonContractTrx.getCompanyId();
            String sourceType = gldLonContractTrx.getSourceType();
            lonContractRepayment.setWithdrawId(sourceId);
            lonContractRepayment.setCfItem(302L);//融资还款利息
            List<HlsCusLonContractRepayment> repayments = lonContractRepaymentMapper.select(lonContractRepayment);
            HlsCusLonContractRepayment repayment = repayments.stream().findAny().orElse(new HlsCusLonContractRepayment());
            Long contractId = repayment.getContractId();
            if (contractId == null)//当合同id为空时,直接return
                return;
            final List<HlsCusGldLonContractRepayment> gldLonContractRepayments = new ArrayList<>();
            if (gldLonContractHd == null || gldLonContractHd.getStatus() == null) {
                gldLonContractHd = new HlsCusGldLonContractHd();
                gldLonContractHd.setWithdrawId(sourceId);
                gldLonContractHd.setContractId(gldLonContractTrx.getContractId());
                gldLonContractHd.setCompanyId(companyId);
                gldLonContractHd.setStatus(ONGOING);
                gldLonContractHd.set__status(DTOStatus.ADD);
                gldLonContractHdService.insertSelective(request, gldLonContractHd);
                repayments.forEach(element -> {
                    HlsCusGldLonContractRepayment gldLonContractRepayment = new HlsCusGldLonContractRepayment();
                    gldLonContractRepayment.setWithdrawId(element.getWithdrawId());
                    gldLonContractRepayment.setContractId(element.getContractId());
                    gldLonContractRepayment.setCfItem(element.getCfItem());
                    gldLonContractRepayment.setRepaymentId(element.getRepaymentId());
                    gldLonContractRepayment.setTimes(element.getTimes());
                    gldLonContractRepayment.setFinanceCost(element.getPlannedDueAmount());
                    gldLonContractRepayment.setFinCostDate(element.getPlannedDueDate());
                    gldLonContractRepaymentService.insertSelective(request, gldLonContractRepayment);
                    gldLonContractRepayments.add(gldLonContractRepayment);
                });
            }
            if (gldLonContractRepayments.isEmpty()) {
                HlsCusGldLonContractRepayment queryParam = new HlsCusGldLonContractRepayment();
                queryParam.setWithdrawId(sourceId);
                List<HlsCusGldLonContractRepayment> select = gldLonContractRepaymentMapper.select(queryParam);
                if (LOAN_CHANGE.equalsIgnoreCase(sourceType)) {
                    //获取HlsCusGldLonContractRepayment的集合
                    Map<Long, HlsCusGldLonContractRepayment> idCollect = select.stream().collect(Collectors.toMap(HlsCusGldLonContractRepayment::getRepaymentId, rep -> rep));
                    repayments.forEach(element -> {
                        Long repaymentId = element.getRepaymentId();
                        if (!idCollect.containsKey(repaymentId)) {//判断还款记录是否在gld_lon_contract_repayment中有相应的记录,无则插入,有则更新
                            HlsCusGldLonContractRepayment gldLonContractRepayment = new HlsCusGldLonContractRepayment();
                            gldLonContractRepayment.setWithdrawId(element.getWithdrawId());
                            gldLonContractRepayment.setContractId(element.getContractId());
                            gldLonContractRepayment.setCfItem(element.getCfItem());
                            gldLonContractRepayment.setRepaymentId(repaymentId);
                            gldLonContractRepayment.setTimes(element.getTimes());
                            gldLonContractRepayment.setFinanceCost(element.getDueAmount());
                            gldLonContractRepayment.setFinCostDate(element.getDueDate());
                            gldLonContractRepaymentService.insertSelective(request, gldLonContractRepayment);
                            gldLonContractRepayments.add(gldLonContractRepayment);
                        } else {
                            HlsCusGldLonContractRepayment gldLonContractRepayment = idCollect.get(repaymentId);
                            gldLonContractRepayment.setSubAmount(CalculateUtil.sub(element.getDueAmount(), gldLonContractRepayment.getFinanceCost()));
                            gldLonContractRepayment.setTimes(element.getTimes());
                            gldLonContractRepayment.setFinanceCost(element.getDueAmount());
                            gldLonContractRepayment.setFinCostDate(element.getDueDate());
                            gldLonContractRepaymentService.updateByPrimaryKeySelective(request, gldLonContractRepayment);
                            gldLonContractRepayments.add(gldLonContractRepayment);
                        }
                    });
                } else if (LOAN_INCEPT.equalsIgnoreCase(sourceType)) {
                    select.forEach(element -> {
                        gldLonContractRepayments.add(element);
                    });
                }
            }
            List<HlsCusGldLonContractRepayment> gldRepaymentsWithStartDate = getGldRepaymentsWithStartDate(sourceId, gldLonContractRepayments);
            if (LOAN_INCEPT.equalsIgnoreCase(sourceType)) {
                apportion(request, gldRepaymentsWithStartDate, companyId);
            } else if (LOAN_CHANGE.equalsIgnoreCase(sourceType)) {
                reApportion(request, gldRepaymentsWithStartDate, companyId);
            }
            gldLonContractTrx.setStatus(POST);
            gldLonContractTrx.setMsgLog("计算成功");
            gldLonContractTrxService.updateByPrimaryKeySelective(request, gldLonContractTrx);
        });
    }

    /**
     * 获取新的还款记录,根据期数和日期排序,并赋值起始日期startLocalDate
     *
     * @param withdrawId
     * @param repayments
     * @return
     */
    private List<HlsCusGldLonContractRepayment> getGldRepaymentsWithStartDate(Long withdrawId, List<HlsCusGldLonContractRepayment> repayments) {
        HlsCusLonContractWithdraw withdraw = lonContractWithdrawMapper.selectByPrimaryKey(withdrawId);
        Date dueDate = withdraw.getDueDate();
        LocalDate startDate = LocalDateUtils.date2LocalDate(dueDate);
        Comparator<HlsCusGldLonContractRepayment> comparator = (a, b) -> {
            if (a.getTimes().compareTo(b.getTimes()) == 0) {
                return a.getFinCostDate().compareTo(b.getFinCostDate());
            }
            return Long.compare(a.getTimes(), b.getTimes());
        };
        List<HlsCusGldLonContractRepayment> result = repayments.stream().sorted(comparator).collect(Collectors.toList());
        result.stream().findFirst().get().setStartLocalDate(startDate);
        result.stream().reduce((a, b) -> {
            b.setStartLocalDate(LocalDateUtils.date2LocalDate(a.getFinCostDate()));
            Date finCostDate = a.getFinCostDate();
            LocalDate date = LocalDateUtils.date2LocalDate(finCostDate);
            a.setFinCostDate(LocalDateUtils.localDate2Date(date.minusDays(1)));
            return b;
        });
        return result;
    }

    /**
     * 分摊重算
     *
     * @param request
     * @param repayments
     * @param companyId
     */
    private void reApportion(IRequest request, List<HlsCusGldLonContractRepayment> repayments, Long companyId) {
        Long withdrawId = repayments.stream().findAny().get().getWithdrawId();
        HlsCusGldLonContractFinCost queryParam = new HlsCusGldLonContractFinCost();
        queryParam.setWithdrawId(withdrawId);
        List<HlsCusGldLonContractFinCost> select = mappers.get(0).select(queryParam);
        List<HlsCusGldLonContractFinCost> unPostRecord = select.stream().filter(item -> !POST_FLAG_Y.equalsIgnoreCase(item.getPostFlag())).collect(Collectors.toList());
        batchDelete(unPostRecord);//删除未确认的数据
        apportion(request, repayments, companyId);
//        for (HlsCusGldLonContractRepayment repayment : repayments) {
//            Long gldLonCashflowId = repayment.getGldLonCashflowId();
//            Date finCostDate = repayment.getFinCostDate();
//            Double subAmount = repayment.getSubAmount();
//            HlsCusGldLonContractFinCost finCost = new HlsCusGldLonContractFinCost();
//            finCost.setGldCashflowId(gldLonCashflowId);
//            List<HlsCusGldLonContractFinCost> gldLonContractFinCostList = mappers.get(0).select(finCost);
//            List<HlsCusGldLonContractFinCost> postSortList = gldLonContractFinCostList.stream().filter(item -> POST_FLAG_Y.equalsIgnoreCase(item.getPostFlag())).sorted((a, b) -> a.getEndDate().compareTo(b.getEndDate())).collect(Collectors.toList());
//            Date endDate = null;
//            if (!postSortList.isEmpty())
//                endDate = postSortList.get(postSortList.size() - 1).getEndDate();
//            if (endDate != null && finCostDate.compareTo(endDate) == 0) {//表示该融资现金流下的分摊数据完全确认
//                if (subAmount != null && subAmount.compareTo(0D) != 0) {//当还款金额发生改变时
//                    HlsCusGldLonContractFinCost gldLonContractFinCost = new HlsCusGldLonContractFinCost();
//                    gldLonContractFinCost.setStartDate(endDate);
//                    gldLonContractFinCost.setEndDate(endDate);
//                    gldLonContractFinCost.setCompanyId(companyId);
//                    gldLonContractFinCost.setPeriodName(postSortList.get(postSortList.size() - 1).getPeriodName());
//                    gldLonContractFinCost.setPostFlag(POST_FLAG_N);
//                    gldLonContractFinCost.setDays(1L);
//                    gldLonContractFinCost.setContractId(repayment.getContractId());
//                    gldLonContractFinCost.setRepaymentId(repayment.getRepaymentId());
//                    gldLonContractFinCost.setGldCashflowId(repayment.getGldLonCashflowId());
//                    self().insertSelective(request, gldLonContractFinCost);
//                } else {
//                    try {
//                        HlsCusGldLonContractRepayment cloneRepayment = (HlsCusGldLonContractRepayment) BeanUtils.cloneBean(repayment);
//                        cloneRepayment.setFinanceCost(subAmount);
//                        LocalDate start = LocalDateUtils.date2LocalDate(endDate).plusDays(1);
//                        cloneRepayment.setStartLocalDate(start);
//                        cloneRepayment.setGldLonCashflowId(repayment.getGldLonCashflowId());
//                        apportionList.add(cloneRepayment);
//                    } catch (IllegalAccessException e) {
//                        e.printStackTrace();
//                    } catch (InstantiationException e) {
//                        e.printStackTrace();
//                    } catch (InvocationTargetException e) {
//                        e.printStackTrace();
//                    } catch (NoSuchMethodException e) {
//                        e.printStackTrace();
//                    }
//                }
//            } else {
//                apportionList.add(repayment);
//            }
//        }
    }

    /**
     * 分摊计算
     *
     * @param repayments
     * @param companyId
     */
    private void apportion(IRequest request, List<HlsCusGldLonContractRepayment> repayments, Long companyId) {
        final List<HlsCusGldLonContractFinCost> gldLonContractFinCosts = new ArrayList<>();
        FinancialAttribute financialAttribute = new FinancialAttribute();
        financialAttribute.setCompanyId(companyId);
        financialAttribute = financialAttributeMapper.selectOne(financialAttribute);//根据合同上的公司id找到对应的公司财务属性
        Long setOfBooksId = financialAttribute.getSetOfBooksId();
        SetOfBooks setOfBooks = setOfBooksMapper.selectByPrimaryKey(setOfBooksId);//根据公司财务属性上对应的账套id找到对应的账套
        Long periodSetId = setOfBooks.getPeriodSetId();
        for (int i = 0; i < repayments.size(); i++) {
            HlsCusGldLonContractRepayment repayment = repayments.get(i);
            Date dueDate = repayment.getFinCostDate();
            LocalDate startLocalDate = repayment.getStartLocalDate();//获取起始日期
            LocalDate endLocalDate = LocalDateUtils.date2LocalDate(dueDate);//获取结束日期
            long between = ChronoUnit.DAYS.between(startLocalDate, endLocalDate) + 1;
            Double leftAmount = repayment.getFinanceCost();
            //找到符合开始和结束日期的区间
            int startMonth = startLocalDate.getMonthValue();//开始月份
            int startYear = startLocalDate.getYear();//开始年份
            int endMonth = endLocalDate.getMonthValue();//结束月份
            int endYear = endLocalDate.getYear();//结束年份
            Map paraMap = new HashMap<>();
            paraMap.put("periodSetId", periodSetId);
            paraMap.put("startMonth", startMonth);
            paraMap.put("startYear", startYear);
            paraMap.put("endMonth", endMonth);
            paraMap.put("endYear", endYear);
            List<Period> periodBetween = periodMapper.getPeriodBetween(paraMap);
            List<HlsCusGldLonContractFinCost> sumList = new ArrayList<>();
            for (int j = 0; j < periodBetween.size(); j++) {
                Period period = periodBetween.get(j);
                String periodName = period.getPeriodName();
                Long periodMonth = period.getPeriodMonth();
                Long periodYear = period.getPeriodYear();
                HlsCusGldLonContractFinCost gldLonContractFinCost = new HlsCusGldLonContractFinCost();
                if (new Long(startMonth).compareTo(periodMonth) == 0 && new Long(startYear).compareTo(periodYear) == 0 && !(startMonth == endMonth && startYear == endYear)) {
                    LocalDate from = startLocalDate;
                    LocalDate to = LocalDateUtils.date2LocalDate(period.getEndDate());
                    gldLonContractFinCost = getGldLonContractFinCost(gldLonContractFinCost, repayment, periodName, between, from, to);
                } else if (new Long(endMonth).compareTo(periodMonth) == 0 && new Long(endYear).compareTo(periodYear) == 0 && !(startMonth == endMonth && startYear == endYear)) {
                    LocalDate from = LocalDateUtils.date2LocalDate(period.getStartDate());
                    LocalDate to = endLocalDate;
                    gldLonContractFinCost = getGldLonContractFinCost(gldLonContractFinCost, repayment, periodName, between, from, to);
                } else if (startMonth == endMonth && startYear == endYear) {
                    LocalDate from = startLocalDate;
                    LocalDate to = endLocalDate;
                    gldLonContractFinCost = getGldLonContractFinCost(gldLonContractFinCost, repayment, periodName, between, from, to);
                } else {
                    LocalDate from = LocalDateUtils.date2LocalDate(period.getStartDate());
                    LocalDate to = LocalDateUtils.date2LocalDate(period.getEndDate());
                    gldLonContractFinCost = getGldLonContractFinCost(gldLonContractFinCost, repayment, periodName, between, from, to);
                }
                if (gldLonContractFinCost != null) {
                    if (j == periodBetween.size() - 1 && j != 0) {
                        gldLonContractFinCost.setFinanceCost(leftAmount);
                    } else {
                        leftAmount = CalculateUtil.sub(leftAmount, gldLonContractFinCost.getFinanceCost());
                    }
                    gldLonContractFinCost.setCompanyId(companyId);
                    gldLonContractFinCost.setPeriodName(periodName);
                    if (gldLonContractFinCost.getSumFlag() != null && true == gldLonContractFinCost.getSumFlag()) {
                        sumList.add(gldLonContractFinCost);
                    } else {
                        gldLonContractFinCosts.add(gldLonContractFinCost);
                    }
                }
            }
            if (!sumList.isEmpty()) {
                List<HlsCusGldLonContractFinCost> changeList = getChangeList(sumList);
                gldLonContractFinCosts.addAll(changeList);
            }

        }
        gldLonContractFinCosts.forEach(item -> self().insertSelective(request, item));
    }

    /**
     * 获取融资合同成本分摊对象
     *
     * @param gldLonContractFinCost
     * @param repayment
     * @param betweenTotal
     * @param from
     * @param to
     */
    private HlsCusGldLonContractFinCost getGldLonContractFinCost(HlsCusGldLonContractFinCost gldLonContractFinCost, HlsCusGldLonContractRepayment repayment, String periodName, long betweenTotal, LocalDate from, LocalDate to) {
        Long gldCashflowId = repayment.getGldLonCashflowId();
        HlsCusGldLonContractFinCost finCost = new HlsCusGldLonContractFinCost();
        finCost.setGldCashflowId(gldCashflowId);
        finCost.setPeriodName(periodName);
        finCost.setPostFlag(POST_FLAG_Y);
        finCost = mappers.get(0).selectOne(finCost);
        if (finCost == null) {
            long between = ChronoUnit.DAYS.between(from, to) + 1;
            gldLonContractFinCost.setDays(between);
            gldLonContractFinCost.setContractId(repayment.getContractId());
            gldLonContractFinCost.setRepaymentId(repayment.getRepaymentId());
            gldLonContractFinCost.setGldCashflowId(repayment.getGldLonCashflowId());
            gldLonContractFinCost.setWithdrawId(repayment.getWithdrawId());
            gldLonContractFinCost.setStartDate(LocalDateUtils.localDate2Date(from));
            gldLonContractFinCost.setEndDate(LocalDateUtils.localDate2Date(to));
            gldLonContractFinCost.setPostFlag(POST_FLAG_N);
            Double percent = CalculateUtil.div(Double.valueOf(between), Double.valueOf(betweenTotal), 6);
            Double amount = CalculateUtil.mul(repayment.getFinanceCost(), percent);
            gldLonContractFinCost.setFinanceCost(amount);
            return gldLonContractFinCost;
        } else {
            Date endDate = finCost.getEndDate();
            Date startDate = finCost.getStartDate();
            long between = ChronoUnit.DAYS.between(LocalDateUtils.date2LocalDate(startDate), LocalDateUtils.date2LocalDate(endDate)) + 1;
            Double percent = CalculateUtil.div(Double.valueOf(between), Double.valueOf(betweenTotal), 6);
            Double amount = CalculateUtil.mul(repayment.getFinanceCost(), percent);
            Double sub = CalculateUtil.sub(amount, finCost.getFinanceCost());
            if (sub.compareTo(0D) == 0) {
                return null;
            }
            try {
                HlsCusGldLonContractFinCost o = (HlsCusGldLonContractFinCost) BeanUtils.cloneBean(finCost);
                o.setFinanceCost(sub);
                o.setStartDate(finCost.getStartDate());
                o.setEndDate(finCost.getEndDate());
                o.setDays(1L);
                o.setPostFlag(POST_FLAG_N);
                o.setSumFlag(true);
                return o;
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            } catch (NoSuchMethodException e) {
                e.printStackTrace();
            }
            return null;
        }
    }

    /**
     * 当list中日期连续时,需要进行合并
     *
     * @param sumList
     * @return
     */
    private List<HlsCusGldLonContractFinCost> getChangeList(List<HlsCusGldLonContractFinCost> sumList) {
        final List<HlsCusGldLonContractFinCost> result = new ArrayList<>();
        HlsCusGldLonContractFinCost gldLonContractFinCost = sumList.stream().sorted((a, b) -> a.getStartDate().compareTo(b.getStartDate())).reduce((a, b) -> {
            LocalDate lastEndDate = LocalDateUtils.date2LocalDate(a.getEndDate());
            LocalDate startDate = LocalDateUtils.date2LocalDate(b.getStartDate());
            if (lastEndDate.plusDays(1L).compareTo(startDate) == 0) {//当上个结束日的下一天为当前起始日时,可以进行合并
                Double sumAmount = CalculateUtil.add(a.getFinanceCost(), b.getFinanceCost());
                b.setFinanceCost(sumAmount);
                b.setDays(1L);
                b.setStartDate(b.getEndDate());
            } else {
                result.add(a);
            }
            return b;
        }).get();
        if (!result.contains(gldLonContractFinCost))
            result.add(gldLonContractFinCost);
        return result;
    }

    /**
     * 根据日期获取期间
     *
     * @param date
     * @return
     */
    private String getPeriodNameByDate(Date date) {
        if (date == null)
            return null;
        LocalDate localDate = LocalDateUtils.date2LocalDate(date);
        int monthValue = localDate.getMonthValue();
        int year = localDate.getYear();
        if (monthValue < 10) {
            return year + "-0" + monthValue;
        }
        return year + "-" + monthValue;
    }


    /*计提查询*/
    public List<HlsCusGldLonContractFinCost> countDrawQuery(IRequest request, HlsCusGldLonContractFinCost dto, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        List<HlsCusGldLonContractFinCost> list = gldLonContractFinCostMapper.countDrawQuery(dto);
        if (list.isEmpty()) {
            return list;
        }
        if ("N".equals(dto.getPostFlag())) {
            //如果查询条件是待计提的数据 已确认计提利息直接为0
            list.get(0).setFinanceCostSum(0D);
        } else {
            //如果查询条件是所有数据或者已计提的数据 查询已计提的利息
            dto.setPostFlag("Y");
            List<HlsCusGldLonContractFinCost> amountList = gldLonContractFinCostMapper.querySumCost(dto);
            if (amountList.isEmpty() || amountList.get(0) == null) {
                list.get(0).setFinanceCostSum(0D);
            } else {
                list.get(0).setFinanceCostSum(amountList.get(0).getFinanceCostSum());
            }
        }

        for (HlsCusGldLonContractFinCost item : list) {
            if (item.getRepaymentDateFrom() == null) {
                Long withdrawId = item.getWithdrawId();
                Date dueDate = lonContractWithdrawMapper.selectByPrimaryKey(withdrawId).getDueDate();
                item.setRepaymentDateFrom(dueDate);
            }
        }

        return list;
    }

}