package hls.core.gld.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fnd.mapper.FirmDefineMapper;
import hls.core.fnd.mapper.HLSCurrencyMapper;
import hls.core.gld.dto.JeLine;
import hls.core.gld.dto.PeriodStatus;
import hls.core.gld.mapper.*;
import hls.core.gld.service.IJeLineService;
import hls.core.hls.components.CalculateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.gld.dto.JeHead;
import hls.core.gld.service.IJeHeadService;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class JeHeadServiceImpl extends BaseServiceImpl<JeHead> implements IJeHeadService {
    @Autowired
    private IJeLineService jeLineService;
    @Autowired
    private PeriodStatusMapper periodStatusMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private JeHeadMapper mapper;
    @Autowired
    private FirmDefineMapper firmDefineMapper;
    @Autowired
    private CostCenterMapper centerMapper;
    @Autowired
    private AccountMapper accountMapper;
    @Autowired
    private HLSCurrencyMapper hlsCurrencyMapper;
    @Autowired
    private JeLineMapper lineMapper;

    public List<JeHead> jeListQuery(Map<String, Object> map, int page, int pageSize) {
        PageHelper.startPage(page,pageSize);
        return mapper.jeListQuery(map);
    }

    @Override
    public List<JeLine> jeConfirm(List<JeHead> list, IRequest requestCtx, Long companyId, Long userId) {
        Map validate = this.validate(list, companyId);
        if ("false".equals(validate.get("success")))
            throw new IllegalArgumentException((String) validate.get("message"));

        List<JeLine> listLine = new ArrayList<>();
        //更新行表
        //先查询出所有的行
        for (int i = 0; i < list.size(); i++) {
            List<JeLine> jeLineList = new ArrayList<>();
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("jeHeadId", list.get(i).getJeHeadId());
            //查询
            jeLineList = jeLineService.queryNoPage(map);
            for (int j = 0; j < jeLineList.size(); j++) {
                //锁表
                databaseLockProvider.lock(jeLineList.get(j));
                jeLineList.get(j).setJeConfirmDate(new Date());
                jeLineList.get(j).setJeConfirmedBy(userId);
                jeLineList.get(j).setJeHeadId(list.get(i).getJeHeadId());
                jeLineList.get(j).set__status("update");
                jeLineList.get(j).setJeStatus("CONFIRM");
            }
            listLine = jeLineService.self().batchUpdate(requestCtx, jeLineList);
            //更新头表
            list.get(i).setCompanyId(companyId);
            list.get(i).setJeConfirmedBy(userId);
            list.get(i).setJeConfirmDate(new Date());
            self().updateByPrimaryKeySelective(requestCtx, list.get(i));

        }
        return listLine;
    }


    /**
     * 凭证确认校验
     *
     * @param list
     * @param companyId
     * @return
     */
    public Map validate(List<JeHead> list, Long companyId) {
        Map result = new HashMap<>();
        outer:
        for (JeHead item : list) {//外层循环
            item = mapper.selectByPrimaryKey(item.getJeHeadId());
            if (!"NEW".equalsIgnoreCase(item.getJeStatus())) {
                result.put("success", "false");
                result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证已被确认,不可重复确认");
                break;
            }
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("jeHeadId", item.getJeHeadId());
            List<JeLine> jeLines = jeLineService.queryNoPage(map);
            Double cr = 0D;
            Double dr = 0D;
            Double crf = 0D;
            Double drf = 0D;

            for (JeLine jeline : jeLines) {//内层
                cr = CalculateUtil.add(cr, jeline.getCrAmount());
                dr = CalculateUtil.add(dr, jeline.getDrAmount());
                crf = CalculateUtil.add(crf, jeline.getCrFunctionalAmount());
                drf = CalculateUtil.add(drf, jeline.getDrFunctionalAmount());
                if (jeline.getCompanyId() == null) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的公司id为空");
                    break outer;
                }
                if (firmDefineMapper.selectByPrimaryKey(jeline.getCompanyId()) == null) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证公司id取值不合法");
                    break outer;
                }
                if (jeline.getCostCenterId() == null) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证成本中心id为空");
                    break outer;
                }
                if (centerMapper.selectByPrimaryKey(jeline.getCostCenterId()) == null) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证成本中心id取值不合法");
                    break outer;
                }
                if (jeline.getAccountId() == null) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证科目id为空");
                    break outer;
                }
                if (accountMapper.selectByPrimaryKey(jeline.getAccountId()) == null) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证科目id取值不合法");
                    break outer;
                }
                if (jeline.getCurrency() == null) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证币种为空");
                }
                if (hlsCurrencyMapper.selectByPrimaryKey(jeline.getCurrency()) == null) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证币种取值不合法");
                }
                if (!(((jeline.getCrAmount() != null && jeline.getCrFunctionalAmount() != null) && (jeline.getDrAmount() == null && jeline.getDrFunctionalAmount() == null)) ||
                        ((jeline.getCrAmount() == null && jeline.getCrFunctionalAmount() == null) && (jeline.getDrAmount() != null && jeline.getDrFunctionalAmount() != null)))) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证借贷取值不合法");
                }
                if (!(((jeline.getCrAmount() == null && jeline.getCrFunctionalAmount() == null) && (jeline.getDrAmount() != null && jeline.getDrFunctionalAmount() != null)) ||
                        ((jeline.getCrAmount() != null && jeline.getCrFunctionalAmount() != null) && (jeline.getDrAmount() == null && jeline.getDrFunctionalAmount() == null)))) {
                    result.put("success", "false");
                    result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证借贷取值不合法");
                }
            }
            if (cr.compareTo(dr) != 0) {
                result.put("success", "false");
                result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证借贷金额不等");
                break;
            }
            if (crf.compareTo(drf) != 0) {
                result.put("success", "false");
                result.put("message", "凭证号为:" + item.getJeNumber() + "的凭证借贷金额不等");
                break;
            }
            PeriodStatus periodStatus = new PeriodStatus();
            periodStatus.setPeriodName(item.getPeriodName());
            periodStatus.setCompanyId(companyId);
            periodStatus = periodStatusMapper.selectOne(periodStatus);
            if (periodStatus == null) {
                result.put("success", "false");
                result.put("message", "凭证号为:" + item.getJeNumber() + "未找到对应的期间状态");
                break;
            }
            if ("CLOSE".equalsIgnoreCase(periodStatus.getStatus())) {
                result.put("success", "false");
                result.put("message", "凭证号为:" + item.getJeNumber() + "对应的期间状态为关闭");
                break;
            }


        }

        return result;
    }
}