package hls.core.gld.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.DTOStatus;
import com.hand.hap.system.service.IBaseService;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.hls.dto.HlsDbDataSourceColumn;
import hls.core.hls.service.IHlsDbDataSourceColumnService;
import hls.core.ruleengine.dto.HlsRuleEngine;
import hls.core.ruleengine.mapper.HlsRuleEngineMapper;
import hls.core.ruleengine.service.IHlsRuleEngineService;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.gld.dto.JeRuleEngine;
import hls.core.gld.service.IJeRuleEngineService;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

@Service
@Transactional
public class JeRuleEngineServiceImpl extends BaseServiceImpl<JeRuleEngine> implements IJeRuleEngineService {

    @Autowired
    private IHlsRuleEngineService service;

    @Autowired
    private HlsRuleEngineMapper ruleEngineMapper;

    @Autowired
    private IHlsDbDataSourceColumnService hlsDbDataSourceColumnService;

    @Override
    public List<JeRuleEngine> batchUpdate(IRequest request, List<JeRuleEngine> list) {
        IJeRuleEngineService self = (IJeRuleEngineService) AopContext.currentProxy();
        for (JeRuleEngine t : list) {
            HlsRuleEngine ruleEngine = new HlsRuleEngine();
            switch (t.get__status()) {
                case DTOStatus.ADD:
                    ruleEngine.setDataSourceId(t.getDataSourceId());
                    ruleEngine.setRuleEngineCode("JE_" + t.getCompanyId().toString() + "_" + t.getJeTrx());
                    ruleEngine.setRuleEngineName("JE_" + t.getCompanyId().toString() + "_" + t.getJeTrx());
                    ruleEngine.setRuleEngineType("GLD.JE_TEMPLATE");
                    ruleEngine = service.self().insertSelective(request, ruleEngine);
                    t.setRuleEngineId(ruleEngine.getRuleEngineId());
                    self.insertSelective(request, t);
                    break;
                case DTOStatus.UPDATE:
                    ruleEngine.setRuleEngineId(t.getRuleEngineId());
                    ruleEngine = ruleEngineMapper.selectOne(ruleEngine);
                    ruleEngine.setDataSourceId(t.getDataSourceId());
                    service.updateByPrimaryKeySelective(request, ruleEngine);
                    self.updateByPrimaryKeySelective(request, t);
                    break;
                case DTOStatus.DELETE:
                    ruleEngine.setRuleEngineId(t.getRuleEngineId());
                    service.deleteByPrimaryKey(ruleEngine);
                    self.deleteByPrimaryKey(t);
                    break;
                default:
                    break;
            }
        }
        return list;
    }


    @Override
    public List<HlsDbDataSourceColumn> jeRuleEngineerLovData(IRequest request, JeRuleEngine condition, int pageNum, int pageSize) {
        condition = mappers.get(0).selectOne(condition);
        Long dataSourceId = condition.getDataSourceId();
        HlsDbDataSourceColumn column = new HlsDbDataSourceColumn();
        column.setDataSourceId(dataSourceId);
        List<HlsDbDataSourceColumn> columns = hlsDbDataSourceColumnService.select(request, column, pageNum, pageSize);
        columns.forEach(item -> {
            if (item.getColumnName() != null && !"".equalsIgnoreCase(item.getColumnName()))
                item.setColumnName("{$" + item.getColumnName() + "$}");
        });
        return columns;
    }
}