package hls.core.hls.bp.components;

import com.hand.hap.core.IRequest;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsBpMasterMapper;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.bp.service.HlsBpMasterService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventCommonService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.sys.service.SysUserService;
import org.activiti.engine.delegate.DelegateExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public abstract class AbstractHlsBpMasterEventHandler implements SysEventCommonService {

    private static final String DOCUMENT_CATEGORY = "HLS_BP_MASTER";
    private static final String URL = "bp/BP100/bp_master_index.html";

    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private HlsCusBpMasterMapper bpCusMasterMapper;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private HlsBpMasterService hlsBpMasterService;

    abstract String getNoticeTitle();

    abstract String getMessage(String userDesc, String bpName, String bpCode);

    public long getLevel() {
        return 2L;
    }

    public String getSourceModel() {
        return SysEventCodeUtil.PROPERTITY_SOURCEMODULE_FND3;
    }

    public String getRelation() {
        return SysEventCodeUtil.PROPERTITY_RELATIONTYPE_P2D;
    }

    @Override
    public String getDocumentCategory() {
        return null;
    }

    @Override
    public void process(IRequest iRequest, Long documentId, String documentCategory, String documentType, Map<String, Object> params) {
        if (params == null) {
            params = new HashMap<>();
        }
        HlsCusBpMaster bpMaster = new HlsCusBpMaster();
        bpMaster.setBpId(documentId);
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        List<HlsCusBpMaster> list = hlsBpMasterService.selectHlsBpMasterById(bpMaster);
        if(list != null && list.size() > 0){
            bpMaster = list.get(0);
        }
        String message = getMessage(sysUser.getDescription(), bpMaster.getBpName(), bpMaster.getBpCode());
        String noticeTitle = getNoticeTitle();
        params.put("message", message);
        params.put("noticeTitle", noticeTitle);
        params.put("url", URL);
        params.put("level", getLevel());
        params.put("noticeType", "NOTICE");

        bpMaster = bpCusMasterMapper.selectHlsBpMasterById(bpMaster).get(0);

        sysEventService.eventSave(iRequest, documentId, DOCUMENT_CATEGORY, bpMaster.getBpClass(), getSourceModel(), getEventCode(), getRelation(), params);
    }

    @Override
    public void notice(DelegateExecution delegateExecution, String status) {

    }
}
