package hls.core.hls.bp.controllers;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.fnd.dto.FndCity;
import hls.core.fnd.dto.FndDistrict;
import hls.core.fnd.service.FndDistrictService;
import hls.core.fnd.service.IFndCityService;
import hls.core.hls.bp.dto.HlsCusBpMasterAddress;
import hls.core.hls.bp.service.HlsBpMasterAddressService;
import hls.core.hls.dto.FndProvince;
import hls.core.hls.service.FndProvinceService;

@Controller
public class HlsBpMasterAddressController extends BaseController
{
	@Autowired
	private HlsBpMasterAddressService service;
	
	@Autowired
	private FndProvinceService provinceService;
	@Autowired
	private IFndCityService cityService;
	@Autowired
	private FndDistrictService districtService;
	
	
	@RequestMapping(value = "/hls/bp/address/query")
	@ResponseBody
	public ResponseData query(final HlsCusBpMasterAddress bpMasterAddress, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.selectAll(requestContext, bpMasterAddress, page, pagesize));
	}
	
	@RequestMapping(value = "/hls/bp/address/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody final List<HlsCusBpMasterAddress> bpMasterAddresses, final BindingResult result,
                               final HttpServletRequest request){
		 	getValidator().validate(bpMasterAddresses, result);
	        if (result.hasErrors()) {
	            ResponseData responseData = new ResponseData(false);
	            responseData.setMessage(getErrorMessage(result, request));
	            return responseData;
	        }
	        IRequest requestContext = createRequestContext(request);
	        String province = null;
	        String city = null;
	        String district = null;
	        
	        if(bpMasterAddresses != null)
	        {
	        	for(HlsCusBpMasterAddress address:bpMasterAddresses)
	        	{
	        		if(address.getProvince() != null && address.getProvince() !="")
	        		{
	        			province = provinceService.queryProvinceCode(address.getProvince());
	        			address.setProvince(province);
	        		}else
	        		{
	        			//address.setProvince("-1");
	        		}
	        		
	        		if(address.getCity() != null && address.getCity() != "")
	        		{
						province = provinceService.queryProvinceCode(address.getProvince());
	        			city = cityService.queryCityCodeByAdd(address.getCity(),province);
	        			address.setCity(city);
	        		}else
	        		{
	        			//address.setCity("-1");
	        		}
	        		
	        		if(address.getDistrict() != null && address.getDistrict() !="")
	        		{
						province = provinceService.queryProvinceCode(address.getProvince());
	        			district =districtService.queryDistrictCodeByPro(address.getDistrict(),province);
	        			address.setDistrict(district);
	        		}else
	        		{
	        			//address.setDistrict("-1");
	        		}
	        	}
	        }
		for(int i =0; i<bpMasterAddresses.size(); i++){
			bpMasterAddresses.get(i).setBpId(bpMasterAddresses.get(0).getBpId());
		}
		List<HlsCusBpMasterAddress> lists = service.batchUpdate(requestContext, bpMasterAddresses);
	        if(lists != null)
	        {
		        for(HlsCusBpMasterAddress list:lists)
		        {
		        	if(list.getProvince() != null)
	        		{
		        		FndProvince pro = new FndProvince();
		        		pro.setProvince(list.getProvince());
		        		FndProvince fndPro = provinceService.selectByPrimaryKey(requestContext, pro);
		        		if(fndPro != null)
		        		{
		        			list.setProvince(fndPro.getDescription());
		        		}else
		        		{
		        			list.setProvince("");
		        		}
	        			
	        		}
	        		if(list.getCity() != null)
	        		{
	        			FndCity ci = new FndCity();
	        			ci.setCity(list.getCity());
	        			FndCity fndCi = cityService.selectByPrimaryKey(requestContext, ci);
	        			if(fndCi != null)
	        			{
	        				list.setCity(fndCi.getDescription());
	        			}else
	        			{
	        				list.setCity("");
	        			}
	        			
	        		}
	        		if(list.getDistrict() != null)
	        		{
	        			FndDistrict dis = new FndDistrict();
	        			dis.setDistrict(list.getDistrict());
	        			FndDistrict fndDis =districtService.selectByPrimaryKey(requestContext, dis);
	        			if(fndDis != null)
	        			{
	        				list.setDistrict(fndDis.getDescription());
	        			}else
	        			{
	        				list.setDistrict("");
	        			}
	        			
	        		}
		        }
	        }

	        
	        return new ResponseData(lists);
	}
	
}
