package hls.core.hls.bp.controllers;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.fnd.dto.FndCity;
import hls.core.fnd.dto.FndDistrict;
import hls.core.fnd.service.FndDistrictService;
import hls.core.fnd.service.IFndCityService;
import hls.core.hls.bp.dto.HlsCusBpMasterBankAccount;
import hls.core.hls.bp.service.HlsBpMasterBankAccountService;
import hls.core.hls.dto.FndProvince;
import hls.core.hls.service.FndProvinceService;

@Controller
public class HlsBpMasterBankAccountController extends BaseController
{
	@Autowired
	private HlsBpMasterBankAccountService service;

	@Autowired
	private FndProvinceService provinceService;
	@Autowired
	private IFndCityService cityService;
	@Autowired
	private FndDistrictService districtService;
	
	@RequestMapping(value = "/hls/bp/bankAccount/query")
	@ResponseBody
	public ResponseData query(final HlsCusBpMasterBankAccount bpMasterBankAccount, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
							  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.selectAll(requestContext, bpMasterBankAccount, page, pagesize));
	}

	@RequestMapping(value = "/hls/debt/account/query")
	@ResponseBody
	public ResponseData debtAccountQuery(final HlsCusBpMasterBankAccount bpMasterBankAccount, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
										 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.debtAccountQuery(requestContext, bpMasterBankAccount, page, pagesize));
	}

	@RequestMapping(value = "/hls/bp/bankAccount/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody final List<HlsCusBpMasterBankAccount> bpMasterBankAccounts, final BindingResult result,
							   final HttpServletRequest request){
		 	getValidator().validate(bpMasterBankAccounts, result);

	         if (result.hasErrors()) {
	            ResponseData responseData = new ResponseData(false);
	            responseData.setMessage(getErrorMessage(result, request));
	            return responseData;
	        }
	        IRequest requestContext = createRequestContext(request);
	        
	        String province = null;
	        String city = null;
	        String district = null;
	        
	        if(bpMasterBankAccounts != null)
	        {
	        	for(HlsCusBpMasterBankAccount account:bpMasterBankAccounts)
	        	{
					if(account.get__status().equals("update") && account.getDebtBankId() == null){
						service.modifyBpMasterBankAccountDeptBankId(account);
					}
	        		if(account.getProvince() != null && account.getProvince() !="")
	        		{
	        			province = provinceService.queryProvinceCode(account.getProvince());
	        			account.setProvince(province);
	        		}else
	        		{
	        			//account.setProvince("-1");
	        		}
	        		
	        		if(account.getCity() != null && account.getCity() != "")
	        		{
	        			city = cityService.queryCityCode(account.getCity());
	        			account.setCity(city);
	        		}else
	        		{
	        			//account.setCity("-1");
	        		}
	        		
	        		if(account.getDistrict() != null && account.getDistrict() !="")
	        		{
	        			district =districtService.queryDistrictCode(account.getDistrict());
	        			account.setDistrict(district);
	        		}else
	        		{
	        			//account.setDistrict("-1");
	        		}
	        	}
	        }
	        List<HlsCusBpMasterBankAccount> lists = service.batchUpdate(requestContext, bpMasterBankAccounts);
	        
	        if(lists != null)
	        {
		        for(HlsCusBpMasterBankAccount list:lists)
		        {
		        	if(list.getProvince() != null)
	        		{
		        		FndProvince pro = new FndProvince();
		        		pro.setProvince(list.getProvince());
		        		FndProvince fndPro = provinceService.selectByPrimaryKey(requestContext, pro);
		        		if(fndPro != null)
		        		{
		        			list.setProvince(fndPro.getDescription());
		        		}else
		        		{
		        			list.setProvince("");
		        		}
	        		}
	        		if(list.getCity() != null)
	        		{
	        			FndCity ci = new FndCity();
	        			ci.setCity(list.getCity());
	        			FndCity fndCi = cityService.selectByPrimaryKey(requestContext, ci);
	        			if(fndCi != null)
	        			{
	        				list.setCity(fndCi.getDescription());
	        			}else
	        			{
	        				list.setCity("");
	        			}
	        		}
	        		if(list.getDistrict() != null)
	        		{
	        			FndDistrict dis = new FndDistrict();
	        			dis.setDistrict(list.getDistrict());
	        			FndDistrict fndDis =districtService.selectByPrimaryKey(requestContext, dis);
	        			if(fndDis != null)
	        			{
	        				list.setDistrict(fndDis.getDescription());
	        			}else
	        			{
	        				list.setDistrict("");
	        			}
	        		}
		        }
	        }
	        
	        return new ResponseData(lists);
	}
}
