package hls.core.hls.bp.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.csh.dto.HlsCusCshBank;
import hls.core.csh.service.CshBankAccountService;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.service.HlsBpMasterEventService;
import hls.core.hls.bp.service.HlsBpMasterFavoriteService;
import hls.core.hls.bp.service.HlsBpMasterRoleService;
import hls.core.hls.bp.service.HlsBpMasterService;
import hls.core.hls.dto.HlsUserRelationship;
import hls.core.hls.service.IHlsUserRelationshipService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
public class HlsBpMasterController extends BaseController {
    @Autowired
    private HlsBpMasterService service;

    @Autowired
    private HlsBpMasterRoleService roleService;

    @Autowired
    private CshBankAccountService bankService;

    @Autowired
    private HlsBpMasterFavoriteService bpMasterFavoriteService;

    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;

    @Autowired
    private HlsBpMasterEventService hlsBpMasterEventService;

    @Autowired
    private IHlsUserRelationshipService hlsUserRelationshipService;

    @RequestMapping(value = "/hls/bp/master/by/id/query")
    @ResponseBody
    public ResponseData queryByBpId(final HlsCusBpMaster bpMaster, final HttpServletRequest request) {
        List<HlsCusBpMaster> hlsCusBpMasters = service.queryById(bpMaster.getBpId());

        return new ResponseData(hlsCusBpMasters);
    }

    @RequestMapping(value = "/hls/bp/master/credit/detail/query")
    @ResponseBody
    public ResponseData queryBpMasterCredit(HlsCusBpMaster bpMaster) {
        List<HlsCusBpMaster> hlsCusBpMasters = service.masterCreditDetailQuery(bpMaster);
        return new ResponseData(hlsCusBpMasters);
    }

    @RequestMapping(value = "/hls/bp/master/query")
    @ResponseBody
    public ResponseData query(final HlsCusBpMaster bpMaster, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request, final HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        Long userId = requestContext.getUserId();
        //查询所符合条件的bpId
        List<HlsCusBpMaster> hlsCusBpMasters = service.queryBpIds(requestContext, bpMaster, page, pagesize, companyId, userId);

        return new ResponseData(hlsCusBpMasters);
    }

    @RequestMapping(value = "/hls/bp/master/submit")
    @ResponseBody
    public ResponseData submit(@RequestBody final List<HlsCusBpMaster> bpMasters, final BindingResult result,
                               final HttpServletRequest request, final HttpSession session) {
        getValidator().validate(bpMasters, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }

        if (bpMasters != null) {
            for (HlsCusBpMaster bpMaster : bpMasters) {
                if (bpMaster.getBpId() == null || bpMaster.getBpId() == 0) {
                    bpMaster.set__status("add");
                } else {
                    bpMaster.set__status("update");
                }
            }
        }
        IRequest requestContext = RequestHelper.getCurrentRequest();
        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        List<HlsCusBpMaster> datas = service.masterUpdate(requestContext, bpMasters.get(0), companyId);
        return new ResponseData(datas);
    }

    @RequestMapping(value = "/hls/bp/master/submitCredit")
    @ResponseBody
    public ResponseData submitCredit(@RequestBody final List<HlsCusBpMaster> bpMasters, final BindingResult result,
                                final HttpServletRequest request, final HttpSession session) {
        getValidator().validate(bpMasters, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }

        if (bpMasters != null) {
            for (HlsCusBpMaster bpMaster : bpMasters) {
                if (bpMaster.getBpId() == null || bpMaster.getBpId() == 0) {
                    bpMaster.set__status("add");
                } else {
                    bpMaster.set__status("update");
                }
            }
        }
        IRequest requestContext = createRequestContext(request);
        Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
        List<HlsCusBpMaster> datas = service.masterUpdate2(requestContext, bpMasters.get(0), companyId);
        return new ResponseData(datas);
    }

    @RequestMapping(value = "/hls/favorite/collect/submit")
    @ResponseBody
    public ResponseData collect(String collectBpId, final HttpServletRequest request) {

        IRequest requestContext = createRequestContext(request);

        Long userId = requestContext.getUserId();

        String[] arrs = collectBpId.split(",");

        HlsUserRelationship hlsUserRelationships = null;
        for (int i = 0; i < arrs.length; i++) {
            hlsUserRelationships = new HlsUserRelationship();
            hlsUserRelationships.setRelationDocumentId(Long.parseLong(arrs[i]));
            hlsUserRelationships.setUserId(userId);
            hlsUserRelationships.setRelationDocumentCategory("HLS_BP_MASTER");
            hlsUserRelationships.setRelationType("P2D");

            hlsUserRelationshipService.insertSelective(requestContext, hlsUserRelationships);
        }
//		HlsCusBpMasterFavorite bpMasterFavorite = null;
//		for(int i=0;i<arrs.length;i++)
//		{
//			bpMasterFavorite = new HlsCusBpMasterFavorite();
//
//			bpMasterFavorite.setBpId(Long.parseLong(arrs[i]));
//			bpMasterFavorite.setUserId(userId);
//			bpMasterFavorite.setFavoriteDate(new Date());
//           //商业伙伴收藏事件
//			bpMasterFavoriteService.insertSelective(requestContext, bpMasterFavorite);
//			hlsBpMasterEventService.bpEvent(requestContext, "HLS_BP_MASTER.COLLECT", bpMasterFavorite.getBpId());
//
//		}

        return new ResponseData();
    }

    @RequestMapping(value = "/hls/favorite/unCollect/submit")
    @ResponseBody
    public ResponseData collected(String unCollectBpId, final HttpServletRequest request) {
        String[] arrs = unCollectBpId.split(",");

        for (int i = 0; i < arrs.length; i++) {
            hlsUserRelationshipService.uncollect(Long.parseLong(arrs[i]));
//			bpMasterFavoriteService.unCollect(Long.parseLong(arrs[i]));
            //商业伙伴取消收藏事件
//			hlsBpMasterEventService.bpEvent(createRequestContext(request), "HLS_BP_MASTER.UNCOLLECT", Long.parseLong(arrs[i]));
        }

        return new ResponseData();
    }

    //验证统一社会信用代码
    @RequestMapping(value = "/hls/bp/creditCode/query")
    @ResponseBody
    public ResponseData validateCredit(String enterpriseUnifiedCreditCode) {
        return new ResponseData(service.validateCredit(enterpriseUnifiedCreditCode));
    }

    //证件号
    @RequestMapping(value = "/hls/bp/cardNo/query")
    @ResponseBody
    public ResponseData validateCardNo(String idCardNo) {
        return new ResponseData(service.validateCardNo(idCardNo));
    }

    //首页报表数据
    @RequestMapping(value = "/hls/bp/count/query")
    @ResponseBody
    public ResponseData selectBpCount() {
        return new ResponseData(service.selectBpCount());
    }

    @RequestMapping(value = "/bp/master/detail/query")
    @ResponseBody
    public ResponseData masterDetailQuery(final HlsCusBpMaster hlsCusBpMaster) {
        return new ResponseData(service.masterDetailQuery(hlsCusBpMaster));
    }

    @RequestMapping(value = "/bp/master/bpid/query")
    @ResponseBody
    public ResponseData selectHlsBpMaster(final HlsCusBpMaster hlsCusBpMaster) {
        return new ResponseData(service.selectHlsBpMaster(hlsCusBpMaster));
    }


    @RequestMapping(value = "/hls/bp/edit/query")
    @ResponseBody
    public ResponseData editQuery(Long bpId) {
        List<HlsCusBpMaster> list = service.editQuery(bpId);

        List<String> types = roleService.queryType(bpId);

        String type = "";

        for (int j = 0; j < types.size(); j++) {
            type += types.get(j) + "、";
        }

        if (type != null) {
            type = type.substring(0, type.length() - 1);
        }

        list.get(0).setBpType(type);

        return new ResponseData(list);
    }

    @RequestMapping(value = "/hls/bp/edit/query/lon")
    @ResponseBody
    public ResponseData editQueryLon(Long bpId) {
        List<HlsCusBpMaster> list = service.editQuery(bpId);

        List<HlsCusCshBank> bank = bankService.bankInfoQuery2(bpId);
        HlsCusCshBank cshBank = bank.stream().findFirst().orElseGet(HlsCusCshBank::new);
        list.get(0).setBankShortName(cshBank.getBankShortName());
        list.get(0).setStartActiveDate(cshBank.getStartActiveDate());
        list.get(0).setEndActiveDate(cshBank.getEndActiveDate());
        list.get(0).setOrgType(cshBank.getOrgType());
        list.get(0).setBankCode(cshBank.getBankCode());

        return new ResponseData(list);
    }

    /**
     * 通过bpCode精确查找或者bpName模糊查询
     *
     * @return
     */
    @RequestMapping("/hls/bp/master/queryInfo")
    @ResponseBody
    public ResponseData queryBpMasterInfos(final HlsCusBpMaster bpMaster,
                                           @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                           @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize,
                                           final HttpServletRequest request) {
        List<HlsCusBpMaster> list = service.queryByCodeOrName(bpMaster, page, pagesize);
        return new ResponseData(list);
    }

	@RequestMapping("/hls/bp/master/home/first/query")
	@ResponseBody
	public ResponseData homeFirstQuery(String bpCategory,String bpType, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		Long userId = requestContext.getUserId() ;
		List<HlsCusBpMaster> list = new ArrayList<>();
		if (!StringUtils.isEmpty(bpCategory) || !StringUtils.isEmpty(bpType)){
			list = service.homeFirstQuery(userId,bpCategory,bpType);
		}else {
			list = service.homeFirstQuery(userId);
		}
		return new ResponseData(list);
	}

    @RequestMapping("/hls/bp/master/home/secord/query")
    @ResponseBody
    public ResponseData homeSecordQuery() {
        List<HlsCusBpMaster> list = service.homeSecordQuery();
        return new ResponseData(list);
    }

	@RequestMapping(value = "/hls/bp/master/home/third/query")
	@ResponseBody
	public ResponseData homeThirdQuery( HttpServletRequest request,String bpCategory,String bpType){

		IRequest requestContext = createRequestContext(request);
		List<HlsCusBpMaster> list = new ArrayList<>();
		if (!StringUtils.isEmpty(bpCategory) || !StringUtils.isEmpty(bpType)){
			list = service.homeThirdQuery(requestContext.getUserId(),bpCategory,bpType);
		}else {
			list = service.homeThirdQuery(requestContext.getUserId());
		}

		return new ResponseData(list);
	}

	@RequestMapping("/hls/bp/master/home/four/query")
	@ResponseBody
	public ResponseData homeFourQuery(){
		List<HlsCusBpMaster> list = service.homeFourQuery();
		return new ResponseData(list);
	}

    @RequestMapping("/hls/bp/master/home/query")
    @ResponseBody
    public ResponseData homeQuery(String inputValue) {

/*		if("".equals(inputValue))
        {
			return new ResponseData();
		}*/
        List<HlsCusBpMaster> list = service.homeQuery(inputValue);
        return new ResponseData(list);

    }

    @RequestMapping("/hls/bp/master/home/query2")
    @ResponseBody
    public ResponseData homeQuery2(String inputValue) {

/*		if("".equals(inputValue))
		{
			return new ResponseData();
		}*/
        List<HlsCusBpMaster> list = service.homeQuery2(inputValue);
        return new ResponseData(list);

    }

    @RequestMapping("/hls/bp/master/count/query")
    @ResponseBody
    public ResponseData countQuery() {


        List<HlsCusBpMaster> list = service.countQuery();
        return new ResponseData(list);

    }

/*	@RequestMapping("/hls/bp/master/code/value/query")
	@ResponseBody
	public String bpCodeValueQuery(HttpServletRequest request,final HttpSession session){
		IRequest requestContext = createRequestContext(request);
		Map<String,String> params = new HashMap<String,String>();

		String value = codingRuleValuesService.getCodeRuleValue(requestContext,1L, params);

		return  value;

	}*/


    /**
	 *  回款查询控制器
	 * @param map
	 * @param page
	 * @param pagesize
	 * @param request
     * @return
     */
    @RequestMapping("/hls/bp/master/recieivedPayments")
    @ResponseBody
    public ResponseData recieivedPayments(HlsCusBpMaster map, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                          @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        Long bpId = map.getBpId();
        Map mp = new HashMap();
        if (bpId != null) {
            mp.put("bpId", bpId);
        }
        List<Map> maps = service.queryRecieivedPayments(mp, iRequest, pagesize, page);

        return new ResponseData(maps);
    }

    @RequestMapping("/hls/bp/master/invoicPageData")
    @ResponseBody
    public ResponseData invoicPageData(HlsCusBpMaster bp, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        return new ResponseData(service.invoicPageData(bp, iRequest, page, pagesize));
    }

    @RequestMapping("/hls/bp/master/invoicMessage")
    @ResponseBody
    public ResponseData invoicMessage(@RequestBody Map bp, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                      @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
        IRequest iRequest = createRequestContext(request);
        List<Map> map = service.invoicMessage(bp);
        return new ResponseData(map);
    }

    /**
     * 授信机构首页各机构数量 包括占比，当月新增等数据
     */
    @RequestMapping(value = "/hls/bp/master/getAllCreditCount")
    @ResponseBody
    public ResponseData getAllCreditCount(@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                          @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.getAllCreditCount(requestContext));
    }

    @RequestMapping(value = "/hls/bp/master/lov")
    @ResponseBody
    public ResponseData queryBpLov(@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HlsCusBpMaster bp) {
        return new ResponseData(service.queryBpLov(bp));
    }

    @RequestMapping(value = "/hls/bp/master/type/lov")
    @ResponseBody
    public ResponseData queryBpTypeLov(@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HlsCusBpMaster bp) {
        return new ResponseData(service.queryBpByBpType(bp));
    }


}
