package hls.core.hls.bp.controllers;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.hls.bp.dto.HlsCusBpMasterRelation;
import hls.core.hls.bp.service.HlsBpMasterRelationService;

@Controller
public class HlsBpMasterRelationController extends BaseController
{
	@Autowired
	private HlsBpMasterRelationService service;
	
	
	@RequestMapping(value = "/hls/bp/relation/query")
	@ResponseBody
	public ResponseData query(final HlsCusBpMasterRelation bpMasterRelation, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.queryAll(requestContext, bpMasterRelation, page, pagesize));
	}
	
	@RequestMapping(value = "/hls/bp/relation/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody final List<HlsCusBpMasterRelation> bpMasterRelations, final BindingResult result,
                               final HttpServletRequest request){
		 	getValidator().validate(bpMasterRelations, result);
	        if (result.hasErrors()) {
	            ResponseData responseData = new ResponseData(false);
	            responseData.setMessage(getErrorMessage(result, request));
	            return responseData;
	        }
	        IRequest requestContext = createRequestContext(request);
	        
	        if(bpMasterRelations != null)
	        {
	        	for(HlsCusBpMasterRelation bpMasterRelation :bpMasterRelations)
	        	{
		        	Long relationBpId = service.queryBpId(bpMasterRelation.getBpCode(),bpMasterRelation.getBpName());
		        	
		        	bpMasterRelation.setRelationBpId(relationBpId);
	        	}
	        }
	        
	        List<HlsCusBpMasterRelation> datas = service.batchRelationUpdate(requestContext, bpMasterRelations);
	        
	        return new ResponseData(datas);
	}
	
	@RequestMapping(value="/hls/bp/relation/remove")
	@ResponseBody
	public ResponseData remove(@RequestBody List<HlsCusBpMasterRelation> lists){
		service.batchRelationDelete(lists);
		return new ResponseData();
	}
	
	@RequestMapping(value = "/hls/bp/relationType/query")
	@ResponseBody
	public ResponseData queryType(Long bpId){
		return new ResponseData(service.selectRelationType(bpId));
	}
	
	@RequestMapping(value = "/hls/bp/relationCode/query")
	@ResponseBody
	public ResponseData queryCode(Long bpId){
		return new ResponseData(service.selectRelationCode(bpId));
	}

	@RequestMapping(value = "/hls/bp/relation/selectBpRelationType")
	@ResponseBody
	public ResponseData selectBpRelationType(final HlsCusBpMasterRelation bpMasterRelation,  final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.selectBpRelationType(requestContext,bpMasterRelation));
	}
	
}
