package hls.core.hls.bp.controllers;

import java.io.*;
import java.net.URLEncoder;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.hand.hap.attachment.exception.AttachmentException;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hand.hap.attachment.FileInfo;
import com.hand.hap.attachment.UpConstants;
import com.hand.hap.attachment.Uploader;
import com.hand.hap.attachment.UploaderFactory;
import com.hand.hap.attachment.controllers.AttachmentController;
import com.hand.hap.attachment.dto.AttachCategory;
import com.hand.hap.attachment.dto.Attachment;
import com.hand.hap.attachment.dto.SysFile;
import com.hand.hap.attachment.exception.StoragePathNotExsitException;
import com.hand.hap.attachment.exception.UniqueFileMutiException;
import com.hand.hap.attachment.service.IAttachCategoryService;
import com.hand.hap.attachment.service.ISysFileService;
import com.hand.hap.core.BaseConstants;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.util.UploadUtil;
import com.hand.hap.security.TokenUtils;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.hls.bp.dto.HlsCusSysAttachment;
import hls.core.hls.bp.service.HlsSysAttachmentService;

@Controller
public class HlsSysAttachmentController extends BaseController
{
	@Autowired
	private HlsSysAttachmentService service;
    @Autowired
    private IAttachCategoryService categoryService;
	
    @Autowired
    private ISysFileService fileService;

    @Autowired
    private MessageSource messageSource;
    
    @Autowired
    private ObjectMapper objectMapper;
    
    /**
     * 提示信息名.
     */
    private static final String MESSAGE_NAME = "message";
    /**
     * 提示信息 name.
     **/
    private static final String INFO_NAME = "info";
    /**
     * file对象名.
     */
    private static final String FILE_NAME = "file";
    
    private static Logger logger = LoggerFactory.getLogger(AttachmentController.class);
	
	@RequestMapping(value = "/hls/bp/attachment/query")
	@ResponseBody
	public ResponseData query(final HlsCusSysAttachment sysAttachment, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.select(requestContext, sysAttachment, page, pagesize));
	}
	
	@RequestMapping(value = "/hls/bp/attachment/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody final List<HlsCusSysAttachment> sysAttachments, final BindingResult result,
                               final HttpServletRequest request){
		 	getValidator().validate(sysAttachments, result);
	        if (result.hasErrors()) {
	            ResponseData responseData = new ResponseData(false);
	            responseData.setMessage(getErrorMessage(result, request));
	            return responseData;
	        }
	        IRequest requestContext = createRequestContext(request);
	        return new ResponseData(service.batchUpdate(requestContext, sysAttachments));
	}

    @RequestMapping(value = "/hls/sys/attach/upload")
    public String upload(HttpServletRequest request, Locale locale, String contextPath)
            throws StoragePathNotExsitException, UniqueFileMutiException, JsonProcessingException, AttachmentException {
        Map<String, Object> response = new HashMap<String, Object>();
        Uploader uploader = UploaderFactory.getMutiUploader();
        uploader.init(request);
        String sourceType = uploader.getParams("sourceType");
        String sourceKey = uploader.getParams("sourceKey");
        response.put("success", false);
        if (StringUtils.isBlank(sourceType) || StringUtils.isBlank(sourceKey)) {
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.typeorkey_empty", null, locale));// TYPEORKEY_EMPTY
            response.put(INFO_NAME, messageSource.getMessage("hap.typeorkey_empty", null, locale));
            return objectMapper.writeValueAsString(response);
        }
        IRequest requestContext = createRequestContext(request);
        // 设置上传参数
        AttachCategory category = categoryService.selectAttachByCode(requestContext,sourceType);
        UploadUtil.initUploaderParams(uploader, category);
        // TYPE 设置不对
        if (category == null) {
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.type_error", null, locale));
            response.put(INFO_NAME, messageSource.getMessage("hap.type_error", null, locale));
            return objectMapper.writeValueAsString(response);
        }
        List<FileInfo> fileInfos = uploader.upload();
        // 出错了
        if (!UpConstants.SUCCESS.equals(fileInfos.get(0).getStatus())) {
            response.put(MESSAGE_NAME, fileInfos.get(0).getStatus());
            response.put(INFO_NAME, fileInfos.get(0).getStatus());
            return objectMapper.writeValueAsString(response);
        } else {
            response.put("success", true);
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.upload_success", null, locale));// uploader.getStatus()
        }
        // WebUploader 每次只上传一个文件
        FileInfo f = fileInfos.get(0);
        try {
            SysFile sysFile = UploadUtil.genSysFile(f, requestContext.getUserId(), requestContext.getUserId());
            Attachment attach = UploadUtil.genAttachment(category, sourceKey, requestContext.getUserId(),
                    requestContext.getUserId());
            // 分类如果是唯一类型
            if (BaseConstants.YES.equals(category.getIsUnique())) {
                sysFile = fileService.updateOrInsertFile(requestContext, attach, sysFile);
            } else {
                fileService.insertFileAndAttach(requestContext, attach, sysFile);
            }
            sysFile.setFilePath(null);
            TokenUtils.generateAndSetToken(TokenUtils.getSecurityKey(request.getSession(false)), sysFile);
            response.put(FILE_NAME, sysFile);
        } catch (UniqueFileMutiException ex) {
            response.put("success", false);
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.mesg_unique", null, locale));
            return objectMapper.writeValueAsString(response);
        } catch (Exception e) {
            if (logger.isErrorEnabled()) {
                logger.error("database error", e);
            }
            File file = f.getFile();
            if (file.exists()) {
                file.delete();
            }
            response.put("success", false);
            response.put(MESSAGE_NAME, messageSource.getMessage("hap.database_error", null, locale));
            response.put(INFO_NAME, messageSource.getMessage("hap.database_error", null, locale));
        }
        return objectMapper.writeValueAsString(response);
    }

    @RequestMapping(value = "/hls/bp/attachment/batch/download")
    @ResponseBody
    public void batchDownload(HttpServletRequest request, HttpServletResponse response){
        String tmpFileName="bpMaster.zip";
        String FilePath="C:\\Users\\Administrator\\Desktop\\222";
        byte[] buffer=new byte[1024];
        String strZipPath = FilePath+"\\"+tmpFileName;
        try{
            ZipOutputStream out=new ZipOutputStream(new FileOutputStream( strZipPath));

            File file=new File(FilePath);
            File[] file1=file.listFiles();
            for(int i=0;i<file1.length;i++){
                FileInputStream fis=new FileInputStream(file1[i]);
                out.putNextEntry(new ZipEntry(file1[i].getName()));
                //设置压缩文件内的字符编码
                int len;
                while((len=fis.read(buffer))>0){
                    out.write(buffer,0,len);
                }
                out.closeEntry();fis.close();
            }
            out.close();
            this.downFile(response,FilePath,tmpFileName);
        }catch(Exception e){
            System.out.println("错误信息："+e.getMessage());
        }
    }

    private void downFile(HttpServletResponse response, String FilePath,String tmpFileName) {
        try{
            String path=FilePath+"\\"+tmpFileName;
            File file=new File(path);
            if(file.exists())
            {
                InputStream ins=new FileInputStream(path);
                BufferedInputStream bins=new BufferedInputStream(ins);
                // 放到缓冲流里面
                OutputStream outs=response.getOutputStream();
                //获取文件 输出IO流
                BufferedOutputStream bouts=new BufferedOutputStream(outs);
                response.setContentType("application/x-download");
                //设 置response内容的类型
                response.setHeader( "Content-disposition","attachment;filename=" + URLEncoder.encode(tmpFileName,"UTF-8"));
                //设置头 部信息 intbytesRead=0; byte[]buffer=newbyte[8192];
                //开始向网络传输文件流
                int bytesRead=0;
                byte[]buffer=new byte[8192];
                while((bytesRead=bins.read(buffer,0,8192))!=-1){
                    bouts.write(buffer,0,bytesRead);
                }
                bouts.flush();
                //这里一定要调用flush()方法ins.close();
                bins.close();
                outs.close();
                bouts.close();
            }else{
            } }catch(IOException e) {
                System.out.println("错误信息："+e.getMessage());
            }
    }


}
