package hls.core.hls.bp.controllers;

import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletRequest;

import hls.core.hls.bp.dto.HlsCusSysFile;
import hls.core.hls.bp.mapper.HlsCusSysFileMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.hand.hap.attachment.dto.SysFile;
import com.hand.hap.core.IRequest;
import com.hand.hap.security.TokenUtils;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.hls.bp.service.HlsSysFileService;

@Controller
public class HlsSysFileController extends BaseController {
	@Autowired
	private HlsSysFileService service;
	@Autowired
	private HlsCusSysFileMapper mapper;

	@RequestMapping(value = "/hls/bp/sys/file/query")
	@ResponseBody
	public ResponseData query(final HlsCusSysFile sysFile, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
		IRequest requestContext = createRequestContext(request);
		sysFile.setSourceType(sysFile.getSourceType().toUpperCase());
		List<HlsCusSysFile> list = service.queryAll(requestContext, sysFile, page, pagesize);
		String securityKey = TokenUtils.getSecurityKey(request.getSession());
		SysFile file = new SysFile();
		for (HlsCusSysFile f : list) {
			file.setFileId(f.getFileId());
			f.set_token(TokenUtils.generateToken(securityKey, file));
		}
		return new ResponseData(list);
	}

	@RequestMapping(value = "/hls/bp/sys/file/uk/query")
	@ResponseBody
	public ResponseData queryUk(final HlsCusSysFile sysFile, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
		IRequest requestContext = createRequestContext(request);

		List<HlsCusSysFile> list = service.selectAllContent(requestContext,sysFile,page,pagesize);

		String securityKey = TokenUtils.getSecurityKey(request.getSession());
		SysFile file = new SysFile();
		for (HlsCusSysFile f : list) {
			file.setFileId(f.getFileId());
			f.set_token(TokenUtils.generateToken(securityKey, file));
		}
		return new ResponseData(list);
	}

	@RequestMapping(value = "/hls/bp/sys/file/con/query")
	@ResponseBody
	public ResponseData queryConTemplete(final HlsCusSysFile sysFile, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
								@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
		IRequest requestContext = createRequestContext(request);

		List<HlsCusSysFile> list = service.selectAllConContent(requestContext,sysFile,page,pagesize);

		String securityKey = TokenUtils.getSecurityKey(request.getSession());
		SysFile file = new SysFile();
		for (HlsCusSysFile f : list) {
			file.setFileId(f.getFileId());
			f.set_token(TokenUtils.generateToken(securityKey, file));
		}
		return new ResponseData(list);
	}


	@RequestMapping(value = "/hls/bp/sys/file/uka/query")
	@ResponseBody
	public ResponseData queryUka(final HlsCusSysFile sysFile, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request) {
		IRequest requestContext = createRequestContext(request);
		sysFile.setSourceType(sysFile.getSourceType().toUpperCase());
		List<HlsCusSysFile> list = service.queryUka(requestContext,sysFile,page,pagesize);

		String securityKey = TokenUtils.getSecurityKey(request.getSession());
		SysFile file = new SysFile();
		for (HlsCusSysFile f : list) {
			file.setFileId(f.getFileId());
			f.set_token(TokenUtils.generateToken(securityKey, file));
		}
		return new ResponseData(list);
	}

	@RequestMapping(value = "/hls/bp/sys/file/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody final List<HlsCusSysFile> sysFiles, final BindingResult result,
                               final HttpServletRequest request) {
		getValidator().validate(sysFiles, result);
		if (result.hasErrors()) {
			ResponseData responseData = new ResponseData(false);
			responseData.setMessage(getErrorMessage(result, request));
			return responseData;
		}
		IRequest requestContext = createRequestContext(request);
		for(HlsCusSysFile sysFile:sysFiles){
			if(sysFile.getSourceType() != null)
				sysFile.setSourceType(sysFile.getSourceType().toUpperCase());
		}
		return new ResponseData(service.batchUpdate(requestContext, sysFiles));
	}
	@RequestMapping(value = "/hls/bp/sys/file/queryById")
	@ResponseBody
	public ResponseData queryById(HlsCusSysFile sysFile,HttpServletRequest request){
		if(sysFile.getSourceKey()!=null){
			sysFile.setSourceType(sysFile.getSourceType().toUpperCase());
		}
		HlsCusSysFile hlsCusSysFile = mapper.selectByPrimaryKey(sysFile);
		List<HlsCusSysFile> list = new ArrayList<HlsCusSysFile>();
		list.add(hlsCusSysFile);
		return new ResponseData(list);
	}
}
