package hls.core.hls.bp.crawler;

import edu.uci.ics.crawler4j.crawler.Page;
import edu.uci.ics.crawler4j.crawler.WebCrawler;
import edu.uci.ics.crawler4j.parser.HtmlParseData;
import edu.uci.ics.crawler4j.parser.ParseData;
import edu.uci.ics.crawler4j.url.WebURL;
import hls.core.hls.bp.dto.HlsCusBpMasterBadRecord;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by yang on 2016/11/28.
 */
public class Crawler extends WebCrawler {

    private static Pattern pattern = Pattern.compile("http://www.police.sh.cn/shga/wzXxfbZfgkml/pagaList\\?[a-zA-Z0-9=]*&page[12]=\\d+&type=([12])");
    private static String urlTemplate = "http://www.police.sh.cn/shga/wzXxfbZfgkml/pagaList?pa=";
    private static Logger logger = LoggerFactory.getLogger(Crawler.class);
    private static List<String> visitedUrl = new ArrayList<String>();
    private static boolean noCompanyInfo = true;
    private static final int COMPANY_INFO_LENGTH = 11; // 网站表格中公司信息的列数
    private static List<HlsCusBpMasterBadRecord> records = new ArrayList<>();

    public static List<HlsCusBpMasterBadRecord> getRecords() {
        return records;
    }

    public static void clearRecords(){
        if(records != null && records.size() > 0){
            records.clear();
        }
    }

    public static void setNoCompanyInfo(boolean bool){
        noCompanyInfo = bool;
    }

    @Override
    public boolean shouldVisit(Page referringPage, WebURL url) {
        if (pattern.matcher(url.getURL()).matches() && !visitedUrl.contains(url.getPath())) {
            logger.info("match:" + url.getURL());
            visitedUrl.add(url.getPath());
            return true;
        }
        return false;
    }

    @Override
    public void visit(Page page) {
        int type = 2;
        String tableId = "#chuxian1";
        ParseData parseData = page.getParseData();

        if (parseData instanceof HtmlParseData) {
            HtmlParseData hpd = (HtmlParseData) parseData;
            String html = hpd.getHtml();
            Document doc = Jsoup.parse(html);
            Elements select = doc.select(tableId);
            if (!select.isEmpty()) {
                Elements trs = select.select(".jiequ_");
                HlsCusBpMasterBadRecord record = null;
                for (Element tr : trs) {
                    Elements tds = tr.select("td");
                    if(tds.size() >= COMPANY_INFO_LENGTH) {
                        // 需要依次对公司信息设置
                        record = new HlsCusBpMasterBadRecord();
                        String order = tds.get(0).text();
                        String docNumber = tds.get(1).text();
                        String companyName = tds.get(2).text();
                        String representName = tds.get(3).text();
                        String code = tds.get(4).text();
                        String caseReason = tds.get(5).text();
                        String punishBasis = tds.get(6).text();
                        String result = tds.get(7).text();
                        String policeDepart = tds.get(8).text();
                        String date = tds.get(9).text();
                        String detailUrl = tds.get(10).text();

                        Elements as = tr.select("a");
                        if (!as.isEmpty()) {
                            detailUrl = as.first().attr("href");
                        }
                        record.setRecordOrder(order);
                        record.setDocNumber(docNumber);
                        record.setCompanyName(companyName);
                        record.setRepresentName(representName);
                        record.setCode(code);
                        record.setCaseReason(caseReason);
                        record.setPunishBasis(punishBasis);
                        record.setPunishResult(result);
                        record.setPoliceDepart(policeDepart);
                        record.setDate(date);
                        record.setDetailUrl(detailUrl);

                        records.add(record);
                    }
                }
                if (noCompanyInfo) {
                    addCompanyPageUrlList(html);
                }
            }
        }
    }

    private void addCompanyPageUrlList(String html){
        Pattern page1Pat = Pattern.compile(".*initPageBar\\((\\d+),\\s*(\\d+),\\s*(\\d+),\\s*(\\w+),\\s*\"(page1)\"\\);.*");
        Pattern pa1 = Pattern.compile("var\\s*pa=\"(\\w+)\";");
        Matcher page1Mat = page1Pat.matcher(html);
        if(page1Mat.find()){ // 公司信息页
            String pa1Str = "";
            Matcher matcher1 = pa1.matcher(html);
            if(matcher1.find()){
                pa1Str = matcher1.group(1);
            }
            int start = Integer.valueOf(page1Mat.group(1));
            int total = Integer.valueOf(page1Mat.group(2));
            for(int i = start; i<=total; i++){
                String url = urlTemplate+pa1Str+"&page="+i+"&type=2";
                logger.info("found: "+url);
                getMyController().addSeed(url);
            }
            noCompanyInfo = false;
        }
    }
}
