package hls.core.hls.bp.service.impl;

import java.util.Date;
import java.util.List;

import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.bp.service.IBpMasterEventService;
import hls.core.hls.bp.service.IBpMasterUpdateService;
import hls.core.hls.mapper.HlsUserRelationshipMapper;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.service.SysNoticePushService;
import hls.core.sys.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.hls.bp.mapper.HlsBpMasterMapper;
import hls.core.hls.bp.service.HlsBpMasterEventService;
import hls.core.hls.bp.service.HlsBpMasterService;
import hls.core.hls.dto.HlsDocAuthority;
import hls.core.hls.dto.HlsSystemEvent;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsUserRelationship;
import hls.core.hls.service.HlsSystemEventService;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.hls.service.IHlsDocAuthorityService;
import hls.core.hls.service.IHlsUserRelationshipService;


@Service
@Transactional
public class HlsBpMasterEventServiceImpl extends BaseServiceImpl<HlsCusBpMaster> implements HlsBpMasterEventService {

    @Autowired
    private HlsCusBpMasterMapper bpCusMasterMapper;
    @Autowired
    private HlsSystemEventService hlsSystemEventService;
    @Autowired
    private IHlsDocAuthorityService docAuthorityService;
    @Autowired
    private IHlsUserRelationshipService hlsUserRelationshipService;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;
    @Autowired
    private SysNoticePushService sysNoticePushService;
    @Autowired
    private HlsUserRelationshipMapper userRelationshipMapper;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private HlsBpMasterService hlsBpMasterService;
    @Autowired
    private SysEventService sysEventService;

    @Autowired(required = false)
    private IBpMasterEventService iBpMasterEventService;

    private static final String HLS_BP_MASTER_CREATE = "创建了商业伙伴";
    private static final String HLS_BP_MASTER_MODIFY = "修改了商业伙伴";
    private static final String HLS_BP_MASTER_DELETE = "删除了商业伙伴";
    private static final String HLS_BP_MASTER_COLLECT = "收藏了商业伙伴";
    private static final String HLS_BP_MASTER_UNCOLLECT = "取消了商业伙伴收藏";

    private static String documentCategory = "HLS_BP_MASTER";
    private static String URL = "bp/BP100/bp_master_index.html";
    private static String sourceModule = "FND3";
    private static String relationType = "P2D";

    /*
     * (non-Javadoc)
     *
     * @see
     * hls.core.hls.bp.service.HlsBpMasterEventService#bpEvent(java.lang.String,
     * java.lang.Long, java.lang.Long)
     */
    @Override
    public void bpEvent(IRequest iRequest, String eventCode, Long bpId) {
        String message = "";
        String noticeTitle = "";
        HlsCusBpMaster bpMaster = new HlsCusBpMaster();
        bpMaster.setBpId(bpId);
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        List<HlsCusBpMaster> list = hlsBpMasterService.selectHlsBpMasterById(bpMaster);

        if (iBpMasterEventService != null) {
            iBpMasterEventService.beforeUpdate(iRequest, eventCode, bpId);
        }
        switch (eventCode) {
            case "HLS_BP_MASTER.CREATE":
                message = sysUser.getDescription() + HLS_BP_MASTER_CREATE + '【' + list.get(0).getBpName() + '】';
                noticeTitle = "商业伙伴创建";
                break;
            case "HLS_BP_MASTER.MODIFY":
                message = sysUser.getDescription() + HLS_BP_MASTER_MODIFY + '【' + list.get(0).getBpName() + '】';
                noticeTitle = "商业伙伴修改";
                break;
            case "HLS_BP_MASTER.DELETE":
                message = sysUser.getDescription() + HLS_BP_MASTER_DELETE + '【' + list.get(0).getBpName() + '】';
                noticeTitle = "商业伙伴删除";
                break;
            case "HLS_BP_MASTER.COLLECT":
                message = sysUser.getDescription() + HLS_BP_MASTER_COLLECT + '【' + list.get(0).getBpName() + '】';
                noticeTitle = "商业伙伴收藏";
                break;
            case "HLS_BP_MASTER.UNCOLLECT":
                message = sysUser.getDescription() + HLS_BP_MASTER_UNCOLLECT + '【' + list.get(0).getBpName() + '】';
                noticeTitle = "商业伙伴取消收藏";
                break;
            default:
                break;
        }
        Long userId = iRequest.getUserId();
        String documentType = "";
        bpMaster = bpCusMasterMapper.selectHlsBpMasterById(bpMaster).get(0);
        documentType = bpMaster.getBpClass();
        // 事件表
        HlsSystemEvent event = new HlsSystemEvent();
        event.setEventCode(eventCode);
        event.setSourceDocumentId(bpId);
        event.setEventUserId(userId);
        event.setSourceDocumentCategory(documentCategory);
        event.setLastUpdatedBy(userId);
        event.setLastUpdateDate(new Date());
        // 系统通知
        HlsSystemNotice hlsSystemNotice = new HlsSystemNotice();
        hlsSystemNotice.setNotice_title(noticeTitle);
        hlsSystemNotice.setNotice_message(message);
        hlsSystemNotice.setNotice_datetime(new Date());
        hlsSystemNotice.setNotice_type("NOTICE");
        hlsSystemNotice.setNotice_url(URL);
        hlsSystemNotice.setNotice_level(2L);
        hlsSystemNotice.setSource_module(sourceModule);
        hlsSystemNotice.setSource_event(eventCode);
        hlsSystemNotice.setSource_document_category(documentCategory);
        hlsSystemNotice.setSource_document_id(bpId);
        hlsSystemNotice.setSource_user_id(iRequest.getUserId());
        hlsSystemNotice.setLastUpdatedBy(userId);
        hlsSystemNotice.setLastUpdateDate(new Date());
//		hlsSystemNotice.setRead_flag("Y");
        //权限
        HlsDocAuthority authority = new HlsDocAuthority();
        authority.setUserId(userId);
        authority.setDocumentCategory(documentType);// 单据类型
        authority.setDocumentId(bpId);
        authority.setAuthorityType("2");// 1查询，2，修改
        authority.setSourceModule(sourceModule);
        authority.setSourceEvent(eventCode);
        authority.setSourceDocumentCategory(documentCategory);
        authority.setSourceDocumentId(bpId);
        authority.setSourceUserId(iRequest.getUserId());
        authority.setLastUpdatedBy(userId);
        authority.setLastUpdateDate(new Date());

        // 关系
        HlsUserRelationship hlsUserRelationship = new HlsUserRelationship();
        hlsUserRelationship.setUserId(userId);
        hlsUserRelationship.setRelationType(relationType);
        hlsUserRelationship.setSourceModule(sourceModule);
        hlsUserRelationship.setSourceEvent(eventCode);
        hlsUserRelationship.setSourceDocumentCategory(documentCategory);
        hlsUserRelationship.setSourceDocumentId(bpId);
        hlsUserRelationship.setSourceUserId(iRequest.getUserId());
        hlsUserRelationship.setLastUpdatedBy(userId);
        hlsUserRelationship.setLastUpdateDate(new Date());

        hlsSystemEventService.insert(iRequest, event);
        if (eventCode.equals("HLS_BP_MASTER.COLLECT")) {
            //docAuthorityService.insert(iRequest, authority );
            if ("P2P".equalsIgnoreCase(hlsUserRelationship.getRelationType())) {
                hlsUserRelationship.setRelationUserId(iRequest.getUserId());

                HlsUserRelationship userRelationship = new HlsUserRelationship();
                userRelationship.setUserId(hlsUserRelationship.getUserId());
                userRelationship.setRelationType(hlsUserRelationship.getRelationType());
                userRelationship.setRelationUserId(hlsUserRelationship.getRelationUserId());
                List<HlsUserRelationship> list2 = userRelationshipMapper.select(userRelationship);
                if (list2.size() == 0) {
                    hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship);
                }
            } else if ("P2D".equalsIgnoreCase(hlsUserRelationship.getRelationType())) {
                hlsUserRelationship.setRelationDocumentCategory(documentCategory);// 关联单据类别
                hlsUserRelationship.setRelationDocumentId(bpId); // 关系单据ID

                HlsUserRelationship userRelationship1 = new HlsUserRelationship();
                userRelationship1.setUserId(hlsUserRelationship.getUserId());
                userRelationship1.setRelationType(hlsUserRelationship.getRelationType());
                userRelationship1.setSourceDocumentCategory(hlsUserRelationship.getSourceDocumentCategory());
                userRelationship1.setSourceDocumentId(hlsUserRelationship.getSourceDocumentId());
                List<HlsUserRelationship> lists = userRelationshipMapper.select(userRelationship1);
                if (lists.size() == 0) {
                    hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship);
                }
            }
        } else if (eventCode.equals("HLS_BP_MASTER.UNCOLLECT")) {
            hlsUserRelationshipService.uncollectByPara(hlsUserRelationship);
        }
        hlsSystemNoticeService.insert(iRequest, hlsSystemNotice);
        sysEventService.setNoticeCache(hlsSystemNotice);
        if (iBpMasterEventService != null) {
            iBpMasterEventService.afterUpdate(iRequest, eventCode, bpId);
        }
    }
}
