package hls.core.hls.bp.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.csh.dto.HlsCusCshBank;
import hls.core.csh.service.CshBankService;
import hls.core.fnd.dto.FndCompany;
import hls.core.fnd.mapper.FndCompanyMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.bp.dto.HlsBpMaster;
import hls.core.hls.bp.dto.HlsBpMasterRole;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.hls.bp.mapper.HlsCusBpMasterRoleMapper;
import hls.core.hls.bp.service.HlsBpMasterCompanyService;
import hls.core.hls.bp.service.HlsBpMasterRoleService;
import hls.core.hls.bp.service.HlsBpMasterService;
import hls.core.hls.bp.service.IBpMasterUpdateService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class HlsBpMasterServiceImpl extends BaseServiceImpl<HlsCusBpMaster> implements HlsBpMasterService {


    @Autowired
    private SysEventService sysEventService;

    @Autowired(required = false)
    private IBpMasterUpdateService iBpMasterUpdateService;

    @Autowired
    HlsCusBpMasterMapper mapper;
    @Autowired
    HlsCusBpMasterRoleMapper roleMapper;
    @Autowired
    HlsBpMasterRoleService roleService;
    @Autowired
    CshBankService bankService;
    @Autowired
    HlsBpMasterRoleService service;
    @Autowired
    HlsBpMasterCompanyService companyService;
    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private FndCompanyMapper fndCompanyMapper;

    @Override
    public List<HlsCusBpMaster> queryById(Long bpId) {
        return mapper.queryById(bpId);
    }

    @Override
    public List<HlsCusBpMaster> queryBpIds(IRequest requestContext, HlsCusBpMaster bpMaster, int page, int pagesize, Long companyId, Long userId) {
        PageHelper.startPage(page, pagesize);

        String[] bpTypes = null;
        String[] bpClasses = null;
        String[] flag = null;
        String[] collect = null;
        if (bpMaster.getBpType() != null) {
            bpTypes = bpMaster.getBpType().split("、");
        }
        if (bpMaster.getBpClass() != null) {
            bpClasses = bpMaster.getBpClass().split("、");
        }
        if (bpMaster.getEnabledFlag() != null) {
            flag = bpMaster.getEnabledFlag().split("、");
        }
        if (bpMaster.getCollectFlag() != null) {
            collect = bpMaster.getCollectFlag().split("、");
        }

        Map<String, Object> params = new HashMap<String, Object>(8);

        params.put("bpTypes", bpTypes);
        params.put("bpClasses", bpClasses);
        params.put("bpCode", bpMaster.getBpCode());
        params.put("bpFullName", bpMaster.getBpName());
        params.put("bpName", bpMaster.getExtraNam());
        params.put("collect", collect);
        params.put("flag", flag);
        //params.put("companyId", companyId);
        params.put("userId", userId);

        return mapper.queryBpIds(params);
    }

    @Override
    public boolean validateCredit(String enterpriseUnifiedCreditCode) {

        String bpCode = mapper.validateCredit(enterpriseUnifiedCreditCode);

        if (bpCode == null) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public boolean validateCardNo(String idCardNo) {

        String bpCode = mapper.validateCardNo(idCardNo);

        if (bpCode == null) {
            return true;
        } else {
            return false;
        }
    }

    public void setAuthorityRuleString(IRequest requestContext, HlsCusBpMaster bpMaster, Long companyId){
        FndCompany fndCompany = new FndCompany();
        fndCompany.setCompany_id(companyId);
        fndCompany.setCompanyId(companyId);
        fndCompany = fndCompanyMapper.selectByPrimaryKey(fndCompany);
        String userName = requestContext.getUserName();
        String positionCode = requestContext.getAttribute("positionCode")==null?"":requestContext.getAttribute("positionCode");
        String unitCode = requestContext.getAttribute("unitCode")==null?"":requestContext.getAttribute("unitCode");
        bpMaster.setAuthorityRuleString('"' + fndCompany.getCompany_code() + '"' + "." + '"'+unitCode+ '"' + "." + '"' + '"' + "." + '"'+'"' + "." + '"' + '"' + "." + '"' + positionCode + '"' + "." + '"' + userName + '"');
    }

    @Override
    public List<HlsCusBpMaster> masterUpdate(IRequest requestContext, HlsCusBpMaster bpMaster, Long companyId) {

        Map<String, String> params = new HashMap<String, String>();
        if (iBpMasterUpdateService != null) {
            iBpMasterUpdateService.beforeUpdate(requestContext, bpMaster, companyId);
        }
        if ("add".equals(bpMaster.get__status())) {
            String value = codingRuleValuesService.getCodeRuleValue(requestContext, "HLS_BP_MASTER", bpMaster.getBpClass(), bpMaster.getBpClass(), params);

            bpMaster.setBpCode(value);
            setAuthorityRuleString(requestContext,bpMaster,companyId);

            self().insertSelective(requestContext, bpMaster);
            //商业伙伴创建事件
            sysEventService.createEvent(requestContext, bpMaster.getBpId(), "HLS_BP_MASTER", bpMaster.getBpClass(), null, SysEventCodeUtil.PROPERTY_HLS_BP_MASTER_CREATE);
//			hlsBpMasterEventService.bpEvent(requestContext, "HLS_BP_MASTER.CREATE", bpMaster.getBpId());

            HlsBpMasterRole role = new HlsBpMasterRole();

            role.setBpId(bpMaster.getBpId());
            role.setBpType(bpMaster.getBpType());
            role.setEnabledFlag("Y");

            String bpCategory = service.selectBbpCategory(bpMaster.getBpType());
            role.setBpCategory(bpCategory);

            roleService.insertSelective(requestContext, role);


/*			HlsCusBpMasterCompany company = new HlsCusBpMasterCompany();

			company.setBpId(bpMaster.getBpId());
			company.setCompanyId(companyId);
			company.setEnabledFlag("Y");
			companyService.insertSelective(requestContext, company);*/
        }
        if ("update".equals(bpMaster.get__status())) {
            setAuthorityRuleString(requestContext,bpMaster,companyId);
            self().updateByPrimaryKeySelective(requestContext, bpMaster);
            //商业伙伴修改事件
            sysEventService.createEvent(requestContext, bpMaster.getBpId(), "HLS_BP_MASTER", bpMaster.getBpClass(), null, SysEventCodeUtil.PROPERTY_HLS_BP_MASTER_MODIFY);

//          hlsBpMasterEventService.bpEvent(requestContext, "HLS_BP_MASTER.MODIFY", bpMaster.getBpId());
        }
        if (iBpMasterUpdateService != null) {
            iBpMasterUpdateService.afterUpdate(requestContext, bpMaster, companyId);
        }
        List<HlsCusBpMaster> bpMasters = new ArrayList<HlsCusBpMaster>();
        bpMasters.add(bpMaster);

        return bpMasters;
    }


    @Override
    public List<HlsCusBpMaster> masterUpdate2(IRequest requestContext, HlsCusBpMaster bpMaster, Long companyId) {

        Map<String, String> params = new HashMap<String, String>();
        if ("add".equals(bpMaster.get__status())) {
            String value = codingRuleValuesService.getCodeRuleValue(requestContext, "HLS_BP_MASTER", bpMaster.getBpClass(), bpMaster.getBpClass(), params);

            bpMaster.setBpCode(value);
            setAuthorityRuleString(requestContext,bpMaster,companyId);
            bpMaster.setExtraNam(bpMaster.getBankShortName());
            self().insertSelective(requestContext, bpMaster);
            //商业伙伴创建事件
            sysEventService.createEvent(requestContext, bpMaster.getBpId(), "HLS_BP_MASTER", bpMaster.getBpClass(), null, SysEventCodeUtil.PROPERTY_HLS_BP_MASTER_CREATE);
//			hlsBpMasterEventService.bpEvent(requestContext, "HLS_BP_MASTER.CREATE", bpMaster.getBpId());

            HlsBpMasterRole role = new HlsBpMasterRole();

            role.setBpId(bpMaster.getBpId());
            role.setBpType(bpMaster.getBpType());
            role.setEnabledFlag("Y");

            String bpCategory = service.selectBbpCategory(bpMaster.getBpType());
            role.setBpCategory(bpCategory);

            roleService.insertSelective(requestContext, role);


            HlsCusCshBank bank = new HlsCusCshBank();
            bank.setBpId(bpMaster.getBpId());
            bank.setBankName(bpMaster.getBpName());
            bank.setBankShortName(bpMaster.getBankShortName());
            bank.setOrgType(bpMaster.getOrgType());
            bank.setBankCode(bpMaster.getBankCode());
            bank.setStartActiveDate(bpMaster.getStartActiveDate());
            bank.setEndActiveDate(bpMaster.getEndActiveDate());
            bank.setBankType(bpMaster.getBpType());
            bankService.insertSelective(requestContext, bank);
            Long bankId = bank.getBankId();
            bpMaster.setBankId(bankId);

        }
        if ("update".equals(bpMaster.get__status())) {
            bpMaster.setExtraNam(bpMaster.getBankShortName());
            self().updateByPrimaryKeySelective(requestContext, bpMaster);
            //商业伙伴修改事件
            sysEventService.createEvent(requestContext, bpMaster.getBpId(), "HLS_BP_MASTER", bpMaster.getBpClass(), null, SysEventCodeUtil.PROPERTY_HLS_BP_MASTER_MODIFY);

//          hlsBpMasterEventService.bpEvent(requestContext, "HLS_BP_MASTER.MODIFY", bpMaster.getBpId());
            setAuthorityRuleString(requestContext,bpMaster,companyId);
            self().updateByPrimaryKeySelective(requestContext, bpMaster);

            HlsBpMasterRole role = new HlsBpMasterRole();
            role.setBpId(bpMaster.getBpId());
            role.setBpType(bpMaster.getBpType());
            role.setEnabledFlag("Y");

            String bpCategory = service.selectBbpCategory(bpMaster.getBpType());
            role.setBpCategory(bpCategory);

            roleService.updateByPrimaryKeySelective(requestContext, role);


            HlsCusCshBank bank = new HlsCusCshBank();
            bank.setBpId(bpMaster.getBpId());
            bank.setBankName(bpMaster.getBpName());
            bank.setBankShortName(bpMaster.getBankShortName());
            bank.setOrgType(bpMaster.getOrgType());
            bank.setBankCode(bpMaster.getBankCode());
            bank.setStartActiveDate(bpMaster.getStartActiveDate());
            bank.setEndActiveDate(bpMaster.getEndActiveDate());
            bank.setBankId(bpMaster.getBankId());
            bank.setBankType(bpMaster.getBpType());
            bankService.updateByPrimaryKeySelective(requestContext, bank);
            Long bankId = bank.getBankId();
            bpMaster.setBankId(bankId);
        }

        List<HlsCusBpMaster> bpMasters = new ArrayList<HlsCusBpMaster>();
        bpMasters.add(bpMaster);

        return bpMasters;
    }

    @Override
    public String selectBpClass(String bpClass) {

        return mapper.selectBpClass(bpClass);
    }

    @Override
    public List<HlsCusBpMaster> selectBpCount() {
        // TODO Auto-generated method stub
        return mapper.selectBpCount();
    }

    @Override
    public List<HlsCusBpMaster> editQuery(Long bpId) {
        // TODO Auto-generated method stub
        return mapper.editQuery(bpId);
    }

    @Override
    public List<HlsCusBpMaster> masterDetailQuery(HlsCusBpMaster hlsCusBpMaster) {
        // TODO Auto-generated method stub
        return mapper.masterDetailQuery(hlsCusBpMaster);
    }

    @Override
    public List<HlsCusBpMaster> queryByCodeOrName(HlsCusBpMaster bpMaster, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        return mapper.queryByCodeOrName(bpMaster);
    }

    @Override
    public List<HlsCusBpMaster> selectHlsBpMaster(HlsCusBpMaster hlsCusBpMaster) {
        return mapper.selectHlsBpMaster(hlsCusBpMaster);
    }

    @Override
    public List<HlsCusBpMaster> homeFirstQuery(Long userId) {
        // TODO Auto-generated method stub
        return mapper.homeFirstQuery(userId);
    }

    @Override
    public List<HlsCusBpMaster> homeFirstQuery(Long userId,String bpCategory,String bpType) {
        // TODO Auto-generated method stub
        return mapper.homeFirstQuery2(userId,bpCategory,bpType);
    }

    @Override
    public List<HlsCusBpMaster> homeThirdQuery(Long userId,String bpCategory,String bpType) {
        // TODO Auto-generated method stub

        return mapper.homeThirdQuery2(userId,bpCategory,bpType);
    }

    @Override
    public List<HlsCusBpMaster> homeThirdQuery(Long userId) {
        // TODO Auto-generated method stub

        return mapper.homeThirdQuery(userId);
    }

    @Override
    public List<HlsCusBpMaster> homeFourQuery() {
        // TODO Auto-generated method stub
        return mapper.homeFourQuery();
    }

    @Override
    public List<HlsCusBpMaster> homeSecordQuery() {
        // TODO Auto-generated method stub
        return mapper.homeSecordQuery();
    }

    @Override
    public List<HlsCusBpMaster> homeQuery(String inputValue) {
        // TODO Auto-generated method stub
        return mapper.homeQuery(inputValue);
    }

    @Override
    public List<HlsCusBpMaster> homeQuery2(String inputValue) {
        // TODO Auto-generated method stub
        return mapper.homeQuery2(inputValue);
    }

    @Override
    public List<HlsCusBpMaster> countQuery() {
        // TODO Auto-generated method stub
        return mapper.countQuery();
    }

    /**
     * 回款查询
     *
     * @param map
     * @param iRequest
     * @param pagesize
     * @param page
     * @return
     */
    @Override
    public List<Map> queryRecieivedPayments(Map map, IRequest iRequest, int pagesize, int page) {
        PageHelper.startPage(page, pagesize);
        return mapper.queryRecieivedPayments(map);
    }

    @Override
    public List<HlsCusBpMaster> selectAllBpMaster(HlsCusBpMaster hlsCusBpMaster, HttpSession Session) {
        return mapper.selectAllBpMaster(hlsCusBpMaster);
    }

    @Override
    public List<HlsCusBpMaster> selectGeneralBpMaster(HlsCusBpMaster hlsCusBpMaster, HttpSession session) {
        return mapper.selectGeneralBpMaster(hlsCusBpMaster);
    }

    /**
     * 商业伙伴开票信息页面数据
     *
     * @param hlsCusBpMaster
     * @return
     */
    @Override
    public List<HlsCusBpMaster> invoicPageData(HlsCusBpMaster hlsCusBpMaster, IRequest iRequest, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);//分页
        List<HlsCusBpMaster> down = mapper.invoicDetail(hlsCusBpMaster);//开票记录

        return down;
    }

    /**
     * 商业伙伴开票信息
     *
     * @param map
     * @return
     */
    @Override
    public List<Map> invoicMessage(Map map) {
        List<Map> top = mapper.invoicMessage(map);//开票信息,
        return top;
    }

    @Override
    public List<HlsCusBpMaster> getTenantBpInfo(HlsCusBpMaster bpMaster) {
        return mapper.getTenantBpInfo(bpMaster);
    }

    @Override
    public List<HlsCusBpMaster> getGuarantorInfo(HlsCusBpMaster bpMaster) {
        return mapper.getGuarantorInfo(bpMaster);
    }

    @Override
    public List<HlsCusBpMaster> selectHlsBpMasterById(HlsCusBpMaster bpMaster) {
        return mapper.selectHlsBpMasterById(bpMaster);
    }

    @Override
    public List<Map> getAllCreditCount(IRequest iRequest) {
        return mapper.getAllCreditCount();
    }

    @Override
    public List<HlsCusBpMaster> queryBpLov(HlsCusBpMaster bpMaster) {
        return mapper.queryBpLov(bpMaster);
    }

    @Override
    public List<HlsCusBpMaster> masterCreditDetailQuery(HlsCusBpMaster bpMaster){
        return mapper.masterCreditDetailQuery(bpMaster);
    }

    @Override
    public List<HlsCusBpMaster> queryBpByBpType(HlsCusBpMaster bpMaster) {
        return mapper.queryBpByBpType(bpMaster);
    }
}
