package hls.core.hls.components;

import java.math.BigDecimal;
import java.util.Optional;

/**
 * Created by fjm on 2017/7/2.
 */
public class CalculateUtil {

    private CalculateUtil(){
        super();
    }

    private static final int DEF_DIV_SCALE = 2; //这个类不能实例化

    private static final int DEF_MUL_SCALE = 4;

    private static final int DEF_ADD_SCALE = 2;

    /**
     * 提供精确的加法运算。
     * @param v1 被加数
     * @param v2 加数
     * @return 两个参数的和
     */
    public static Double add(Double v1,Double v2){
        if (v1 == null)
            v1 = 0D;
        if (v2 == null)
            v2 = 0D;
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.add(b2).setScale(DEF_ADD_SCALE,BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    /**
     * 提供精确的减法运算。
     * @param v1 被减数
     * @param v2 减数
     * @return 两个参数的差
     */
    public static Double sub(Double v1,Double v2){
        if (v1 == null)
            v1 = 0D;
        if (v2 == null)
            v2 = 0D;
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.subtract(b2).setScale(DEF_MUL_SCALE,BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    /**
     * 提供精确的乘法运算。
     * @param v1 被乘数
     * @param v2 乘数
     * @return 两个参数的积
     */
    public static Double mul(Double v1,Double v2){
        if (v1 == null)
            v1 = 0D;
        if (v2 == null)
            v2 = 0D;
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.multiply(b2).setScale(DEF_MUL_SCALE,BigDecimal.ROUND_HALF_UP).doubleValue();
    }


    /**
     * 提供（相对）精确的除法运算，当发生除不尽的情况时，精确到
     * 小数点以后10位，以后的数字四舍五入。
     * @param v1 被除数
     * @param v2 除数
     * @return 两个参数的商
     */
    public static Double div(Double v1,Double v2){
        return div(v1,v2,DEF_DIV_SCALE);
    }
    /**
     * 提供（相对）精确的除法运算。当发生除不尽的情况时，由scale参数指
     * 定精度，以后的数字四舍五入。
     * @param v1 被除数
     * @param v2 除数
     * @param scale 表示表示需要精确到小数点以后几位。
     * @return 两个参数的商
     */
    public static Double div(Double v1,Double v2,int scale){
        if (v1 == null)
            v1 = 0D;
        if (v2 == null)
            v2 = 0D;
        if(scale<0){
            throw new IllegalArgumentException(
                    "The scale must be a positive integer or zero");
        }
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.divide(b2,scale,BigDecimal.ROUND_HALF_UP).doubleValue();
    }
}
