package hls.core.hls.controllers;

import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.exception.TokenException;
import com.hand.hap.excel.dto.ColumnInfo;
import com.hand.hap.excel.dto.ExportConfig;
import com.hand.hap.excel.service.IExportService;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.fnd.service.HLSIEmployeeService;
import hls.core.hls.dto.Division;
import hls.core.hls.mapper.DivisionMapper;
import hls.core.hls.service.DivisionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.List;

//import com.hand.hap.excel.service.IHapExcelImportService;


@Controller
public class DivisionController extends BaseController{
	
	@Autowired
	private DivisionService divisionService;
	
	@Autowired
    HLSIEmployeeService employeeService ;
	@Autowired
	IExportService excelService;

	@Autowired
	ObjectMapper objectMapper;
	
	
	@RequestMapping(value = "/fnd/division/query")
	@ResponseBody
	public ResponseData query(final Division division, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
            @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request,HttpSession session){
		IRequest requestContext = createRequestContext(request);
		division.setCompanyId((Long)session.getAttribute("companyId"));
		return new ResponseData(divisionService.selectList(division,page,pagesize));
	}

	@RequestMapping(value = "/fnd/division/export")
	public void createXLS(HttpServletRequest request, @RequestParam String config,
						  HttpServletResponse httpServletResponse,HttpSession session) {
		IRequest requestContext = createRequestContext(request);
		try {

			JavaType type = objectMapper.getTypeFactory().constructParametrizedType(ExportConfig.class,
					ExportConfig.class, Division.class, ColumnInfo.class);
			ExportConfig<Division, ColumnInfo> exportConfig = objectMapper.readValue(config, type);
			exportConfig.getParam().setCompanyId((Long)session.getAttribute("companyId"));
			excelService.exportAndDownloadExcel("hls.core.hls.mapper.DivisionMapper.selectList",
					exportConfig, request, httpServletResponse, requestContext);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@RequestMapping(value = "/fnd/division/submit")
	@ResponseBody
	public ResponseData submit(@RequestBody final List<Division> divisions, final BindingResult result,
            final HttpServletRequest request)throws TokenException {
		
//		 	getValidator().validate(divisions, result);
//	        if (result.hasErrors()) {
//	            ResponseData responseData = new ResponseData(false);
//	            responseData.setMessage(getErrorMessage(result, request));
//	            return responseData;
//	        }
        ResponseData rd = null;
	 	getValidator().validate(divisions, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage("此信息编号已经存在，请重新输入");
            return responseData;
        }
	       try {
               IRequest requestContext = createRequestContext(request);
               for (int i = 0; i < divisions.size(); i++) {
                   divisions.get(i).setCompanyId(requestContext.getCompanyId());
               }

               return new ResponseData(divisionService.batchUpdate(requestContext, divisions));
           }catch (Exception e) {
               rd = new ResponseData(false);
               rd.setMessage("产品线编码不可重复！");
               return rd;
           }
	}
	
	
	@RequestMapping(value="/fnd/division/remove")
	@ResponseBody
	public ResponseData remove(@RequestBody final List<Division> divisions, final BindingResult result,
            final HttpServletRequest request){
		divisionService.batchDelete(divisions);
		return new  ResponseData();
	}
	
	
}
