package hls.core.hls.controllers;

import hls.core.hls.components.Datasource2Json;
import hls.core.hls.dto.HlsDbDataSource;
import hls.core.hls.service.IHlsDbDataSourceService;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.List;

@Controller
public class HlsDbDataSourceController extends BaseController {

    @Autowired
    private IHlsDbDataSourceService service;

    @Autowired
    private Datasource2Json d2j;


    @RequestMapping(value = "/hls/db/data/source/query")
    @ResponseBody
    public ResponseData query(HlsDbDataSource dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.query(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/hls/db/data/source/submit")
    @ResponseBody
    public ResponseData update(HttpSession session, HttpServletRequest request, @RequestBody List<HlsDbDataSource> dto) {
        ResponseData rd = null;
        for(int i=0; i<dto.size(); i++){
            if(dto.get(i).getDataSourceCode() == null){
                rd = new ResponseData(false);
                rd.setMessage("数据来源代码为必填项！");

                return rd;
            }
            if(dto.get(i).getDescription() == null){
                rd = new ResponseData(false);
                rd.setMessage("描述为必填项！");

                return rd;
            }
            if(dto.get(i).getUsageType() == ""){
                rd = new ResponseData(false);
                rd.setMessage("数据源用途为必填项！");

                return rd;
            }
        }
        try{
            IRequest requestCtx = createRequestContext(request);
            Long companyId = (Long) session.getAttribute("companyId");
            for (HlsDbDataSource tar :
                    dto) {
                if (companyId != null) {
                    tar.setCompanyId(companyId);
                } else {
                    ResponseData rs = new ResponseData(false);
                    rs.setMessage("公司ID获取失败,请联系管理员");
                    return rs;
                }
            }
            rd = new ResponseData(service.batchUpdate(requestCtx, dto));
        }catch (Exception e){
            rd = new ResponseData(false);

            rd.setMessage("数据来源代码不可重复");
        }

        return rd;
    }

    @RequestMapping(value = "/hls/db/data/source/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsDbDataSource> dto) {
        //service.batchDelete(dto);
        service.deleteChild(dto);
        return new ResponseData();
    }

    @RequestMapping("/hls/db/data/source/json")
    @ResponseBody
    public String json(Long dataSourceId) {
        try {
            return d2j.executeSQL4Json(dataSourceId);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return "";
    }

    @RequestMapping("/hls/db/tables")
    @ResponseBody
    public ResponseData getAllTables() {
        List<String> allTables = d2j.getAllTables();
        return new ResponseData(allTables);
    }

    @RequestMapping("/hls/db/columns")
    @ResponseBody
    public ResponseData getAllColumns(String tableName) {
        List<String> allColumns = d2j.getAllColumns(tableName);
        return new ResponseData(allColumns);
    }

    @RequestMapping("/hls/db/combox")
    @ResponseBody
    public ResponseData comboxDs(HlsDbDataSource dto, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.dataSource( dto));
    }
}