package hls.core.hls.controllers;

import com.hand.hap.core.IRequest;
import com.hand.hap.excel.ExcelException;
import com.hand.hap.excel.service.IHapExcelImportService;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.hls.dto.BPRelation;
import hls.core.hls.service.BPRelationService;
import hls.core.hls.service.HlsExcelCommonService;
import hls.core.hls.service.HlsExcelService;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


@Controller
public class HlsExcelController extends BaseController {

    @Autowired
    HlsExcelService hlsExcelService;
    @Autowired
    HlsExcelCommonService hlsExcelCommonService;
    private Workbook wb;
    private Sheet sheet;

    @RequestMapping(value = "/hls/excel/import", method = RequestMethod.POST)
    public Map<String, Object> excelImport(HttpServletRequest request,String templateCode) throws IOException,ExcelException {
        IRequest iRequest = createRequestContext(request);
        Map<String, Object> response = new HashMap<String, Object>();
        response.put("success", false);
        // 在解析请求之前先判断请求类型是否为文件上传类型
        boolean isMultipart = ServletFileUpload.isMultipartContent(request);
        if (!isMultipart) {
            return null;
        }
        // 文件上传处理工厂
        FileItemFactory factory = new DiskFileItemFactory();

        // 创建文件上传处理器
        ServletFileUpload upload = new ServletFileUpload(factory);

        List items = null;
        try {
            items = upload.parseRequest(request);
        } catch (FileUploadException e) {
            response.put("success", false);
            response.put("message", "文件错误！");
            e.printStackTrace();
        }


        // 对所有请求信息进行判断
        Iterator iter = items.iterator();

        while (iter.hasNext()) {
            FileItem item = (FileItem) iter.next();
            // 信息为文件的格式
            if (!item.isFormField()) {
                try {
                    InputStream fs = item.getInputStream();
                    String filename = new String(item.getName());
                    //插入接口表 返回头表ID
                    Long headerId = hlsExcelService.loadExcel(iRequest, fs, templateCode, filename);
                    response.put("headerId", headerId);
                    response.put("message", "导入成功");
                    response.put("success", true);
                } catch (Exception e) {
                    response.put("success", false);
                    response.put("message", "导入失败！" + e.getMessage());
                    e.printStackTrace();
                }

            }
        }
        return response;
    }

    @RequestMapping(value = "/hls/common/excel/import", method = RequestMethod.POST)
    public Map<String, Object> commonImport(HttpServletRequest request) throws IOException,ExcelException {
        IRequest iRequest = createRequestContext(request);
        Map<String, Object> response = new HashMap<String, Object>();
        response.put("success", false);
        // 文件上传处理工厂
        FileItemFactory factory = new DiskFileItemFactory();
        // 创建文件上传处理器
        ServletFileUpload upload = new ServletFileUpload(factory);
        List items = null;
        try {
            items = upload.parseRequest(request);
        } catch (FileUploadException e) {
            response.put("success", false);
            response.put("message", "文件错误！");
            e.printStackTrace();
        }
        // 对所有请求信息进行判断
        Iterator iter = items.iterator();
        while (iter.hasNext()) {
            FileItem item = (FileItem) iter.next();
            // 信息为文件的格式
            if (!item.isFormField()) {
                try {
                    InputStream fs = item.getInputStream();
                    try {
                        wb = WorkbookFactory.create(fs);
                    } catch (InvalidFormatException e) {
                        e.printStackTrace();
                        throw new ExcelException((String) null, "不是正确的Excel文件！", (Object[]) null);
                    }
                    for (int i = 0; i < wb.getNumberOfSheets(); ++i) {
                        sheet = wb.getSheetAt(i);
                        String sheetName = sheet.getSheetName();
                        hlsExcelCommonService.loadExcel(fs, sheetName,sheet);
                    }
                    response.put("message", "导入成功");
                    response.put("success", true);
                } catch (Exception e) {
                    response.put("success", false);
                    response.put("message", "导入失败！" + e.getMessage());
                    e.printStackTrace();
                }
            }
        }
        return response;
    }

    @RequestMapping(value = "/hls/excel/importData", method = RequestMethod.POST)
    public Map<String, Object> excelImportForNormal(HttpServletRequest request) throws IOException,ExcelException {
        IRequest iRequest = createRequestContext(request);
        Map<String, Object> response = new HashMap<String, Object>();
        response.put("success", false);
        // 在解析请求之前先判断请求类型是否为文件上传类型
        boolean isMultipart = ServletFileUpload.isMultipartContent(request);
        if (!isMultipart) {
            return null;
        }
        // 文件上传处理工厂
        FileItemFactory factory = new DiskFileItemFactory();

        // 创建文件上传处理器
        ServletFileUpload upload = new ServletFileUpload(factory);

        List items = null;
        try {
            items = upload.parseRequest(request);
        } catch (FileUploadException e) {
            response.put("success", false);
            response.put("message", "文件错误！");
            e.printStackTrace();
        }


        // 对所有请求信息进行判断
        Iterator iter = items.iterator();

        while (iter.hasNext()) {
            FileItem item = (FileItem) iter.next();
            // 信息为文件的格式
            if (!item.isFormField()) {
                try {
                    InputStream fs = item.getInputStream();
                    String filename = new String(item.getName());
                    hlsExcelService.ImportData(iRequest,fs);//导入excel中的表格数据
                    response.put("message", "导入成功");
                    response.put("success", true);
                } catch (Exception e) {
                    response.put("success", false);
                    response.put("message", "导入失败！" + e.getMessage());
                    e.printStackTrace();
                }

            }
        }
        return response;
    }

}
