package hls.core.hls.job;

import com.hand.hap.job.AbstractJob;
import hls.core.hls.service.HlsDayEndService;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class HlsDayEndJob extends AbstractJob {
    private Logger logger = LoggerFactory.getLogger(HlsDayEndJob.class);
    @Autowired
    private HlsDayEndService hlsDayEndService;

    private Exception exception = null;
    private Object result;
    @Override
    public void safeExecute(JobExecutionContext context ) throws Exception {
        String dayEndDate = context.getMergedJobDataMap().getString("dayEndDate");
        String dayEndType = context.getMergedJobDataMap().getString("dayEndType");
        Long companyId = context.getMergedJobDataMap().getLong("companyId");
        Long contractId = context.getMergedJobDataMap().getLong("contractId");
        Long withdrawId = context.getMergedJobDataMap().getLong("withdrawId");
        Map<String, Object> param = new HashMap<>();
        if(dayEndDate == null || dayEndDate.equals("")){
            param.put("dayEndDate", new Date());
        }else{
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Date date = sdf.parse(dayEndDate);
            param.put("dayEndDate", date);
        }
        if(dayEndType == null || dayEndType.equals("")){
            param.put("dayEndType", "ALL");
        }else{
            param.put("dayEndType", dayEndType);
        }
        //先固定公司
        if(companyId == null || companyId.equals("")){
            param.put("companyId", 1);
        }else{
            param.put("companyId", companyId);
        }

        if(contractId == null || contractId.equals("")){
            param.put("contractId", "");
        }else{
            param.put("contractId", contractId);
        }

        if(withdrawId == null || withdrawId.equals("")){
            param.put("withdrawId", "");
        }else{
            param.put("withdrawId", withdrawId);
        }


        param.put("jobExecute", 1);
        try {
           hlsDayEndService.excEndDay(param);
        } catch (Exception e) {
            if (logger.isErrorEnabled()) {
                logger.error(e.getMessage(), e);
            }
            exception = e;
            throw e;
        }
        if (exception != null) {
            setExecutionSummary(exception.getClass().getName() + ":" + exception.getMessage());
        } else {
            setExecutionSummary("执行完成！" );
        }
    }
    @Override
    public boolean isRefireImmediatelyWhenException() {
        //任务发生异常时候进行的动作
        //false 挂起我JOB等待处理
        //true 继续执行
        return false;
    }
}
