package hls.core.hls.office.service.impl;

import com.hand.hap.attachment.dto.SysFile;
import com.hand.hap.attachment.mapper.SysFileMapper;
import com.hand.hap.attachment.service.ISysFileService;
import com.hand.hap.core.IRequest;
import hls.core.hls.office.service.IHlsEditOfficeOnlineService;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2017-10-26
 * Time: 10:04
 */
@Service
@Transactional
public class HlsEditOfficeOnlineServiceImpl implements IHlsEditOfficeOnlineService {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ISysFileService iSysFileService;
    @Autowired
    private SysFileMapper sysFileMapper;

    @Override
    public void openServerFile(IRequest requestContext, String filePath, HttpServletResponse response) {
        try {
            File file = new File(filePath);
            ServletOutputStream outputStream = response.getOutputStream();
            FileUtils.copyFile(file, outputStream);
        } catch (IOException e) {
            logger.error("file trans error,file path:{}", filePath, e);
        }
    }

    @Override
    public void saveServerFile(IRequest requestContext, HttpServletRequest request, String filePath) {

        filePath = filePath.replace("/","\\");

        List<FileItem> fileItems = null;
        DiskFileItemFactory factory = new DiskFileItemFactory();
        factory.setSizeThreshold(4096);
        ServletFileUpload upload = new ServletFileUpload(factory);
        //设置允许用户上传文件大小,单位:字节
        upload.setSizeMax(1024 * 1024 * 4);
        try {
            fileItems = upload.parseRequest(request);
        } catch (FileUploadException e) {
            logger.error("file size too large");
        }

        SysFile sysFile = new SysFile();
        sysFile.setFilePath(filePath);
        List<SysFile> files = sysFileMapper.select(sysFile);
        if (files.size() < 1) {
            logger.error("file not found in system_file,file path:{}", filePath);
            return;
        }

        try {
            Iterator iterator = fileItems.iterator();
            while (iterator.hasNext()) {
                FileItem item = (FileItem) iterator.next();
                File fil = new File(filePath);
                item.write(fil);
                files.forEach(f -> {
                    f.setFileSize(item.getSize());
                    sysFileMapper.updateByPrimaryKeySelective(f);
                });

            }
        } catch (Exception e) {
            logger.error("file upload error");
        }
    }
}
