package hls.core.hls.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.dto.HlsCusHlsCreditLineTrx;
import hls.core.hls.dto.HlsCusHlsCreditLineTrxMaster;
import hls.core.hls.mapper.HlsCusHlsCreditLineTrxMapper;
import hls.core.hls.mapper.HlsCusHlsCreditLineTrxMasterMapper;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.mapper.HlsCusHlsCreditLineMapper;
import hls.core.prj.service.HlsCreditLineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.hls.service.IHlsCreditLineTrxService;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional(rollbackFor = Exception.class)
public class HlsCreditLineTrxServiceImpl extends BaseServiceImpl<HlsCusHlsCreditLineTrx> implements IHlsCreditLineTrxService {

    public static final String USE = "USE";
    public static final String RELEASE = "RELEASE";
    public static final String FREEZE = "FREEZE";
    public static final String THAW = "THAW";

    @Autowired
    private HlsCusHlsCreditLineTrxMasterMapper hlsCreditLineTrxMasterMapper;
    @Autowired
    private HlsCusHlsCreditLineTrxMapper hlsCreditLineTrxMapper;
    @Autowired
    private HlsCusHlsCreditLineMapper hlsCreditLineMapper;
    @Autowired
    private HlsCreditLineService hlsCreditLineService;

    @Override
    public void calcCreditMethod(IRequest request, Long trxId) {
        boolean updateFlag = false;//更新标志
        HlsCusHlsCreditLineTrx hlsCreditLineTrx = mappers.get(0).selectByPrimaryKey(trxId);
        String trxCode = hlsCreditLineTrx.getTrxCode();
        Long creditLineId = hlsCreditLineTrx.getCreditLineId();
        HlsCusHlsCreditLineTrxMaster hlsCreditLineTrxMaster = hlsCreditLineTrxMasterMapper.selectByPrimaryKey(trxCode);
        String trxType = hlsCreditLineTrxMaster.getTrxType();
        HlsCusHlsCreditLine hlsCreditLine = hlsCreditLineMapper.selectByPrimaryKey(creditLineId);
        if (hlsCreditLine == null)
            return;
        switch (trxType) {
            case USE:
                hlsCreditLine.setCreditExposureAmt(CalculateUtil.add(hlsCreditLine.getCreditExposureAmt(), hlsCreditLineTrx.getTrxAmount()));
                updateFlag = true;
                break;
            case RELEASE:
                String documentType = hlsCreditLine.getDocumentType();
                if ("NON_REVOLVING".equalsIgnoreCase(documentType)) {//非循环

                } else if ("REVOLVING".equalsIgnoreCase(documentType)) {//循环
                    hlsCreditLine.setCreditExposureAmt(CalculateUtil.sub(hlsCreditLine.getCreditExposureAmt(), hlsCreditLineTrx.getTrxAmount()));
                    updateFlag = true;
                }
                break;
            case FREEZE:
                break;
            case THAW:
                break;
            default:
                break;
        }
        if (updateFlag)
            hlsCreditLineService.self().updateByPrimaryKeySelective(request, hlsCreditLine);
    }

    @Override
    public List<HlsCusHlsCreditLineTrx> selectCreditLineTrxDynamic(IRequest request, HlsCusHlsCreditLineTrx dto, int page, int pageSize) {
        dto.setCompanyId(request.getCompanyId());
        PageHelper.startPage(page, pageSize);
        List<HlsCusHlsCreditLineTrx> hlsCusHlsCreditLineTrxes = hlsCreditLineTrxMapper.selectCreditLineTrxDynamic(dto);
        hlsCusHlsCreditLineTrxes.forEach(item -> {
            item.setEventMessage(item.getTrxTypeDesc().concat(item.getTrxAmount()!=null?item.getTrxAmount().toString():""));
        });
        return hlsCusHlsCreditLineTrxes;
    }

}