package hls.core.hls.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.mybatis.entity.Example;
import com.hand.hap.system.dto.ResponseData;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.hls.dto.HlsDbDataSource;
import hls.core.hls.mapper.HlsDbDataSourceMapper;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.support.rowset.SqlRowSet;
import org.springframework.jdbc.support.rowset.SqlRowSetMetaData;
import org.springframework.stereotype.Service;
import hls.core.hls.dto.HlsDbDataSourceColumn;
import hls.core.hls.service.IHlsDbDataSourceColumnService;
import org.springframework.transaction.annotation.Transactional;
import hls.core.hls.mapper.HlsDbDataSourceColumnMapper;

import java.util.ArrayList;
import java.util.List;

@Service
@Transactional
public class HlsDbDataSourceColumnServiceImpl extends BaseServiceImpl<HlsDbDataSourceColumn> implements IHlsDbDataSourceColumnService {
    @Autowired
    private HlsDbDataSourceColumnMapper hlsDbDataSourceColumnMapper;

    @Autowired
    private HlsDbDataSourceMapper dataSourceMapper;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public List<HlsDbDataSourceColumn> query(IRequest requestContext, HlsDbDataSourceColumn dto, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        return hlsDbDataSourceColumnMapper.query(dto);
    }

    @Override
    public ResponseData generateColumns(IRequest requestContext, HlsDbDataSource dataSource) {
        ResponseData responseData = new ResponseData(false);
        if (dataSource == null || dataSource.getDataSourceId() == null) {
            responseData.setMessage("非法的参数");
            return responseData;
        }
        // 先通过datasourceId查询其对应的sql语句
        HlsDbDataSource hlsDbDataSource = dataSourceMapper.selectByPrimaryKey(dataSource);
        if (hlsDbDataSource == null || StringUtils.isEmpty(hlsDbDataSource.getSqlContext())) {
            responseData.setMessage("没有找到该条记录");
            return responseData;
        }
        String sqlContext = hlsDbDataSource.getSqlContext();
        // 替换其中的${xx}为''
        if (sqlContext.contains("$")) {
            sqlContext = sqlContext.replaceAll("\\$\\{.*?}", "''");
        }
        // 执行sql，获取各列信息
        //   获取结果集
        try {
            SqlRowSet rowSet = jdbcTemplate.queryForRowSet(sqlContext.trim());
            //   获取结果集元数据
            SqlRowSetMetaData metaData = rowSet.getMetaData();
            //   遍历每一列
            int columnCount = metaData.getColumnCount();

            List<HlsDbDataSourceColumn> columns = new ArrayList<>(columnCount);
            List<String> conds = new ArrayList<>(columnCount);
            for (int i = 0; i < columnCount; i++) {
                String columnName = metaData.getColumnLabel(i + 1);
                String columnTypeName = metaData.getColumnTypeName(i + 1);

                // 封装成dto
                HlsDbDataSourceColumn column = new HlsDbDataSourceColumn();
                column.setColumnName(columnName);
                column.setDataSourceId(hlsDbDataSource.getDataSourceId());
                List<HlsDbDataSourceColumn> select = hlsDbDataSourceColumnMapper.select(column);
                column.setColumnType(columnTypeName);
                column.setAliasName(columnName);
                column.setDescription(columnName);
                column.setEnabledFlag("Y");
                if (select == null || select.size() < 1) {
                    // 保存新的列信息
                    hlsDbDataSourceColumnMapper.insertSelective(column);
                }
                conds.add(columnName);
                columns.add(column);
            }
            Example example = new Example(HlsDbDataSourceColumn.class);
            example.createCriteria().andNotIn("columnName",conds).andEqualTo("dataSourceId",dataSource.getDataSourceId());
            hlsDbDataSourceColumnMapper.deleteByExample(example);
            responseData.setSuccess(true);
        }catch (Exception e){
            responseData.setMessage(e.getMessage());
        }
        return responseData;
    }
}