package hls.core.hls.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.excel.ExcelException;
import com.hand.hap.excel.dto.ColumnInfo;
import com.hand.hap.generator.service.impl.DBUtil;
import com.hand.hap.mybatis.common.Mapper;
import hls.core.fnd.dto.FndInterfaceHeaders;
import hls.core.fnd.dto.FndInterfaceLines;
import hls.core.fnd.mapper.FndInterfaceHeadersMapper;
import hls.core.fnd.mapper.FndInterfaceLinesMapper;
import hls.core.fnd.service.FndInterfaceHeadersService;

import hls.core.fnd.service.FndInterfaceLinesService;
import hls.core.hls.service.HlsBeanRefUtilService;
import hls.core.hls.service.HlsExcelService;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.sql.DataSource;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.*;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.Date;
import java.util.regex.Pattern;

import static org.apache.poi.hwpf.model.FileInformationBlock.logger;

/**
 * Created by syj on 2017/3/6.
 */
@Service
@Transactional
public class HlsExcelServiceImpl implements HlsExcelService{
    //private POIFSFileSystem fs;

    @Autowired
    @Qualifier("dataSource")
    DataSource dataSource;
    @Autowired
    FndInterfaceHeadersMapper fndInterfaceHeadersMapper;
    @Autowired
    FndInterfaceLinesMapper fndInterfaceLinesMapper;
    @Autowired
    FndInterfaceHeadersService fndInterfaceHeadersService;
    @Autowired
    FndInterfaceLinesService fndInterfaceLinesService;

    @Autowired
    HlsBeanRefUtilService hlsBeanRefUtilService;
    private static final int BATCH_COUNT = 100;
    private Workbook wb;
    private Sheet sheet;
    private Row row;
    private static Pattern linePattern = Pattern.compile("_(\\w)");
    private StringBuilder sql = null;//sql语句
    private Connection conn = null;
    List<ColumnInfo> columnInfos = new ArrayList<>();
    private Row rows; // 当前行索引

    private Row row1; //excel中表名行

    private int startRow = 2;

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Override
    public Long loadExcel(IRequest iRequest, InputStream is, String templateCode, String fileName) throws IOException, SQLException, ExcelException, ParseException, InvocationTargetException {

        try {
            wb = WorkbookFactory.create(is);
        } catch (InvalidFormatException e) {
            e.printStackTrace();
            throw new ExcelException((String) null, "不是正确的Excel文件！", (Object[]) null);
        }

        //

        //插入接口头表
        FndInterfaceHeaders fndInterfaceHeaders = new FndInterfaceHeaders();

        fndInterfaceHeaders.setTemplateCode(templateCode);
        fndInterfaceHeaders.setJobId(null);
        fndInterfaceHeaders.setStatus("NEW");
        fndInterfaceHeaders.setFileName(fileName);
        fndInterfaceHeaders.set__status("add");
        List<FndInterfaceHeaders> fndInterfaceHeadersList = new ArrayList<FndInterfaceHeaders>();

        fndInterfaceHeadersList.add(fndInterfaceHeaders);
        fndInterfaceHeadersService.batchUpdate(iRequest, fndInterfaceHeadersList);

        Long hd_id = fndInterfaceHeadersList.get(0).getHeaderId();


        Map<String, String> LineMap = new HashMap<String, String>();
        List<FndInterfaceLines> fndInterfaceLinesList = new ArrayList<FndInterfaceLines>();
        //sheetName
        String str = "";
        FndInterfaceLines fndInterfaceLines;
        for (int i = 0; i < wb.getNumberOfSheets(); i++) {
            sheet = wb.getSheetAt(i);
            String sheetName = sheet.getSheetName();
            //取行数
            int rowNum = sheet.getLastRowNum() + 1;
            try {
                //插入行表
                for (int j = 0; j < rowNum; j++) {
                    fndInterfaceLines = new FndInterfaceLines();
                    int k = 0;
                    //获取当前行的列数
                    row = sheet.getRow(j);
                    int colNum;
                    if (row == null) {
                        colNum = -1;
                    } else {
                        colNum = row.getLastCellNum();
                    }
                    LineMap.clear();
                    while (k < colNum) {
                        str = getCellFormatValue(row.getCell(k));
                        String attribute_name = "attributes_" + (k + 1L);
                        LineMap.put(attribute_name, str);
                        k++;
                    }
                    hlsBeanRefUtilService.setFieldValues(fndInterfaceLines, LineMap);
                    fndInterfaceLines.setHeaderId(hd_id);
                    fndInterfaceLines.setLineNumber(j + 1L);

                    fndInterfaceLines.setStatus("NEW");
                    fndInterfaceLines.setSheetName(sheetName);
                    fndInterfaceLines.set__status("add");
                    fndInterfaceLinesList.add(fndInterfaceLines);
                }

            } catch (Exception e) {
                e.printStackTrace();
                throw new ExcelException((String) null, "", (Object[]) null);
            }


        }
        fndInterfaceLinesService.batchUpdate(iRequest, fndInterfaceLinesList);

        return hd_id;

    }

    /**
     *  以sheet页为表名,单个sheet页为表数据进行excel数据导入  by fjm11330
     * @param iRequest
     * @param is
     * @throws IOException
     * @throws ExcelException
     * @throws SQLException
     */
    @Override
    public void ImportData(IRequest iRequest, InputStream is) throws IOException, ExcelException, SQLException {
        try {
            wb = WorkbookFactory.create(is);
        } catch (InvalidFormatException e) {
            logger.error("excel流构建workbook出错");
            throw new ExcelException(null, "excel流构建workbook出错", null);
        }

        for (int i = 0; i < wb.getNumberOfSheets(); i++) {

            //获取excel中表格的列名
            sheet = wb.getSheetAt(i);
            String tableName = sheet.getSheetName();
            readColumnType(tableName);
            startRow = 2;
            row1 = sheet.getRow(startRow - 1);
            if (!isContains(columnInfos, row1)) {
                logger.error("excel数据验证失败：非法的列存在");
                throw new ExcelException(null, "excel数据验证失败：非法的列存在", null);
            }

            // 构造sql 插入的列
            getInsertSql(row1, tableName);

            try (Connection connection = dataSource.getConnection()) {
                conn = connection;
                connection.setAutoCommit(false);
                try (PreparedStatement stmt = connection.prepareStatement(sql.toString())) {
                    createPreparedStatement(stmt);
                    conn.commit();
                    logger.info("数据导入成功");
                }
            } catch (SQLException e) {
                logger.error(e.getMessage());
                logger.info("插入数据失败，数据回滚..");
                try {
                    conn.rollback();
                    logger.info("数据已回滚");
                } catch (SQLException e1) {
                    logger.error("数据回滚失败");
                }
                throw new SQLException();
            }
        }

        wb.close();
    }

   /* @Override
    public void ImportData(IRequest iRequest, InputStream is) throws IOException, ExcelException, SQLException {
        try {
            wb = WorkbookFactory.create(is);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InvalidFormatException e) {
            e.printStackTrace();
        }
        String sheetName;
        for (int i = 0; i < wb.getNumberOfSheets(); i++) {
            sheet = wb.getSheetAt(i);//获取sheet页
            sheetName = sheet.getSheetName();//sheetName为表名
            String tableName = sheetName;
            readColumnType(tableName);
            List<String> tableColumn = new ArrayList<>();
            Map cellData;
            List<Map> columnData = new ArrayList<>();
            int rowNum = sheet.getLastRowNum() + 1;
            for (int j = 0; j < rowNum; j++) {
                int k = 0;
                //获取当前行的列数
                row = sheet.getRow(j);
                int colNum;
                if (row == null) {
                    colNum = -1;
                } else {
                    colNum = row.getLastCellNum();
                }
                cellData = new HashMap<>();
                while (k < colNum) {
                   getCellFormatValue(row.getCell(k));
                    if (j == 1){//第二行为字段名
                    tableColumn.add(getCellFormatValue(row.getCell(k)));
                    }
                    if (j>1){
                    cellData.put(tableColumn.get(k),getCellFormatValue(row.getCell(k)));
                    }
                    k++;
                }
                if (j>=2){
                    columnData.add(j-2,cellData);
                }
            }
            row1 = sheet.getRow(startRow - 1);
            getInsertSql(tableColumn,tableName);
            try (Connection connection = dataSource.getConnection()) {
                conn = connection;
                connection.setAutoCommit(false);
                try (PreparedStatement stmt = connection.prepareStatement(sql.toString())) {
                    createPreparedStatement(stmt);
                    conn.commit();
                }
            } catch (SQLException e) {
                try {
                    conn.rollback();
                } catch (SQLException e1) {
                }

            }
            wb.close();
            System.out.println(tableName);
        }
    }*/




    /**
     * 获取单元格数据内容为字符串类型的数据
     *
     * @param cell Excel单元格
     * @return String 单元格数据内容
     */

    private String getStringCellValue(Cell cell) {
        String strCell = "";
        switch (cell.getCellType()) {
            case Cell.CELL_TYPE_STRING:
                strCell = cell.getStringCellValue();
                break;
            case Cell.CELL_TYPE_NUMERIC:
                // strCell = String.valueOf(cell.getNumericCellValue());
                DecimalFormat df = new DecimalFormat("0");
                strCell = df.format(cell.getNumericCellValue());
                break;
            case Cell.CELL_TYPE_BOOLEAN:
                strCell = String.valueOf(cell.getBooleanCellValue());
                break;
            case Cell.CELL_TYPE_BLANK:
                strCell = "";
                break;
            default:
                strCell = "";
                break;
        }
        if (strCell.equals("") || strCell == null) {
            return "";
        }
        if (cell == null) {
            return "";
        }
        return strCell;
    }

    /**
     * 获取单元格数据内容为日期类型的数据
     *
     * @param cell Excel单元格
     * @return String 单元格数据内容
     */

    private String getDateCellValue(Cell cell) throws ExcelException {
        String result = "";
        try {
            int cellType = cell.getCellType();
            if (cellType == Cell.CELL_TYPE_NUMERIC) {
                Date date = cell.getDateCellValue();
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                result = sdf.format(date);
               /* result = (date.getYear() + 1900) + "-" + (date.getMonth() + 1)
                        + "-" + date.getDate();*/

            } else if (cellType == Cell.CELL_TYPE_STRING) {
                String date = getStringCellValue(cell);
                result = date.replaceAll("[年月]", "-").replace("日", "").trim();
            } else if (cellType == Cell.CELL_TYPE_BLANK) {
                result = "";
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ExcelException((String) null, "日期格式不正确！", (Object[]) null);
        }
        return result;
    }

    /**
     * 根据Cell类型设置数据
     *
     * @param cell
     * @return
     */

    private String getCellFormatValue(Cell cell) {
        String cellvalue = "";
        if (cell != null) {
            // 判断当前Cell的Type
            switch (cell.getCellType()) {
                // 如果当前Cell的Type为NUMERIC
                case Cell.CELL_TYPE_NUMERIC: {

                    short dateFormat = cell.getCellStyle().getDataFormat();
                    SimpleDateFormat sdf = null;
                    if (HSSFDateUtil.isCellDateFormatted(cell)) {// 处理日期格式、时间格式

                        if (cell.getCellStyle().getDataFormat() == HSSFDataFormat.getBuiltinFormat("h:mm")) {
                            sdf = new SimpleDateFormat("HH:mm");
                        } else {// 日期
                            sdf = new SimpleDateFormat("yyyy-MM-dd");
                        }
                        Date date = cell.getDateCellValue();
                        cellvalue = sdf.format(date);
                    } else if (dateFormat == 14 || dateFormat == 31 || dateFormat == 57 || dateFormat == 58 || dateFormat == 177) {

                        sdf = new SimpleDateFormat("yyyy-MM-dd");
                        double value = cell.getNumericCellValue();
                        Date date = org.apache.poi.ss.usermodel.DateUtil.getJavaDate(value);
                        cellvalue = sdf.format(date);
                    } else if (dateFormat == 20 || dateFormat == 32) {
                        sdf = new SimpleDateFormat("HH:mm");
                        double value = cell.getNumericCellValue();
                        Date date = org.apache.poi.ss.usermodel.DateUtil.getJavaDate(value);
                        cellvalue = sdf.format(date);
                    } else {
                        double value = cell.getNumericCellValue();
                        CellStyle style = cell.getCellStyle();
                        DecimalFormat format = new DecimalFormat();
                        String temp = style.getDataFormatString();
                        // 单元格设置成常规
                        if (temp.equals("General")) {
                            format.applyPattern("#");
                            cellvalue = format.format(value);
                        } else {
                            cellvalue = value + "";
                        }
                    }
                    break;
                }
                case Cell.CELL_TYPE_FORMULA: {
                    try {
                        boolean a = HSSFDateUtil.isCellDateFormatted(cell);
                        // 判断当前的cell是否为Date
                        if (HSSFDateUtil.isCellDateFormatted(cell)) {
                            // 如果是Date类型则，转化为Data格式
                            //方法1：这样子的data格式是带时分秒的：2011-10-12 0:00:00
                            //cellvalue = cell.getDateCellValue().toLocaleString();
                            //方法2：这样子的data格式是不带带时分秒的：2011-10-12
                            Date date = cell.getDateCellValue();
                            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                            cellvalue = sdf.format(date);
                        }
                        // 如果是纯数字
                        else {
                            // 取得当前Cell的数值
                            cellvalue = String.valueOf(cell.getNumericCellValue());
                        }
                    } catch (Exception e) {
                        //字符串
                        cellvalue = cell.getCellFormula();
                        break;
                    }
                    break;
                }
                // 如果当前Cell的Type为STRIN
                case Cell.CELL_TYPE_STRING:
                    // 取得当前的Cell字符串
                    cellvalue = cell.getRichStringCellValue().getString();
                    break;
                // 默认的Cell值
                default:
                    cellvalue = " ";
            }
        } else {
            cellvalue = "";
        }
        return cellvalue;

    }

    public static String lineToHump(String str){
        str = str.toLowerCase();
        Matcher matcher = linePattern.matcher(str);
        StringBuffer sb = new StringBuffer();
        while(matcher.find()){
            matcher.appendReplacement(sb, matcher.group(1).toUpperCase());
        }
        matcher.appendTail(sb);
        return sb.substring(0,1).toUpperCase()+sb.substring(1);
    }

//    private void getInsertSql(List<String> tableColumn, String tableName) {
//        sql = new StringBuilder();
//        sql.append("insert into " + tableName + " (");
//        Iterator<String> iterator = tableColumn.iterator();
//        while (iterator.hasNext()) {
//            String cell = iterator.next();
//            if (!(null == cell || "".equals(cell) || " ".equals(cell))) {
//                sql.append(cell.toString() + ",");
//            }
//        }
//        sql.deleteCharAt(sql.length() - 1);
//        sql.append(") values(");
//
//        Iterator<String> iterator1 = tableColumn.iterator();
//
//        while (iterator1.hasNext()) {
//            sql.append("?,");
//            iterator1.next();
//        }
//        sql.deleteCharAt(sql.length() - 1);
//        sql.append(")");
//    }


    private void insertData(PreparedStatement statement) throws SQLException {
        statement.executeBatch();
    }

    // 通过数据库构造columninfo信息类
    private void readColumnType(String tableName) throws SQLException {
        try (Connection connection = dataSource.getConnection()) {
            ResultSet resultSet = DBUtil.getTableColumnInfo(tableName, connection.getMetaData());
            while (resultSet.next()) {
                ColumnInfo columnInfo = new ColumnInfo();
                String typeName = resultSet.getString("TYPE_NAME");
                String columnName = resultSet.getString("COLUMN_NAME");
                columnInfo.setName(columnName);
                columnInfo.setType(typeName);
                this.columnInfos.add(columnInfo);
            }
            resultSet.close();
        } catch (SQLException e) {
            logger.error("读取列信息失败");
            throw new SQLException();
        }
    }

    // 判断excel的列是否被数据表列包含。
    private boolean isContains(List<ColumnInfo> columnInfos, Row rowTalbeName) {
        boolean result = true;
        Iterator<Cell> iterator = rowTalbeName.cellIterator();

        while (iterator.hasNext()) {
            int key = 0;
            Cell cell = iterator.next();
            for (ColumnInfo columnInfo : columnInfos) {
                if ((!(null == cell || "".equals(cell.toString()) || " ".equals(cell.toString()))) && columnInfo.getName().equalsIgnoreCase(cell.toString())) {
                    key = 1;
                }
            }
            if (key == 0) {
                result = false;
                break;
            }
        }
        return result;
    }

    /*
    * 通过第一行构造sql语句
    * */
    private void getInsertSql(Row row1, String tableName) {
        sql = new StringBuilder();
        sql.append("insert into " + tableName + " (");
        Iterator<Cell> iterator = row1.cellIterator();
        while (iterator.hasNext()) {
            Cell cell = iterator.next();
            if (!(null == cell || "".equals(cell.toString()) || " ".equals(cell.toString()))) {
                sql.append(cell.toString() + ",");
            }
        }
        sql.deleteCharAt(sql.length() - 1);
        sql.append(") values(");

        Iterator<Cell> iterator1 = row1.cellIterator();

        while (iterator1.hasNext()) {
            sql.append("?,");
            iterator1.next();
        }
        sql.deleteCharAt(sql.length() - 1);
        sql.append(")");
    }

    private void createPreparedStatement(PreparedStatement stmt) throws SQLException, ExcelException {

        int batchCount = 0;//stmtBatch数量

        for (int j = startRow; j < sheet.getLastRowNum() + 1; j++) {
            batchCount = addBatch(stmt, batchCount, j);
            // 读取excel读取列数据插入batch
        }
        if (batchCount > 0) {
            insertData(stmt);
        }


    }

    private int addBatch(PreparedStatement stmt, int batchCount, int rowNumber) throws SQLException, ExcelException {

        rows = sheet.getRow(rowNumber);
        Iterator<Cell> iterator3 = row1.cellIterator();
        int ii = 1;
        while (iterator3.hasNext()) {
            Cell cell = iterator3.next();
            if (!(null == cell || "".equals(cell.toString()) || " ".equals(cell.toString()))) {
                String type = "STRING";
                for (ColumnInfo columnInfo : columnInfos) {
                    if (columnInfo.getName().toUpperCase().equals(cell.toString().toUpperCase())) {
                        type = columnInfo.getType();
                        break;
                    }
                }
                Cell cells = rows.getCell(cell.getColumnIndex());
                if (null != cells) {
                    switch (type) {
                        case "DATE":
                        case "TIMESTAMP":
                        case "DATETIME":
                        case "TIME":
                            try {
                                if (cells.getDateCellValue()!=null) {
                                    stmt.setDate(ii, new java.sql.Date(cells.getDateCellValue().getTime()));
                                }else {
                                    stmt.setDate(ii, null);
                                }

                            } catch (IllegalStateException e) {
                                throw new ExcelException(null, cells.toString() + " 数据类型错误: " + (rowNumber + 1) + "行"
                                        + (cells.getColumnIndex() + 1) + "列,期待的类型: ", null);
                            }
                            break;
                        case "BIGINT":
                        case "INT":
                        case "DOUBLE":
                        case "NUMBER":
                        case "INTEGER":
                        case "FLOAT":
                        case "DECIMAL":
                        case "TINYINT":
                            try {
                                stmt.setInt(ii, (int) cells.getNumericCellValue());
                            } catch (IllegalStateException e) {
                                throw new ExcelException(null, cells.toString() + " 数据类型错误: " + (rowNumber + 1) + "行"
                                        + (cells.getColumnIndex() + 1) + "列,期待的类型: " + type, null);
                            }
                            break;
                        default:
                            cells.setCellType(Cell.CELL_TYPE_STRING);
                            stmt.setString(ii, cells.getStringCellValue());
                            break;
                    }

                } else {
                    switch (type.toUpperCase()) {
                        case "VARCHAR":
                        case "STRING":
                            stmt.setNull(ii, Types.VARCHAR);
                            break;
                        case "CHAR":
                            stmt.setNull(ii, Types.CHAR);
                            break;
                        case "INTEGER":
                            stmt.setNull(ii, Types.INTEGER);
                            break;
                        case "BIGINT":
                            stmt.setNull(ii, Types.BIGINT);
                            break;
                        case "DOUBLE":
                            stmt.setNull(ii, Types.DOUBLE);
                            break;
                        case "DECIMAL":
                            stmt.setNull(ii, Types.DECIMAL);
                            break;
                        case "DATE":
                        case "DATETIME":
                        case "TIME":
                        case "TIMESTAMP":
                            stmt.setNull(ii, Types.DATE);
                            break;
                        default:
                            stmt.setNull(ii, Types.VARCHAR);
                            break;
                    }

                }
                if (null != cells) {
                    logger.info("正在将数据装入batch... 第" + (rowNumber + 1) + "行第" + (cells.getColumnIndex() + 1) + "列....当前值：" + cells);
                }
            }
            ii++;
        }

        // 写入数据库
        stmt.addBatch();
        batchCount++;
        if (batchCount >= BATCH_COUNT) {
            insertData(stmt);
            batchCount = 0;
        }
        return batchCount;
    }



}
