package hls.core.hls.service.impl;

import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusConDebtExemptionReqCf;
import hls.core.csh.dto.HlsCusCshWriteOff;
import hls.core.csh.mapper.*;
import hls.core.fnd.dto.HlsPenaltyProfileDtl;
import hls.core.fnd.mapper.HlsPenaltyProfileDtlMapper;
import hls.core.hls.service.HlsDayEndCommon;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class HlsPenaltyCalServiceImpl implements HlsDayEndCommon {

    @Autowired
    HlsCusConContractMapper conContractMapper;

    @Autowired
    HlsCusConContractCashflowMapper conContractCashflowMapper;

    @Autowired
    HlsCusCshWriteOffMapper cshWriteOffMapper;

    @Autowired
    HlsPenaltyProfileDtlMapper hlsPenaltyProfileDtlMapper;

    @Autowired
    HlsCusConDebtExemptionReqCfMapper conDebtExemptionReqCfMapper;

    private static final String DayEndType = "PENALTY";

    @Override
    public String getDayEndType() {
        return DayEndType;
    }

    @Override
    public void process(Map map) {
        if (map.get("contractId") != null && map.get("contractId") != "" ) { //对特定合同计算罚息
            Long contractId = (Long) map.get("contractId");
            HlsCusConContract contract = new HlsCusConContract();
            contract.setContractId(contractId);
            contract = conContractMapper.selectByPrimaryKey(contract);
            if (contract == null || contract.getContractStatus().equals("INCEPT")) {
                calculatePenalty(contract, (Date) map.get("dayEndDate"));
            } else {
                //抛出异常合同状态错误不能计算罚息！
                throw new IllegalArgumentException("合同状态错误不能计算罚息！");
            }
        } else { //对所有合同计算罚息
            List<HlsCusConContract> conContractList = conContractMapper.selectNeedCalcPenaltyContract(Long.parseLong(map.get("companyId").toString()));

            HlsCusConContract contract = new HlsCusConContract();
            contract.setCompanyId(Long.parseLong(map.get("companyId").toString()));
            contract.setDayEndDate((Date) map.get("dayEndDate"));
            List<HlsCusConContract> overContractList = conContractMapper.selectOverContract(contract);

            if(overContractList!= null && overContractList.size() > 0){
                for (HlsCusConContract overCon : overContractList) {
                    overCon.setOverdueStatus("Y");
                    conContractMapper.updateByPrimaryKey(overCon);
                }
            }

            if(conContractList!= null && conContractList.size() > 0){
                for (HlsCusConContract con : conContractList) {
                    //计算罚息
                    calculatePenalty(con, (Date) map.get("dayEndDate"));
                }
            }

        }
    }

    //计算一条合同的罚息
    public void calculatePenalty(HlsCusConContract contract, Date dayEndDate) {
                //找出该合同对应的现金流
                HlsCusConContractCashflow contractCashflow = new HlsCusConContractCashflow();
                //contractCashflow.setCf_item(hppd.getCfItem());
                contractCashflow.setDayEndDate(dayEndDate);
                contractCashflow.setContract_id(contract.getContractId());
                List<HlsCusConContractCashflow> contractCashflowsList = conContractCashflowMapper.selectCalcPenaltyCashFlowByConId(contractCashflow);
                if(contractCashflowsList != null && contractCashflowsList.size() > 0 ) {
                    for (HlsCusConContractCashflow ccc : contractCashflowsList) {
                        //更新现金流overdue_status 为Y
                        ccc.setOverdueStatus("Y");
                        conContractCashflowMapper.updateByPrimaryKeySelective(ccc);


                        //取得该现金流对应的罚息计算规则
                        HlsPenaltyProfileDtl hppd = new HlsPenaltyProfileDtl();
                        hppd.setCfItem(ccc.getCf_item());
                        hppd.setPenaltyProfile(contract.getPenaltyProfile());
                        hppd = hlsPenaltyProfileDtlMapper.selectByPrimaryKey(hppd);


                        //计算未收部分 (未核销罚息)
                        //double unreceived = (ccc.getDue_amount() - ccc.getReceived_amount()) * ((int) (dayEndDate.getTime() - ccc.getDue_date().getTime() / (24 * 3600 * 1000)) - hppd.getGracePeriod()) * hppd.getPenaltyRate();

                        //计算已收部分
                        HlsCusCshWriteOff cshWriteOff = new HlsCusCshWriteOff();
                        cshWriteOff.setCashflowId(ccc.getCashflow_id());
                        cshWriteOff.setDueDate(ccc.getDue_date());
                        List<HlsCusCshWriteOff> cshWriteOffList = cshWriteOffMapper.selectPenaltyWriteOff(cshWriteOff);
                        double received = 0.0d;
                        double writeOffDueAmount = 0.0d;
                        for (HlsCusCshWriteOff cwo : cshWriteOffList) {
                            writeOffDueAmount = writeOffDueAmount + cwo.getWriteOffDueAmount();
                            received = received + cwo.getWriteOffDueAmount() * (int)(((cwo.getPenaltyCalcDate().getTime() - ccc.getDue_date().getTime()) / (24 * 3600 * 1000)) - hppd.getGracePeriod()) * hppd.getPenaltyRate();
                        }
                        //查询罚息减免部分
                        double derateAmount=0;
                        HlsCusConDebtExemptionReqCf conDebtExemptionReqCf=new HlsCusConDebtExemptionReqCf();
                        conDebtExemptionReqCf.setCashflowId(ccc.getCashflow_id());
                        List<HlsCusConDebtExemptionReqCf> list=conDebtExemptionReqCfMapper.selectReqCfByCashflowId(conDebtExemptionReqCf);
                        for(HlsCusConDebtExemptionReqCf cf:list){
                            if(cf.getExemptionAmount()!=null){
                                derateAmount+=cf.getExemptionAmount();
                            }
                        }
                        //计算未收部分 (未核销罚息)
                        double unreceived = (ccc.getDue_amount() - writeOffDueAmount) * ((int)((dayEndDate.getTime() - ccc.getDue_date().getTime())/(24 * 3600 * 1000) -  hppd.getGracePeriod())) * hppd.getPenaltyRate();

                        double profileTotalAmount = unreceived + received - derateAmount;

                        //查询该现金流是否存在罚息现金流，有就修改，没有就添加
                        HlsCusConContractCashflow contractCashflow2 = new HlsCusConContractCashflow();
                        contractCashflow2.setGenerated_source_doc_id(ccc.getCashflow_id());
                        contractCashflow2.setGenerated_source("DAYEND");
                        contractCashflow2.setOverdueStatus("N");
                        List<HlsCusConContractCashflow> contractCashflowsList2 = conContractCashflowMapper.selectCalcPenaltyCashFlowBySourceId(contractCashflow2);
                        if (contractCashflowsList2.size() > 0) {
                            for (HlsCusConContractCashflow c : contractCashflowsList2) {
                                c.setDue_date(dayEndDate);
                                c.setFin_income_date(dayEndDate);
                                c.setDue_amount(profileTotalAmount);
                                if (new BigDecimal(c.getReceived_amount()).compareTo(new BigDecimal(0))==0) {
                                    c.setWrite_off_flag("NOT");
                                } else if (new BigDecimal(c.getReceived_amount()).compareTo(new BigDecimal(c.getDue_amount())) >= 0) {
                                    c.setWrite_off_flag("FULL");
                                } else {
                                    c.setWrite_off_flag("PARTIAL");
                                }
                                conContractCashflowMapper.updateByPrimaryKeySelective(c);
                            }

                        } else {
                            HlsCusConContractCashflow contractCashflowIns = new HlsCusConContractCashflow();
                            contractCashflowIns.setContract_id(ccc.getContract_id());
                            contractCashflowIns.setCf_item(9L);
                            contractCashflowIns.setCf_type(9L);
                            contractCashflowIns.setCf_direction("INFLOW");
                            contractCashflowIns.setCf_status("RELEASE");
                            contractCashflowIns.setTimes(ccc.getTimes());
                            contractCashflowIns.setDue_date(dayEndDate);
                            contractCashflowIns.setFin_income_date(dayEndDate);
                            contractCashflowIns.setDue_amount(profileTotalAmount);
                            contractCashflowIns.setWrite_off_flag("NOT");
                            contractCashflowIns.setPenalty_process_status("NORMAL");
                            contractCashflowIns.setBilling_status("NOT");
                            contractCashflowIns.setOverdueStatus("N");
                            contractCashflowIns.setGenerated_source("DAYEND");
                            contractCashflowIns.setGenerated_source_doc_id(ccc.getCashflow_id());
                            conContractCashflowMapper.insertSelective(contractCashflowIns);
                        }
                    }
                }

                //遍历该合同对应的所有现金流
//        List<HlsCusConContractCashflow> contractCashflows = conContractCashflowMapper.selectCashFlowByContractId(contract.getContractId());
//        if(contractCashflows != null ){
//            boolean flag = false;
//            for(HlsCusConContractCashflow ctemp : contractCashflows){
//                if("Y".equals(ctemp.getOverdueStatus())){
//                    flag = true;
//                }
//            }
//            String overdueStatus = contract.getOverdueStatus() == null ? "":contract.getOverdueStatus();
//            if(flag){
//                contract.setOverdueStatus("Y");
//            }else{
//                contract.setOverdueStatus("N");
//            }
//            if( overdueStatus == null || !overdueStatus.equals(contract.getOverdueStatus())){
//                conContractMapper.updateByPrimaryKeySelective(contract);
//            }
//        }

    }
}
