package hls.core.job.controllers;

import org.quartz.SchedulerException;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.job.dto.JobInformation;
import hls.core.job.service.IJobInformationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;

import org.springframework.validation.BindingResult;

import java.util.HashMap;
import java.util.List;

@Controller
public class JobInformationController extends BaseController {

    @Autowired
    private IJobInformationService service;


    @RequestMapping(value = "/job/information/query")
    @ResponseBody
    public ResponseData query(JobInformation dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/job/information/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<JobInformation> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/job/information/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<JobInformation> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }


    @RequestMapping(value = "/hls/job/information/query")
    @ResponseBody
    public ResponseData queryInformation(HttpServletRequest request, JobInformation dto,@RequestParam(defaultValue = DEFAULT_PAGE) int page,
    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        IRequest requestContext = createRequestContext(request);
        List<HashMap> list = service.queryInformation(requestContext,dto,page,pageSize);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/job/information/deleteBoth")
    @ResponseBody
    public ResponseData deleteInformationAndJob(HttpServletRequest request, @RequestBody List<JobInformation> dto) {
        try {
            service.deleteInformationAndJob(dto);
        } catch (SchedulerException e) {
            e.printStackTrace();
            ResponseData rs = new ResponseData(false);
            rs.setMessage("任务删除失败,请联系管理员");
            return rs;
        }
        return new ResponseData();
    }

    /**
     * 传入一个jobId暂停job
     * @param request
     * @param dto
     * @return
     */
    @RequestMapping(value = "/hls/job/pause")
    @ResponseBody
    public ResponseData pauseJobByJobId(HttpServletRequest request,JobInformation dto){
        ResponseData rs;
        try {
            service.pauseJob(dto);
            rs = new ResponseData();
        } catch (SchedulerException e) {
            e.printStackTrace();
            rs = new ResponseData(false);
            rs.setMessage("暂停job出错,请联系管理员");
        }
        return rs;
    }

    /**
     * 传入一个jobId恢复job
     * @param request
     * @param dto
     * @return
     */
    @RequestMapping(value = "/hls/job/resume")
    @ResponseBody
    public ResponseData resumeJobByJobId(HttpServletRequest request,JobInformation dto){
        ResponseData rs;
        try {
            service.resumeJob(dto);
            rs = new ResponseData();
        } catch (SchedulerException e) {
            e.printStackTrace();
            rs = new ResponseData(false);
            rs.setMessage("暂停job出错,请联系管理员");
        }
        return rs;
    }
}