package hls.core.job.controllers;

import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.job.dto.JobSchedule;
import hls.core.job.service.ISchScheduleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;

import org.springframework.validation.BindingResult;

import java.util.List;

@Controller
public class SchScheduleController extends BaseController {

    @Autowired
    private ISchScheduleService service;


    @RequestMapping(value = "/hls/sch/schedule/query")
    @ResponseBody
    public ResponseData query(JobSchedule dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/hls/sch/schedule/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<JobSchedule> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }

        try {
            IRequest requestCtx = createRequestContext(request);
            return new ResponseData(service.batchUpdate(requestCtx, dto));
        } catch (Exception e) {
            e.printStackTrace();
            ResponseData data = new ResponseData(false);
            data.setMessage("任务周期计划代码或计划名称重复，请重新定义！");
            return data;
        }
    }

    @RequestMapping(value = "/hls/sch/schedule/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<JobSchedule> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }
}