package hls.core.job.controllers;

import com.hand.hap.core.exception.FieldRequiredException;
import com.hand.hap.job.dto.JobCreateDto;
import com.hand.hap.job.exception.JobException;
import hls.core.job.dto.HlsJobDto;
import hls.core.job.dto.JobTask;
import hls.core.job.service.ISchTaskService;
import hls.core.job.service.impl.HlsJobCreateService;
import org.quartz.SchedulerException;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
    public class SchTaskController extends BaseController{

    @Autowired
    private ISchTaskService service;

    @Autowired
    private HlsJobCreateService hlsJobCreateService;


    @RequestMapping(value = "/hls/sch/task/query")
    @ResponseBody
    public ResponseData query(JobTask dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext,dto,page,pageSize));
    }

    @RequestMapping(value = "/hls/sch/task/submit")
    @ResponseBody
    public ResponseData update(HttpServletRequest request,@RequestBody List<JobTask> dto){
        IRequest requestCtx = createRequestContext(request);
        for (JobTask task:
             dto) {
            try {
                Class clazz = Class.forName(task.getJobClassName());
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
                ResponseData data = new ResponseData(false);
                data.setMessage("任务程序未定义,请确认程序名称是否正确，联系系统管理员！");
                return data;
            }
        }
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/hls/sch/task/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request,@RequestBody List<JobTask> dto){
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/hls/sch/task/queryLov")
    @ResponseBody
    public ResponseData queryTaskByLov(JobTask dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.queryTaskByLov(requestContext,dto,page,pageSize));
    }


    /**
     * create by fjm 11330  2017 7.26
     * 创建定时任务,在配置了JobStoreTX的情况下,
     * job和trigger定义会自动持久化到qutz自带的表中,
     * 此方法创建job时把参数放入jobDataMap之中,并向jobInformation表中插入一条job的详细信息的数据
     * @param hlsJobDto
     * @param result
     * @param request
     * @return
     * @throws SchedulerException
     * @throws JobException
     * @throws ClassNotFoundException
     * @throws FieldRequiredException
     */
    @RequestMapping(value = "/hls/job/create")
    @ResponseBody
    public Map createJob(@RequestBody HlsJobDto hlsJobDto, BindingResult result,
                         HttpServletRequest request) throws SchedulerException, JobException, ClassNotFoundException, FieldRequiredException {
            IRequest requestCtx = createRequestContext(request);
            Map map = new HashMap<>();

            try{
                JobCreateDto jobCreateDto = hlsJobDto.getJobCreateDto();
                jobCreateDto.setTriggerGroup(jobCreateDto.getJobGroup());//trigger的组名与job的组名一致,触发器名加上_trigger后缀
                jobCreateDto.setTriggerName(jobCreateDto.getJobName() + "_trigger");
                getValidator().validate(jobCreateDto, result);
                Long jobId = hlsJobCreateService.createHlsJob(hlsJobDto,requestCtx);
                map.put("jobId",jobId);
                map.put("success","true");
            }catch (Exception e){
                map.put("message",e.getMessage());
                map.put("success","false");
            }
            return map;
    }
    }