package hls.core.job.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.job.dto.InformationPara;
import hls.core.job.dto.TaskParam;
import hls.core.job.mapper.InformationParaMapper;
import hls.core.job.mapper.JobInformationMapper;
import hls.core.job.mapper.TaskParamMapper;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.job.dto.JobInformation;
import hls.core.job.service.IJobInformationService;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

@Service
@Transactional(rollbackFor = Exception.class)
public class JobInformationServiceImpl extends BaseServiceImpl<JobInformation> implements IJobInformationService{


    @Autowired
    private JobInformationMapper mapper;

    @Autowired
    private InformationParaMapper paraMapper;

    @Autowired
    private Scheduler scheduler;


    /**
     * 查询计划任务工作台的明细情况
     * @param request
     * @param information
     * @param page
     * @param pageSize
     * @return
     */
    @Override
    public List<HashMap> queryInformation(IRequest request, JobInformation information,int page,int pageSize) {
        PageHelper.startPage(page,pageSize);
        List<HashMap> maps = mapper.queryInfomation(information);
        //在明细中增加job当前的状态
        for (HashMap map:
             maps) {
            try {
               Trigger trigger = scheduler.getTrigger(TriggerKey.triggerKey(map.get("jobName").toString()+"_trigger",map.get("jobGroup").toString()));
               //上一次执行时间和下一次执行时间应当从触发器中获取,如果当期触发器和job已经结束了,那么触发器是无法取到的
                if (trigger!=null) {
                   Date startTime = trigger.getStartTime();
                   Date endTime = trigger.getEndTime();
                   Date nextFireTime = trigger.getNextFireTime();
                   Date previousFireTime = trigger.getPreviousFireTime();
                   map.put("nextFireTime", dateFormat(nextFireTime));
                   map.put("previousFireTime", dateFormat(previousFireTime));
                   map.put("startTime",dateFormat(startTime));
                   map.put("endTime",dateFormat(endTime));
               }

               if (map.get("jobId")!=null){//获取实际的参数
                   List<HashMap> list = new ArrayList<>();

                   InformationPara informationPara = new InformationPara();
                   informationPara.setJobId(Long.parseLong(map.get("jobId").toString()));
                   List<InformationPara> params = paraMapper.select(informationPara);


                   for (InformationPara child:
                   params) {
                       HashMap dataMap = new HashMap();
                       dataMap.put("objectVersionNumber",child.getObjectVersionNumber());
                       dataMap.put("name",child.getParameterName());
                       dataMap.put("value",child.getParameterValue());
                       list.add(dataMap);
                   }

                   map.put("jobDatas",list);
               }
            } catch (SchedulerException e) {
                e.printStackTrace();
            }
            map.put("runningState",getTriggerState(map.get("jobName").toString(),map.get("jobGroup").toString()));
        }
        return maps;
    }

    /**
     * 由于job建立的时候,对应的触发器的name和group也是按job的name和group创建,所以触发器和job是一对一的
     * 故可以通过触发器的状态获取job当前执行的状态
     * @param jobName
     * @param jobGroup
     * @return
     */
    private String getTriggerState(String jobName,String jobGroup){
        TriggerKey triggerKey = TriggerKey.triggerKey(jobName+"_trigger",jobGroup);//获取触发器的唯一表示TriggerKey
        String triggerState;
        try {
            triggerState = scheduler.getTriggerState(triggerKey).name();
        } catch (SchedulerException e) {
            e.printStackTrace();
            triggerState = Trigger.TriggerState.ERROR.name();
        }
        return triggerState;
    }


    /**
     * 删除jobInformation的信息时,也对应的删除调度器中的job
     * @param dto
     * @throws SchedulerException
     */
    @Override
    public void deleteInformationAndJob(List<JobInformation> dto) throws SchedulerException {
        for (JobInformation information:
             dto) {
            mapper.deleteByPrimaryKey(information);
            scheduler.deleteJob(new JobKey(information.getJobName(),information.getJobGroup()));
        }
    }

    @Override
    public void pauseJob(JobInformation dto) throws SchedulerException {
        dto = mapper.selectOne(dto);
        scheduler.pauseJob(JobKey.jobKey(dto.getJobName(),dto.getJobGroup()));
    }

    @Override
    public void resumeJob(JobInformation dto) throws SchedulerException {
        dto = mapper.selectOne(dto);
        scheduler.resumeJob(JobKey.jobKey(dto.getJobName(),dto.getJobGroup()));
    }

    //时间转换
    private static String dateFormat(Date date){
        String time = "";
        if (date!=null) {
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            time = formatter.format(date);
        }
        return time;
    }
}