package hls.core.job.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.exception.FieldRequiredException;
import com.hand.hap.job.AbstractJob;
import com.hand.hap.job.dto.JobCreateDto;
import com.hand.hap.job.exception.JobException;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.job.dto.HlsJobDto;
import hls.core.job.dto.JobInformation;
import hls.core.job.dto.TaskParam;
import hls.core.job.mapper.SchTaskMapper;
import hls.core.job.mapper.TaskParamMapper;
import hls.core.job.service.IJobInformationService;
import hls.core.job.service.ISchTaskService;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.job.dto.JobTask;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.HashMap;
import java.util.List;

import static com.hand.hap.job.exception.JobException.JOB_EXCEPTION;
import static com.hand.hap.job.exception.JobException.NOT_SUB_CLASS;

@Service
@Transactional
public class SchTaskServiceImpl extends BaseServiceImpl<JobTask> implements ISchTaskService {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IJobInformationService jobInformationService;

    @Autowired
    private SchTaskMapper mapper;



    @Override
    public List<JobTask> queryTaskByLov(IRequest request, JobTask condition, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum,pageSize);
        HashMap<String,Long> map = new HashMap();
        map.put("roleId",request.getRoleId());
        Long taskId;
        if ((taskId = condition.getTaskId())!=null)
            map.put("taskId",taskId);
        return mapper.queryTaskByLov(map);
    }



    /**
     * 将job的创建信息保存至jobinformation表之中
     * @param hlsJobDto
     * @param requestCtx
     */
    public Long saveJobInformation(HlsJobDto hlsJobDto, IRequest requestCtx) {
        JobInformation information = new JobInformation();

        JobCreateDto jobCreateDto = hlsJobDto.getJobCreateDto();

        information.setTaskId(hlsJobDto.getTaskId());
        information.setCronExpression(jobCreateDto.getCronExpression());
        information.setJobName(jobCreateDto.getJobName());
        information.setJobGroup(jobCreateDto.getJobGroup());
        information.setScheduleName(hlsJobDto.getScheduleName());
        information.setSubmitType(jobCreateDto.getTriggerType());
        information.setTaskClassName(jobCreateDto.getJobClassName());
        information.setDescription(jobCreateDto.getDescription());

        information = jobInformationService.self().insertSelective(requestCtx,information);

        return information.getJobId();
    }

}