package hls.core.lon.components;

import com.hand.hap.core.IRequest;
import hls.core.lon.dto.HlsCusCreditContract;
import hls.core.lon.dto.HlsCusLonContract;
import hls.core.lon.mapper.HlsCusCreditContractMapper;
import hls.core.lon.mapper.HlsCusLonContractMapper;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventCommonService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.sys.service.SysUserService;
import org.activiti.engine.delegate.DelegateExecution;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

abstract class AbstractLonCreditContractEventHandler implements SysEventCommonService {
    private static final String DOCUMENT_CATEGORY = "LON_CREDIT_CONTRACT";

    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private HlsCusCreditContractMapper creditContractMapper;

    public String getMessage(String userDesc, String documentName, String documentNumber) {
        return userDesc + getSuffixMsg() + documentName + "(" + documentNumber + ") " + "。";
    }

    abstract String getSuffixMsg();

    abstract String getNoticeTitle();

    public long getLevel() {
        return 2L;
    }

    public String getSourceModel() {
        return SysEventCodeUtil.PROPERTITY_SOURCEMODULE_LON;
    }

    public String getRelation() {
        return SysEventCodeUtil.PROPERTITY_RELATIONTYPE_P2D;
    }

    @Override
    public String getDocumentCategory() {
        return null;
    }

    public String getDocumentType() {
        return "";
    }

    @Override
    public void process(IRequest iRequest, Long documentId, String documentCategory, String documentType, Map<String, Object> params) {
        if (params == null) {
            params = new HashMap<>();
        }
        HlsCusCreditContract creditContract = creditContractMapper.selectByPrimaryKey(documentId);
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String message = getMessage(sysUser.getDescription(), creditContract.getCreditContractName(), creditContract.getCreditContractNumber());
        params.put("message", message);
        params.put("noticeTitle", getNoticeTitle());
        params.put("level", getLevel());
        params.put("noticeType", "NOTICE");
        sysEventService.eventSave(iRequest, documentId, DOCUMENT_CATEGORY, getDocumentType(), getSourceModel(), getEventCode(), getRelation(), params);
    }

    @Override
    public void notice(DelegateExecution delegateExecution, String status) {

    }
}
