package hls.core.lon.controllers;

import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.lon.dto.HlsCusLonContract;
import hls.core.lon.dto.HlsCusLonContractPkg;
import hls.core.lon.dto.LonContractPkg;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.lon.dto.LonContract;
import hls.core.lon.service.ILonContractService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.validation.BindingResult;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Controller
public class LonContractController extends BaseController {

    @Autowired
    private ILonContractService service;

    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;


    @RequestMapping(value = "/lon/contract/query")
    @ResponseBody
    public ResponseData query(HlsCusLonContract dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/lon/contract/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusLonContract> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/lon/contract/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusLonContract> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/lon/contract/board")
    @ResponseBody
    public ResponseData query4LonConBoard(HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = requestContext.getCompanyId();
        return new ResponseData(service.queryLonConBoardData(companyId));
    }

    @RequestMapping(value = "/lon/contract/list")
    @ResponseBody
    public ResponseData query4LonConList(HlsCusLonContract dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                         @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.queryLonContract(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/lon/contract/create")
    @ResponseBody
    public HlsCusLonContractPkg createLonContract(HttpServletRequest request, @RequestBody(required = false) HlsCusLonContractPkg contractPkg) {
        IRequest requestContext = createRequestContext(request);
        if (contractPkg != null)
            try {
                service.saveLonContract(requestContext, contractPkg);
                contractPkg.setSuccess(true);
            } catch (Exception e) {
                contractPkg.setSuccess(false);
                contractPkg.setMessage(e.getMessage());
            }
        return contractPkg;
    }

    @RequestMapping(value = "/lon/contract/formData")
    @ResponseBody
    public HlsCusLonContractPkg getLonContractFormData(HttpServletRequest request, @RequestBody HlsCusLonContract lonContract) {
        IRequest requestContext = createRequestContext(request);
        return service.selectLonContractFormData(requestContext, lonContract);
    }

    /*提交审批*/
    @RequestMapping(value = "/lon/contract/submitApproval")
    @ResponseBody
    public ResponseData submitApproval(@RequestBody List<HlsCusLonContract> list, HttpServletRequest request) {
        IRequest requestCtx = createRequestContext(request);
        List<HlsCusLonContract> result = service.submitApproval(requestCtx, list);
        return new ResponseData(result);
    }

    @RequestMapping(value = "/lon/contract/detail")
    @ResponseBody
    public HlsCusLonContract getLonContractDetail(HttpServletRequest request, @RequestBody(required = false) HlsCusLonContract dto) {
        IRequest requestCtx = createRequestContext(request);
        return service.queryLonContractDetail(requestCtx, dto);
    }

    @RequestMapping(value = "/lon/contract/event")
    @ResponseBody
    public List<HlsSystemNotice> getLonContractEvent(HlsSystemNotice hlsSystemNotice, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                     @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestCtx = createRequestContext(request);
        hlsSystemNotice.setCompanyId(requestCtx.getCompanyId());
        return hlsSystemNoticeService.selectLonNotice(hlsSystemNotice, page, pageSize);
    }

    @RequestMapping(value = "/lon/contract/lonContractQuery")
    @ResponseBody
    public ResponseData lonContractQuery(HlsCusLonContract lonContract, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HttpSession session){
        lonContract.setCompanyId((Long) session.getAttribute("companyId"));
        IRequest requestContext = createRequestContext(request);
        return  new ResponseData(service.lonContractQuery(requestContext,lonContract,page,pageSize));
    }

}