package hls.core.lon.controllers;

import com.hand.hap.system.dto.DTOStatus;
import hls.core.gld.dto.HlsCusGldLonContractTrx;
import hls.core.gld.service.IGldLonContractTrxService;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.dto.HlsCusLonContractWithdraw;
import org.apache.http.HttpRequest;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.lon.service.ILonContractRepaymentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.validation.BindingResult;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.LongStream;

@Controller
public class LonContractRepaymentController extends BaseController {

    public static final String NOT = "NOT";
    public static final String RELEASE = "RELEASE";

    @Autowired
    private ILonContractRepaymentService service;

    @Autowired
    private IGldLonContractTrxService gldLonContractTrxService;


    @RequestMapping(value = "/lon/contract/repayment/query")
    @ResponseBody
    public ResponseData query(HlsCusLonContractRepayment dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.selectLonContractRep(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/lon/contract/repayment/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusLonContractRepayment> dto,Boolean changeFlag, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        List<HlsCusLonContractRepayment> resultList = service.batchUpdate(requestCtx, dto);
        List<Long> list = new ArrayList<>();
        resultList.forEach(item -> list.add(item.getRepaymentId()));
        //重算计息余额
        resultList = service.resetInterestBalance(resultList);
        //更新表
        if (!resultList.isEmpty() && changeFlag){
            HlsCusLonContractRepayment repayment = resultList.stream().findAny().get();
            //插入融资合同成本事物
            HlsCusGldLonContractTrx gldLonContractTrx = new HlsCusGldLonContractTrx();
            gldLonContractTrx.setCompanyId(requestCtx.getCompanyId());
            gldLonContractTrx.setContractId(repayment.getContractId());
            gldLonContractTrx.setSourceType("LOAN_CHANGE");
            gldLonContractTrx.setSourceId(repayment.getWithdrawId());
            List<HlsCusGldLonContractTrx> select = gldLonContractTrxService.select(requestCtx, gldLonContractTrx, 1, 10);
            if(select.isEmpty()){
                gldLonContractTrx.setStatus("NEW");
                gldLonContractTrxService.insertSelective(requestCtx,gldLonContractTrx);
            }else {
                gldLonContractTrx.setContractTrxId(select.stream().findFirst().get().getContractTrxId());
                gldLonContractTrx.setStatus("NEW");
                gldLonContractTrxService.updateByPrimaryKeySelective(requestCtx,gldLonContractTrx);
            }
        }
        resultList = service.batchUpdate(requestCtx, resultList);
        final List<HlsCusLonContractRepayment> repayments = new ArrayList<>();
        resultList.forEach(item -> {
            if (list.contains(item.getRepaymentId()))
                repayments.add(item);
        });
        return new ResponseData(repayments);
    }

    @RequestMapping(value = "/lon/contract/repayment/default/submit")
    @ResponseBody
    public ResponseData defaultUpdate(@RequestBody List<HlsCusLonContractRepayment> dto, BindingResult result, HttpServletRequest request){
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        dto.forEach(item -> {
            if (DTOStatus.ADD.equalsIgnoreCase(item.get__status())){
                item.setDueAmount(item.getPlannedDueAmount());
                item.setDueDate(item.getPlannedDueDate());
                item.setWriteOffFlag(NOT);
                item.setCfStatus(RELEASE);
            }else if (DTOStatus.UPDATE.equalsIgnoreCase(item.get__status())){
                item.setDueAmount(item.getPlannedDueAmount());
                item.setDueDate(item.getPlannedDueDate());
            }
        });
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/lon/contract/repayment/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusLonContractRepayment> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/lon/contract/repayment/default/query")
    @ResponseBody
    public ResponseData queryByDefault(HlsCusLonContractRepayment dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/lon/contract/repayment/queryRep")
    @ResponseBody
    public ResponseData queryRep(@RequestBody HlsCusLonContractRepayment hlsCusLonContractRepayment, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        hlsCusLonContractRepayment.setCompanyId(companyId);
        return new ResponseData(service.queryRep(requestContext, hlsCusLonContractRepayment, page, pageSize));
    }

    @RequestMapping(value = "/lon/contract/repayment/calc")
    @ResponseBody
    public ResponseData calc4Repayment(@RequestBody HlsCusLonContractWithdraw lonContractWithdraw, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.createRepaymentRecord(requestContext, lonContractWithdraw));
    }

    /*提交审批*/
    @RequestMapping("/lon/contract/repayment/approval")
    @ResponseBody
    public ResponseData paymentApproval(HttpServletRequest httpRequest, @RequestBody List<HlsCusLonContractRepayment> list, HttpSession session) {
        ResponseData data = new ResponseData();
        IRequest iRequest = createRequestContext(httpRequest);
        try {
            service.repaymentApproval(iRequest, session, list);
            data = new ResponseData(true);
        } catch (Exception e) {
            data = new ResponseData(false);
            data.setMessage(e.getMessage());
        }
        return data;
    }

    /**
     * 还款情况明细(如果没有对应的还款数据,则根据初始化算法计算)
     *
     * @param request
     * @param lonContractRepayment
     * @return
     */
    @RequestMapping(value = "/lon/contract/repayment/detail")
    @ResponseBody
    public ResponseData withdrawDetail(HttpServletRequest request, HlsCusLonContractRepayment lonContractRepayment, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.getOrCreateLonContractRepayment(requestContext, lonContractRepayment, page, pageSize));
    }


    /*还款综合查询*/
    @RequestMapping(value = "/lon/contract/repayment/unitQueryRep")
    @ResponseBody
    public ResponseData unitQueryRep(HttpServletRequest request,HlsCusLonContractRepayment hlsCusLonContractRepayment, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                     @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize,HttpSession session){
        hlsCusLonContractRepayment.setCompanyId((Long) session.getAttribute("companyId"));
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.unitQueryRep(requestContext,hlsCusLonContractRepayment,page,pageSize));
    }
}