package hls.core.lon.controllers;

import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.lon.dto.HlsCusLonContractWithdraw;
import org.apache.http.HttpRequest;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.lon.service.ILonContractWithdrawService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.springframework.validation.BindingResult;

import java.util.List;
import java.util.Map;

@Controller
public class LonContractWithdrawController extends BaseController {

    @Autowired
    private ILonContractWithdrawService service;

    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;


    @RequestMapping(value = "/lon/contract/withdraw/query")
    @ResponseBody
    public ResponseData query(HlsCusLonContractWithdraw dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/lon/contract/withdraw/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<HlsCusLonContractWithdraw> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        ResponseData rs;
        try{
            List<HlsCusLonContractWithdraw> list = service.batchUpdate4Withdraw(requestCtx, dto);
            rs = new ResponseData(list);
        }catch (IllegalArgumentException e){
            rs = new ResponseData(false);
            rs.setMessage(e.getMessage());
        }
        return rs;
    }

    @RequestMapping(value = "/lon/contract/withdraw/default/submit")
    @ResponseBody
    public ResponseData defaultUpdate(@RequestBody List<HlsCusLonContractWithdraw> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }


    @RequestMapping(value = "/lon/contract/withdraw/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusLonContractWithdraw> dto) {
        for (HlsCusLonContractWithdraw item :dto) {
            if (item == null || !"NEW".equalsIgnoreCase(item.getWithdrawStatus())){
//                ResponseData rs = new ResponseData(false);
//                rs.setMessage("只允许删除新建状态的提款");
                return new ResponseData();
            }
        }
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/lon/contract/withdraw/default/remove")
    @ResponseBody
    public ResponseData defaultDelete(HttpServletRequest request, @RequestBody List<HlsCusLonContractWithdraw> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/lon/contract/withdraw/list")
    @ResponseBody
    public ResponseData query4LonConWithdrawList(HlsCusLonContractWithdraw dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.queryContractWithdraw(requestCtx, dto, page, pageSize));
    }

    @RequestMapping(value = "/lon/contract/withdraw/board")
    @ResponseBody
    public Map query4LonWdBoard(HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = requestContext.getCompanyId();
        return service.queryLonWdBoardData(companyId);
    }

    @RequestMapping(value = "/lon/contract/withdraw/lineChart")
    @ResponseBody
    public ResponseData query4LonWdLineChart(HttpServletRequest request, HlsCusLonContractWithdraw dto) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.queryLonWdLineChartData(requestContext, dto));
    }

    @RequestMapping(value = "/lon/contract/withdraw/selectwd")
    @ResponseBody
    public ResponseData selectWd(HttpServletRequest request, @RequestBody HlsCusLonContractWithdraw hlsCusLonContractWithdraw, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpSession session) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = (Long) session.getAttribute("companyId");
        hlsCusLonContractWithdraw.setCompanyId(companyId);
        return new ResponseData(service.selectWd(requestContext, hlsCusLonContractWithdraw, page, pageSize));
    }

    /**
     * 提款工作流启动
     *
     * @param request
     * @param lonContractWithdraw
     * @return
     */
    @RequestMapping(value = "/lon/contract/withdraw/workflow", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData withdrawWorkflowStart(HttpServletRequest request, @RequestBody HlsCusLonContractWithdraw lonContractWithdraw) {
        IRequest requestContext = createRequestContext(request);
        ResponseData rs;
        try {
            service.lonContractWithdrawWorkFlowStart(requestContext, lonContractWithdraw);
            rs = new ResponseData();
            rs.setMessage("工作流提交成功");
        } catch (Exception e) {
            rs = new ResponseData(false);
            rs.setMessage(e.getMessage());
        }
        return rs;
    }

    @RequestMapping(value = "/lon/contract/withdraw/event")
    @ResponseBody
    public List<HlsSystemNotice> getLonContractWithdrawEvent(HlsSystemNotice hlsSystemNotice, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                     @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestCtx = createRequestContext(request);
        hlsSystemNotice.setCompanyId(requestCtx.getCompanyId());
        return hlsSystemNoticeService.selectLonWithdrawNotice(hlsSystemNotice, page, pageSize);
    }

    /*-----------融资提款综合查询-----------*/
    @RequestMapping(value = "/lon/contract/withdraw/WithdrawComprehensiveQuery")
    @ResponseBody
    public ResponseData WithdrawComprehensiveQuery(HlsCusLonContractWithdraw dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.withdrawComprehensiveQuery(requestCtx, dto, page, pageSize));
    }
}