package hls.core.lon.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.hls.components.CalculateUtil;
import hls.core.lon.dto.HlsCusCreditContract;
import hls.core.lon.dto.HlsCusCreditLineAttachment;
import hls.core.lon.mapper.HlsCusCreditContractMapper;
import hls.core.lon.service.ICreditLineAttachmentService;
import hls.core.lon.service.IUpdateCreditContractService;
import hls.core.lon.service.IUpdateCreditLineAttachService;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.service.HlsCreditLineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.lon.dto.CreditContract;
import hls.core.lon.service.ICreditContractService;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional(rollbackFor = Exception.class)
public class CreditContractServiceImpl extends BaseServiceImpl<HlsCusCreditContract> implements ICreditContractService {

    @Autowired
    private HlsCusCreditContractMapper creditContractMapper;
    @Autowired
    private HlsCreditLineService hlsCreditLineService;
    @Autowired
    private ICreditLineAttachmentService iCreditLineAttachmentService;
    @Autowired(required = false)
    private IUpdateCreditContractService updateCreditContractService;
    @Autowired(required = false)
    private IUpdateCreditLineAttachService updateCreditLineAttachService;

    @Override
    public void save(IRequest iRequest, HlsCusCreditContract creditContract) {
        if (updateCreditContractService != null && !updateCreditContractService.beforeUpdate(iRequest, creditContract)) {
            return;
        }
        HlsCusHlsCreditLine HlsCusHlsCreditLine = new HlsCusHlsCreditLine();
        creditContract.setBusinessType("LON_CREDIT_CONTRACT");
        HlsCusHlsCreditLine.setDocumentType(creditContract.getDocumentType2());
        HlsCusHlsCreditLine.setCreditLineName(creditContract.getCreditContractName());
        HlsCusHlsCreditLine.setCreditLineAmt(Double.valueOf(creditContract.getCreditLineAmt()));
        HlsCusHlsCreditLine.setValidFrom(creditContract.getValidFrom());
        HlsCusHlsCreditLine.setValidTo(creditContract.getValidTo());
        HlsCusHlsCreditLine.setBpId(creditContract.getCreditBpId());
        HlsCusHlsCreditLine.setDocumentCategory("HLS_CREDIT_LINE");
        HlsCusHlsCreditLine.setBusinessType("FINANCING_LOAN");
        HlsCusHlsCreditLine.setCreditLineStatus(creditContract.getCreditContractStatus());
        HlsCusHlsCreditLine.setCompanyId(creditContract.getCompanyId());
        HlsCusHlsCreditLine.setProposerUserId(creditContract.getProposerUserId());
        HlsCusHlsCreditLine.setProposerEmployeeId(creditContract.getProposerEmployeeId());
        HlsCusHlsCreditLine.setProposerEmployeeAssignId(creditContract.getProposerEmployeeAssignId());
        HlsCusHlsCreditLine.setCurrencyCode(String.valueOf(creditContract.getCurrency()));
        HlsCusHlsCreditLine.setCreditLineId(creditContract.getCreditLineId());
        if (creditContract.getCreditLineId() != null) {
            hlsCreditLineService.updateByPrimaryKeySelective(iRequest, HlsCusHlsCreditLine);
        } else {
            hlsCreditLineService.insertSelective(iRequest, HlsCusHlsCreditLine);
            creditContract.setCreditLineId(HlsCusHlsCreditLine.getCreditLineId());
        }
        if (creditContract.getCreditContractId() != null) {
            self().updateByPrimaryKeySelective(iRequest, creditContract);
        } else {
            self().insertSelective(iRequest, creditContract);
        }
        if (creditContract.getCreditLineAttachments() != null) {
            for (HlsCusCreditLineAttachment creditLineAttachment : creditContract.getCreditLineAttachments()) {
                if (updateCreditLineAttachService != null && !updateCreditLineAttachService.beforeUpdate(iRequest, creditLineAttachment)) {
                    return;
                }
                if (creditLineAttachment.getCreditLineAttachmentId() != null) {
                    creditLineAttachment.setCreditLineId(creditContract.getCreditLineId());
                    creditLineAttachment.setBpId(creditContract.getCreditBpId());
                    creditLineAttachment.setBpCategory(creditContract.getBpCategory());
                    iCreditLineAttachmentService.updateByPrimaryKeySelective(iRequest, creditLineAttachment);
                } else {
                    creditLineAttachment.setCreditLineId(creditContract.getCreditLineId());
                    creditLineAttachment.setBpId(creditContract.getCreditBpId());
                    creditLineAttachment.setBpCategory(creditContract.getBpCategory());
                    iCreditLineAttachmentService.insertSelective(iRequest, creditLineAttachment);
                }
                if (updateCreditLineAttachService != null) {
                    updateCreditLineAttachService.afterUpdate(iRequest, creditLineAttachment);
                }
            }
        }
        if (updateCreditContractService != null) {
            updateCreditContractService.afterUpdate(iRequest, creditContract);
        }
    }

    @Override
    public List<HlsCusCreditContract> unitSelect(HlsCusCreditContract creditContract, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        List<HlsCusCreditContract> list = creditContractMapper.unitSelect(creditContract);
        for (HlsCusCreditContract contract : list) {
            Double openToBuy = setNoUsedAmt(contract.getCreditLineAmt(), contract.getCreditExposureAmt(), contract.getDocumentType2());
            contract.setOpenToBuy(openToBuy);
        }
        return list;
    }

    @Override
    public List<HlsCusCreditContract> selectBp(HlsCusCreditContract creditContract) {
        List<HlsCusCreditContract> list = creditContractMapper.selectBp(creditContract);

        return list;
    }

    @Override
    public List<HlsCusCreditContract> selectBpChart(HlsCusCreditContract creditContract) {
        int year = Math.toIntExact(creditContract.getYear());
        String bpName = creditContract.getBpName();
        Long companyId = creditContract.getCompanyId();
        String currencyName = creditContract.getCurrencyName();
        Map map1 = new HashMap();
        map1.put("year", year);
        map1.put("bpName", bpName);
        map1.put("companyId", companyId);
        map1.put("currencyName", currencyName);
        List<HlsCusCreditContract> list = new ArrayList<HlsCusCreditContract>();
        for (int i = 0; i < 12; i++) {
            map1.put("month", i + 1);
            List<HlsCusCreditContract> result = creditContractMapper.selectBpChart(map1);
            if (result.size() != 0) {
                Double sumAmt = Double.valueOf(0);
                Double usedCreditAmt = Double.valueOf(0);
                Double creditAmt = Double.valueOf(0);
                for (CreditContract contract : result
                        ) {
                    usedCreditAmt += contract.getUsedAmt();
                    creditAmt += contract.getCreditLineAmt();
                }
                sumAmt = CalculateUtil.sub(creditAmt, usedCreditAmt);
                result.get(0).setUsedAmt(usedCreditAmt);
                result.get(0).setNoUsedAmt(sumAmt);
                result.get(0).setCreditLineAmt(creditAmt);
                list.add(result.get(0));
            } else {
                list.add(new HlsCusCreditContract());
            }
        }
        return list;
    }

    @Override
    public List<HlsCusCreditContract> selectBpName(HlsCusCreditContract creditContract) {
        return creditContractMapper.selectBpName(creditContract);
    }


    /*
    * creditLineAmt 额度
    * withdrawAmt 提款金额
    * repaymentAmt 还款金额
    * documentType 额度单据类型
    * */
    public Double setNoUsedAmt(Double creditLineAmt, Double withdrawAmt, Double repaymentAmt, String documentType) {
        Double noUsedAmt = null;
        if ("REVOLVING".equals(documentType)) {
            noUsedAmt = CalculateUtil.add(CalculateUtil.sub(creditLineAmt, withdrawAmt), repaymentAmt);
        } else if ("NON_REVOLVING".equals(documentType)) {
            noUsedAmt = CalculateUtil.sub(creditLineAmt, withdrawAmt);
        } else if ("ONCE".equals(documentType)) {
            if (!withdrawAmt.equals(Double.valueOf(0))) {
                noUsedAmt = Double.valueOf(0);
            } else {
                noUsedAmt = creditLineAmt;
            }
        }
        return noUsedAmt;
    }

    public Double setNoUsedAmt(Double creditLineAmt, Double creditExposureAmt, String documentType) {
        Double noUsedAmt = null;
        if ("REVOLVING".equals(documentType)) {
            noUsedAmt = CalculateUtil.sub(creditLineAmt, creditExposureAmt);
        } else if ("NON_REVOLVING".equals(documentType)) {
            noUsedAmt = CalculateUtil.sub(creditLineAmt, creditExposureAmt);
        } else if ("ONCE".equals(documentType)) {
            if (!creditExposureAmt.equals(Double.valueOf(0))) {
                noUsedAmt = Double.valueOf(0);
            } else {
                noUsedAmt = creditLineAmt;
            }
        }
        return noUsedAmt;
    }

    /*合同创建维护-lov框*/
    public List<HlsCusCreditContract> queryLovInfo(HlsCusCreditContract creditContract, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        return creditContractMapper.queryLovInfo(creditContract);
    }

    public void delete(IRequest iRequest, HlsCusCreditContract creditContract) {
        Long creditLineId = creditContract.getCreditLineId();
        creditContractMapper.deleteFromCreditLine(creditLineId);
        creditContractMapper.deleteFromCreditContract(creditLineId);
    }
}