package hls.core.lon.service.impl;

import hls.core.hls.components.CalculateUtil;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.service.ILonContractInterestCalcService;
import hls.core.lon.utils.LocalDateUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

/**
 * 到期一次性还本
 *
 * Created by fjm on 2017/11/13.
 */
@Service
public class LonContractOneTimeCalcServiceImpl implements ILonContractInterestCalcService {

    private static final String OUTFLOW = "OUTFLOW";
    public static final String NOT = "NOT";
    public static final String ONE_TIME = "ONE_TIME";

    public String getInterestCalcMethodName() {
        return ONE_TIME;
    }

    public List<HlsCusLonContractRepayment> getLonContractRepaymentsByCalc(List<HlsCusLonContractRepayment> lonContractRepaymentList, Double financeAmount, Double loanRate) {
        List<HlsCusLonContractRepayment> result = new ArrayList<>();
        lonContractRepaymentList.forEach(item -> {
            Date plannedDueDate = item.getPlannedDueDate();
            LocalDate startLocalDate = item.getStartLocalDate();
            LocalDate endLocalDate = LocalDateUtils.date2LocalDate(plannedDueDate);
            long between = ChronoUnit.DAYS.between(startLocalDate, endLocalDate);
            Double resultAmount = CalculateUtil.mul(financeAmount, loanRate);
            resultAmount = CalculateUtil.mul(resultAmount, Double.valueOf(between));
            resultAmount = CalculateUtil.div(resultAmount, 360D);
            item.setInterestPeriodDays(between);
            item.setDueAmount(resultAmount);
            item.setPlannedDueAmount(resultAmount);
            item.setWriteOffFlag(NOT);
            item.setCfType(30L);//还款
            item.setCfItem(302L);//利息
            item.setCfDirection(OUTFLOW);
            item.setInterestAccrualBalance(financeAmount);
            result.add(item);
        });
        lonContractRepaymentList.get(lonContractRepaymentList.size()-1).setInterestAccrualBalance(0D);
        //找到最后一条还款现金流
        HlsCusLonContractRepayment lastContractRepayment = lonContractRepaymentList.stream().max(Comparator.comparingLong(HlsCusLonContractRepayment::getTimes)).get();
        HlsCusLonContractRepayment principal = new HlsCusLonContractRepayment();
        principal.setCfItem(301L);//还款
        principal.setCfType(30L);//本金
        principal.setDueAmount(financeAmount);
        principal.setPlannedDueAmount(financeAmount);
        principal.setTimes(lastContractRepayment.getTimes());
        principal.setDueDate(lastContractRepayment.getDueDate());
        principal.setPlannedDueDate(lastContractRepayment.getPlannedDueDate());
        principal.setInterestAccrualBalance(0D);
        principal.setWriteOffFlag(NOT);
        principal.setCfDirection(OUTFLOW);
        result.add(principal);
        return result;
    }

}
