package hls.core.lon.service.impl;

import hls.core.hls.components.CalculateUtil;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.service.ILonContractInterestCalcService;
import hls.core.lon.utils.LocalDateUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 归还本金法
 * <p>
 * Created by fjm on 2017/11/13.
 */
@Service
public class LonContractRepaymentCalcServiceImpl implements ILonContractInterestCalcService {

    public static final String REPAYMENT_OF_PRINCIPAL = "REPAYMENT_OF_PRINCIPAL";
    public static final String OUTFLOW = "OUTFLOW";
    public static final String NOT = "NOT";

    @Override
    public String getInterestCalcMethodName() {
        return REPAYMENT_OF_PRINCIPAL;
    }

    @Override
    public List<HlsCusLonContractRepayment> getLonContractRepaymentsByCalc(List<HlsCusLonContractRepayment> lonContractRepaymentList, Double financeAmount, Double loanRate) {
        Double leftAmount = financeAmount;
        Long paramDays = 0L;
        long maxTime = lonContractRepaymentList.stream().mapToLong(HlsCusLonContractRepayment::getTimes).max().getAsLong();//找出最大的期数
        List<HlsCusLonContractRepayment> interestList = new ArrayList<>();
        for (int i = 0; i < lonContractRepaymentList.size(); i++) {
            HlsCusLonContractRepayment lonContractRepayment = lonContractRepaymentList.get(i);
            Double plannedDueAmount = lonContractRepayment.getPlannedDueAmount();
            LocalDate startLocalDate = lonContractRepayment.getStartLocalDate();
            Date dueDate = lonContractRepayment.getPlannedDueDate();
            LocalDate dueLocalDate = LocalDateUtils.date2LocalDate(dueDate);
            long between = ChronoUnit.DAYS.between(startLocalDate, dueLocalDate);
            if (lonContractRepayment.getCfType() == 30 && lonContractRepayment.getCfItem() == 301) {//还款本金
                paramDays = between;
                if (lonContractRepayment.getTimes().compareTo(maxTime) == 0) {
                    lonContractRepayment.setDueAmount(leftAmount);
                    lonContractRepayment.setPlannedDueAmount(leftAmount);
                    //leftAmount = 0D;
                } else {
                    Double interestAmount = CalculateUtil.mul(plannedDueAmount, Double.valueOf(between));
                    interestAmount = CalculateUtil.mul(interestAmount, loanRate);
                    interestAmount = CalculateUtil.div(interestAmount, Double.valueOf(360));
                    HlsCusLonContractRepayment interestRepayment = getInterestRepayment(lonContractRepayment, interestAmount, between);
                    interestList.add(interestRepayment);
                    leftAmount = CalculateUtil.sub(leftAmount, plannedDueAmount);
                }
            } else if (lonContractRepayment.getCfType() == 30 && lonContractRepayment.getCfItem() == 302) {//还款利息

                Double interestAmount = CalculateUtil.mul(leftAmount, CalculateUtil.add(Double.valueOf(between), Double.valueOf(paramDays)));
                interestAmount = CalculateUtil.mul(interestAmount, loanRate);
                interestAmount = CalculateUtil.div(interestAmount, Double.valueOf(360));
                lonContractRepayment.setDueAmount(interestAmount);
                lonContractRepayment.setPlannedDueAmount(interestAmount);
                lonContractRepayment.setInterestPeriodDays(between);
                paramDays = 0L;
            }
        }
        final List<HlsCusLonContractRepayment> result = new ArrayList<>();
        Comparator<HlsCusLonContractRepayment> comparator = (a, b) -> {
            if (a.getPlannedDueDate().compareTo(b.getPlannedDueDate()) == 0) {
                return Long.compare(a.getCfItem(), b.getCfItem());
            }
            return a.getPlannedDueDate().compareTo(b.getPlannedDueDate());
        };
        List<HlsCusLonContractRepayment> collect = Stream.concat(lonContractRepaymentList.stream(), interestList.stream()).sorted(comparator).collect(Collectors.toList());
        for (int i = 0; i < collect.size(); i++) {
            HlsCusLonContractRepayment lonContractRepayment = collect.get(i);
            if (i > 0) {
                HlsCusLonContractRepayment lastLonContractRepayment = collect.get(i - 1);
                if (lonContractRepayment.getTimes().compareTo(lastLonContractRepayment.getTimes()) == 0 && lastLonContractRepayment.getCfItem().compareTo(lonContractRepayment.getCfItem()) == 0) {
                    Double lastPlannedDueAmount = lastLonContractRepayment.getPlannedDueAmount();
                    Double plannedDueAmount = lonContractRepayment.getPlannedDueAmount();
                    lastLonContractRepayment.setPlannedDueAmount(CalculateUtil.add(lastPlannedDueAmount,plannedDueAmount));
                    lastLonContractRepayment.setDueAmount(lastLonContractRepayment.getPlannedDueAmount());
                }else {
                    result.add(lonContractRepayment);
                }
            }else {
                result.add(lonContractRepayment);
            }
        }
        return result;
    }

    /**
     * 获取新生成的还款利息类型现金流对象
     *
     * @param param
     * @param dueAmount
     * @return
     */
    private HlsCusLonContractRepayment getInterestRepayment(HlsCusLonContractRepayment param, Double dueAmount, Long between) {
        HlsCusLonContractRepayment lonContractRepayment = new HlsCusLonContractRepayment();
        lonContractRepayment.setContractId(param.getContractId());
        lonContractRepayment.setWithdrawId(param.getWithdrawId());
        lonContractRepayment.setDueDate(param.getDueDate());
        lonContractRepayment.setPlannedDueDate(param.getPlannedDueDate());
        lonContractRepayment.setTimes(param.getTimes());
        lonContractRepayment.setCfType(30L);
        lonContractRepayment.setCfItem(302L);
        lonContractRepayment.setInterestPeriodDays(between);
        lonContractRepayment.setDueAmount(dueAmount);
        lonContractRepayment.setPlannedDueAmount(dueAmount);
        lonContractRepayment.setCfDirection(OUTFLOW);
        lonContractRepayment.setWriteOffFlag(NOT);
        return lonContractRepayment;
    }
}
