package hls.core.lon.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.csh.dto.HlsCusCshPaymentReqHd;
import hls.core.csh.dto.HlsCusCshPaymentReqLn;
import hls.core.csh.mapper.HlsCusCshPaymentReqHdMapper;
import hls.core.csh.mapper.HlsCusCshPaymentReqLnMapper;
import hls.core.csh.service.ICshPaymentReqHdService;
import hls.core.csh.service.ICshPaymentReqLnService;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.components.CalculateUtil;
import hls.core.lon.dto.*;
import hls.core.lon.mapper.HlsCusLonContractMapper;
import hls.core.lon.mapper.HlsCusLonContractQuotationMapper;
import hls.core.lon.mapper.HlsCusLonContractRepaymentMapper;
import hls.core.lon.mapper.HlsCusLonContractWithdrawMapper;
import hls.core.lon.service.ILonContractInterestCalcService;
import hls.core.lon.service.IUpdateLonContractRepaymentService;
import hls.core.lon.utils.LocalDateUtils;
import hls.core.wfl.service.IActivitiStartService;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import hls.core.lon.service.ILonContractRepaymentService;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpSession;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = Exception.class)
public class LonContractRepaymentServiceImpl extends BaseServiceImpl<HlsCusLonContractRepayment> implements ILonContractRepaymentService {

    private static final String YEAR = "YEAR";//年
    private static final String HALF_A_YEAR = "HALF_A_YEAR";//半年
    private static final String QUARTER = "QUARTER";//一季度
    private static final String MONTH = "MONTH";//一个月
    public static final String ONE_TIME = "ONE_TIME";
    public static final String REPAYMENT_OF_PRINCIPAL = "REPAYMENT_OF_PRINCIPAL";
    public static final String RESIDUAL_OF_PRINCIPAL = "RESIDUAL_OF_PRINCIPAL";

    @Autowired
    private HlsCusLonContractRepaymentMapper lonContractRepaymentMapper;
    @Autowired
    private HlsCusLonContractQuotationMapper lonContractQuotationMapper;
    @Autowired
    private HlsCusLonContractMapper lonContractMapper;
    @Autowired
    private List<ILonContractInterestCalcService> calcServices;
    @Autowired
    private HlsCusCshPaymentReqHdMapper cshPaymentReqHdMapper;
    @Autowired
    private HlsCusCshPaymentReqLnMapper cshPaymentReqLnMapper;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private FndCodingRuleValuesService codingRuleValuesService;
    @Autowired
    private HlsCusLonContractWithdrawMapper lonContractWithdrawMapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private ICshPaymentReqHdService paymentReqHdService;
    @Autowired
    private ICshPaymentReqLnService cshPaymentReqLnService;
    @Autowired(required = false)
    private IUpdateLonContractRepaymentService updateLonContractRepaymentService;

    @Override
    public List<HlsCusLonContractRepayment> selectLonContractRep(IRequest request, HlsCusLonContractRepayment lonContractRepayment, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        //构造参数
        if (!"".equals(lonContractRepayment.getRepaymentIds()) && lonContractRepayment.getRepaymentIds() != null) {
            lonContractRepayment.setRepaymentIdList(lonContractRepayment.getRepaymentIds().split("-"));
        }
        List<HlsCusLonContractRepayment> list = lonContractRepaymentMapper.selectLonContractRep(lonContractRepayment);

        //获取已申请金额
        for (HlsCusLonContractRepayment item : list) {
            Double approvedAmount = 0D;
            HlsCusCshPaymentReqLn reqLn = new HlsCusCshPaymentReqLn();
            reqLn.setCompanyId(item.getCompanyId());
            reqLn.setSource_doc_category("LON_CONTRACT_WITHDRAW");
            reqLn.setSource_doc_id(item.getWithdrawId());
            reqLn.setSource_doc_line_id(item.getRepaymentId());
            //获取当前还款id的所有付款申请行信息
            List<HlsCusCshPaymentReqLn> reqList = cshPaymentReqLnMapper.select(reqLn);
            if (reqList.size() > 0) {
                for (HlsCusCshPaymentReqLn req : reqList) {
                    //判断对应的头上的申请状态
                    HlsCusCshPaymentReqHd hd = new HlsCusCshPaymentReqHd();
                    hd.setPayment_req_id(req.getPayment_req_id());
                    String paymentStatus = "";
                    if (cshPaymentReqHdMapper.selectOne(hd) != null) {
                        paymentStatus = cshPaymentReqHdMapper.selectOne(hd).getPayment_req_status();
                    }

                    //如果申请状态不是被拒绝 计算申请总额
                    if (!"REJECTED".equals(paymentStatus)) {
                        approvedAmount = CalculateUtil.add(approvedAmount, req.getAmount());
                    }
                    //如果获取了参数 申请头ID 判断当前申请行是否属于当前申请头  如果属于 获取本次申请金额和申请行ID(明细页面)
                    if (lonContractRepayment.getPaymentReqId() != null && lonContractRepayment.getPaymentReqId().compareTo(0L) != 0 && req.getPayment_req_id().compareTo(lonContractRepayment.getPaymentReqId()) == 0) {
                        item.setApplyAmount(req.getAmount());
                        item.setPaymentReqId(lonContractRepayment.getPaymentReqId());
                        item.setPaymentReqLineId(req.getPayment_req_ln_id());
                        item.setPaymentStatus(paymentStatus);

                    }
                }
            } else {
                item.setPaymentStatus("NEW");
            }
            //获取当前还款行的已申请金额
            item.setApprovedAmount(approvedAmount);
            //如果本次申请金额为null 初始化本次申请金额为 实际还款金额-已申请金额
            if (item.getApplyAmount() == null) {
                item.setApplyAmount(CalculateUtil.sub(item.getDueAmount(), item.getApprovedAmount()));
            }
        }
        return list;
    }

    @Override
    public List<HlsCusLonContractRepayment> getOrCreateLonContractRepayment(IRequest request, HlsCusLonContractRepayment lonContractRepayment, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        List<HlsCusLonContractRepayment> result = lonContractRepaymentMapper.selectLonContractRep(lonContractRepayment);
        return result;
    }

    @Override
    public List<HlsCusLonContractRepayment> queryRep(IRequest request, HlsCusLonContractRepayment hlsCusLonContractRepayment, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        return lonContractRepaymentMapper.queryRep(hlsCusLonContractRepayment);
    }

    @Override
    public List<HlsCusLonContractRepayment> createRepaymentRecord(IRequest request, HlsCusLonContractWithdraw lonContractWithdraw) {
        ILonContractInterestCalcService calcService = null;
        List<HlsCusLonContractRepayment> groupResultList = null;//单独构建分组后的list,修改原list导致影响主体逻辑
        Long contractId = lonContractWithdraw.getContractId();
        Long withdrawId = lonContractWithdraw.getWithdrawId();
        lonContractWithdraw = lonContractWithdrawMapper.selectByPrimaryKey(withdrawId);
        HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(contractId);
        HlsCusLonContractQuotation contractQuotation = new HlsCusLonContractQuotation();
        contractQuotation.setContractId(contractId);
        contractQuotation = lonContractQuotationMapper.selectOne(contractQuotation);
        HlsCusLonContractRepayment lonContractRepayment = new HlsCusLonContractRepayment();
        lonContractRepayment.setWithdrawId(withdrawId);
        lonContractRepayment.setContractId(contractId);
        List<HlsCusLonContractRepayment> lonContractRepaymentList = mappers.get(0).select(lonContractRepayment);
        Double financeAmount = lonContractWithdraw.getDueAmount();//融资额
        Double loanRate = contractQuotation.getLoanRate();//执行利率
        Date startActiveDate = contractQuotation.getStartActiveDate();//开始日期
        Date endActiveDate = contractQuotation.getEndActiveDate();//结束日期
        Long loanTimes = contractQuotation.getLoanTimes();//融资期数
        String interestCycle = contractQuotation.getInterestCycle();//结息周期
        Long interestCalcDate = contractQuotation.getInterestCalcDate();//结息日期
        String interestCalcMethod = contractQuotation.getInterestCalcMethod();//计息方式
        if (lonContractRepaymentList.isEmpty() || lonContractRepaymentList == null || ONE_TIME.equalsIgnoreCase(interestCalcMethod)) {//当提款记录下没有对应的还款现金流记录时,生成新的记录
            interestCalcMethod = ONE_TIME;
            HlsCusLonContractRepayment repayment = new HlsCusLonContractRepayment();
            repayment.setWithdrawId(withdrawId);
            lonContractRepaymentMapper.delete(repayment);
            //获取根据期数构建的现金流(未插表)
            groupResultList = getRepaymentRecordsByInterestCycle(loanTimes, interestCalcDate, startActiveDate, endActiveDate, interestCycle);
        } else {//如果已有记录,则根据保存的记录,重新对还款现金流进行分组
//            if (lonContractRepaymentList.stream().filter(item -> item.getCfItem().compareTo(301L) == 0).collect(Collectors.toList()).size() == 1) {//如果其中只有一条本金,那么是原始记录,不计算直接返回
//                return lonContractRepaymentList;
//            }
            if (interestCalcMethod.equalsIgnoreCase(REPAYMENT_OF_PRINCIPAL)) {
                mappers.get(0).delete(lonContractRepayment);
                List<HlsCusLonContractRepayment> collect = lonContractRepaymentList.stream().filter(item -> {
                    item.setRepaymentId(null);
                    return item.getCfType() == 30 && item.getCfItem() == 301;
                }).collect(Collectors.toList());
                HlsCusLonContractRepayment maxTimeRepayment = collect.stream().max(Comparator.comparingLong(item -> item.getTimes() == null ? 0L : item.getTimes())).get();
                collect.remove(maxTimeRepayment);
                groupResultList = getRepaymentRecordsByInterestCycle(loanTimes, interestCalcDate, startActiveDate, endActiveDate, interestCycle);
                for (ILonContractInterestCalcService item : calcServices) {
                    if (ONE_TIME.equalsIgnoreCase(item.getInterestCalcMethodName())) {
                        groupResultList = item.getLonContractRepaymentsByCalc(groupResultList, financeAmount, loanRate);
                        break;
                    }
                }
                groupResultList.addAll(collect);
                groupResultList = getRepaymentRecordsByInterestCycle(groupResultList, startActiveDate, endActiveDate);
            } else if (interestCalcMethod.equalsIgnoreCase(RESIDUAL_OF_PRINCIPAL)) {
                groupResultList = getRepaymentRecordsByInterestCycle(lonContractRepaymentList, startActiveDate, endActiveDate);
            }
        }
        for (ILonContractInterestCalcService item : calcServices) {
            if (item.getInterestCalcMethodName().equalsIgnoreCase(interestCalcMethod)) {
                calcService = item;
                break;
            }
        }
        List<HlsCusLonContractRepayment> lonContractRepaymentsByCalc = calcService.getLonContractRepaymentsByCalc(groupResultList, financeAmount, loanRate);
        lonContractRepaymentsByCalc = makeDaysRight(lonContractRepaymentsByCalc, financeAmount, startActiveDate, endActiveDate);
        lonContractRepaymentsByCalc.forEach(item -> {
            if (updateLonContractRepaymentService != null && !updateLonContractRepaymentService.beforeUpdate(request, item)) {
                return;
            }
            item.setContractId(contractId);
            item.setWithdrawId(withdrawId);
            if (item.getRepaymentId() == null)
                self().insertSelective(request, item);
            else
                self().updateByPrimaryKeySelective(request, item);
            if (updateLonContractRepaymentService != null) {
                updateLonContractRepaymentService.afterUpdate(request, item);
            }
        });
        return lonContractRepaymentsByCalc;
    }

    /**
     * 重新构建还款现金流的分组
     *
     * @param lonContractRepaymentList
     * @param startActiveDate
     * @param endActiveDate
     * @return
     */
    public List<HlsCusLonContractRepayment> getRepaymentRecordsByInterestCycle(List<HlsCusLonContractRepayment> lonContractRepaymentList, Date startActiveDate, Date endActiveDate) {
        Comparator<HlsCusLonContractRepayment> comparator = (a, b) -> {
            if (a.getPlannedDueDate().compareTo(b.getPlannedDueDate()) == 0) {
                return Long.compare(a.getCfItem(), b.getCfItem());
            }
            return a.getPlannedDueDate().compareTo(b.getPlannedDueDate());
        };
        List<HlsCusLonContractRepayment> result = lonContractRepaymentList.stream().sorted(comparator).collect(Collectors.toList());//先将新的现金流根据时间排序,并生成一个新的list
        LocalDate start = LocalDateUtils.date2LocalDate(startActiveDate);
        LocalDate end = LocalDateUtils.date2LocalDate(endActiveDate);
        int time = 0;//初始化期数
        for (int i = 0; i < result.size(); i++) {
            HlsCusLonContractRepayment lonContractRepayment = result.get(i);
            Date lastDueDate = null;
            if (i == 0) {
                lonContractRepayment.setStartLocalDate(start);
            } else {
                lastDueDate = result.get(i - 1).getPlannedDueDate();
                lonContractRepayment.setStartLocalDate(LocalDateUtils.date2LocalDate(lastDueDate));
                Date plannedDueDate = lonContractRepayment.getPlannedDueDate();
                LocalDate date = LocalDateUtils.date2LocalDate(plannedDueDate);
                if (!date.isBefore(end)) {
                    Date endDate = LocalDateUtils.localDate2Date(end);
                    lonContractRepayment.setDueDate(endDate);
                    lonContractRepayment.setPlannedDueDate(endDate);
                }
            }
            //当前dueDate和上一个dueDate不相同时
            if (lastDueDate == null || lonContractRepayment.getPlannedDueDate().compareTo(lastDueDate) != 0) {
                time++;
            }
            lonContractRepayment.setTimes(Long.valueOf(time));
        }
        return result;
    }


    /**
     * 获得还款现金流的初始分组
     *
     * @param times
     * @param interestCalcDate
     * @param startDate
     * @param endDate
     * @param interestCycle
     * @return
     */
    public List<HlsCusLonContractRepayment> getRepaymentRecordsByInterestCycle(Long times, Long interestCalcDate, Date startDate, Date endDate, String interestCycle) {
        List<HlsCusLonContractRepayment> result = new ArrayList<>();
        LocalDate start = LocalDateUtils.date2LocalDate(startDate);
        LocalDate end = LocalDateUtils.date2LocalDate(endDate);
        for (int i = 1; i <= times; i++) {
            HlsCusLonContractRepayment lonContractRepayment = new HlsCusLonContractRepayment();
            lonContractRepayment.setTimes(Long.valueOf(i));
            LocalDate dueDateByInterestCycle;
            Date dueDate;
            if (i == 1) {
                lonContractRepayment.setStartLocalDate(start);
                dueDateByInterestCycle = getDueDateByInterestCycle(start, interestCycle, interestCalcDate);
            } else {
                HlsCusLonContractRepayment lastRepayment = result.get(i - 2);//因为i从1开始,所以此处减去2才是正确的下标
                LocalDate startLocalDate = LocalDateUtils.date2LocalDate(lastRepayment.getPlannedDueDate());
                lonContractRepayment.setStartLocalDate(startLocalDate);
                dueDateByInterestCycle = getDueDateByInterestCycle(startLocalDate, interestCycle);
            }
            //当计算出的结束日期在end之后或者期数为最后一期时,直接使用endDate作为DueDate
            if (dueDateByInterestCycle.isAfter(end) || i == times) {
                lonContractRepayment.setDueDate(endDate);
                lonContractRepayment.setPlannedDueDate(endDate);
            } else {
                dueDate = LocalDateUtils.localDate2Date(dueDateByInterestCycle);
                lonContractRepayment.setDueDate(dueDate);
                lonContractRepayment.setPlannedDueDate(dueDate);
            }
            result.add(lonContractRepayment);
        }
        return result;
    }

    /**
     * 重新计算计息天数
     *
     * @param list
     * @param financeAmount
     * @param startActiveDate
     * @param endActiveDate
     */
    private List<HlsCusLonContractRepayment> makeDaysRight(List<HlsCusLonContractRepayment> list, Double financeAmount, Date startActiveDate, Date endActiveDate) {
        Comparator<HlsCusLonContractRepayment> comparator = (a, b) -> {
            if (a.getPlannedDueDate().compareTo(b.getPlannedDueDate()) == 0) {
                return Long.compare(a.getCfItem(), b.getCfItem());
            }
            return a.getPlannedDueDate().compareTo(b.getPlannedDueDate());
        };
        List<HlsCusLonContractRepayment> collect = list.stream().sorted(comparator).collect(Collectors.toList());
        Long param = null;
        for (int i = 0; i < collect.size(); i++) {
            HlsCusLonContractRepayment lonContractRepayment = collect.get(i);
            LocalDate start, end;
            if (i == 0) {
                start = LocalDateUtils.date2LocalDate(startActiveDate);
                end = LocalDateUtils.date2LocalDate(lonContractRepayment.getPlannedDueDate());
            } else {
                start = LocalDateUtils.date2LocalDate(collect.get(i - 1).getPlannedDueDate());
                if (i == collect.size() - 1) {
                    end = LocalDateUtils.date2LocalDate(endActiveDate);
                } else {
                    end = LocalDateUtils.date2LocalDate(lonContractRepayment.getPlannedDueDate());
                }
            }
            if (lonContractRepayment.getCfItem() == 302) {
                if (param == null) {
                    lonContractRepayment.setInterestPeriodDays(ChronoUnit.DAYS.between(start, end));
                } else {
                    lonContractRepayment.setInterestPeriodDays(param + ChronoUnit.DAYS.between(start, end));
                    param = null;
                }
                lonContractRepayment.setInterestAccrualBalance(financeAmount);
            } else {
                param = ChronoUnit.DAYS.between(start, end);
                lonContractRepayment.setInterestPeriodDays(null);
                financeAmount = CalculateUtil.sub(financeAmount, lonContractRepayment.getPlannedDueAmount());
                lonContractRepayment.setInterestAccrualBalance(financeAmount);
            }
            lonContractRepayment.setDueDate(lonContractRepayment.getPlannedDueDate());
            lonContractRepayment.setDueAmount(lonContractRepayment.getPlannedDueAmount());
        }
        return list;
    }

    /**
     * 根据不同额结息周期,计算下次的日期
     *
     * @param date
     * @param interestCycle
     * @return
     */
    public LocalDate getDueDateByInterestCycle(LocalDate date, String interestCycle) {
        LocalDate result = null;
        switch (interestCycle) {
            case YEAR:
                result = date.plusYears(1);
                break;
            case HALF_A_YEAR:
                result = date.plusMonths(6);
                break;
            case QUARTER:
                result = date.plusMonths(3);
                break;
            case MONTH:
                result = date.plusMonths(1);
                break;
            default:
                break;
        }
        return result;
    }

    /**
     * 根据不同额结息周期,计算第一次的日期
     *
     * @param startLocalDate
     * @param interestCycle
     * @param interestCalcDate
     * @return
     */
    public LocalDate getDueDateByInterestCycle(LocalDate startLocalDate, String interestCycle, Long interestCalcDate) {
        int dayOfMonth = startLocalDate.getDayOfMonth();//获得开始日期所在当前月的天数
        int monthValue = startLocalDate.getMonthValue();//获得开始日期所在当前年的月份
        int yearValue = startLocalDate.getYear();//获得开始日期的年份
        LocalDate result = startLocalDate.withDayOfMonth(interestCalcDate.intValue());
        switch (interestCycle) {
            case YEAR:
                if (dayOfMonth >= interestCalcDate && monthValue == 12) {
                    yearValue = yearValue + 1;
                    result = result.withYear(yearValue).withMonth(12);
                }
                break;
            case HALF_A_YEAR:
                if (monthValue >= 1 && monthValue < 6) {
                    result = result.withMonth(6);
                } else if (monthValue == 6) {
                    if (interestCalcDate <= dayOfMonth)
                        result = result.withMonth(12);
                } else if (monthValue >= 7 && monthValue < 12) {
                    result = result.withMonth(12);
                } else if (monthValue == 12) {
                    if (interestCalcDate <= dayOfMonth)
                        result = result.withMonth(6).withYear(yearValue + 1);
                }
                break;
            case QUARTER:
                if (monthValue >= 1 && monthValue < 3) {
                    result = result.withMonth(3);
                } else if (monthValue == 3) {
                    if (interestCalcDate <= dayOfMonth)
                        result = result.withMonth(6);
                } else if (monthValue >= 4 && monthValue < 6) {
                    result = result.withMonth(6);
                } else if (monthValue == 6) {
                    if (interestCalcDate <= dayOfMonth)
                        result = result.withMonth(9);
                } else if (monthValue >= 7 && monthValue < 9) {
                    result = result.withMonth(9);
                } else if (monthValue == 9) {
                    if (interestCalcDate <= dayOfMonth)
                        result = result.withMonth(12);
                } else if (monthValue >= 10 && monthValue < 12) {
                    result = result.withMonth(12);
                } else if (monthValue == 12) {
                    if (interestCalcDate <= dayOfMonth)
                        result = result.withMonth(3).withYear(yearValue + 1);
                }
                break;
            case MONTH:
                if (dayOfMonth >= interestCalcDate) {//如果开始日期大于等于结息日,那么应该取下个月的结息日为结束日期
                    if (monthValue + 1 > 12) {//如果当前月份为12月,则取1月
                        int newMonthValue = (monthValue + 1) % 12;
                        int newYearValue = yearValue + 1;
                        result = startLocalDate.withYear(newYearValue).withMonth(newMonthValue).withDayOfMonth(interestCalcDate.intValue());
                    } else {
                        int newMonthValue = monthValue + 1;
                        result = startLocalDate.withMonth(newMonthValue).withDayOfMonth(interestCalcDate.intValue());
                    }
                } else {
                    result = startLocalDate.withDayOfMonth(interestCalcDate.intValue());
                }
                break;
            default:
                break;
        }
        return result;
    }

    /*还款创建明细-重置计息余额*/
    public List<HlsCusLonContractRepayment> resetInterestBalance(List<HlsCusLonContractRepayment> list) {
        //获取最新的还款数据(根据期数升序排列)
        HlsCusLonContractRepayment dto = new HlsCusLonContractRepayment();
        dto.setContractId(list.get(0).getContractId());
        dto.setWithdrawId(list.get(0).getWithdrawId());
        List<HlsCusLonContractRepayment> listAll = lonContractRepaymentMapper.selectLonContractRep(dto);
        //获取合同上的融资金额
        HlsCusLonContract contract = new HlsCusLonContract();
        contract.setContractId(listAll.get(0).getContractId());
        Double interestAmount = lonContractMapper.selectByPrimaryKey(contract).getFinanceAmount();
        ;//计息余额
        for (int i = 0; i < listAll.size(); i++) {
            HlsCusLonContractRepayment lonContractRepayment = listAll.get(i);
            Double dueAmount = lonContractRepayment.getDueAmount();//实际还款金额
            //还款本金
            if (lonContractRepayment.getCfType() == 30 && lonContractRepayment.getCfItem() == 301) {
                interestAmount = CalculateUtil.sub(interestAmount, dueAmount);
            }
            lonContractRepayment.setInterestAccrualBalance(interestAmount);
            lonContractRepayment.set__status("update");
        }
        return listAll;
    }

    /*生成付款申请编号*/
    public String getCodeValue(IRequest requestContext) {
        Map<String, String> params = new HashMap<String, String>();
        String value = codingRuleValuesService.getCodeRuleValue(requestContext, "CSH_PAYMENT_REQ", "PAYMENT_REQ", "PAYMENT_REQ", params);
        return value;
    }

    /*创建申请信息*/
    /*创建申请信息*/
    public List<HlsCusCshPaymentReqLn> createApplyInfo(List<HlsCusLonContractRepayment> list, Long companyId, IRequest request) {
        //创建申请头信息
        HlsCusCshPaymentReqHd cshPaymentReqHd = new HlsCusCshPaymentReqHd();
        cshPaymentReqHd.setCompany_id(companyId);
        cshPaymentReqHd.setDocument_type("REPAYMENT_REQ");
        cshPaymentReqHd.setDocument_category("CSH_PAYMENT_REQ");
        cshPaymentReqHd.setBusiness_type("PAYMENT_REQ");
        cshPaymentReqHd.setPayment_req_date(list.get(0).getCreationDate());//申请日期
        cshPaymentReqHd.setPayment_req_status("APPROVING");
        cshPaymentReqHd.setApply_pay_date(new Date());
        cshPaymentReqHd.setCurrency(list.get(0).getCurrency());
        cshPaymentReqHd.setDescription(list.get(0).getDescription());
        if (list.get(0).getPaymentReqId() != null && list.get(0).getPaymentReqId().compareTo(0L) != 0) {
            //如果获取到了头id 表示是一条被申请拒绝的重新提交申请
            cshPaymentReqHd.setPayment_req_id(list.get(0).getPaymentReqId());
            //获取原编号
            HlsCusCshPaymentReqHd dto = new HlsCusCshPaymentReqHd();
            dto.setPayment_req_id(list.get(0).getPaymentReqId());
            cshPaymentReqHd.setPayment_req_number(paymentReqHdService.selectByPrimaryKey(request, dto).getPayment_req_number());
            cshPaymentReqHd.set__status("update");
        } else {
            String value = getCodeValue(request);//获取付款申请编号
            cshPaymentReqHd.setPayment_req_id(null);
            cshPaymentReqHd.set__status("add");
            cshPaymentReqHd.setPayment_req_number(value);
        }
        List<HlsCusCshPaymentReqHd> listUpdate = new ArrayList<>();
        listUpdate.add(cshPaymentReqHd);
        paymentReqHdService.batchUpdate(request, listUpdate);
        //创建申请行信息
        List<HlsCusCshPaymentReqLn> ReqLnList = new ArrayList<>();
        Double sumAmount = 0D;
        for (int i = 0; i < list.size(); i++) {
            sumAmount = CalculateUtil.add(sumAmount, list.get(i).getApplyAmount());
            HlsCusCshPaymentReqLn cshPaymentReqLn = new HlsCusCshPaymentReqLn();
            if (list.get(i).getPaymentReqLineId() != null && list.get(i).getPaymentReqLineId() != 0) {
                cshPaymentReqLn.set__status("update");
                cshPaymentReqLn.setPayment_req_ln_id(list.get(i).getPaymentReqLineId());
            } else {
                cshPaymentReqLn.set__status("add");
            }
            cshPaymentReqLn.setPayment_req_id(cshPaymentReqHd.getPayment_req_id());
            cshPaymentReqLn.setSource_doc_category("LON_CONTRACT_WITHDRAW");
            cshPaymentReqLn.setSource_doc_id(list.get(i).getWithdrawId());
            cshPaymentReqLn.setSource_doc_line_id(list.get(i).getRepaymentId());
            cshPaymentReqLn.setAmount(list.get(i).getApplyAmount());
            cshPaymentReqLn.setPayment_flag("NOT");
            cshPaymentReqLn.setBp_id(list.get(i).getBpId());
            cshPaymentReqLn.setBp_bank_account_id(list.get(i).getWithdrawBankAccountId());
            cshPaymentReqLn.setBp_bank_account_name(list.get(i).getWithdrawBankAccountName());//账户名
            cshPaymentReqLn.setBp_bank_account_num(list.get(i).getWithdrawBankAccountNumber());
            cshPaymentReqLn.setBp_bank_branch_name(list.get(i).getWithdrawBankAccountBranchName());//银行分行
            cshPaymentReqLn.setBp_bank_name(list.get(i).getWithdrawBankName());
            ReqLnList.add(cshPaymentReqLn);

        }
        //修改申请头的汇总金额
        cshPaymentReqHd.setAmount(sumAmount);
        paymentReqHdService.updateByPrimaryKey(request, cshPaymentReqHd);
        //创建申请行信息
        List<HlsCusCshPaymentReqLn> resultList = cshPaymentReqLnService.batchUpdate(request, ReqLnList);
        return resultList;
    }

    /*还款提交审批*/
    public void repaymentApproval(IRequest iRequest, HttpSession session, List<HlsCusLonContractRepayment> list) {
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
        Long companyId = ((Long) session.getAttribute("companyId"));//公司ID
        if (list == null || list.size() < 1) {
            return;
        }
//        //获取还款id
//        Long withdrawId = list.get(0).getWithdrawId();
//        //锁表
//        HlsCusLonContractWithdraw withdraw = new HlsCusLonContractWithdraw();
//        withdraw = lonContractWithdrawMapper.selectByPrimaryKey(withdrawId);
//        databaseLockProvider.lock(withdraw);
        //创建申请信息
        List<HlsCusCshPaymentReqLn> cshPaymentReqLnList = createApplyInfo(list, companyId, iRequest);
        //开始流程
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "PAY_REPAYMENT");
        params.put("cshPaymentReqLnList", cshPaymentReqLnList);
        params.put("paymentReqHdId", cshPaymentReqLnList.get(0).getPayment_req_id());//申请头id
        activitiStartService.start(iRequest, list, params);
        //修改申请头表信息
        HlsCusCshPaymentReqHd hd = new HlsCusCshPaymentReqHd();
        hd.setPayment_req_id(cshPaymentReqLnList.get(0).getPayment_req_id());
        hd.setPayment_req_status("APPROVING");
        paymentReqHdService.updateByPrimaryKeySelective(iRequest, hd);
    }

    @Override
    public List<HlsCusLonContractRepayment> unitQueryRep(IRequest request, HlsCusLonContractRepayment hlsCusLonContractRepayment, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        Map<String, Object> params = new HashMap<String, Object>();
        List<String> listCashItem = new ArrayList<String>();
        List<String> listStatus = new ArrayList<String>();
        String[] RepCashitem = null;
        String[] RepStatus = null;
        Long companyId = hlsCusLonContractRepayment.getCompanyId();
        params.put("companyId", companyId);
        if (hlsCusLonContractRepayment.getCshItems() != null) {
            RepCashitem = hlsCusLonContractRepayment.getCshItems().split(",");
            for (int i = 0; i < RepCashitem.length; i++) {
                listCashItem.add(RepCashitem[i]);
            }
            params.put("listCashItem", listCashItem);
        }
        if (hlsCusLonContractRepayment.getStatuses() != null) {
            RepStatus = hlsCusLonContractRepayment.getStatuses().split(",");
            for (int i = 0; i < RepStatus.length; i++) {
                listStatus.add(RepStatus[i]);
            }
            params.put("listStatus", listStatus);
        }
        params.put("repAmtFirst", hlsCusLonContractRepayment.getRepAmtFirst());
        params.put("repAmtSecond", hlsCusLonContractRepayment.getRepAmtSecond());
        params.put("repAmtThird", hlsCusLonContractRepayment.getRepAmtThird());
        params.put("amtFrom", hlsCusLonContractRepayment.getAmtFrom());
        params.put("amtTo", hlsCusLonContractRepayment.getAmtTo());
        params.put("creditContractName", hlsCusLonContractRepayment.getCreditContractName());
        params.put("bpName", hlsCusLonContractRepayment.getBpName());
        params.put("paymentReqNumber", hlsCusLonContractRepayment.getPaymentReqNumber());
        params.put("contractNumber", hlsCusLonContractRepayment.getContractNumber());
        params.put("contractName", hlsCusLonContractRepayment.getContractName());
        return lonContractRepaymentMapper.unitQueryRep(params);
    }
}