package hls.core.lon.service.impl;

import hls.core.hls.components.CalculateUtil;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.service.ILonContractInterestCalcService;
import hls.core.lon.utils.LocalDateUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;
import java.util.OptionalLong;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 剩余本金法
 * <p>
 * Created by fjm on 2017/11/13.
 */
@Service
public class LonContractResidualCalcServiceImpl implements ILonContractInterestCalcService {

    public static final String RESIDUAL_OF_PRINCIPAL = "RESIDUAL_OF_PRINCIPAL";

    @Override
    public String getInterestCalcMethodName() {
        return RESIDUAL_OF_PRINCIPAL;
    }

    @Override
    public List<HlsCusLonContractRepayment> getLonContractRepaymentsByCalc(List<HlsCusLonContractRepayment> lonContractRepaymentList, Double financeAmount, Double loanRate) {
        Double leftAmount = financeAmount;
        Double paramAmount = 0D;
        long maxTime = lonContractRepaymentList.stream().mapToLong(HlsCusLonContractRepayment::getTimes).max().getAsLong();//找出最大的期数
        for (int i = 0; i < lonContractRepaymentList.size(); i++) {
            HlsCusLonContractRepayment lonContractRepayment = lonContractRepaymentList.get(i);
            Double plannedDueAmount = lonContractRepayment.getPlannedDueAmount();
            LocalDate startLocalDate = lonContractRepayment.getStartLocalDate();
            Date plannedDueDate = lonContractRepayment.getPlannedDueDate();
            LocalDate plannedDueLocalDate = LocalDateUtils.date2LocalDate(plannedDueDate);
            long between = ChronoUnit.DAYS.between(startLocalDate, plannedDueLocalDate);
            if (lonContractRepayment.getCfType() == 30 && lonContractRepayment.getCfItem() == 301) {//还款本金
                if (i == 0) {
                    Long interestPeriodDays = between;
                    Double interestAccrualBalance = financeAmount;
                    paramAmount = CalculateUtil.mul(interestAccrualBalance, Double.valueOf(interestPeriodDays));
                    paramAmount = CalculateUtil.mul(paramAmount, loanRate);
                    paramAmount = CalculateUtil.div(paramAmount, Double.valueOf(360));
                } else {
                    HlsCusLonContractRepayment lastLonContractRepayment = lonContractRepaymentList.get(i - 1);
                    Long interestPeriodDays = between;
                    Double interestAccrualBalance = lastLonContractRepayment.getInterestAccrualBalance();
                    paramAmount = CalculateUtil.mul(interestAccrualBalance, Double.valueOf(interestPeriodDays));
                    paramAmount = CalculateUtil.mul(paramAmount, loanRate);
                    paramAmount = CalculateUtil.div(paramAmount, Double.valueOf(360));
                }

                if (lonContractRepayment.getTimes().compareTo(maxTime) == 0) {
                    lonContractRepayment.setDueAmount(leftAmount);
                    lonContractRepayment.setPlannedDueAmount(leftAmount);
                    leftAmount = 0D;
                } else {
                    leftAmount = CalculateUtil.sub(leftAmount, plannedDueAmount);
                }
            } else if (lonContractRepayment.getCfType() == 30 && lonContractRepayment.getCfItem() == 302) {//还款利息
                Double interestAmount = CalculateUtil.mul(leftAmount, Double.valueOf(between));
                interestAmount = CalculateUtil.mul(interestAmount, loanRate);
                interestAmount = CalculateUtil.div(interestAmount, Double.valueOf(360));
                interestAmount = CalculateUtil.add(interestAmount, paramAmount);
                lonContractRepayment.setDueAmount(interestAmount);
                lonContractRepayment.setPlannedDueAmount(interestAmount);
                lonContractRepayment.setInterestPeriodDays(between);
                paramAmount = 0D;
            }
            if (lonContractRepayment.getTimes().compareTo(maxTime) == 0) {
                lonContractRepayment.setInterestAccrualBalance(0D);
            } else {
                lonContractRepayment.setInterestAccrualBalance(leftAmount);
            }
            lonContractRepayment.setObjectVersionNumber(null);//避免乐观锁导致报错
        }
        return lonContractRepaymentList;
    }
}
