package hls.core.lon.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.components.DocumentChangeCommon;
import hls.core.hls.dto.BusinessType;
import hls.core.hls.dto.DocumentType;
import hls.core.hls.mapper.BusinessTypeMapper;
import hls.core.hls.mapper.DocumentTypeMapper;
import hls.core.lon.annotation.KeyWord;
import hls.core.lon.components.LonSaveComponent;
import hls.core.lon.dto.*;
import hls.core.lon.exception.AmoutOverdueException;
import hls.core.lon.exception.DateInvalidException;
import hls.core.lon.mapper.*;
import hls.core.lon.service.*;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.mapper.HlsCusHlsCreditLineMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.wfl.service.IActivitiStartService;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class LonContractServiceImpl extends BaseServiceImpl<HlsCusLonContract> implements ILonContractService {

    public static final String LON_CONTRACT_WITHDRAW = "LON_CONTRACT_WITHDRAW";
    public static final String LON_CONTRACT = "LON_CONTRACT";
    public static final String NEW = "NEW";
    public static final String NOT = "NOT";
    public static final String RELEASE = "RELEASE";
    @Autowired
    private HlsCusLonContractMapper lonContractMapper;

    @Autowired
    private HlsCusLonContractQuotationMapper lonContractQuotationMapper;

    @Autowired
    private HlsCusLonContractRepaymentMapper lonContractRepaymentMapper;

    @Autowired
    private BusinessTypeMapper businessTypeMapper;

    @Autowired
    private DocumentTypeMapper documentTypeMapper;

    @Autowired
    private HlsCusCreditContractMapper hlsCusCreditContractMapper;

    @Autowired
    private LonSaveComponent lonSaveComponent;

    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;

    @Autowired
    private ILonContractPledgeService lonContractPledgeService;

    @Autowired
    private ILonContractPurposeService lonContractPurposeService;

    @Autowired
    private ILonContractQuotationService lonContractQuotationService;

    @Autowired
    private ILonContractRepaymentService lonContractRepaymentService;

    @Autowired
    private ILonContractWithdrawService lonContractWithdrawService;

    @Autowired
    private ILonContractAttachmentService lonContractAttachmentService;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;

    @Autowired
    private HlsEmployeeMapper employeeMapper;

    @Autowired
    private IActivitiStartService activitiStartService;

    @Autowired
    private SysEventService sysEventService;

    @Autowired
    private HlsCusHlsCreditLineMapper hlsCreditLineMapper;

    @Autowired(required = false)
    private IUpdateLonContractService updateLonContractService;
    @Autowired(required = false)
    private IUpdateLonContractQuotationService updateLonContractQuotationService;
    @Autowired(required = false)
    private IUpdateLonContractWithdrawService updateLonContractWithdrawService;
    @Autowired(required = false)
    private IUpdateLonContractRepaymentService updateLonContractRepaymentService;
    @Autowired(required = false)
    private IUpdateLonContractPurposeService updateLonContractPurposeService;
    @Autowired(required = false)
    private IUpdateLonContractPledgeService updateLonContractPledgeService;
    @Autowired(required = false)
    private IUpdateLonContractAttachService updateLonContractAttachService;
    @Autowired(required = false)
    private IUpdateLonContractPkgService updateLonContractPkgService;

    @Override
    public List<Map> queryLonConBoardData(Long companyId) {
        return lonContractMapper.lonConBoard(companyId);
    }

    @Override
    public List<HlsCusLonContract> queryLonContract(IRequest request, HlsCusLonContract lonContract, int page, int pageSize) {
        lonContract.setCompanyId(request.getCompanyId());
        PageHelper.startPage(page, pageSize);
        return lonContractMapper.selectLonContract(lonContract);
    }

    /**
     * 保存融资合同
     *
     * @param request
     * @param lonContractPkg
     */
    @Override
    public void saveLonContract(IRequest request, HlsCusLonContractPkg lonContractPkg) throws AmoutOverdueException, DateInvalidException {
        Long companyId = request.getCompanyId();
        HlsCusLonContract lonContract = lonContractPkg.getLonContractByOptional();//融资合同
        HlsCusLonContractQuotation lonContractQuotation = lonContractPkg.getLonContractQuotationByOptional();//融资合同报价
        List<HlsCusLonContractWithdraw> lonContractWithdraws = lonContractPkg.getLonContractWithdrawsByOptional();//融资合同提款
        List<HlsCusLonContractPledge> lonContractPledges = lonContractPkg.getLonContractPledgesByOptional();//融资合同质押
        List<HlsCusLonContractPurpose> lonContractPurposes = lonContractPkg.getLonContractPurposesByOptional();//融资合同用途
        List<HlsCusLonContractRepayment> lonContractRepayments = lonContractPkg.getLonContractRepaymentsByOptional();//融资合同还款
        List<HlsCusLonContractAttachment> lonContractAttachments = lonContractPkg.getLonContractAttachmentsByOptional();//融资合同附件

        if (updateLonContractPkgService != null && !updateLonContractPkgService.beforeUpdate(request, lonContractPkg)) {
            return;
        }

        //数据校验
        HlsCusCreditContract creditContract = new HlsCusCreditContract();
        creditContract.setCreditContractId(lonContract.getCreditContractId());
        creditContract.setCompanyId(companyId);
        //获取融资合同相关信息
        List<HlsCusCreditContract> list = hlsCusCreditContractMapper.unitSelect(creditContract);
        Double creditLineAmt = 0D;
        Date validFrom = null;
        Date validTo = null;
        if (list.size() > 0) {
            creditLineAmt = list.get(0).getOpenToBuy();//可用授信额度
            validFrom = list.get(0).getValidFrom();
            validTo = list.get(0).getValidTo();
        }

        //当合同ID为空时 合同状态为新建时 合同状态为拒绝时 校验融资金额小于可用授信金额
        if (lonContract.getContractId()==null&&creditLineAmt.compareTo(lonContract.getFinanceAmount()) < 0) {
            throw new AmoutOverdueException();
        }else if(lonContract.getContractId()!=null&&creditLineAmt.compareTo(lonContract.getFinanceAmount()) < 0){
            if("NEW".equals(lonContract.getContractStatus())||"REJECT".equals(lonContract.getContractStatus())){
                throw new AmoutOverdueException();
            }
        }
        //校验合同生效日在授信期间内
        if (validFrom != null && validTo != null && lonContractQuotation.getStartActiveDate() != null && lonContractQuotation.getEndActiveDate() != null) {
            if (validFrom.compareTo(lonContractQuotation.getStartActiveDate()) > 0) {
                throw new DateInvalidException();
            }
            if (lonContractQuotation.getStartActiveDate().compareTo(lonContractQuotation.getEndActiveDate()) > 0) {
                throw new DateInvalidException();
            }
        } else {
            throw new DateInvalidException();
        }

        Double sumPurposesAmount = 0D;
        for (HlsCusLonContractPurpose p:lonContractPurposes) {
            sumPurposesAmount = CalculateUtil.add(sumPurposesAmount,p.getFinancingAmount());
        }
        if (sumPurposesAmount.compareTo(lonContract.getFinanceAmount()) >0 ){
            throw new IllegalArgumentException("投放融资额总和已超过融资金额");
        }

        boolean flag = false;//提款金额是否改变的标志
        boolean eventFlag = true;//创建事件标志,新增时插入事件
        Double beforeFinanceAmount = 0D;
        if (lonContract.getContractId() != null) {
            eventFlag = false;
            HlsCusLonContract hlsCusLonContract = mappers.get(0).selectByPrimaryKey(lonContract.getContractId());
            beforeFinanceAmount = hlsCusLonContract.getFinanceAmount() == null ? 0D : hlsCusLonContract.getFinanceAmount();
            lonContract.setContractStatus(hlsCusLonContract.getContractStatus());
        }
        /*当合同ID为空时,既表示整个合同及相关数据都尚未创建,需要先插入合同表作为头表*/
        lonSaveComponent.commonSaveMethod(request, lonContract, self(), item -> {
            //获取编码规则
            item.setCompanyId(companyId);
            item.setContractStatus(lonContract.getContractStatus());
            if (lonContract.getContractId() == null)
                item.setContractStatus(NEW);
            item.setDocumentCategory(LON_CONTRACT);
            BusinessType businessType = new BusinessType();
            businessType.setDocumentCategory(LON_CONTRACT);
            item.setBusinessType(businessTypeMapper.selectList(businessType).stream().findAny().get().getBusinessType());
            DocumentType documentType = new DocumentType();
            documentType.setDocumentCategory(LON_CONTRACT);
            Map<String, String> params = new HashMap<>();
            //item.setDocumentType(documentTypeMapper.selectList(documentType).stream().findAny().get().getDocumentType());
            if (lonContract.getContractId() == null) {
                item.setContractNumber(fndCodingRuleValuesService.getCodeRuleValue(request, lonContract.getDocumentCategory(), lonContract.getDocumentType(), lonContract.getBusinessType(), params));
            }
        }, false);
        Long contractId = lonContract.getContractId();
        if (eventFlag) {
            sysEventService.createEvent(request, contractId, LON_CONTRACT, "", null, SysEventCodeUtil.PROPERTY_LON_CONTRACT_CREATE);
        }
        final HlsCusLonContract lonContractParam = mappers.get(0).selectByPrimaryKey(contractId);
        Double afterFinanceAmount = lonContractParam.getFinanceAmount();
        if (beforeFinanceAmount.compareTo(afterFinanceAmount) != 0) {
            flag = true;
        }
        final boolean finalFlag = flag;
        lonSaveComponent.commonSaveMethod(request, lonContractQuotation, lonContractQuotationService, item -> {
            item.setContractId(contractId);
            item.setLoanRate(CalculateUtil.div(item.getLoanRate(), 100D, 4));
            if (item.getQuotationId() != null) {
                HlsCusLonContractQuotation quotationRecord = lonContractQuotationMapper.selectByPrimaryKey(item.getQuotationId());//更新前的记录
                if (compareChange(quotationRecord, item) || finalFlag) {//比较关键字段是否发生改变,如果改变了,那么删除以前的还款记录,新建一次性还款记录
                    HlsCusLonContractRepayment repayment = new HlsCusLonContractRepayment();
                    repayment.setContractId(contractId);
                    lonContractRepaymentMapper.delete(repayment);
                }
            }
        }, false);
        List<HlsCusLonContractWithdraw> autoList = new ArrayList<>();
        if (lonContractWithdraws == null || lonContractWithdraws.isEmpty()) {//初始化时,默认生成一条提款数据
            HlsCusLonContractWithdraw lonContractWithdraw = new HlsCusLonContractWithdraw();
            lonContractWithdraw.setContractId(contractId);
            autoList.add(lonContractWithdraw);
        } else {//如果grid中有数据,且合同的financeAmount发生了变化,那么更新对应的Withdraw记录
            if (finalFlag) {
                lonSaveComponent.commonSaveMethod(request, lonContractWithdraws.get(0), lonContractWithdrawService, item -> {
                    item.setDueAmount(afterFinanceAmount);
                    item.setDueDate(lonContractQuotation.getStartActiveDate());
                }, true);
            }
        }
        //插入提款数据
        autoList.forEach(child -> {
            lonSaveComponent.commonSaveMethod(request, child, lonContractWithdrawService, item -> {
                item.setWithdrawStatus(NEW);
                item.setDocumentCategory(LON_CONTRACT_WITHDRAW);
                BusinessType businessType = new BusinessType();
                businessType.setDocumentCategory(LON_CONTRACT_WITHDRAW);
                item.setBusinessType(businessTypeMapper.selectList(businessType).stream().findAny().get().getBusinessType());
                DocumentType documentType = new DocumentType();
                documentType.setDocumentCategory(LON_CONTRACT_WITHDRAW);
                item.setDocumentType(documentTypeMapper.selectList(documentType).stream().findAny().get().getDocumentType());
                item.setContractId(contractId);//设置合同id
                item.setCompanyId(companyId);//设置公司id
                item.setCfItem(20L);
                item.setCfType(20L);
                item.setWithdrawNumber(lonContractParam.getContractNumber().concat("-01"));//拼接编号,默认创建时,值先新建一条提款记录
                item.setDueAmount(lonContractParam.getFinanceAmount());
                item.setDueDate(lonContractQuotation.getStartActiveDate());
            }, true);
        });
        lonContractPledges.forEach(child -> {
            lonSaveComponent.commonSaveMethod(request, child, lonContractPledgeService, item -> item.setContractId(contractId), true);
        });
        lonContractPurposes.forEach(child -> {
            lonSaveComponent.commonSaveMethod(request, child, lonContractPurposeService, item -> item.setContractId(contractId), true);
        });
        lonContractRepayments.forEach(child -> {
            lonSaveComponent.commonSaveMethod(request, child, lonContractRepaymentService, item -> {
                item.setContractId(contractId);
                item.setDueAmount(item.getPlannedDueAmount());
                item.setWriteOffFlag(NOT);
                item.setCfStatus(RELEASE);
            }, true);
        });
        lonContractAttachments.forEach(child -> {
            lonSaveComponent.commonSaveMethod(request, child, lonContractAttachmentService, item -> item.setContractId(contractId), true);
        });

        if (updateLonContractPkgService != null) {
            updateLonContractPkgService.afterUpdate(request, lonContractPkg);
        }
    }

    @Override
    public HlsCusLonContractPkg selectLonContractFormData(IRequest request, HlsCusLonContract lonContract) {
        List<HlsCusLonContract> hlsCusLonContracts = lonContractMapper.selectLonContractFormData(lonContract);
        HlsCusLonContractQuotation lonContractQuotation = new HlsCusLonContractQuotation();
        lonContractQuotation.setContractId(lonContract.getContractId());
        lonContractQuotation = lonContractQuotationMapper.selectOne(lonContractQuotation);
        HlsCusLonContractPkg result = new HlsCusLonContractPkg();
        result.setLonContract(hlsCusLonContracts.stream().findAny().orElse(null));
        result.setLonContractQuotation(lonContractQuotation);
        return result;
    }

    /**
     * 判断报价中关键字段是否发生变化
     *
     * @param oldQuotation
     * @param newQuotation
     * @return
     */
    private boolean compareChange(HlsCusLonContractQuotation oldQuotation, HlsCusLonContractQuotation newQuotation) {
        List<Field> allField = DocumentChangeCommon.getAllField(HlsCusLonContractQuotation.class);
        for (Field field : allField) {
            KeyWord[] annotationsByType = field.getAnnotationsByType(KeyWord.class);
            if (annotationsByType.length > 0) {
                try {
                    String oldProperty = BeanUtils.getProperty(oldQuotation, field.getName()) == null ? "" : BeanUtils.getProperty(oldQuotation, field.getName());
                    String newProperty = BeanUtils.getProperty(newQuotation, field.getName()) == null ? "" : BeanUtils.getProperty(newQuotation, field.getName());
                    if (!oldProperty.equalsIgnoreCase(newProperty)) {
                        return true;
                    }
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                }
            }
        }
        return false;
    }

    /*提交审批*/
    public List<HlsCusLonContract> submitApproval(IRequest iRequest, List<HlsCusLonContract> lonContractList) {
        if (updateLonContractService != null && !updateLonContractService.beforeUpdate(iRequest, lonContractList.get(0))) {
            return new ArrayList<>();
        }
        //获取需要的合同
        Long contractId = lonContractList.get(0).getContractId();
        HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(contractId);
        List<HlsCusLonContract> lonContractListParam = new ArrayList<>();
        lonContractListParam.add(lonContract);
        databaseLockProvider.lock(lonContract);
        //获取申请人
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
        //开始流程
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "LON_CONTRACT");
        activitiStartService.start(iRequest, lonContractListParam, params);
        //修改合同状态
        lonContract.setContractStatus("APPROVING");
        self().updateByPrimaryKeySelective(iRequest, lonContract);
        if (updateLonContractService != null && !updateLonContractService.afterUpdate(iRequest, lonContractListParam.get(0))) {
            return new ArrayList<>();
        }
        return lonContractListParam;
    }

    @Override
    public HlsCusLonContract queryLonContractDetail(IRequest request, HlsCusLonContract lonContract) {
        return lonContractMapper.selectLonContractFormData(lonContract).stream().findAny().orElseGet(HlsCusLonContract::new);
    }

    @Override
    public List<HlsCusLonContract> lonContractQuery(IRequest request, HlsCusLonContract lonContract, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        Map<String, Object> params = new HashMap<String, Object>();
        String[] listDocumentType = null;
        String[] listContractStatus = null;

        if (lonContract.getDocumentTypes() != null) {
            listDocumentType = lonContract.getDocumentTypes().split(",");
        }
        if (lonContract.getContractStatuses() != null) {
            listContractStatus = lonContract.getContractStatuses().split(",");
        }
        params.put("companyId", lonContract.getCompanyId());
        params.put("listDocumentType", listDocumentType);
        params.put("listContractStatus", listContractStatus);
        params.put("amtFirst", lonContract.getAmtFirst());
        params.put("amtSecond", lonContract.getAmtSecond());
        params.put("amtThird", lonContract.getAmtThird());
        params.put("amtFrom", lonContract.getAmtFrom());
        params.put("amtTo", lonContract.getAmtTo());
        params.put("contractNumber", lonContract.getContractNumber());
        params.put("contractName", lonContract.getContractName());
        params.put("creditBpName", lonContract.getCreditBpName());
        params.put("creditContractName", lonContract.getCreditContractName());
        return lonContractMapper.lonContractQuery(params);
    }

}