package hls.core.lon.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.DTOStatus;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.hls.components.CalculateUtil;
import hls.core.hls.dto.BusinessType;
import hls.core.hls.dto.DocumentType;
import hls.core.hls.mapper.BusinessTypeMapper;
import hls.core.hls.mapper.DocumentTypeMapper;
import hls.core.lon.dto.HlsCusLonContract;
import hls.core.lon.dto.HlsCusLonContractQuotation;
import hls.core.lon.dto.HlsCusLonContractRepayment;
import hls.core.lon.dto.HlsCusLonContractWithdraw;
import hls.core.lon.mapper.HlsCusLonContractMapper;
import hls.core.lon.mapper.HlsCusLonContractQuotationMapper;
import hls.core.lon.mapper.HlsCusLonContractWithdrawMapper;
import hls.core.lon.service.*;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class LonContractWithdrawServiceImpl extends BaseServiceImpl<HlsCusLonContractWithdraw> implements ILonContractWithdrawService {

    public static final String WEEK = "WEEK";
    public static final String MONTH = "MONTH";
    public static final String QUARTER = "QUARTER";
    public static final String NOT = "NOT";
    public static final String ONE_TIME = "ONE_TIME";
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private List<ILonContractInterestCalcService> calcServices;
    @Autowired
    private HlsCusLonContractWithdrawMapper lonContractWithdrawMapper;
    @Autowired
    private ILonContractRepaymentService lonContractRepaymentService;
    @Autowired
    private HlsCusLonContractMapper lonContractMapper;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private BusinessTypeMapper businessTypeMapper;
    @Autowired
    private DocumentTypeMapper documentTypeMapper;
    @Autowired
    private HlsCusLonContractQuotationMapper lonContractQuotationMapper;
    @Autowired(required = false)
    private IUpdateLonContractRepaymentService updateLonContractRepaymentService;
    @Autowired(required = false)
    private IUpdateLonContractWithdrawService updateLonContractWithdrawService;

    @Override
    public List<HlsCusLonContractWithdraw> queryContractWithdraw(IRequest request, HlsCusLonContractWithdraw lonContractWithdraw, int page, int pageSize) {
        lonContractWithdraw.setCompanyId(request.getCompanyId());
        String timeCondition = lonContractWithdraw.getTimeCondition();
        Date dateByCondition = getDateByCondition(timeCondition);
        lonContractWithdraw.setConditionDate(dateByCondition);
        //PageHelper.startPage(1,0);//参数为1,0时,不分页
        return lonContractWithdrawMapper.selectContractWithdraw(lonContractWithdraw);
    }

    @Override
    public Map queryLonWdBoardData(Long companyId) {
        return lonContractWithdrawMapper.lonWdBoard(companyId);
    }

    @Override
    public List<Map> queryLonWdLineChartData(IRequest request, HlsCusLonContractWithdraw lonContractWithdraw) {
        lonContractWithdraw.setCompanyId(request.getCompanyId());
        String timeCondition = lonContractWithdraw.getTimeCondition();
        Date dateByCondition = getDateByCondition(timeCondition);
        if ("null".equalsIgnoreCase(timeCondition))
            lonContractWithdraw.setTimeCondition(null);
        lonContractWithdraw.setConditionDate(dateByCondition);
        return lonContractWithdrawMapper.lonWdLineChart(lonContractWithdraw);
    }

    @Override
    public List<HlsCusLonContractWithdraw> selectWd(IRequest request, HlsCusLonContractWithdraw hlsCusLonContractWithdraw, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        List<HlsCusLonContractWithdraw> list = lonContractWithdrawMapper.selectWd(hlsCusLonContractWithdraw);
        return list;
    }

    @Override
    public List<HlsCusLonContractWithdraw> batchUpdate4Withdraw(IRequest request, List<HlsCusLonContractWithdraw> list) {
        ILonContractWithdrawService self = self();
        for (HlsCusLonContractWithdraw item : list) {
            if(updateLonContractWithdrawService!=null &&!updateLonContractWithdrawService.beforeUpdate(request,item)){
                return new ArrayList<>();
            }
            switch (item.get__status()) {
                case DTOStatus.ADD:
                    Long contractId = item.getContractId();
                    HlsCusLonContractWithdraw param = new HlsCusLonContractWithdraw();
                    param.setContractId(contractId);
                    List<HlsCusLonContractWithdraw> result = lonContractWithdrawMapper.select(param);
                    HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(contractId);
                    //找出该合同的提款记录中,id最大的的数据
                    HlsCusLonContractWithdraw lonContractWithdraw = result.stream().sorted(Comparator.comparingLong(HlsCusLonContractWithdraw::getWithdrawId).reversed()).findFirst().orElseGet(HlsCusLonContractWithdraw::new);
                    String lastWithdrawNumber = lonContractWithdraw.getWithdrawNumber();
                    if (!StringUtils.isEmpty(lastWithdrawNumber)) {
                        //withdrawNumber.substring();
                        item.setWithdrawNumber(getWithdrawNumber(lastWithdrawNumber));
                    } else {
                        item.setWithdrawNumber(lonContract.getContractNumber().concat("-01"));
                    }
                    item.setCfItem(20L);
                    item.setCfType(20L);
                    item.setWriteOffFlag(NOT);
                    item.setCompanyId(request.getCompanyId());
                    item.setWithdrawStatus("NEW");
                    item.setDocumentCategory("LON_CONTRACT_WITHDRAW");
                    BusinessType businessType = new BusinessType();
                    businessType.setDocumentCategory("LON_CONTRACT_WITHDRAW");
                    item.setBusinessType(businessTypeMapper.selectList(businessType).stream().findAny().get().getBusinessType());
                    DocumentType documentType = new DocumentType();
                    documentType.setDocumentCategory("LON_CONTRACT_WITHDRAW");
                    item.setDocumentType(documentTypeMapper.selectList(documentType).stream().findAny().get().getDocumentType());
                    item = self().insertSelective(request, item);
                    Long withdrawId = item.getWithdrawId();
                    HlsCusLonContractQuotation contractQuotation = new HlsCusLonContractQuotation();
                    contractQuotation.setContractId(contractId);
                    contractQuotation = lonContractQuotationMapper.selectOne(contractQuotation);
                    Double financeAmount = lonContractWithdraw.getDueAmount();//融资额
                    Double loanRate = contractQuotation.getLoanRate();//执行利率
                    Date startActiveDate = contractQuotation.getStartActiveDate();//开始日期
                    Date endActiveDate = contractQuotation.getEndActiveDate();//结束日期
                    Long loanTimes = contractQuotation.getLoanTimes();//融资期数
                    String interestCycle = contractQuotation.getInterestCycle();//结息周期
                    Long interestCalcDate = contractQuotation.getInterestCalcDate();//结息日期
                    List<HlsCusLonContractRepayment> repaymentRecordsByInterestCycle = lonContractRepaymentService.getRepaymentRecordsByInterestCycle(loanTimes, interestCalcDate, startActiveDate, endActiveDate, interestCycle);

                    List<HlsCusLonContractRepayment> lonContractRepaymentsByCalc = null;

                    for (ILonContractInterestCalcService element : calcServices) {
                        if (element.getInterestCalcMethodName().equalsIgnoreCase(ONE_TIME)) {
                            lonContractRepaymentsByCalc = element.getLonContractRepaymentsByCalc(repaymentRecordsByInterestCycle, financeAmount, loanRate);
                            break;
                        }
                    }

                    lonContractRepaymentsByCalc.forEach(element -> {
                        if (updateLonContractRepaymentService != null && !updateLonContractRepaymentService.beforeUpdate(request, element)) {
                            return;
                        }
                        element.setContractId(contractId);
                        element.setWithdrawId(withdrawId);
                        lonContractRepaymentService.insertSelective(request, element);
                        if (updateLonContractRepaymentService != null) {
                            updateLonContractRepaymentService.afterUpdate(request, element);
                        }
                    });
                    break;
                case DTOStatus.UPDATE:
                    if (useSelectiveUpdate()) {
                        self.updateByPrimaryKeySelective(request, item);
                    } else {
                        self.updateByPrimaryKey(request, item);
                    }
                    break;
                case DTOStatus.DELETE:
                    self.deleteByPrimaryKey(item);
                    break;
                default:
                    break;
            }
        }

        HlsCusLonContractWithdraw lonContractWithdraw = new HlsCusLonContractWithdraw();
        Long contractId = list.stream().findAny().get().getContractId();
        lonContractWithdraw.setContractId(contractId);
        List<HlsCusLonContractWithdraw> select = mappers.get(0).select(lonContractWithdraw);
        HlsCusLonContract lonContract = lonContractMapper.selectByPrimaryKey(contractId);
        Double financeAmount = lonContract.getFinanceAmount();
        double sum = select.stream().mapToDouble(HlsCusLonContractWithdraw::getDueAmount).reduce(0D, (a, b) -> CalculateUtil.add(a, b));
        if (financeAmount.compareTo(sum) < 0)
            throw new IllegalArgumentException("提款金额超出限制");
        if(updateLonContractWithdrawService!=null){
            for(HlsCusLonContractWithdraw index : list){
                if(!updateLonContractWithdrawService.afterUpdate(request,index)){
                    return new ArrayList<>();

                }
            }
        }
        return list;
    }

    /**
     * 根据条件,获取对应的时间戳
     *
     * @param timeCondition
     * @return
     */
    private static Date getDateByCondition(String timeCondition) {
        if (!StringUtils.isEmpty(timeCondition)) {
            LocalDateTime now = LocalDateTime.now();
            ZoneId zoneId = ZoneId.systemDefault();
            String condition = timeCondition.toUpperCase();
            Date date = null;
            switch (condition) {
                case WEEK:
                    date = Date.from(now.plusDays(-7).atZone(zoneId).toInstant());
                    break;
                case MONTH:
                    date = Date.from(now.plusMonths(-1).atZone(zoneId).toInstant());
                    break;
                case QUARTER:
                    date = Date.from(now.plusMonths(-3).atZone(zoneId).toInstant());
                    break;
                default:
                    break;
            }
            return date;
        }
        return null;
    }

    /**
     * 获取提款编号
     *
     * @param lastNumber
     * @return
     */
    private static String getWithdrawNumber(String lastNumber) {
        int i = lastNumber.indexOf("-");
        String part1 = lastNumber.substring(0, i + 1);
        String part2 = lastNumber.substring(i + 1, lastNumber.length());
        long l = Long.parseLong(part2) + 1;
        if (l > 10) {
            part1 = part1.concat(new Long(l).toString());
        } else {
            part1 = part1.concat("0").concat(new Long(l).toString());
        }
        return part1;
    }


    /**
     * 提款流程启动
     *
     * @param request
     * @param lonContractWithdraw
     */
    public void lonContractWithdrawWorkFlowStart(IRequest request, HlsCusLonContractWithdraw lonContractWithdraw) {
        Long withdrawId = lonContractWithdraw.getWithdrawId();
        lonContractWithdraw = mappers.get(0).selectByPrimaryKey(withdrawId);
        List<HlsCusLonContractWithdraw> list = new ArrayList<>();
        list.add(lonContractWithdraw);
        //获取申请人
        HlsEmployee employee = employeeMapper.getEmployeeCode(request.getUserId());
        String employeeCode = employee.getEmployeeCode();
        request.setEmployeeCode(employeeCode);
        //开始流程
        Map<String, Object> params = new HashMap<>();
        params.put("workFlowType", "LON_CONTRACT_WITHDRAW");
        activitiStartService.start(request, list, params);
    }

    /*---------------融资提款综合查询----------------*/
    @Override
    public List<HlsCusLonContractWithdraw> withdrawComprehensiveQuery(IRequest request, HlsCusLonContractWithdraw lonContractWithdraw, int page, int pageSize) {

        PageHelper.startPage(page, pageSize);
        Long companyId = request.getCompanyId();

        String[] withdrawStatusArray = null;
        if(lonContractWithdraw.getWithdrawStatus()!= null){
            withdrawStatusArray = lonContractWithdraw.getWithdrawStatus().split("、");
        }
        Map<String, Object> params = new HashMap<String, Object>(8);
        params.put("companyId",companyId);
        params.put("withdrawStatusArray", withdrawStatusArray);
        params.put("amountFrom", lonContractWithdraw.getAmountFrom());
        params.put("amountTo", lonContractWithdraw.getAmountTo());
        params.put("contractNumber", lonContractWithdraw.getContractNumber());
        params.put("contractName", lonContractWithdraw.getContractName());
        params.put("creditBpName", lonContractWithdraw.getCreditBpName());
        params.put("withdrawNumber", lonContractWithdraw.getWithdrawNumber());

        return lonContractWithdrawMapper.withdrawComprehensiveQuery(params);
    }
}