package hls.core.mail;

import com.hand.hap.core.IRequest;
import com.hand.hap.mail.MessageTypeEnum;
import com.hand.hap.mail.ReceiverTypeEnum;
import com.hand.hap.mail.SendTypeEnum;
import com.hand.hap.mail.dto.MessageReceiver;
import com.hand.hap.mail.service.IMessageService;
import hls.core.job.AbstractJobWithIRequest;
import hls.core.mail.dto.MailingList;
import hls.core.mail.service.IMailingListService;
import org.apache.commons.lang.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.quartz.PersistJobDataAfterExecution;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;

@DisallowConcurrentExecution
@PersistJobDataAfterExecution
public class MailJob extends AbstractJobWithIRequest {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IMessageService messageService;
    @Autowired
    private IMailingListService iMailingListService;


    @Override
    public void safeExecuteWithIRequest(JobExecutionContext jobExecutionContext, IRequest iRequest) {
        //取回前台传回的参数accountCode
        String accountCode = jobExecutionContext.getMergedJobDataMap().get("accountCode").toString();
        HashMap<String, Object> param = new HashMap<String, Object>();
        List<MailingList> mailingListArrayList = new ArrayList<MailingList>();
        MailingList mailingList = new MailingList();
        mailingListArrayList = iMailingListService.selectSendMailList();//从表里取出需要发送的邮件列表
        Iterator mailLingListIterator = mailingListArrayList.iterator();
        while (mailLingListIterator.hasNext()) {
            mailingList = (MailingList) mailLingListIterator.next();
            param.put("receivers", mailingList.getReceivers());
            param.put("templateCode", mailingList.getTemplateCode());
            param.put("mode", mailingList.getMode());
            param.put("attachments", mailingList.getAttachments());
            param.put("subject", mailingList.getSubject());
            param.put("content", mailingList.getBody());
            String str = param.get("receivers").toString();
            String[] receivers = StringUtils.split(str, ";");
            ArrayList receiverList = new ArrayList();
            String[] attachments = receivers;
            String subject = param.get("subject").toString();
            String content = param.get("content").toString();
            String messageSource = "逾期合同邮件";

            int attachment = receivers.length;
            for (int var9 = 0; var9 < attachment; ++var9) {
                String s = attachments[var9];
                MessageReceiver mr = new MessageReceiver();
                mr.setMessageAddress(s);
                mr.setMessageType(ReceiverTypeEnum.NORMAL.getCode());
                receiverList.add(mr);
            }
            List var13 = (List) param.get("attachme nts");
            ArrayList var14 = null;
            if (null != var13) {
                var14 = new ArrayList();
                Iterator var15 = var13.iterator();

                while (var15.hasNext()) {
                    Integer var12 = (Integer) var15.next();
                    var14.add(Long.valueOf((long) var12.intValue()));
                }
            }
            try {
                //如果是模版邮件，则按照模版发送，不需要自定义内容，如果是自定义邮件，则传入自定义主题和内容
                if ("template".equals(param.get("mode"))) {
                    if (param.get("accountCode") != null && StringUtils.isNotEmpty(param.get("accountCode").toString())) {
                        this.messageService.addEmailMessage(iRequest.getUserId(), param.get("accountCode").toString(), param.get("templateCode").toString(), (Map) null, var14, receiverList);
                    } else {
                        this.messageService.sendMessage(iRequest, param.get("templateCode").toString(), (Map) null, receiverList, var14);
                    }
                } else {
                    this.messageService.sendMessage(iRequest, accountCode, subject, content, MessageTypeEnum.EMAIL.getCode(), SendTypeEnum.DIRECT.getCode(), messageSource, receiverList, null);
                }
                setExecutionSummary("success");
                mailingList.setSentFlag("Y");
                mailingList.set__status("update");
                iMailingListService.updateSendMail(mailingList);//更新已发送邮件
                logger.info("------------------------------邮件发送成功-----------------------------------");
            } catch (Exception e) {
                logger.info("------------------------------邮件发送失败-----------------------------------");
                mailingList.setSentFlag("N");
                mailingList.set__status("update");
                iMailingListService.updateUnsendMail(mailingList);//更新未发送邮件
                e.printStackTrace();
                setExecutionSummary(e.getClass().getName() + ":" + e.getMessage());
            }
        }
    }

    /**
     * 任务发生异常的时候挂起
     *
     * @return
     */
    @Override
    protected boolean isRefireImmediatelyWhenException() {
        return false;
    }

}