package hls.core.mq.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.core.AppContextInitListener;
import hls.core.mq.service.IRabbitMessageConsumerService;
import hls.core.mq.util.RabbitMQUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by gaoyang on 2017/4/21.
 */
@Service
public class IRabbitMessageConsumerListenerService implements MessageListener, AppContextInitListener {

    private Map<String, IRabbitMessageConsumerService> consumers = new HashMap<>();

    @Autowired(required = false)
    private MessageConverter msgConverter;

    @Override
    public void onMessage(Message message) {
        message.getMessageProperties().setContentType("application/json");
        Object obj = msgConverter.fromMessage(message);
        JSONObject json = JSON.parseObject(JSON.toJSONString(obj));
        JSONObject header = JSON.parseObject(JSON.toJSONString(json.get("header")));
        if (header == null){
            return;
        }
        String orgCode = header.getString("orgCode");
        if ("hand".equalsIgnoreCase(orgCode)) {
            return;
        }
        JSONObject jsonData = RabbitMQUtils.decrypt(json);
        try {
            String apiName = header.getString("apiName");
            IRabbitMessageConsumerService consumerService = consumers.get(apiName);
            if (consumerService != null) {
                consumerService.process(jsonData);
            } else {
                throw new Exception(header.getString("apiName") + apiName + ",该接口尚未实现，请联系管理员！");
            }
        } catch (Exception e) {
            return;
        }


    }

    @Override
    public void contextInitialized(ApplicationContext appCtx) {
        Map<String, IRabbitMessageConsumerService> map = appCtx.getBeansOfType(IRabbitMessageConsumerService.class);

        map.forEach((k, v) -> {
            consumers.put(v.getApiName(), v);
        });

    }
}
