package hls.core.mq.service.impl;

import com.alibaba.fastjson.JSON;
import hls.core.mq.dto.HlsRabbitExchange;
import hls.core.mq.dto.HlsRabbitExchangeQueue;
import hls.core.mq.service.IRabbitProducerService;
import hls.core.mq.util.RabbitMQUtils;
import com.alibaba.fastjson.JSONObject;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.rabbit.listener.adapter.MessageListenerAdapter;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Created by gaoyang on 2017/4/28.
 */
@Service
@Transactional
public class IRabbitProducerServiceImpl implements IRabbitProducerService{

    @Autowired(required = false)
    private RabbitAdmin rabbitAdmin;

    @Autowired(required = false)
    private ConnectionFactory connectionFactory;
    @Autowired(required = false)
    private AmqpTemplate amqpTemplate;
    @Autowired(required = false)
    private Jackson2JsonMessageConverter jackson2JsonMessageConverter;

    @Autowired(required = false)
    private IRabbitMessageConsumerListenerService messageReceiver;

    private String defaultTopicType = "TOPIC";
    private String defaultDirectType = "DIRECT";
    private String defaultFanoutType = "FANOUT";
    private String defaultMessageHandle = "onMessage";

    private boolean convertBoolean(String flag){
        if("Y".equals(flag)){
            return true;
        }else{
            return false;
        }
    }

    @Override
    public void createExchange(HlsRabbitExchange hlsRabbitExchange) {
        String exchangeType = hlsRabbitExchange.getExchangeType();
        if(defaultTopicType.equals(exchangeType)){
            TopicExchange topicExchange = new TopicExchange(hlsRabbitExchange.getExchangeName(),convertBoolean(hlsRabbitExchange.getDurable()),convertBoolean(hlsRabbitExchange.getAutoDetele()));
            rabbitAdmin.declareExchange(topicExchange);
        }else if(defaultDirectType.equals(exchangeType)){
            DirectExchange directExchange = new DirectExchange(hlsRabbitExchange.getExchangeName(),convertBoolean(hlsRabbitExchange.getDurable()),convertBoolean(hlsRabbitExchange.getAutoDetele()));
            rabbitAdmin.declareExchange(directExchange);
        }else if(defaultFanoutType.equals(exchangeType)){
            FanoutExchange fanoutExchange = new FanoutExchange(hlsRabbitExchange.getExchangeName(),convertBoolean(hlsRabbitExchange.getDurable()),convertBoolean(hlsRabbitExchange.getAutoDetele()));
            rabbitAdmin.declareExchange(fanoutExchange);
        }
    }

    @Override
    public void createQueue(HlsRabbitExchangeQueue hlsRabbitExchangeQueue) {
        Queue queue = new Queue(hlsRabbitExchangeQueue.getQueueName(),convertBoolean(hlsRabbitExchangeQueue.getDurable()),convertBoolean(hlsRabbitExchangeQueue.getExclusive()),convertBoolean(hlsRabbitExchangeQueue.getAutoDetele()));
        rabbitAdmin.declareQueue(queue);
        String exchangeType = hlsRabbitExchangeQueue.getExchangeType();
        if(defaultTopicType.equals(exchangeType)){
            TopicExchange topicExchange = new TopicExchange(hlsRabbitExchangeQueue.getExchangeName(),convertBoolean(hlsRabbitExchangeQueue.getDurableH()),convertBoolean(hlsRabbitExchangeQueue.getAutoDeteleH()));
            rabbitAdmin.declareExchange(topicExchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(topicExchange).with(hlsRabbitExchangeQueue.getRoutingKey()));
        }else if(defaultDirectType.equals(exchangeType)){
            DirectExchange directExchange = new DirectExchange(hlsRabbitExchangeQueue.getExchangeName(),convertBoolean(hlsRabbitExchangeQueue.getDurableH()),convertBoolean(hlsRabbitExchangeQueue.getAutoDeteleH()));
            rabbitAdmin.declareExchange(directExchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(directExchange).with(hlsRabbitExchangeQueue.getRoutingKey()));
        }else if(defaultFanoutType.equals(exchangeType)){
            FanoutExchange fanoutExchange = new FanoutExchange(hlsRabbitExchangeQueue.getExchangeName(),convertBoolean(hlsRabbitExchangeQueue.getDurableH()),convertBoolean(hlsRabbitExchangeQueue.getAutoDeteleH()));
            rabbitAdmin.declareExchange(fanoutExchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(fanoutExchange));
        }
        SimpleMessageListenerContainer simpleMessageListenerContainer = new SimpleMessageListenerContainer(connectionFactory);
        simpleMessageListenerContainer.setMessageConverter(jackson2JsonMessageConverter);
        MessageListenerAdapter adapter = new MessageListenerAdapter(messageReceiver,defaultMessageHandle);//messageReceiver为接收消息的类的实例,defaultMessageHandle为接收消息的方法
        simpleMessageListenerContainer.setMessageListener(adapter);
        simpleMessageListenerContainer.setQueueNames(hlsRabbitExchangeQueue.getQueueName());
        simpleMessageListenerContainer.start();//通过在listener中设置监听适配器和队列名字,将队列和消息监听器绑定在一起
    }

    @Override
    public void sendMessage(JSONObject json) {
        // convertAndSend 将Java对象转换为消息发送至匹配key的交换机中Exchange
        JSONObject header = JSON.parseObject(JSON.toJSONString(json.get("header")));
        //String exchangeName = orgCode+"." + "sysCode"+"."+header.getString("apiName");
        String exchangeName ="hls_topic_exchange";
        String sysCode = header.getString("sysCode");
        String apiName = header.getString("apiName");
        String routingKey = header.getString("routingKey");

        if(sysCode==null){
            try {
                throw new Exception("请指定系统名称sysName！");
            }catch (Exception e){
                e.printStackTrace();
            }
        }
        if(apiName==null){
            try {
                throw new Exception("请指定系统接口名称apiName！");
            }catch (Exception e){
                e.printStackTrace();
            }
        }

        JSONObject jsonData = RabbitMQUtils.encryption(json);

        amqpTemplate.convertAndSend(exchangeName, routingKey, jsonData);
    }
}
