package hls.core.mq.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import hls.core.mq.components.RsaKeyComponent;
import hls.core.mq.dto.Header;
import hls.core.mq.dto.ResponseInfo;
import hls.core.mq.dto.SecurityInfo;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * Created by Administrator on 2017/5/26.
 */
public class RabbitMQUtils {

    public static ResponseInfo getResposeInfo(JSONObject var) throws ParseException {
        ResponseInfo responseInfo = new ResponseInfo();
        JSONObject header = JSON.parseObject(JSON.toJSONString(var.get("header")));
        Header rheader = new Header();
        rheader.setSysCode(header.getString("sysCode"));
        //rheader.setOrgCode(header.getString("orgCode"));
        rheader.setOrgCode("hand");
        rheader.setApiName(header.getString("apiName"));
        try {
            rheader.setRespCode(header.getString("respCode"));
            rheader.setRespMsg(header.getString("respMsg"));
        } catch (Exception e) {

        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date date = sdf.parse(header.getString("transDate"));
        rheader.setTransDate(date);
        rheader.setTransNo(header.getString("transNo"));
        rheader.setRoutingKey(header.getString("routingKey"));

        SecurityInfo securityInfo = new SecurityInfo();
        securityInfo.setSignatureValue("");
        securityInfo.setUserName("admin");
        securityInfo.setUserPassword("admin");

        responseInfo.setHeader(rheader);
        responseInfo.setBusiData(var.getString("busiData"));
        responseInfo.setSecurityInfo(securityInfo);

        return responseInfo;
    }

    public static JSONObject encryption(JSONObject json) {
        JSONObject jsonData = new JSONObject();
        JSONObject busiData = JSON.parseObject(JSON.toJSONString(json.get("busiData")));
        JSONObject securityInfo = JSON.parseObject(JSON.toJSONString(json.get("securityInfo")));

        byte[] encryptData = null;
        SecurityInfo security = new SecurityInfo();
        String privateKey = RsaKeyComponent.getServerPrivateKey();
        String publicKey = RsaKeyComponent.getClientPublicKey();
        byte[] pwd = null;
        try {
//            Map<String, Object> keyMap = RSAUtils.genKeyPair();
//            privateKey = RSAUtils.getPrivateKey(keyMap);
//            publicKey = RSAUtils.getPublicKey(keyMap);

            //公钥加密
            encryptData = RSAUtils.encryptByPublicKey(busiData.toString().getBytes(), publicKey);
            pwd = RSAUtils.encryptByPublicKey(securityInfo.getString("userPassword").getBytes(), publicKey);
            byte[] source = RSAUtils.MdigestSHA(Base64Utils.encode(encryptData));
            String sign = RSAUtils.sign(source, privateKey);
            security.setSignatureValue(sign);
        } catch (Exception e) {
            e.printStackTrace();
        }
        security.setUserName(securityInfo.getString("userName"));
        security.setUserPassword(Base64Utils.encode(pwd));

        jsonData.put("header", json.get("header"));
        jsonData.put("busiData", Base64Utils.encode(encryptData));
        jsonData.put("securityInfo", security);
        return jsonData;
    }

    public static JSONObject decrypt(JSONObject json) {
        JSONObject jsonData = new JSONObject();
        Map<String, Object> keyMap = null;
        String publicKey = RsaKeyComponent.getClientPublicKey();
        String privateKey = RsaKeyComponent.getServerPrivateKey();
//        try {
//            keyMap = RSAUtils.genKeyPair();
//            publicKey = RSAUtils.getPublicKey(keyMap);
//            privateKey = RSAUtils.getPrivateKey(keyMap);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }

        JSONObject securityInfo = JSON.parseObject(JSON.toJSONString(json.get("securityInfo")));
        String busiData = json.get("busiData").toString();
        if (!checkSecurity(securityInfo, busiData, publicKey)) {
            try {
                throw new Exception("验证签名失败");
            } catch (Exception e) {
            }
        }

        byte[] decodeData = null;
        byte[] decodepwd = null;
        try {
            decodeData = Base64Utils.decode(busiData.toString());
            decodepwd = Base64Utils.decode(securityInfo.getString("userPassword"));
        } catch (Exception e) {
            e.printStackTrace();
        }
        //私钥解密
        byte[] data = null;
        byte[] pwd = null;
        try {
            data = RSAUtils.decryptByPrivateKey(decodeData, privateKey);
            pwd = RSAUtils.decryptByPrivateKey(decodepwd, privateKey);
        } catch (Exception e) {
            e.printStackTrace();
        }
        String dBusiData = new String(data);

        SecurityInfo security = new SecurityInfo();
        security.setSignatureValue(securityInfo.getString("signatureValue"));
        security.setUserName(securityInfo.getString("userName"));
        security.setUserPassword(new String(pwd));

        jsonData.put("header", json.get("header"));
        jsonData.put("busiData", dBusiData);
        jsonData.put("securityInfo", security);
        return jsonData;
    }

    private static boolean checkSecurity(JSONObject securityInfo, String busiData, String publicKey) {
        boolean flag = false;
        try {
            String signValue = securityInfo.get("signatureValue").toString();
            byte[] source = RSAUtils.MdigestSHA(busiData.toString());
            flag = RSAUtils.verify(source, publicKey, signValue);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return flag;
    }
}
