package hls.core.prj.components;

import com.hand.hap.core.IRequest;
import hls.core.prj.dto.HlsCusPrjProject;
import hls.core.prj.service.PrjProjectService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventCommonService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.sys.service.SysUserService;
import org.activiti.engine.delegate.DelegateExecution;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

abstract class AbstractPrjProjectEventHandler implements SysEventCommonService {

    private static final String DOCUMENT_CATEGORY = "PRJ_PROJECT";

    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private PrjProjectService projectService;
    @Autowired
    private SysEventService sysEventService;

    abstract String getMessage(String userDesc, String prjName, String prjNum);

    abstract String getNoticeTitle();

    public long getLevel() {
        return 2L;
    }

    public String getSourceModel() {
        return SysEventCodeUtil.PROPERTITY_SOURCEMODULE_PRJ;
    }

    public String getRelation() {
        return SysEventCodeUtil.PROPERTITY_RELATIONTYPE_P2D;
    }

    @Override
    public String getDocumentCategory() {
        return null;
    }

    public String getDocumentType(){
        return "";
    }

    @Override
    public void process(IRequest iRequest, Long documentId, String documentCategory, String documentType, Map<String, Object> params) {
        if (params == null) {
            params = new HashMap<>();
        }
        HlsCusPrjProject prjProject = new HlsCusPrjProject();
        prjProject.setProjectId(documentId);
        prjProject = projectService.selectByPrimaryKey(iRequest, prjProject);
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String message = getMessage(sysUser.getDescription(), prjProject.getProjectName(), prjProject.getProjectNumber());
        params.put("message", message);
        params.put("noticeTitle", getNoticeTitle());
        params.put("level", getLevel());
        params.put("noticeType", "NOTICE");

        sysEventService.eventSave(iRequest, documentId, DOCUMENT_CATEGORY, getDocumentType(), getSourceModel(), getEventCode(), getRelation(), params);
    }

    @Override
    public void notice(DelegateExecution delegateExecution, String status) {

    }
}
