package hls.core.prj.controllers;

import hls.core.fnd.service.FndCodingRuleService;
import hls.core.fnd.service.FndCodingRuleValuesService;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.service.HlsCreditLineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.servlet.http.HttpSession;
import java.text.ParseException;
import java.util.Map;


@Controller
    public class HlsCreditLineController extends BaseController{
    public static final String DOCUMENT_CATEGORY = "HLS_CREDIT_LINE";
    public static String DOCUMENT_TYPE = "STD";
    public static final String BUSINESS_TYPE = "CREDIT_LINE";

    @Autowired
    private HlsCreditLineService service;
    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;

    @RequestMapping(value = "/hls/credit/line/statisticalChart/query")
    @ResponseBody
    public List<HlsCusHlsCreditLine> statisticalChart(final HlsCusHlsCreditLine hlsCusHlsCreditLine,@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                      @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize,HttpSession session)  throws ParseException{
        hlsCusHlsCreditLine.setCompanyId((Long) session.getAttribute("companyId"));
        List<HlsCusHlsCreditLine> clist = service.selectCreditLineStatusInfo(hlsCusHlsCreditLine,page,pageSize,session);
        return clist;
    }
    /*@RequestMapping(value = "/hls/credit/line/query")
    @ResponseBody
    public ResponseData query(HlsCusHlsCreditLine dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext,dto,page,pageSize));
    }*/
    @RequestMapping(value = "/hls/credit/line/query")
    @ResponseBody
    public ResponseData creditLineDetailQuery(HlsCusHlsCreditLine dto, @RequestParam Long creditLineId,HttpSession session) {
        dto.setCompanyId((Long) session.getAttribute("companyId"));
        return new ResponseData(service.creditLineDetailQuery(dto));
    }

    @RequestMapping(value = "/hls/credit/line/contract/query")
    @ResponseBody
    public ResponseData creditLineDetailContractQuery(HlsCusHlsCreditLine dto, @RequestParam Long creditLineId,HttpSession session) {
        if(creditLineId == null){
            creditLineId = 0L;
        }
        dto.setCreditLineId(creditLineId);
        dto.setCompanyId((Long) session.getAttribute("companyId"));
        return new ResponseData(service.hlsCreditContractQuery(dto));
    }

    @RequestMapping(value = "/hls/credit/line/detail/query")
    @ResponseBody
    public ResponseData selectCreditLineInfo(HlsCusHlsCreditLine hlsCusHlsCreditLine,HttpSession session,@RequestParam(defaultValue = DEFAULT_PAGE) int page,
    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) throws ParseException {
        hlsCusHlsCreditLine.setCompanyId((Long) session.getAttribute("companyId"));
        List<HlsCusHlsCreditLine> list = service.selectCreditLineInfo(hlsCusHlsCreditLine,page,pageSize);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/hls/credit/line/bpQuery")
    @ResponseBody
    public ResponseData hlsCreditQuery(Long creditLineId) throws ParseException {
        HlsCusHlsCreditLine hlsCusHlsCreditLine=new HlsCusHlsCreditLine();
        hlsCusHlsCreditLine.setCreditLineId(creditLineId);
        List<HlsCusHlsCreditLine> list =service.hlsCreditQuery(hlsCusHlsCreditLine);
        return new ResponseData(list);
    }
    //额度占用查询
    @RequestMapping(value = "/hls/credit/line/bpInfoQuery")
    @ResponseBody
    public ResponseData hlsCreditBpQuery(Long bpId) throws ParseException {
        HlsCusHlsCreditLine hlsCusHlsCreditLine=new HlsCusHlsCreditLine();
        hlsCusHlsCreditLine.setCreditLineId(bpId);
        List<HlsCusHlsCreditLine> list =service.hlsCreditBpQuery(hlsCusHlsCreditLine);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/hls/credit/line/submit")
    @ResponseBody
    public ResponseData update(HttpServletRequest request,@RequestBody List<HlsCusHlsCreditLine> dto){
        IRequest requestCtx = createRequestContext(request);
        if(dto != null)
        {
            for(HlsCusHlsCreditLine dt: dto)
            {
                if(dt.getCreditLineId()== null || dt.getCreditLineId() == 0)
                {
                    dt.set__status("add");
                    DOCUMENT_TYPE=dt.getDocumentType();
                    dt.setCreditLineStatus("NEW");
                    Map<String, String> params = new HashMap<String, String>();
                    dt.setCreditLineNumber(fndCodingRuleValuesService.getCodeRuleValue(requestCtx, DOCUMENT_CATEGORY, DOCUMENT_TYPE, BUSINESS_TYPE, params));

                }else
                {
                    dt.set__status("update");
                }

            }
        }
        List<HlsCusHlsCreditLine> hlsCreditLineList=new ArrayList<>();
        hlsCreditLineList=service.batchUpdate(requestCtx, dto);
        if(hlsCreditLineList.get(0).get__status()=="add"){
            service.addCreditEvent(requestCtx,hlsCreditLineList.get(0));
        }else if(hlsCreditLineList.get(0).get__status()=="update"){
            service.updateCreditEvent(requestCtx,hlsCreditLineList.get(0));
        }
        return new ResponseData(hlsCreditLineList);
    }
    @RequestMapping(value = "/hls/credit/line/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request,@RequestBody List<HlsCusHlsCreditLine> dto){
        service.batchDelete(dto);
        return new ResponseData();
    }

    /*提交审批*/
    @RequestMapping(value = "/hls/credit/line/submitApproval")
    @ResponseBody
    public ResponseData submitApproval(@RequestBody List<HlsCusHlsCreditLine> list, HttpServletRequest request) {
        try {
            IRequest requestCtx = createRequestContext(request);
            List<HlsCusHlsCreditLine> result = service.submitApproval(requestCtx,list);
            return new ResponseData(result);
        } catch (IllegalArgumentException e) {
            ResponseData error = new ResponseData(false);
            error.setMessage(e.getMessage());
            return error;
        }
    }
    /*授信综合查询*/
    @RequestMapping(value = "/hls/credit/line/integratedQuery")
    @ResponseBody
    public ResponseData creditLineIntegratedQuery(@RequestBody HlsCusHlsCreditLine dto,@RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize,HttpServletRequest request) {
        IRequest requestCtx = createRequestContext(request);
        dto.setCompanyId(requestCtx.getCompanyId());
        return new ResponseData(service.creditLineIntegratedQuery(dto,page,pageSize));
    }
}