package hls.core.prj.controllers;

import com.bstek.ureport.utils.PrjProjectDocumentValidate;
import hls.core.cont.dto.ConContract;
import hls.core.document.utils.DocumentUtil;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsUserRelationship;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.hls.service.IHlsUserRelationshipService;
import hls.core.prj.dto.PrjProject;
import hls.core.prj.dto.PrjQuotation;
import hls.core.prj.dto.HlsCusPrjProject;
import hls.core.prj.exception.LeaseItemException;
import hls.core.prj.service.PrjProjectService;
import hls.core.prj.service.PrjTaskService;
import hls.core.sys.dto.HlsCusSysUserVisitHistory;
import hls.core.sys.service.ISysUserVisitHistoryService;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
public class PrjProjectController extends BaseController {

    public static final String DOCUMENT = "DOCUMENT";
    @Autowired
    private PrjProjectService service;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;
    @Autowired
    private ISysUserVisitHistoryService sysUserVisitHistoryService;
    @Autowired
    private IHlsUserRelationshipService hlsUserRelationshipService;

    @RequestMapping(value = "/prj/project/detail/query")
    @ResponseBody
    public ResponseData detailQuery(HlsCusPrjProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusPrjProject> list = service.prjProjectDetailQuery(requestContext, dto, page, pageSize);
        return new ResponseData(list);
    }

    @RequestMapping(value = "/prj/project/integratedQuery")
    @ResponseBody
    public ResponseData prjIntegratedQuery(@RequestBody HlsCusPrjProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                           @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        if (dto.getPage() != page) {
            page = dto.getPage();
        }
        if (dto.getPageSize() != pageSize) {
            pageSize = dto.getPageSize();
        }
        List<Map> map = service.prjIntegratedQuery(requestContext, dto, page, pageSize);
        return new ResponseData(map);
    }

    /**
     * 项目保存
     *
     * @param prjProject
     * @param request
     * @return
     */
    @RequestMapping(value = "/prj/project/save", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData save(@RequestBody HlsCusPrjProject prjProject, HttpServletRequest request) throws Exception {
        IRequest requestContext = createRequestContext(request);
        List<String> list = new ArrayList<>();
        list.add("NEW");
        list.add("APPROVED_RETURN");
        list.add("CANCEL");
        PrjProject validate = new PrjProject();
        if (prjProject.getProjectId() != null) {
            validate = service.selectByPrimaryKey(requestContext, prjProject);
        }
        try {
            PrjProjectDocumentValidate.statusValidate(validate, list);
        } catch (Exception e) {
            throw new Exception("单据状态错误");
        }
        try {
            service.save(requestContext, prjProject);
            HlsCusSysUserVisitHistory sysUserVisitHistory = new HlsCusSysUserVisitHistory();
            String visitType = DOCUMENT;
            String visitTitle = prjProject.getProjectName();
//            String documentCategory = "PRJ_PROJECT";
//            Map<String, Object> map = new HashMap<>();
//            map.put("visitType", visitType);
//            map.put("visitTitle", visitTitle);
//            map.put("documentId", prjProject.getProjectId());
//            map.put("documentCategory", documentCategory);
//            map.put("visitTypeSource", "PRJ");
//            map.put("visitTypeSourceId", prjProject.getProjectId());
            String url = "/document_entrance?documentCategory=PRJ_PROJECT";
            if (prjProject.getProjectId() != null) {
                url = url + "&documentId=" + prjProject.getProjectId();
            }
            sysUserVisitHistory.setVisitUrl(url);
            sysUserVisitHistory.setVisitTitle(visitTitle);
            sysUserVisitHistory.setVisitType(visitType);
            sysUserVisitHistory.setVisitTypeSource("PRJ");
            sysUserVisitHistory.setVisitTypeSourceId(prjProject.getProjectId());
            sysUserVisitHistoryService.saveHistory(requestContext, sysUserVisitHistory);

        } catch (LeaseItemException e) {

        }
        List<HlsCusPrjProject> prjProjectList = new ArrayList<HlsCusPrjProject>();
        //插入项目表
//        prjProject.setFinanceAmount(Long.valueOf(prjQuotation.getFinanceAmount().toString()));
        prjProjectList.add(prjProject);
        return new ResponseData(prjProjectList);
    }

    /**
     * 删除承租人
     *
     * @param dto
     * @return
     */
    @RequestMapping(value = "/prj/project/tenant/delete", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData tenantDelete(HlsCusPrjProject dto) {
        Long projectId = dto.getProjectId();
        Long bpId = dto.getBpId();
        service.deleteTenant(projectId, bpId);
        return new ResponseData();
    }

    /**
     * 删除担保人
     *
     * @param dto
     * @return
     */
    @RequestMapping(value = "/prj/project/guarantor/delete", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData guarantorDelete(HlsCusPrjProject dto) {
        Long projectId = dto.getProjectId();
        Long bpId = dto.getBpId();
        service.deleteGuarantor(projectId, bpId);
        return new ResponseData();
    }


    /**
     * 项目首页项目数量
     */
    @RequestMapping(value = "/prj/project/prjHomePageGetProjectCount")
    @ResponseBody
    public ResponseData prjHomePageGetProjectCount(HlsCusPrjProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = requestContext.getCompanyId();
        dto.setCompanyId(companyId);
        return new ResponseData(service.prjHomePageGetProjectCount(requestContext, dto));
    }

    /**
     * 项目首页各状态项目数量
     */
    @RequestMapping(value = "/prj/project/prjHomePageGetProjectStatusCount")
    @ResponseBody
    public ResponseData prjHomePageGetProjectStatusCount(HlsCusPrjProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                         @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = requestContext.getCompanyId();
        dto.setCompanyId(companyId);
        return new ResponseData(service.prjHomePageGetProjectStatusCount(requestContext, dto));
    }

    /**
     * 项目首页各状态项目数量 包括占比，当月新增等数据
     */
    @RequestMapping(value = "/prj/project/prjHomePageGetAllStatusProjectCount")
    @ResponseBody
    public ResponseData prjHomePageGetAllStatusProjectCount(HlsCusPrjProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                            @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = requestContext.getCompanyId();
        dto.setCompanyId(companyId);
        return new ResponseData(service.prjHomePageGetAllStatusProjectCount(requestContext, dto));
    }

    @RequestMapping(value = "/prj/project/prjHomePageProjectInfoGrid")
    @ResponseBody
    public ResponseData prjHomePageProjectInfoGrid(HlsCusPrjProject dto, HttpSession session, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                                   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        Long companyId = requestContext.getCompanyId();
        dto.setCompanyId(companyId);
        dto.setUserId((Long) session.getAttribute("userId"));
        return new ResponseData(service.prjHomePageProjectInfoGrid(requestContext, dto, page, pageSize));


    }

    /**
     * 通过额度项目创建普通项目
     *
     * @param prjProject
     * @param request
     * @return
     */
    @RequestMapping(value = "/prj/project/saveNormalProjectFromCreditProject", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData saveNormalProjectFromCreditProject(HlsCusPrjProject prjProject, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        List<HlsCusPrjProject> prjProjectList = service.saveNormalProjectFromCreditProject(prjProject, requestContext);
        return new ResponseData(prjProjectList);
    }


    /**
     * 项目商业伙伴查询
     */
    @RequestMapping(value = "/prj/project/bp/query")
    @ResponseBody
    public ResponseData prjBpDetailQuery(HlsCusPrjProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                         @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.prjBpDetailQuery(requestContext, dto));
    }

    /**
     * 项目提交审批
     *
     * @param prjProject
     * @param request
     * @return
     */
    @RequestMapping(value = "/prj/project/updatePrjProjectApproving", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData updatePrjProjectApproving(@RequestBody HlsCusPrjProject prjProject, HttpServletRequest request, HttpSession session) {

        ResponseData rd = null;
        IRequest iRequest = createRequestContext(request);
        try {
            List<HlsCusPrjProject> projects = new ArrayList<HlsCusPrjProject>();
            projects.add(prjProject);

            service.PrjProjectApproving(iRequest, projects, session);
            rd = new ResponseData(true);
        } catch (Exception e) {
            // TODO: handle exception
            rd = new ResponseData(false);
            rd.setMessage(e.getMessage());
        }
        return rd;
    }

    /**
     * 项目动态信息
     *
     * @return
     */
    @RequestMapping(value = "/prj/project/dynamicInfo/query")
    @ResponseBody
    public ResponseData dynamicInfo(final HttpServletRequest request, final HlsSystemNotice hlsSystemNotice, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize) {
        IRequest iRequest = createRequestContext(request);
        List<HlsSystemNotice> list = hlsSystemNoticeService.selectPrjNotice(iRequest, page, pageSize);
        return new ResponseData(list);
    }
   /* public List<HlsSystemNotice> dynamicInfo(final HlsSystemNotice hlsSystemNotice, final HttpServletRequest request){
        IRequest iRequest = createRequestContext(request);
        List<HlsSystemNotice> hlist = hlsSystemNoticeService.selectPrjNotice(iRequest);
        return hlist;
    }*/

    /**
     * 项目收藏
     *
     * @return
     */
    @RequestMapping(value = "/hls/project/collect/submit")
    @ResponseBody
    public ResponseData collect(String collectProjectId, final HttpServletRequest request) {

        IRequest requestContext = createRequestContext(request);

        Long userId = requestContext.getUserId();

        String[] arrs = collectProjectId.split(",");

        HlsUserRelationship hlsUserRelationships = null;
        for (int i = 0; i < arrs.length; i++) {
            hlsUserRelationships = new HlsUserRelationship();
            hlsUserRelationships.setRelationDocumentId(Long.parseLong(arrs[i]));
            hlsUserRelationships.setUserId(userId);
            hlsUserRelationships.setSourceUserId(userId);
            hlsUserRelationships.setRelationDocumentCategory("PRJ_PROJECT");
            hlsUserRelationships.setRelationType("P2D");
            hlsUserRelationships.setSourceModule("PRJ");
            hlsUserRelationships.setSourceDocumentCategory("PRJ_PROJECT");
            hlsUserRelationships.setSourceDocumentId(Long.parseLong(arrs[i]));

            hlsUserRelationshipService.insertSelective(requestContext, hlsUserRelationships);
        }

        return new ResponseData();
    }

    /**
     * 项目取消收藏
     *
     * @return
     */
    @RequestMapping(value = "/hls/project/unCollect/submit")
    @ResponseBody
    public ResponseData collected(String unCollectProjectId, final HttpServletRequest request) {
        String[] arrs = unCollectProjectId.split(",");

        for (int i = 0; i < arrs.length; i++) {
            hlsUserRelationshipService.uncollect(Long.parseLong(arrs[i]));
        }

        return new ResponseData();
    }

    /**
     * CreateUser：Jeffery
     * Date：2017-06-20 10:44:39
     * 查询租赁申请
     *
     * @return
     */
    @RequestMapping(value = "/hls/project/query/apply")
    @ResponseBody
    public ResponseData queryApplyRecord(HlsCusPrjProject dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                         @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        List<HlsCusPrjProject> listPrj = service.queryApplyRecord(dto.getBpId());
        return new ResponseData(listPrj);
    }

}