package hls.core.prj.controllers;

import com.hand.hap.core.exception.TokenException;
import hls.core.calc.service.HlsCalcExcelImportUtilService;
import hls.core.prj.dto.HlsCusPrjQuotation;
import hls.core.prj.mapper.HlsCusPrjQuotationMapper;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.prj.service.PrjQuotationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Controller
public class PrjQuotationController extends BaseController {

    @Autowired
    private PrjQuotationService service;

    @Autowired
    private HlsCalcExcelImportUtilService hlsCalcExcelImportUtilService;
    @Autowired
    private PrjQuotationService prjQuotationService;
    @Autowired
    private HlsCusPrjQuotationMapper prjQuotationMapper;

    @RequestMapping(value = "/prj/quotation/query")
    @ResponseBody
    public ResponseData query(HlsCusPrjQuotation dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/prj/quotation/submit")
    @ResponseBody
    public ResponseData update(HttpServletRequest request, @RequestBody List<HlsCusPrjQuotation> dto) {
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/prj/quotation/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<HlsCusPrjQuotation> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }

    @RequestMapping(value = "/prj/quotation/detail/query")
    @ResponseBody
    public ResponseData detailQuery(HlsCusPrjQuotation dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                    @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, @RequestParam Long projectId) {
        if(projectId == null){
            projectId = 0L;
        }

        dto.setProjectId(projectId);
        return new ResponseData(service.prjQuotationDetailQuery(dto, page, pageSize));
    }

    @RequestMapping(value = "/prj/quotation/submitSheettTest", method = RequestMethod.POST)
    @ResponseBody
    public ResponseData submitSheettTest(HttpServletRequest request, String priceList,@RequestBody String sheets)
            throws TokenException {
        ResponseData rd = null;
        IRequest requestCtx = createRequestContext(request);
        try {
            hlsCalcExcelImportUtilService.parseExcelJsonToArray(requestCtx,sheets);
        } catch (Exception e) {
            // TODO: handle exception
            rd = new ResponseData(false);
            rd.setMessage(e.getMessage());
        }

        return rd;
    }


    @RequestMapping(value = "/prj/quotation/querySheetByquotation",produces="application/javascript;charset=utf-8")
    @ResponseBody
    public String querySheet(HttpServletRequest request,HlsCusPrjQuotation prjQuotation) {
        IRequest requestCtx = createRequestContext(request);
        List<HlsCusPrjQuotation> quotationList=prjQuotationMapper.getPrjProjectQuotationSheets(prjQuotation);
        if(quotationList.size()>0){
            return  quotationList.get(0).getSheets();
        }else{
            //没有定义sheet时，初始化一个sheet让其可以维护
            return "[{\"name\":\"Sheet1\",\"rows\":[]}]";
        }

    }
}