package hls.core.prj.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.mapper.HlsCusHlsCreditLineMapper;
import hls.core.prj.service.HlsCreditLineService;
import hls.core.sys.event.service.SysEventService;
import hls.core.user.service.LoginUserInfoService;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpSession;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Transactional
public class HlsCreditLineServiceImpl extends BaseServiceImpl<HlsCusHlsCreditLine> implements HlsCreditLineService{

    @Autowired
    private HlsCusHlsCreditLineMapper mapper;
    @Autowired
    private DatabaseLockProvider databaseLockProvider;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private LoginUserInfoService loginUserInfoService;
    @Autowired
    private HlsCreditLineService service;

    @Override
    public List<HlsCusHlsCreditLine> creditLineDetailQuery(HlsCusHlsCreditLine hlsCusHlsCreditLine) {
        return mapper.creditLineDetailQuery(hlsCusHlsCreditLine);
    }

    @Override
    public List<HlsCusHlsCreditLine> hlsCreditContractQuery(HlsCusHlsCreditLine hlsCusHlsCreditLine) {
        return mapper.hlsCreditContractQuery(hlsCusHlsCreditLine);
    }

    @Override
    public void updateCreditLine(HlsCusHlsCreditLine creditLine, Long userId) {
        mapper.updateCreditLine(creditLine.getSourceDocumentCategory(),creditLine.getSourceDocumentId(),userId,new Date());
    }

    @Override
    public List<HlsCusHlsCreditLine> selectCreditLineStatusInfo(HlsCusHlsCreditLine hlsCusHlsCreditLine,int page,int pageSize,HttpSession session) throws ParseException{
        PageHelper.startPage(page, pageSize);
        hlsCusHlsCreditLine.setCompanyId((Long)session.getAttribute("companyId"));
        String beginDate = hlsCusHlsCreditLine.getDate_startup();
        String endDate = hlsCusHlsCreditLine.getDate_out();
        SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");

        if (beginDate != null && beginDate != "") {
            hlsCusHlsCreditLine.setBeginDate(dft.parse(beginDate));
        }
        if (endDate != null && endDate != "") {
            hlsCusHlsCreditLine.setEndDate(dft.parse(endDate));
        }
        List<HlsCusHlsCreditLine> list=mapper.selectCreditLineStatusInfo(hlsCusHlsCreditLine);
        Long countPercent=0L;
        for(int i=0;i<=list.size()-2;i++){
            countPercent=list.get(i).getCreditLineCountPercent()+countPercent;
        }
        list.get(list.size()-1).setCreditLineCountPercent(100-countPercent);
        return list;
    }
    @Override
    public List<HlsCusHlsCreditLine> selectCreditLineInfo(HlsCusHlsCreditLine hlsCusHlsCreditLine,int page,int pageSize) throws ParseException {
        PageHelper.startPage(page, pageSize);
        String beginDate = hlsCusHlsCreditLine.getDate_startup();
        String endDate = hlsCusHlsCreditLine.getDate_out();
        String limitFrom = hlsCusHlsCreditLine.getLimit_from();
        String limitTo = hlsCusHlsCreditLine.getLimit_to();
        String creditObjectName = hlsCusHlsCreditLine.getCredit_object_name();
        String creditLineStatus = hlsCusHlsCreditLine.getCreditLineStatus();
        String businessType = hlsCusHlsCreditLine.getBusinessType();
        String fctCreditLineStatus = hlsCusHlsCreditLine.getFctCreditLineStatus();

        SimpleDateFormat dft = new SimpleDateFormat("yyyy-MM-dd");

        if (beginDate != null && beginDate != "") {
            hlsCusHlsCreditLine.setBeginDate(dft.parse(beginDate));
        }
        if (endDate != null && endDate != "") {
            hlsCusHlsCreditLine.setEndDate(dft.parse(endDate));
        }
        if (limitFrom != null && limitFrom != "") {
            hlsCusHlsCreditLine.setLimitFrom(limitFrom);
        }
        if (limitTo != null && limitTo != "") {
            hlsCusHlsCreditLine.setLimitTo(limitTo);
        }
        if (creditObjectName != null && creditObjectName != "") {
            hlsCusHlsCreditLine.setCreditObjectName(creditObjectName);
        }
        if(creditLineStatus !=null &&creditLineStatus!=""){
                hlsCusHlsCreditLine.setCreditLineStatus(creditLineStatus);
        }
        if(businessType!=null&&businessType!=""){
            hlsCusHlsCreditLine.setBusinessType(businessType);
        }
        if(fctCreditLineStatus!=null&&fctCreditLineStatus!=""){
            hlsCusHlsCreditLine.setFctCreditLineStatus(fctCreditLineStatus);
        }
        List<HlsCusHlsCreditLine> list = mapper.selectCreditLineInfo(hlsCusHlsCreditLine);
        return list;
    }
    @Override
    public List<HlsCusHlsCreditLine> hlsCreditQuery(HlsCusHlsCreditLine hlsCusHlsCreditLine){
        return mapper.hlsCreditQuery(hlsCusHlsCreditLine);
    }

    @Override
    public List<HlsCusHlsCreditLine> hlsCreditBpQuery(HlsCusHlsCreditLine hlsCusHlsCreditLine){
        return mapper.hlsCreditBpQuery(hlsCusHlsCreditLine);
    }


    /*提交审批*/
    @Override
    public List<HlsCusHlsCreditLine> submitApproval(IRequest iRequest, List<HlsCusHlsCreditLine> cusHlsCreditLineList) {
        //先保存页面
        List<HlsCusHlsCreditLine> list = new ArrayList<>();
        list.add(cusHlsCreditLineList.get(0).getHlsCreditLine());
        list.get(0).set__status("update");
        service.batchUpdate(iRequest, list);
        //获取需要的额度
        Long creditLineId = cusHlsCreditLineList.get(0).getCreditLineId();
        HlsCusHlsCreditLine creditLine = mapper.selectByPrimaryKey(creditLineId);
        List<HlsCusHlsCreditLine> creditLineListParam = new ArrayList<>();
        creditLineListParam.add(creditLine);
        databaseLockProvider.lock(creditLine);
        //获取申请人
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
        //修改合同状态
        creditLine.setCreditLineStatus("APPROVING");
        self().updateByPrimaryKeySelective(iRequest, creditLine);
        //开始流程
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "CREDIT_LINE");
        params.put("creditLineStatus",creditLine.getCreditLineStatus());
        params.put("creditLineId",creditLine.getCreditLineId());
        Map<String, Object> evenParams = new HashMap<>();
        //插入事件
        //String userName=(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg=userName+"提交了一条综合授信审批:"+creditLine.getCreditLineName()+creditLine.getCreditLineNumber();
        evenParams.put("message",msg);
        evenParams.put("noticeTitle", "授信审批");
        evenParams.put("url", "");
        evenParams.put("level", 2L);
        evenParams.put("noticeType", "NOTICE");
        evenParams.put("sourceUserId",iRequest.getUserId());
        sysEventService.eventSave(iRequest, creditLine.getCreditLineId(), creditLine.getDocumentCategory(), creditLine.getDocumentType(), "CRE", "HLS_CREDIT_LINE", "P2D", evenParams);
        activitiStartService.start(iRequest, creditLineListParam, params);
        return creditLineListParam;
    }


    //新增综合授信事件
    @Override
    public void addCreditEvent(IRequest iRequest,HlsCusHlsCreditLine hlsCusHlsCreditLine){
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        //String userName=(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg=userName+"创建了综合授信:"+hlsCusHlsCreditLine.getCreditLineName()+",编号:"+hlsCusHlsCreditLine.getCreditLineNumber();
        evenParams.put("message", msg);
        evenParams.put("noticeTitle", "综合授信创建");
        evenParams.put("url", "");
        evenParams.put("level", 2L);
        evenParams.put("noticeType", "NOTICE");
        evenParams.put("sourceUserId",iRequest.getUserId());
        sysEventService.eventSave(iRequest, hlsCusHlsCreditLine.getCreditLineId(), hlsCusHlsCreditLine.getDocumentCategory(), hlsCusHlsCreditLine.getDocumentType(), "CRE", "HLS_CREDIT_LINE.CREATE", "P2D", evenParams);
    }

    @Override
    public List<HlsCusHlsCreditLine> updateCreditlineInfo(HlsCusHlsCreditLine hlsCusHlsCreditLine){
        return mapper.updateCreditlineInfo(hlsCusHlsCreditLine);
    }


    //更新综合授信事件
    @Override
    public void updateCreditEvent(IRequest iRequest,HlsCusHlsCreditLine hlsCusHlsCreditLine){
        //插入事件
        Map<String, Object> evenParams = new HashMap<>();
        //String userName=(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        String userName ="";
        if(loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode()).size()>0){
            userName = (loginUserInfoService.queryUserInfo(iRequest.getEmployeeCode())).get(0).getUserName();
        }
        String msg=userName+"更新了综合授信:"+hlsCusHlsCreditLine.getCreditLineName()+",编号:"+hlsCusHlsCreditLine.getCreditLineNumber();
        evenParams.put("message", msg);
        evenParams.put("noticeTitle", "综合授信更新");
        evenParams.put("url", "");
        evenParams.put("level", 2L);
        evenParams.put("noticeType", "NOTICE");
        evenParams.put("sourceUserId",iRequest.getUserId());
        sysEventService.eventSave(iRequest, hlsCusHlsCreditLine.getCreditLineId(), hlsCusHlsCreditLine.getDocumentCategory(), hlsCusHlsCreditLine.getDocumentType(), "CRE", "HLS_CREDIT_LINE.UPDATE", "P2D", evenParams);
    }

    //授信综合查询信息
    @Override
    public List<HlsCusHlsCreditLine> creditLineIntegratedQuery(HlsCusHlsCreditLine hlsCusHlsCreditLine,int page,int pageSize){
        PageHelper.startPage(page, pageSize);
        return mapper.creditLineIntegratedQuery(hlsCusHlsCreditLine);
    }
}