package hls.core.prj.service.impl;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.hls.dto.HlsDocAuthority;
import hls.core.hls.dto.HlsSystemEvent;
import hls.core.hls.dto.HlsSystemNotice;
import hls.core.hls.dto.HlsUserRelationship;
import hls.core.hls.mapper.HlsUserRelationshipMapper;
import hls.core.hls.service.HlsSystemEventService;
import hls.core.hls.service.HlsSystemNoticeService;
import hls.core.hls.service.IHlsDocAuthorityService;
import hls.core.hls.service.IHlsUserRelationshipService;
import hls.core.prj.dto.HlsCusPrjProject;
import hls.core.prj.service.PrjProjectEventService;
import hls.core.prj.service.PrjProjectService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.service.SysNoticePushService;
import hls.core.sys.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * Created by Administrator on 2017/4/25.
 */
@Service
@Transactional
public class PrjProjectEventServiceImpl extends BaseServiceImpl<HlsCusPrjProject> implements PrjProjectEventService {
    @Autowired
    private HlsSystemEventService hlsSystemEventService;
    @Autowired
    private IHlsDocAuthorityService docAuthorityService;
    @Autowired
    private IHlsUserRelationshipService hlsUserRelationshipService;
    @Autowired
    private HlsUserRelationshipMapper userRelationshipMapper;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;
    @Autowired
    private PrjProjectService projectService;
    @Autowired
    private SysNoticePushService sysNoticePushService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private SysEventService sysEventService;

    private static final String PRJ_PROJECT_CREATE = "项目创建";
    private static final String PRJ_PROJECT_MODIFY= "项目修改";
    private static final String PRJ_PROJECT_SUBMIT = "项目提交申请";
    private static final String PRJ_PROJECT_APPROVE = "项目审批通过";
    private static final String PRJ_PROJECT_REJECT = "项目审批拒绝";
    private static final String PRJ_PROJECT_CANCEL = "项目取消";
    private static final String sourceModule ="PRJ";
    private static final String RelationType = "P2D";
    private static final String url = "";

    @Override
    public void eventSave(IRequest iRequest, String eventCode,HlsCusPrjProject project) {
        project=projectService.selectByPrimaryKey(iRequest,project);
        Long userId = iRequest.getUserId();
        String documentCategory = project.getDocumentCategory();
        Long documentId = project.getProjectId();
        String message = "";
        String noticeTitle = "";
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());

        switch (eventCode) {
            case "PRJ_PROJECT.CREATE":
                message =sysUser.getDescription() + "创建了项目" + "【"+ project.getProjectName() + "】";
                noticeTitle = PRJ_PROJECT_CREATE;
                break;
            case "PRJ_PROJECT.MODIFY":
                message = sysUser.getDescription() + "修改了项目" + "【"+ project.getProjectName() + "】";
                noticeTitle = PRJ_PROJECT_MODIFY;
                break;
            case "PRJ_PROJECT.SUBMIT":
                message = sysUser.getDescription() + "提交了项目申请" + "【"+ project.getProjectName() + "】";
                noticeTitle = PRJ_PROJECT_SUBMIT;
                break;
            case "PRJ_PROJECT.APPROVE":
                message = sysUser.getDescription() + "提交了项目申请" + "【"+ project.getProjectName() + "】审批通过";
                noticeTitle = PRJ_PROJECT_APPROVE;
                break;
            case "PRJ_PROJECT.REJECT":
                message = sysUser.getDescription() + "提交了项目申请" + "【"+ project.getProjectName() + "】已被拒绝";
                noticeTitle = PRJ_PROJECT_REJECT;
                break;
            case "PRJ_PROJECT.CANCEL":
                message = sysUser.getDescription() + "提交了项目申请" + "【"+ project.getProjectName() + "】已被取消";
                noticeTitle = PRJ_PROJECT_CANCEL;
                break;
            default:
                break;
        }


        HlsSystemEvent event = new HlsSystemEvent();
        event.setEventCode(eventCode);
        event.setSourceDocumentId(project.getProjectId());
        event.setEventUserId(userId);
        event.setSourceDocumentCategory(project.getDocumentCategory());
        event.setLastUpdatedBy(userId);
        event.setLastUpdateDate(new Date());

        // 系统通知
        HlsSystemNotice hlsSystemNotice = new HlsSystemNotice();
        hlsSystemNotice.setNotice_message(message);
        hlsSystemNotice.setNotice_title(noticeTitle);
        hlsSystemNotice.setNotice_datetime(new Date());
        hlsSystemNotice.setNotice_type("NOTICE");
        hlsSystemNotice.setNotice_url(url);
        hlsSystemNotice.setNotice_level(2L);
        hlsSystemNotice.setSource_module(sourceModule);
        hlsSystemNotice.setSource_event(eventCode);
        hlsSystemNotice.setSource_document_category(documentCategory);
        hlsSystemNotice.setSource_document_id(documentId);
        hlsSystemNotice.setSource_user_id(iRequest.getUserId());
//        hlsSystemNotice.setRead_flag("N");
        hlsSystemNotice.setLastUpdatedBy(userId);
        hlsSystemNotice.setLastUpdateDate(new Date());

        HlsDocAuthority authority = new HlsDocAuthority();
        authority.setUserId(userId);
        authority.setDocumentCategory(documentCategory);// 单据类型
        authority.setDocumentId(documentId);
        authority.setAuthorityType("1");// 1查询，2，修改
        authority.setSourceModule(sourceModule);
        authority.setSourceEvent(eventCode);
        authority.setSourceDocumentCategory(documentCategory);
        authority.setSourceDocumentId(documentId);
        authority.setSourceUserId(iRequest.getUserId());
        authority.setLastUpdatedBy(userId);
        authority.setLastUpdateDate(new Date());

        HlsUserRelationship hlsUserRelationship = new HlsUserRelationship();
        hlsUserRelationship.setUserId(userId);
        hlsUserRelationship.setRelationType(RelationType);

        hlsUserRelationship.setSourceModule(sourceModule);
        hlsUserRelationship.setSourceEvent(eventCode);
        hlsUserRelationship.setSourceDocumentCategory(documentCategory);
        hlsUserRelationship.setSourceDocumentId(documentId);
        hlsUserRelationship.setSourceUserId(iRequest.getUserId());
        hlsUserRelationship.setLastUpdatedBy(userId);
        hlsUserRelationship.setLastUpdateDate(new Date());

        hlsSystemEventService.insertSelective(iRequest, event);
        docAuthorityService.insertSelective(iRequest, authority );

        if("P2P".equalsIgnoreCase(hlsUserRelationship.getRelationType()) && (eventCode.equals("PRJ_PROJECT.CREATE") || eventCode.equals("PRJ_PROJECT.MODIFY")))
        {
            hlsUserRelationship.setRelationUserId(iRequest.getUserId()); // 关联用户ID

            HlsUserRelationship userRelationship = new HlsUserRelationship();
            userRelationship.setUserId(hlsUserRelationship.getUserId());
            userRelationship.setRelationType(hlsUserRelationship.getRelationType());
            userRelationship.setRelationUserId(hlsUserRelationship.getRelationUserId());
            List<HlsUserRelationship> list =  userRelationshipMapper.select(userRelationship);
            if(list.size() == 0)
            {
                hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship );
            }
        }else if("P2D".equalsIgnoreCase(hlsUserRelationship.getRelationType()) && (eventCode.equals("PRJ_PROJECT.CREATE") || eventCode.equals("PRJ_PROJECT.MODIFY"))){

            hlsUserRelationship.setRelationDocumentCategory(documentCategory);// 关联单据类别
            hlsUserRelationship.setRelationDocumentId(documentId); // 关系单据ID

            HlsUserRelationship userRelationship1 = new HlsUserRelationship();
            userRelationship1.setUserId(hlsUserRelationship.getUserId());
            userRelationship1.setRelationType(hlsUserRelationship.getRelationType());
            userRelationship1.setSourceDocumentCategory(hlsUserRelationship.getSourceDocumentCategory());
            userRelationship1.setSourceDocumentId(hlsUserRelationship.getSourceDocumentId());
            List<HlsUserRelationship> lists =  userRelationshipMapper.select(userRelationship1);
            if(lists.size() == 0)
            {
                hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship );
            }
        }
        hlsSystemNoticeService.insertSelective(iRequest, hlsSystemNotice );
        sysEventService.setNoticeCache(hlsSystemNotice);
    }
}
