package hls.core.prj.service.impl;


import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.lock.components.DatabaseLockProvider;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.fnd.dto.FndCompany;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.dto.HlsEmployeeAssigns;
import hls.core.fnd.mapper.FndCompanyMapper;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.fnd.mapper.IHlsEmployeeAssignsMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.bp.dto.HlsCusBpReport;
import hls.core.hls.bp.mapper.HlsCusBpReportMapper;
import hls.core.hls.bp.service.HlsBpReportService;
import hls.core.prj.dto.*;
import hls.core.prj.exception.LeaseItemException;
import hls.core.prj.mapper.*;
import hls.core.prj.service.*;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.wfl.service.IActivitiStartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpSession;
import java.math.BigDecimal;
import java.util.*;

@Service
@Transactional
public class PrjProjectServiceImpl extends BaseServiceImpl<HlsCusPrjProject> implements PrjProjectService {
    @Autowired
    private HlsCusPrjProjectMapper prjProjectMapper;

    @Autowired
    private PrjProjectTenantService prjProjectTenantService;

    @Autowired
    private HlsCusPrjProjectTenantMapper prjProjectTenantMapper;

    @Autowired
    private PrjProjectGuarantorService prjProjectGuarantorService;

    @Autowired
    private HlsCusPrjProjectGuarantorMapper prjProjectGuarantorMapper;

    @Autowired
    private PrjProjectLeaseItemService prjProjectLeaseItemService;

    @Autowired
    private PrjProjectMortgageService prjProjectMortgageService;

    @Autowired
    private PrjQuotationService prjQuotationService;

    @Autowired
    private FndCodingRuleValuesService fndCodingRuleValuesService;

    @Autowired
    private HlsBpReportService hlsBpReportService;

    @Autowired
    private HlsCusBpReportMapper hlsCusBpReportMapper;

    @Autowired
    private PrjTenantHistEvolutionService prjTenantHistEvolutionService;

    @Autowired
    private HlsCusPrjTenantHistEvolutionMapper prjTenantHistEvolutionMapper;

    @Autowired
    private PrjTenantStockStructureService prjTenantStockStructureService;

    @Autowired
    private HlsCusPrjTenantStockStructureMapper prjTenantStockStructureMapper;

    @Autowired
    private PrjTenantStockHoldingService prjTenantStockHoldingService;

    @Autowired
    private HlsCusPrjTenantStockHoldingMapper prjTenantStockHoldingMapper;

    @Autowired
    private PrjTenantSalesCustomerService prjTenantSalesCustomerService;

    @Autowired
    private HlsCusPrjTenantSalesCustomerMapper prjTenantSalesCustomerMapper;

    @Autowired
    private PrjTenantSupplierService prjTenantSupplierService;

    @Autowired
    private HlsCusHlsCreditLineMapper hlsCreditLineMapper;

    @Autowired
    private HlsCreditLineService hlsCreditLineService;

    @Autowired
    private HlsCusPrjTenantSupplierMapper prjTenantSupplierMapper;

    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private IActivitiStartService activitiStartService;
    @Autowired
    private PrjProjectEventService projectEventService;
    @Autowired
    private HlsCreditLineService creditLineService;
    @Autowired
    private PrjProjectMeetingService prjProjectMeetingService;

    @Autowired
    private PrjProjectAttachmentService prjProjectAttachmentService;

    @Autowired
    private IHlsEmployeeAssignsMapper iHlsEmployeeAssignsMapper;

    @Autowired
    private DatabaseLockProvider databaseLockProvider;

    @Autowired
    private SysEventService sysEventService;
    @Autowired
    private FndCompanyMapper fndCompanyMapper;
    @Autowired(required = false)
    private PrjProjectCustomService prjProjectCustomService;


    @Override
    public List<Map> prjIntegratedQuery(IRequest iRequest, HlsCusPrjProject prjProject, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        return prjProjectMapper.prjIntegratedQuery(prjProject);
    }

    @Override
    public List<HlsCusPrjProject> prjProjectDetailQuery(IRequest iRequest, HlsCusPrjProject prjProject, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        return prjProjectMapper.prjProjectDetailQuery(prjProject);
    }

    private Long getReportId(Long projectId, Long bpId) {
        HlsCusBpReport hlsCusBpReport = new HlsCusBpReport();
        hlsCusBpReport.setSourceDocumentCategory("PRJ_PROJECT");
        hlsCusBpReport.setSourceDocumentId(projectId);
        hlsCusBpReport.setBpId(bpId);
        hlsCusBpReport = hlsCusBpReportMapper.selectOne(hlsCusBpReport);
        return hlsCusBpReport.getBpReportId();
    }

    //删除承租人
    @Override
    public void deleteTenant(Long projectId, Long bpId) {

        HlsCusPrjProjectTenant prjProjectTenant = new HlsCusPrjProjectTenant();
        prjProjectTenant.setProjectId(projectId);
        prjProjectTenant.setTenantBpId(bpId);

        prjProjectTenantMapper.delete(prjProjectTenant);

        HlsCusBpReport hlsCusBpReport = new HlsCusBpReport();
        hlsCusBpReport.setBpId(bpId);
        hlsCusBpReport.setSourceDocumentCategory("PRJ_PROJECT");
        hlsCusBpReport.setSourceDocumentId(projectId);


        if (hlsCusBpReportMapper.select(hlsCusBpReport).size() > 0) {
            Long bpReportId = hlsCusBpReportMapper.select(hlsCusBpReport).get(0).getBpReportId();
            hlsCusBpReportMapper.delete(hlsCusBpReport);

            HlsCusPrjTenantHistEvolution prjTenantHistEvolution = new HlsCusPrjTenantHistEvolution();
            prjTenantHistEvolution.setBpReportId(bpReportId);
            prjTenantHistEvolutionMapper.delete(prjTenantHistEvolution);

            HlsCusPrjTenantStockStructure prjTenantStockStructure = new HlsCusPrjTenantStockStructure();
            prjTenantStockStructure.setBpReportId(bpReportId);
            prjTenantStockStructureMapper.delete(prjTenantStockStructure);

            HlsCusPrjTenantStockHolding prjTenantStockHolding = new HlsCusPrjTenantStockHolding();
            prjTenantStockHolding.setBpReportId(bpReportId);
            prjTenantStockHoldingMapper.delete(prjTenantStockHolding);

            HlsCusPrjTenantSupplier prjTenantSupplier = new HlsCusPrjTenantSupplier();
            prjTenantSupplier.setBpReportId(bpReportId);
            prjTenantSupplierMapper.delete(prjTenantSupplier);

            HlsCusPrjTenantSalesCustomer prjTenantSalesCustomer = new HlsCusPrjTenantSalesCustomer();
            prjTenantSalesCustomer.setBpReportId(bpReportId);
            prjTenantSalesCustomerMapper.delete(prjTenantSalesCustomer);
        }

    }

    //担保人
    @Override
    public void deleteGuarantor(Long projectId, Long bpId) {

        HlsCusPrjProjectGuarantor prjProjectGuarantor = new HlsCusPrjProjectGuarantor();
        prjProjectGuarantor.setProjectId(projectId);
        prjProjectGuarantor.setGuarantorBpId(bpId);
        prjProjectGuarantorMapper.delete(prjProjectGuarantor);

        HlsCusBpReport hlsCusBpReport = new HlsCusBpReport();
        hlsCusBpReport.setBpId(bpId);
        hlsCusBpReport.setSourceDocumentCategory("PRJ_PROJECT");
        hlsCusBpReport.setSourceDocumentId(projectId);
        if (hlsCusBpReportMapper.select(hlsCusBpReport).size() > 0) {
            hlsCusBpReportMapper.delete(hlsCusBpReport);
        }

    }


    private HlsCusBpReport insertHlsBpReport(IRequest iRequest, Long bpId, String bpCode, PrjProject prjProject) {
        //商业伙伴调查报告表
        HlsCusBpReport hlsCusBpReport = new HlsCusBpReport();
        hlsCusBpReport.setBpId(bpId);
        hlsCusBpReport.setReportNumber(prjProject.getProjectId() + bpCode);
        hlsCusBpReport.setReportName(prjProject.getProjectName() + bpCode + "的调查报告表");
        hlsCusBpReport.setDocumentType(prjProject.getDocumentType());
        hlsCusBpReport.setDocumentCategory(prjProject.getDocumentCategory());
        hlsCusBpReport.setBusinessType(prjProject.getBusinessType());
        hlsCusBpReport.setSourceDocumentCategory(prjProject.getDocumentCategory());
        hlsCusBpReport.setSourceDocumentId(prjProject.getProjectId());
        hlsCusBpReport.setStatus("NEW");
        hlsCusBpReport.setReportBy(iRequest.getUserId());
        hlsCusBpReport.setReportDate(new Date());
        hlsBpReportService.insertSelective(iRequest, hlsCusBpReport);
        return hlsCusBpReport;
    }


    @Override
    public void save(IRequest iRequest, HlsCusPrjProject prjProject)throws LeaseItemException {

        if (prjProjectCustomService != null && !prjProjectCustomService.beforeSave(iRequest, prjProject)) {
            return;
        }
        prjProject.setCompanyId(iRequest.getCompanyId());
        prjProject.setProjectStatus("NEW");
        //prjProject.setStatus("NEW");
        prjProject.setCurrency("CNY");

        //授权规则字符串设置
        FndCompany fndCompany = new FndCompany();
        fndCompany.setCompany_id(prjProject.getCompanyId());
        fndCompany.setCompanyId(prjProject.getCompanyId());
        fndCompany = fndCompanyMapper.selectByPrimaryKey(fndCompany);
        prjProject.setAuthorityRuleString('"' + fndCompany.getCompany_code() + '"' + "." + '"' + '"' + "." + '"' + '"' + "." + '"' + prjProject.getDocumentType() + '"' + "." + '"' + prjProject.getBusinessType() + '"' + "." + '"' + prjProject.getDivision() + '"' + "." + '"' + '"');
        Map<String, String> params = new HashMap<String, String>();
        PrjProject project = null;
        if (prjProject.getProjectId() == null) {
            HlsEmployeeAssigns hlsEmployeeAssigns = new HlsEmployeeAssigns();
            hlsEmployeeAssigns.setCompany_id(iRequest.getCompanyId());
            hlsEmployeeAssigns.setUser_id(iRequest.getUserId());
            hlsEmployeeAssigns.setRole_id(iRequest.getRoleId());

            hlsEmployeeAssigns = iHlsEmployeeAssignsMapper.selectEmployeeAndUnit(hlsEmployeeAssigns);


            if (hlsEmployeeAssigns != null) {
                prjProject.setEmployeeId(hlsEmployeeAssigns.getEmployee_id());
                prjProject.setUnitId(hlsEmployeeAssigns.getUnit_id());
            }
            prjProject.setProjectNumber(fndCodingRuleValuesService.getCodeRuleValue(iRequest, prjProject.getDocumentCategory(), prjProject.getDocumentType(), prjProject.getBusinessType(), params));
            project = self().insertSelective(iRequest, prjProject);

            sysEventService.createEvent(iRequest, project.getProjectId(), "PRJ_PROJECT", "", null, SysEventCodeUtil.PROPERTY_PRJ_PROJECT_CREATE);

//            projectEventService.eventSave(iRequest, "PRJ_PROJECT.CREATE", project);
        } else {
            project = self().updateByPrimaryKeySelective(iRequest, prjProject);
            sysEventService.createEvent(iRequest, project.getProjectId(), "PRJ_PROJECT", "", null, SysEventCodeUtil.PROPERTY_PRJ_PROJECT_MODIFY);

//            projectEventService.eventSave(iRequest, "PRJ_PROJECT.MODIFY", project);
        }

        Long projectId = prjProject.getProjectId();

        //报价信息
        HlsCusPrjQuotation prjQuotation = new HlsCusPrjQuotation();

        prjQuotation.setQuotationId(prjProject.getQuotationId());
        prjQuotation.setSourceDocumentCategory(prjProject.getDocumentCategory());
        prjQuotation.setSourceDocumentId(projectId);
        prjQuotation.setDescription(prjProject.getDescription());
        prjQuotation.setStatus("NEW");
        prjQuotation.setPriceList(prjProject.getPriceList());
        prjQuotation.setLeaseItemAmount(prjProject.getLeaseItemAmount());
        prjQuotation.setLeaseStartDate(prjProject.getLeaseStartDate());
        prjQuotation.setLeaseTimes(prjProject.getLeaseTimes());
        prjQuotation.setAnnualPayTimes(prjProject.getAnnualPayTimes());
        prjQuotation.setIrr(prjProject.getIrr());

        if (prjQuotation.getQuotationId() == null) {
            prjQuotationService.insertSelective(iRequest, prjQuotation);
            prjProject.setPrjQuotation(prjQuotation);
        } else {
            prjQuotationService.updateByPrimaryKeySelective(iRequest, prjQuotation);
            prjProject.setPrjQuotation(prjQuotation);
        }

        //租赁物
        if (prjProject.getPrjProjectLeaseItems() != null) {
            for (int i = 0; i < prjProject.getPrjProjectLeaseItems().size(); i++) {
                if (prjProject.getPrjProjectLeaseItems().get(i).getProjectLeaseItemId() == null) {
                    prjProject.getPrjProjectLeaseItems().get(i).setProjectId(projectId);
                    prjProjectLeaseItemService.insertSelective(iRequest, prjProject.getPrjProjectLeaseItems().get(i));
                    prjProject.getPrjProjectLeaseItems().set(i, prjProject.getPrjProjectLeaseItems().get(i));

                } else {
                    try {
                        prjProject.getPrjProjectLeaseItems().get(i).setProjectId(projectId);
                        prjProjectLeaseItemService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjProjectLeaseItems().get(i));
                    }catch (Exception e){
                        throw new LeaseItemException();
                    }
                }
            }
        }

        //抵质押物
        if (prjProject.getPrjProjectMortgages() != null) {
            for (int i = 0; i < prjProject.getPrjProjectMortgages().size(); i++) {
                if (prjProject.getPrjProjectMortgages().get(i).getProjectMortgageId() == null) {
                    prjProject.getPrjProjectMortgages().get(i).setProjectId(projectId);
                    prjProjectMortgageService.insertSelective(iRequest, prjProject.getPrjProjectMortgages().get(i));
                    prjProject.getPrjProjectMortgages().set(i, prjProject.getPrjProjectMortgages().get(i));
                } else {
                    prjProject.getPrjProjectMortgages().get(i).setProjectId(projectId);
                    prjProjectMortgageService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjProjectMortgages().get(i));
                }
            }
        }

        //承租人
        if (prjProject.getPrjProjectTenants() != null) {
            for (int i = 0; i < prjProject.getPrjProjectTenants().size(); i++) {
                prjProject.getPrjProjectTenants().get(i).setProjectId(projectId);
                if (prjProject.getPrjProjectTenants().get(i).getProjectTenantId() == null) {
                    prjProjectTenantService.insertSelective(iRequest, prjProject.getPrjProjectTenants().get(i));
                    insertHlsBpReport(iRequest, prjProject.getPrjProjectTenants().get(i).getTenantBpId(), prjProject.getPrjProjectTenants().get(i).getTenantName(), prjProject);
                    prjProject.getPrjProjectTenants().set(i, prjProject.getPrjProjectTenants().get(i));
                } else {
                    prjProject.getPrjProjectTenants().get(i).setProjectId(projectId);
                    prjProjectTenantService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjProjectTenants().get(i));
                }

            }
        }

        //担保人
   if (prjProject.getPrjProjectGuarantors() != null) {
            for (int i = 0; i < prjProject.getPrjProjectGuarantors().size(); i++) {
                prjProject.getPrjProjectGuarantors().get(i).setProjectId(projectId);

                if (prjProject.getPrjProjectGuarantors().get(i).getProjectGuarantorId() == null) {
                    prjProjectGuarantorService.insertSelective(iRequest, prjProject.getPrjProjectGuarantors().get(i));
                    insertHlsBpReport(iRequest, prjProject.getPrjProjectGuarantors().get(i).getGuarantorBpId(), prjProject.getPrjProjectGuarantors().get(i).getGuarantorName(), prjProject);
                    prjProject.getPrjProjectGuarantors().set(i, prjProject.getPrjProjectGuarantors().get(i));
                } else {
                    prjProjectGuarantorService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjProjectGuarantors().get(i));
                }
            }
        }

        //历史沿革
        if (prjProject.getPrjTenantHistEvolutions() != null) {
            for (int i = 0; i < prjProject.getPrjTenantHistEvolutions().size(); i++) {
                if (prjProject.getPrjTenantHistEvolutions().get(i).getEvolutionId() == null) {
                    Long reportId = getReportId(projectId, prjProject.getPrjTenantHistEvolutions().get(i).getBpId());
                    prjProject.getPrjTenantHistEvolutions().get(i).setProjectId(projectId);
                    prjProject.getPrjTenantHistEvolutions().get(i).setBpReportId(reportId);
                    prjTenantHistEvolutionService.insertSelective(iRequest, prjProject.getPrjTenantHistEvolutions().get(i));
                    prjProject.getPrjTenantHistEvolutions().set(i, prjProject.getPrjTenantHistEvolutions().get(i));

                } else {
                    prjTenantHistEvolutionService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjTenantHistEvolutions().get(i));
                }
            }
        }

        //承租人股权结构-主要股东
        if (prjProject.getPrjTenantStockStructures() != null) {
            for (int i = 0; i < prjProject.getPrjTenantStockStructures().size(); i++) {
                if (prjProject.getPrjTenantStockStructures().get(i).getStructureId() == null) {
                    Long reportId = getReportId(projectId, prjProject.getPrjTenantStockStructures().get(i).getBpId());
                    prjProject.getPrjTenantStockStructures().get(i).setProjectId(projectId);
                    prjProject.getPrjTenantStockStructures().get(i).setBpReportId(reportId);
                    prjTenantStockStructureService.insertSelective(iRequest, prjProject.getPrjTenantStockStructures().get(i));
                    prjProject.getPrjTenantStockStructures().set(i, prjProject.getPrjTenantStockStructures().get(i));
                } else {
                    prjTenantStockStructureService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjTenantStockStructures().get(i));
                }
            }
        }

        //承租人股权结构-控股情况
        if (prjProject.getPrjTenantStockHoldings() != null) {
            for (int i = 0; i < prjProject.getPrjTenantStockHoldings().size(); i++) {
                if (prjProject.getPrjTenantStockHoldings().get(i).getHoldingId() == null) {
                    Long reportId = getReportId(projectId, prjProject.getPrjTenantStockHoldings().get(i).getBpId());
                    prjProject.getPrjTenantStockHoldings().get(i).setProjectId(projectId);
                    prjProject.getPrjTenantStockHoldings().get(i).setBpReportId(reportId);
                    prjTenantStockHoldingService.insertSelective(iRequest, prjProject.getPrjTenantStockHoldings().get(i));
                    prjProject.getPrjTenantStockHoldings().set(i, prjProject.getPrjTenantStockHoldings().get(i));

                } else {
                    prjTenantStockHoldingService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjTenantStockHoldings().get(i));
                }
            }
        }

        //承租人经营情况-前五大供应商
        if (prjProject.getPrjTenantSuppliers() != null) {
            for (int i = 0; i < prjProject.getPrjTenantSuppliers().size(); i++) {
                if (prjProject.getPrjTenantSuppliers().get(i).getSupplierId() == null) {
                    Long reportId = getReportId(projectId, prjProject.getPrjTenantSuppliers().get(i).getBpId());
                    prjProject.getPrjTenantSuppliers().get(i).setProjectId(projectId);
                    prjProject.getPrjTenantSuppliers().get(i).setBpReportId(reportId);
                    prjTenantSupplierService.insertSelective(iRequest, prjProject.getPrjTenantSuppliers().get(i));
                    prjProject.getPrjTenantSuppliers().set(i, prjProject.getPrjTenantSuppliers().get(i));
                } else {
                    prjTenantSupplierService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjTenantSuppliers().get(i));
                }
            }
        }
        //承租人经营情况-前五大销售客户
        if (prjProject.getPrjTenantSalesCustomers() != null) {
            for (int i = 0; i < prjProject.getPrjTenantSalesCustomers().size(); i++) {
                if (prjProject.getPrjTenantSalesCustomers().get(i).getCustomerId() == null) {
                    Long reportId = getReportId(projectId, prjProject.getPrjTenantSalesCustomers().get(i).getBpId());
                    prjProject.getPrjTenantSalesCustomers().get(i).setProjectId(projectId);
                    prjProject.getPrjTenantSalesCustomers().get(i).setBpReportId(reportId);
                    prjTenantSalesCustomerService.insertSelective(iRequest, prjProject.getPrjTenantSalesCustomers().get(i));
                    prjProject.getPrjTenantSalesCustomers().set(i, prjProject.getPrjTenantSalesCustomers().get(i));
                } else {
                    prjTenantSalesCustomerService.updateByPrimaryKeySelective(iRequest, prjProject.getPrjTenantSalesCustomers().get(i));
                }
            }

        }
        //插入用户额度表
        HlsCusHlsCreditLine param = new HlsCusHlsCreditLine();
        HlsCusPrjProjectTenant bp = new HlsCusPrjProjectTenant();
        param.setSourceDocumentCategory("PRJ_PROJECT");
        param.setSourceDocumentId(prjProject.getProjectId());
        List<HlsCusHlsCreditLine> data = hlsCreditLineMapper.select(param);
        if ("CREDIT_LINE".equals(prjProject.getBusinessType()) && data.size() == 0) {
            bp.setProjectId(prjProject.getProjectId());
            if (prjProject.getBpId() != null)//设置bpId
                param.setBpId(prjProject.getBpId());
            else
                param.setBpId(prjProjectTenantMapper.select(bp).get(0).getTenantBpId());
            param.setCreditLineName(prjProject.getProjectName() != null ? prjProject.getProjectName() : "");
            param.setCompanyId(prjProject.getCompanyId() != null ? prjProject.getCompanyId() : 0);
            param.setDocumentType(prjProject.getDocumentType() != null ? prjProject.getDocumentType() : "");
            param.setDocumentCategory("NON_REVOLVING");
            param.setBusinessType(prjProject.getBusinessType() != null ? prjProject.getBusinessType() : "");
            param.setCreditLineStatus("NEW");
            param.setValidFrom(prjProject.getValidFrom() != null ? prjProject.getValidFrom() : new Date());
            param.setValidTo(prjProject.getValidTo() != null ? prjProject.getValidTo() : new Date());
            param.setDescription(prjProject.getHclDescription() != null ? prjProject.getHclDescription() : "");
            param.setCreditLineAmt(prjProject.getCreditLineAmt());
            String number = fndCodingRuleValuesService.getCodeRuleValue(iRequest, "PRJ_PROJECT", "CREDIT_LINE", "CREDIT_LINE", new HashMap());
            param.setCreditLineNumber(number);

            hlsCreditLineService.self().insertSelective(iRequest, param);
        }else {
            if(data.size()>0){
                param.setCreditLineId(data.get(0).getCreditLineId());
            }
            param.setBpId(prjProjectTenantMapper.select(bp).get(0).getTenantBpId());
            param.setCreditLineName(prjProject.getProjectName() != null ? prjProject.getProjectName() : "");
            param.setCompanyId(prjProject.getCompanyId() != null ? prjProject.getCompanyId() : 0);
            param.setDocumentType(prjProject.getDocumentType() != null ? prjProject.getDocumentType() : "");
            param.setDocumentCategory("NON_REVOLVING");
            param.setBusinessType(prjProject.getBusinessType() != null ? prjProject.getBusinessType() : "");
            param.setCreditLineStatus("NEW");
            param.setValidFrom(prjProject.getValidFrom() != null ? prjProject.getValidFrom() : new Date());
            param.setValidTo(prjProject.getValidTo() != null ? prjProject.getValidTo() : new Date());
            param.setDescription(prjProject.getHclDescription() != null ? prjProject.getHclDescription() : "");
            param.setCreditLineAmt(prjProject.getCreditLineAmt());
            param.setSourceDocumentId(prjProject.getProjectId());

            hlsCreditLineService.updateCreditlineInfo(param);
        }


        //更新项目附件表

        List<HlsCusPrjProjectAttachment> prjProjectAttachments = prjProject.getPrjProjectAttachments();
        if (prjProjectAttachments != null) {
            for (int i = 0; i < prjProjectAttachments.size(); i++) {
                if (prjProjectAttachments.get(i).getProjectAttachmentId() == null) {
                    prjProjectAttachments.get(i).set__status("add");
                } else {
                    prjProjectAttachments.get(i).set__status("update");
                }
            }
            prjProjectAttachmentService.batchUpdate(iRequest, prjProjectAttachments);

        }
        if (prjProjectCustomService != null) {
            prjProjectCustomService.afterSave(iRequest, prjProject);
        }
    }

    @Override
    public List<Map> prjHomePageGetProjectCount(IRequest iRequest, HlsCusPrjProject prjProject) {
        return prjProjectMapper.prjHomePageGetProjectCount(prjProject);
    }

    @Override
    public List<Map> prjHomePageGetProjectStatusCount(IRequest iRequest, HlsCusPrjProject prjProject) {
        return prjProjectMapper.prjHomePageGetProjectStatusCount(prjProject);
    }
    @Override
    public List<Map> prjHomePageGetAllStatusProjectCount(IRequest iRequest, HlsCusPrjProject prjProject) {
        return prjProjectMapper.prjHomePageGetAllStatusProjectCount(prjProject);
    }

    @Override
    public List<HlsCusPrjProject> prjHomePageProjectInfoGrid(IRequest iRequest, HlsCusPrjProject prjProject, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        return prjProjectMapper.prjHomePageProjectInfoGrid(prjProject);
    }

    @Override
    public List<HlsCusPrjProject> selectAllTenantPrjProhect(HlsCusPrjProjectTenant prjProjectTenant, IRequest iRequest, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        List<HlsCusPrjProject> list = prjProjectMapper.selectAllTenantPrjProhect(prjProjectTenant);
        return list;
    }

    //通过额度项目创建普通项目
    @Override
    public List<HlsCusPrjProject> saveNormalProjectFromCreditProject(HlsCusPrjProject prjProject, IRequest iRequest) {
        List<HlsCusPrjProject> list = new ArrayList<HlsCusPrjProject>();
        Long projectId = prjProject.getProjectId();
        String projectName = prjProject.getProjectName();
        String businessType = prjProject.getBusinessType();
        Double leaseItemAmount = prjProject.getLeaseItemAmount();
        if (prjProjectCustomService != null && !prjProjectCustomService.beforeSave(iRequest, prjProject)) {
        }
        if (new BigDecimal(leaseItemAmount.toString()).compareTo(BigDecimal.ZERO) <= 0) {
            throw new IllegalArgumentException("项目额度错误！");
        }
        HlsCusPrjProject creditProject = self().selectByPrimaryKey(iRequest, prjProject);
        if (creditProject == null) {
            throw new IllegalArgumentException("当前项目不存在！");
        }
        if (!creditProject.getDocumentType().equals("CREDIT_LINE")) {
            throw new IllegalArgumentException("当前项目不是额度项目！");
        }
        if (!creditProject.getProjectStatus().equals("APPROVED")) {
            throw new IllegalArgumentException("当前额度项目未审批通过！");
        }

        creditProject.setBusinessType(businessType);
        if (businessType.equals("LEASE")) {
            creditProject.setDocumentType("PRJL");
        } else if (businessType.equals("LEASEBACK")) {
            creditProject.setDocumentType("PRJLB");
        } else {
            throw new IllegalArgumentException("业务类型只能为直租或者回租!");
        }
        //额度校验及更新
        checkHlsCreditInfoAndUpadate(creditProject, leaseItemAmount, iRequest);

        creditProject.setProjectId(null);

        Map<String, String> params = new HashMap<String, String>();
        //获取编码规则
        creditProject.setProjectNumber(fndCodingRuleValuesService.getCodeRuleValue(iRequest, creditProject.getDocumentCategory(), creditProject.getDocumentType(), creditProject.getBusinessType(), params));
        creditProject.setProjectStatus("NEW");
        creditProject.setProjectName(projectName);
        creditProject.setLeaseItemAmount(leaseItemAmount);
        self().insertSelective(iRequest, creditProject);
        long normalProjectId = creditProject.getProjectId();
        //承租人复制
        copyPrjProjectTenant(projectId, normalProjectId, iRequest);
        //担保人复制
        copyPrjProjectGuarantor(projectId, normalProjectId, iRequest);
        //租赁物复制
        copyPrjProjectLeaseItem(projectId, normalProjectId, iRequest);
        //抵质押物复制
        copyPrjProjectMortgage(projectId, normalProjectId, iRequest);
        //复制商业伙伴调查报告
        copyHlsBpReport(creditProject.getDocumentCategory(), projectId, normalProjectId, iRequest);
        list.add(creditProject);
        if (prjProjectCustomService != null) {
            prjProjectCustomService.afterSave(iRequest, prjProject);
        }
        return list;
    }

    @Override
    public List<Map> prjBpDetailQuery(IRequest iRequest, HlsCusPrjProject prjProject) {
        return prjProjectMapper.prjBpDetailQuery(prjProject);
    }

    //额度项目校验及额度更新
    public void checkHlsCreditInfoAndUpadate(PrjProject prjProject, Double leaseItemAmount, IRequest iRequest) {

        HlsCusHlsCreditLine hlsCreditLineQuery = new HlsCusHlsCreditLine();
        hlsCreditLineQuery.setSourceDocumentId(prjProject.getProjectId());
        hlsCreditLineQuery.setSourceDocumentCategory(prjProject.getDocumentCategory());
        List<HlsCusHlsCreditLine> hlsCreditLineList = hlsCreditLineMapper.hlsCreditLineQueryBySource(hlsCreditLineQuery);
        if (hlsCreditLineList.size() == 0) {
            throw new IllegalArgumentException("当前项目不存在额度信息！");
        }
        //默认一个项目只有一个额度信息
        HlsCusHlsCreditLine hlsCreditLine = hlsCreditLineList.get(0);
        //总额度
        BigDecimal creditLineAmt = new BigDecimal(hlsCreditLine.getCreditLineAmt() != null ? hlsCreditLine.getCreditLineAmt().toString() : "0");
        //已占用额度
        BigDecimal creditExposureAmt = new BigDecimal(hlsCreditLine.getCreditExposureAmt() != null ? hlsCreditLine.getCreditExposureAmt().toString() : "0");
        //本次占用额度
        BigDecimal creditExposureAmtB = new BigDecimal(leaseItemAmount.toString());
        BigDecimal billingAmountTotal = creditExposureAmt.add(creditExposureAmtB);
        //超出额度
        if (billingAmountTotal.subtract(creditLineAmt).compareTo(BigDecimal.ZERO) > 0) {
            throw new IllegalArgumentException("当前项目额度超出！");
        } else {
            //更新额度表
            hlsCreditLine.setCreditExposureAmt(billingAmountTotal.doubleValue());
            Long creditExposureQty = hlsCreditLine.getCreditExposureQty() != null ? hlsCreditLine.getCreditExposureQty() : 0L;
            hlsCreditLine.setCreditExposureQty(creditExposureQty + 1L);
            hlsCreditLineService.updateByPrimaryKeySelective(iRequest, hlsCreditLine);
        }

    }

    //项目承租人商业伙伴复制
    public void copyPrjProjectTenant(Long projectId, Long normalProjectId, IRequest iRequest) {
        //通过原项目id查询复制
        HlsCusPrjProjectTenant prjProjectTenantQuery = new HlsCusPrjProjectTenant();
        prjProjectTenantQuery.setProjectId(projectId);
        List<HlsCusPrjProjectTenant> prjProjectTenantList = prjProjectTenantService.prjProjectTenantDetailInfo(prjProjectTenantQuery, 1, 100);
        if (prjProjectTenantList.size() > 0) {
            for (int i = 0; i < prjProjectTenantList.size(); i++) {
                prjProjectTenantList.get(i).setProjectId(normalProjectId);
                prjProjectTenantList.get(i).setProjectTenantId(null);
                prjProjectTenantService.insertSelective(iRequest, prjProjectTenantList.get(i));

            }
        }
    }

    //项目担保人商业伙伴复制
    public void copyPrjProjectGuarantor(Long projectId, Long normalProjectId, IRequest iRequest) {
        //通过原项目id查询复制
        HlsCusPrjProjectGuarantor prjProjectGuarantorQuery = new HlsCusPrjProjectGuarantor();
        prjProjectGuarantorQuery.setProjectId(projectId);
        List<HlsCusPrjProjectGuarantor> prjProjectGuarantorList = prjProjectGuarantorService.prjProjectGuarantorDetailQuery(prjProjectGuarantorQuery, 1, 100);
        if (prjProjectGuarantorList.size() > 0) {
            for (int i = 0; i < prjProjectGuarantorList.size(); i++) {
                prjProjectGuarantorList.get(i).setProjectId(normalProjectId);
                prjProjectGuarantorList.get(i).setProjectGuarantorId(null);
                prjProjectGuarantorService.insertSelective(iRequest, prjProjectGuarantorList.get(i));

            }
        }
    }

    //项目租赁物复制
    public void copyPrjProjectLeaseItem(Long projectId, Long normalProjectId, IRequest iRequest) {
        HlsCusPrjProjectLeaseItem prjProjectLeaseItem = new HlsCusPrjProjectLeaseItem();
        prjProjectLeaseItem.setProjectId(projectId);
        List<HlsCusPrjProjectLeaseItem> prjProjectLeaseItemList = prjProjectLeaseItemService.prjProjectLeaseItemDetailQuery(prjProjectLeaseItem, 1, 100);
        if (prjProjectLeaseItemList.size() > 0) {
            for (int i = 0; i < prjProjectLeaseItemList.size(); i++) {
                prjProjectLeaseItemList.get(i).setProjectId(normalProjectId);
                prjProjectLeaseItemList.get(i).setProjectLeaseItemId(null);
                prjProjectLeaseItemService.insertSelective(iRequest, prjProjectLeaseItemList.get(i));

            }
        }
    }

    //项目抵质押物复制
    public void copyPrjProjectMortgage(Long projectId, Long normalProjectId, IRequest iRequest) {
        HlsCusPrjProjectMortgage prjProjectMortgageQuery = new HlsCusPrjProjectMortgage();
        prjProjectMortgageQuery.setProjectId(projectId);
        List<HlsCusPrjProjectMortgage> prjProjectMortgageList = prjProjectMortgageService.prjProjectMortgageMapperDetailQuery(prjProjectMortgageQuery, 1, 100);
        if (prjProjectMortgageList.size() > 0) {
            for (int i = 0; i < prjProjectMortgageList.size(); i++) {
                prjProjectMortgageList.get(i).setProjectId(normalProjectId);
                prjProjectMortgageList.get(i).setProjectMortgageId(null);
                prjProjectMortgageService.insertSelective(iRequest, prjProjectMortgageList.get(i));

            }
        }
    }

    //商业伙伴调查报告表复制
    public void copyHlsBpReport(String documentCategory, Long projectId, Long normalProjectId, IRequest iRequest) {
        HlsCusBpReport hlsCusBpReportQuery = new HlsCusBpReport();
        hlsCusBpReportQuery.setDocumentCategory(documentCategory);
        hlsCusBpReportQuery.setSourceDocumentId(projectId);
        List<HlsCusBpReport> hlsCusBpReportList = hlsCusBpReportMapper.hlsBpReportDetailQuery(hlsCusBpReportQuery);
        if (hlsCusBpReportList.size() > 0) {
            for (int i = 0; i < hlsCusBpReportList.size(); i++) {
                Long oldBpReportId = hlsCusBpReportList.get(i).getBpReportId();
                hlsCusBpReportList.get(i).setSourceDocumentId(normalProjectId);
                hlsCusBpReportList.get(i).setDocumentCategory(documentCategory);
                hlsCusBpReportList.get(i).setReportNumber(normalProjectId + hlsCusBpReportList.get(i).getReportNumber());
                hlsBpReportService.insertSelective(iRequest, hlsCusBpReportList.get(i));
                //遍历插入子表
                Long bpReportId = hlsCusBpReportList.get(i).getBpReportId();
                //承租人历史沿革
                copyPrjTenantHistEvolution(oldBpReportId, bpReportId, iRequest);
                //承租人股权结构
                copyPrjTenantStockStructure(oldBpReportId, bpReportId, iRequest);
                //承租人股权结构-控股情况
                copyPrjTenantStockHolding(oldBpReportId, bpReportId, iRequest);
                //承租人经营情况-前五大供应商
                copyPrjTenantSupplier(oldBpReportId, bpReportId, iRequest);
            }
        }

    }

    //承租人历史沿革
    public void copyPrjTenantHistEvolution(Long oldBpReportId, Long bpReportId, IRequest iRequest) {
        HlsCusPrjTenantHistEvolution prjTenantHistEvolutionQuery = new HlsCusPrjTenantHistEvolution();
        prjTenantHistEvolutionQuery.setBpReportId(oldBpReportId);
        List<HlsCusPrjTenantHistEvolution> prjTenantHistEvolutionList = prjTenantHistEvolutionMapper.prjTenantHistEvolutionQueryByReportId(prjTenantHistEvolutionQuery);
        if (prjTenantHistEvolutionList.size() > 0) {
            for (int i = 0; i < prjTenantHistEvolutionList.size(); i++) {
                prjTenantHistEvolutionList.get(i).setBpReportId(bpReportId);
                prjTenantHistEvolutionList.get(i).setEvolutionId(null);
                prjTenantHistEvolutionService.insertSelective(iRequest, prjTenantHistEvolutionList.get(i));

            }
        }
    }

    //承租人持股情况
    public void copyPrjTenantStockStructure(Long oldBpReportId, Long bpReportId, IRequest iRequest) {
        HlsCusPrjTenantStockStructure prjTenantStockStructureQuery = new HlsCusPrjTenantStockStructure();
        prjTenantStockStructureQuery.setBpReportId(oldBpReportId);
        List<HlsCusPrjTenantStockStructure> prjTenantStockStructureList = prjTenantStockStructureMapper.prjTenantStockStructureQueryByReportId(prjTenantStockStructureQuery);
        if (prjTenantStockStructureList.size() > 0) {
            for (int i = 0; i < prjTenantStockStructureList.size(); i++) {
                prjTenantStockStructureList.get(i).setBpReportId(bpReportId);
                prjTenantStockStructureList.get(i).setStructureId(null);
                prjTenantStockStructureService.insertSelective(iRequest, prjTenantStockStructureList.get(i));

            }
        }
    }

    //承租人股权结构-控股情况
    public void copyPrjTenantStockHolding(Long oldBpReportId, Long bpReportId, IRequest iRequest) {
        HlsCusPrjTenantStockHolding prjTenantStockHoldingQuery = new HlsCusPrjTenantStockHolding();
        prjTenantStockHoldingQuery.setBpReportId(oldBpReportId);
        List<HlsCusPrjTenantStockHolding> prjTenantStockHoldingList = prjTenantStockHoldingMapper.prjTenantStockHoldingQueryByReportId(prjTenantStockHoldingQuery);
        if (prjTenantStockHoldingList.size() > 0) {
            for (int i = 0; i < prjTenantStockHoldingList.size(); i++) {
                prjTenantStockHoldingList.get(i).setBpReportId(bpReportId);
                prjTenantStockHoldingList.get(i).setHoldingId(null);
                prjTenantStockHoldingService.insertSelective(iRequest, prjTenantStockHoldingList.get(i));

            }
        }
    }

    //承租人经营情况-前五大供应商
    public void copyPrjTenantSupplier(Long oldBpReportId, Long bpReportId, IRequest iRequest) {
        HlsCusPrjTenantSupplier prjTenantSupplier = new HlsCusPrjTenantSupplier();
        prjTenantSupplier.setBpReportId(oldBpReportId);
        List<HlsCusPrjTenantSupplier> prjTenantSupplierList = prjTenantSupplierMapper.prjTenantSupplierQueryByReportId(prjTenantSupplier);
        if (prjTenantSupplierList.size() > 0) {
            for (int i = 0; i < prjTenantSupplierList.size(); i++) {
                prjTenantSupplierList.get(i).setBpReportId(bpReportId);
                prjTenantSupplierList.get(i).setSupplierId(null);
                prjTenantSupplierService.insertSelective(iRequest, prjTenantSupplierList.get(i));

            }
        }
    }

    //承租人经营情况-前五大销售客户
    public void copyPrjTenantSalesCustomer(Long oldBpReportId, Long bpReportId, IRequest iRequest) {
        HlsCusPrjTenantSalesCustomer prjTenantSalesCustomerQuery = new HlsCusPrjTenantSalesCustomer();
        prjTenantSalesCustomerQuery.setBpReportId(oldBpReportId);
        List<HlsCusPrjTenantSalesCustomer> prjTenantSalesCustomerList = prjTenantSalesCustomerMapper.prjTenantSalesCustomerQueryByReportId(prjTenantSalesCustomerQuery);
        if (prjTenantSalesCustomerList.size() > 0) {
            for (int i = 0; i < prjTenantSalesCustomerList.size(); i++) {
                prjTenantSalesCustomerList.get(i).setBpReportId(bpReportId);
                prjTenantSalesCustomerList.get(i).setCustomerId(null);
                prjTenantSalesCustomerService.insertSelective(iRequest, prjTenantSalesCustomerList.get(i));

            }
        }
    }

    @Override
    public void updatePrjProjectApproved(IRequest iRequest, HlsCusPrjProject prjProject, String approvedResult) {
        if (prjProjectCustomService != null && !prjProjectCustomService.beforeSave(iRequest, prjProject)) {
            return;
        }
        HlsCusPrjProject prjProjectDetail = self().selectByPrimaryKey(iRequest, prjProject);
        if (prjProjectDetail == null) {
            throw new IllegalArgumentException("当前项目不存在！");
        }
        if (prjProjectDetail.getProjectStatus() == null) {
            throw new IllegalArgumentException("项目状态错误，审批通过！");
        } else {
            if (!prjProjectDetail.getProjectStatus().equals("APPROVING")) {
                throw new IllegalArgumentException("项目状态错误，审批通过！");
            }
        }
        prjProjectMapper.updatePrjProjectStatus(prjProjectDetail.getProjectId(), approvedResult, iRequest.getUserId(), new Date());
        if (prjProjectCustomService != null) {
            prjProjectCustomService.afterSubmit(iRequest, prjProject);
        }
    }

    @Override
    public void updatePrjProjectApproving(IRequest iRequest, HlsCusPrjProject prjProject) {
        if (prjProjectCustomService != null && !prjProjectCustomService.beforeSave(iRequest, prjProject)) {
            return;
        }
        HlsCusPrjProject prjProjectDetail = self().selectByPrimaryKey(iRequest, prjProject);
        if (prjProjectDetail == null) {
            throw new IllegalArgumentException("当前项目不存在！");
        }
        String projectStatus = prjProjectDetail.getProjectStatus() == null ? "NEW" : prjProjectDetail.getProjectStatus();

        if (!projectStatus.equals("NEW") && !projectStatus.equals("APPROVED_RETURN") && !projectStatus.equals("CANCEL")) {
            throw new IllegalArgumentException("项目状态错误，无法提交审批！");
        }

        prjProjectMapper.updatePrjProjectStatus(prjProjectDetail.getProjectId(), "APPROVING", iRequest.getUserId(), new Date());
        if (prjProjectCustomService != null) {
            prjProjectCustomService.afterSubmit(iRequest, prjProject);
        }
    }

    @Override
    public void PrjProjectApproving(IRequest iRequest, List<HlsCusPrjProject> projects, HttpSession session) {
        HlsCusPrjProject prjProject = projects.get(0);
        databaseLockProvider.lock(prjProject);
        if (prjProjectCustomService != null && !prjProjectCustomService.beforeSubmit(iRequest, prjProject)) {
            return;
        }
       //修改项目状态
        updatePrjProjectApproving(iRequest,prjProject);
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);
//        projectEventService.eventSave(iRequest, "PRJ_PROJECT.SUBMIT", prjProject);
        sysEventService.createEvent(iRequest, prjProject.getProjectId(), "PRJ_PROJECT", "", null, SysEventCodeUtil.PROPERTY_PRJ_PROJECT_SUBMIT);


        //插入项目上会表
        HlsCusPrjProjectMeeting prjProjectMeeting = new HlsCusPrjProjectMeeting();
        prjProjectMeeting.setProjectId(prjProject.getProjectId());
        prjProjectMeeting = prjProjectMeetingService.insertSelective(iRequest, prjProjectMeeting);


        Map<String, Object> params = new HashMap<String, Object>();
        params.put("workFlowType", "PRJ_PROJECT");
        params.put("projectMeetingId", prjProjectMeeting.getProjectMeetingId());
        activitiStartService.start(iRequest, projects, params);
        if (prjProjectCustomService != null) {
            prjProjectCustomService.afterSubmit(iRequest, prjProject);
        }
    }


    @Override
    public HlsCusPrjProject selectPrjById(Long projectId) {
        return prjProjectMapper.selectPrjById(projectId);
    }

    @Override
    public List<HlsCusPrjProject> queryApplyRecord(Long bpId) {
        return prjProjectMapper.queryApplyRecord(bpId);
    }
}