package hls.core.reportform.controllers;

import com.bstek.ureport.provider.report.ReportFile;
import com.bstek.ureport.provider.report.file.FileReportProvider;
import com.hand.hap.system.dto.Lov;
import com.hand.hap.system.mapper.LovMapper;
import com.hand.hap.system.service.ILovService;
import hls.core.reportform.dto.Attr;
import hls.core.reportform.service.IAttrService;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicHeader;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.reportform.dto.Reportform;
import hls.core.reportform.service.IReportformService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import org.springframework.validation.BindingResult;
import org.springframework.web.context.ContextLoader;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Controller
    public class ReportformController extends BaseController{

    @Value("${ureport.fileStoreDir}")
    private String filedir;
    @Autowired
    private IReportformService service;
    @Autowired
    private ILovService commonLovService;
    @Autowired
    private LovMapper lovMapper;
    @Autowired
    private IAttrService iAttrService;
    @RequestMapping(value = "/reportform/getlov/{lov}")
    @ResponseBody
    public ResponseData getlov(@PathVariable("lov") String lovcode, @RequestParam Map<String, String> params, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        JSONArray jsonArray = new JSONArray();
        Lov lov = lovMapper.selectByCode(lovcode);
        if (null != lov) {
            String textField = lov.getTextField();
            String valueField = lov.getValueField();
            if ((null != lov.getCustomUrl()) && (!lov.getCustomUrl().equals(""))) {
                Cookie[] cookies = request.getCookies();
                StringBuilder sb = new StringBuilder();
                for (Cookie cookie : cookies) {
                    sb.append(cookie.getName() + "=" + cookie.getValue() + ";");

                }
                String cookieValue = sb.toString();


                CloseableHttpClient httpclient = HttpClients.createDefault();
                String path = request.getContextPath();
                String basePath = request.getScheme()+"://"+request.getServerName()+":"+request.getServerPort()+path+"/";
                HttpGet httpget = new HttpGet(basePath+lov.getCustomUrl());
                httpget.addHeader(new BasicHeader("Cookie", cookieValue));
                HttpEntity entity;

                try {
                    CloseableHttpResponse response = httpclient.execute(httpget);
                    entity = response.getEntity();
                    JSONObject jsonObject = JSONObject.fromObject(EntityUtils.toString(entity));
                    String rows = String.valueOf(jsonObject.get("rows"));
                    jsonArray = JSONArray.fromObject(rows);
                } catch (java.io.IOException e) {
                    e.printStackTrace();
                }

            } else  {
                ResponseData lovdata = new ResponseData(commonLovService.selectDatas(requestContext, lovcode, params, 1, 30));
                jsonArray = JSONArray.fromObject(lovdata.getRows());
            }

            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject jsonObject = (JSONObject) (jsonArray.get(i));
                jsonObject.put("valueField", jsonObject.get(valueField));
                jsonObject.put("textField", jsonObject.get(textField));
            }


            ResponseData responseData = new ResponseData(jsonArray);
            return responseData;
        } else {
            return null;
        }

    }

    @RequestMapping(value = "/reportform/loadReportFile")
    @ResponseBody
    public ResponseData loadReportFile() {
//        String basePath = ContextLoader.getCurrentWebApplicationContext().getServletContext().getRealPath("/");
        FileReportProvider fileReportProvider = new FileReportProvider();
//        String filedir = basePath + "/WEB-INF/ureportfiles";
        fileReportProvider.setFileStoreDir(filedir);
        List<ReportFile> files = fileReportProvider.getReportFiles();
        return new ResponseData(files);
    }



    @RequestMapping(value = "/reportform/query")
    @ResponseBody
    public ResponseData query(Reportform dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/reportform/query/{id}")
    @ResponseBody
    public ResponseData queryById(Reportform dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                                  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request, @PathVariable("id") long id) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.queryById(id));
    }


    @RequestMapping(value = "/reportform/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<Reportform> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/reportform/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<Reportform> dto) {
        List<Attr> attrs=new ArrayList<>();
        for(Reportform reportform:dto){
            attrs.addAll(iAttrService.queryById(reportform.getId()));
        }

        service.batchDelete(dto);
        iAttrService.batchDelete(attrs);
        return new ResponseData();
    }
}