package hls.core.rpt.job;

import com.hand.hap.core.IRequest;
import hls.core.job.AbstractJobWithIRequest;
import hls.core.rpt.dto.RptReportDtl;
import hls.core.rpt.dto.RptReportHd;
import hls.core.rpt.job.service.RptTempReceivablesService;
import hls.core.rpt.service.RptReportDtlService;
import hls.core.rpt.service.IRptReportGetDataService;
import hls.core.rpt.service.RptReportEventService;
import hls.core.rpt.service.RptReportHdService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.quartz.JobExecutionContext;
import org.quartz.SchedulerException;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

import java.util.*;

/**
 * Created with IntelliJ IDEA.
 * User: Jeffery
 * Date: 2017/7/31
 * Time: 14:58
 * To change this template use File | Settings | File Templates.
 */
public class RptDayJob extends AbstractJobWithIRequest implements ApplicationContextAware {
    @Autowired
    private RptReportDtlService rptReportDtlService;
    @Autowired
    private RptReportHdService rptReportHdService;
    @Autowired
    private RptReportEventService rptReportEventService;
    @Autowired
    private SysEventService sysEventService;
    private Map<String, IRptReportGetDataService> serviceMap;

    @Override
    public void safeExecuteWithIRequest(JobExecutionContext jobExecutionContext, IRequest iRequest) {
        Long rptId = Long.parseLong(jobExecutionContext.getMergedJobDataMap().get("rptId").toString());
        RptReportHd rptReportHd = new RptReportHd();
        rptReportHd.setReportId(rptId);
        List<RptReportHd> rptReportHdList = rptReportHdService.queryRptReportHdInfoById(rptReportHd);
        RptReportDtl rptReportDtl = new RptReportDtl();
        rptReportDtl.setReleaseDate(new Date());
        rptReportDtl.setReportId(rptId);
        rptReportDtl.setReleaseUserId(rptReportHdList.get(0).getReleaseUserId());
        rptReportDtl.set__status("add");
        List<RptReportDtl> listDtl = new ArrayList<>();
        listDtl.add(rptReportDtl);
        listDtl = rptReportDtlService.batchUpdate(iRequest, listDtl);
        rptReportEventService.saveRelation(iRequest,SysEventCodeUtil.PROPERTY_RPT_REPORT_DTL_RUNNING,listDtl.get(0));
        sysEventService.createEvent(iRequest, listDtl.get(0).getReportDtlId(), "REPORT_DTL", "", null, SysEventCodeUtil.PROPERTY_RPT_REPORT_DTL_RUNNING);
        rptReportHdList.get(0).setReportDtlId(listDtl.get(0).getReportDtlId());
        IRptReportGetDataService iRptReportGetDataService = serviceMap.get(rptReportHdList.get(0).getRptProgram());
        iRptReportGetDataService.tempData(iRequest, rptReportHdList.get(0));
    }

    @Override
    public boolean isRefireImmediatelyWhenException() {
        //任务发生异常时候进行的动作
        //false 挂起我JOB等待处理
        //true 继续执行
        return false;
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        Map<String, IRptReportGetDataService> beansOfType = applicationContext.getBeansOfType(IRptReportGetDataService.class);
        serviceMap = new HashMap<>(beansOfType.size());
        beansOfType.forEach((k, v) -> {
            serviceMap.put(v.getCode(), v);
        });
    }
}
