package hls.core.ruleengine.service.impl;

import com.alibaba.fastjson.JSON;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.ruleengine.dto.GldJeRuleEngineExtend;
import hls.core.ruleengine.dto.HlsRuleEngineCondition;
import hls.core.ruleengine.dto.HlsRuleEngineRoute;
import hls.core.ruleengine.dto.HlsRuleEngineRouteResult;
import hls.core.ruleengine.mapper.GldJeRuleEngineExtendMapper;
import hls.core.ruleengine.mapper.HlsRuleEngineConditionMapper;
import hls.core.ruleengine.mapper.HlsRuleEngineRouteMapper;
import hls.core.ruleengine.mapper.HlsRuleEngineRouteResultMapper;
import hls.core.ruleengine.service.IHlsRuleEngineRouteService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@Transactional
public class HlsRuleEngineRouteServiceImpl extends BaseServiceImpl<HlsRuleEngineRoute> implements IHlsRuleEngineRouteService {

    @Autowired
    private HlsRuleEngineRouteMapper hlsRuleEngineRouteMapper;
    @Autowired
    private HlsRuleEngineConditionMapper hlsRuleEngineConditionMapper;
    @Autowired
    private HlsRuleEngineRouteResultMapper resultMapper;
    @Autowired
    private GldJeRuleEngineExtendMapper gldJeRuleEngineExtendMapper;

    private Map<String, Integer> levelMap = new HashMap<String, Integer>();

    private String getRuleDesc(List<HlsRuleEngineCondition> conditions) {
        String ruleStr = null;
        int i = 0;
        for (HlsRuleEngineCondition condition : conditions) {
            ruleStr = condition.getXconditionValue() + condition.getCalulateSymbol() + condition.getYconditionValue1();
            if ("between".equalsIgnoreCase(condition.getCalulateSymbol())) {
                ruleStr = ruleStr + "and " + condition.getYconditionValue2();
            }
            if (i != conditions.size() - 1 && conditions.size() > 1) {
                ruleStr = ruleStr + ";";
            }
            i = i++;
        }
        return ruleStr;
    }

    private void setLevel(HlsRuleEngineRoute hlsRuleEngineRoute, List<HlsRuleEngineRoute> list) {
        if (hlsRuleEngineRoute.getParentRouteId() == null && hlsRuleEngineRoute.getCurrentLevel() == null) {
            levelMap.put("0", levelMap.get("0") == null ? 1 : (levelMap.get("0") + 1));
            hlsRuleEngineRoute.setCurrentLevel(Integer.toString(levelMap.get("0")));
            return;
        }
        for (int j = 0; j < list.size(); j++) {
            HlsRuleEngineRoute result = list.get(j);
            if (result.getRouteId().equals(hlsRuleEngineRoute.getParentRouteId())) {
                String parentLevel = result.getCurrentLevel();
                if (parentLevel != null) {
                    levelMap.put(parentLevel, levelMap.get(parentLevel) == null ? 1 : levelMap.get(parentLevel) + 1);
                    hlsRuleEngineRoute.setCurrentLevel(result.getCurrentLevel() + "-" + levelMap.get(parentLevel));
                    break;
                } else {
                    setLevel(result, list);
                    setLevel(hlsRuleEngineRoute, list);
                }
            }
        }
    }

    @Override
    public List<HlsRuleEngineRoute> getEngineRouteTree(IRequest requestContext, Long ruleEngineId) {
        List<HlsRuleEngineRoute> routeList = hlsRuleEngineRouteMapper.selectRuleEnginerRouteTree(ruleEngineId);
        levelMap = new HashMap<String, Integer>();
        for (HlsRuleEngineRoute result : routeList) {
            Long routeId = result.getRouteId();
            Boolean parentFlag = false;
            setLevel(result, routeList);
            for (HlsRuleEngineRoute result1 : routeList) {
                if (result1.getParentRouteId() != null && result1.getParentRouteId().equals(routeId)) {
                    parentFlag = true;
                    break;
                }
            }
            String ruleStr = null;
            if ("Y".equals(result.getFinnalFlag())) {
                List<HlsRuleEngineRouteResult> dataList = resultMapper.selectRouteResultByRouteId(routeId);
                GldJeRuleEngineExtend gldJeRuleEngineExtend = new GldJeRuleEngineExtend();
                gldJeRuleEngineExtend.setRouteId(routeId);
                List<GldJeRuleEngineExtend> gldJeRuleEngineExtendList = gldJeRuleEngineExtendMapper.select(gldJeRuleEngineExtend);
                ruleStr = "结果集";
                for (int i = 0; i < dataList.size(); i++) {
                    ruleStr = (i==0?"":",")+ dataList.get(i).getRouteResultValue()+"-"+ dataList.get(i).getRouteResultDescription();
                    if (i != dataList.size() - 1 && dataList.size() > 1) {
                        ruleStr = ruleStr + ";";
                    }
                }
                if(gldJeRuleEngineExtendList.size()>0){
                    ruleStr = "GLD-EXTEND"+ruleStr;
                }
            } else {
                List<HlsRuleEngineCondition> conditions = hlsRuleEngineConditionMapper.selectRuleEngineConditionByRouteId(routeId);
                ruleStr = getRuleDesc(conditions);
            }
            if (ruleStr == null) {
                ruleStr = "规则条件";
            }
            result.setRuleDesc(ruleStr);
        }
        return routeList;
    }

    private String getParentLevel(String currentLevel) {
        String[] levelArr = currentLevel.split("-");
        String parentLevel = null;
        for (int i = 0; i < levelArr.length - 1; i++) {
            if (i == 0) {
                parentLevel = levelArr[i];
            } else {
                parentLevel = parentLevel + "-" + levelArr[i];
            }
        }
        return parentLevel;
    }

    private String getJsonValue(String key, JSONObject json) {
        if (json.containsKey(key)) {
            return json.getString(key);
        } else {
            return null;
        }
    }

    private void setHlsRuleEngineRoute(HlsRuleEngineRoute hlsRuleEngineRoute, JSONObject json) {
        hlsRuleEngineRoute.setFinnalFlag(getJsonValue("finnalFlag", json));
        hlsRuleEngineRoute.setRuleDesc(getJsonValue("ruleDesc", json));
        hlsRuleEngineRoute.setCurrentLevel(getJsonValue("currentLevel", json));
        if (getJsonValue("routeId", json) != null && !"null".equals(json.getString("routeId")) && !"undefined".equals(json.getString("routeId"))) {
            hlsRuleEngineRoute.setRouteId(Long.parseLong(json.getString("routeId")));
        }
        if (getJsonValue("parentRouteId", json) != null && !"null".equals(json.getString("parentRouteId")) && !"undefined".equals(json.getString("parentRouteId"))) {
            hlsRuleEngineRoute.setParentRouteId(Long.parseLong(json.getString("parentRouteId")));
        }
        if (getJsonValue("ruleEngineId", json) != null && !"null".equals(json.getString("ruleEngineId"))) {
            hlsRuleEngineRoute.setRuleEngineId(Long.parseLong(json.getString("ruleEngineId")));
        }
    }

    private void saveLevel(IRequest requestContext, JSONObject json, JSONArray jsonArray) {
        HlsRuleEngineRoute hlsRuleEngineRoute = new HlsRuleEngineRoute();
        setHlsRuleEngineRoute(hlsRuleEngineRoute, json);
        if ((json.containsKey("parentRouteId") && json.getString("parentRouteId") != null && !"undefined".equals(json.getString("parentRouteId")) && json.getString("currentLevel").indexOf("-") != -1) || json.getString("currentLevel").indexOf("-") == -1) {
            hlsRuleEngineRoute = self().insertSelective(requestContext, hlsRuleEngineRoute);
            json.put("routeId", Long.toString(hlsRuleEngineRoute.getRouteId()));
        } else {
            Boolean childFlag = false;
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject json1 = JSON.parseObject(JSON.toJSONString(jsonArray.get(i)));
                if (json1.getString("currentLevel").equals(getParentLevel(hlsRuleEngineRoute.getCurrentLevel()))) {
                    childFlag = true;
                    if (!json1.containsKey("routeId") || (json1.containsKey("routeId") && json1.getString("routeId") == null)) {
                        saveLevel(requestContext, json1, jsonArray);
                    }
                    json.put("parentRouteId", Long.parseLong(json1.getString("routeId")));
                    saveLevel(requestContext, json, jsonArray);
                    break;
                }
            }
            if (!childFlag) {
                hlsRuleEngineRoute = self().insertSelective(requestContext, hlsRuleEngineRoute);
                json.put("routeId", Long.toString(hlsRuleEngineRoute.getRouteId()));
            }
        }
    }

    @Override
    public List<HlsRuleEngineRoute> saveEngineRouteTree(IRequest requestContext, JSONArray jsonArray,JSONArray deleteJsonArray) {
        if(deleteJsonArray!=null){
            for(int j = 0; j < deleteJsonArray.size(); j++){
                JSONObject deleteJson = JSON.parseObject(JSON.toJSONString(deleteJsonArray.get(j)));
                if(deleteJson.getString("routeId")!=null){
                    removeEngineRouteTree(requestContext,Long.parseLong(deleteJson.getString("routeId")));
                }
            }
        }
        List<HlsRuleEngineRoute> routeList = new ArrayList<>();
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject json = JSON.parseObject(JSON.toJSONString(jsonArray.get(i)));
            if (!json.containsKey("routeId") || (json.containsKey("routeId") && json.getString("routeId") == null)) {
                String currentLevel = json.getString("currentLevel");
                saveLevel(requestContext, json, jsonArray);
                jsonArray.remove(jsonArray.get(i));
                jsonArray.add(i,json);
            }
            HlsRuleEngineRoute hlsRuleEngineRoute = new HlsRuleEngineRoute();
            setHlsRuleEngineRoute(hlsRuleEngineRoute, json);
            routeList.add(hlsRuleEngineRoute);
        }
        return routeList;
    }

    @Override
    public void removeEngineRouteTree(IRequest requestContext, Long routeId) {
        hlsRuleEngineRouteMapper.removeRuleEngineByRouteId1(routeId);
        hlsRuleEngineRouteMapper.removeRuleEngineByRouteId2(routeId);
        hlsRuleEngineRouteMapper.removeRuleEngineByRouteId3(routeId);
    }
}