package hls.core.ruleengine.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.intergration.beans.HapJDBCSqlSessionFactory;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.hls.components.Datasource2Json;
import hls.core.ruleengine.dto.HlsRuleEngine;
import hls.core.ruleengine.dto.RuleEngineType;
import hls.core.ruleengine.mapper.HlsRuleEngineMapper;
import hls.core.ruleengine.mapper.RuleEngineTypeMapper;
import hls.core.ruleengine.service.IHlsRuleEngineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Transactional
public class HlsRuleEngineServiceImpl extends BaseServiceImpl<HlsRuleEngine> implements IHlsRuleEngineService {

    @Autowired
    private HlsRuleEngineMapper hlsRuleEngineMapper;
    @Autowired
    private HapJDBCSqlSessionFactory hapSqlSessionFactory;
    @Autowired
    private RuleEngineTypeMapper ruleEngineTypeMapper;
    @Autowired
    private Datasource2Json datasource2Json;

    private String analyzSql(String sql,IRequest iRequest,Map param) {
        String oldStr = "\\u007B\\u0024" + "session\\u002E"+"userId" + "\\}";
        String oldStr1 = "\\u007B\\u0024" + "session\\u002E"+"companyId" + "\\}";
        String newSql = sql.replaceAll(oldStr,Long.toString(iRequest.getUserId()))
                .replaceAll(oldStr1,Long.toString(iRequest.getCompanyId()));//userId和companyId先区分处理
        String result = paramReplace(newSql,param);
        return result;
    }

    @Override
    public List<Map<String, Object>> getHlsRuleEngine(IRequest iRequest,Map param) {
        Long ruleEngineId = Long.valueOf(param.get("ruleEngineId").toString());
        HlsRuleEngine hlsRuleEngine = hlsRuleEngineMapper.selectByPrimaryKey(ruleEngineId);
        Long dataSourceId = hlsRuleEngine.getDataSourceId();
        // 更改为从数据源获取
        String engineType = hlsRuleEngine.getRuleEngineType();
        RuleEngineType ruleEngineType = ruleEngineTypeMapper.selectByPrimaryKey(engineType);
        List<Map<String, Object>> resultList = hapSqlSessionFactory.getJdbcTemplateObject().queryForList(analyzSql(ruleEngineType.getRuleResultSql(),iRequest,param));

        // LOV查询条件 , 前台传过来NAME和CODE
        if(param.get("NAME") != null){
            resultList = resultList.stream()
                    .filter(item -> param.get("NAME").toString().indexOf(item.get("NAME").toString()) != -1)
                    .collect(Collectors.toList());
        }
        if(param.get("CODE") != null){
            resultList = resultList.stream()
                    .filter(item -> param.get("CODE").toString().indexOf(item.get("CODE").toString()) != -1)
                    .collect(Collectors.toList());
        }
        return resultList;
    }

    @Override
    public HlsRuleEngine queryByType(String type) {
        return hlsRuleEngineMapper.queryByType(type);
    }

    @Override
    public List<HlsRuleEngine> selectInfo(IRequest requestContext, HlsRuleEngine dto, int page, int pageSize) {
        PageHelper.startPage(page, pageSize);
        return hlsRuleEngineMapper.selectInfo(dto);
    }

    /**
     * map为参数集合,格式固定为{$parameter.xxx}
     * @param paramsql
     * @param param
     * @return
     */
    private static String paramReplace(String paramsql, Map param){

        StringBuffer sql = new StringBuffer(paramsql);
        String paramName; //定义参数名
        String paramValue; //定义参数值
        while (sql.lastIndexOf("{$")>0){
            paramName = sql.subSequence(sql.lastIndexOf("{$parameter.")+12, sql.lastIndexOf("}")).toString();//获取标示中的参数名
            if(param.get(paramName)!=null) {
                //区分参数类型
                if (param.get(paramName) instanceof Long || param.get(paramName) instanceof Integer){
                    paramValue = param.get(paramName).toString();//从HashMap中将参数取出
                }
                else {
                    paramValue = "\'"+param.get(paramName)+"\'";
                }
            }else{
                paramValue = "null"; //若没有得到该参数的值，则在sql上替换为"null"这样使得sql执行结果为空避免报错
            }
            sql.replace(sql.lastIndexOf("{$parameter."), sql.lastIndexOf("}")+12,paramValue);

        }
        return sql.toString();
    }
}