package hls.core.ruleengine.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hand.hap.core.IRequest;
import hls.core.ruleengine.dto.GldJeRuleEngineExtend;
import hls.core.ruleengine.dto.HlsRuleEngineCondition;
import hls.core.ruleengine.dto.HlsRuleEngineRoute;
import hls.core.ruleengine.dto.HlsRuleEngineRouteResult;
import hls.core.ruleengine.mapper.GldJeRuleEngineExtendMapper;
import hls.core.ruleengine.mapper.HlsRuleEngineConditionMapper;
import hls.core.ruleengine.mapper.HlsRuleEngineRouteMapper;
import hls.core.ruleengine.mapper.HlsRuleEngineRouteResultMapper;
import hls.core.ruleengine.service.IHLSRuleEngineInitService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by gaoyang on 2017/5/12.
 */
@Service
public class IHLSRuleEngineInitServiceImpl implements IHLSRuleEngineInitService {

    public static final String PROPERTITY_CONDITION_FILED = "FIELD";

    @Autowired
    private HlsRuleEngineRouteMapper hlsRuleEngineRouteMapper;
    @Autowired
    private HlsRuleEngineRouteResultMapper routeResultMapper;
    @Autowired
    private HlsRuleEngineConditionMapper conditionMapper;
    @Autowired
    private GldJeRuleEngineExtendMapper gldJeRuleEngineExtendMapper;
    private int resultLength = 0;
    private List<HlsRuleEngineRoute> routeList = new ArrayList<>();

    private String getConditionValue(String conditionType, String conditionValue, JSONObject jsonObject) {
        String value = null;
        if (PROPERTITY_CONDITION_FILED.equals(conditionType)) {
            if (jsonObject.containsKey(conditionValue)) {
                value = jsonObject.getString(conditionValue);
            }
        } else {
            value = conditionValue;
        }
        return value;
    }

    private boolean checkRuleCondition(HlsRuleEngineRoute engineRoute, JSONObject jsonObject) {
        boolean flag = true;
        List<HlsRuleEngineCondition> conditionList = conditionMapper.selectRuleEngineConditionByRouteId(engineRoute.getRouteId());
        if (conditionList != null) {
            for (HlsRuleEngineCondition condition : conditionList) {
                String xValue = getConditionValue(condition.getXconditionType(), condition.getXconditionValue(), jsonObject),
                        yValue1 = condition.getYconditionValue1(),
                        yValue2 = condition.getYconditionValue2();
                String calulateSymbol = condition.getCalulateSymbol();
                if ("=".equals(calulateSymbol)) {
                    if (xValue != null && yValue1 != null && xValue.equals(yValue1)) {
                        flag = true;
                    } else {
                        flag = false;
                        break;
                    }
                } else if ("!=".equals(calulateSymbol)) {
                    if (xValue != null && yValue1 != null && !xValue.equals(yValue1)) {
                        flag = true;
                    } else {
                        flag = false;
                        break;
                    }
                } else if ("<".equals(calulateSymbol)) {
                    try {
                        if (xValue != null && yValue1 != null && Float.parseFloat(xValue) < Float.parseFloat(yValue1)) {
                            flag = true;
                        } else {
                            flag = false;
                            break;
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        flag = false;
                        break;
                    }
                } else if ("<=".equals(calulateSymbol)) {
                    try {
                        if (xValue != null && yValue1 != null && Float.parseFloat(xValue) <= Float.parseFloat(yValue1)) {
                            flag = true;
                        } else {
                            flag = false;
                            break;
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        flag = false;
                        break;
                    }
                } else if (">".equals(calulateSymbol)) {
                    try {
                        if (xValue != null && yValue1 != null && Float.parseFloat(xValue) > Float.parseFloat(yValue1)) {
                            flag = true;
                        } else {
                            flag = false;
                            break;
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        flag = false;
                        break;
                    }
                } else if (">=".equals(calulateSymbol)) {
                    try {
                        if (xValue != null && yValue1 != null && Float.parseFloat(xValue) >= Float.parseFloat(yValue1)) {
                            flag = true;
                        } else {
                            flag = false;
                            break;
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        flag = false;
                        break;
                    }
                } else if ("between".equals(calulateSymbol)) {
                    try {
                        if (xValue != null && yValue1 != null && yValue2 != null & Float.parseFloat(xValue) >= Float.parseFloat(yValue1) && Float.parseFloat(xValue) <= Float.parseFloat(yValue2)) {
                            flag = true;
                        } else {
                            flag = false;
                            break;
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        flag = false;
                        break;
                    }
                }
            }
        }
        return flag;
    }

    private void getRuleResult(HlsRuleEngineRoute engineRoute, String[] strArray) {
        List<HlsRuleEngineRouteResult> resultList = routeResultMapper.selectRouteResultByRouteId(engineRoute.getRouteId());
        if (resultList != null) {
            int i = 0;
            for (HlsRuleEngineRouteResult result : resultList) {
                strArray[resultLength + i] = result.getRouteResultValue();
                i++;
            }
            resultLength = resultLength + resultList.size();
        }
    }

    private void checkEngineRoute(HlsRuleEngineRoute engineRoute, String[] strArray, JSONObject jsonObject) {
        List<HlsRuleEngineRoute> childrenList = hlsRuleEngineRouteMapper.selectRuleEngineChildRoute(engineRoute.getRouteId());
        if (childrenList.size() == 0) {
            getRuleResult(engineRoute, strArray);
            routeList.add(engineRoute);
        } else {
            boolean checkResult = checkRuleCondition(engineRoute, jsonObject);
            if (checkResult) {
                for (HlsRuleEngineRoute child : childrenList) {
                    checkEngineRoute(child, strArray, jsonObject);
                }
            }
        }
    }

    private void calcResult(IRequest iRequest, JSONObject jsonObject, String[] strArray) {
        Long ruleEngineId = jsonObject.getLong("ruleEngineId");
        List<HlsRuleEngineRoute> routeList = hlsRuleEngineRouteMapper.selectRuleEngineRoute(ruleEngineId);
        if (routeList != null) {
            for (HlsRuleEngineRoute engineRoute : routeList) {
                checkEngineRoute(engineRoute, strArray, jsonObject);
            }
        }
    }

    private void getGldRuleResult(HlsRuleEngineRoute engineRoute, Map<String,String> mapList) {
        GldJeRuleEngineExtend gldJeRuleEngineExtend = new GldJeRuleEngineExtend();
        gldJeRuleEngineExtend.setRouteId(engineRoute.getRouteId());
        List<GldJeRuleEngineExtend> gldResultList = gldJeRuleEngineExtendMapper.select(gldJeRuleEngineExtend);
        List<HlsRuleEngineRouteResult> routeResultList = routeResultMapper.selectRouteResultByRouteId(engineRoute.getRouteId());
        if (gldResultList.size()>0) {
            for(int i=0;i<routeResultList.size();i++){
                String resultValue = routeResultList.get(i).getRouteResultValue();
                String ruleEngineExtendId = gldResultList.get(0).getRuleEngineExtendId().toString();
                if(mapList.get(resultValue)!=null){
                    mapList.put(resultValue,resultValue+"$"+ruleEngineExtendId);
                }else {
                    mapList.put(resultValue, ruleEngineExtendId);
                }
            }
        }
    }

    private void gldCalcResult(IRequest iRequest, JSONObject jsonObject, Map<String,String> mapList) {
        if (routeList != null) {
            for (HlsRuleEngineRoute engineRoute : routeList) {
                if("Y".equals(engineRoute.getFinnalFlag())){
                    getGldRuleResult(engineRoute, mapList);
                }
            }
        }
    }

    @Override
    public String[] ruleEngineInit(IRequest iRequest, JSONObject jsonObject) {
        resultLength = 0;
        routeList.clear();
        String[] strArray = new String[1000];
        calcResult(iRequest, jsonObject, strArray);
        String[] resultArray = new String[resultLength];
        for (int i = 0; i < strArray.length; i++) {
            if (strArray[i] != null) {
                resultArray[i] = strArray[i];
            }
        }
        return resultArray;
    }

    private Map<String,String> gldRuleEngine(IRequest iRequest, JSONObject jsonObject){
        Map<String,String> mapList = new HashMap<>();
        gldCalcResult(iRequest,jsonObject,mapList);
        return mapList;
    }

    @Override
    public Map<String,Object> gldRuleEngineInit(IRequest iRequest, JSONObject jsonObject) {
        Map<String,Object> resultMap = new HashMap();
        resultMap.put("routeResult",ruleEngineInit(iRequest,jsonObject));
        resultMap.put("gldRouteResult",gldRuleEngine(iRequest,jsonObject));
        return resultMap;
    }
}
