package hls.core.svc.components;

import com.alibaba.fastjson.JSON;
import hls.core.svc.beans.*;
import hls.core.svc.utils.ModelParser;
import hls.core.svc.utils.ParamUtils;
import hls.core.svc.utils.SqlCreator;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.xmlpull.v1.XmlPullParserException;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2018-01-03
 * Time: 15:24
 */
@Component
public class ModelExecutor {
    @Autowired
    private SqlExecutor sqlExecutor;
    private Logger logger = LoggerFactory.getLogger(getClass());
    private ThreadLocal<HttpServletRequest> localRequest = new ThreadLocal<>();

    public int delete(BusinessModel model, Map<String, Object> params) {
        SqlCreator sqlCreator = new SqlCreator(model);
        String sql = sqlCreator.getDeleteByPrimaryKey();
        logger.debug(sql);
        logger.debug("params: {}", JSON.toJSONString(params));
        return sqlExecutor.executeDelete(sql, params);
    }


    public List<Map<String, Object>> query(BusinessModel model) {
        SqlCreator sqlCreator = new SqlCreator(model);
        String sql = sqlCreator.getSelectAll();
        logger.debug(sql);
        return sqlExecutor.executeQuery(sql, null);
    }

    public List<Map<String, Object>> query(BusinessModel model, Map<String, Object> params) {
        SqlCreator sqlCreator = new SqlCreator(model);
        String sql = sqlCreator.getSelect();
        logger.debug(sql);
        return sqlExecutor.executeQuery(sql, params);
    }

    public Map<String, Object> save(BusinessModel model, Map<String, Object> params) {
        SqlCreator sqlCreator = new SqlCreator(model);
        String insert = sqlCreator.getInsert();
        return sqlExecutor.executeInsert(insert, params);
    }

    public Map<String, Object> update(BusinessModel model, Map<String, Object> params) {
        SqlCreator sqlCreator = new SqlCreator(model);
        String sql = sqlCreator.getUpdateByPrimaryKey();
        return sqlExecutor.executeUpdate(sql, params);
    }

    public Map<String, Object> updateSelective(BusinessModel model, Map<String, Object> params) {
        SqlCreator sqlCreator = new SqlCreator(model);
        String sql = sqlCreator.getUpdateByPrimaryKeyWithParams(params);
        return sqlExecutor.executeUpdate(sql, params);
    }

    private Map<String, Object> prepareSourceParams(String sourcePath) {
        return dive(ParamUtils.getParamMap(), sourcePath);
    }

    private Map<String, Object> dive(Map<String, Object> map, String path) {
        if (StringUtils.isBlank(path) || "/".equals(path)) {
            return map;
        }
        StringBuilder sb = new StringBuilder(path);
        if (sb.charAt(0) == '/') {
            sb.deleteCharAt(0);
        }
        int lastSplash = sb.lastIndexOf("/");
        // last level
        if (lastSplash < 0) {
            Object o = map.get(sb.toString());
            if (o instanceof Map) {
                return (Map) o;
            } else {
                return Collections.emptyMap();
            }

        }
        String level = sb.substring(0, lastSplash);
        Object o = map.get(level);
        if (o instanceof Map) {
            return dive((Map) o, sb.toString());
        }
        return Collections.emptyMap();

    }

    public Object delete(ModelDelete modelDelete) throws IOException, XmlPullParserException {
        Assert.notNull(modelDelete, "modelDelete can not be null");
        String model = modelDelete.getModel();
        Assert.notNull(model, "model can not be null");
        String sourcePath = modelDelete.getSourcePath();
        if (StringUtils.isBlank(sourcePath)) {
            sourcePath = "/";
        }
        BusinessModel businessModel = ModelParser.load(model);
        Map<String, Object> params = prepareSourceParams(sourcePath);
        return this.delete(businessModel, params);
    }

    public Object insert(ModelInsert modelInsert) throws IOException, XmlPullParserException {
        Assert.notNull(modelInsert, "modelInsert can not be null");
        String model = modelInsert.getModel();
        Assert.notNull(model, "model can not be null");
        String sourcePath = modelInsert.getSourcePath();
        if (StringUtils.isBlank(sourcePath)) {
            sourcePath = "/";
        }
        BusinessModel businessModel = ModelParser.load(model);
        Map<String, Object> params = prepareSourceParams(sourcePath);
        return this.save(businessModel, params);
    }

    public Object query(ModelQuery modelQuery) throws IOException, XmlPullParserException {
        Assert.notNull(modelQuery, "modelQuery can not be null");
        String model = modelQuery.getModel();
        Assert.notNull(model, "model can not be null");
        String sourcePath = modelQuery.getSourcePath();
        if (StringUtils.isBlank(sourcePath)) {
            sourcePath = "/";
        }
        BusinessModel businessModel = ModelParser.load(model);
        Map<String, Object> params = prepareSourceParams(sourcePath);
        return this.query(businessModel, params);
    }

    public Object update(ModelUpdate modelUpdate) throws IOException, XmlPullParserException {
        Assert.notNull(modelUpdate, "modelUpdate can not be null");
        String model = modelUpdate.getModel();
        Assert.notNull(model, "model can not be null");
        String sourcePath = modelUpdate.getSourcePath();
        if (StringUtils.isBlank(sourcePath)) {
            sourcePath = "/";
        }
        BusinessModel businessModel = ModelParser.load(model);
        Map<String, Object> params = prepareSourceParams(sourcePath);
        Boolean updatePassedFieldOnly = modelUpdate.getUpdatePassedFieldOnly();
        if (updatePassedFieldOnly != null && updatePassedFieldOnly) {
            return this.updateSelective(businessModel, params);
        }
        return this.update(businessModel, params);
    }


}
