package hls.core.svc.service.impl;

import hls.core.svc.components.SqlExecutor;
import hls.core.svc.service.BmService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.RowCallbackHandler;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2017-12-21
 * Time: 11:17
 */
@Service
public class BmServiceImpl implements BmService {
    @Autowired
    private SqlExecutor executor;

    @Override
    public List<Map<String, Object>> executeQuery(String sql, Map<String, Object> params) {
        return executor.executeQuery(sql, params);
    }

    @Override
    public void executeQuery(String sql, Map<String, Object> params, RowCallbackHandler rowCallbackHandler) {
        executor.executeQuery(sql, params, rowCallbackHandler);
    }

    @Override
    public Map<String, Object> executeUpdate(String sql, Map<String, Object> params) {
        return executor.executeUpdate(sql, params);
    }

    @Override
    @Transactional
    public List<Map<String, Object>> batchUpdate(String sql, List<Map<String, Object>> params) {
        List<Map<String, Object>> list = new ArrayList<>();
        if (params != null && params.size() > 0) {
            for (Map<String, Object> param : params) {
                Map<String, Object> map = executor.executeUpdate(sql, param);
                list.add(map);
            }
        }
        return list;
    }

    @Override
    public Map<String, Object> executeUpdate(String sql, SqlParameterSource parameterSource) {
        return executor.executeUpdate(sql, parameterSource);
    }

    @Override
    public Map<String, Object> insert(String sql, Map<String, Object> params) {
        return executor.executeInsert(sql, params);
    }

    @Override
    public List<Map<String, Object>> batchInsert(String sql, List<Map<String, Object>> params) {
        List<Map<String, Object>> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(params)) {
            for (Map<String, Object> param : params) {
                list.add(executor.executeInsert(sql, param));
            }
        }
        return list;
    }

    @Override
    public int executeDelete(String sql, Map<String, Object> params) {
        return executor.executeDelete(sql,params);
    }
}
