package hls.core.svc.utils;

import com.alibaba.fastjson.JSONObject;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.impl.RequestHelper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * Created with IntelliJ IDEA.
 * User: yang
 * Date: 2017-12-20
 * Time: 16:40
 */
public class ParamUtils {
    private static ThreadLocal<HttpServletRequest> requestThreadLocal = new ThreadLocal<>();
    private static Logger logger = LoggerFactory.getLogger(ParamUtils.class);

    public static HttpServletRequest getCurrentHttpRequest() {
        return requestThreadLocal.get();
    }

    public static void setCurrentHttpRequest(HttpServletRequest request) {
        requestThreadLocal.set(request);

    }

    public static Map<String, Object> getParamMap() {
        Map<String, Object> params = new HashMap<>();
        HttpServletRequest request = requestThreadLocal.get();

        if (request == null) {
            return params;
        }

        String param = request.getParameter("_param");
        if (StringUtils.isNotBlank(param)) {
            try {
                Map map = JSONObject.parseObject(param, Map.class);
                params.put("request", map);
            } catch (Throwable throwable) {
                logger.debug("params cast error", throwable);
            }
        }

        HttpSession session = request.getSession(false);
        if (session != null) {
            Enumeration<String> attributeNames = session.getAttributeNames();
            if (attributeNames != null) {
                Map<String, Object> sessionMap = new HashMap<>();
                while (attributeNames.hasMoreElements()) {
                    String name = attributeNames.nextElement();
                    Object attribute = session.getAttribute(name);
                    sessionMap.put(name, attribute);
                }
                params.put("session", sessionMap);
            }
        }
        return params;
    }

    public static Map<String, Object> prepareParams() {
        IRequest currentRequest = RequestHelper.getCurrentRequest();
        Map<String, Object> attributeMap = currentRequest.getAttributeMap();
        HttpServletRequest httpServletRequest = requestThreadLocal.get();
        HashMap<String, Object> param = new HashMap<>();
        if (httpServletRequest != null) {
            Map<String, String[]> parameterMap = httpServletRequest.getParameterMap();
            if (parameterMap != null && parameterMap.size() > 0) {
                parameterMap.forEach((k, v) -> {
                    if (v != null && v.length == 1) {
                        param.put(k, v[0]);
                    } else {
                        param.put(k, v);
                    }
                });
            }
        }
        HashMap<String, Object> map = new HashMap<>(attributeMap.size() + param.size());
        attributeMap.forEach((key, value) -> map.put("$session." + key, value));
        param.forEach((key, value) -> map.put("$params." + key, value));
        map.put("$session.userId", currentRequest.getUserId());
        map.put("$session.roleId", currentRequest.getRoleId());
        map.put("$session.employeeCode", currentRequest.getEmployeeCode());
        map.put("$session.userName", currentRequest.getUserName());
        map.put("$session.roleIds", currentRequest.getAllRoleId());

        return map;
    }
}
