package hls.core.sys.controllers;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.hand.hap.system.dto.SysConfig;
import com.hand.hap.system.dto.SysPreferences;
import com.hand.hap.system.mapper.SysConfigMapper;
import com.hand.hap.system.service.ISysPreferencesService;
import com.hand.hap.system.service.impl.SysPreferencesServiceImpl;
import hls.core.fnd.dto.FndCompanyAttribute;
import hls.core.fnd.mapper.FndCompanyAttributeMapper;
import hls.core.sys.components.HlsAuthenticationComponent;
import hls.core.sys.dto.SysUser;
import hls.core.sys.dto.SysUserAuthorityRule;
import hls.core.sys.mapper.SysUserAuthorityRuleMapper;
import hls.core.sys.service.SysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import com.hand.hap.account.exception.RoleException;
import com.hand.hap.system.controllers.BaseController;
import hls.core.sys.dto.SysUserAllocation;
import hls.core.sys.service.SysUserAllocationService;
import org.springframework.web.servlet.ThemeResolver;
import org.springframework.web.servlet.support.RequestContextUtils;

/**
 * Created by gaoyang on 16/11/29.
 */
@Controller
public class HlsRoleController extends BaseController {
    public static final String DEFAULT_INDEX = "/index";
    public static final String DEFAULT_TARGET_URL = "DEFAULT_TARGET_URL";
    @Autowired
    private SysUserAllocationService service;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private ISysPreferencesService iSysPreferencesService;
    @Autowired
    private FndCompanyAttributeMapper fndCompanyAttributeMapper;
    @Autowired
    private SysUserAuthorityRuleMapper sysUserAuthorityRuleMapper;
    @Autowired
    private SysConfigMapper sysConfigMapper;
    @Autowired
    private HlsAuthenticationComponent component;

    protected static final String REDIRECT = "redirect:";

    /**
     * 角色选择逻辑.
     * <p>
     * 角色对象
     *
     * @param request  HttpServletRequest
     * @param response HttpServletResponse
     * @return view
     * @throws RoleException
     */
    @RequestMapping(value = "/hlsRole", method = RequestMethod.POST)
    public ModelAndView selectRole(final SysUserAllocation sua,
                                   final HttpServletRequest request, final HttpServletResponse response)
            throws RoleException {
        ModelAndView result = new ModelAndView();
        List<SysUserAllocation> list = service.editQuery(sua.getUserId(), sua.getAllocationId());
        HttpSession session = request.getSession(false);
        if (session != null && sua != null && sua.getAllocationId() != null) {
            SysUserAllocation sa = (SysUserAllocation) list.get(0);
            SysUser sysUser = sysUserService.selectUserById(sa.getUserId());
            session.setAttribute(SysUserAllocation.FIELD_ALLOCATION_ID,
                    sa.getAllocationId());
            session.setAttribute(SysUserAllocation.FIELD_ROLE_ID,
                    sa.getRoleId());
            session.setAttribute(SysUserAllocation.FIELD_ROLE_NAME,
                    sa.getRoleName());
            session.setAttribute(SysUserAllocation.FIELD_COMPANY_ID,
                    sa.getCompanyId());
            session.setAttribute(SysUserAllocation.FIELD_COMPANY_NAME,
                    sa.getCompanyShortName());
            session.setAttribute(SysUserAllocation.FIELD_POSITION_ID,
                    sa.getPositionId());
            session.setAttribute(SysUserAllocation.FIELD_UNIT_ID,
                    sa.getUnitId());
            session.setAttribute(SysUserAllocation.FIELD_EMPLOYEEASSIGNID,
                    sa.getEmployeeAssignId());
            session.setAttribute(SysUserAllocation.FIELD_POSITIONNAME,
                    sa.getPositionName());
            session.setAttribute(SysUserAllocation.FIELD_POSITIONCODE,
                    sa.getPositionCode());
            session.setAttribute(SysUserAllocation.FIELD_EMPLOYEENAME,
                    sa.getName());
            session.setAttribute(SysUserAllocation.FIELD_UNITCODE,
                    sa.getUnitCode());
            session.setAttribute(SysUserAllocation.FIELD_UNITNAME,
                    sa.getUnitName());
            session.setAttribute(SysUserAllocation.FIELD_EMAIL,
                    sa.getEmail());
            session.setAttribute(SysUserAllocation.SYS_DATE,
                    sa.getSysDate());
            session.setAttribute(SysUserAllocation.DESCRIPTION,
                    sysUser.getDescription());
            session.setAttribute("employeeId", sa.getEmployeeId());
            session.setAttribute("employeeCode", sa.getEmployeeCode());

            //皮肤
            SysPreferences sysPreferences = iSysPreferencesService.selectUserPreference("theme", sa.getUserId());
            if (sysPreferences != null) {
                ThemeResolver themeResolver = RequestContextUtils.getThemeResolver(request);
                themeResolver.setThemeName(request, response, sysPreferences.getPreferencesValue());
            }
            FndCompanyAttribute fndCompanyAttribute = new FndCompanyAttribute();
            fndCompanyAttribute.setCompanyId((Long) session.getAttribute("companyId"));
            List<FndCompanyAttribute> attrList = fndCompanyAttributeMapper.select(fndCompanyAttribute);
            if (attrList.size() > 0) {
                session.setAttribute("authorityRuleFlag", attrList.get(0).getAuthorityRuleFlag());
            }
            SysUserAuthorityRule sysUserAuthorityRule = new SysUserAuthorityRule();
            sysUserAuthorityRule.setAllocationId(sa.getAllocationId());
            List<SysUserAuthorityRule> ruleList = sysUserAuthorityRuleMapper.select(sysUserAuthorityRule);
            if (ruleList.size() > 0) {
                session.setAttribute("userRules",ruleList);
            }
            List<SysConfig> sysConfigs = sysConfigMapper.selectAll();
            String index = sysConfigs.stream().filter(item ->
                    DEFAULT_TARGET_URL.equalsIgnoreCase(item.getConfigCode())
            ).findFirst().map(item -> item.getConfigValue()).orElse(DEFAULT_INDEX);
            result.setViewName(REDIRECT + index);
        } else {
            result.setViewName(REDIRECT + "/login");
        }
        return result;
    }

    /**
     * 根据来源系统和用户名直接进入角色访问(需要通过获取access_token访问)
     *
     * @param request
     * @param response
     * @param sourceSystem 来源系统
     * @param username 用户名
     */
    @RequestMapping(value = "/hls/sso/{sourceSystem}/{username}/login",method = {RequestMethod.GET,RequestMethod.POST})
    public void createNewSession4OASystem(HttpServletRequest request, HttpServletResponse response
            , @PathVariable(name = "sourceSystem") String sourceSystem,@PathVariable(name = "username") String username ){
        String hapSessionId = component.handlerLogin(request, response, username);
    }
}
