package hls.core.sys.controllers;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import hls.core.sys.dto.SysRoleModuleExcludeFunc;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;

import hls.core.sys.dto.SysFunction;
import hls.core.sys.service.SysFunctionService;
import hls.core.sys.service.SysModuleService;

@Controller()
public class SysFunctionController extends BaseController {

	@Autowired
	private SysModuleService SmService;
	
	@Autowired
	private SysFunctionService SfService;

	@RequestMapping(value = "/sysFunction/query")
	@ResponseBody
	public ResponseData query(HttpServletRequest request, SysFunction sysFunction,
			@RequestParam(defaultValue = DEFAULT_PAGE) int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {

		IRequest iRequest = createRequestContext(request);
		List<SysFunction> datas = SfService.queryAll(iRequest, sysFunction, page, pagesize);
		return new ResponseData(datas);
	}

	@RequestMapping(value = "/sysFunction/widget/query")
	@ResponseBody
	public ResponseData widgeQuery(HttpServletRequest request,@RequestParam(required=false)Long roleId,@RequestParam(required=false)String moduleCode) {
		IRequest iRequest = createRequestContext(request);
		List<SysFunction> datas = SfService.widgeQuery(iRequest,roleId,moduleCode);
		return new ResponseData(datas);
	}

	@RequestMapping(value = "/sysFunction/submit")
	@ResponseBody
	public ResponseData update(HttpServletRequest request,
			@RequestBody List<SysFunction> list) {
		ResponseData rd = null;
		try {
			IRequest iRequest = createRequestContext(request);
			for(int i=0;i<list.size();i++)
			{
			  Long 	homeResourceId = list.get(i).getHomepageResourceId();
			  Long resourceId = list.get(i).getResourceId();
			if (homeResourceId == null && list.get(i).get__status().equals("update")) {
					list.get(i).setHomepageResourceId(new Long(-99999));
				}else {
					list.get(i).setHomepageResourceId(homeResourceId);
				}
				if (resourceId == null && list.get(i).get__status().equals("update")) {
					list.get(i).setResourceId(new Long(-99999));
				}else {
					list.get(i).setResourceId(resourceId);
				}
			}
			List<SysFunction> lists = SfService.batchUpdate(iRequest, list);
			rd = new ResponseData(lists);
		} catch (Exception e) {
			// TODO: handle exception
			rd = new ResponseData(false);
			rd.setMessage("功能代码不可重复");
		}
		
		return rd;
	}

	@RequestMapping(value = "/sysFunction/remove")
	@ResponseBody
	public ResponseData delete(HttpServletRequest request,
			@RequestBody List<SysFunction> list) {
		SfService.batchDelete(list);
		for(int i= 0;i<list.size();i++)
		{
			SfService.deleteById(list.get(i).getFunctionId());
		}
		return new ResponseData();
	}
	
	@RequestMapping(value="/sys/module/modulecode/query")
	@ResponseBody
	public ResponseData queryByModuleCode(HttpServletRequest request, String moduleCode,String isHomePage){
		List<SysFunction> sf =  SfService.queryByModuleCode(moduleCode,isHomePage);
		return new ResponseData(sf);
	}



	/*
	 *	查询
     *  模块分配-》 功能分配 关联role
	 */
	@RequestMapping(value = "/sysFunction/role/query")
	@ResponseBody
	public ResponseData queryAllFunctionRelateRole(HttpServletRequest request, SysFunction sysFunction,
							  @RequestParam(defaultValue = DEFAULT_PAGE) int page,
							  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pagesize) {

		IRequest iRequest = createRequestContext(request);


		List<SysFunction> datas = SfService.queryAllFunctionRelateRole(iRequest, sysFunction, page, pagesize);
		return new ResponseData(datas);
	}


	/*
	 *  修改
	 *  模块分配-》 功能分配 关联role
	 */
	@RequestMapping(value = "/sysFunction/role/update")
	@ResponseBody
	public ResponseData updateFunctionRelateRole(@RequestBody List<SysFunction> dto, BindingResult result, HttpServletRequest request){
		getValidator().validate(dto, result);
		if (result.hasErrors()) {
			ResponseData responseData = new ResponseData(false);
			responseData.setMessage(getErrorMessage(result, request));
			return responseData;
		}
		IRequest requestCtx = createRequestContext(request);
		return new ResponseData(SfService.updateFunctionRelateRole(requestCtx, dto));
	}
}
