package hls.core.sys.event.service.impl;

import com.hand.hap.core.AppContextInitListener;
import com.hand.hap.core.IRequest;
import hls.core.hls.dto.*;
import hls.core.hls.mapper.HlsUserRelationshipMapper;
import hls.core.hls.service.*;
import hls.core.sys.event.service.SysEventCommonService;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.service.SysNoticePushService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import org.activiti.engine.delegate.DelegateExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * Created by Administrator on 2017/4/19.
 */
@Service
@Transactional
public class SysEventServiceImpl implements AppContextInitListener, SysEventService {

    @Autowired
    private HlsSystemEventService hlsSystemEventService;
    @Autowired
    private IHlsDocAuthorityService docAuthorityService;
    @Autowired
    private IHlsUserRelationshipService hlsUserRelationshipService;
    @Autowired
    private HlsSystemNoticeService hlsSystemNoticeService;
    @Autowired
    private SysNoticePushService sysNoticePushService;
    @Autowired
    private HlsUserRelationshipMapper userRelationshipMapper;

    private Map<String, SysEventCommonService> commonService = new HashMap<String, SysEventCommonService>();

    @Override
    public void contextInitialized(ApplicationContext applicationContext) {
        Map<String, SysEventCommonService> map = applicationContext.getBeansOfType(SysEventCommonService.class);

        map.forEach((k, v) -> {
            commonService.put(v.getEventCode(), v);
        });
    }

    @Override
    public void createEvent(IRequest iRequest, Long documentId, String documentCategory, String documentType, Map params) {
        SysEventCommonService service = commonService.get(params.get("eventCode"));

        if(service != null)
        {
            service.process(iRequest,documentId,documentCategory,documentType,params);
        }else{
            String message = "事件："+params.get("eventCode")+"对应的实体类尚未实现!";
            try {
                throw new Exception(message);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }


    @Override
    public void eventSave(IRequest iRequest, Long documentId, String documentCategory, String documentType, String sourceModule, String eventCode, String RelationType, Map params) {
        String message = params.get("message")!=null?(String)params.get("message"):"";
        String noticeTitle = (String)params.get("noticeTitle");
        String url = (String)params.get("url");
        Long level = Long.parseLong (params.get("level").toString());
        String noticeType = (String)params.get("noticeType");
        String authorityType = params.getOrDefault("authorityType", "2").toString();
        Long userId = iRequest.getUserId();
        if(("").equals(level) || null==level){
            level =2L;
        }
        HlsSystemEvent event = new HlsSystemEvent();
        event.setEventCode(eventCode);
        event.setSourceDocumentId(documentId);
        event.setEventUserId(userId);
        event.setSourceDocumentCategory(documentCategory);
        event.setLastUpdatedBy(userId);
        event.setLastUpdateDate(new Date());

        // 系统通知
        HlsSystemNotice hlsSystemNotice = new HlsSystemNotice();
        hlsSystemNotice.setNotice_message(message);
        hlsSystemNotice.setNotice_title(noticeTitle);
        hlsSystemNotice.setNotice_datetime(new Date());
        hlsSystemNotice.setNotice_type(noticeType);
        hlsSystemNotice.setNotice_url(url);
        hlsSystemNotice.setNotice_level(level);
        hlsSystemNotice.setSource_module(sourceModule);
        hlsSystemNotice.setSource_event(eventCode);
        if(eventCode.equals(SysEventCodeUtil.PROPERTY_WFL_TO_DO)){
            String taskId = (String)params.get("taskId");
            hlsSystemNotice.setSource_document_category(eventCode);
            hlsSystemNotice.setSource_document_id(Long.parseLong(taskId.toString()));
        }else {
            hlsSystemNotice.setSource_document_category(documentCategory);
            hlsSystemNotice.setSource_document_id(documentId);
        }
        hlsSystemNotice.setSource_user_id(iRequest.getUserId());
        hlsSystemNotice.setLastUpdatedBy(userId);
        hlsSystemNotice.setLastUpdateDate(new Date());

        HlsDocAuthority authority = new HlsDocAuthority();
        authority.setUserId(userId);
        authority.setDocumentCategory(documentCategory);// 单据类型
        authority.setDocumentId(documentId);
        authority.setAuthorityType(authorityType);// 1查询，2，修改
        authority.setSourceModule(sourceModule);
        authority.setSourceEvent(eventCode);
        authority.setSourceDocumentCategory(documentCategory);
        authority.setSourceDocumentId(documentId);
        authority.setSourceUserId(iRequest.getUserId());
        authority.setLastUpdatedBy(userId);
        authority.setLastUpdateDate(new Date());

        HlsUserRelationship hlsUserRelationship = new HlsUserRelationship();
        hlsUserRelationship.setUserId(userId);
        hlsUserRelationship.setRelationType(RelationType);
        hlsUserRelationship.setSourceModule(sourceModule);
        hlsUserRelationship.setSourceEvent(eventCode);
        hlsUserRelationship.setSourceDocumentCategory(documentCategory);
        hlsUserRelationship.setSourceDocumentId(documentId);
        hlsUserRelationship.setSourceUserId(iRequest.getUserId());
        hlsUserRelationship.setLastUpdatedBy(userId);
        hlsUserRelationship.setLastUpdateDate(new Date());

        hlsSystemEventService.insertSelective(iRequest, event);
        if(Boolean.valueOf(params.getOrDefault("needAuthority","false").toString())) {
            docAuthorityService.insertSelective(iRequest, authority );
        }
        if("P2P".equalsIgnoreCase(hlsUserRelationship.getRelationType())) {
            hlsUserRelationship.setRelationUserId(iRequest.getUserId()); // 关联用户ID

            HlsUserRelationship userRelationship = new HlsUserRelationship();
            userRelationship.setUserId(hlsUserRelationship.getUserId());
            userRelationship.setRelationType(hlsUserRelationship.getRelationType());
            userRelationship.setRelationUserId(hlsUserRelationship.getRelationUserId());
            List<HlsUserRelationship> list =  userRelationshipMapper.select(userRelationship);
            if(list.size() == 0) {
                hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship );
            }
        }else if("P2D".equalsIgnoreCase(hlsUserRelationship.getRelationType())){
            hlsUserRelationship.setRelationDocumentCategory(documentCategory);// 关联单据类别
            hlsUserRelationship.setRelationDocumentId(documentId); // 关系单据ID

            HlsUserRelationship userRelationship1 = new HlsUserRelationship();
            userRelationship1.setUserId(hlsUserRelationship.getUserId());
            userRelationship1.setRelationType(hlsUserRelationship.getRelationType());
            userRelationship1.setSourceDocumentCategory(hlsUserRelationship.getSourceDocumentCategory());
            userRelationship1.setSourceDocumentId(hlsUserRelationship.getSourceDocumentId());
            List<HlsUserRelationship> lists =  userRelationshipMapper.select(userRelationship1);
            if(lists.size() == 0 && !eventCode.equals("WFL.TO_DO")) {
                hlsUserRelationshipService.insertSelective(iRequest, hlsUserRelationship );
            }
        }
        hlsSystemNotice.setSourceEventId(event.getEventId());
        HlsSystemNotice hsn = hlsSystemNoticeService.insertSelective(iRequest, hlsSystemNotice );
        setNoticeCache(hlsSystemNotice);
    }

    @Override
    public void setNoticeCache(HlsSystemNotice hlsSystemNotice) {
        sysNoticePushService.sendUserNotice(hlsSystemNotice.getSource_user_id(),hlsSystemNotice);
        Long currentUserId = hlsSystemNotice.getSource_user_id();
        List<HlsSystemNotice> noticeToList= hlsSystemNoticeService.selectSendNotice(hlsSystemNotice);
        if(noticeToList!=null && noticeToList.size() > 0){
            for(HlsSystemNotice hlsSystemNotice1:noticeToList){
                if(!currentUserId.equals(hlsSystemNotice1.getUserId())){
                    sysNoticePushService.sendUserNotice(hlsSystemNotice1.getUserId(),hlsSystemNotice);
                }
            }
        }
    }

    @Override
    public void createEvent(IRequest iRequest, Long documentId, String documentCategory, String documentType, Map params, String eventCode) {
        if(params == null){
            params = new HashMap();
        }
        params.put("eventCode",eventCode);
        createEvent(iRequest,documentId,documentCategory,documentType,params);
    }

    @Override
    public void createEvent(IRequest iRequest, Long documentId, String documentCategory, String documentType, Map params, String eventCode, boolean needAuthority) {
        if(params == null){
            params = new HashMap();
        }
        params.put("eventCode",eventCode);
        params.put("needAuthority",needAuthority);
        createEvent(iRequest,documentId,documentCategory,documentType,params);
    }

    @Override
    public void createNotice(DelegateExecution delegateExecution,String status) {
        IRequest iRequest = (IRequest)delegateExecution.getVariable("iRequest");
        String documentCategory = (String)delegateExecution.getVariable("documentCategory");
        Long documentId = (Long)delegateExecution.getVariable("documentId");
        if (StringUtils.isEmpty(documentCategory))
            return;
        SysEventCommonService service = commonService.get(documentCategory);
        if(service != null) {
            service.notice(delegateExecution,status);
        }
    }
}
