package hls.core.sys.service.impl;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.github.pagehelper.PageHelper;
import hls.core.hls.dto.HlsSystemNotice;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.core.IRequest;
import com.hand.hap.security.PasswordManager;
import com.hand.hap.security.service.IAESClientService;
import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.sys.dto.SysUser;
import hls.core.sys.dto.SysUserAllocation;
import hls.core.sys.mapper.SysUserAllocationMapper;
import hls.core.sys.mapper.SysUserMapper;
import hls.core.sys.service.SysUserService;

@Service
@Transactional
public class SysUserServiceImpl extends BaseServiceImpl<SysUser> implements SysUserService {

	@Autowired
	SysUserMapper mapper;
	
	@Autowired
	SysUserAllocationMapper allocationMapper;
	
	@Autowired
	PasswordManager passwordManager ;
	
	@Override
	public List<SysUser> selectEdit(Long id) {
		
		List<SysUser> userList = new ArrayList<SysUser>();
		
		SysUser user = mapper.selectByPrimaryKey(id);
		
		userList.add(user) ;
		
		return userList;
		
	}

	@Override
	protected boolean useSelectiveUpdate() {
		return false;
	}

	@Override
	public List<SysUser> batchUpdate(IRequest arg0, List<SysUser> arg1) {
		// TODO Auto-generated method stub
		for (SysUser sysUser : arg1) {
			sysUser.setPasswordEncrypted(passwordManager.encode(passwordManager.getDefaultPassword()));
		}
		return super.batchUpdate(arg0, arg1);
	}

	public void batchChildUpdate(IRequest iRequest, SysUser sysUser, List<SysUserAllocation> allocations) {
		if(sysUser != null)
		{
			/*if(sysUser.getPasswordLifespanDays() != null)
			{
				sysUser.setPasswordLifespanType("登陆天数");
				
			}else if(sysUser.getPasswordLifespanTimes() != null)
			{
				sysUser.setPasswordLifespanType("登陆次数");
			}else
			{
				sysUser.setPasswordLifespanType("永不过期");
			}*/
			//更新和插入头表
			if(sysUser.getUserId() == null)
			{
				 sysUser.setPasswordEncrypted(passwordManager.encode(passwordManager.getDefaultPassword()));
				
				 self().insertSelective(iRequest,sysUser);
			}else
			{
				self().updateByPrimaryKeySelective(iRequest,sysUser);
			}
			
			//更新和插入行表
			if(sysUser.getSysUserAllocation() != null)
			{
				for(SysUserAllocation allocation :allocations)
				{
					if (allocation!=null) {
						allocation.setUserId(sysUser.getUserId());
						
						if(allocation.getAllocationId() ==null)
						{
							allocationMapper.insertSelective(allocation);
						}else
						{
							allocationMapper.updateByPrimaryKey(allocation);
						}
					}
				}
			}
			
		}		
		
		
		
	}

	@Override
	public void batchDeleteByKey(List<SysUser> list) {
		if(list.size() !=0)
		{
			for(SysUser user:list)
			{
				mapper.deleteByKey(user.getUserId());
				
//				allocationMapper.deleteChild(user.getUserId());
			}
		}
	}

	@Override
	public void deleteChildByUserId(Long userId) {
		// TODO Auto-generated method stub
		mapper.deleteChildByUserId(userId);
	}

	@Override
	public List<SysUser> selectList(SysUser sysUser) {
		return mapper.selectList(sysUser);
	}

	@Override
	public SysUser queryUserByCode(String code) {
		return mapper.queryUserByCode(code);
	}

	@Override
	public List<SysUser> selectUserFriends(SysUser user) {
		return mapper.selectUserFriends(user);
	}

	@Override
	public SysUser selectUserById(Long userId) {
		return mapper.selectUserById(userId);
	}

	@Override
	public List<HlsSystemNotice> selectAllPerson(SysUser user) {
		return mapper.selectAllPerson(user);
	}

	@Override
	public SysUser selectUserPositionById(Long userId) {
		return mapper.selectUserPositionById(userId);
	}

	@Override
	public List<SysUser> userQuery(IRequest iRequest, SysUser sysUser, int page, int pageSize) {
		PageHelper.startPage(page, pageSize);
		return mapper.userQuery(sysUser);
	}
}
