package hls.core.vat.controllers;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hand.hap.attachment.exception.FileReadIOException;
import com.hand.hap.core.IRequest;
import com.hand.hap.core.exception.TokenException;
import com.hand.hap.excel.dto.ExportConfig;
import com.hand.hap.excel.service.IExportService;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.fnd.dto.FndInterfaceHeaders;
import hls.core.fnd.dto.FndInterfaceLines;
import hls.core.vat.dto.HlsCusAcrInvoiceHd;
import hls.core.vat.service.AcrInvoiceHdService;
import hls.core.vat.service.ITaxService;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.*;
import java.net.URLEncoder;
import java.util.*;

import static org.springframework.util.StreamUtils.BUFFER_SIZE;

@Controller
public class AcrInvoiceHdController extends BaseController {
	@Autowired
	private AcrInvoiceHdService service;

	@Autowired
	ObjectMapper objectMapper;

	@Autowired
	IExportService excelService;

	@Autowired
	ITaxService iTaxService;

	@RequestMapping(value = "/vat/acr/invoicehd/query")
	@ResponseBody
	public ResponseData query(final HlsCusAcrInvoiceHd acrInvoiceHd, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
							  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		acrInvoiceHd.setCompanyId(Long.valueOf(String.valueOf(request.getSession().getAttribute("companyId"))));
		return new ResponseData(service.select(requestContext, acrInvoiceHd, page, pagesize));
	}

	@RequestMapping(value = "/vat/acr/invoicehd/sumbit")
	@ResponseBody
	public ResponseData sumbit(@RequestBody final JSONObject params, final BindingResult result,
							   final HttpServletRequest request, final HttpSession session) throws BeyondAmountLimitException{
//		getValidator().validate(acrInvoiceHds, result);
//		if (result.hasErrors()) {
//			ResponseData responseData = new ResponseData(false);
//			responseData.setMessage(getErrorMessage(result, request));
//			return responseData;
//		}
		Long companyId = Long.valueOf(String.valueOf(session.getAttribute("companyId")));
		Long userId = Long.valueOf(String.valueOf(session.getAttribute("userId")));
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.batchSubmit(requestContext, params,companyId,userId));
	}

	/**
	 * 发票头分组显示
	 * @param
	 * @return
	 */
	@RequestMapping(value = "/vat/queryAcrHdFromConCash")
	@ResponseBody
	public ResponseData queryAcrHdFromConCash(@RequestBody HlsCusAcrInvoiceHd acrInvoiceHd, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
											  @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		return new ResponseData(service.queryAcrInfo(acrInvoiceHd,iRequest));
	}
	
	
	@RequestMapping(value = "/vat/acr/invoicehd/detail/query")
	@ResponseBody
	public ResponseData invoicehdDetailQuery(HttpServletRequest request,@RequestBody Map<String,String> map,
			@RequestParam(defaultValue = DEFAULT_PAGE) final int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize){
		IRequest requestContext = createRequestContext(request);
		map.put("companyId",requestContext.getCompanyId().toString());
		return new ResponseData(service.invoicehdDetailQuery(map));
	}

	@RequestMapping(value = "/vat/acr/invoicehd/receipt/query")
	@ResponseBody
	public ResponseData invoicehdReceiptQuery(HttpServletRequest request,@RequestBody Map<String,String> map,
			@RequestParam(defaultValue = DEFAULT_PAGE) final int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize){
		return new ResponseData(service.invoicehdReceiptQuery(map));
	}

	/**
	 * 发票首页待开票清单
	 *已开票比例
	 * @param
	 * @return
	 */
	@RequestMapping(value = "/vat/VAT000H/vat000hQuerybillingAmountRadio")
	@ResponseBody
	public ResponseData vat000hQuerybillingAmountRadio(HlsCusAcrInvoiceHd acrInvoiceHd, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
													   @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		List<HlsCusAcrInvoiceHd> list=new ArrayList<HlsCusAcrInvoiceHd>();
		acrInvoiceHd.setCompanyId(iRequest.getCompanyId());
        list= service.vat000hQuerybillingAmountRadio(acrInvoiceHd);
        return new ResponseData(list);
	}

        @RequestMapping(value = "/vat/attach/file/upload")
        public void detail(HttpServletRequest request, HttpServletResponse response) throws FileReadIOException, TokenException {
            IRequest requestContext = createRequestContext(request);
		try {
            String fileName = request.getSession().getServletContext().getRealPath("/")+ "resources/excel/VAT/进项发票导入.xlsx";
			File file = new File(fileName);
			if (file.exists()) {
				response.addHeader("Content-Disposition","attachment;filename=\"" + URLEncoder.encode("进项发票导入.xlsx", "UTF-8") + "\"");
				response.setContentType("EXCEL;charset=UTF-8");
				response.setHeader("Accept-Ranges", "bytes");
				int fileLength = (int) file.length();
				response.setContentLength(fileLength);
				if (fileLength > 0) {
					writeFileToResp(response, file);
				}
			} else {
				response.getWriter().write("文件不存在！");
			}
		} catch (IOException e) {
			throw new FileReadIOException();
		}
	}

    private void writeFileToResp(HttpServletResponse response, File file) throws FileNotFoundException, IOException {
        byte[] buf = new byte[BUFFER_SIZE];
        try (InputStream inStream = new FileInputStream(file);
             ServletOutputStream outputStream = response.getOutputStream()) {
            int readLength;
            while (((readLength = inStream.read(buf)) != -1)) {
                outputStream.write(buf, 0, readLength);
            }
            outputStream.flush();
        }
    }


	/**
	 * 销项发票综合查询
	 * @param
	 * @return
	 */
	@RequestMapping(value = "/vat/VAT220A/acr220aAcrInvoiceHdQuery")
	@ResponseBody
	public ResponseData acr220aAcrInvoiceHdQuery(@RequestBody HlsCusAcrInvoiceHd acrInvoiceHd, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
												 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		List<HlsCusAcrInvoiceHd> list=new ArrayList<HlsCusAcrInvoiceHd>();
		list= service.acr220aAcrInvoiceHdQuery(acrInvoiceHd,iRequest,acrInvoiceHd.getPage(),acrInvoiceHd.getPagesize());
		return new ResponseData(list);
	}
	@RequestMapping(value = "/vat/acrInvoiceHd/Receipt/selectAllReceipt")
	@ResponseBody
	public ResponseData selectAllReceipt(@RequestBody HlsCusAcrInvoiceHd acrInvoiceHd, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
										 @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		List<HlsCusAcrInvoiceHd> list=service.selectAllReceipt(acrInvoiceHd,iRequest,acrInvoiceHd.getPage(),acrInvoiceHd.getPagesize());
		return new ResponseData(list);
	}

	@RequestMapping(value = "/vat/acrInvoiceDetail")
	@ResponseBody
	public ResponseData acrInvoiceDetail(@RequestBody HlsCusAcrInvoiceHd acrInvoiceHd, final HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		acrInvoiceHd.setCompanyId(iRequest.getCompanyId());
		return new ResponseData(service.acrInvoiceDetail(acrInvoiceHd,iRequest));
	}

	@RequestMapping(value = "/acr/inf/index/txt/import")
	public ResponseData acrIndexExcelImport(HttpServletRequest request) throws IOException {
		/*IRequest iRequest = createRequestContext(request);
	    InputStream in  = file.getInputStream();
		String fileName = file.getOriginalFilename();
		InputStreamReader isr = new InputStreamReader(in);// 字符流
		FndInterfaceHeaders fndInterfaceHeaders = iTaxService.readInvoice("fileName", isr,iRequest);
		List<FndInterfaceHeaders> list = new ArrayList<FndInterfaceHeaders>();
		list.add(fndInterfaceHeaders);
		return new ResponseData(list);*/
		IRequest iRequest = createRequestContext(request);
		Map<String, Object> response = new HashMap<String, Object>();
		response.put("success", false);
		// 在解析请求之前先判断请求类型是否为文件上传类型
		boolean isMultipart = ServletFileUpload.isMultipartContent(request);
		if (!isMultipart) {
			return null;
		}
		// 文件上传处理工厂
		FileItemFactory factory = new DiskFileItemFactory();

		// 创建文件上传处理器
		ServletFileUpload upload = new ServletFileUpload(factory);

		List items = null;
		try {
			items = upload.parseRequest(request);
		} catch (FileUploadException e) {
			e.printStackTrace();
		}


		// 对所有请求信息进行判断
		Iterator iter = items.iterator();
		List<FndInterfaceHeaders> list = new ArrayList<FndInterfaceHeaders>();
		while (iter.hasNext()) {
			FileItem item = (FileItem) iter.next();
			// 信息为文件的格式
			if (!item.isFormField()) {
				try {
					InputStream fs = item.getInputStream();
					String filename = new String(item.getName());
					InputStreamReader isr = new InputStreamReader(fs,"utf-8");// 字符流
					FndInterfaceHeaders fndInterfaceHeaders = iTaxService.readInvoice("fileName", isr,iRequest);
					list.add(fndInterfaceHeaders);
				} catch (Exception e) {
					e.printStackTrace();
				}

			}
		}
		return new ResponseData(list);
	}
	@RequestMapping(value = "/acr/inf/index/txt/import/merge")
	public ResponseData acrIndexTxtMerge(HttpServletRequest request, FndInterfaceLines fndInterfaceLines) {
		IRequest iRequest = createRequestContext(request);
		service.acrIndexTxtMerge(iRequest,fndInterfaceLines);
		List list = new ArrayList();
		list.add("success");
		return new ResponseData(list);
	}


	@RequestMapping(value = "/acr/inf/index/excel/export")
	public void acrIndexExcelExport(HttpServletRequest request,HlsCusAcrInvoiceHd acrInvoiceHd,
									HttpServletResponse httpServletResponse){
		IRequest requestContext = createRequestContext(request);
		String [] arr = acrInvoiceHd.getParamStr().split(",");
		List idList = new ArrayList();
		for(int i = 0; i < arr.length; i++){
			if(!arr[i].equals("")) {
				idList.add(Integer.parseInt(arr[i]));
			}
		}
		iTaxService.exportInvoice(idList,"gold.txt",httpServletResponse);
		/*try {
			JavaType type = objectMapper.getTypeFactory().constructParametrizedType(ExportConfig.class,
					ExportConfig.class, HlsCusAcrInvoiceHd.class, ColumnInfo.class);
			ExportConfig<HlsCusAcrInvoiceHd, ColumnInfo> exportConfig = objectMapper.readValue(config, type);
			excelService.exportAndDownloadExcel("hls.core.vat.mapper.HlsCusAcrInvoiceHdMapper.vatInfIndexInvoiceDetailQuery",
					exportConfig, request, httpServletResponse, requestContext);
		} catch (IOException e) {
			e.printStackTrace();
		}*/

	}

	@RequestMapping(value = "/vat/inf/index/invoice/detail/query")
	@ResponseBody
	public ResponseData vatInfIndexInvoiceDetailQuery(
			@RequestBody(required = false) HlsCusAcrInvoiceHd acrInvoiceHd,
			@RequestParam(defaultValue = DEFAULT_PAGE) final int page,
			@RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pageSize,
			final HttpServletRequest request
	){
		if (acrInvoiceHd==null){
			acrInvoiceHd = new HlsCusAcrInvoiceHd();
		}
		IRequest reuestContext = createRequestContext(request);
		return new ResponseData(service.vatInfIndexInvoiceDetailQuery(reuestContext,acrInvoiceHd,page,pageSize));
	}

	@RequestMapping(value = "/vat/inf/index/invoice/status/query")
	@ResponseBody
	public ResponseData vatInfIndexInvoiceStatusQuery(@RequestBody HlsCusAcrInvoiceHd acrInvoiceHd, HttpServletRequest request){
		return new ResponseData(service.vatInfIndexInvoiceStatusQuery(acrInvoiceHd));
	}

	@RequestMapping(value = "/vat/inf/index/invoice/reversed/query")
	@ResponseBody
	public ResponseData vatInfIndexInvoiceReversedQuery(@RequestBody HlsCusAcrInvoiceHd acrInvoiceHd, HttpServletRequest request){
		return new ResponseData(service.vatInfIndexInvoiceReversedQuery(acrInvoiceHd));
	}

	@RequestMapping(value = "/vat/inf/invoice/by/id/reversed")
	@ResponseBody
	public ResponseData vatInfByIdInvoiceReversed(HlsCusAcrInvoiceHd acrInvoiceHd, HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.vatInfByIdInvoiceReversed(requestContext,acrInvoiceHd));
	}

}
