package hls.core.vat.controllers;

import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;

import hls.core.vat.dto.HlsCusAcrInvoiceLn;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.system.dto.ResponseData;
import hls.core.vat.service.AcrInvoiceLnService;

@Controller
public class AcrInvoiceLnController extends BaseController {
	@Autowired
	private AcrInvoiceLnService service;

	@RequestMapping(value = "/vat/acr/invoiceln/query")
	@ResponseBody
	public ResponseData query(final HlsCusAcrInvoiceLn acrInvoiceLn, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.select(requestContext, acrInvoiceLn, page, pagesize));
	}
	
	@RequestMapping(value = "/vat/acr/invoiceln/sumbit")
	@ResponseBody
	public ResponseData sumbit(@RequestBody final List<HlsCusAcrInvoiceLn> acrInvoiceLns, final BindingResult result,
                               final HttpServletRequest request){
		getValidator().validate(acrInvoiceLns, result);
		if (result.hasErrors()) {
			ResponseData responseData = new ResponseData(false);
			responseData.setMessage(getErrorMessage(result, request));
			return responseData;
		}
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.batchUpdate(requestContext, acrInvoiceLns));
	}

	/**
	 * 回租本金
	 * @param acrInvoiceLn
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/vat/acr/invoiceln/leasePrincipal")
	@ResponseBody
	public ResponseData leasePrincipal(HlsCusAcrInvoiceLn acrInvoiceLn, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                       @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){

		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.leasePrincipal(acrInvoiceLn,requestContext,page,pagesize));
	}

	/**
	 * 收款
	 * @param acrInvoiceLn
	 * @param page
	 * @param pagesize
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/vat/acr/invoiceln/reciveAmount")
	@ResponseBody
	public ResponseData reciveAmount(HlsCusAcrInvoiceLn acrInvoiceLn, @RequestParam(defaultValue = DEFAULT_PAGE) final int page,
                                     @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) final int pagesize, final HttpServletRequest request){
		IRequest requestContext = createRequestContext(request);
		return new ResponseData(service.reciveAmount(acrInvoiceLn,requestContext,page,pagesize));
	}

	@RequestMapping( value = "/vat/acr/stepOne",method = RequestMethod.POST)
	@ResponseBody
	public ResponseData stepOne(@RequestBody List<Map> map,HttpServletRequest request){
		IRequest iRequest = createRequestContext(request);
		long companyId = request.getSession().getAttribute("companyId")!=null? (long) request.getSession().getAttribute("companyId") :0;
		try {
            List<Map> list=service.stepOne(map,iRequest,companyId);
        }catch (Exception e){
            ResponseData erda = new ResponseData(false);
            erda.setMessage(e.getMessage());
            return erda;
        }
        ResponseData erda = new ResponseData(true);
        return erda;
	}

    @RequestMapping( value = "/vat/acr/acrHLRecINS")
	public void acrHLRecINS(@RequestBody List<Map> maps,HttpServletRequest request){
		long companyId = request.getSession().getAttribute("companyId")!=null? (long) request.getSession().getAttribute("companyId") :0;
		for (Map map:maps) {
			map.put("companyId",companyId);
		}
		service.acrHLRecINS(maps);
	}
}
