package hls.core.vat.controllers;

import org.springframework.stereotype.Controller;
import com.hand.hap.system.controllers.BaseController;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.dto.ResponseData;
import hls.core.vat.dto.AcrReceiptHd;
import hls.core.vat.service.IAcrReceiptHdService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;

import org.springframework.validation.BindingResult;

import java.util.List;

@Controller
public class AcrReceiptHdController extends BaseController {

    @Autowired
    private IAcrReceiptHdService service;


    @RequestMapping(value = "/acr/receipt/hd/query")
    @ResponseBody
    public ResponseData query(AcrReceiptHd dto, @RequestParam(defaultValue = DEFAULT_PAGE) int page,
                              @RequestParam(defaultValue = DEFAULT_PAGE_SIZE) int pageSize, HttpServletRequest request) {
        IRequest requestContext = createRequestContext(request);
        return new ResponseData(service.select(requestContext, dto, page, pageSize));
    }

    @RequestMapping(value = "/acr/receipt/hd/submit")
    @ResponseBody
    public ResponseData update(@RequestBody List<AcrReceiptHd> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        return new ResponseData(service.batchUpdate(requestCtx, dto));
    }

    @RequestMapping(value = "/acr/receipt/hd/remove")
    @ResponseBody
    public ResponseData delete(HttpServletRequest request, @RequestBody List<AcrReceiptHd> dto) {
        service.batchDelete(dto);
        return new ResponseData();
    }


    @RequestMapping(value = "/acr/receipt/hd/create")
    @ResponseBody
    public ResponseData receiptCreate(@RequestBody List<AcrReceiptHd> dto, BindingResult result, HttpServletRequest request) {
        getValidator().validate(dto, result);
        if (result.hasErrors()) {
            ResponseData responseData = new ResponseData(false);
            responseData.setMessage(getErrorMessage(result, request));
            return responseData;
        }
        IRequest requestCtx = createRequestContext(request);
        service.receiptCreate(requestCtx, dto);
        return new ResponseData();
    }


//    @RequestMapping(value = "/acr/receipt/hd/create/gathering")
//    @ResponseBody
//    public ResponseData gatheringReceiptCreate(@RequestBody List<AcrReceiptHd> dto, BindingResult result, HttpServletRequest request) {
//        getValidator().validate(dto, result);
//        if (result.hasErrors()) {
//            ResponseData responseData = new ResponseData(false);
//            responseData.setMessage(getErrorMessage(result, request));
//            return responseData;
//        }
//        IRequest requestCtx = createRequestContext(request);
//        service.gatheringReceiptCreate(requestCtx, dto);
//        return new ResponseData();
//    }
}