package hls.core.vat.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import com.hand.hap.excel.ExcelException;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.service.IConContractCashflowService;
import hls.core.fnd.dto.FndInterfaceLines;
import hls.core.fnd.mapper.FndInterfaceLinesMapper;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.hls.service.HlsBeanRefUtilService;
import hls.core.vat.dto.HlsCusAcpInvoiceHd;
import hls.core.vat.dto.HlsCusAcpInvoiceLn;
import hls.core.vat.mapper.HlsCusAcpInvoiceLnMapper;
import hls.core.vat.service.AcpInvoiceHdService;
import hls.core.vat.service.AcpInvoiceLnService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class AcpInvoiceLnServiceImpl extends BaseServiceImpl<HlsCusAcpInvoiceLn> implements AcpInvoiceLnService {
    @Autowired
    HlsCusAcpInvoiceLnMapper mapper;
    @Autowired
    FndInterfaceLinesMapper fndInterfaceLinesMapper;
    @Autowired
    HlsBeanRefUtilService hlsBeanRefUtilService;
    @Autowired
    AcpInvoiceHdService acpInvoiceHdService;
    /*@Autowired
    AcpInvoiceLnService acpInvoiceLnService;*/
    @Autowired
    FndCodingRuleValuesService fndCodingRuleValuesService;
    @Autowired
    private IConContractCashflowService conContractCashflowService;



    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Override
    public List<HlsCusAcpInvoiceLn> acpInvoiceLnQuery(HlsCusAcpInvoiceLn acpInvoiceLn) {

        return mapper.acpInvoiceLnQuery(acpInvoiceLn);
    }

    @Override
    public void acpInvoiceExcelImport(IRequest iRequest, Long hdId) throws ExcelException, SQLException {
        insertTable(iRequest, hdId);
    }

    @Override
    public List<Map> selectAcpInvoiceDetail(Map map, IRequest iRequest, int page, int pagesize) {
        PageHelper.startPage(page,pagesize);
        return mapper.selectAcpInvoiceDetail(map);
    }

    /**
     *判断String里是数字还是字符
     */
//    public static boolean isNumber(String str) {
//        for (int i = 0; i < str.length(); i++) {
//            System.out.println(str.charAt(i));
//            if (!Character.isDigit(str.charAt(i))) {
//                return false;
//            }
//        }
//        return true;
//    }


    public static boolean isNumber(String str)
    {
        java.util.regex.Pattern pattern=java.util.regex.Pattern.compile("^(([1-9]{1}\\d*)|([0]{1}))(\\.(\\d){0,10})?$"); // 判断小数点后2位的数字的正则表达式
        java.util.regex.Matcher match=pattern.matcher(str);
        if(match.matches()==false)
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    private void insertTable(IRequest iRequest, Long hdId) throws SQLException, ExcelException  {
        //获取接口表数据
        FndInterfaceLines acpInvoiceLn = new FndInterfaceLines();
        FndInterfaceLines acpInvoiceHd = new FndInterfaceLines();
        acpInvoiceHd.setHeaderId(hdId);
        List<FndInterfaceLines> acpInvoiceHdList = fndInterfaceLinesMapper.acpInvoiceHdQuery(acpInvoiceHd);
        Map<String, String> params = new HashMap<String, String>();


        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        //插入头表
        for (int i = 0; i < acpInvoiceHdList.size(); i++) {
            HlsCusAcpInvoiceHd invoiceHd = new HlsCusAcpInvoiceHd();
            String attributes3 = acpInvoiceHdList.get(i).getAttributes_3();
            Date invoiceDate = null;
            try {
                invoiceDate = df.parse(attributes3);
            } catch (ParseException e) {
                e.printStackTrace();
            }
            acpInvoiceHdList.get(i).setInvoiceDate(invoiceDate);

                if(acpInvoiceHdList.get(i).getInvoiceKind() == null){
                    throw new ExcelException((String) null, "开票种类不能为空！", (Object[]) null);
                } else if(acpInvoiceHdList.get(i).getBpId() == null){
                    throw new ExcelException((String) null, "对方商业伙伴不存在！", (Object[]) null);
                }else if(acpInvoiceHdList.get(i).getBpName() == null){
                    throw new ExcelException((String) null, "商业伙伴名称为空！", (Object[]) null);
                } else if(acpInvoiceHdList.get(i).getBpTaxRegistryNum() == null){
                    throw new ExcelException((String) null,  acpInvoiceHdList.get(i).getBpName() + "的纳税人登记号不能为空！", (Object[]) null);
                }else if(acpInvoiceHdList.get(i).getBpAddressPhoneNum() == null){
                    throw new ExcelException((String) null,  acpInvoiceHdList.get(i).getBpName() + "的地址及电话不能为空！", (Object[]) null);
                }else if(acpInvoiceHdList.get(i).getBpBankAccount() == null){
                    throw new ExcelException((String) null,  acpInvoiceHdList.get(i).getBpName() + "的开户行及账号不能为空！", (Object[]) null);
                }

                invoiceHd.setCompanyId(iRequest.getCompanyId());
                invoiceHd.setDocumentNumber(fndCodingRuleValuesService.getCodeRuleValue(iRequest, "AP_INVOICE","STD","ACP", params));
                invoiceHd.setDocumentType("STD");
                invoiceHd.setDocumentCategory("AP_INVOICE");
                invoiceHd.setBusinessType("ACP");
                invoiceHd.setBillingMethod("Excel");
                invoiceHd.setInvoiceKind(acpInvoiceHdList.get(i).getInvoiceKind());
                invoiceHd.setBpId(acpInvoiceHdList.get(i).getBpId());
                invoiceHd.setBpName(acpInvoiceHdList.get(i).getBpName());
                invoiceHd.setBpTaxRegistryNum(acpInvoiceHdList.get(i).getBpTaxRegistryNum());
                invoiceHd.setBpAddressPhoneNum(acpInvoiceHdList.get(i).getBpAddressPhoneNum());
                invoiceHd.setBpBankAccount(acpInvoiceHdList.get(i).getBpBankAccount());
                invoiceHd.setDescription("");


                if(isNumber(acpInvoiceHdList.get(i).getTotalAmountStr())){
                    invoiceHd.setTotalAmount(Double.parseDouble(acpInvoiceHdList.get(i).getTotalAmountStr()));
                }
                if(isNumber(acpInvoiceHdList.get(i).getTaxAmountStr())){
                    invoiceHd.setTaxAmount(Double.parseDouble(acpInvoiceHdList.get(i).getTaxAmountStr()));
                }

                if(isNumber(acpInvoiceHdList.get(i).getTaxAmountStr()) && isNumber(acpInvoiceHdList.get(i).getTotalAmountStr())){
                    invoiceHd.setNetAmount(sub(invoiceHd.getTotalAmount(),invoiceHd.getTaxAmount()));

                }
//                if(isNumber(acpInvoiceHdList.get(i).getNetAmountBjs())){
//                    invoiceHd.setNetAmount(sub(Double.parseDouble(acpInvoiceHdList.get(i).getTotalAmountStr()),Double.parseDouble(acpInvoiceHdList.get(i).getNetAmountJs())));
//                }

                //invoiceHd.setCurrency(acpInvoiceHdList.get(i).getCurrency());
                invoiceHd.setCurrency("CNY");
                invoiceHd.setInvoiceDate(acpInvoiceHdList.get(i).getInvoiceDate());
                invoiceHd.setInvoiceNumber(acpInvoiceHdList.get(i).getInvoiceNumber());
                invoiceHd.setInvoiceStatus("APPROVED");
                invoiceHd.setReversedFlag("N");



            invoiceHd = acpInvoiceHdService.insertSelective(iRequest, invoiceHd);

            acpInvoiceLn.setHeaderId(hdId);
            acpInvoiceLn.setAttributes_5(invoiceHd.getInvoiceNumber());
            List<FndInterfaceLines> acpInvoiceLnList = fndInterfaceLinesMapper.acpInvoiceLnQuery(acpInvoiceLn);

            HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
            //插入行表
            for (int j = 0; j < acpInvoiceLnList.size(); j++) {

                HlsCusAcpInvoiceLn invoiceLN = new HlsCusAcpInvoiceLn();

                invoiceLN.setInvoiceHdId(invoiceHd.getInvoiceHdId());
                invoiceLN.setLineType("SERVICE");
                invoiceLN.setLineNumber(j + 1L);
                invoiceLN.setSourceDocumentCategory("CON_CONTRACT");
                invoiceLN.setContractId(acpInvoiceLnList.get(j).getContractId());
                invoiceLN.setCashflowId(acpInvoiceLnList.get(j).getCashflowId());
                invoiceLN.setCfItem(acpInvoiceLnList.get(j).getCfItem());
                invoiceLN.setCfType(acpInvoiceLnList.get(j).getCfType());
                invoiceLN.setTimes(acpInvoiceLnList.get(j).getTimes());

                if(isNumber(acpInvoiceLnList.get(j).getTimesStr().trim())){
                    invoiceLN.setTimes(Long.parseLong(acpInvoiceLnList.get(j).getTimesStr().trim()));
                }

                if(isNumber(acpInvoiceLnList.get(j).getTaxTypeRateStr().trim())){
                    invoiceLN.setTaxTypeRate(Double.parseDouble(acpInvoiceLnList.get(j).getTaxTypeRateStr().trim()));
                }
                if(isNumber(acpInvoiceLnList.get(j).getTaxAmountStr().trim())){
                    invoiceLN.setTaxAmount(Double.parseDouble(acpInvoiceLnList.get(j).getTaxAmountStr().trim()));
                }
                if(isNumber(acpInvoiceLnList.get(j).getNetAmountBjs().trim())){
                    Double bjs = Double.parseDouble(acpInvoiceLnList.get(j).getNetAmountBjs().trim());
                    Double js = Double.parseDouble(acpInvoiceLnList.get(j).getNetAmountJs().trim());
                    invoiceLN.setNetAmount(sub(bjs,js));
                }
                if(isNumber(acpInvoiceLnList.get(j).getNetAmountBjs().trim())){
                    invoiceLN.setTotalAmount(Double.parseDouble(acpInvoiceLnList.get(j).getNetAmountBjs().trim()));
                }

                invoiceLN.setDescription(acpInvoiceLnList.get(j).getDescription());

                //acpInvoiceLnService.insertSelective(iRequest, invoiceLN);
                self().insertSelective(iRequest, invoiceLN);

                if(acpInvoiceLnList.get(j).getCashflowId() != null){
                    cashflow.setCashflow_id(acpInvoiceLnList.get(j).getCashflowId());
                    cashflow = conContractCashflowService.selectByPrimaryKey(iRequest,cashflow);
                    double billAmount = add(cashflow.getBilling_amount()!=null?cashflow.getBilling_amount():0d,invoiceLN.getTotalAmount());//现金流上更新后的开票金额

                    if (compareTo(billAmount, 0d) == 0) {
                        cashflow.setBilling_status("NOT");

                    } else if (compareTo(billAmount, cashflow.getDue_amount()) < 0) {
                        cashflow.setBilling_status("PARTIAL");

                    } else if (compareTo(billAmount, cashflow.getDue_amount()) == 0) {
                        cashflow.setBilling_status("FULL");

                    } else {
                        throw new IllegalArgumentException("开票金额有误");
                    }

                    cashflow.setBilling_amount(billAmount);
                    conContractCashflowService.self().updateByPrimaryKeySelective(iRequest,cashflow);
                }
            }

        }


    }

    private double add(Double a, Double b) {
        BigDecimal aa = new BigDecimal(a.toString());
        BigDecimal bb = new BigDecimal(b.toString());

        return aa.add(bb).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    private int compareTo(Double a, Double b) {
        BigDecimal aa = new BigDecimal(a.toString());
        BigDecimal bb = new BigDecimal(b.toString());

        return aa.compareTo(bb);
    }

    public Double multiply(Double a, Double b) {
        BigDecimal aa = new BigDecimal(a.toString());
        BigDecimal bb = new BigDecimal(b.toString());

        return aa.multiply(bb).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    public static double sub(double v1, double v2) {

        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));

        return b1.subtract(b2).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }
}
