package hls.core.vat.service.impl;

import java.math.BigDecimal;
import java.util.*;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.hand.hap.lock.components.DatabaseLockProvider;
import hls.core.cont.dto.HlsCusConContract;
import hls.core.cont.mapper.HlsCusConContractMapper;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.mapper.HlsCusConContractCashflowMapper;
import hls.core.fnd.dto.FndInterfaceLines;
import hls.core.fnd.service.FndInterfaceLinesService;
import hls.core.gld.components.AbstractJeTrxService;
import hls.core.gld.components.JeTrxCommonService;
import hls.core.gld.service.IJeTrxDtlService;
import hls.core.hls.bp.dto.HlsCusBpMaster;
import hls.core.hls.bp.mapper.HlsCusBpMasterMapper;
import hls.core.sys.event.service.SysEventService;
import hls.core.sys.event.utils.SysEventCodeUtil;
import hls.core.vat.dto.AcrReceiptHd;
import hls.core.vat.dto.HlsCusAcrInvoiceRelationship;
import hls.core.vat.dto.HlsCusAcrInvoiceHd;
import hls.core.vat.mapper.HlsCusAcrInvoiceLnMapper;
import hls.core.vat.service.AcrInvoiceRelationshipService;
import hls.core.vat.mapper.HlsCusAcrInvoiceRelationshipMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.hand.hap.core.AppContextInitListener;
import com.hand.hap.core.IRequest;
import com.hand.hap.system.service.impl.BaseServiceImpl;
import hls.core.csh.exception.BeyondAmountLimitException;
import hls.core.csh.service.IConContractCashflowService;
import hls.core.fnd.service.FndCodingRuleValuesService;
import hls.core.vat.dto.HlsCusAcrInvoiceLn;
import hls.core.vat.mapper.HlsCusAcrInvoiceHdMapper;
import hls.core.vat.service.AcrInvoiceCommon;
import hls.core.vat.service.AcrInvoiceHdService;
import hls.core.vat.service.AcrInvoiceLnService;

import static hls.core.hls.components.CalculateUtil.div;

@Service
@Transactional(rollbackFor = {Exception.class, RuntimeException.class})
public class AcrInvoiceHdServiceImpl extends BaseServiceImpl<HlsCusAcrInvoiceHd> implements AcrInvoiceHdService, AppContextInitListener {
    @Autowired
    HlsCusAcrInvoiceHdMapper mapper;

    @Autowired
    private HlsCusAcrInvoiceRelationshipMapper relationshipMapper;

    @Autowired
    HlsCusConContractCashflowMapper conContractCashflowMapper;

    @Autowired
    HlsCusConContractMapper conContractMapper;
    @Autowired
    IConContractCashflowService cashflowHService;
    @Autowired
    FndCodingRuleValuesService codingRuleValuesService;

    @Autowired
    HlsCusConContractCashflowMapper cashflowMapper;

    @Autowired
    HlsCusAcrInvoiceRelationshipMapper acrInvoiceRelationshipMapper;

    @Autowired
    IConContractCashflowService iConContractCashflowService;

    @Autowired
    AcrInvoiceRelationshipService acrInvoiceRelationshipService;

    @Autowired
    FndInterfaceLinesService fndInterfaceLinesService;

    @Autowired
    AcrInvoiceLnService lnService;

    @Autowired
    private SysEventService sysEventService;

    @Autowired
    private IJeTrxDtlService jeTrxDtlService;

    @Autowired
    private DatabaseLockProvider databaseLockProvider;

    @Autowired
    private HlsCusAcrInvoiceLnMapper acrInvoiceLnMapper;

    @Autowired
    private JeTrxCommonService jeTrxCommonService;


    @Autowired
    private HlsCusBpMasterMapper hlsCusBpMasterMapper;

    private Map<String, AcrInvoiceCommon> invoiceRegistion = new HashMap<String, AcrInvoiceCommon>();

    @Override
    public List<HlsCusAcrInvoiceHd> invoicehdDetailQuery(Map<String, String> map) {

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("count", map.get("count"));
        params.put("invoiceKind", map.get("invoiceKind"));
        params.put("companyId",map.get("companyId"));
        String[] rate = null;

        if (map.get("invoiceTypesRate") != "") {
            rate = map.get("invoiceTypesRate").split("、");
        }
        params.put("invoiceTypesRate", rate);
        return mapper.invoicehdDetailQuery(params);
    }

    @Override
    public List<AcrReceiptHd> invoicehdReceiptQuery(Map<String, String> map) {
        Map<String, Object> params = new HashMap<String, Object>();

        params.put("count", map.get("count"));
        params.put("contractInfo", map.get("contractInfo"));
        params.put("totalAmountFrom", map.get("totalAmountFrom"));
        params.put("totalAmountTo", map.get("totalAmountTo"));
        String[] cfItems = null;
        if (map.get("cfItems") != null) {
            cfItems = map.get("cfItems").split("、");
        }
        params.put("cfItems", cfItems);
        params.put("cfItem", map.get("cfItem"));
        return mapper.invoicehdReceiptQuery(params);
    }

    @Override
    public List<HlsCusAcrInvoiceHd> vat000hQuerybillingAmountRadio(HlsCusAcrInvoiceHd acrInvoiceHd) {
        List<HlsCusAcrInvoiceHd> maps = mapper.vat000hQuerybillingAmountRadio(acrInvoiceHd);
        return maps;
    }


//    @Override
//    public List<HlsCusAcrInvoiceHd> queryAcr(HlsCusAcrInvoiceHd acrInvoiceHd, IRequest iRequest, int page, int pagesize) {
//        AcrInvoiceCommon acrInvoice = invoiceRegistion.get(acrInvoiceHd.getGroupByFlag());
//        acrInvoiceHd.setCompanyId(iRequest.getCompanyId());
//        List<Long> listTemp = new ArrayList<Long>();
//        if (acrInvoiceHd.getCashflowIdStr() != null) {
//            String[] docTypeArray = acrInvoiceHd.getCashflowIdStr().split(",");
//            for (int i = 0; i < docTypeArray.length; i++) {
//                listTemp.add(Long.parseLong(docTypeArray[i]));
//            }
//            acrInvoiceHd.setQueryList(listTemp);
//        }
//        List<HlsCusAcrInvoiceHd> list = acrInvoice.query(iRequest, acrInvoiceHd, page, pagesize);
//        return list;
//    }


    @Override
    public List<HlsCusAcrInvoiceHd> queryAcrInfo(HlsCusAcrInvoiceHd acrInvoiceHd, IRequest iRequest) {
        //1.查询出来所有可以开票的现金流以及该现金流所发生的核销记录
        acrInvoiceHd.setQueryList(stringConvertLongList(acrInvoiceHd.getCashflowIdStr(), "cashflowId"));
        acrInvoiceHd.setCfItemList(stringConvertLongList(acrInvoiceHd.getCashflowIdStr(), "cfItem"));
        List<HlsCusAcrInvoiceHd> ccrInvoiceHdList = principalAndInterestSplit(mapper.selectCanInvoiceCashFlow(acrInvoiceHd));
        return ccrInvoiceHdList;
    }


    public List<Long> stringConvertLongList(String str, String type) {
        List<Long> list = new ArrayList<Long>();
        String[] tempArr = str.split(",");
        for (int i = 0; i < tempArr.length; i++) {
            String[] tempArr2 = tempArr[i].split("-");
            if (type.equals("cashflowId")) {
                list.add(Long.parseLong(tempArr2[0]));
            } else if (type.equals("cfItem")) {
                list.add(Long.parseLong(tempArr2[1]));
            }
        }
        return list;
    }


    @Override
    public void contextInitialized(ApplicationContext applicationContext) {
        Map<String, AcrInvoiceCommon> map = applicationContext.getBeansOfType(AcrInvoiceCommon.class);

        map.forEach((k, v) -> {
            invoiceRegistion.put(v.getGroupByFlag(), v);
        });

    }

    @Override
    public List<HlsCusAcrInvoiceHd> batchSubmit(IRequest requestContext, JSONObject params, Long companyId, Long userId) throws BeyondAmountLimitException {

        //1.校验 原始List
        List<HlsCusAcrInvoiceHd> originalAcrInvoiceHds = JSON.parseArray(JSON.toJSONString(params.get("originalList")), HlsCusAcrInvoiceHd.class);
        validate(originalAcrInvoiceHds);


        //2.新增记录到发票头行表
        //用一个变量保存插入到acrInvoiceLn表中的记录
        List<Map> list = new ArrayList<Map>();
        List<HlsCusAcrInvoiceHd> acrInvoiceHds = JSON.parseArray(JSON.toJSONString(params.get("groupingsList")), HlsCusAcrInvoiceHd.class);

        List<HlsCusAcrInvoiceHd> invoiceHds = getAcrInvoiceHds(requestContext, acrInvoiceHds, companyId, userId);
        if (acrInvoiceHds != null) {
            for (HlsCusAcrInvoiceHd acrInvoiceHd : invoiceHds) {
                self().insertSelective(requestContext, acrInvoiceHd);
                for (HlsCusAcrInvoiceLn acrInvoiceLn : acrInvoiceHd.getAcrInvoiceLnList()) {
                    Map map = new HashMap();
                    map.put("cashflowId",acrInvoiceLn.getCashflowId());
                    map.put("cfItem",acrInvoiceLn.getCfItem());
                    HlsCusConContractCashflow cash = new HlsCusConContractCashflow();
                    cash.setCashflow_id(acrInvoiceLn.getCashflowId());
                    // lock con_contract_cashflow
                    databaseLockProvider.lock(cash);

                    if(acrInvoiceLn.getQuantity() == null){
                        acrInvoiceLn.setQuantity(1L);
                    }else{
                        acrInvoiceLn.setQuantity(acrInvoiceLn.getQuantity());
                    }
                    acrInvoiceLn.setProductName(acrInvoiceLn.getTaxableServiceName());
                    acrInvoiceLn.setPrice(acrInvoiceLn.getTotalAmount()/acrInvoiceLn.getQuantity());
                    acrInvoiceLn.setNetPrice(acrInvoiceLn.getNetAmount()/acrInvoiceLn.getQuantity());
                    acrInvoiceLn.setTaxIncludedFlag("N");
                    acrInvoiceLn.setDescription("N");
                    acrInvoiceLn.setInvoiceHdId(acrInvoiceHd.getInvoiceHdId());
                    lnService.insertSelective(requestContext, acrInvoiceLn);
                    map.put("acrInvoiceLnId",acrInvoiceLn.getInvoiceLnId());
                    map.put("lnBoxNumber",acrInvoiceLn.getLnBoxNumber());
                    list.add(map);
                }
            }
        }

        //3.新增记录到发票relationship表
        //  更新cashflow
        //  更新合同
        HlsCusConContractCashflow cashflow = null;
        if (originalAcrInvoiceHds != null) {
            for( HlsCusAcrInvoiceHd acrInvoiceHdTmp :originalAcrInvoiceHds){
                HlsCusAcrInvoiceRelationship acrInvoiceRelationship = new HlsCusAcrInvoiceRelationship();
                //获得acrInvoiceLnId
                Long acrInvoiceLnId = getAcrInvoiceLnIdByLnNumber(list,acrInvoiceHdTmp.getLnBoxNumber(),acrInvoiceHdTmp.getCashflowId(),acrInvoiceHdTmp.getCfItem());
                acrInvoiceRelationship.setInvoiceLnId(acrInvoiceLnId);
                acrInvoiceRelationship.setSourceDocumentType("CON_CONTRACT_CASHFLOW");
                acrInvoiceRelationship.setSourceDocumentId(acrInvoiceHdTmp.getCashflowId());
                acrInvoiceRelationship.setBillingAmount(acrInvoiceHdTmp.getTotalAmount());
                acrInvoiceRelationship.setTaxTypeRate(acrInvoiceHdTmp.getTaxTypeRate());
                acrInvoiceRelationship.setTaxAmount(acrInvoiceHdTmp.getTaxAmount());
                if(new BigDecimal(acrInvoiceHdTmp.getTaxAmount()).compareTo(new BigDecimal(0)) != 0 ){
                    acrInvoiceRelationship.setTaxIncludedFlag("Y");
                }else{
                    acrInvoiceRelationship.setTaxIncludedFlag("N");
                }
                acrInvoiceRelationship.setNetAmount(acrInvoiceHdTmp.getNetAmount());
                acrInvoiceRelationship.setInvoiceKind(acrInvoiceHdTmp.getInvoiceKind());
                acrInvoiceRelationship.setBpId(acrInvoiceHdTmp.getBpId());
//                acrInvoiceRelationshipMapper.insertSelective(acrInvoiceRelationship);
                acrInvoiceRelationshipService.insert(requestContext,acrInvoiceRelationship);

                //插入租赁合同收入事务表
                Map param = new HashMap<>();
                AbstractJeTrxService acrInvoiceConfirmJeTrxService = jeTrxCommonService.map.get("ACR_INVOICE_CONFIRM");
                param.put("jeTrxId",acrInvoiceRelationship.getInvoiceRelationId());
                param.put("companyId",companyId);
                param.put("sourceDoc","CON_CONTRACT");
                param.put("contractId",acrInvoiceHdTmp.getContractId());
                acrInvoiceConfirmJeTrxService.process(requestContext,param);

                double totalAmount = acrInvoiceHdTmp.getTotalAmount();


                HlsCusAcrInvoiceLn acrInvoiceLn = new HlsCusAcrInvoiceLn();
                acrInvoiceLn.setInvoiceLnId(acrInvoiceRelationship.getInvoiceLnId());
                acrInvoiceLn = acrInvoiceLnMapper.selectByPrimaryKey(acrInvoiceLn);

                HlsCusAcrInvoiceHd acrInvoiceHd = new HlsCusAcrInvoiceHd();
                acrInvoiceHd.setInvoiceHdId(acrInvoiceLn.getInvoiceHdId());
                acrInvoiceHd = mapper.selectByPrimaryKey(acrInvoiceHd);

                String documentNumber = acrInvoiceHd.getDocumentNumber();
                HashMap<String, Object> map = new HashMap<>();
                map.put("amt",totalAmount);
                map.put("invoiceNum",documentNumber);
                map.put("eventCode", SysEventCodeUtil.PROPERTY_ACR_INVOICE_HD_CREATE);
                sysEventService.createEvent(requestContext,acrInvoiceHdTmp.getContractId(),"CON_CONTRACT",acrInvoiceHd.getDocumentType(),map);

                cashflow = getCashflow(acrInvoiceRelationship,acrInvoiceHdTmp.getCfItem());
                cashflowHService.updateByPrimaryKeySelective(requestContext, cashflow);
                updateContractBillingStatus(cashflow);
            }
        }




//        List<HlsCusAcrInvoiceHd> groupingsList= (List<HlsCusAcrInvoiceHd>)params.get("groupingsList");

//        validate()
//        validate(acrInvoiceHds);
//        List<HlsCusAcrInvoiceHd> invoiceHds = getAcrInvoiceHds(requestContext, acrInvoiceHds, companyId, userId);
//        List<HlsCusAcrInvoiceHd> hdList = new ArrayList<HlsCusAcrInvoiceHd>();
//        List<HlsCusAcrInvoiceLn> lnList = new ArrayList<HlsCusAcrInvoiceLn>();
//
//        HlsCusConContractCashflow cashflow = null;
//
//        if (invoiceHds != null) {
//            for (HlsCusAcrInvoiceHd acrInvoiceHd : invoiceHds) {
//                self().insertSelective(requestContext, acrInvoiceHd);
//
//                for (HlsCusAcrInvoiceLn acrInvoiceLn : acrInvoiceHd.getAcrInvoiceLnList()) {
//                    acrInvoiceLn.setInvoiceHdId(acrInvoiceHd.getInvoiceHdId());
//                    lnService.insertSelective(requestContext, acrInvoiceLn);
//
//                    HlsCusAcrInvoiceRelationship acrInvoiceRelationship = new HlsCusAcrInvoiceRelationship();
//                    acrInvoiceRelationship.setInvoiceLnId(acrInvoiceLn.getInvoiceLnId());
//                    acrInvoiceRelationship.setSourceDocumentType("CON_CONTRACT_CASHFLOW");
//                    acrInvoiceRelationship.setSourceDocumentId(acrInvoiceLn.getCashflowId());
//                    acrInvoiceRelationship.setBillingAmount(acrInvoiceLn.getTotalAmount());
//                    acrInvoiceRelationship.setTaxTypeRate(acrInvoiceLn.getTaxTypeRate());
//                    acrInvoiceRelationship.setTaxAmount(acrInvoiceLn.getTaxAmount());
//                    if(acrInvoiceLn.getTaxAmount().compareTo(0d) > 0 ){
//                        acrInvoiceRelationship.setTaxIncludedFlag("Y");
//                    }else{
//                        acrInvoiceRelationship.setTaxIncludedFlag("N");
//                    }
//                    acrInvoiceRelationship.setNetAmount(acrInvoiceLn.getNetAmount());
//                    acrInvoiceRelationship.setInvoiceKind(acrInvoiceLn.getInvoiceKind());
//                    acrInvoiceRelationship.setBpId(acrInvoiceLn.getBpId());
//                    acrInvoiceRelationshipMapper.insertSelective(acrInvoiceRelationship);
//                    lnList.add(acrInvoiceLn);
//                    cashflow = getCashflow(acrInvoiceLn);
//                    cashflowHService.updateByPrimaryKeySelective(requestContext, cashflow);
//                }
//                acrInvoiceHd.setAcrInvoiceLnList(lnList);
//                hdList.add(acrInvoiceHd);
//                updateContractBillingStatus(cashflow);
//            }
//        }
        return acrInvoiceHds;
    }


    private Long getAcrInvoiceLnIdByLnNumber(List<Map> list,String lnBoxNumber , Long cashflowId , Long cfItem){
        Long acrInvoiceLnResult  = 0L;
        if(list != null ){
           for(Map map : list) {
               if("".equals(map.get("lnBoxNumber")) || "".equals(lnBoxNumber) || lnBoxNumber == null ){
                   if(new BigDecimal(cashflowId).compareTo(new BigDecimal(map.get("cashflowId").toString())) == 0 &&
                           new BigDecimal(cfItem).compareTo(new BigDecimal(map.get("cfItem").toString())) == 0
                           ){
                       acrInvoiceLnResult = Long.valueOf(map.get("acrInvoiceLnId").toString());
                   }
               }
               else {
                   if (lnBoxNumber.equals(map.get("lnBoxNumber").toString())) {
                       acrInvoiceLnResult = Long.valueOf(map.get("acrInvoiceLnId").toString());
                   }
               }
           }
       }

       return acrInvoiceLnResult;
    }

    private HlsCusConContractCashflow getCashflow(HlsCusAcrInvoiceRelationship acrInvoiceRelationship, Long cfItem) {

//        //拿到发票行对应的cf
        HlsCusConContractCashflow cash = conContractCashflowMapper.selectByPrimaryKey(acrInvoiceRelationship.getSourceDocumentId());
//
//
        //更新cf信息
        cash.setBilling_amount(add(cash.getBilling_amount(),acrInvoiceRelationship.getBillingAmount()));
        if(cfItem.compareTo(100L) == 0){
            cash.setBilling_principal(add(cash.getBilling_principal(),acrInvoiceRelationship.getBillingAmount()));
        }
        if(cfItem.compareTo(101L) == 0){
            cash.setBilling_interest(add(cash.getBilling_interest(),acrInvoiceRelationship.getBillingAmount()));
        }
        //更新现金流开票状态
        if(sub(cash.getDue_amount(),cash.getBilling_amount()) == 0){
            cash.setBilling_status("FULL");
        }else{
            cash.setBilling_status("PARTIAL");
        }

//
//        Double amount = add(acrInvoiceLn.getReceivedInterest(), multiply(acrInvoiceLn.getReceivedInterest(), acrInvoiceLn.getTaxTypeRate()));
//        if (compareTo(acrInvoiceLn.getNetAmount(), 0d) == 0) {
//            cash.setBilling_status("NOT");
//            cash.setBilling_principal(0d);
//            cash.setBilling_interest(0d);
//        } else if (compareTo(acrInvoiceLn.getNetAmount(), acrInvoiceLn.getDueAmount()) < 0) {
//            cash.setBilling_status("PARTIAL");
//            if (compareTo(acrInvoiceLn.getNetAmount(), amount) <= 0) {
//                cash.setBilling_principal(0d);
//                cash.setBilling_interest(amount);
//            } else {
//                cash.setBilling_principal(sub(acrInvoiceLn.getNetAmount(), amount));
//                cash.setBilling_interest(amount);
//            }
//        } else if (compareTo(acrInvoiceLn.getNetAmount(), acrInvoiceLn.getDueAmount()) == 0) {
//            cash.setBilling_status("FULL");
//            cash.setBilling_principal(sub(acrInvoiceLn.getNetAmount(), amount));
//            cash.setBilling_interest(amount);
//        }
//
//        cash.setBilling_amount(acrInvoiceLn.getTotalAmount());
//        cash.setContract_id(acrInvoiceLn.getContractId());
//        cash.setTimes(acrInvoiceLn.getTimes().toString());
//        cash.setCf_item(acrInvoiceLn.getCfItem());
//        cash.setCf_type(acrInvoiceLn.getCfType());
        return cash;
    }

    private List<HlsCusAcrInvoiceHd> getAcrInvoiceHds(IRequest requestContext, List<HlsCusAcrInvoiceHd> acrInvoiceHds, Long companyId, Long userId) {
        Map<String, String> params = new HashMap<String, String>();
        if (acrInvoiceHds != null) {
            for (HlsCusAcrInvoiceHd acrInvoiceHd : acrInvoiceHds) {
                acrInvoiceHd.setCompanyId(companyId);
                acrInvoiceHd.setReversedFlag("N");
                acrInvoiceHd.setDocumentCategory("AR_INVOICE");
                acrInvoiceHd.setDocumentType("ACR");
                acrInvoiceHd.setBusinessType("ACR");
                acrInvoiceHd.setCurrency("CNY");

                acrInvoiceHd.setInvoiceStatus("NEW");
                acrInvoiceHd.setDocumentNumber(codingRuleValuesService.getCodeRuleValue(requestContext, "AR_INVOICE", "ACR", "ACR", params));
                acrInvoiceHd.setTaxAmount(sub(acrInvoiceHd.getTotalAmount(), acrInvoiceHd.getNetAmount()));
                if (acrInvoiceHd.getAcrInvoiceLnList() != null) {
                    for (int i = 0; i < acrInvoiceHd.getAcrInvoiceLnList().size(); i++) {
                        acrInvoiceHd.getAcrInvoiceLnList().get(i).setLineNumber(Long.valueOf(String.valueOf(i + 1)));
                    }
                }
            }
        }

        return acrInvoiceHds;
    }

    private void validate(List<HlsCusAcrInvoiceHd> acrInvoiceHds) throws BeyondAmountLimitException {
        if (acrInvoiceHds != null) {
            for (HlsCusAcrInvoiceHd acrInvoiceHd : acrInvoiceHds) {
                HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
                cashflow.setCashflow_id(acrInvoiceHd.getCashflowId());
                cashflow = conContractCashflowMapper.selectByPrimaryKey(cashflow);
                if (acrInvoiceHd.getCfItem().compareTo(100L) == 0) {
                    if (compareTo(acrInvoiceHd.getTotalAmount(), sub(cashflow.getPrincipal(), cashflow.getBilling_principal())) > 0) {
                        throw new BeyondAmountLimitException();
                    }
                } else if (acrInvoiceHd.getCfItem().compareTo(101L) == 0) {
                    if (compareTo(acrInvoiceHd.getTotalAmount(), sub(cashflow.getInterest(), cashflow.getBilling_interest())) > 0) {
                        throw new BeyondAmountLimitException();
                    }
                } else {
                    if (compareTo(acrInvoiceHd.getTotalAmount(), sub(cashflow.getDue_amount(), cashflow.getBilling_amount())) > 0) {
                        throw new BeyondAmountLimitException();
                    }
                }
            }
        }
//                Double hdTotalAmount = 0d;
//                Double lnTotalAmount = 0d;
//                hdTotalAmount = acrInvoiceHd.getTotalAmount();
//                if (acrInvoiceHd.getAcrInvoiceLnList() != null) {
//                    for (HlsCusAcrInvoiceLn acrInvoiceLn : acrInvoiceHd.getAcrInvoiceLnList()) {
//                        //拿到发票行对应的cashflow
//                        HlsCusConContractCashflow cashflow = new HlsCusConContractCashflow();
//                        cashflow.setCashflow_id(acrInvoiceLn.getCashflowId());
//                        cashflow =  conContractCashflowMapper.selectByPrimaryKey(cashflow);
//
//                        if(acrInvoiceLn.getCfItem().compareTo(100L) == 0){
//                                if (compareTo(acrInvoiceLn.getTotalAmount(), sub(cashflow.getPrincipal(),cashflow.getBilling_principal())) > 0) {
//                                throw new BeyondAmountLimitException();
//                            }
//                        }else if (acrInvoiceLn.getCfItem().compareTo(101L) == 0){
//                            if (compareTo(acrInvoiceLn.getTotalAmount(), sub(cashflow.getInterest(),cashflow.getBilling_interest())) > 0) {
//                                throw new BeyondAmountLimitException();
//                            }
//                        }else{
//                            if (compareTo(acrInvoiceLn.getTotalAmount(), sub(cashflow.getDue_amount(),cashflow.getBilling_amount())) > 0) {
//                                throw new BeyondAmountLimitException();
//                            }
//                        }
//                        lnTotalAmount = add(lnTotalAmount, acrInvoiceLn.getTotalAmount());
//                    }
//                }
//
//                if (compareTo(hdTotalAmount, lnTotalAmount) != 0) {
//                    throw new BeyondAmountLimitException();
//                }
//            }
//
//        }
    }

    private double add(Double a, Double b) {
        BigDecimal aa = new BigDecimal(a.toString());
        BigDecimal bb = new BigDecimal(b.toString());

        return aa.add(bb).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    private int compareTo(Double a, Double b) {
        BigDecimal aa = new BigDecimal(a.toString());
        BigDecimal bb = new BigDecimal(b.toString());

        return aa.compareTo(bb);
    }

    public Double multiply(Double a, Double b) {
        BigDecimal aa = new BigDecimal(a.toString());
        BigDecimal bb = new BigDecimal(b.toString());

        return aa.multiply(bb).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    public static double sub(double v1, double v2) {
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.subtract(b2).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    @Override
    public List<HlsCusAcrInvoiceHd> acr220aAcrInvoiceHdQuery(HlsCusAcrInvoiceHd acrInvoiceHd, IRequest iRequest, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
        if ("on".equalsIgnoreCase(acrInvoiceHd.getInvoiceKind())) {
            acrInvoiceHd.setInvoiceKind(null);
        }
        if ("1".equalsIgnoreCase(acrInvoiceHd.getNetAmountFlag1()) || "1".equalsIgnoreCase(acrInvoiceHd.getNetAmountFlag2())
                || "1".equalsIgnoreCase(acrInvoiceHd.getNetAmountFlag3()) || "1".equalsIgnoreCase(acrInvoiceHd.getNetAmountFlag4())
                || "1".equalsIgnoreCase(acrInvoiceHd.getNetAmountFlag5()) || "1".equalsIgnoreCase(acrInvoiceHd.getNetAmountFlag6())
                || "1".equalsIgnoreCase(acrInvoiceHd.getNetAmountFlag7())) {
            acrInvoiceHd.setNetAmountbz("1");
        }
        //设置税率
        setTaxAmount(acrInvoiceHd);
        //税价合计设置
        setTaxAmountTotal(acrInvoiceHd);
        return mapper.acr220aAcrInvoiceHdQuery(acrInvoiceHd);
    }

    @Override
    public List<HlsCusAcrInvoiceHd> selectAllReceipt(HlsCusAcrInvoiceHd acrInvoiceHd, IRequest iRequest, int page, int pagesize) {
        PageHelper.startPage(page, pagesize);
       /* if ("1".equalsIgnoreCase(acrInvoiceHd.getAmountFlag1()) || "1".equalsIgnoreCase(acrInvoiceHd.getAmountFlag2())
                || "1".equalsIgnoreCase(acrInvoiceHd.getAmountFlag3()) || "1".equalsIgnoreCase(acrInvoiceHd.getAmountFlag4())) {
            acrInvoiceHd.setNetAmountbz("1");
        }*/
        return mapper.selectAllReceipt(acrInvoiceHd);
    }

    /**
     * 税价合计设置
     *
     * @param acrInvoiceHd
     */
    public void setTaxAmountTotal(HlsCusAcrInvoiceHd acrInvoiceHd) {
        if ("1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountTotalFlag1()) || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountTotalFlag2())
                || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountTotalFlag3()) || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountTotalFlag4())
                || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountTotalFlag5()) || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountTotalFlag6())
                || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountTotalFlag7())) {
            acrInvoiceHd.setTaxAmountTotalbz("1");
        }
    }

    /**
     * 税率设置
     *
     * @param acrInvoiceHd
     */
    private void setTaxAmount(HlsCusAcrInvoiceHd acrInvoiceHd) {
        if ("1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountFlag1()) || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountFlag2())
                || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountFlag3()) || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountFlag4())
                || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountFlag5()) || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountFlag6())
                || "1".equalsIgnoreCase(acrInvoiceHd.getTaxAmountFlag7())) {
            acrInvoiceHd.setTaxAmountbz("1");
        }
    }

    @Override
    public List<HashMap> acrInvoiceDetail(HlsCusAcrInvoiceHd acrInvoiceHd, IRequest iRequest) {
        return mapper.acrInvoiceDetail(acrInvoiceHd);
    }


    @Override
    public void updateContractBillingStatus(HlsCusConContractCashflow cashflow) {
        //修改合同上的billing_status字段
        HlsCusConContract c = new HlsCusConContract();
        c.setContractId(cashflow.getContract_id());
        c = conContractMapper.selectByPrimaryKey(c);
        List<HlsCusConContractCashflow> cccList = cashflowMapper.selectCashFlowByContractId(cashflow.getContract_id());
        boolean flag = true;
        if (cccList != null && cccList.size() > 0) {
            for (HlsCusConContractCashflow ccc : cccList) {
                if (!"FULL".equals(ccc.getBilling_status())) {
                    flag = false;
                }
            }
        }
        if (flag) {
            c.setBillingStatus("FULL");
        } else {
            c.setBillingStatus("PARTIAL");
        }
        conContractMapper.updateByPrimaryKey(c);
    }

    @Override
    public List<HlsCusAcrInvoiceHd> vatInfIndexInvoiceDetailQuery(IRequest requestContext , HlsCusAcrInvoiceHd acrInvoiceHd,int page, int pagesize) {
        if (requestContext.getCompanyId()!=null){
            acrInvoiceHd.setCompanyId(requestContext.getCompanyId());
        }
        PageHelper.startPage(page, pagesize);
        return mapper.vatInfIndexInvoiceDetailQuery(acrInvoiceHd);
    }

    @Override
    public List<HlsCusAcrInvoiceHd> vatInfIndexInvoiceStatusQuery(HlsCusAcrInvoiceHd acrInvoiceHd) {
        return mapper.vatInfIndexInvoiceStatusQuery(acrInvoiceHd);
    }

    @Override
    public List<HlsCusAcrInvoiceHd> vatInfIndexInvoiceReversedQuery(HlsCusAcrInvoiceHd acrInvoiceHd) {
        return mapper.vatInfIndexInvoiceReversedQuery(acrInvoiceHd);
    }

    private List<HlsCusAcrInvoiceHd> principalAndInterestSplit(List<HlsCusAcrInvoiceHd> AcrInvoiceHdList) {
        if (AcrInvoiceHdList != null && AcrInvoiceHdList.size() > 0) {
            Iterator<HlsCusAcrInvoiceHd> iterator = AcrInvoiceHdList.iterator();
            while (iterator.hasNext()) {
                HlsCusAcrInvoiceHd next = iterator.next();
                // 以现金流为维度，判断cfItem 是否等于1,否不处理
                if (next.getCfItem().compareTo(1L) == 0 && next.getCfItem().compareTo(next.getBsCfItem()) != 0 && next.getVatModifyFlag() != 1) {
                    //BsCfItem不等于cfItem说明配置的开票规则将租金分成了本金和利息
                    //找到对应的两条现金流,确定它们的 mergeRentalFlag 字段是否都为 Y
                    HlsCusAcrInvoiceHd acrInvoiceHd = null;
                    for (HlsCusAcrInvoiceHd ccc2 : AcrInvoiceHdList) {
                        if (next.getCashflowId().compareTo(ccc2.getCashflowId()) == 0 && next.getBsCfItem().compareTo(ccc2.getBsCfItem()) != 0) {
                            acrInvoiceHd = ccc2;
                        }
                    }
                    if (acrInvoiceHd != null) {
                        //mergeRentalFlag 字段都为 Y ,则从List中删除一条cashflow
                        if (acrInvoiceHd.getMergeRentalFlag().equals("Y") && next.getMergeRentalFlag().equals("Y")) {
                            iterator.remove();
                        }
                        //否则修改销项发票头,100的改为本金,101的改为利息, 更新modifyFlag为1
                        else {
                            if (next.getBsCfItem().compareTo(100L) == 0) {
                                setInvoiceHdByCfItem(next, 100L);
                                setInvoiceHdByCfItem(acrInvoiceHd, 101L);
                            } else if (next.getBsCfItem().compareTo(101L) == 0) {
                                setInvoiceHdByCfItem(acrInvoiceHd, 100L);
                                setInvoiceHdByCfItem(next, 101L);
                            }
                        }
                    } else {
                        if (next.getBsCfItem().compareTo(100L) == 0) {
                            setInvoiceHdByCfItem(next, 100L);

                        } else if (next.getBsCfItem().compareTo(101L) == 0) {
                            setInvoiceHdByCfItem(next, 101L);
                        }
                    }
                }
                if("BP_TYPE".equals(next.getInvoiceKind())){
                    // 查询出该合同承租人的规模
                    HlsCusBpMaster bp = new HlsCusBpMaster();
                    bp.setBpId(next.getBpId());
                    bp = hlsCusBpMasterMapper.selectByPrimaryKey(bp);
                    if(bp != null){
                        switch (bp.getEnterpriseScale()){
                            case "XS":
                            case "S":
                                next.setInvoiceKind("2");
                                next.setInvoiceKindDesc("普通增票");
                                break;
                            case "M":
                            case "L":
                                next.setInvoiceKind("0");
                                next.setInvoiceKindDesc("专用增票");
                                break;
                        }
                    }else{

                    }
                }
            }
        }
//        Iterator<HlsCusAcrInvoiceHd> iterator2 = AcrInvoiceHdList.iterator();
//        while (iterator2.hasNext()) {
//            HlsCusAcrInvoiceHd next2 = iterator2.next();
//            if(next2.getReceivedAmount().compareTo(0d) == 0 ){
//                iterator2.remove();
//            }
//        }
        return AcrInvoiceHdList;
    }

    public void setInvoiceHdByCfItem(HlsCusAcrInvoiceHd acrInvoiceHd, Long cfItem) {
        if (cfItem.compareTo(100L) == 0) {
            acrInvoiceHd.setCfItem(100L);
//            acrInvoiceHd.setReceivedAmount(acrInvoiceHd.getPrincipal() - acrInvoiceHd.getBillingPrincipal());
//            acrInvoiceHd.setReceivedInterest(0d);
            acrInvoiceHd.setTotalAmount(acrInvoiceHd.getPrincipal() - acrInvoiceHd.getBillingPrincipal());

        } else if (cfItem.compareTo(101L) == 0) {
            acrInvoiceHd.setCfItem(101L);
//            acrInvoiceHd.setReceivedAmount(acrInvoiceHd.getReceivedInterest());
//            acrInvoiceHd.setReceivedPrincipal(0d);
            acrInvoiceHd.setTotalAmount(acrInvoiceHd.getInterest() - acrInvoiceHd.getBillingInterest());
//            acrInvoiceHd.setTotalAmount(acrInvoiceHd.getReceivedInterest());
        }
        acrInvoiceHd.setTaxAmount(acrInvoiceHd.getTotalAmount() * acrInvoiceHd.getTaxTypeRate());
        acrInvoiceHd.setNetAmount(acrInvoiceHd.getTotalAmount() - acrInvoiceHd.getTaxAmount());
        acrInvoiceHd.setVatModifyFlag(1);
    }

    @Override
    public List<HlsCusAcrInvoiceHd> vatInfByIdInvoiceReversed(IRequest iRequest, HlsCusAcrInvoiceHd acrInvoiceHd) {
        HlsCusAcrInvoiceLn acrInvoiceLn = new HlsCusAcrInvoiceLn();
        acrInvoiceLn.setInvoiceHdId(acrInvoiceHd.getInvoiceHdId());
        List<HlsCusAcrInvoiceLn> acrInvoiceLns = lnService.select(iRequest, acrInvoiceLn, 1, 1000000);
        acrInvoiceHd = self().selectByPrimaryKey(iRequest, acrInvoiceHd);
        acrInvoiceHd.setReversedFlag("W");
        acrInvoiceHd.setReverseDate(new Date());
        self().updateByPrimaryKeySelective(iRequest, acrInvoiceHd);
        acrInvoiceHd.setTotalAmount(acrInvoiceHd.getTotalAmount() * (-1));
        acrInvoiceHd.setNetAmount(acrInvoiceHd.getNetAmount() * (-1));
        acrInvoiceHd.setTaxAmount(acrInvoiceHd.getTaxAmount() * (-1));
        acrInvoiceHd.setReversedFlag("R");
        self().insertSelective(iRequest, acrInvoiceHd);
        HlsCusConContractCashflow dto = new HlsCusConContractCashflow();
        for (int i = 0; i < acrInvoiceLns.size(); i++) {
            HlsCusAcrInvoiceRelationship acrInvoiceRelationship = new HlsCusAcrInvoiceRelationship();
            acrInvoiceRelationship.setInvoiceLnId(acrInvoiceLns.get(i).getInvoiceLnId());
            List<HlsCusAcrInvoiceRelationship> acrInvoiceRelationships = acrInvoiceRelationshipService.select(iRequest, acrInvoiceRelationship, 1, 1000000);
            for (int j = 0; j < acrInvoiceRelationships.size(); j++) {
                dto.setCashflow_id(acrInvoiceRelationships.get(j).getSourceDocumentId());
                HlsCusConContractCashflow contractCashflow = iConContractCashflowService.selectByPrimaryKey(iRequest, dto);
                contractCashflow.setBilling_status("NOT");
                contractCashflow.setBilling_amount((double) 0);
                contractCashflow.setBilling_principal((double) 0);
                contractCashflow.setBilling_interest((double) 0);
                iConContractCashflowService.updateByPrimaryKeySelective(iRequest, contractCashflow);

                // 新增事件
                double totalAmount = acrInvoiceHd.getTotalAmount();
                String documentNumber = acrInvoiceHd.getDocumentNumber();
                HashMap<String, Object> map = new HashMap<>();
                map.put("amt",totalAmount);
                map.put("invoiceNum",documentNumber);
                map.put("eventCode", SysEventCodeUtil.PROPERTY_ACR_INVOICE_HD_REVERSE);
                sysEventService.createEvent(iRequest,contractCashflow.getContract_id(),"CON_CONTRACT","",map);

            }
        }
        List<HlsCusAcrInvoiceHd> list = new ArrayList<HlsCusAcrInvoiceHd>();
        list.add(acrInvoiceHd);
        return list;
    }

    @Override
    public void acrIndexTxtMerge(IRequest iRequest,FndInterfaceLines fndInterfaceLines){
        List<FndInterfaceLines> lines = fndInterfaceLinesService.fndInterfaceLinesTxtQuery(fndInterfaceLines);
        for(FndInterfaceLines line : lines){
            String[] idAndDoc = line.getAttributes_9().split("__");
            Long id = (long)Integer.parseInt(idAndDoc[0]);
            HlsCusAcrInvoiceHd hd = new HlsCusAcrInvoiceHd();
            hd.setInvoiceHdId(id);
            hd = self().selectByPrimaryKey(iRequest,hd);

            if(hd == null || hd.getInvoiceHdId() == null){
                return;
            }
            // lock acr_invoice_hd
            databaseLockProvider.lock(hd);

            hd.setInvoiceStatus(line.getAttributes_1());
            hd.setInvoiceNumber(line.getAttributes_5());
            self().updateByPrimaryKeySelective(iRequest,hd);
        }
    }
}
