package hls.core.vat.service.impl;

import com.github.pagehelper.PageHelper;
import com.hand.hap.core.IRequest;
import hls.core.csh.dto.HlsCusConContractCashflow;
import hls.core.csh.dto.HlsCusCshConContract;
import hls.core.csh.dto.HlsCusCshTransaction;
import hls.core.csh.service.IConContractCashflowService;
import hls.core.vat.dto.HlsCusAcrInvoiceLn;
import hls.core.vat.mapper.HlsCusAcrInvoiceHdMapper;
import hls.core.vat.mapper.HlsCusAcrInvoiceLnMapper;
import hls.core.vat.mapper.HlsCusAcrInvoiceRelationshipMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.hand.hap.system.service.impl.BaseServiceImpl;

import hls.core.vat.service.AcrInvoiceLnService;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Transactional(rollbackFor = Exception.class)
public class AcrInvoiceLnServiceImpl extends BaseServiceImpl<HlsCusAcrInvoiceLn> implements AcrInvoiceLnService {

    @Autowired
    HlsCusAcrInvoiceLnMapper acrInvoiceLnMapper;

    @Autowired
    HlsCusAcrInvoiceHdMapper acrInvoiceHdMapper;

    @Autowired
    IConContractCashflowService conContractCashflowService;

    @Autowired
    private HlsCusAcrInvoiceRelationshipMapper relationshipMapper;

    @Override
    public List<Map> leasePrincipal(HlsCusAcrInvoiceLn acrInvoiceLn, IRequest irequest, int page, int pagesize) {
        PageHelper.startPage(page,pagesize);
        HashMap map = new HashMap();
        if(acrInvoiceLn.getBpName()!=null){
            map.put("bpName",acrInvoiceLn.getBpName());
        }
        if(acrInvoiceLn.getContractName()!=null){
            map.put("contractName",acrInvoiceLn.getContractName());
        }
        return acrInvoiceLnMapper.leasePrincipal(acrInvoiceLn);
    }

    @Override
    public List<Map> reciveAmount(HlsCusAcrInvoiceLn acrInvoiceLn, IRequest irequest, int page, int pagesize) {
        PageHelper.startPage(page,pagesize);
        HashMap map = new HashMap();
        return acrInvoiceLnMapper.reciveAmount(acrInvoiceLn);
    }

    @Override
    public List<Map> stepOne(List<Map> maps, IRequest iRequest,long companyId) {

        //List<Map> lists = acrInvoiceLnMapper.stepOne(maps);//先获取要插入头表的数据
        for (Map map:maps) {
            map.put("companyId",companyId);
            map.put("documentType","ACR");
            map.put("documentCategory","AR_INVOICE");
            //待修改
            map.put("documentNumber","");

            //已收金额或者回租本金
            BigDecimal receivedAmountAcr = new BigDecimal(Double.toString(Double.valueOf(map.get("receivedAmountAcr").toString())));
            BigDecimal billingAmountAcr =  new BigDecimal(Double.toString(Double.valueOf(map.get("billingAmountAcr").toString())));
            int r=receivedAmountAcr.subtract(billingAmountAcr).compareTo(BigDecimal.ZERO); //和0，Zero比较
            if(r==0||r==-1) { //等于//小于
                throw new IllegalArgumentException(map.get("contractName")+"的"+map.get("times")+"期现金流不存在可开票金额");
            }
            map.put("writeOffDueAmount",receivedAmountAcr.subtract(billingAmountAcr).doubleValue());
            acrInvoiceHdMapper.stepTwo(map);//插入头表
            long headId = (long) map.get("invoice_hd_id");//获取返回的头表的id
            map.put("headId",headId);
            map.put("lineNumber",1);
            acrInvoiceLnMapper.stepFour(map);//插入发票的行表

            //插入发票relationship 表
            relationshipMapper.insertRelationship(map);

            //判断开票状态
            long cashflowId = Long.parseLong(map.get("cashflowId").toString());
            long cfItem=Long.parseLong(map.get("cfItem").toString());
            //回租本金
            BigDecimal dueAmount = new BigDecimal(Double.toString(Double.valueOf(map.get("dueAmount").toString())));
            HlsCusConContractCashflow obj = new HlsCusConContractCashflow();
            obj.setCashflow_id(cashflowId);
            obj = conContractCashflowService.selectByPrimaryKey(iRequest,obj);//获取该条现金流上的数据
            if(cfItem==1){
                BigDecimal billingInterest=   new BigDecimal(obj.getBilling_interest()!=null?obj.getBilling_interest().toString():"0");
                BigDecimal billingPrincipal=   new BigDecimal(obj.getBilling_principal()!=null?obj.getBilling_principal().toString():"0");
                BigDecimal billingAmountP=billingInterest.add(billingPrincipal).add(receivedAmountAcr.subtract(billingAmountAcr));
                if(billingAmountP.compareTo(dueAmount)==0){
                    obj.setBilling_status("FULL");
                }else if(billingAmountP.compareTo(dueAmount)<0&&billingAmountP.compareTo(BigDecimal.ZERO)==0){
                    obj.setBilling_status("NOT");
                }else if(billingAmountP.compareTo(dueAmount)<0&&billingAmountP.compareTo(BigDecimal.ZERO)>0){
                    obj.setBilling_status("PARTIAL");
                }else{
                    throw new IllegalArgumentException(map.get("contractName")+"的"+map.get("times")+"期开票金额已超出！");
                }
                obj.setBilling_amount(billingAmountP.doubleValue());
                obj.setBilling_principal(billingPrincipal.add(receivedAmountAcr.subtract(billingAmountAcr)).doubleValue());
            }else{
                //其他费用项
                BigDecimal billingAmountOther=   new BigDecimal(obj.getBilling_amount()!=null?obj.getBilling_amount().toString():"0");
                BigDecimal billingAmountOtherTotal=billingAmountOther.add(receivedAmountAcr.subtract(billingAmountAcr));
                if(billingAmountOtherTotal.compareTo(dueAmount)==0){
                    obj.setBilling_status("FULL");
                }else if(billingAmountOtherTotal.compareTo(dueAmount)<0&&receivedAmountAcr.compareTo(BigDecimal.ZERO)==0){
                    obj.setBilling_status("NOT");
                }else if(billingAmountOtherTotal.compareTo(dueAmount)<0&&receivedAmountAcr.compareTo(BigDecimal.ZERO)>0){
                    obj.setBilling_status("PARTIAL");
                }else{
                    throw new IllegalArgumentException(map.get("contractName")+"的"+map.get("times")+"期开票金额已超出！");
                }
                obj.setBilling_amount(billingAmountOtherTotal.doubleValue());
            }
            conContractCashflowService.updateByPrimaryKey(iRequest,obj);


        }
        return maps;
    }



       public void acrHLRecINS(List<Map> maps){
           for (Map map:maps) {
               acrInvoiceHdMapper.acrHeadRecive(map);
               acrInvoiceLnMapper.acrLineRecive(map);
           }
       }
    @Override
    public   List<HlsCusAcrInvoiceLn> selectAllAcrInvoiceLn(IRequest iRequest, HlsCusCshTransaction cshTransaction) throws ParseException {
        Long fate =  cshTransaction.getQueryTimeSolt();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        HlsCusCshConContract cshConContract = new HlsCusCshConContract();
        if(fate!=null) {
            String strDate = calculationDate(fate);
            cshConContract.setQueryCondition(cshTransaction.getQueryCondition());
            cshConContract.setDocument_type(cshTransaction.getDocumentType());
            cshConContract.setDue_date(sdf.parse(strDate));
        }else{
            String strDate = sdf.format(new Date())+"01";
            Date date = sdf.parse(strDate);
            cshConContract.setDue_date(date);
        }
        cshConContract.setCompany_id(iRequest.getCompanyId());
        return  acrInvoiceLnMapper.selectAllAcrInvoiceLn(cshConContract);
    }

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    private String calculationDate(Long fate) {

        Date beginDate = new Date();
        Calendar date = Calendar.getInstance();
        date.setTime(beginDate);
        if (fate == 7) {
            date.set(Calendar.DATE, date.get(Calendar.DATE) - 7);
        }
        if (fate == 30) {
            date.add(Calendar.MONTH, -1);
        }
        if (fate == 90) {
            date.add(Calendar.MONTH, -3);
        }
        String endDate = sdf.format(date.getTime());
        return endDate;
    }
}
