package hls.core.vat.service.impl;

import com.hand.hap.core.IRequest;
import freemarker.template.Configuration;
import freemarker.template.Template;
import hls.core.fnd.dto.FndInterfaceHeaders;
import hls.core.fnd.dto.FndInterfaceLines;
import hls.core.fnd.service.FndInterfaceHeadersService;
import hls.core.fnd.service.FndInterfaceLinesService;
import hls.core.vat.dto.HlsCusAcrInvoiceHd;
import hls.core.vat.mapper.HlsCusAcrInvoiceHdMapper;
import hls.core.vat.mapper.HlsCusAcrInvoiceLnMapper;
import hls.core.vat.mapper.HlsCusTaxMapper;
import hls.core.vat.service.ITaxService;
import org.apache.commons.lang.StringUtils;
import org.hibernate.validator.constraints.NotEmpty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.freemarker.FreeMarkerConfigurationFactory;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class TaxServiceImpl implements ITaxService {
    public static final String FILE_SEPARATOR = "fileSeparator";
    @Value("${file.separator}")
    private String fileSeparator;


    @Autowired
    private HlsCusAcrInvoiceHdMapper acrInvoiceHdMapper;

    @Autowired
    private HlsCusAcrInvoiceLnMapper acrInvoiceLnMapper;

    @Autowired
    private FndInterfaceLinesService fndInterfaceLinesService;

    @Autowired
    private FndInterfaceHeadersService fndInterfaceHeadersService;

    @Autowired
    private HlsCusTaxMapper taxMapper;

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Override
    public String exportInvoice(@NotEmpty List<Long> headerIds, String fileName,HttpServletResponse httpServletResponse) {
        // 按照头表的ID列表进行查询
        List<Map> headers = new ArrayList<Map>();
        if(headerIds != null && headerIds.size() != 0) {
             headers = taxMapper.findAcrInvoiceHdByIds(headerIds);
        }
        List<Map<String, Object>> invoices = new ArrayList<>(headers.size());

        for (Map header : headers) {
            Map<String, Object> map = new HashMap<>();
            Object hdId = header.get("invoiceHdId");
            if (hdId == null || !StringUtils.isNumeric(hdId.toString())) {
                continue;
            }
            List<Map> lines = taxMapper.findAcrInvoiceLnByHeadId(Long.valueOf(hdId.toString()));
            header.put("lines", lines.size());

            map.put("header", header);
            map.put("lines", lines);
            invoices.add(map);
        }

        Map<String, Object> infos = new HashMap<>();
        infos.put(FILE_SEPARATOR, fileSeparator);
        infos.put("note", "");
        infos.put("invoices", invoices);

        try {
            return generateFtl(infos, httpServletResponse);
        } catch (Exception e) {
            logger.error("生成单据文件出错", e);
        }
        return null;
    }

    @Transactional
    public FndInterfaceHeaders readInvoice(@NotEmpty String fileName,InputStreamReader isr,IRequest iRequest) {
        FndInterfaceHeaders fndInterfaceHeaders = new FndInterfaceHeaders();
        try (BufferedReader bufferedReader = new BufferedReader(isr)) {
            fndInterfaceHeaders.setFileName(fileName);
            fndInterfaceHeaders.setStatus("NEW");
            fndInterfaceHeaders.setTemplateCode("GOLD");
            fndInterfaceHeadersService.insertSelective(iRequest,fndInterfaceHeaders);

            String line = null;
            boolean isHeader = true;
            boolean isRange = false;
            boolean isInvoiceHeader = false;
            boolean isInvoiceLine = false;
            String tableHeader = null;
            int invoiceLine = 0; //发票总数
            int lineNum = 0; // 单个发票的明细行数
            while ((line = bufferedReader.readLine()) != null) {
                if ("".equals(line.trim()) || line.startsWith("//")) {
                    // 注释内容或者空行，不需要处理
                    continue;
                }
                // 正文数据，分为表头，范围，(发票头，多个发票明细行)+
                String[] parts = StringUtils.split(line, "~~");
                if (isHeader && "SJJK0201".equals(parts[0])) {
                    // 已开发票传出文件，表头数据
                    isHeader = false; // 表头只读取一次，置为false
                    isRange = true;
                    tableHeader = line;
                } else if (isRange) {
                    // 范围
                    isRange = false;  // 范围只读取一次，置为false
                    if (parts.length < 3 || !StringUtils.isNumeric(parts[0])) {
                        throw new IOException("file format error");
                    }
                    invoiceLine = Integer.valueOf(parts[0]);
                    isInvoiceHeader = true;
                } else if (isInvoiceHeader && invoiceLine > 0) {
                    // 发票头数据
                    isInvoiceHeader = false;
                    if (parts.length < 21 || !StringUtils.isNumeric(parts[5])) {
                        throw new IOException("file format error");
                    }
                    // 先设置发票的行数
                    lineNum = Integer.valueOf(parts[5]);

                    // 对发票头的数据进行处理
                    String idAndDocNum = parts[8];
                    String[] split = StringUtils.split(idAndDocNum,"__");
                    if (split.length < 2 || !StringUtils.isNumeric(split[0])) {
                        throw new IOException("file format error");
                    }
                    Long hdId = Long.valueOf(split[0]);
                    String docNum = split[1];

                    HlsCusAcrInvoiceHd acrInvoiceHd = new HlsCusAcrInvoiceHd();
                    acrInvoiceHd.setInvoiceHdId(hdId);
                    acrInvoiceHd.setDocumentNumber(docNum);
                    HlsCusAcrInvoiceHd target = acrInvoiceHdMapper.selectByPrimaryKey(acrInvoiceHd);
                    if(target == null || target.getInvoiceHdId() == null){
                        throw new IOException("unknown invoice id__documentNumber: "+parts[8]);
                    }
                    FndInterfaceLines fndInterfaceLines = new FndInterfaceLines();
                    fndInterfaceLines.setHeaderId(fndInterfaceHeaders.getHeaderId());
                    fndInterfaceLines.setAttributes_1(parts[0]);
                    fndInterfaceLines.setAttributes_2(parts[1]);
                    fndInterfaceLines.setAttributes_3(parts[2]);
                    fndInterfaceLines.setAttributes_4(parts[3]);
                    fndInterfaceLines.setAttributes_5(parts[4]);
                    fndInterfaceLines.setAttributes_6(parts[5]);
                    fndInterfaceLines.setAttributes_7(parts[6]);
                    fndInterfaceLines.setAttributes_8(parts[7]);
                    fndInterfaceLines.setAttributes_9(parts[8]);
                    fndInterfaceLines.setAttributes_10(parts[9]);
                    fndInterfaceLines.setAttributes_11(parts[10]);
                    fndInterfaceLines.setAttributes_12(parts[11]);
                    fndInterfaceLines.setAttributes_13(parts[12]);
                    fndInterfaceLines.setAttributes_14(parts[13]);
                    fndInterfaceLines.setAttributes_15(parts[14]);
                    fndInterfaceLines.setAttributes_16(parts[15]);
                    fndInterfaceLines.setAttributes_17(parts[16]);
                    fndInterfaceLines.setAttributes_18(parts[17]);
                    fndInterfaceLines.setAttributes_19(parts[18]);
                    fndInterfaceLines.setAttributes_20(parts[19]);
                    fndInterfaceLinesService.insertSelective(iRequest,fndInterfaceLines);
                    /*target.setInvoiceStatus(parts[0]);
                    target.setInvoiceNumber(parts[4]);
                    acrInvoiceHdMapper.updateByPrimaryKeySelective(target);*/
                    isInvoiceLine = true;
                } else if (isInvoiceLine && lineNum > 0) {
                    // 发票行数据
                    //FIXME 发票行数据的处理
                    lineNum--;
                    // 单个发票所有行读取完成，需要将发票头数量减 1
                    if (lineNum <= 0) {
                        isInvoiceLine = false;
                        isInvoiceHeader = true;
                        invoiceLine--;
                    }
                } else {
                    // 未知的数据
                    throw new IOException("unknown line: " + line);
                }

            }
        } catch (FileNotFoundException e) {
            logger.error("file[" + fileName + "] not exists or cannot be read", e);
        } catch (IOException e) {
            logger.error("read file[" + fileName + "] error", e);
        }
        return fndInterfaceHeaders;
    }

    public String generateFtl(Map<String, Object> map,HttpServletResponse httpServletResponse) throws Exception {
        Configuration cfg = new FreeMarkerConfigurationFactory().createConfiguration();
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
                .getRequest();
        cfg.setServletContextForTemplateLoading(request.getServletContext(), "WEB-INF/view/vat");
        cfg.setDefaultEncoding("utf-8");
        Template template = cfg.getTemplate("soldNote.ftl");
        template.setEncoding("utf-8");


        httpServletResponse.reset();
        httpServletResponse.setContentType("application/x-download");
        String filename = "gold.txt";
        filename = URLEncoder.encode(filename,"UTF-8");
        httpServletResponse.setCharacterEncoding("UTF-8");
        httpServletResponse.addHeader("Content-Disposition","attachment;filename=" + filename);
        PrintWriter printwriter = new PrintWriter(httpServletResponse.getOutputStream());
        template.process(map, printwriter);
        printwriter.close();
        return "success";
    }
}
