package hls.core.wfl.service.impl;

import com.hand.hap.activiti.dto.ReProcdef;
import com.hand.hap.activiti.service.IActivitiService;
import com.hand.hap.activiti.service.IReProcdefService;
import com.hand.hap.core.IRequest;
import hls.core.cont.dto.HlsCusConFloatingRateReq;
import hls.core.cont.service.IConFloatingRateReqService;
import hls.core.csh.dto.HlsCusConDebtExemptionReq;
import hls.core.fnd.dto.HlsEmployee;
import hls.core.fnd.mapper.HlsEmployeeMapper;
import hls.core.sys.dto.SysUser;
import hls.core.sys.service.SysUserService;
import hls.core.wfl.service.IActivitiCommonService;
import org.activiti.rest.service.api.engine.variable.RestVariable;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceCreateRequest;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by fjm on 2017/10/27.
 */
@Service
public class ConFloatingRateActivitiStartServiceImpl implements IActivitiCommonService {
    private static final String workFlowType = "CON_FLOATING_RATE";
    public static final String APPROVING = "APPROVING";

    @Autowired
    private IActivitiService activitiService;
    @Autowired
    private IReProcdefService reProcdefService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private HlsEmployeeMapper employeeMapper;
    @Autowired
    private IConFloatingRateReqService conFloatingRateReqService;

    @Override
    public String getWorkFlowType() {
        return workFlowType;
    }

    @Override
    public void process(IRequest iRequest, List list, Map params) {
        ProcessInstanceCreateRequest processInstance = new ProcessInstanceCreateRequest();
        ReProcdef reProcdefs = reProcdefService.queryReProcdef("CON_FLOATING_RATE_WORK_FLOW", "CON_CONTRACT");
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        HlsEmployee employee = employeeMapper.getEmployeeCode(iRequest.getUserId());
        HlsCusConFloatingRateReq conFloatingRateReq = (HlsCusConFloatingRateReq) list.get(0);
        conFloatingRateReq = conFloatingRateReqService.selectByPrimaryKey(iRequest,conFloatingRateReq);
        String employeeCode = employee.getEmployeeCode();
        iRequest.setEmployeeCode(employeeCode);

        String id = reProcdefs.getId_();
        String name = reProcdefs.getName_();
        processInstance.setProcessDefinitionId(id);
        processInstance.setBusinessKey(conFloatingRateReq.getFltReqId().toString());

        List<RestVariable> variables = new ArrayList<RestVariable>();
        List<RestVariable> transientVariables = new ArrayList<RestVariable>();

        RestVariable restVariable1 = new RestVariable();
        restVariable1.setName("startUserDescription");
        restVariable1.setValue(sysUser.getDescription());
        variables.add(restVariable1);

        RestVariable restVariable2 = new RestVariable();
        restVariable2.setName("iRequest");
        restVariable2.setValue(iRequest);
        variables.add(restVariable2);

        RestVariable restVariable3 = new RestVariable();
        restVariable3.setName("documentCategory");
        restVariable3.setValue(conFloatingRateReq.getDocumentCategory());
        variables.add(restVariable3);

        RestVariable restVariable4 = new RestVariable();
        restVariable4.setName("workflowType");
        restVariable4.setValue(workFlowType);
        variables.add(restVariable4);

        processInstance.setVariables(variables);
        processInstance.setTransientVariables(transientVariables);

        ProcessInstanceResponse processInstanceResponse = activitiService.startProcess(iRequest, processInstance);

        Long processInstanceId = Long.valueOf(processInstanceResponse.getId());
        conFloatingRateReq.setProcessInstanceId(processInstanceId);
        conFloatingRateReq.setStatus(APPROVING);
        conFloatingRateReqService.updateByPrimaryKeySelective(iRequest,conFloatingRateReq);
    }

    @Override
    public void cancel(IRequest iRequest, Map params) {
        String businessKey = (String) params.get("businessKey");
        String processInstanceId = (String) params.get("processInstanceId");
        long id = Long.parseLong(businessKey);
        long prcId = Long.parseLong(processInstanceId);
        HlsCusConFloatingRateReq conFloatingRateReq = new HlsCusConFloatingRateReq();
        conFloatingRateReq.setFltReqId(id);
        conFloatingRateReq.setStatus(CANCEL_STATUS);
        conFloatingRateReqService.updateByPrimaryKeySelective(iRequest,conFloatingRateReq);
    }
}
