package hls.core.wfl.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.activiti.dto.ReProcdef;
import com.hand.hap.activiti.service.IActivitiService;
import com.hand.hap.activiti.service.IReProcdefService;
import com.hand.hap.core.IRequest;
import hls.core.fct.dto.HlsCusHlsCreditLineApproval;
import hls.core.fct.service.HlsCreditLineApprovalService;
import hls.core.fct.service.HlsCusFctWflCustomService;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.service.HlsCreditLineService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.service.SysUserService;
import hls.core.wfl.service.IActivitiCommonService;
import org.activiti.rest.service.api.engine.variable.RestVariable;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceCreateRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by wangyan on 2017/11/13.
 */
@Service
@Transactional
public class CreditLineActivitiStartServiceImpl implements IActivitiCommonService {
    private static final String workFlowType = "CREDIT_LINE";
    @Autowired
    private IActivitiService activitiService;
    @Autowired
    private IReProcdefService reProcdefService;
    @Autowired
    private HlsCreditLineService hlsCreditLineService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private HlsCreditLineApprovalService hlsCreditLineApprovalService;
    @Autowired(required = false)
    private HlsCusFctWflCustomService fctWflCustomService;

    @Override
    public String getWorkFlowType() {
        return workFlowType;
    }

    @Override
    public void process(IRequest iRequest, List list, Map params) {
        //增加综合授信二开逻辑
        ProcessInstanceCreateRequest processInstanceCreateRequest = getProcessInstanceCreateRequest((HlsCusHlsCreditLine)list.get(0),iRequest);
        if (fctWflCustomService != null && !fctWflCustomService.beforeCreditSubmit(iRequest, (HlsCusHlsCreditLine) list.get(0))) {
            return;
        }
        activitiService.startProcess(iRequest, processInstanceCreateRequest);
        if (fctWflCustomService != null) {
            fctWflCustomService.afterCreditSubmit(iRequest, (HlsCusHlsCreditLine) list.get(0));
        }
    }


    private ProcessInstanceCreateRequest getProcessInstanceCreateRequest(HlsCusHlsCreditLine creditLine, IRequest iRequest) {
        ProcessInstanceCreateRequest createRequest = new ProcessInstanceCreateRequest();
        ReProcdef reProcdefs = reProcdefService.queryReProcdef("HLS_CREDIT_LINE_WORK_FLOW","CREDIT_LINE");
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String id = reProcdefs.getId_();
        String name = reProcdefs.getName_();
        createRequest.setProcessDefinitionId(id);

        createRequest.setBusinessKey(creditLine.getCreditLineId().toString());
        //设置参数
        List<RestVariable> variables = new ArrayList<RestVariable>();
        List<RestVariable> transientVariables = new ArrayList<RestVariable>();
        RestVariable restVariable1 = new RestVariable();
        restVariable1.setName("processDefinitionId");
        restVariable1.setValue(id);
        variables.add(restVariable1);
        RestVariable restVariable2 = new RestVariable();
        restVariable2.setName("startUserDescription");
        restVariable2.setValue(sysUser.getDescription());
        variables.add(restVariable2);
        RestVariable restVariable3 = new RestVariable();
        restVariable3.setName("iRequest");
        restVariable3.setValue(iRequest);
        variables.add(restVariable3);
        RestVariable restVariable4 = new RestVariable();
        restVariable4.setName("startUserName");
        restVariable4.setValue(iRequest.getEmployeeCode());
        variables.add(restVariable4);
        RestVariable restVariable5 = new RestVariable();
        restVariable5.setName("creditLine");
        JSONObject jsonObject= JSON.parseObject(JSON.toJSONString(creditLine));
        restVariable5.setValue(jsonObject.toString());
        variables.add(restVariable5);
        RestVariable restVariable6 = new RestVariable();
        restVariable6.setName("documentCategory");
        restVariable6.setValue("CREDIT_LINE");
        variables.add(restVariable6);
        RestVariable restVariable7 = new RestVariable();
        restVariable7.setName("documentType");
        restVariable7.setValue("");
        variables.add(restVariable7);
        RestVariable restVariable8 = new RestVariable();
        restVariable8.setName("documentId");
        restVariable8.setValue(creditLine.getCreditLineId());
        variables.add(restVariable8);
        RestVariable restVariable9 = new RestVariable();
        restVariable9.setName("pName");
        restVariable9.setValue(name);
        variables.add(restVariable9);
        RestVariable restVariable10 = new RestVariable();
        restVariable10.setName("creditLineName");
        restVariable10.setValue(creditLine.getCreditLineName());
        variables.add(restVariable10);
        RestVariable restVariable11 = new RestVariable();
        restVariable11.setName("creditLineId");
        restVariable11.setValue(creditLine.getCreditLineId());
        variables.add(restVariable11);
        RestVariable restVariable12 = new RestVariable();
        restVariable12.setName("creditLineStatus");
        restVariable12.setValue(creditLine.getCreditLineStatus());
        variables.add(restVariable12);

        RestVariable restVariable13 = new RestVariable();
        restVariable13.setName("workflowType");
        restVariable13.setValue(workFlowType);
        variables.add(restVariable13);

        createRequest.setVariables(variables);
        createRequest.setTransientVariables(transientVariables);

        //插入审批表
        HlsCusHlsCreditLineApproval hlsCreditLineApproval = new HlsCusHlsCreditLineApproval();
        hlsCreditLineApproval.setCreditLineId(creditLine.getCreditLineId());
        hlsCreditLineApproval.setStatus("APPROVING");

        hlsCreditLineApprovalService.insertSelective(iRequest,hlsCreditLineApproval);
        return createRequest;
    }

    @Override
    public void cancel(IRequest iRequest, Map params) {
        String businessKey = (String) params.get("businessKey");
        String processInstanceId = (String) params.get("processInstanceId");
        long id = Long.parseLong(businessKey);
        long prcId = Long.parseLong(processInstanceId);
        HlsCusHlsCreditLine hlsCreditLine = new HlsCusHlsCreditLine();
        hlsCreditLine.setCreditLineId(id);
        hlsCreditLine.setCreditLineStatus(CANCEL_STATUS);
        //hlsCreditLineService.updateByPrimaryKeySelective(iRequest,hlsCreditLine);
        HlsCusHlsCreditLineApproval hlsCusHlsCreditLineApproval = new HlsCusHlsCreditLineApproval();
        hlsCusHlsCreditLineApproval.setCreditLineId(id);
        hlsCusHlsCreditLineApproval.setStatus(CANCEL_STATUS);
        hlsCusHlsCreditLineApproval.setProcessInstanceId(prcId);
        hlsCreditLineApprovalService.insertSelective(iRequest,hlsCusHlsCreditLineApproval);
    }
}
