package hls.core.wfl.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.activiti.dto.ReProcdef;
import com.hand.hap.activiti.service.IActivitiService;
import com.hand.hap.activiti.service.IReProcdefService;
import com.hand.hap.core.IRequest;
import hls.core.fct.dto.HlsCusFctContract;
import hls.core.fct.dto.HlsCusFctContractTermination;
import hls.core.fct.service.FctContractService;
import hls.core.fct.service.FctContractTerminationService;
import hls.core.fct.service.HlsCusFctWflCustomService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.service.SysUserService;
import hls.core.wfl.service.IActivitiCommonService;
import org.activiti.rest.service.api.engine.variable.RestVariable;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceCreateRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
@Transactional
public class FactorProjectWriteOffActivitStartServiceImpl implements IActivitiCommonService {

    private static final String workFlowType = "FCT_PROJECT_WRITEOFF";

    @Autowired
    private IActivitiService activitiService;
    @Autowired
    private IReProcdefService reProcdefService;
    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private FctContractService fctContractService;
    @Autowired(required = false)
    private HlsCusFctWflCustomService fctWflCustomService;
    @Autowired
    private FctContractTerminationService fctContractTerminationService;

    @Override
    public String getWorkFlowType() {
        return workFlowType;
    }

    @Override
    public void process(IRequest iRequest, List list, Map map) {
        Long contractId = Long.parseLong(map.get("contractId").toString());
        ProcessInstanceCreateRequest processInstanceCreateRequest = getProcessInstanceCreateRequest((HlsCusFctContract) list.get(0),  iRequest);
        if (fctWflCustomService != null && !fctWflCustomService.beforeContractEndSubmit(iRequest, (HlsCusFctContract) list.get(0))) {
            return;
        }
        activitiService.startProcess(iRequest, processInstanceCreateRequest);
        if (fctWflCustomService != null) {
            fctWflCustomService.afterContractEndSubmit(iRequest, (HlsCusFctContract) list.get(0));
        }

    }

    private ProcessInstanceCreateRequest getProcessInstanceCreateRequest(HlsCusFctContract fctContract, IRequest iRequest) {
        ProcessInstanceCreateRequest createRequest = new ProcessInstanceCreateRequest();

        ReProcdef reProcdefs;

        reProcdefs = reProcdefService.queryReProcdef("FCT_PROJECT_WRITEOFF_WORK_FLOW", "FCT_PROJECT_WRITEOFF");
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String id = reProcdefs.getId_();
        String name = reProcdefs.getName_();

        createRequest.setProcessDefinitionId(id);
        createRequest.setBusinessKey(fctContract.getContractId().toString());

        List<RestVariable> variables = new ArrayList<RestVariable>();
        List<RestVariable> transientVariables = new ArrayList<RestVariable>();
        RestVariable restVariable1 = new RestVariable();
        RestVariable restVariable2 = new RestVariable();
        RestVariable restVariable3 = new RestVariable();
        RestVariable restVariable4 = new RestVariable();
        RestVariable restVariable5 = new RestVariable();
        RestVariable restVariable6 = new RestVariable();
        RestVariable restVariable7 = new RestVariable();
        RestVariable restVariable8 = new RestVariable();
        RestVariable restVariable9 = new RestVariable();
        RestVariable restVariable10 = new RestVariable();
        RestVariable restVariable11 = new RestVariable();
        RestVariable restVariable12 = new RestVariable();
        RestVariable restVariable13 = new RestVariable();
        RestVariable restVariable14 = new RestVariable();
        RestVariable restVariable15 = new RestVariable();


        restVariable1.setName("processDefinitionId");
        restVariable1.setValue(id);
        variables.add(restVariable1);

        restVariable2.setName("fctContract");
        JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(fctContract));
        restVariable2.setValue(jsonObject.toString());
        variables.add(restVariable2);

        restVariable3.setName("iRequest");
        restVariable3.setValue(iRequest);
        variables.add(restVariable3);

        restVariable4.setName("startUserName");
        restVariable4.setValue(sysUser.getDescription());
        variables.add(restVariable4);


        restVariable5.setName("userId");
        restVariable5.setValue(sysUser.getUserId());
        variables.add(restVariable11);

        restVariable6.setName("pName");
        restVariable6.setValue(name);
        variables.add(restVariable6);


        //根据是否常规核销
        restVariable7.setName("routineVerification");
        restVariable7.setValue("NO_NORMAL");
        variables.add(restVariable7);

        String routineVerification = "NO_NORMAL";
        String isRoutineVerification;
        if ("NO_NORMAL".equals(routineVerification)) {
            isRoutineVerification = "Y";
        } else {
            isRoutineVerification = "N";
        }
        restVariable8.setName("isRoutineVerification");
        restVariable8.setValue(isRoutineVerification);
        variables.add(restVariable8);

        //这是工作流中配置启动 结束监听器后触发消息盒子
        restVariable9.setName("documentCategory");
        restVariable9.setValue(fctContract.getDocumentCategory());
        variables.add(restVariable9);
        restVariable10.setName("documentType");
        restVariable10.setValue(fctContract.getDocumentType());
        variables.add(restVariable10);
        restVariable11.setName("documentId");
        restVariable11.setValue(fctContract.getContractId());
        variables.add(restVariable11);

        //流程中节点的名称 自定义
        restVariable12.setName("showName");
        restVariable12.setValue("中交建");
        variables.add(restVariable12);


        restVariable13.setName("contractId");
        restVariable13.setValue(fctContract.getContractId());
        variables.add(restVariable13);

        //获取projectId
        HlsCusFctContract fctContract1=new HlsCusFctContract();
        fctContract1.setContractId(fctContract.getContractId());
        fctContract1=fctContractService.selectByPrimaryKey(iRequest,fctContract1);
        restVariable15.setName("projectId");
        restVariable15.setValue(fctContract1.getProjectId());
        variables.add(restVariable15);

        //插入审批表
        HlsCusFctContractTermination fctContractTermination=new HlsCusFctContractTermination();
        fctContractTermination.setContractId(fctContract.getContractId());
        fctContractTermination.setDescription("合同结束");
        fctContractTermination.setStatus("APPROVING");
        fctContractTermination.setTerminationDate(new Date());
        fctContractTermination.set__status("add");
        fctContractTerminationService.insertSelective(iRequest,fctContractTermination);

        restVariable14.setName("fctContractTermination");
        JSONObject jsonObject4 = JSON.parseObject(JSON.toJSONString(fctContractTermination));
        restVariable14.setValue(jsonObject4.toString());
        variables.add(restVariable14);

        createRequest.setVariables(variables);
        createRequest.setTransientVariables(transientVariables);
        return createRequest;

    }
}