package hls.core.wfl.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.activiti.dto.ReProcdef;
import com.hand.hap.activiti.service.IActivitiService;
import com.hand.hap.activiti.service.IReProcdefService;
import com.hand.hap.core.IRequest;
import hls.core.fct.dto.HlsCusFctProject;
import hls.core.fct.service.FctProjectConFileAprlService;
import hls.core.fct.service.FctProjectService;
import hls.core.fct.service.HlsCusFctProjectCustomService;
import hls.core.fct.service.HlsCusFctWflCustomService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.service.SysUserService;
import hls.core.wfl.service.IActivitiCommonService;
import org.activiti.rest.service.api.engine.variable.RestVariable;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceCreateRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author: syj
 * @Description:
 * @Date: Created in 2017/11/16 10:20
 * @Modified By:
 */
@Service
@Transactional
public class FctContractContentActivitiStartServiceImpl implements IActivitiCommonService {
    private static final String workFlowType = "FCT_CONTRACT_CONTENT";

    @Autowired
    private IReProcdefService reProcdefService;
    @Autowired
    private IActivitiService activitiService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private FctProjectConFileAprlService fctProjectConFileAprlService;
    @Autowired
    private FctProjectService fctProjectService;
    @Autowired(required = false)
    private HlsCusFctWflCustomService fctWflCustomService;


    @Override
    public String getWorkFlowType() {
        return workFlowType;
    }

    @Override
    public void process(IRequest iRequest, List list, Map params) {
        Long projectId = Long.parseLong(params.get("projectId").toString());
        ProcessInstanceCreateRequest processInstanceCreateRequest = getProcessInstanceCreateRequest((HlsCusFctProject) list.get(0), iRequest);
        if (fctWflCustomService != null && !fctWflCustomService.beforeContentSubmit(iRequest, list)) {
            return;
        }
        activitiService.startProcess(iRequest, processInstanceCreateRequest);
        if (fctWflCustomService != null) {
            fctWflCustomService.afterContentSubmit(iRequest, list);
        }
    }

    private ProcessInstanceCreateRequest getProcessInstanceCreateRequest(HlsCusFctProject fctProject, IRequest iRequest) {
        ProcessInstanceCreateRequest createRequest = new ProcessInstanceCreateRequest();

        ReProcdef reProcdefs;

        reProcdefs = reProcdefService.queryReProcdef("FCT_CONTRACT_CONTENT_WORK_FLOW", "FCT_CONTRACT_CONTENT");
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String id = reProcdefs.getId_();
        String name = reProcdefs.getName_();
        createRequest.setProcessDefinitionId(id);
        createRequest.setBusinessKey(fctProject.getProjectId().toString());

        List<RestVariable> variables = new ArrayList<RestVariable>();
        List<RestVariable> transientVariables = new ArrayList<RestVariable>();
        RestVariable restVariable1 = new RestVariable();
        RestVariable restVariable2 = new RestVariable();
        RestVariable restVariable3 = new RestVariable();
        RestVariable restVariable4 = new RestVariable();
        RestVariable restVariable5 = new RestVariable();
        RestVariable restVariable6 = new RestVariable();
        RestVariable restVariable7 = new RestVariable();
        RestVariable restVariable8 = new RestVariable();
        RestVariable restVariable9 = new RestVariable();
        RestVariable restVariable10 = new RestVariable();


        restVariable1.setName("processDefinitionId");
        restVariable1.setValue(id);
        variables.add(restVariable1);

        restVariable2.setName("fctProject");
        JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(fctProject));
        restVariable2.setValue(jsonObject.toString());
        variables.add(restVariable2);

        restVariable3.setName("iRequest");
        restVariable3.setValue(iRequest);
        variables.add(restVariable3);

        restVariable4.setName("startUserName");
        restVariable4.setValue(sysUser.getDescription());
        variables.add(restVariable4);


        restVariable5.setName("documentCategory");
        restVariable5.setValue("FCT_CONTRACT_CONTENT");
        variables.add(restVariable5);
        restVariable6.setName("documentType");
        restVariable6.setValue(fctProject.getDocumentType());
        variables.add(restVariable6);
        restVariable7.setName("documentId");
        restVariable7.setValue(fctProject.getProjectId());
        variables.add(restVariable7);

        restVariable8.setName("pName");
        restVariable8.setValue(name);
        variables.add(restVariable8);


        restVariable9.setName("projectName");
        restVariable9.setValue(fctProject.getProjectName());
        variables.add(restVariable9);

        restVariable10.setName("workflowType");
        restVariable10.setValue(workFlowType);
        variables.add(restVariable10);

        RestVariable restVariable11 = new RestVariable();
        restVariable11.setName("projectId");
        restVariable11.setValue(fctProject.getProjectId());
        variables.add(restVariable11);

        createRequest.setVariables(variables);
        createRequest.setTransientVariables(transientVariables);


        /*//插入审批表
        HlsCusFctProjectConFileAprl fctProjectConFileAprl = new HlsCusFctProjectConFileAprl();
        fctProjectConFileAprl.setProjectId(fctProject.getProjectId());
        fctProjectConFileAprl.setStatus("APPROVING");
        fctProjectConFileAprl.setProcessInstanceId(Long.parseLong(id));

        fctProjectConFileAprlService.insertSelective(iRequest,fctProjectConFileAprl);*/

        return createRequest;
    }

    @Override
    public void cancel(IRequest iRequest, Map params) {
        String businessKey = (String) params.get("businessKey");
        String processInstanceId = (String) params.get("processInstanceId");
        long id = Long.parseLong(businessKey);
        long prcId = Long.parseLong(processInstanceId);
        HlsCusFctProject fctProject = new HlsCusFctProject();
        fctProject.setProjectId(id);
        fctProject.setContractStatus(CANCEL_STATUS);
        fctProjectService.updateByPrimaryKeySelective(iRequest,fctProject);
    }
}
