package hls.core.wfl.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.activiti.dto.ReProcdef;
import com.hand.hap.activiti.service.IActivitiService;
import com.hand.hap.activiti.service.IReProcdefService;
import com.hand.hap.core.IRequest;
import hls.core.fct.dto.HlsCusFctContract;
import hls.core.fct.dto.HlsCusFctContractWithdraw;
import hls.core.fct.dto.HlsCusFctContractWithdrawAprl;
import hls.core.fct.service.FctContractService;
import hls.core.fct.service.FctContractWithdrawAprlService;
import hls.core.fct.service.FctContractWithdrawService;
import hls.core.fct.service.HlsCusFctWflCustomService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.service.SysUserService;
import hls.core.wfl.service.IActivitiCommonService;
import org.activiti.rest.service.api.engine.variable.RestVariable;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceCreateRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Author: syj
 * @Description:保理付款申请 工作流启动
 * @Date: Created in 2017/12/7 13:36
 * @Modified By:
 */
@Service
@Transactional
public class FctPaymentRequestActivitiStartServiceImpl implements IActivitiCommonService {
    private static final String workFlowType = "FCT_PAYMENT_REQUEST";

    @Autowired
    private IReProcdefService reProcdefService;
    @Autowired
    private IActivitiService activitiService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private FctContractWithdrawAprlService fctContractWithdrawAprlService;

    @Autowired
    private FctContractWithdrawService fctContractWithdrawService;
    @Autowired(required = false)
    private HlsCusFctWflCustomService fctWflCustomService;

    @Autowired
    private FctContractService fctContractService;


    @Override
    public String getWorkFlowType() {
        return workFlowType;
    }

    @Override
    public void process(IRequest iRequest, List list, Map params) {
        ProcessInstanceCreateRequest processInstanceCreateRequest = getProcessInstanceCreateRequest((HlsCusFctContractWithdraw) list.get(0), iRequest,params);
        if (fctWflCustomService != null && !fctWflCustomService.beforePaymentSubmit(iRequest, (HlsCusFctContractWithdraw) list.get(0))) {
            return;
        }
        activitiService.startProcess(iRequest, processInstanceCreateRequest);
        if (fctWflCustomService != null) {
            fctWflCustomService.afterPaymentSubmit(iRequest, (HlsCusFctContractWithdraw) list.get(0));
        }
    }

    private ProcessInstanceCreateRequest getProcessInstanceCreateRequest(HlsCusFctContractWithdraw hlsCusFctContractWithdraw, IRequest iRequest,Map params) {
        ProcessInstanceCreateRequest createRequest = new ProcessInstanceCreateRequest();
        ReProcdef reProcdefs;

        reProcdefs = reProcdefService.queryReProcdef("FCT_PAYMENT_REQUEST_WORK_FLOW", "FCT_PAYMENT_REQUEST");
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String id = reProcdefs.getId_();
        String name = reProcdefs.getName_();
        createRequest.setProcessDefinitionId(id);
        createRequest.setBusinessKey(hlsCusFctContractWithdraw.getWithdrawId().toString());

        List<RestVariable> variables = new ArrayList<RestVariable>();
        List<RestVariable> transientVariables = new ArrayList<RestVariable>();
        RestVariable restVariable1 = new RestVariable();
        RestVariable restVariable2 = new RestVariable();
        RestVariable restVariable3 = new RestVariable();
        RestVariable restVariable4 = new RestVariable();
        RestVariable restVariable5 = new RestVariable();
        RestVariable restVariable6 = new RestVariable();
        RestVariable restVariable7 = new RestVariable();
        RestVariable restVariable8 = new RestVariable();
        RestVariable restVariable9 = new RestVariable();
        RestVariable restVariable10 = new RestVariable();
        RestVariable restVariable11 = new RestVariable();
        RestVariable restVariable12 = new RestVariable();
        RestVariable restVariable13 = new RestVariable();


        restVariable1.setName("processDefinitionId");
        restVariable1.setValue(id);
        variables.add(restVariable1);

        restVariable2.setName("hlsCusFctContractWithdraw");
        JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(hlsCusFctContractWithdraw));
        restVariable2.setValue(jsonObject.toString());
        variables.add(restVariable2);

        restVariable3.setName("iRequest");
        restVariable3.setValue(iRequest);
        variables.add(restVariable3);

        restVariable4.setName("startUserName");
        restVariable4.setValue(sysUser.getDescription());
        variables.add(restVariable4);


        restVariable5.setName("documentCategory");
        restVariable5.setValue("FCT_CONTRACT_WITHDRAW");
        variables.add(restVariable5);
        restVariable6.setName("documentType");
        restVariable6.setValue(hlsCusFctContractWithdraw.getDocumentType());
        variables.add(restVariable6);
        restVariable7.setName("documentId");
        restVariable7.setValue(hlsCusFctContractWithdraw.getWithdrawId());
        variables.add(restVariable7);

        restVariable8.setName("pName");
        restVariable8.setValue(name);
        variables.add(restVariable8);


        restVariable9.setName("withdrawName");
        restVariable9.setValue(hlsCusFctContractWithdraw.getWithdrawName());
        variables.add(restVariable9);

        //插入审批表
        HlsCusFctContractWithdrawAprl hlsCusFctContractWithdrawAprl = new HlsCusFctContractWithdrawAprl();
        hlsCusFctContractWithdrawAprl.setWithdrawApprovalType("PAYMENT_REQ");
        hlsCusFctContractWithdrawAprl.setWithdrawId(hlsCusFctContractWithdraw.getWithdrawId());
        hlsCusFctContractWithdrawAprl.setStatus("APPROVING");

        fctContractWithdrawAprlService.insertSelective(iRequest,hlsCusFctContractWithdrawAprl);


        restVariable10.setName("withdrawApprovalId");
        restVariable10.setValue(hlsCusFctContractWithdrawAprl.getWithdrawApprovalId());
        variables.add(restVariable10);

        restVariable11.setName("paymentReqId");
        restVariable11.setValue(params.get("paymentReqId"));
        variables.add(restVariable11);

        restVariable12.setName("paymentReqLnId");
        restVariable12.setValue(params.get("paymentReqLnId"));
        variables.add(restVariable12);

        restVariable13.setName("workflowType");
        restVariable13.setValue(workFlowType);
        variables.add(restVariable13);

        //获取projectId
        RestVariable restVariable14 = new RestVariable();
        HlsCusFctContract fctContract1=new HlsCusFctContract();
        fctContract1.setContractId(hlsCusFctContractWithdraw.getContractId());
        fctContract1=fctContractService.selectByPrimaryKey(iRequest,fctContract1);
        restVariable14.setName("projectId");
        restVariable14.setValue(fctContract1.getProjectId());
        variables.add(restVariable14);

        createRequest.setVariables(variables);
        createRequest.setTransientVariables(transientVariables);

        return createRequest;
    }

    @Override
    public void cancel(IRequest iRequest, Map params) {
        String businessKey = (String) params.get("businessKey");
        String processInstanceId = (String) params.get("processInstanceId");
        long id = Long.parseLong(businessKey);
        long prcId = Long.parseLong(processInstanceId);
        HlsCusFctContractWithdraw fctContractWithdraw = new HlsCusFctContractWithdraw();
        fctContractWithdraw.setWithdrawId(id);
        fctContractWithdraw.setWithdrawStatus(CANCEL_STATUS);
        fctContractWithdrawService.updateByPrimaryKeySelective(iRequest,fctContractWithdraw);
    }
}
