package hls.core.wfl.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.activiti.dto.ReProcdef;
import com.hand.hap.activiti.service.IActivitiService;
import com.hand.hap.activiti.service.IReProcdefService;
import com.hand.hap.core.IRequest;
import hls.core.fct.dto.HlsCusFctProject;
import hls.core.fct.dto.HlsCusFctProjectApproval;
import hls.core.fct.dto.HlsCusHlsCreditLineApproval;
import hls.core.fct.mapper.HlsCusFctProjectApprovalMapper;
import hls.core.fct.service.*;
import hls.core.prj.dto.HlsCusHlsCreditLine;
import hls.core.prj.service.HlsCreditLineService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.service.SysUserService;
import hls.core.wfl.service.IActivitiCommonService;
import org.activiti.rest.service.api.engine.variable.RestVariable;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceCreateRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by wangyan on 2017/11/13.
 */
@Service
@Transactional
public class FctProjectActivitiStartServiceImpl implements IActivitiCommonService {
    private static final String workFlowType = "FCT_PROJECT_HLSCUS";
    @Autowired
    private IActivitiService activitiService;
    @Autowired
    private IReProcdefService reProcdefService;
    @Autowired
    private FctProjectService fctProjectService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private FctProjectApprovalService fctProjectApprovalService;
    @Autowired
    private HlsCusFctProjectApprovalMapper fctProjectApprovalMapper;
    @Autowired(required = false)
    private HlsCusFctWflCustomService fctWflCustomService;


    @Override
    public String getWorkFlowType() {
        return workFlowType;
    }

    @Override
    public void process(IRequest iRequest, List list, Map params) {
        ProcessInstanceCreateRequest processInstanceCreateRequest = getProcessInstanceCreateRequest((HlsCusFctProject)list.get(0),iRequest);
        if (fctWflCustomService != null && !fctWflCustomService.beforeProjectSubmit(iRequest,list)) {
            return;
        }
        activitiService.startProcess(iRequest, processInstanceCreateRequest);
        if (fctWflCustomService != null) {
            fctWflCustomService.afterProjectSubmit(iRequest,list);
        }
    }


    private ProcessInstanceCreateRequest getProcessInstanceCreateRequest(HlsCusFctProject fctProject, IRequest iRequest) {
        ProcessInstanceCreateRequest createRequest = new ProcessInstanceCreateRequest();
        ReProcdef reProcdefs = reProcdefService.queryReProcdef("FCT_PROJECT_WORK_FLOW","FCT_PROJECT");
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String id = reProcdefs.getId_();
        String name = reProcdefs.getName_();
        createRequest.setProcessDefinitionId(id);
        createRequest.setBusinessKey(fctProject.getProjectId().toString());
        //设置参数
        List<RestVariable> variables = new ArrayList<RestVariable>();
        List<RestVariable> transientVariables = new ArrayList<RestVariable>();
        RestVariable restVariable1 = new RestVariable();
        restVariable1.setName("processDefinitionId");
        restVariable1.setValue(id);
        variables.add(restVariable1);
        RestVariable restVariable2 = new RestVariable();
        restVariable2.setName("startUserDescription");
        restVariable2.setValue(sysUser.getDescription());
        variables.add(restVariable2);
        RestVariable restVariable3 = new RestVariable();
        restVariable3.setName("iRequest");
        restVariable3.setValue(iRequest);
        variables.add(restVariable3);
        RestVariable restVariable4 = new RestVariable();
        restVariable4.setName("startUserName");
        restVariable4.setValue(iRequest.getEmployeeCode());
        variables.add(restVariable4);
        RestVariable restVariable5 = new RestVariable();
        restVariable5.setName("fctProject");
        JSONObject jsonObject= JSON.parseObject(JSON.toJSONString(fctProject));
        restVariable5.setValue(jsonObject.toString());
        variables.add(restVariable5);
        RestVariable restVariable6 = new RestVariable();
        restVariable6.setName("documentCategory");
        restVariable6.setValue("FCT_PROJECT");
        variables.add(restVariable6);
        RestVariable restVariable7 = new RestVariable();
        restVariable7.setName("documentType");
        restVariable7.setValue("");
        variables.add(restVariable7);
        RestVariable restVariable8 = new RestVariable();
        restVariable8.setName("documentId");
        restVariable8.setValue(fctProject.getProjectId());
        variables.add(restVariable8);
        RestVariable restVariable9 = new RestVariable();
        restVariable9.setName("pName");
        restVariable9.setValue(name);
        variables.add(restVariable9);
        RestVariable restVariable10 = new RestVariable();
        restVariable10.setName("projectName");
        restVariable10.setValue(fctProject.getProjectName());
        variables.add(restVariable10);

        RestVariable restVariable11 = new RestVariable();
        restVariable11.setName("workflowType");
        restVariable11.setValue(workFlowType);
        variables.add(restVariable11);

        RestVariable restVariable12 = new RestVariable();
        restVariable12.setName("projectId");
        restVariable12.setValue(fctProject.getProjectId());
        variables.add(restVariable12);

        createRequest.setVariables(variables);
        createRequest.setTransientVariables(transientVariables);

        //插入审批表
        HlsCusFctProjectApproval fctProjectApproval=new HlsCusFctProjectApproval();
       // HlsCusHlsCreditLineApproval hlsCreditLineApproval = new HlsCusHlsCreditLineApproval();
        fctProjectApproval.setProjectId(fctProject.getProjectId());
        fctProjectApproval.setStatus("APPROVING");

        fctProjectApprovalService.insertSelective(iRequest,fctProjectApproval);
        return createRequest;
    }

    @Override
    public void cancel(IRequest iRequest, Map params) {
        String businessKey = (String) params.get("businessKey");//ProjectId
        String processInstanceId = (String) params.get("processInstanceId");
        long projectId = Long.parseLong(businessKey);
        long prcId = Long.parseLong(processInstanceId);
        //修改审批表
        HlsCusFctProjectApproval fctProjectApproval=new HlsCusFctProjectApproval();
        fctProjectApproval = fctProjectApprovalMapper.queryFctProjectApprovalByProjectId(projectId);
        fctProjectApproval.setStatus(CANCEL_STATUS);
        fctProjectApproval.setProcessInstanceId(prcId);
        fctProjectApprovalService.updateByPrimaryKeySelective(iRequest,fctProjectApproval);
        //更改fct_project状态
        HlsCusFctProject fctProject = new HlsCusFctProject();
        fctProject.setProjectId(projectId);
        String status = "NEW";
        fctProject.setProjectStatus(status);
        fctProjectService.updateByPrimaryKeySelective(iRequest,fctProject);
    }
}
