package hls.core.wfl.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hand.hap.activiti.dto.ReProcdef;
import com.hand.hap.activiti.service.IActivitiService;
import com.hand.hap.activiti.service.IReProcdefService;
import com.hand.hap.core.IRequest;
import hls.core.fct.dto.HlsCusFctProject;
import hls.core.fct.dto.HlsCusHlsCreditLineApproval;
import hls.core.fct.mapper.HlsCusFctProjectMapper;
import hls.core.fct.mapper.HlsCusHlsCreditLineApprovalMapper;
import hls.core.fct.service.FctProjectService;
import hls.core.fct.service.HlsCreditLineApprovalService;
import hls.core.fct.service.HlsCusFctWflCustomService;
import hls.core.sys.dto.SysUser;
import hls.core.sys.service.SysUserService;
import hls.core.wfl.service.IActivitiCommonService;
import org.activiti.rest.service.api.engine.variable.RestVariable;
import org.activiti.rest.service.api.runtime.process.ProcessInstanceCreateRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author: syj
 * @Description:
 * @Date: Created in 2017/11/9 20:29
 * @Modified By:
 */
@Service
@Transactional
public class FctProjectCreditActivitiStartServiceImpl implements IActivitiCommonService {
    private static final String workFlowType = "FCT_PROJECT_CREDIT";

    @Autowired
    private IReProcdefService reProcdefService;
    @Autowired
    private IActivitiService activitiService;
    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private HlsCreditLineApprovalService hlsCreditLineApprovalService;
    @Autowired
    private FctProjectService fctProjectService;
    @Autowired
    private HlsCusFctProjectMapper mapper;
    @Autowired
    private HlsCusFctProjectMapper hlsCusFctProjectMapper;
    @Autowired
    private HlsCusHlsCreditLineApprovalMapper hlsCreditLineApprovalMapper;
    @Autowired(required = false)
    private HlsCusFctWflCustomService fctWflCustomService;


    @Override
    public String getWorkFlowType() {
        return workFlowType;
    }

    @Override
    public void process(IRequest iRequest, List list, Map params) {
        Long projectId = Long.parseLong(params.get("projectId").toString());
        ProcessInstanceCreateRequest processInstanceCreateRequest = getProcessInstanceCreateRequest((HlsCusFctProject) list.get(0), iRequest);
        if (fctWflCustomService != null && !fctWflCustomService.beforeFctCreditSubmit(iRequest, (HlsCusFctProject) list.get(0))) {
            return;
        }
        activitiService.startProcess(iRequest, processInstanceCreateRequest);
        if (fctWflCustomService != null) {
            fctWflCustomService.afterFctCreditSubmit(iRequest, (HlsCusFctProject) list.get(0));
        }
    }

    private ProcessInstanceCreateRequest getProcessInstanceCreateRequest(HlsCusFctProject fctProject, IRequest iRequest) {
        ProcessInstanceCreateRequest createRequest = new ProcessInstanceCreateRequest();

        ReProcdef reProcdefs;

        reProcdefs = reProcdefService.queryReProcdef("FCT_PROJECT_CREDIT_WORK_FLOW", "FCT_PROJECT_CREDIT");
        SysUser sysUser = sysUserService.selectUserById(iRequest.getUserId());
        String id = reProcdefs.getId_();
        String name = reProcdefs.getName_();
        createRequest.setProcessDefinitionId(id);
        createRequest.setBusinessKey(fctProject.getProjectId().toString());

        List<RestVariable> variables = new ArrayList<RestVariable>();
        List<RestVariable> transientVariables = new ArrayList<RestVariable>();
        RestVariable restVariable1 = new RestVariable();
        RestVariable restVariable2 = new RestVariable();
        RestVariable restVariable3 = new RestVariable();
        RestVariable restVariable4 = new RestVariable();
        RestVariable restVariable5 = new RestVariable();
        RestVariable restVariable6 = new RestVariable();
        RestVariable restVariable7 = new RestVariable();
        RestVariable restVariable8 = new RestVariable();
        RestVariable restVariable9 = new RestVariable();
        RestVariable restVariable10 = new RestVariable();
        RestVariable restVariable11 = new RestVariable();


        restVariable1.setName("processDefinitionId");
        restVariable1.setValue(id);
        variables.add(restVariable1);

        restVariable2.setName("fctProject");
        JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(fctProject));
        restVariable2.setValue(jsonObject.toString());
        variables.add(restVariable2);

        restVariable3.setName("iRequest");
        restVariable3.setValue(iRequest);
        variables.add(restVariable3);

        restVariable4.setName("startUserName");
        restVariable4.setValue(sysUser.getDescription());
        variables.add(restVariable4);


        restVariable5.setName("documentCategory");
        restVariable5.setValue("FCT_PROJECT_CREDIT");
        variables.add(restVariable5);
        restVariable6.setName("documentType");
        restVariable6.setValue(fctProject.getDocumentType());
        variables.add(restVariable6);
        restVariable7.setName("documentId");
        restVariable7.setValue(fctProject.getProjectId());
        variables.add(restVariable7);

        restVariable8.setName("pName");
        restVariable8.setValue(name);
        variables.add(restVariable8);


        restVariable9.setName("projectName");
        restVariable9.setValue(fctProject.getProjectName());
        variables.add(restVariable9);

        //插入审批表
        HlsCusHlsCreditLineApproval hlsCreditLineApproval = new HlsCusHlsCreditLineApproval();
        hlsCreditLineApproval.setCreditLineId(fctProject.getCreditLineId());
        hlsCreditLineApproval.setStatus("APPROVING");
        //hlsCreditLineApproval.setProcessInstanceId(Long.parseLong(id));
        hlsCreditLineApprovalService.insertSelective(iRequest,hlsCreditLineApproval);


        restVariable10.setName("approvalId");
        restVariable10.setValue(hlsCreditLineApproval.getApprovalId());
        variables.add(restVariable10);

        restVariable11.setName("workflowType");
        restVariable11.setValue(workFlowType);
        variables.add(restVariable11);

        RestVariable restVariable12 = new RestVariable();
        restVariable12.setName("projectId");
        restVariable12.setValue(fctProject.getProjectId());
        variables.add(restVariable12);

        createRequest.setVariables(variables);
        createRequest.setTransientVariables(transientVariables);

        return createRequest;
    }

    @Override
    public void cancel(IRequest iRequest, Map params) {
        String businessKey = (String) params.get("businessKey");
        String processInstanceId = (String) params.get("processInstanceId");
        long projectId = Long.parseLong(businessKey);
        long prcId = Long.parseLong(processInstanceId);
        //获取credit_line_id
        HlsCusFctProject fctProject = new HlsCusFctProject();
        fctProject = hlsCusFctProjectMapper.queryFctProjectByProjectId(projectId);
        long creditLineId = fctProject.getCreditLineId();
        HlsCusHlsCreditLineApproval hlsCreditLineApproval = new HlsCusHlsCreditLineApproval();
        hlsCreditLineApproval = hlsCreditLineApprovalMapper.queryCreditLineByCreditLineId(creditLineId);
        //更改审批状态
        hlsCreditLineApproval.setStatus(CANCEL_STATUS);
        hlsCreditLineApproval.setProcessInstanceId(prcId);
        hlsCreditLineApprovalService.updateByPrimaryKeySelective(iRequest,hlsCreditLineApproval);
        //更改授信状态为NEW
        mapper.updateCreditStatus(projectId, "NEW", iRequest.getUserId(), new Date());

    }
}
